-- Data_to_LCD.vhd

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity lcd is
    Port ( lcd_data 	: out std_logic_vector(7 downto 4);
           clk        	: in std_logic;
		 reset      	: in std_logic;
		 SW2      	: in std_logic;                              
		 RTC_year		: in std_logic_vector(7 downto 0);           
	      RTC_month	: in std_logic_vector(7 downto 0);           
	      RTC_date		: in std_logic_vector(7 downto 0);           
		 RTC_hours	: in std_logic_vector(7 downto 0);
	      RTC_minutes	: in std_logic_vector(7 downto 0);
	      RTC_seconds	: in std_logic_vector(7 downto 0);
		 lcd_enable 	: out std_logic;
           lcd_rs 		: out std_logic;
 		 lcd_rw 		: out std_logic
 		 );

end lcd ;


architecture behavioural of lcd is

type state_type is (warmup, setfunc, clear1, clear2, setmode1, setmode2,
			     write1a_I, write1b_I, write1a_D, write1b_D,
			     write2a_I, write2b_I, write2a_D, write2b_D,
			     write3a_I, write3b_I, write3a_D, write3b_D,
			     write4a_I, write4b_I, write4a_D, write4b_D,
			     write5a_I, write5b_I, write5a_D, write5b_D,
			     write6a_I, write6b_I, write6a_D, write6b_D,
			     write7a_I, write7b_I, write7a_D, write7b_D,
			     write8a_I, write8b_I, write8a_D, write8b_D
			     );

signal state : state_type;

attribute syn_state_machine : boolean;
attribute syn_state_machine of state : signal is true;  

signal count : std_logic_vector(3 downto 0);


begin
    lcd_rw <= '0';
    lcd_enable <= clk; --not clk; if enable is not pulsed, lcd will not write
    
    state_set: process (clk, reset)
	begin
		if reset = '1' then
			count 		<= (others => '0');
            	state 		<= warmup;
            	lcd_data 		<= "0000"; 
            	lcd_rs 		<= '0';

		elsif (clk'event and clk = '1') then
			case state is
				when warmup =>
					lcd_rs <= '0';
		-- wait for LCD to power-up					
					lcd_data <= "0011"; 
					if count = "1111" then
						count <= (others => '0');
						state <= setfunc;
					else
						count <= count + '1';
						state <= warmup;
					end if;
		-- set the LCD mode to 4-bit mode					
				when setfunc =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "0010";
                    	
                    	if count = "0011" then
						count <= (others => '0');
						state <= clear1;
                    	else
						count <= count + '1';
                    	    	state <= setfunc;
					end if;
		-- set cursor settings and turn display ON				
                	when clear1 =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "0000";
                    	state 	<= clear2;				
                	when clear2 =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "1100";  -- turn cursor increment off
                    	state 	<= setmode1;
		-- set entry mode (no dispaly shift)	                    				
				when setmode1 	=>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "0000";
					state 	<= setmode2;				
                	when setmode2 =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "0110";
					state 	<= write1a_I;
					
	-- Write all 8 segments of the LCD display
	-- Write the MS digit of hour to the 1st char. position
          -- Set DDram memory to address 00h                          											
				when write1a_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "1000";
                    	state 	<= write1b_I;				
                	when write1b_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "0000";  
                    	state 	<= write1a_D;
		-- Now set control signal to data and write                   	
				when write1a_D =>
					lcd_rs 	<= '1';
					if (RTC_hours(4) = '1') or (SW2 = '1') then  --load ascii 1 (31h)					
                    		lcd_data 	<= "0011";
					else						--load blank (88h)
						lcd_data 	<= "1000";	
					end if;											                    		
                    	state 	<= write1b_D;				
                	when write1b_D =>
					lcd_rs 	<= '1';
					if (SW2 = '1') then
						lcd_data 	<= RTC_month(7 downto 4);					
					elsif (RTC_hours(4) = '1') then  --load ascii 1 (31h)					
                    		lcd_data 	<= "0001";
					else						--load blank (88h)
						lcd_data 	<= "1000";	
					end if;
					state 	<= write2a_I;					
	-- Write the LS digit of hour to the 2nd char. position
          -- Set DDram memory to address 01h                          											
				when write2a_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "1000";
                    	state 	<= write2b_I;				
                	when write2b_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "0001";  
                    	state 	<= write2a_D;
		-- Now set control signal to data and write 3xh                 	
				when write2a_D =>
					lcd_rs 	<= '1';
					lcd_data 	<= "0011";											                    		
                    	state 	<= write2b_D;				
                	when write2b_D =>
					lcd_rs 	<= '1';
					if (SW2 = '1') then
						lcd_data 	<= RTC_month(3 downto 0);
					else									
						lcd_data 	<= RTC_hours(3 downto 0);
					end if;						
					state 	<= write3a_I;					
	-- Write a ":" to the 3rd char. position
          -- Set DDram memory to address 02h                          											
				when write3a_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "1000";
                    	state 	<= write3b_I;				
                	when write3b_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "0010";  
                    	state 	<= write3a_D;
		-- Now set control signal to data and write ascii : 3a                   	
				when write3a_D =>
					lcd_rs 	<= '1';
					if (SW2 = '1') then
						lcd_data 	<= "0010";	-- 2D is a "-"
					else
                    		lcd_data 	<= "0011";	
					end if;                    												                    		
                    	state 	<= write3b_D;				
                	when write3b_D =>
					lcd_rs 	<= '1';
					if (SW2 = '1') then
						lcd_data 	<= "1101";	-- 2D is a "-"
					else
                    		lcd_data 	<= "1010";
					end if;                    				
					state 	<= write4a_I;	
	-- Write the MS digit of minutes to the 4th char. position
          -- Set DDram memory to address 03h                          											
				when write4a_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "1000";
                    	state 	<= write4b_I;				
                	when write4b_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "0011";  
                    	state 	<= write4a_D;
		-- Now set control signal to data and write ascii A                    	
				when write4a_D =>
					lcd_rs 	<= '1';
					lcd_data 	<= "0011"; 											                    		
                    	state 	<= write4b_D;				
                	when write4b_D =>
					lcd_rs 	<= '1';
					if (SW2 = '1') then
						lcd_data 	<= "00" & RTC_date(5 downto 4);
					else
						lcd_data 	<= '0' & RTC_minutes(6 downto 4); 	
					end if;						
					state 	<= write5a_I;
	-- Write the LS digit of minutes to the 5th char. position
          -- Set DDram memory to address 04h                          											
				when write5a_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "1000";
                    	state 	<= write5b_I;				
                	when write5b_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "0100";  
                    	state 	<= write5a_D;
		-- Now set control signal to data and write                    	
				when write5a_D =>
					lcd_rs 	<= '1';
					lcd_data 	<= "0011";					                    		
                    	state 	<= write5b_D;				
                	when write5b_D =>
					lcd_rs 	<= '1';
					if (SW2 = '1') then
						lcd_data 	<= RTC_date(3 downto 0);
					else					
						lcd_data 	<= RTC_minutes(3 downto 0);
					end if;	
					state 	<= write6a_I;
	-- Write a ":" to the 6th char. position
          -- Set DDram memory to address 05h                          											
				when write6a_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "1000";
                    	state 	<= write6b_I;				
                	when write6b_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "0101";  
                    	state 	<= write6a_D;
		-- Now set control signal to data and write ascii : 3a                   	
				when write6a_D =>
					lcd_rs 	<= '1';
 					if (SW2 = '1') then
						lcd_data 	<= "0010";	-- 2D is a "-"
					else
                    		lcd_data 	<= "0011";	
					end if; 											                    		
                    	state 	<= write6b_D;				
                	when write6b_D =>
					lcd_rs 	<= '1';
					if (SW2 = '1') then
						lcd_data 	<= "1101";	-- 2D is a "-"
					else
                    		lcd_data 	<= "1010";
					end if;  		
					state 	<= write7a_I;	
	-- Write the MS digit of seconds to the 7th char. position
          -- Set DDram memory to address 06h                          											
				when write7a_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "1000";
                    	state 	<= write7b_I;				
                	when write7b_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "0110";  
                    	state 	<= write7a_D;
		-- Now set control signal to data and write ascii A                    	
				when write7a_D =>
					lcd_rs 	<= '1';
					lcd_data 	<= "0011"; 											                    		
                    	state 	<= write7b_D;				
                	when write7b_D =>
					lcd_rs 	<= '1';
					if (SW2 = '1') then
						lcd_data 	<= "0000";
					else
						lcd_data 	<= '0' & RTC_seconds(6 downto 4);
					end if;	 	
					state 	<= write8a_I;
	-- Write the LS digit of seconds to the 8th char. position
          -- Set DDram memory to address 07h                          											
				when write8a_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "1000";
                    	state 	<= write8b_I;				
                	when write8b_I =>
					lcd_rs 	<= '0';
                    	lcd_data 	<= "0111";  
                    	state 	<= write8a_D;
		-- Now set control signal to data and write                    	
				when write8a_D =>
					lcd_rs 	<= '1';
					lcd_data 	<= "0011";					                    		
                    	state 	<= write8b_D;				
                	when write8b_D =>
					lcd_rs 	<= '1';
					if (SW2 = '1') then
						lcd_data 	<= RTC_year(3 downto 0);
					else
						lcd_data 	<= RTC_seconds(3 downto 0);
					end if;						
					state 	<= write1a_I;																													
											                  		                
           		when others =>               
			end case;
		end if;
	end process;
	
end behavioural;
