-- Version: 7.0 SP1 7.0.1.14

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity XTAL_RTC is

    port(RTC_MODE : in std_logic_vector(1 downto 0); XTAL1, 
        SELMODE : in std_logic; RTCCLK : out std_logic);

end XTAL_RTC;

architecture DEF_ARCH of XTAL_RTC is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component XTLOSC
    port(XTL : in std_logic := 'U'; CLKOUT : out std_logic; 
        SELMODE : in std_logic := 'U'; MODE : 
        in std_logic_vector(1 downto 0) := (others => 'U'); 
        RTCMODE : 
        in std_logic_vector(1 downto 0) := (others => 'U'));
  end component;

    signal \GND\, \VCC\, GND_net_1, VCC_net_1 : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    crystal : XTLOSC
      port map(XTL => XTAL1, CLKOUT => RTCCLK, SELMODE => SELMODE, 
        MODE(1) => \GND\, MODE(0) => \GND\, RTCMODE(1) => 
        RTC_MODE(1), RTCMODE(0) => RTC_MODE(0));
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity PLL_100_10 is

    port(sys_reset_gl, sys_clk : in std_logic; ACM_CLK_c : out
         std_logic);

end PLL_100_10;

architecture DEF_ARCH of PLL_100_10 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component PLL
    generic (VCOFREQUENCY:real := 0.0);

    port(CLKA, EXTFB, POWERDOWN : in std_logic := 'U'; GLA, LOCK, 
        GLB, YB, GLC, YC : out std_logic; OADIV0, OADIV1, OADIV2, 
        OADIV3, OADIV4, OADIVHALF, OADIVRST, OAMUX0, OAMUX1, 
        OAMUX2, DLYGLA0, DLYGLA1, DLYGLA2, DLYGLA3, DLYGLA4, 
        OBDIV0, OBDIV1, OBDIV2, OBDIV3, OBDIV4, OBMUX0, OBMUX1, 
        OBMUX2, DLYYB0, DLYYB1, DLYYB2, DLYYB3, DLYYB4, DLYGLB0, 
        DLYGLB1, DLYGLB2, DLYGLB3, DLYGLB4, OCDIV0, OCDIV1, 
        OCDIV2, OCDIV3, OCDIV4, OCMUX0, OCMUX1, OCMUX2, DLYYC0, 
        DLYYC1, DLYYC2, DLYYC3, DLYYC4, DLYGLC0, DLYGLC1, DLYGLC2, 
        DLYGLC3, DLYGLC4, FINDIV0, FINDIV1, FINDIV2, FINDIV3, 
        FINDIV4, FINDIV5, FINDIV6, FBDIV0, FBDIV1, FBDIV2, FBDIV3, 
        FBDIV4, FBDIV5, FBDIV6, FBDLY0, FBDLY1, FBDLY2, FBDLY3, 
        FBDLY4, FBSEL0, FBSEL1, XDLYSEL, VCOSEL0, VCOSEL1, 
        VCOSEL2 : in std_logic := 'U');
  end component;

  component GND
    port(Y : out std_logic);
  end component;

    signal Core_GLB, Core_GLC, Core_LOCK, Core_YB, Core_YC, 
        \GND\, \VCC\, GND_net_1, VCC_net_1 : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    Core : PLL
      generic map(VCOFREQUENCY => 100.000000)

      port map(CLKA => sys_clk, EXTFB => \GND\, POWERDOWN => 
        \VCC\, GLA => ACM_CLK_c, LOCK => Core_LOCK, GLB => 
        Core_GLB, YB => Core_YB, GLC => Core_GLC, YC => Core_YC, 
        OADIV0 => \VCC\, OADIV1 => \GND\, OADIV2 => \GND\, OADIV3
         => \VCC\, OADIV4 => \GND\, OADIVHALF => \GND\, OADIVRST
         => sys_reset_gl, OAMUX0 => \GND\, OAMUX1 => \GND\, 
        OAMUX2 => \VCC\, DLYGLA0 => \GND\, DLYGLA1 => \GND\, 
        DLYGLA2 => \GND\, DLYGLA3 => \GND\, DLYGLA4 => \GND\, 
        OBDIV0 => \GND\, OBDIV1 => \GND\, OBDIV2 => \GND\, OBDIV3
         => \GND\, OBDIV4 => \GND\, OBMUX0 => \GND\, OBMUX1 => 
        \GND\, OBMUX2 => \GND\, DLYYB0 => \GND\, DLYYB1 => \GND\, 
        DLYYB2 => \GND\, DLYYB3 => \GND\, DLYYB4 => \GND\, 
        DLYGLB0 => \GND\, DLYGLB1 => \GND\, DLYGLB2 => \GND\, 
        DLYGLB3 => \GND\, DLYGLB4 => \GND\, OCDIV0 => \GND\, 
        OCDIV1 => \GND\, OCDIV2 => \GND\, OCDIV3 => \GND\, OCDIV4
         => \GND\, OCMUX0 => \GND\, OCMUX1 => \GND\, OCMUX2 => 
        \GND\, DLYYC0 => \GND\, DLYYC1 => \GND\, DLYYC2 => \GND\, 
        DLYYC3 => \GND\, DLYYC4 => \GND\, DLYGLC0 => \GND\, 
        DLYGLC1 => \GND\, DLYGLC2 => \GND\, DLYGLC3 => \GND\, 
        DLYGLC4 => \GND\, FINDIV0 => \GND\, FINDIV1 => \VCC\, 
        FINDIV2 => \GND\, FINDIV3 => \GND\, FINDIV4 => \VCC\, 
        FINDIV5 => \GND\, FINDIV6 => \GND\, FBDIV0 => \GND\, 
        FBDIV1 => \VCC\, FBDIV2 => \GND\, FBDIV3 => \GND\, FBDIV4
         => \VCC\, FBDIV5 => \GND\, FBDIV6 => \GND\, FBDLY0 => 
        \GND\, FBDLY1 => \GND\, FBDLY2 => \GND\, FBDLY3 => \GND\, 
        FBDLY4 => \GND\, FBSEL0 => \VCC\, FBSEL1 => \GND\, 
        XDLYSEL => \GND\, VCOSEL0 => \GND\, VCOSEL1 => \GND\, 
        VCOSEL2 => \VCC\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity VR_mon is

    port(PWR_dwn, PUB : in std_logic; PUCORE_c, FPGAGOOD_c : out
         std_logic);

end VR_mon;

architecture DEF_ARCH of VR_mon is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component VRPSM
    port(PUB, VRPU, VRINITSTATE, RTCPSMMATCH : in
         std_logic := 'U'; FPGAGOOD, PUCORE : out std_logic);
  end component;

    signal \GND\, \VCC\, GND_net_1, VCC_net_1 : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    VRPSM1 : VRPSM
      port map(PUB => PUB, VRPU => PWR_dwn, VRINITSTATE => \GND\, 
        RTCPSMMATCH => \GND\, FPGAGOOD => FPGAGOOD_c, PUCORE => 
        PUCORE_c);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv is

    port(net : 
        inout std_logic_vector(7 downto 6) := (others => 'Z'); 
        SW1_c_0 : in std_logic);

end clockdiv;

architecture DEF_ARCH of clockdiv is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \net_i[7]_net_1\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \net_i[7]_net_1\, CLK => net(6), CLR => 
        SW1_c_0, Q => net(7));
    
    \net_i[7]\ : INV
      port map(A => net(7), Y => \net_i[7]_net_1\);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv_11 is

    port(net : 
        inout std_logic_vector(5 downto 4) := (others => 'Z'); 
        SW1_c_1 : in std_logic);

end clockdiv_11;

architecture DEF_ARCH of clockdiv_11 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \net_i[5]_net_1\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \net_i[5]_net_1\, CLK => net(4), CLR => 
        SW1_c_1, Q => net(5));
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    \net_i[5]\ : INV
      port map(A => net(5), Y => \net_i[5]_net_1\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv_8 is

    port(net : 
        inout std_logic_vector(4 downto 3) := (others => 'Z'); 
        SW1_c_1 : in std_logic);

end clockdiv_8;

architecture DEF_ARCH of clockdiv_8 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

    signal \net_i[4]_net_1\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    \net_i[4]\ : INV
      port map(A => net(4), Y => \net_i[4]_net_1\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \net_i[4]_net_1\, CLK => net(3), CLR => 
        SW1_c_1, Q => net(4));
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv_10 is

    port(net : 
        inout std_logic_vector(10 downto 9) := (others => 'Z'); 
        SW1_c_1 : in std_logic);

end clockdiv_10;

architecture DEF_ARCH of clockdiv_10 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \net_i[10]_net_1\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \net_i[10]_net_1\, CLK => net(9), CLR => 
        SW1_c_1, Q => net(10));
    
    \net_i[10]\ : INV
      port map(A => net(10), Y => \net_i[10]_net_1\);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv_7 is

    port(net : 
        inout std_logic_vector(9 downto 8) := (others => 'Z'); 
        SW1_c_1 : in std_logic);

end clockdiv_7;

architecture DEF_ARCH of clockdiv_7 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \net_i[9]_net_1\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \net_i[9]_net_1\, CLK => net(8), CLR => 
        SW1_c_1, Q => net(9));
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    \net_i[9]\ : INV
      port map(A => net(9), Y => \net_i[9]_net_1\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv_4 is

    port(net : 
        inout std_logic_vector(8 downto 7) := (others => 'Z'); 
        SW1_c_0 : in std_logic);

end clockdiv_4;

architecture DEF_ARCH of clockdiv_4 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \net_i[8]_net_1\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \net_i[8]_net_1\, CLK => net(7), CLR => 
        SW1_c_0, Q => net(8));
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    \net_i[8]\ : INV
      port map(A => net(8), Y => \net_i[8]_net_1\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv_1 is

    port(net : 
        inout std_logic_vector(3 downto 2) := (others => 'Z'); 
        SW1_c_0 : in std_logic);

end clockdiv_1;

architecture DEF_ARCH of clockdiv_1 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

    signal \net_i[3]_net_1\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    \net_i[3]\ : INV
      port map(A => net(3), Y => \net_i[3]_net_1\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \net_i[3]_net_1\, CLK => net(2), CLR => 
        SW1_c_0, Q => net(3));
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv_9 is

    port(net : 
        inout std_logic_vector(2 downto 1) := (others => 'Z'); 
        SW1_c_1 : in std_logic);

end clockdiv_9;

architecture DEF_ARCH of clockdiv_9 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

    signal \net_i[2]_net_1\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    \net_i[2]\ : INV
      port map(A => net(2), Y => \net_i[2]_net_1\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \net_i[2]_net_1\, CLK => net(1), CLR => 
        SW1_c_1, Q => net(2));
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv_2 is

    port(net : 
        inout std_logic_vector(11 downto 10) := (others => 'Z'); 
        SW1_c_0 : in std_logic);

end clockdiv_2;

architecture DEF_ARCH of clockdiv_2 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \net_i[11]_net_1\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \net_i[11]_net_1\, CLK => net(10), CLR => 
        SW1_c_0, Q => net(11));
    
    \net_i[11]\ : INV
      port map(A => net(11), Y => \net_i[11]_net_1\);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv_3 is

    port(net : 
        inout std_logic_vector(12 downto 11) := (others => 'Z'); 
        SW1_c_0 : in std_logic);

end clockdiv_3;

architecture DEF_ARCH of clockdiv_3 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

    signal \net_i[12]_net_1\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    \net_i[12]\ : INV
      port map(A => net(12), Y => \net_i[12]_net_1\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \net_i[12]_net_1\, CLK => net(11), CLR => 
        SW1_c_0, Q => net(12));
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv_12 is

    port(net : 
        inout std_logic_vector(6 downto 5) := (others => 'Z'); 
        SW1_c_1 : in std_logic);

end clockdiv_12;

architecture DEF_ARCH of clockdiv_12 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \net_i[6]_net_1\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \net_i[6]_net_1\, CLK => net(5), CLR => 
        SW1_c_1, Q => net(6));
    
    \net_i[6]\ : INV
      port map(A => net(6), Y => \net_i[6]_net_1\);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv_5 is

    port(\net[1]\ : out std_logic; SW1_c_0, ACM_CLK_c : in
         std_logic);

end clockdiv_5;

architecture DEF_ARCH of clockdiv_5 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

    signal \net[1]_net_1\, \net_i[1]_net_1\, \VCC\, \GND\, 
        GND_net_1, VCC_net_1 : std_logic;

begin 

    \net[1]\ <= \net[1]_net_1\;

    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    \net_i[1]\ : INV
      port map(A => \net[1]_net_1\, Y => \net_i[1]_net_1\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \net_i[1]_net_1\, CLK => ACM_CLK_c, CLR => 
        SW1_c_0, Q => \net[1]_net_1\);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity clockdiv_6 is

    port(\net[12]\, SW1_c_1 : in std_logic; lcd_en, lcd_en_0 : 
        out std_logic);

end clockdiv_6;

architecture DEF_ARCH of clockdiv_6 is 

  component VCC
    port(Y : out std_logic);
  end component;

  component BUFF
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

    signal \lcd_en\, \lcd_en_i\, \VCC\, \GND\, GND_net_1, 
        VCC_net_1 : std_logic;

begin 

    lcd_en <= \lcd_en\;

    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    Q_net_0 : BUFF
      port map(A => \lcd_en\, Y => lcd_en_0);
    
    lcd_en_i : INV
      port map(A => \lcd_en\, Y => \lcd_en_i\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Q_net : DFN1C1
      port map(D => \lcd_en_i\, CLK => \net[12]\, CLR => SW1_c_1, 
        Q => \lcd_en\);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity CLK_DIVIDER is

    port(lcd_en_0, lcd_en : out std_logic; SW1_c_1, ACM_CLK_c, 
        SW1_c_0 : in std_logic);

end CLK_DIVIDER;

architecture DEF_ARCH of CLK_DIVIDER is 

  component clockdiv
    port(net : inout std_logic_vector(7 downto 6); SW1_c_0 : in
         std_logic := 'U');
  end component;

  component clockdiv_11
    port(net : inout std_logic_vector(5 downto 4); SW1_c_1 : in
         std_logic := 'U');
  end component;

  component clockdiv_8
    port(net : inout std_logic_vector(4 downto 3); SW1_c_1 : in
         std_logic := 'U');
  end component;

  component clockdiv_10
    port(net : inout std_logic_vector(10 downto 9); SW1_c_1 : in
         std_logic := 'U');
  end component;

  component clockdiv_7
    port(net : inout std_logic_vector(9 downto 8); SW1_c_1 : in
         std_logic := 'U');
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component clockdiv_4
    port(net : inout std_logic_vector(8 downto 7); SW1_c_0 : in
         std_logic := 'U');
  end component;

  component clockdiv_1
    port(net : inout std_logic_vector(3 downto 2); SW1_c_0 : in
         std_logic := 'U');
  end component;

  component clockdiv_9
    port(net : inout std_logic_vector(2 downto 1); SW1_c_1 : in
         std_logic := 'U');
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component clockdiv_2
    port(net : inout std_logic_vector(11 downto 10); SW1_c_0 : in
         std_logic := 'U');
  end component;

  component clockdiv_3
    port(net : inout std_logic_vector(12 downto 11); SW1_c_0 : in
         std_logic := 'U');
  end component;

  component clockdiv_12
    port(net : inout std_logic_vector(6 downto 5); SW1_c_1 : in
         std_logic := 'U');
  end component;

  component clockdiv_5
    port(\net[1]\ : out std_logic; SW1_c_0, ACM_CLK_c : in
         std_logic := 'U');
  end component;

  component clockdiv_6
    port(\net[12]\, SW1_c_1 : in std_logic := 'U'; lcd_en, 
        lcd_en_0 : out std_logic);
  end component;

    signal \net[6]\, \net[7]\, \net[2]\, \net[3]\, \net[10]\, 
        \net[11]\, \net[12]\, \net[8]\, \net[1]\, \net[9]\, 
        \net[4]\, \net[5]\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

    for all : clockdiv
	Use entity work.clockdiv(DEF_ARCH);
    for all : clockdiv_11
	Use entity work.clockdiv_11(DEF_ARCH);
    for all : clockdiv_8
	Use entity work.clockdiv_8(DEF_ARCH);
    for all : clockdiv_10
	Use entity work.clockdiv_10(DEF_ARCH);
    for all : clockdiv_7
	Use entity work.clockdiv_7(DEF_ARCH);
    for all : clockdiv_4
	Use entity work.clockdiv_4(DEF_ARCH);
    for all : clockdiv_1
	Use entity work.clockdiv_1(DEF_ARCH);
    for all : clockdiv_9
	Use entity work.clockdiv_9(DEF_ARCH);
    for all : clockdiv_2
	Use entity work.clockdiv_2(DEF_ARCH);
    for all : clockdiv_3
	Use entity work.clockdiv_3(DEF_ARCH);
    for all : clockdiv_12
	Use entity work.clockdiv_12(DEF_ARCH);
    for all : clockdiv_5
	Use entity work.clockdiv_5(DEF_ARCH);
    for all : clockdiv_6
	Use entity work.clockdiv_6(DEF_ARCH);
begin 


    GEN_label_7_clk_div : clockdiv
      port map(net(7) => \net[7]\, net(6) => \net[6]\, SW1_c_0
         => SW1_c_0);
    
    GEN_label_5_clk_div : clockdiv_11
      port map(net(5) => \net[5]\, net(4) => \net[4]\, SW1_c_1
         => SW1_c_1);
    
    GEN_label_4_clk_div : clockdiv_8
      port map(net(4) => \net[4]\, net(3) => \net[3]\, SW1_c_1
         => SW1_c_1);
    
    GEN_label_10_clk_div : clockdiv_10
      port map(net(10) => \net[10]\, net(9) => \net[9]\, SW1_c_1
         => SW1_c_1);
    
    GEN_label_9_clk_div : clockdiv_7
      port map(net(9) => \net[9]\, net(8) => \net[8]\, SW1_c_1
         => SW1_c_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    GEN_label_8_clk_div : clockdiv_4
      port map(net(8) => \net[8]\, net(7) => \net[7]\, SW1_c_0
         => SW1_c_0);
    
    GEN_label_3_clk_div : clockdiv_1
      port map(net(3) => \net[3]\, net(2) => \net[2]\, SW1_c_0
         => SW1_c_0);
    
    GEN_label_2_clk_div : clockdiv_9
      port map(net(2) => \net[2]\, net(1) => \net[1]\, SW1_c_1
         => SW1_c_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GEN_label_11_clk_div : clockdiv_2
      port map(net(11) => \net[11]\, net(10) => \net[10]\, 
        SW1_c_0 => SW1_c_0);
    
    GND_i : GND
      port map(Y => \GND\);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GEN_label_12_clk_div : clockdiv_3
      port map(net(12) => \net[12]\, net(11) => \net[11]\, 
        SW1_c_0 => SW1_c_0);
    
    GEN_label_6_clk_div : clockdiv_12
      port map(net(6) => \net[6]\, net(5) => \net[5]\, SW1_c_1
         => SW1_c_1);
    
    GEN_label_1_clk_div : clockdiv_5
      port map(\net[1]\ => \net[1]\, SW1_c_0 => SW1_c_0, 
        ACM_CLK_c => ACM_CLK_c);
    
    GEN_label_13_clk_div : clockdiv_6
      port map(\net[12]\ => \net[12]\, SW1_c_1 => SW1_c_1, lcd_en
         => lcd_en, lcd_en_0 => lcd_en_0);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity lcd is

    port(LCD_net : out std_logic_vector(7 downto 4); RTC_hours_c
         : in std_logic_vector(4 downto 0); RTC_year_c : 
        in std_logic_vector(3 downto 0); RTC_seconds_c : 
        in std_logic_vector(6 downto 0); RTC_date_c : 
        in std_logic_vector(5 downto 0); RTC_month_c : 
        in std_logic_vector(7 downto 0); RTC_minutes_c : 
        in std_logic_vector(6 downto 0); lcd_en : in std_logic; 
        lcd_rs : out std_logic; SW2_c, SW2_c_0, SW1_c, SW1_c_1, 
        lcd_en_0 : in std_logic);

end lcd;

architecture DEF_ARCH of lcd is 

  component OA1C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component OR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AOI1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AOI1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XNOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component OAI1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AND2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AXO6
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XAI1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1D
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \state_0[0]_net_1\, \state_ns[0]\, \state_0[1]_net_1\, 
        \state_ns[1]\, \state_0[2]_net_1\, N_207_i_0, 
        \state_0[4]_net_1\, \state_ns[4]\, 
        \DWACT_ADD_CI_0_g_array_1[0]\, \DWACT_ADD_CI_0_TMP[0]\, 
        \count[1]_net_1\, \DWACT_ADD_CI_0_g_array_12[0]\, 
        \count[2]_net_1\, \lcd_data_32_1_iv_11[4]\, 
        \lcd_data_32_1_iv_9[4]\, \lcd_data_0_sqmuxa_7\, 
        \state_d[15]\, \lcd_data_32_1_iv_10[4]\, 
        \lcd_data_32_1_iv_7[4]\, \lcd_data_32_1_iv_6[4]\, 
        \RTC_date_m[0]_net_1\, N_334, \RTC_month_m[4]_net_1\, 
        \lcd_data_32_1_iv_3[4]\, \lcd_data_32_1_iv_2[4]\, 
        \lcd_data_32_1_iv_4[4]\, N_196, \lcd_data_1_sqmuxa_1\, 
        N_429_1, \RTC_seconds_m[0]_net_1\, 
        \RTC_minutes_m[4]_net_1\, N_44_d, \RTC_year_m[0]_net_1\, 
        \RTC_seconds_m[4]_net_1\, \RTC_hours_m[0]_net_1\, N_288, 
        \RTC_date_m[4]_net_1\, \lcd_data_32_1_iv_10[6]\, 
        \lcd_data_32_1_iv_8[6]\, \lcd_data_32_1_iv_9[6]\, 
        \lcd_data_32_1_iv_6[6]\, \lcd_data_32_1_iv_5[6]\, 
        \RTC_date_m[2]_net_1\, \RTC_month_m[6]_net_1\, 
        \state_d[9]\, \lcd_data_32_1_iv_2[6]\, \state_d[35]\, 
        \lcd_data_32_1_iv_1[6]\, \RTC_minutes_m[6]_net_1\, 
        N_360_1, \RTC_year_m[2]_net_1\, \RTC_seconds_m[6]_net_1\, 
        \RTC_hours_m[2]_net_1\, \RTC_seconds_m[2]_net_1\, 
        \lcd_data_32_1_iv_10[5]\, N_328, \lcd_data_32_1_iv_8[5]\, 
        \lcd_data_32_1_iv_9[5]\, \RTC_month_m[5]_net_1\, 
        \lcd_data_32_1_iv_6[5]\, \lcd_data_32_1_iv_5[5]\, N_261, 
        \RTC_minutes_m[5]_net_1\, \lcd_data_32_1_iv_1[5]\, 
        \RTC_date_m[5]_net_1\, \RTC_seconds_m[1]_net_1\, 
        \lcd_data_32_1_iv_3[5]\, \RTC_hours_m[1]_net_1\, 
        \RTC_seconds_m[5]_net_1\, \RTC_year_m[1]_net_1\, N_303, 
        \lcd_data_32_1_iv_16[7]\, \RTC_date_m[3]_net_1\, 
        \RTC_minutes_m[3]_net_1\, \lcd_data_32_1_iv_13[7]\, 
        \lcd_data_32_1_iv_14[7]\, \lcd_data_32_1_iv_9[7]\, 
        \RTC_month_m[7]_net_1\, \lcd_data_32_1_iv_8[7]\, 
        \lcd_data_32_1_iv_10[7]\, \lcd_data_32_1_iv_2[7]\, 
        \lcd_data_32_1_iv_1[7]\, \lcd_data_32_1_iv_7[7]\, 
        \state_s17_0_a3_0\, \state_s29_0_a3_0\, \state_d_1[7]\, 
        \state_d[32]\, \state_d[14]\, \lcd_data_32_1_iv_5[7]\, 
        \RTC_month_m[3]_net_1\, \lcd_data_32_1_iv_0[7]\, 
        \lcd_data_2_sqmuxa\, N_418_1, \RTC_hours_m[3]_net_1\, 
        N_353, \state_d[10]\, N_348, \state_d[22]\, N_43_d_1, 
        N_428, \state_ns_0_0_1[1]_net_1\, N_401_i, 
        \state_ns_0_0_a2_1_0[1]_net_1\, \state_ns_0_0_0[1]_net_1\, 
        N_481_i_0, \state_ns_0_0_a2_0[1]_net_1\, 
        \un1_state_6_0_5\, N_420, N_421, \un1_state_6_0_3\, 
        \un1_state_6_0_4\, N_422, \un1_state_6_0_0\, N_415, 
        N_419_1, N_482, N_475, N_148_i_0, \state[5]_net_1\, 
        \state_ns_0_0_0[3]_net_1\, N_472_i_0_i_0, 
        \state[3]_net_1\, \state_ns_0_0_2[0]_net_1\, 
        \state_ns_0_0_a2_0_0[0]_net_1\, N_442, 
        \state_ns_0_0_1[0]_net_1\, \state_ns_0_0_a2_3_0[0]_net_1\, 
        \state_ns_0_0_0[0]_net_1\, N_404_i_0, 
        \state_ns_i_2[2]_net_1\, \state_ns_i_0[2]_net_1\, 
        \state_ns_i_a3_2[2]_net_1\, \state_ns_i_a3[2]_net_1\, 
        N_493_1, N_425, \state_s28_0_a3_0\, \I_196_0\, 
        un1_state_6_0_a2_0, N_410_i_i_0, \un1_state_6_0_a2_4_0\, 
        N_495, N_237_1, N_500, \I_183_0\, N_319_1, \I_174_0\, 
        \count_5_i_a2_0_3_0[2]_net_1\, N_497, \I_156_0\, 
        \I_136_0\, \I_203_0\, \count[3]_net_1\, N_189, N_493, 
        N_334_1, un4_rtc_hours, un1_state_6, N_448, N_484, N_445, 
        N_492, N_490, N_260, N_263, N_262, 
        \un1_state_2_0_a2_0_a2\, \state_ns[3]\, N_488_1, N_238, 
        N_239, \count_5_i[2]_net_1\, N_423, N_424, I_18, N_429, 
        N_489, N_488, lcd_rs_2, N_483, N_328_1, N_294, 
        \lcd_data_32[4]\, \lcd_data_32[5]\, \lcd_data_32[6]\, 
        \lcd_data_32[7]\, N_281_1, \state[0]_net_1\, 
        \state[1]_net_1\, \state_ns[5]\, N_473, \state[2]_net_1\, 
        \state[4]_net_1\, N_183, N_184, N_213, \count[0]_net_1\, 
        \DWACT_ADD_CI_0_partial_sum[0]\, I_15, I_17, \VCC\, \GND\, 
        GND_net_1, VCC_net_1 : std_logic;

begin 


    \state_ns_0_0_2[0]\ : OA1C
      port map(A => \state_ns_0_0_a2_0_0[0]_net_1\, B => N_401_i, 
        C => N_442, Y => \state_ns_0_0_2[0]_net_1\);
    
    \state_set_lcd_data_32_1_iv_10[7]\ : NOR3C
      port map(A => \lcd_data_32_1_iv_2[7]\, B => 
        \lcd_data_32_1_iv_1[7]\, C => \lcd_data_32_1_iv_7[7]\, Y
         => \lcd_data_32_1_iv_10[7]\);
    
    \count_5_i[2]\ : NOR3C
      port map(A => N_423, B => N_424, C => I_18, Y => 
        \count_5_i[2]_net_1\);
    
    \state_ns_0_o3_1_0_a2[1]\ : OR3
      port map(A => \count[2]_net_1\, B => \count[3]_net_1\, C
         => N_497, Y => N_495);
    
    \state[0]\ : DFN1C1
      port map(D => \state_ns[0]\, CLK => lcd_en, CLR => SW1_c_1, 
        Q => \state[0]_net_1\);
    
    state_set_un4_rtc_hours : OR2
      port map(A => SW2_c, B => RTC_hours_c(4), Y => 
        un4_rtc_hours);
    
    \RTC_date_m[4]\ : OR3B
      port map(A => N_500, B => RTC_date_c(4), C => \I_174_0\, Y
         => \RTC_date_m[4]_net_1\);
    
    I_183_0 : OR2
      port map(A => N_319_1, B => \state_0[0]_net_1\, Y => 
        \I_183_0\);
    
    un1_count_1_I_19 : NOR2B
      port map(A => \DWACT_ADD_CI_0_TMP[0]\, B => 
        \count[1]_net_1\, Y => \DWACT_ADD_CI_0_g_array_1[0]\);
    
    \state_ns_0_0_a2_1_0[1]\ : AO1A
      port map(A => \state[3]_net_1\, B => N_495, C => N_237_1, Y
         => \state_ns_0_0_a2_1_0[1]_net_1\);
    
    state_s6_0_a2_0_a2_0_a2 : NOR2
      port map(A => \state[3]_net_1\, B => \state[1]_net_1\, Y
         => N_500);
    
    \state_ns_0_0_a2_0[1]\ : OR2A
      port map(A => \state[5]_net_1\, B => N_475, Y => 
        \state_ns_0_0_a2_0[1]_net_1\);
    
    \count_5_i_a2_0[2]\ : OR3B
      port map(A => \state_0[0]_net_1\, B => 
        \un1_state_2_0_a2_0_a2\, C => N_495, Y => N_424);
    
    \state_set_lcd_data_32_1_iv_9[5]\ : AOI1B
      port map(A => N_334, B => RTC_minutes_c(1), C => 
        \RTC_month_m[5]_net_1\, Y => \lcd_data_32_1_iv_9[5]\);
    
    I_196_0 : OR2A
      port map(A => \state_0[2]_net_1\, B => SW2_c_0, Y => 
        \I_196_0\);
    
    \lcd_data[4]\ : DFN1C1
      port map(D => \lcd_data_32[4]\, CLK => lcd_en_0, CLR => 
        SW1_c, Q => LCD_net(4));
    
    un1_state_6_0_a2_4 : OR3B
      port map(A => N_445, B => \un1_state_6_0_a2_4_0\, C => 
        N_482, Y => N_420);
    
    \state_ns_i_a3_2_1_0_a2[2]\ : OR2A
      port map(A => \state[0]_net_1\, B => \state[1]_net_1\, Y
         => N_237_1);
    
    \state_set_lcd_data_32_1_iv_10[6]\ : NOR3A
      port map(A => \lcd_data_32_1_iv_8[6]\, B => 
        \lcd_data_0_sqmuxa_7\, C => \state_d[15]\, Y => 
        \lcd_data_32_1_iv_10[6]\);
    
    state_s8_0_a2 : NOR2A
      port map(A => \state[3]_net_1\, B => \state_0[1]_net_1\, Y
         => N_196);
    
    \state_ns_0_0_a2_0_0[0]\ : NOR2A
      port map(A => N_500, B => \state_0[0]_net_1\, Y => 
        \state_ns_0_0_a2_0_0[0]_net_1\);
    
    \state_set_lcd_data_32_1_iv_0[7]\ : AOI1
      port map(A => N_43_d_1, B => N_428, C => N_303, Y => 
        \lcd_data_32_1_iv_0[7]\);
    
    un39_0_a3 : NOR3B
      port map(A => N_500, B => N_148_i_0, C => 
        \state_0[4]_net_1\, Y => N_189);
    
    state_s17_0_a3_0 : NOR2B
      port map(A => \state_0[0]_net_1\, B => N_196, Y => 
        \state_s17_0_a3_0\);
    
    \RTC_hours_m[2]\ : OR2B
      port map(A => RTC_hours_c(2), B => N_294, Y => 
        \RTC_hours_m[2]_net_1\);
    
    I_174_0 : OR3A
      port map(A => SW2_c_0, B => \state_0[0]_net_1\, C => 
        \state_0[2]_net_1\, Y => \I_174_0\);
    
    state_s2_0_a2_0_0_a2 : NOR2A
      port map(A => \state_0[1]_net_1\, B => \state[3]_net_1\, Y
         => N_425);
    
    un1_count_1_I_11 : XOR2
      port map(A => \count[0]_net_1\, B => 
        \un1_state_2_0_a2_0_a2\, Y => 
        \DWACT_ADD_CI_0_partial_sum[0]\);
    
    \state_ns_0_0_a2_4[0]\ : NOR2
      port map(A => \state[0]_net_1\, B => N_482, Y => N_442);
    
    un1_state_6_0_a2_7 : NOR2A
      port map(A => \un1_state_2_0_a2_0_a2\, B => 
        \state[0]_net_1\, Y => N_448);
    
    \state_set_lcd_data_32_1_iv_7[7]\ : NOR3C
      port map(A => \RTC_month_m[3]_net_1\, B => 
        \lcd_data_32_1_iv_0[7]\, C => \lcd_data_2_sqmuxa\, Y => 
        \lcd_data_32_1_iv_7[7]\);
    
    un1_state_6_0_4 : NOR3C
      port map(A => N_422, B => \un1_state_6_0_0\, C => N_415, Y
         => \un1_state_6_0_4\);
    
    I_136_0 : NOR2A
      port map(A => \state_0[2]_net_1\, B => \state_0[1]_net_1\, 
        Y => \I_136_0\);
    
    state_s30_0_a3_1_0_a2 : OR2A
      port map(A => N_334_1, B => \state[2]_net_1\, Y => 
        \state_d_1[7]\);
    
    \state_ns_0_0[4]\ : OR3
      port map(A => N_492, B => N_334_1, C => N_490, Y => 
        \state_ns[4]\);
    
    \state[4]\ : DFN1C1
      port map(D => \state_ns[4]\, CLK => lcd_en, CLR => SW1_c, Q
         => \state[4]_net_1\);
    
    state_s29_0_a3_0 : NOR2B
      port map(A => \state_0[0]_net_1\, B => N_425, Y => 
        \state_s29_0_a3_0\);
    
    state_set_lcd_rs_2_0_0_a2 : OR2B
      port map(A => \state[3]_net_1\, B => \state[1]_net_1\, Y
         => N_482);
    
    \RTC_month_m[5]\ : OR3B
      port map(A => SW2_c_0, B => RTC_month_c(5), C => N_281_1, Y
         => \RTC_month_m[5]_net_1\);
    
    \RTC_hours_m[0]\ : OR2B
      port map(A => RTC_hours_c(0), B => N_294, Y => 
        \RTC_hours_m[0]_net_1\);
    
    \RTC_seconds_m[0]\ : OR2B
      port map(A => RTC_seconds_c(0), B => N_353, Y => 
        \RTC_seconds_m[0]_net_1\);
    
    \RTC_date_m[0]\ : OR2B
      port map(A => RTC_date_c(0), B => N_328, Y => 
        \RTC_date_m[0]_net_1\);
    
    un1_state_6_0_x2 : XNOR2
      port map(A => \state_0[2]_net_1\, B => \state[1]_net_1\, Y
         => N_410_i_i_0);
    
    \lcd_data[7]\ : DFN1C1
      port map(D => \lcd_data_32[7]\, CLK => lcd_en_0, CLR => 
        SW1_c, Q => LCD_net(7));
    
    \state_set_lcd_data_32_1_iv_5[5]\ : NOR3C
      port map(A => \RTC_date_m[5]_net_1\, B => 
        \RTC_seconds_m[1]_net_1\, C => \lcd_data_32_1_iv_3[5]\, Y
         => \lcd_data_32_1_iv_5[5]\);
    
    \state_set_lcd_data_32_1_iv_10[4]\ : NOR3C
      port map(A => \lcd_data_32_1_iv_7[4]\, B => 
        \lcd_data_32_1_iv_6[4]\, C => \RTC_date_m[0]_net_1\, Y
         => \lcd_data_32_1_iv_10[4]\);
    
    \RTC_year_m[1]\ : OR2B
      port map(A => RTC_year_c(1), B => N_348, Y => 
        \RTC_year_m[1]_net_1\);
    
    \state[5]\ : DFN1C1
      port map(D => \state_ns[5]\, CLK => lcd_en, CLR => SW1_c, Q
         => \state[5]_net_1\);
    
    \state_set_lcd_data_32_1_iv_1[6]\ : AOI1B
      port map(A => N_288, B => RTC_month_c(2), C => 
        \RTC_seconds_m[2]_net_1\, Y => \lcd_data_32_1_iv_1[6]\);
    
    \RTC_month_m[4]\ : OR3B
      port map(A => SW2_c_0, B => RTC_month_c(4), C => N_281_1, Y
         => \RTC_month_m[4]_net_1\);
    
    \state_ns_i_a3_3[2]\ : OR3A
      port map(A => N_425, B => \state_0[0]_net_1\, C => 
        \state_0[4]_net_1\, Y => N_238);
    
    GND_i : GND
      port map(Y => \GND\);
    
    \state_set_lcd_data_32_1_iv_2[6]\ : NOR3C
      port map(A => \RTC_year_m[2]_net_1\, B => 
        \RTC_seconds_m[6]_net_1\, C => \RTC_hours_m[2]_net_1\, Y
         => \lcd_data_32_1_iv_2[6]\);
    
    \state_ns_0_0_i2_i_x2[0]\ : XNOR2
      port map(A => \state[4]_net_1\, B => \state[2]_net_1\, Y
         => N_472_i_0_i_0);
    
    \RTC_minutes_m[3]\ : OR2B
      port map(A => RTC_minutes_c(3), B => N_334, Y => 
        \RTC_minutes_m[3]_net_1\);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    \count[0]\ : DFN1C1
      port map(D => \DWACT_ADD_CI_0_partial_sum[0]\, CLK => 
        lcd_en, CLR => SW1_c, Q => \count[0]_net_1\);
    
    \state_set_lcd_data_32_1_iv_6[4]\ : OAI1
      port map(A => N_196, B => \lcd_data_1_sqmuxa_1\, C => 
        N_429_1, Y => \lcd_data_32_1_iv_6[4]\);
    
    \state_ns_0_0_1[1]\ : OA1A
      port map(A => N_401_i, B => \state_ns_0_0_a2_1_0[1]_net_1\, 
        C => \state_ns_0_0_0[1]_net_1\, Y => 
        \state_ns_0_0_1[1]_net_1\);
    
    \state_set_lcd_data_32_1_iv_8[7]\ : NOR3C
      port map(A => \state_d[32]\, B => \state_d[14]\, C => 
        \lcd_data_32_1_iv_5[7]\, Y => \lcd_data_32_1_iv_8[7]\);
    
    un1_state_6_0_a2_4_0 : NOR2A
      port map(A => \state_0[0]_net_1\, B => SW2_c_0, Y => 
        \un1_state_6_0_a2_4_0\);
    
    \state_set_lcd_data_32_1_iv_4[4]\ : NOR3B
      port map(A => \RTC_seconds_m[0]_net_1\, B => 
        \RTC_minutes_m[4]_net_1\, C => N_44_d, Y => 
        \lcd_data_32_1_iv_4[4]\);
    
    \state_set_lcd_data_32_1_iv_9[4]\ : AOI1B
      port map(A => N_334, B => RTC_minutes_c(0), C => 
        \RTC_month_m[4]_net_1\, Y => \lcd_data_32_1_iv_9[4]\);
    
    \state_set_lcd_data_32_1_iv_2[4]\ : AOI1B
      port map(A => N_288, B => RTC_month_c(0), C => 
        \RTC_date_m[4]_net_1\, Y => \lcd_data_32_1_iv_2[4]\);
    
    N_148_i_0_a2 : NOR2A
      port map(A => \state_0[2]_net_1\, B => \state[0]_net_1\, Y
         => N_148_i_0);
    
    \state_ns_0_0_0[1]\ : AOI1B
      port map(A => N_481_i_0, B => \state_0[1]_net_1\, C => 
        \state_ns_0_0_a2_0[1]_net_1\, Y => 
        \state_ns_0_0_0[1]_net_1\);
    
    un1_count_1_I_15 : XOR2
      port map(A => \count[1]_net_1\, B => 
        \DWACT_ADD_CI_0_TMP[0]\, Y => I_15);
    
    \RTC_year_m[2]\ : OR2B
      port map(A => RTC_year_c(2), B => N_348, Y => 
        \RTC_year_m[2]_net_1\);
    
    \state_set_lcd_data_32_1_iv_1[7]\ : AOI1B
      port map(A => N_348, B => RTC_year_c(3), C => \state_d[22]\, 
        Y => \lcd_data_32_1_iv_1[7]\);
    
    \state_set_lcd_data_32_1_iv_8[5]\ : NOR3C
      port map(A => \lcd_data_32_1_iv_6[5]\, B => 
        \lcd_data_32_1_iv_5[5]\, C => N_261, Y => 
        \lcd_data_32_1_iv_8[5]\);
    
    \state_set_lcd_data_32_1_iv_8[6]\ : AOI1B
      port map(A => N_334, B => RTC_minutes_c(2), C => 
        \RTC_month_m[6]_net_1\, Y => \lcd_data_32_1_iv_8[6]\);
    
    un1_state_6_0_a2_5 : OR3C
      port map(A => N_401_i, B => un4_rtc_hours, C => N_442, Y
         => N_421);
    
    \RTC_minutes_m[6]\ : OR3B
      port map(A => N_500, B => RTC_minutes_c(6), C => \I_183_0\, 
        Y => \RTC_minutes_m[6]_net_1\);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    \state_ns_0_o3_0_i_a2[0]\ : OR2B
      port map(A => \count[1]_net_1\, B => \count[0]_net_1\, Y
         => N_497);
    
    \RTC_minutes_m[4]\ : OR3B
      port map(A => N_500, B => RTC_minutes_c(4), C => \I_183_0\, 
        Y => \RTC_minutes_m[4]_net_1\);
    
    \state_ns_0_0[0]\ : OR3C
      port map(A => \state_ns_0_0_2[0]_net_1\, B => 
        \state_ns_0_0_1[0]_net_1\, C => N_429, Y => \state_ns[0]\);
    
    \RTC_month_m[3]\ : OR2B
      port map(A => RTC_month_c(3), B => N_288, Y => 
        \RTC_month_m[3]_net_1\);
    
    lcd_data_0_sqmuxa_7 : AOI1B
      port map(A => N_183, B => N_184, C => SW2_c, Y => 
        \lcd_data_0_sqmuxa_7\);
    
    state_set_lcd_rs_2_0_0 : OR3C
      port map(A => N_482, B => N_483, C => N_484, Y => lcd_rs_2);
    
    \state_set_lcd_data_32_1_iv_5[7]\ : AOI1B
      port map(A => N_196, B => N_418_1, C => 
        \RTC_hours_m[3]_net_1\, Y => \lcd_data_32_1_iv_5[7]\);
    
    \state_set_lcd_data_32_0_1[6]\ : NOR3
      port map(A => N_189, B => N_493, C => N_44_d, Y => N_360_1);
    
    \state_set_lcd_data_32_1_iv_5[6]\ : NOR3C
      port map(A => \lcd_data_32_1_iv_1[6]\, B => 
        \RTC_minutes_m[6]_net_1\, C => N_360_1, Y => 
        \lcd_data_32_1_iv_5[6]\);
    
    \state_ns_i_a3_0_i_x2[2]\ : XOR2
      port map(A => \state[3]_net_1\, B => \state_0[1]_net_1\, Y
         => N_404_i_0);
    
    \state_ns_0_0_o2_i_a2[0]\ : NOR2
      port map(A => \state[4]_net_1\, B => \state[2]_net_1\, Y
         => N_401_i);
    
    \state_set_lcd_data_32_1_iv_7[4]\ : NOR3C
      port map(A => \lcd_data_32_1_iv_3[4]\, B => 
        \lcd_data_32_1_iv_2[4]\, C => \lcd_data_32_1_iv_4[4]\, Y
         => \lcd_data_32_1_iv_7[4]\);
    
    \state_ns_0_0_0[0]\ : AOI1
      port map(A => N_404_i_0, B => N_472_i_0_i_0, C => N_428, Y
         => \state_ns_0_0_0[0]_net_1\);
    
    un1_state_0_a3 : OR3C
      port map(A => N_473, B => \state[4]_net_1\, C => N_500, Y
         => N_183);
    
    \RTC_hours_m[1]\ : OR2B
      port map(A => RTC_hours_c(1), B => N_294, Y => 
        \RTC_hours_m[1]_net_1\);
    
    \count_5_i_a2[2]\ : OR3B
      port map(A => \count_5_i_a2_0_3_0[2]_net_1\, B => N_448, C
         => N_497, Y => N_423);
    
    \lcd_data[5]\ : DFN1C1
      port map(D => \lcd_data_32[5]\, CLK => lcd_en_0, CLR => 
        SW1_c, Q => LCD_net(5));
    
    I_203_0 : OR2A
      port map(A => \state[5]_net_1\, B => SW2_c_0, Y => 
        \I_203_0\);
    
    \state_ns_i_2[2]\ : NOR3C
      port map(A => \state_ns_i_0[2]_net_1\, B => 
        \state_ns_i_a3_2[2]_net_1\, C => \state_ns_i_a3[2]_net_1\, 
        Y => \state_ns_i_2[2]_net_1\);
    
    un1_state_6_0_a2_3_1 : NOR2B
      port map(A => \state_0[4]_net_1\, B => N_148_i_0, Y => 
        N_419_1);
    
    I_208 : NOR2A
      port map(A => SW2_c, B => N_483, Y => N_348);
    
    \state_0[4]\ : DFN1C1
      port map(D => \state_ns[4]\, CLK => lcd_en_0, CLR => SW1_c, 
        Q => \state_0[4]_net_1\);
    
    I_189_1 : OR2B
      port map(A => SW2_c, B => \state[2]_net_1\, Y => N_328_1);
    
    I_131 : OR3B
      port map(A => N_263, B => N_262, C => 
        \un1_state_2_0_a2_0_a2\, Y => N_260);
    
    \count[1]\ : DFN1C1
      port map(D => I_15, CLK => lcd_en_0, CLR => SW1_c, Q => 
        \count[1]_net_1\);
    
    \state_ns_0_0_1[0]\ : AOI1B
      port map(A => N_401_i, B => \state_ns_0_0_a2_3_0[0]_net_1\, 
        C => \state_ns_0_0_0[0]_net_1\, Y => 
        \state_ns_0_0_1[0]_net_1\);
    
    \lcd_rs\ : DFN1C1
      port map(D => lcd_rs_2, CLK => lcd_en, CLR => SW1_c_1, Q
         => lcd_rs);
    
    I_143_1 : OR2A
      port map(A => N_401_i, B => N_482, Y => N_281_1);
    
    \count[3]\ : DFN1C1
      port map(D => I_17, CLK => lcd_en_0, CLR => SW1_c, Q => 
        \count[3]_net_1\);
    
    un1_count_1_I_1 : AND2
      port map(A => \count[0]_net_1\, B => 
        \un1_state_2_0_a2_0_a2\, Y => \DWACT_ADD_CI_0_TMP[0]\);
    
    un1_count_1_I_17 : XOR2
      port map(A => \count[3]_net_1\, B => 
        \DWACT_ADD_CI_0_g_array_12[0]\, Y => I_17);
    
    I_183_1 : OR2A
      port map(A => \state[4]_net_1\, B => SW2_c, Y => N_319_1);
    
    un1_state_6_0_a2_0_0 : NOR2A
      port map(A => N_410_i_i_0, B => \state[3]_net_1\, Y => 
        un1_state_6_0_a2_0);
    
    \state_set_lcd_data_32_1_iv_6[5]\ : NOR3C
      port map(A => \RTC_minutes_m[5]_net_1\, B => 
        \lcd_data_32_1_iv_1[5]\, C => N_360_1, Y => 
        \lcd_data_32_1_iv_6[5]\);
    
    I_156_0 : OR3A
      port map(A => \state_0[2]_net_1\, B => \state_0[4]_net_1\, 
        C => SW2_c_0, Y => \I_156_0\);
    
    \state_set_lcd_data_32_1_iv_3[5]\ : AOI1B
      port map(A => N_288, B => RTC_month_c(1), C => 
        \RTC_hours_m[1]_net_1\, Y => \lcd_data_32_1_iv_3[5]\);
    
    \state_ns_0_0_a2_3_0[0]\ : NOR3A
      port map(A => \count_5_i_a2_0_3_0[2]_net_1\, B => N_497, C
         => \state_0[1]_net_1\, Y => 
        \state_ns_0_0_a2_3_0[0]_net_1\);
    
    state_s22_0_a3 : NOR3C
      port map(A => N_473, B => \state_0[4]_net_1\, C => N_196, Y
         => \state_d[15]\);
    
    \state_ns_0_0[3]\ : AO1B
      port map(A => N_488_1, B => \state_0[1]_net_1\, C => 
        \state_ns_0_0_0[3]_net_1\, Y => \state_ns[3]\);
    
    \RTC_hours_m[3]\ : OR2B
      port map(A => RTC_hours_c(3), B => N_294, Y => 
        \RTC_hours_m[3]_net_1\);
    
    un1_state_0_a3_0 : OR2B
      port map(A => N_445, B => N_442, Y => N_184);
    
    state_s28_0_a3_0 : NOR2A
      port map(A => N_425, B => \state_0[0]_net_1\, Y => 
        \state_s28_0_a3_0\);
    
    \state_ns_0_0[5]\ : OR2
      port map(A => N_493, B => N_490, Y => \state_ns[5]\);
    
    I_165 : NOR3
      port map(A => N_410_i_i_0, B => N_319_1, C => N_404_i_0, Y
         => N_303);
    
    \state_ns_0_0_a2[5]\ : NOR2B
      port map(A => N_493_1, B => N_475, Y => N_493);
    
    \state_ns_0_0_a2_1[4]\ : NOR3C
      port map(A => \state_0[2]_net_1\, B => \state[3]_net_1\, C
         => N_475, Y => N_492);
    
    \state_set_lcd_data_32_1_iv_13[7]\ : NOR3C
      port map(A => \lcd_data_32_1_iv_8[7]\, B => \state_d[35]\, 
        C => \lcd_data_32_1_iv_10[7]\, Y => 
        \lcd_data_32_1_iv_13[7]\);
    
    state_s34_0_a3 : NOR3C
      port map(A => \state_0[0]_net_1\, B => \state[5]_net_1\, C
         => N_43_d_1, Y => N_44_d);
    
    I_149 : NOR3
      port map(A => N_328_1, B => \state_0[4]_net_1\, C => N_482, 
        Y => N_288);
    
    un1_state_6_0 : OR3B
      port map(A => \un1_state_6_0_5\, B => \un1_state_6_0_4\, C
         => N_448, Y => un1_state_6);
    
    \state_ns_i[2]\ : NOR3C
      port map(A => \state_ns_i_2[2]_net_1\, B => N_238, C => 
        N_239, Y => N_207_i_0);
    
    \state_ns_0_0_0[3]\ : OAI1
      port map(A => \state_0[0]_net_1\, B => N_472_i_0_i_0, C => 
        \state[3]_net_1\, Y => \state_ns_0_0_0[3]_net_1\);
    
    un1_state_6_0_5 : NOR3C
      port map(A => N_420, B => N_421, C => \un1_state_6_0_3\, Y
         => \un1_state_6_0_5\);
    
    \state_set_lcd_data_32_1_iv_1[5]\ : NOR3B
      port map(A => \RTC_seconds_m[5]_net_1\, B => 
        \RTC_year_m[1]_net_1\, C => N_303, Y => 
        \lcd_data_32_1_iv_1[5]\);
    
    \state_ns_0_0_a2_2[1]\ : OR2A
      port map(A => N_488_1, B => \state[3]_net_1\, Y => N_488);
    
    \state[3]\ : DFN1C1
      port map(D => \state_ns[3]\, CLK => lcd_en, CLR => SW1_c_1, 
        Q => \state[3]_net_1\);
    
    \state_0[1]\ : DFN1C1
      port map(D => \state_ns[1]\, CLK => lcd_en_0, CLR => 
        SW1_c_1, Q => \state_0[1]_net_1\);
    
    \state_ns_i_0[2]\ : OA1B
      port map(A => N_404_i_0, B => \state_0[2]_net_1\, C => 
        N_493_1, Y => \state_ns_i_0[2]_net_1\);
    
    state_s27_0_a3 : OR3C
      port map(A => N_334_1, B => N_148_i_0, C => N_425, Y => 
        \state_d[10]\);
    
    \state_ns_0_o3_i_a2_0_o2[4]\ : NOR2B
      port map(A => \state[2]_net_1\, B => \state[0]_net_1\, Y
         => N_473);
    
    state_s35_0_a3_1 : NOR2B
      port map(A => \state[1]_net_1\, B => N_445, Y => N_43_d_1);
    
    \state_ns_0_0_a2_3[1]\ : OR3B
      port map(A => N_473, B => \state[4]_net_1\, C => 
        \state[1]_net_1\, Y => N_489);
    
    lcd_data_2_sqmuxa : OR2
      port map(A => un4_rtc_hours, B => N_281_1, Y => 
        \lcd_data_2_sqmuxa\);
    
    un1_state_6_0_a2_6 : OR3A
      port map(A => N_148_i_0, B => SW2_c_0, C => N_484, Y => 
        N_422);
    
    \state_ns_0_0_x2[1]\ : XOR2
      port map(A => \state[0]_net_1\, B => N_472_i_0_i_0, Y => 
        N_481_i_0);
    
    \state_ns_0_0_a2_1[5]\ : NOR3B
      port map(A => \state[3]_net_1\, B => \state_0[4]_net_1\, C
         => \state[2]_net_1\, Y => N_493_1);
    
    state_s15_0_a3 : OR3B
      port map(A => N_401_i, B => N_196, C => \state_0[0]_net_1\, 
        Y => \state_d[22]\);
    
    \RTC_date_m[2]\ : OR2B
      port map(A => RTC_date_c(2), B => N_328, Y => 
        \RTC_date_m[2]_net_1\);
    
    I_189 : NOR3A
      port map(A => N_334_1, B => N_482, C => N_328_1, Y => N_328);
    
    \state_ns_0_0_a2_1[0]\ : NOR2A
      port map(A => \state[5]_net_1\, B => \state[0]_net_1\, Y
         => N_428);
    
    I_196 : NOR3A
      port map(A => N_334_1, B => N_482, C => \I_196_0\, Y => 
        N_334);
    
    \state_set_lcd_data_32_r[6]\ : AOI1
      port map(A => \lcd_data_32_1_iv_9[6]\, B => 
        \lcd_data_32_1_iv_10[6]\, C => un1_state_6, Y => 
        \lcd_data_32[6]\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    \RTC_month_m[6]\ : OR3B
      port map(A => SW2_c, B => RTC_month_c(6), C => N_281_1, Y
         => \RTC_month_m[6]_net_1\);
    
    \RTC_seconds_m[6]\ : OR3A
      port map(A => RTC_seconds_c(6), B => \state_0[1]_net_1\, C
         => \I_203_0\, Y => \RTC_seconds_m[6]_net_1\);
    
    \state_ns_i_a3_4[2]\ : OR3C
      port map(A => N_334_1, B => \state[0]_net_1\, C => N_425, Y
         => N_239);
    
    un1_count_1_I_18 : XOR2
      port map(A => \count[2]_net_1\, B => 
        \DWACT_ADD_CI_0_g_array_1[0]\, Y => I_18);
    
    \state_0[0]\ : DFN1C1
      port map(D => \state_ns[0]\, CLK => lcd_en_0, CLR => 
        SW1_c_1, Q => \state_0[0]_net_1\);
    
    state_s17_0_a2_0_a2 : NOR2A
      port map(A => \state[4]_net_1\, B => \state[2]_net_1\, Y
         => N_445);
    
    \state_ns_0_0_a2_2_1[0]\ : NOR2B
      port map(A => \state[0]_net_1\, B => N_401_i, Y => N_429_1);
    
    \state_ns_0_o3[1]\ : AXO6
      port map(A => \state[5]_net_1\, B => \state[2]_net_1\, C
         => \state[4]_net_1\, Y => N_213);
    
    un1_state_6_0_a2_2_1 : NOR2A
      port map(A => N_473, B => \state[4]_net_1\, Y => N_418_1);
    
    \state_ns_0_0_a2_0[4]\ : NOR2A
      port map(A => \state_0[4]_net_1\, B => \state[5]_net_1\, Y
         => N_334_1);
    
    \state_ns_i_a3[2]\ : OR3
      port map(A => \state[0]_net_1\, B => \state_0[2]_net_1\, C
         => N_334_1, Y => \state_ns_i_a3[2]_net_1\);
    
    \RTC_seconds_m[1]\ : OR2B
      port map(A => RTC_seconds_c(1), B => N_353, Y => 
        \RTC_seconds_m[1]_net_1\);
    
    \state_set_lcd_data_32_1_iv_16[7]\ : NOR3C
      port map(A => \RTC_date_m[3]_net_1\, B => 
        \RTC_minutes_m[3]_net_1\, C => \lcd_data_32_1_iv_13[7]\, 
        Y => \lcd_data_32_1_iv_16[7]\);
    
    I_130 : XAI1
      port map(A => \state_0[0]_net_1\, B => \state_0[2]_net_1\, 
        C => N_260, Y => N_261);
    
    \state_set_lcd_data_32_1_iv_3[4]\ : NOR3C
      port map(A => \RTC_year_m[0]_net_1\, B => 
        \RTC_seconds_m[4]_net_1\, C => \RTC_hours_m[0]_net_1\, Y
         => \lcd_data_32_1_iv_3[4]\);
    
    \state[2]\ : DFN1C1
      port map(D => N_207_i_0, CLK => lcd_en, CLR => SW1_c_1, Q
         => \state[2]_net_1\);
    
    \state_ns_0_0_o2[1]\ : NOR2
      port map(A => \state[1]_net_1\, B => \state[0]_net_1\, Y
         => N_475);
    
    \RTC_seconds_m[5]\ : OR3A
      port map(A => RTC_seconds_c(5), B => \state_0[1]_net_1\, C
         => \I_203_0\, Y => \RTC_seconds_m[5]_net_1\);
    
    state_s2_0_a3 : OR2B
      port map(A => N_429_1, B => N_425, Y => \state_d[35]\);
    
    \state_ns_0_0[1]\ : OR3C
      port map(A => N_489, B => \state_ns_0_0_1[1]_net_1\, C => 
        N_488, Y => \state_ns[1]\);
    
    \RTC_date_m[3]\ : OR2B
      port map(A => RTC_date_c(3), B => N_328, Y => 
        \RTC_date_m[3]_net_1\);
    
    \state_0[2]\ : DFN1C1
      port map(D => N_207_i_0, CLK => lcd_en_0, CLR => SW1_c_1, Q
         => \state_0[2]_net_1\);
    
    \RTC_date_m[5]\ : OR3B
      port map(A => N_500, B => RTC_date_c(5), C => \I_174_0\, Y
         => \RTC_date_m[5]_net_1\);
    
    \state_set_lcd_data_32_1_iv_9[7]\ : AO1D
      port map(A => \state_s17_0_a3_0\, B => \state_s29_0_a3_0\, 
        C => \state_d_1[7]\, Y => \lcd_data_32_1_iv_9[7]\);
    
    state_s23_0_a3 : OR2B
      port map(A => N_419_1, B => N_196, Y => \state_d[14]\);
    
    \state[1]\ : DFN1C1
      port map(D => \state_ns[1]\, CLK => lcd_en, CLR => SW1_c_1, 
        Q => \state[1]_net_1\);
    
    I_214 : NOR2
      port map(A => SW2_c, B => N_483, Y => N_353);
    
    un1_state_6_0_0 : OAI1
      port map(A => N_475, B => N_148_i_0, C => \state[5]_net_1\, 
        Y => \un1_state_6_0_0\);
    
    \state_set_lcd_data_32_1_iv_10[5]\ : AOI1B
      port map(A => N_328, B => RTC_date_c(1), C => 
        \lcd_data_32_1_iv_8[5]\, Y => \lcd_data_32_1_iv_10[5]\);
    
    \state_set_lcd_data_32_1_iv_2[7]\ : AOI1B
      port map(A => N_353, B => RTC_seconds_c(3), C => 
        \state_d[10]\, Y => \lcd_data_32_1_iv_2[7]\);
    
    I_156 : NOR2
      port map(A => \I_156_0\, B => N_482, Y => N_294);
    
    I_120 : OR2B
      port map(A => \state[1]_net_1\, B => N_493_1, Y => N_263);
    
    \RTC_year_m[0]\ : OR2B
      port map(A => RTC_year_c(0), B => N_348, Y => 
        \RTC_year_m[0]_net_1\);
    
    state_s5_0_a3 : OR2B
      port map(A => N_425, B => N_418_1, Y => \state_d[32]\);
    
    I_136 : XAI1
      port map(A => \state[3]_net_1\, B => \state_0[4]_net_1\, C
         => \I_136_0\, Y => N_262);
    
    state_set_lcd_rs_2_0_0_a2_0 : OR2B
      port map(A => \state[5]_net_1\, B => \state[2]_net_1\, Y
         => N_483);
    
    \state_ns_i_a3_2[2]\ : OR3B
      port map(A => \state[3]_net_1\, B => \state_0[4]_net_1\, C
         => N_237_1, Y => \state_ns_i_a3_2[2]_net_1\);
    
    \state_set_lcd_data_32_r[7]\ : AOI1
      port map(A => \lcd_data_32_1_iv_14[7]\, B => 
        \lcd_data_32_1_iv_16[7]\, C => un1_state_6, Y => 
        \lcd_data_32[7]\);
    
    \state_set_lcd_data_32_1_iv_6[6]\ : NOR3C
      port map(A => \state_d[9]\, B => \lcd_data_32_1_iv_2[6]\, C
         => \state_d[35]\, Y => \lcd_data_32_1_iv_6[6]\);
    
    \state_ns_0_0_a2_2[0]\ : OR3B
      port map(A => N_495, B => N_429_1, C => \state[3]_net_1\, Y
         => N_429);
    
    \RTC_seconds_m[2]\ : OR2B
      port map(A => RTC_seconds_c(2), B => N_353, Y => 
        \RTC_seconds_m[2]_net_1\);
    
    \lcd_data[6]\ : DFN1C1
      port map(D => \lcd_data_32[6]\, CLK => lcd_en_0, CLR => 
        SW1_c, Q => LCD_net(6));
    
    un1_state_6_0_a2 : OR3C
      port map(A => N_334_1, B => \state_0[0]_net_1\, C => 
        un1_state_6_0_a2_0, Y => N_415);
    
    un1_state_2_0_a2_0_a2 : NOR2B
      port map(A => N_500, B => N_401_i, Y => 
        \un1_state_2_0_a2_0_a2\);
    
    \count[2]\ : DFN1C1
      port map(D => \count_5_i[2]_net_1\, CLK => lcd_en_0, CLR
         => SW1_c, Q => \count[2]_net_1\);
    
    \state_ns_0_0_a2_2_1[1]\ : NOR2
      port map(A => \state[0]_net_1\, B => N_213, Y => N_488_1);
    
    \count_5_i_a2_0_3_0[2]\ : NOR2B
      port map(A => \count[3]_net_1\, B => \count[2]_net_1\, Y
         => \count_5_i_a2_0_3_0[2]_net_1\);
    
    \state_ns_0_0_a2[4]\ : NOR2A
      port map(A => \state[5]_net_1\, B => N_473, Y => N_490);
    
    \state_set_lcd_data_32_1_iv_14[7]\ : NOR3B
      port map(A => \lcd_data_32_1_iv_9[7]\, B => 
        \RTC_month_m[7]_net_1\, C => \lcd_data_0_sqmuxa_7\, Y => 
        \lcd_data_32_1_iv_14[7]\);
    
    lcd_data_1_sqmuxa_1 : NOR3A
      port map(A => RTC_hours_c(4), B => SW2_c_0, C => N_482, Y
         => \lcd_data_1_sqmuxa_1\);
    
    \RTC_month_m[7]\ : OR3B
      port map(A => SW2_c, B => RTC_month_c(7), C => N_281_1, Y
         => \RTC_month_m[7]_net_1\);
    
    \state_set_lcd_data_32_1_iv_11[4]\ : NOR3A
      port map(A => \lcd_data_32_1_iv_9[4]\, B => 
        \lcd_data_0_sqmuxa_7\, C => \state_d[15]\, Y => 
        \lcd_data_32_1_iv_11[4]\);
    
    \state_set_lcd_data_32_s[4]\ : OR3B
      port map(A => \lcd_data_32_1_iv_10[4]\, B => 
        \lcd_data_32_1_iv_11[4]\, C => un1_state_6, Y => 
        \lcd_data_32[4]\);
    
    un1_count_1_I_21 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_1[0]\, B => 
        \count[2]_net_1\, Y => \DWACT_ADD_CI_0_g_array_12[0]\);
    
    \state_set_lcd_data_32_1_iv_9[6]\ : NOR3C
      port map(A => \lcd_data_32_1_iv_6[6]\, B => 
        \lcd_data_32_1_iv_5[6]\, C => \RTC_date_m[2]_net_1\, Y
         => \lcd_data_32_1_iv_9[6]\);
    
    \state_set_lcd_data_32_s[5]\ : OR3B
      port map(A => \lcd_data_32_1_iv_9[5]\, B => 
        \lcd_data_32_1_iv_10[5]\, C => un1_state_6, Y => 
        \lcd_data_32[5]\);
    
    state_s28_0_a3 : OR2A
      port map(A => \state_s28_0_a3_0\, B => \state_d_1[7]\, Y
         => \state_d[9]\);
    
    state_set_lcd_rs_2_0_0_a2_1 : OR2B
      port map(A => \state[4]_net_1\, B => N_500, Y => N_484);
    
    un1_state_6_0_3 : AO1D
      port map(A => N_419_1, B => N_418_1, C => N_482, Y => 
        \un1_state_6_0_3\);
    
    \RTC_minutes_m[5]\ : OR3B
      port map(A => N_500, B => RTC_minutes_c(5), C => \I_183_0\, 
        Y => \RTC_minutes_m[5]_net_1\);
    
    \RTC_seconds_m[4]\ : OR3A
      port map(A => RTC_seconds_c(4), B => \state_0[1]_net_1\, C
         => \I_203_0\, Y => \RTC_seconds_m[4]_net_1\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity LCD_DIS is

    port(RTC_minutes_c : in std_logic_vector(6 downto 0); 
        RTC_month_c : in std_logic_vector(7 downto 0); RTC_date_c
         : in std_logic_vector(5 downto 0); RTC_seconds_c : 
        in std_logic_vector(6 downto 0); RTC_year_c : 
        in std_logic_vector(3 downto 0); RTC_hours_c : 
        in std_logic_vector(4 downto 0); DATA_LCD_c : 
        out std_logic_vector(7 downto 4); SW1_c, SW2_c_0, SW2_c, 
        ACM_CLK_c, SW1_c_1 : in std_logic; RS_LCD_c : out
         std_logic; SW1_c_0 : in std_logic; EN_LCD_c : out
         std_logic);

end LCD_DIS;

architecture DEF_ARCH of LCD_DIS is 

  component NOR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component CLK_DIVIDER
    port(lcd_en_0, lcd_en : out std_logic; SW1_c_1, ACM_CLK_c, 
        SW1_c_0 : in std_logic := 'U');
  end component;

  component lcd
    port(LCD_net : out std_logic_vector(7 downto 4); RTC_hours_c
         : in std_logic_vector(4 downto 0) := (others => 'U'); 
        RTC_year_c : 
        in std_logic_vector(3 downto 0) := (others => 'U'); 
        RTC_seconds_c : 
        in std_logic_vector(6 downto 0) := (others => 'U'); 
        RTC_date_c : 
        in std_logic_vector(5 downto 0) := (others => 'U'); 
        RTC_month_c : 
        in std_logic_vector(7 downto 0) := (others => 'U'); 
        RTC_minutes_c : 
        in std_logic_vector(6 downto 0) := (others => 'U'); 
        lcd_en : in std_logic := 'U'; lcd_rs : out std_logic; 
        SW2_c, SW2_c_0, SW1_c, SW1_c_1, lcd_en_0 : in
         std_logic := 'U');
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

    signal lcd_en, \LCD_net[7]\, \LCD_net[6]\, \LCD_net[5]\, 
        \LCD_net[4]\, lcd_rs, lcd_en_0, \VCC\, \GND\, GND_net_1, 
        VCC_net_1 : std_logic;

    for all : CLK_DIVIDER
	Use entity work.CLK_DIVIDER(DEF_ARCH);
    for all : lcd
	Use entity work.lcd(DEF_ARCH);
begin 


    \DATA_LCD_0_a2_0_a2[7]\ : NOR2A
      port map(A => \LCD_net[7]\, B => SW1_c_0, Y => 
        DATA_LCD_c(7));
    
    \DATA_LCD_0_a2_0_a2[4]\ : NOR2A
      port map(A => \LCD_net[4]\, B => SW1_c_0, Y => 
        DATA_LCD_c(4));
    
    CLK_DIVIDER_intance : CLK_DIVIDER
      port map(lcd_en_0 => lcd_en_0, lcd_en => lcd_en, SW1_c_1
         => SW1_c_1, ACM_CLK_c => ACM_CLK_c, SW1_c_0 => SW1_c_0);
    
    lcd_instance : lcd
      port map(LCD_net(7) => \LCD_net[7]\, LCD_net(6) => 
        \LCD_net[6]\, LCD_net(5) => \LCD_net[5]\, LCD_net(4) => 
        \LCD_net[4]\, RTC_hours_c(4) => RTC_hours_c(4), 
        RTC_hours_c(3) => RTC_hours_c(3), RTC_hours_c(2) => 
        RTC_hours_c(2), RTC_hours_c(1) => RTC_hours_c(1), 
        RTC_hours_c(0) => RTC_hours_c(0), RTC_year_c(3) => 
        RTC_year_c(3), RTC_year_c(2) => RTC_year_c(2), 
        RTC_year_c(1) => RTC_year_c(1), RTC_year_c(0) => 
        RTC_year_c(0), RTC_seconds_c(6) => RTC_seconds_c(6), 
        RTC_seconds_c(5) => RTC_seconds_c(5), RTC_seconds_c(4)
         => RTC_seconds_c(4), RTC_seconds_c(3) => 
        RTC_seconds_c(3), RTC_seconds_c(2) => RTC_seconds_c(2), 
        RTC_seconds_c(1) => RTC_seconds_c(1), RTC_seconds_c(0)
         => RTC_seconds_c(0), RTC_date_c(5) => RTC_date_c(5), 
        RTC_date_c(4) => RTC_date_c(4), RTC_date_c(3) => 
        RTC_date_c(3), RTC_date_c(2) => RTC_date_c(2), 
        RTC_date_c(1) => RTC_date_c(1), RTC_date_c(0) => 
        RTC_date_c(0), RTC_month_c(7) => RTC_month_c(7), 
        RTC_month_c(6) => RTC_month_c(6), RTC_month_c(5) => 
        RTC_month_c(5), RTC_month_c(4) => RTC_month_c(4), 
        RTC_month_c(3) => RTC_month_c(3), RTC_month_c(2) => 
        RTC_month_c(2), RTC_month_c(1) => RTC_month_c(1), 
        RTC_month_c(0) => RTC_month_c(0), RTC_minutes_c(6) => 
        RTC_minutes_c(6), RTC_minutes_c(5) => RTC_minutes_c(5), 
        RTC_minutes_c(4) => RTC_minutes_c(4), RTC_minutes_c(3)
         => RTC_minutes_c(3), RTC_minutes_c(2) => 
        RTC_minutes_c(2), RTC_minutes_c(1) => RTC_minutes_c(1), 
        RTC_minutes_c(0) => RTC_minutes_c(0), lcd_en => lcd_en, 
        lcd_rs => lcd_rs, SW2_c => SW2_c, SW2_c_0 => SW2_c_0, 
        SW1_c => SW1_c, SW1_c_1 => SW1_c_1, lcd_en_0 => lcd_en_0);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    EN_LCD_0_a2_0_a2 : NOR2A
      port map(A => lcd_en, B => SW1_c_0, Y => EN_LCD_c);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    RS_LCD_0_a2_0_a2 : NOR2A
      port map(A => lcd_rs, B => SW1_c_0, Y => RS_LCD_c);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    GND_i : GND
      port map(Y => \GND\);
    
    \DATA_LCD_0_a2_0_a2[6]\ : NOR2A
      port map(A => \LCD_net[6]\, B => SW1_c_0, Y => 
        DATA_LCD_c(6));
    
    \DATA_LCD_0_a2_0_a2[5]\ : NOR2A
      port map(A => \LCD_net[5]\, B => SW1_c_0, Y => 
        DATA_LCD_c(5));
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity RCO is

    port(sys_clk : out std_logic);

end RCO;

architecture DEF_ARCH of RCO is 

  component VCC
    port(Y : out std_logic);
  end component;

  component RCOSC
    port(CLKOUT : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    RCOSC1 : RCOSC
      port map(CLKOUT => sys_clk);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity jhmkpjjmdkx is

    port(hrgmtcpsdzr : out std_logic; qdvwjxmzbpf : in std_logic; 
        bfcqzvtnrcn : out std_logic; sys_reset_gl, ACM_CLK_c : in
         std_logic; bfcqzvtnrcn_0 : out std_logic);

end jhmkpjjmdkx;

architecture DEF_ARCH of jhmkpjjmdkx is 

  component VCC
    port(Y : out std_logic);
  end component;

  component DFN1C0
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

    signal \VCC\, \crmctgwbctb\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    crmctgwbctb : DFN1C0
      port map(D => qdvwjxmzbpf, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \crmctgwbctb\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    \hrgmtcpsdzr\ : DFN1C0
      port map(D => \crmctgwbctb\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => hrgmtcpsdzr);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    qqqjcnjzntg_0 : DFN1C0
      port map(D => \VCC\, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        Q => bfcqzvtnrcn_0);
    
    qqqjcnjzntg : DFN1C0
      port map(D => \VCC\, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        Q => bfcqzvtnrcn);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity INITCFG_XF is

    port(bfcqzvtnrcn_0 : out std_logic; ACM_CLK_c, sys_reset_gl : 
        in std_logic; bfcqzvtnrcn : out std_logic; qdvwjxmzbpf : 
        in std_logic; hrgmtcpsdzr : out std_logic);

end INITCFG_XF;

architecture DEF_ARCH of INITCFG_XF is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component jhmkpjjmdkx
    port(hrgmtcpsdzr : out std_logic; qdvwjxmzbpf : in
         std_logic := 'U'; bfcqzvtnrcn : out std_logic; 
        sys_reset_gl, ACM_CLK_c : in std_logic := 'U'; 
        bfcqzvtnrcn_0 : out std_logic);
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : jhmkpjjmdkx
	Use entity work.jhmkpjjmdkx(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    u_jhmkpjjmdkx : jhmkpjjmdkx
      port map(hrgmtcpsdzr => hrgmtcpsdzr, qdvwjxmzbpf => 
        qdvwjxmzbpf, bfcqzvtnrcn => bfcqzvtnrcn, sys_reset_gl => 
        sys_reset_gl, ACM_CLK_c => ACM_CLK_c, bfcqzvtnrcn_0 => 
        bfcqzvtnrcn_0);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity stnpfdkggvx is

    port(gpbbjwsprsj : out std_logic_vector(2 downto 0); 
        mwtvnzdkmwm_3, mwtvnzdkmwm_5, mwtvnzdkmwm_4, 
        \mwtvnzdkmwm_ns_1[6]\, \gpbbjwsprsj_0[0]\, qdvwjxmzbpf, 
        N_70, N_7 : out std_logic; bfcqzvtnrcn, mhxwcfnvbgt : in
         std_logic; xbkjkgdsbfh, dxpbcqjpxtw : out std_logic; 
        NVM_BUSY_net, sys_reset_gl, ACM_CLK_c : in std_logic);

end stnpfdkggvx;

architecture DEF_ARCH of stnpfdkggvx is 

  component NOR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1C0
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component AO1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component AO1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AXOI5
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1P0
    port(D, CLK, PRE : in std_logic := 'U'; Q : out std_logic);
  end component;

  component DFN1E1C0
    port(D, CLK, CLR, E : in std_logic := 'U'; Q : out std_logic);
        
  end component;

  component OR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1D
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \gpbbjwsprsj[0]\, \gpbbjwsprsj[1]\, \gpbbjwsprsj[2]\, 
        \mwtvnzdkmwm[4]_net_1\, \mwtvnzdkmwm[6]_net_1\, 
        \mwtvnzdkmwm[5]_net_1\, \mwtvnzdkmwm_ns_1[6]_net_1\, 
        \gpbbjwsprsj_0[0]_net_1\, \qdvwjxmzbpf\, \N_70\, \N_7\, 
        \xbkjkgdsbfh\, \dxpbcqjpxtw\, \rhjctqnqzsn_1_i_0\, 
        \mwtvnzdkmwm_ns_a3_0_a2_0[6]_net_1\, 
        \mwtvnzdkmwm[2]_net_1\, \mwtvnzdkmwm_ns_i_0_0[1]_net_1\, 
        \mwtvnzdkmwm[7]_net_1\, \mwtvnzdkmwm_ns[0]\, 
        \mwtvnzdkmwm_ns_i_0_a2_1_0[1]_net_1\, 
        \mwtvnzdkmwm_ns_i_0[1]_net_1\, \mwtvnzdkmwm_ns[6]\, 
        \mwtvnzdkmwm_ns[2]\, \mwtvnzdkmwm_ns[5]\, 
        \mwtvnzdkmwm_ns[4]\, \mwtvnzdkmwm_ns[3]\, N_73, N_71_i, 
        \mwtvnzdkmwm_ns_i_a3_0_a2[7]_net_1\, 
        \mwtvnzdkmwm[1]_net_1\, \VCC\, \GND\, GND_net_1, 
        VCC_net_1 : std_logic;

begin 

    gpbbjwsprsj(2) <= \gpbbjwsprsj[2]\;
    gpbbjwsprsj(1) <= \gpbbjwsprsj[1]\;
    gpbbjwsprsj(0) <= \gpbbjwsprsj[0]\;
    mwtvnzdkmwm_3 <= \mwtvnzdkmwm[4]_net_1\;
    mwtvnzdkmwm_5 <= \mwtvnzdkmwm[6]_net_1\;
    mwtvnzdkmwm_4 <= \mwtvnzdkmwm[5]_net_1\;
    \mwtvnzdkmwm_ns_1[6]\ <= \mwtvnzdkmwm_ns_1[6]_net_1\;
    \gpbbjwsprsj_0[0]\ <= \gpbbjwsprsj_0[0]_net_1\;
    qdvwjxmzbpf <= \qdvwjxmzbpf\;
    N_70 <= \N_70\;
    N_7 <= \N_7\;
    xbkjkgdsbfh <= \xbkjkgdsbfh\;
    dxpbcqjpxtw <= \dxpbcqjpxtw\;

    \mwtvnzdkmwm_ns_i_0_a2[1]\ : NOR2A
      port map(A => \mwtvnzdkmwm[7]_net_1\, B => bfcqzvtnrcn, Y
         => \mwtvnzdkmwm_ns[0]\);
    
    \mwtvnzdkmwm[2]\ : DFN1C0
      port map(D => \mwtvnzdkmwm_ns[5]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \mwtvnzdkmwm[2]_net_1\);
    
    \mwtvnzdkmwm_ns_0[3]\ : AO1A
      port map(A => NVM_BUSY_net, B => \mwtvnzdkmwm[5]_net_1\, C
         => \dxpbcqjpxtw\, Y => \mwtvnzdkmwm_ns[3]\);
    
    \mwtvnzdkmwm[6]\ : DFN1C0
      port map(D => \mwtvnzdkmwm_ns_i_0[1]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \mwtvnzdkmwm[6]_net_1\);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    \mwtvnzdkmwm_ns_0[4]\ : AO1
      port map(A => NVM_BUSY_net, B => \xbkjkgdsbfh\, C => 
        \mwtvnzdkmwm[4]_net_1\, Y => \mwtvnzdkmwm_ns[4]\);
    
    \mwtvnzdkmwm_ns_a3_0_a2_1[6]\ : OR2A
      port map(A => \gpbbjwsprsj[2]\, B => \gpbbjwsprsj[1]\, Y
         => \mwtvnzdkmwm_ns_1[6]_net_1\);
    
    \mwtvnzdkmwm[4]\ : DFN1C0
      port map(D => \mwtvnzdkmwm_ns[3]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \mwtvnzdkmwm[4]_net_1\);
    
    \mwtvnzdkmwm_ns_0[2]\ : AO1
      port map(A => NVM_BUSY_net, B => \mwtvnzdkmwm[5]_net_1\, C
         => \mwtvnzdkmwm[6]_net_1\, Y => \mwtvnzdkmwm_ns[2]\);
    
    \mwtvnzdkmwm[3]\ : DFN1C0
      port map(D => \mwtvnzdkmwm_ns[4]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \xbkjkgdsbfh\);
    
    \mwtvnzdkmwm_ns_i_0_a2_1_0[1]\ : OR2
      port map(A => \mwtvnzdkmwm[7]_net_1\, B => 
        \gpbbjwsprsj_0[0]_net_1\, Y => 
        \mwtvnzdkmwm_ns_i_0_a2_1_0[1]_net_1\);
    
    rhjctqnqzsn_1_i_0 : AXOI5
      port map(A => \mwtvnzdkmwm_ns_1[6]_net_1\, B => 
        \mwtvnzdkmwm[2]_net_1\, C => \gpbbjwsprsj[0]\, Y => 
        \rhjctqnqzsn_1_i_0\);
    
    \rhjctqnqzsn[0]\ : DFN1C0
      port map(D => \rhjctqnqzsn_1_i_0\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \gpbbjwsprsj[0]\);
    
    \mwtvnzdkmwm_ns_i_0[1]\ : OA1B
      port map(A => \mwtvnzdkmwm_ns_1[6]_net_1\, B => 
        \mwtvnzdkmwm_ns_i_0_a2_1_0[1]_net_1\, C => 
        \mwtvnzdkmwm_ns_i_0_0[1]_net_1\, Y => 
        \mwtvnzdkmwm_ns_i_0[1]_net_1\);
    
    \rhjctqnqzsn_0[0]\ : DFN1C0
      port map(D => \rhjctqnqzsn_1_i_0\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \gpbbjwsprsj_0[0]_net_1\);
    
    \mwtvnzdkmwm[1]\ : DFN1C0
      port map(D => \mwtvnzdkmwm_ns[6]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \mwtvnzdkmwm[1]_net_1\);
    
    GND_i : GND
      port map(Y => \GND\);
    
    \mwtvnzdkmwm[5]\ : DFN1C0
      port map(D => \mwtvnzdkmwm_ns[2]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \mwtvnzdkmwm[5]_net_1\);
    
    \mwtvnzdkmwm[7]\ : DFN1P0
      port map(D => \mwtvnzdkmwm_ns[0]\, CLK => ACM_CLK_c, PRE
         => sys_reset_gl, Q => \mwtvnzdkmwm[7]_net_1\);
    
    \rhjctqnqzsn[2]\ : DFN1E1C0
      port map(D => N_73, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        E => \mwtvnzdkmwm[2]_net_1\, Q => \gpbbjwsprsj[2]\);
    
    un7_rhjctqnqzsn_1_CO2_0_o2 : OR2B
      port map(A => \gpbbjwsprsj[1]\, B => \gpbbjwsprsj[0]\, Y
         => \N_7\);
    
    \rhjctqnqzsn[1]\ : DFN1E1C0
      port map(D => \N_70\, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        E => \mwtvnzdkmwm[2]_net_1\, Q => \gpbbjwsprsj[1]\);
    
    \mwtvnzdkmwm_ns_a3_0_a2[5]\ : NOR3B
      port map(A => \xbkjkgdsbfh\, B => mhxwcfnvbgt, C => 
        NVM_BUSY_net, Y => \mwtvnzdkmwm_ns[5]\);
    
    \mwtvnzdkmwm_ns_i_a3_0_a2[7]\ : OR2
      port map(A => \qdvwjxmzbpf\, B => \mwtvnzdkmwm[1]_net_1\, Y
         => \mwtvnzdkmwm_ns_i_a3_0_a2[7]_net_1\);
    
    \mwtvnzdkmwm[0]\ : DFN1C0
      port map(D => \mwtvnzdkmwm_ns_i_a3_0_a2[7]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => \qdvwjxmzbpf\);
    
    un7_rhjctqnqzsn_1_SUM1_0_x2 : XOR2
      port map(A => \gpbbjwsprsj[1]\, B => \gpbbjwsprsj[0]\, Y
         => \N_70\);
    
    un7_rhjctqnqzsn_1_SUM2_0_x2 : XOR2
      port map(A => \N_7\, B => \gpbbjwsprsj[2]\, Y => N_71_i);
    
    \mwtvnzdkmwm_ns_a3_0_a2[6]\ : NOR2A
      port map(A => \mwtvnzdkmwm_ns_a3_0_a2_0[6]_net_1\, B => 
        \mwtvnzdkmwm_ns_1[6]_net_1\, Y => \mwtvnzdkmwm_ns[6]\);
    
    \mwtvnzdkmwm_ns_i_0_0[1]\ : AO1D
      port map(A => \mwtvnzdkmwm[2]_net_1\, B => 
        \mwtvnzdkmwm[7]_net_1\, C => \mwtvnzdkmwm_ns[0]\, Y => 
        \mwtvnzdkmwm_ns_i_0_0[1]_net_1\);
    
    \mwtvnzdkmwm_ns_a3_0_a2_0[6]\ : NOR2A
      port map(A => \mwtvnzdkmwm[2]_net_1\, B => 
        \gpbbjwsprsj_0[0]_net_1\, Y => 
        \mwtvnzdkmwm_ns_a3_0_a2_0[6]_net_1\);
    
    \rhjctqnqzsn_3_i[2]\ : NOR2
      port map(A => \mwtvnzdkmwm_ns[6]\, B => N_71_i, Y => N_73);
    
    \mwtvnzdkmwm_ns_0_a2_0[3]\ : NOR3A
      port map(A => \xbkjkgdsbfh\, B => NVM_BUSY_net, C => 
        mhxwcfnvbgt, Y => \dxpbcqjpxtw\);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity INITCFG_XA is

    port(mwtvnzdkmwm : out std_logic_vector(6 downto 4); 
        gpbbjwsprsj : out std_logic_vector(2 downto 0); 
        \gpbbjwsprsj_0[0]\, \mwtvnzdkmwm_ns_1[6]\ : out std_logic; 
        ACM_CLK_c, sys_reset_gl, NVM_BUSY_net : in std_logic; 
        dxpbcqjpxtw, xbkjkgdsbfh : out std_logic; mhxwcfnvbgt, 
        bfcqzvtnrcn : in std_logic; N_7, N_70, qdvwjxmzbpf : out
         std_logic);

end INITCFG_XA;

architecture DEF_ARCH of INITCFG_XA is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component stnpfdkggvx
    port(gpbbjwsprsj : out std_logic_vector(2 downto 0); 
        mwtvnzdkmwm_3, mwtvnzdkmwm_5, mwtvnzdkmwm_4, 
        \mwtvnzdkmwm_ns_1[6]\, \gpbbjwsprsj_0[0]\, qdvwjxmzbpf, 
        N_70, N_7 : out std_logic; bfcqzvtnrcn, mhxwcfnvbgt : in
         std_logic := 'U'; xbkjkgdsbfh, dxpbcqjpxtw : out
         std_logic; NVM_BUSY_net, sys_reset_gl, ACM_CLK_c : in
         std_logic := 'U');
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : stnpfdkggvx
	Use entity work.stnpfdkggvx(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    u_stnpfdkggvx : stnpfdkggvx
      port map(gpbbjwsprsj(2) => gpbbjwsprsj(2), gpbbjwsprsj(1)
         => gpbbjwsprsj(1), gpbbjwsprsj(0) => gpbbjwsprsj(0), 
        mwtvnzdkmwm_3 => mwtvnzdkmwm(4), mwtvnzdkmwm_5 => 
        mwtvnzdkmwm(6), mwtvnzdkmwm_4 => mwtvnzdkmwm(5), 
        \mwtvnzdkmwm_ns_1[6]\ => \mwtvnzdkmwm_ns_1[6]\, 
        \gpbbjwsprsj_0[0]\ => \gpbbjwsprsj_0[0]\, qdvwjxmzbpf => 
        qdvwjxmzbpf, N_70 => N_70, N_7 => N_7, bfcqzvtnrcn => 
        bfcqzvtnrcn, mhxwcfnvbgt => mhxwcfnvbgt, xbkjkgdsbfh => 
        xbkjkgdsbfh, dxpbcqjpxtw => dxpbcqjpxtw, NVM_BUSY_net => 
        NVM_BUSY_net, sys_reset_gl => sys_reset_gl, ACM_CLK_c => 
        ACM_CLK_c);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity dxwqxgdccxc is

    port(mmsccbknsvn : in std_logic_vector(15 downto 12); 
        INIT_ADDR_int : out std_logic_vector(8 downto 0); 
        ggfvvjbkdws_0, ggfvvjbkdws_1, ggfvvjbkdws_2, 
        ggfvvjbkdws_3, ggfvvjbkdws_4, ggfvvjbkdws_5 : out
         std_logic; \wqwwrvhjsjn_i[0]\, cbfkvdxnxdp_i_0, 
        sys_reset_gl, ACM_CLK_c : in std_logic; mhxwcfnvbgt, N_31, 
        N_25, N_20 : out std_logic; N_3943, N_4378, N_4134_1, 
        N_4225, N_4224, N_4221, N_4220, N_4223, N_4222 : in
         std_logic; N_41, N_44 : out std_logic; xkvhpkkjhmx_i_0 : 
        in std_logic; N_24, N178 : out std_logic; N_2772 : in
         std_logic; ADD_13x13_slow_I7_S_0, ADD_13x13_slow_I8_S_0, 
        ADD_13x13_slow_I9_S_0 : out std_logic);

end dxwqxgdccxc;

architecture DEF_ARCH of dxwqxgdccxc is 

  component XOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1C0
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component AX1E
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component AND2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2C
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XA1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XNOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MAJ3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component DFN1E0C0
    port(D, CLK, CLR, E : in std_logic := 'U'; Q : out std_logic);
        
  end component;

  component OA1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \ggfvvjbkdws[4]_net_1\, \ggfvvjbkdws[5]_net_1\, 
        \ggfvvjbkdws[6]_net_1\, \INIT_ADDR_int[0]\, 
        \INIT_ADDR_int[1]\, \INIT_ADDR_int[2]\, 
        \INIT_ADDR_int[3]\, \INIT_ADDR_int[4]\, 
        \INIT_ADDR_int[5]\, \INIT_ADDR_int[6]\, 
        \INIT_ADDR_int[7]\, \INIT_ADDR_int[8]\, \N_25\, \N_20\, 
        \N_41\, \N_44\, \N_24\, \N178\, 
        \DWACT_ADD_CI_0_g_array_2[0]\, 
        \DWACT_ADD_CI_0_g_array_1[0]\, 
        \DWACT_ADD_CI_0_pog_array_1[0]\, 
        \DWACT_ADD_CI_0_g_array_3[0]\, 
        \DWACT_ADD_CI_0_pog_array_2[0]\, 
        \DWACT_ADD_CI_0_g_array_11[0]\, 
        \DWACT_ADD_CI_0_pog_array_1_1[0]\, 
        \DWACT_ADD_CI_0_TMP[0]\, \DWACT_ADD_CI_0_g_array_12_1[0]\, 
        \DWACT_ADD_CI_0_g_array_12[0]\, 
        \DWACT_ADD_CI_0_g_array_12_2[0]\, \ggfvvjbkdws[9]_net_1\, 
        \ggfvvjbkdws[8]_net_1\, \ggfvvjbkdws[7]_net_1\, 
        un10_gnmdxrqrdgs_NE_5, un10_gnmdxrqrdgs_8, 
        un10_gnmdxrqrdgs_NE_2, un10_gnmdxrqrdgs_NE_3, 
        un10_gnmdxrqrdgs_7, un10_gnmdxrqrdgs_NE_0, 
        un10_gnmdxrqrdgs_3, un10_gnmdxrqrdgs_6, 
        un10_gnmdxrqrdgs_NE, un10_gnmdxrqrdgs_5, 
        un10_gnmdxrqrdgs_1, \gnmdxrqrdgs_3[8]\, I_36, 
        \gnmdxrqrdgs_3[7]\, I_34, \gnmdxrqrdgs_3[6]\, I_38, 
        \gnmdxrqrdgs_3[5]\, I_33, \gnmdxrqrdgs_3[4]\, I_30, 
        \gnmdxrqrdgs_3[3]\, I_37, \gnmdxrqrdgs_3[2]\, I_35, 
        \gnmdxrqrdgs_3[1]\, I_32, \gnmdxrqrdgs_3[0]\, 
        \DWACT_ADD_CI_0_partial_sum[0]\, N_41_0, 
        \DWACT_ADD_CI_0_pog_array_1_2[0]\, \VCC\, \GND\, 
        GND_net_1, VCC_net_1 : std_logic;

begin 

    INIT_ADDR_int(8) <= \INIT_ADDR_int[8]\;
    INIT_ADDR_int(7) <= \INIT_ADDR_int[7]\;
    INIT_ADDR_int(6) <= \INIT_ADDR_int[6]\;
    INIT_ADDR_int(5) <= \INIT_ADDR_int[5]\;
    INIT_ADDR_int(4) <= \INIT_ADDR_int[4]\;
    INIT_ADDR_int(3) <= \INIT_ADDR_int[3]\;
    INIT_ADDR_int(2) <= \INIT_ADDR_int[2]\;
    INIT_ADDR_int(1) <= \INIT_ADDR_int[1]\;
    INIT_ADDR_int(0) <= \INIT_ADDR_int[0]\;
    ggfvvjbkdws_3 <= \ggfvvjbkdws[4]_net_1\;
    ggfvvjbkdws_4 <= \ggfvvjbkdws[5]_net_1\;
    ggfvvjbkdws_5 <= \ggfvvjbkdws[6]_net_1\;
    N_25 <= \N_25\;
    N_20 <= \N_20\;
    N_41 <= \N_41\;
    N_44 <= \N_44\;
    N_24 <= \N_24\;
    N178 <= \N178\;

    un1_gnmdxrqrdgs_I_32 : XOR2
      port map(A => \INIT_ADDR_int[1]\, B => 
        \DWACT_ADD_CI_0_TMP[0]\, Y => I_32);
    
    \ggfvvjbkdws[4]\ : MX2
      port map(A => \INIT_ADDR_int[4]\, B => \INIT_ADDR_int[3]\, 
        S => \wqwwrvhjsjn_i[0]\, Y => \ggfvvjbkdws[4]_net_1\);
    
    \gnmdxrqrdgs[2]\ : DFN1C0
      port map(D => \gnmdxrqrdgs_3[2]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \INIT_ADDR_int[2]\);
    
    jvbvzqvvxpc_un10_gnmdxrqrdgs_7 : AX1E
      port map(A => N_4222, B => N_4223, C => \INIT_ADDR_int[7]\, 
        Y => un10_gnmdxrqrdgs_7);
    
    un1_gnmdxrqrdgs_I_34 : XOR2
      port map(A => \INIT_ADDR_int[7]\, B => 
        \DWACT_ADD_CI_0_g_array_12_2[0]\, Y => I_34);
    
    jvbvzqvvxpc_un10_gnmdxrqrdgs_6 : AX1E
      port map(A => N_4220, B => N_4221, C => \INIT_ADDR_int[6]\, 
        Y => un10_gnmdxrqrdgs_6);
    
    un1_gnmdxrqrdgs_I_43 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_1[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_1[0]\, Y => 
        \DWACT_ADD_CI_0_g_array_2[0]\);
    
    un1_dkjzqmrkvwb_ADD_13x13_slow_I8_S_0 : XOR2
      port map(A => mmsccbknsvn(13), B => \ggfvvjbkdws[8]_net_1\, 
        Y => ADD_13x13_slow_I8_S_0);
    
    un1_dkjzqmrkvwb_ADD_13x13_slow_I6_un1_CO1_i_a2 : NOR2
      port map(A => \N_44\, B => \ggfvvjbkdws[6]_net_1\, Y => 
        \N_24\);
    
    jvbvzqvvxpc_un2_tnjvgqqhdkd_i_a2 : NOR2A
      port map(A => xkvhpkkjhmx_i_0, B => un10_gnmdxrqrdgs_NE, Y
         => N_41_0);
    
    \gnmdxrqrdgs[0]\ : DFN1C0
      port map(D => \gnmdxrqrdgs_3[0]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \INIT_ADDR_int[0]\);
    
    \gnmdxrqrdgs[5]\ : DFN1C0
      port map(D => \gnmdxrqrdgs_3[5]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \INIT_ADDR_int[5]\);
    
    jvbvzqvvxpc_un10_gnmdxrqrdgs_8 : AX1E
      port map(A => N_4224, B => N_4225, C => \INIT_ADDR_int[8]\, 
        Y => un10_gnmdxrqrdgs_8);
    
    un1_gnmdxrqrdgs_I_42 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_2[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_1_1[0]\, Y => 
        \DWACT_ADD_CI_0_g_array_11[0]\);
    
    jvbvzqvvxpc_un10_gnmdxrqrdgs_NE_5 : NOR3C
      port map(A => un10_gnmdxrqrdgs_8, B => 
        un10_gnmdxrqrdgs_NE_2, C => un10_gnmdxrqrdgs_NE_3, Y => 
        un10_gnmdxrqrdgs_NE_5);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    un1_gnmdxrqrdgs_I_44 : NOR2B
      port map(A => \DWACT_ADD_CI_0_TMP[0]\, B => 
        \INIT_ADDR_int[1]\, Y => \DWACT_ADD_CI_0_g_array_1[0]\);
    
    un1_gnmdxrqrdgs_I_53 : AND2
      port map(A => \INIT_ADDR_int[4]\, B => \INIT_ADDR_int[5]\, 
        Y => \DWACT_ADD_CI_0_pog_array_1_1[0]\);
    
    un1_gnmdxrqrdgs_I_49 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_1[0]\, B => 
        \INIT_ADDR_int[2]\, Y => \DWACT_ADD_CI_0_g_array_12[0]\);
    
    \jvbvzqvvxpc_gnmdxrqrdgs_3_0_a2[8]\ : NOR2B
      port map(A => xkvhpkkjhmx_i_0, B => I_36, Y => 
        \gnmdxrqrdgs_3[8]\);
    
    un1_dkjzqmrkvwb_ADD_13x13_slow_I7_S_0 : XOR2
      port map(A => mmsccbknsvn(12), B => \ggfvvjbkdws[7]_net_1\, 
        Y => ADD_13x13_slow_I7_S_0);
    
    \gnmdxrqrdgs[1]\ : DFN1C0
      port map(D => \gnmdxrqrdgs_3[1]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \INIT_ADDR_int[1]\);
    
    un1_gnmdxrqrdgs_I_52 : AND2
      port map(A => \INIT_ADDR_int[2]\, B => \INIT_ADDR_int[3]\, 
        Y => \DWACT_ADD_CI_0_pog_array_1[0]\);
    
    un1_gnmdxrqrdgs_I_38 : XOR2
      port map(A => \INIT_ADDR_int[6]\, B => 
        \DWACT_ADD_CI_0_g_array_11[0]\, Y => I_38);
    
    un1_dkjzqmrkvwb_ADD_13x13_slow_I9_S_0 : XOR2
      port map(A => mmsccbknsvn(14), B => \ggfvvjbkdws[9]_net_1\, 
        Y => ADD_13x13_slow_I9_S_0);
    
    \ggfvvjbkdws[3]\ : MX2C
      port map(A => \INIT_ADDR_int[3]\, B => \INIT_ADDR_int[2]\, 
        S => \wqwwrvhjsjn_i[0]\, Y => ggfvvjbkdws_2);
    
    jvbvzqvvxpc_un10_gnmdxrqrdgs_NE_3 : NOR3C
      port map(A => un10_gnmdxrqrdgs_7, B => 
        un10_gnmdxrqrdgs_NE_0, C => un10_gnmdxrqrdgs_3, Y => 
        un10_gnmdxrqrdgs_NE_3);
    
    jvbvzqvvxpc_un10_gnmdxrqrdgs_NE_2 : XA1
      port map(A => \INIT_ADDR_int[2]\, B => N_2772, C => 
        un10_gnmdxrqrdgs_6, Y => un10_gnmdxrqrdgs_NE_2);
    
    \jvbvzqvvxpc_gnmdxrqrdgs_3_0_a2[7]\ : NOR2B
      port map(A => xkvhpkkjhmx_i_0, B => I_34, Y => 
        \gnmdxrqrdgs_3[7]\);
    
    un1_gnmdxrqrdgs_I_30 : XOR2
      port map(A => \INIT_ADDR_int[4]\, B => 
        \DWACT_ADD_CI_0_g_array_2[0]\, Y => I_30);
    
    un1_gnmdxrqrdgs_I_1 : AND2
      port map(A => \INIT_ADDR_int[0]\, B => cbfkvdxnxdp_i_0, Y
         => \DWACT_ADD_CI_0_TMP[0]\);
    
    un1_gnmdxrqrdgs_I_54 : AND2
      port map(A => \INIT_ADDR_int[6]\, B => \INIT_ADDR_int[7]\, 
        Y => \DWACT_ADD_CI_0_pog_array_1_2[0]\);
    
    \gnmdxrqrdgs[6]\ : DFN1C0
      port map(D => \gnmdxrqrdgs_3[6]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \INIT_ADDR_int[6]\);
    
    jvbvzqvvxpc_un10_gnmdxrqrdgs_3 : AX1E
      port map(A => N_4134_1, B => N_4378, C => 
        \INIT_ADDR_int[3]\, Y => un10_gnmdxrqrdgs_3);
    
    \jvbvzqvvxpc_gnmdxrqrdgs_3_0_a2[5]\ : NOR2B
      port map(A => xkvhpkkjhmx_i_0, B => I_33, Y => 
        \gnmdxrqrdgs_3[5]\);
    
    jvbvzqvvxpc_un10_gnmdxrqrdgs_1 : XNOR2
      port map(A => \wqwwrvhjsjn_i[0]\, B => \INIT_ADDR_int[1]\, 
        Y => un10_gnmdxrqrdgs_1);
    
    \ggfvvjbkdws[5]\ : MX2
      port map(A => \INIT_ADDR_int[5]\, B => \INIT_ADDR_int[4]\, 
        S => \wqwwrvhjsjn_i[0]\, Y => \ggfvvjbkdws[5]_net_1\);
    
    un1_dkjzqmrkvwb_ADD_13x13_slow_I8_un1_CO1_i_o2 : MAJ3
      port map(A => \ggfvvjbkdws[8]_net_1\, B => mmsccbknsvn(13), 
        C => \N178\, Y => \N_20\);
    
    \jvbvzqvvxpc_gnmdxrqrdgs_3_0_a2[2]\ : NOR2B
      port map(A => xkvhpkkjhmx_i_0, B => I_35, Y => 
        \gnmdxrqrdgs_3[2]\);
    
    \jvbvzqvvxpc_gnmdxrqrdgs_3_0_a2[6]\ : NOR2B
      port map(A => xkvhpkkjhmx_i_0, B => I_38, Y => 
        \gnmdxrqrdgs_3[6]\);
    
    un1_gnmdxrqrdgs_I_36 : XOR2
      port map(A => \INIT_ADDR_int[8]\, B => 
        \DWACT_ADD_CI_0_g_array_3[0]\, Y => I_36);
    
    jvbvzqvvxpc_un10_gnmdxrqrdgs_NE_0 : NOR2B
      port map(A => \INIT_ADDR_int[4]\, B => \INIT_ADDR_int[0]\, 
        Y => un10_gnmdxrqrdgs_NE_0);
    
    un1_dkjzqmrkvwb_ADD_13x13_slow_I11_S_0_o2 : OR2B
      port map(A => mmsccbknsvn(15), B => \N_25\, Y => N_31);
    
    \gnmdxrqrdgs[4]\ : DFN1C0
      port map(D => \gnmdxrqrdgs_3[4]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \INIT_ADDR_int[4]\);
    
    GND_i : GND
      port map(Y => \GND\);
    
    \ggfvvjbkdws[7]\ : MX2
      port map(A => \INIT_ADDR_int[7]\, B => \INIT_ADDR_int[6]\, 
        S => \wqwwrvhjsjn_i[0]\, Y => \ggfvvjbkdws[7]_net_1\);
    
    un1_dkjzqmrkvwb_ADD_13x13_slow_I5_CO1_i_a2 : OR2B
      port map(A => \N_41\, B => \ggfvvjbkdws[5]_net_1\, Y => 
        \N_44\);
    
    \gnmdxrqrdgs[3]\ : DFN1C0
      port map(D => \gnmdxrqrdgs_3[3]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \INIT_ADDR_int[3]\);
    
    \ggfvvjbkdws[2]\ : MX2C
      port map(A => \INIT_ADDR_int[2]\, B => \INIT_ADDR_int[1]\, 
        S => \wqwwrvhjsjn_i[0]\, Y => ggfvvjbkdws_1);
    
    un1_gnmdxrqrdgs_I_37 : XOR2
      port map(A => \INIT_ADDR_int[3]\, B => 
        \DWACT_ADD_CI_0_g_array_12[0]\, Y => I_37);
    
    \jvbvzqvvxpc_gnmdxrqrdgs_3_0_a2[3]\ : NOR2B
      port map(A => xkvhpkkjhmx_i_0, B => I_37, Y => 
        \gnmdxrqrdgs_3[3]\);
    
    \ggfvvjbkdws[6]\ : MX2C
      port map(A => \INIT_ADDR_int[6]\, B => \INIT_ADDR_int[5]\, 
        S => \wqwwrvhjsjn_i[0]\, Y => \ggfvvjbkdws[6]_net_1\);
    
    jvbvzqvvxpc_un10_gnmdxrqrdgs_NE : NOR3C
      port map(A => un10_gnmdxrqrdgs_5, B => 
        un10_gnmdxrqrdgs_NE_5, C => un10_gnmdxrqrdgs_1, Y => 
        un10_gnmdxrqrdgs_NE);
    
    jvbvzqvvxpc_un10_gnmdxrqrdgs_5 : XNOR2
      port map(A => \wqwwrvhjsjn_i[0]\, B => \INIT_ADDR_int[5]\, 
        Y => un10_gnmdxrqrdgs_5);
    
    un1_gnmdxrqrdgs_I_21 : XOR2
      port map(A => \INIT_ADDR_int[0]\, B => cbfkvdxnxdp_i_0, Y
         => \DWACT_ADD_CI_0_partial_sum[0]\);
    
    un1_gnmdxrqrdgs_I_46 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_2[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_2[0]\, Y => 
        \DWACT_ADD_CI_0_g_array_3[0]\);
    
    \ggfvvjbkdws[9]\ : NOR2B
      port map(A => \wqwwrvhjsjn_i[0]\, B => \INIT_ADDR_int[8]\, 
        Y => \ggfvvjbkdws[9]_net_1\);
    
    un1_gnmdxrqrdgs_I_35 : XOR2
      port map(A => \INIT_ADDR_int[2]\, B => 
        \DWACT_ADD_CI_0_g_array_1[0]\, Y => I_35);
    
    \jvbvzqvvxpc_gnmdxrqrdgs_3_0_a2[1]\ : NOR2B
      port map(A => xkvhpkkjhmx_i_0, B => I_32, Y => 
        \gnmdxrqrdgs_3[1]\);
    
    un1_gnmdxrqrdgs_I_50 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_11[0]\, B => 
        \INIT_ADDR_int[6]\, Y => \DWACT_ADD_CI_0_g_array_12_2[0]\);
    
    un1_gnmdxrqrdgs_I_51 : AND2
      port map(A => \DWACT_ADD_CI_0_pog_array_1_1[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_1_2[0]\, Y => 
        \DWACT_ADD_CI_0_pog_array_2[0]\);
    
    un1_dkjzqmrkvwb_ADD_13x13_slow_I7_CO1_i_o2 : MAJ3
      port map(A => mmsccbknsvn(12), B => \ggfvvjbkdws[7]_net_1\, 
        C => \N_24\, Y => \N178\);
    
    \jvbvzqvvxpc_gnmdxrqrdgs_3_0_a2[4]\ : NOR2B
      port map(A => xkvhpkkjhmx_i_0, B => I_30, Y => 
        \gnmdxrqrdgs_3[4]\);
    
    un1_gnmdxrqrdgs_I_47 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_2[0]\, B => 
        \INIT_ADDR_int[4]\, Y => \DWACT_ADD_CI_0_g_array_12_1[0]\);
    
    \gnmdxrqrdgs[8]\ : DFN1C0
      port map(D => \gnmdxrqrdgs_3[8]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \INIT_ADDR_int[8]\);
    
    un1_dkjzqmrkvwb_ADD_13x13_slow_I4_un1_CO1_i_a2_0 : NOR2A
      port map(A => \ggfvvjbkdws[4]_net_1\, B => N_3943, Y => 
        \N_41\);
    
    \gnmdxrqrdgs[7]\ : DFN1C0
      port map(D => \gnmdxrqrdgs_3[7]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \INIT_ADDR_int[7]\);
    
    kftxkfpnstb : DFN1E0C0
      port map(D => un10_gnmdxrqrdgs_NE, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => N_41_0, Q => mhxwcfnvbgt);
    
    un1_gnmdxrqrdgs_I_33 : XOR2
      port map(A => \INIT_ADDR_int[5]\, B => 
        \DWACT_ADD_CI_0_g_array_12_1[0]\, Y => I_33);
    
    \jvbvzqvvxpc_gnmdxrqrdgs_3_0_a2[0]\ : NOR2B
      port map(A => xkvhpkkjhmx_i_0, B => 
        \DWACT_ADD_CI_0_partial_sum[0]\, Y => \gnmdxrqrdgs_3[0]\);
    
    \ggfvvjbkdws[8]\ : MX2
      port map(A => \INIT_ADDR_int[8]\, B => \INIT_ADDR_int[7]\, 
        S => \wqwwrvhjsjn_i[0]\, Y => \ggfvvjbkdws[8]_net_1\);
    
    un1_dkjzqmrkvwb_ADD_13x13_slow_I10_S_0_o3 : OA1
      port map(A => mmsccbknsvn(14), B => \N_20\, C => 
        \ggfvvjbkdws[9]_net_1\, Y => \N_25\);
    
    \ggfvvjbkdws[1]\ : MX2C
      port map(A => \INIT_ADDR_int[1]\, B => \INIT_ADDR_int[0]\, 
        S => \wqwwrvhjsjn_i[0]\, Y => ggfvvjbkdws_0);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity INITCFG_XD is

    port(INIT_ADDR_int : out std_logic_vector(8 downto 0); 
        mmsccbknsvn : in std_logic_vector(15 downto 12); 
        ggfvvjbkdws : out std_logic_vector(6 downto 1); 
        \wqwwrvhjsjn_i[0]\ : in std_logic; ADD_13x13_slow_I9_S_0, 
        ADD_13x13_slow_I8_S_0, ADD_13x13_slow_I7_S_0 : out
         std_logic; N_2772 : in std_logic; N178, N_24 : out
         std_logic; xkvhpkkjhmx_i_0 : in std_logic; N_44, N_41 : 
        out std_logic; N_4222, N_4223, N_4220, N_4221, N_4224, 
        N_4225, N_4134_1, N_4378, N_3943 : in std_logic; N_20, 
        N_25, N_31, mhxwcfnvbgt : out std_logic; ACM_CLK_c, 
        sys_reset_gl, cbfkvdxnxdp_i_0 : in std_logic);

end INITCFG_XD;

architecture DEF_ARCH of INITCFG_XD is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component dxwqxgdccxc
    port(mmsccbknsvn : 
        in std_logic_vector(15 downto 12) := (others => 'U'); 
        INIT_ADDR_int : out std_logic_vector(8 downto 0); 
        ggfvvjbkdws_0, ggfvvjbkdws_1, ggfvvjbkdws_2, 
        ggfvvjbkdws_3, ggfvvjbkdws_4, ggfvvjbkdws_5 : out
         std_logic; \wqwwrvhjsjn_i[0]\, cbfkvdxnxdp_i_0, 
        sys_reset_gl, ACM_CLK_c : in std_logic := 'U'; 
        mhxwcfnvbgt, N_31, N_25, N_20 : out std_logic; N_3943, 
        N_4378, N_4134_1, N_4225, N_4224, N_4221, N_4220, N_4223, 
        N_4222 : in std_logic := 'U'; N_41, N_44 : out std_logic; 
        xkvhpkkjhmx_i_0 : in std_logic := 'U'; N_24, N178 : out
         std_logic; N_2772 : in std_logic := 'U'; 
        ADD_13x13_slow_I7_S_0, ADD_13x13_slow_I8_S_0, 
        ADD_13x13_slow_I9_S_0 : out std_logic);
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : dxwqxgdccxc
	Use entity work.dxwqxgdccxc(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    u_dxwqxgdccxc : dxwqxgdccxc
      port map(mmsccbknsvn(15) => mmsccbknsvn(15), 
        mmsccbknsvn(14) => mmsccbknsvn(14), mmsccbknsvn(13) => 
        mmsccbknsvn(13), mmsccbknsvn(12) => mmsccbknsvn(12), 
        INIT_ADDR_int(8) => INIT_ADDR_int(8), INIT_ADDR_int(7)
         => INIT_ADDR_int(7), INIT_ADDR_int(6) => 
        INIT_ADDR_int(6), INIT_ADDR_int(5) => INIT_ADDR_int(5), 
        INIT_ADDR_int(4) => INIT_ADDR_int(4), INIT_ADDR_int(3)
         => INIT_ADDR_int(3), INIT_ADDR_int(2) => 
        INIT_ADDR_int(2), INIT_ADDR_int(1) => INIT_ADDR_int(1), 
        INIT_ADDR_int(0) => INIT_ADDR_int(0), ggfvvjbkdws_0 => 
        ggfvvjbkdws(1), ggfvvjbkdws_1 => ggfvvjbkdws(2), 
        ggfvvjbkdws_2 => ggfvvjbkdws(3), ggfvvjbkdws_3 => 
        ggfvvjbkdws(4), ggfvvjbkdws_4 => ggfvvjbkdws(5), 
        ggfvvjbkdws_5 => ggfvvjbkdws(6), \wqwwrvhjsjn_i[0]\ => 
        \wqwwrvhjsjn_i[0]\, cbfkvdxnxdp_i_0 => cbfkvdxnxdp_i_0, 
        sys_reset_gl => sys_reset_gl, ACM_CLK_c => ACM_CLK_c, 
        mhxwcfnvbgt => mhxwcfnvbgt, N_31 => N_31, N_25 => N_25, 
        N_20 => N_20, N_3943 => N_3943, N_4378 => N_4378, 
        N_4134_1 => N_4134_1, N_4225 => N_4225, N_4224 => N_4224, 
        N_4221 => N_4221, N_4220 => N_4220, N_4223 => N_4223, 
        N_4222 => N_4222, N_41 => N_41, N_44 => N_44, 
        xkvhpkkjhmx_i_0 => xkvhpkkjhmx_i_0, N_24 => N_24, N178
         => N178, N_2772 => N_2772, ADD_13x13_slow_I7_S_0 => 
        ADD_13x13_slow_I7_S_0, ADD_13x13_slow_I8_S_0 => 
        ADD_13x13_slow_I8_S_0, ADD_13x13_slow_I9_S_0 => 
        ADD_13x13_slow_I9_S_0);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity gpbdjfttbjd is

    port(\mwtvnzdkmwm_ns_1[6]\, \gpbbjwsprsj[0]\, 
        \vvpkfmmpprk[0]\, \gpbbjwsprsj_0[0]\, N_101 : in
         std_logic; N_12, INIT_EV_WEN : out std_logic; 
        NVM_BUSY_net, xbkjkgdsbfh : in std_logic; N_14, 
        INIT_ASSC_WEN : out std_logic; N_4294, N_4432, 
        npgnwznthhx : in std_logic);

end gpbdjfttbjd;

architecture DEF_ARCH of gpbdjfttbjd is 

  component OR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AOI1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component OR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \N_14\, N_28, N_27, N_26, N_25, N_30, N_29, \VCC\, 
        \GND\, GND_net_1, VCC_net_1 : std_logic;

begin 

    N_14 <= \N_14\;

    \fpppnnsbksx_0_a3_0[4]\ : OR3
      port map(A => \gpbbjwsprsj[0]\, B => npgnwznthhx, C => 
        \mwtvnzdkmwm_ns_1[6]\, Y => N_30);
    
    \fpppnnsbksx_0_a3[4]\ : OR3A
      port map(A => npgnwznthhx, B => \vvpkfmmpprk[0]\, C => 
        N_101, Y => N_29);
    
    tdckqkwvkfk_un7_vcktjmvmkfg_i : OA1B
      port map(A => npgnwznthhx, B => xbkjkgdsbfh, C => 
        NVM_BUSY_net, Y => \N_14\);
    
    \fpppnnsbksx_r[3]\ : AOI1B
      port map(A => N_27, B => N_28, C => \N_14\, Y => 
        INIT_EV_WEN);
    
    \fpppnnsbksx_0_a3[3]\ : OR3C
      port map(A => \vvpkfmmpprk[0]\, B => npgnwznthhx, C => 
        N_4294, Y => N_27);
    
    \fpppnnsbksx_r[2]\ : AOI1B
      port map(A => N_25, B => N_26, C => \N_14\, Y => 
        INIT_ASSC_WEN);
    
    \fpppnnsbksx_0_a3_0[3]\ : OR3A
      port map(A => \gpbbjwsprsj_0[0]\, B => npgnwznthhx, C => 
        N_4432, Y => N_28);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    \fpppnnsbksx_0_a3_0[2]\ : OR3
      port map(A => \gpbbjwsprsj_0[0]\, B => npgnwznthhx, C => 
        N_4432, Y => N_26);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    GND_i : GND
      port map(Y => \GND\);
    
    \fpppnnsbksx_0_a3[2]\ : OR3B
      port map(A => npgnwznthhx, B => N_4294, C => 
        \vvpkfmmpprk[0]\, Y => N_25);
    
    \fpppnnsbksx_0[4]\ : OR2B
      port map(A => N_30, B => N_29, Y => N_12);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity INITCFG_XE is

    port(\gpbbjwsprsj_0[0]\, \vvpkfmmpprk[0]\, \gpbbjwsprsj[0]\, 
        \mwtvnzdkmwm_ns_1[6]\, npgnwznthhx, N_4432, N_4294 : in
         std_logic; INIT_ASSC_WEN, N_14 : out std_logic; 
        xbkjkgdsbfh, NVM_BUSY_net : in std_logic; INIT_EV_WEN, 
        N_12 : out std_logic; N_101 : in std_logic);

end INITCFG_XE;

architecture DEF_ARCH of INITCFG_XE is 

  component VCC
    port(Y : out std_logic);
  end component;

  component gpbdjfttbjd
    port(\mwtvnzdkmwm_ns_1[6]\, \gpbbjwsprsj[0]\, 
        \vvpkfmmpprk[0]\, \gpbbjwsprsj_0[0]\, N_101 : in
         std_logic := 'U'; N_12, INIT_EV_WEN : out std_logic; 
        NVM_BUSY_net, xbkjkgdsbfh : in std_logic := 'U'; N_14, 
        INIT_ASSC_WEN : out std_logic; N_4294, N_4432, 
        npgnwznthhx : in std_logic := 'U');
  end component;

  component GND
    port(Y : out std_logic);
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : gpbdjfttbjd
	Use entity work.gpbdjfttbjd(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    u_gpbdjfttbjd : gpbdjfttbjd
      port map(\mwtvnzdkmwm_ns_1[6]\ => \mwtvnzdkmwm_ns_1[6]\, 
        \gpbbjwsprsj[0]\ => \gpbbjwsprsj[0]\, \vvpkfmmpprk[0]\
         => \vvpkfmmpprk[0]\, \gpbbjwsprsj_0[0]\ => 
        \gpbbjwsprsj_0[0]\, N_101 => N_101, N_12 => N_12, 
        INIT_EV_WEN => INIT_EV_WEN, NVM_BUSY_net => NVM_BUSY_net, 
        xbkjkgdsbfh => xbkjkgdsbfh, N_14 => N_14, INIT_ASSC_WEN
         => INIT_ASSC_WEN, N_4294 => N_4294, N_4432 => N_4432, 
        npgnwznthhx => npgnwznthhx);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity ffppvfwnggt is

    port(ggfvvjbkdws : in std_logic_vector(6 downto 1); 
        vvpkfmmpprk : in std_logic_vector(2 downto 0); 
        gpbbjwsprsj : in std_logic_vector(2 downto 0); 
        mwtvnzdkmwm : in std_logic_vector(6 downto 4); 
        \wqwwrvhjsjn_i[0]\ : out std_logic; \INIT_ADDR_int[0]\ : 
        in std_logic; srdtcqsfmmm_i_0_13, srdtcqsfmmm_i_0_16, 
        srdtcqsfmmm_i_0_15, srdtcqsfmmm_i_0_4, srdtcqsfmmm_i_0_3, 
        srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_12, srdtcqsfmmm_i_0_14, 
        srdtcqsfmmm_i_0_2, srdtcqsfmmm_i_0_1, srdtcqsfmmm_i_0_5, 
        srdtcqsfmmm_i_0_0, mmsccbknsvn_3, mmsccbknsvn_1, 
        mmsccbknsvn_2, mmsccbknsvn_0 : out std_logic; 
        \mwtvnzdkmwm_ns_1[6]\, \gpbbjwsprsj_0[0]\, crpwsdfhcqq_0, 
        crpwsdfhcqq_4 : in std_logic; N_2497_i_0 : out std_logic; 
        ADD_13x13_slow_I8_S_0, N178, N_31 : in std_logic; 
        xkvhpkkjhmx_i_0 : out std_logic; N_25 : in std_logic; 
        twxmgvvpbgm_iv_0_0_o3_i_o3 : out std_logic; dxpbcqjpxtw, 
        fhpsqkqvwpw : in std_logic; cbfkvdxnxdp_i_0, N_2772 : out
         std_logic; xbkjkgdsbfh : in std_logic; 
        twxmgvvpbgm_iv_0_0_0_a2_0 : out std_logic; N_44, 
        ADD_13x13_slow_I7_S_0, N_24, ADD_13x13_slow_I9_S_0, N_20, 
        N_41 : in std_logic; N_4294 : out std_logic; N_7, 
        bfcqzvtnrcn : in std_logic; N_4223, N_4225, N_4224, 
        N_3943, N_4222 : out std_logic; N_70 : in std_logic; 
        N_4221, N_4220, N_4432 : out std_logic; N_101 : in
         std_logic; N_4134_1, N_4378 : out std_logic; qdvwjxmzbpf, 
        bfcqzvtnrcn_0 : in std_logic);

end ffppvfwnggt;

architecture DEF_ARCH of ffppvfwnggt is 

  component OR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2C
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XA1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component NOR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AOI1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component XO1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1D
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XA1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \wqwwrvhjsjn_i[0]_net_1\, \mmsccbknsvn[15]\, 
        twxmgvvpbgm_iv_0_0_o3_i_o3_net_1, \N_4294\, \N_3943\, 
        \N_4432\, \N_4134_1\, \N_4378\, N_4402_0, 
        \hrdbnmzftpn_0_0_0_1[14]_net_1\, N_4135, 
        \hrdbnmzftpn_0_0_0_1[13]_net_1\, N_4081, N_4131_1, N_4380, 
        \hrdbnmzftpn_0_0_0_1[15]_net_1\, N_4375_i, N_4136_i, 
        ttkcwbrgmjd_i_0_a2_0, \ttkcwbrgmjd_i_0_a2_0_0\, 
        \hrdbnmzftpn_0_0_0_3[12]_net_1\, N_4381, N_4382, 
        \hrdbnmzftpn_0_0_0_1[12]_net_1\, 
        \hrdbnmzftpn_0_0_0_2[12]_net_1\, N_4051, N_4135_1, N_4129, 
        \hrdbnmzftpn_i_i_i_2[4]_net_1\, 
        \hrdbnmzftpn_i_i_i_0[4]_net_1\, 
        \cpqqthwwhvc_i_0_a2_0_0[8]_net_1\, N_4102_i, 
        \cpqqthwwhvc_i_0_a2_0[8]_net_1\, 
        \grmnwjxgzvw_i_m_8_i_0_0_a2_0_0[9]_net_1\, 
        \cpqqthwwhvc_i_0_a2_0[6]_net_1\, 
        \hrdbnmzftpn_0_0_a2_0_0[16]_net_1\, N_4070, 
        \cpqqthwwhvc_i_0_a2_0[2]\, 
        \hrdbnmzftpn_0_0_0_a3_0_0[12]_net_1\, 
        \cpqqthwwhvc_i_0_a2_0_1[2]_net_1\, 
        \cpqqthwwhvc_i_0_a2_0_0[7]\, N_4354_1, N_4302_1, N_4036, 
        N_4449, N_4318, N_4141_1, N_4212, N_4030, N_4374, N_4321, 
        N_4379_1, N_4190, N_4406, N_4100_i_0, N_4193, N_4029, 
        N_4048, \mmsccbknsvn[16]\, N_4141, \VCC\, \GND\, 
        GND_net_1, VCC_net_1 : std_logic;

begin 

    \wqwwrvhjsjn_i[0]\ <= \wqwwrvhjsjn_i[0]_net_1\;
    mmsccbknsvn_3 <= \mmsccbknsvn[15]\;
    twxmgvvpbgm_iv_0_0_o3_i_o3 <= 
        twxmgvvpbgm_iv_0_0_o3_i_o3_net_1;
    N_4294 <= \N_4294\;
    N_3943 <= \N_3943\;
    N_4432 <= \N_4432\;
    N_4134_1 <= \N_4134_1\;
    N_4378 <= \N_4378\;

    \hrdbnmzftpn_i_0_0_o3[6]\ : OR2B
      port map(A => gpbbjwsprsj(2), B => gpbbjwsprsj(1), Y => 
        N_4030);
    
    ttkcwbrgmjd_i_0 : MX2C
      port map(A => ttkcwbrgmjd_i_0_a2_0, B => 
        \ttkcwbrgmjd_i_0_a2_0_0\, S => 
        twxmgvvpbgm_iv_0_0_o3_i_o3_net_1, Y => xkvhpkkjhmx_i_0);
    
    \hrdbnmzftpn_0_0_0_a3_0[14]\ : OR3A
      port map(A => N_4100_i_0, B => N_7, C => N_4030, Y => 
        \N_4378\);
    
    \hrdbnmzftpn_0_0_0_a3[12]\ : OR2B
      port map(A => N_4318, B => N_4100_i_0, Y => N_4381);
    
    \hrdbnmzftpn_0_0_0_a3[10]\ : NOR2A
      port map(A => gpbbjwsprsj(0), B => gpbbjwsprsj(1), Y => 
        N_4321);
    
    \cpqqthwwhvc_i_0_a2_0_2[2]\ : NOR2A
      port map(A => N_101, B => bfcqzvtnrcn_0, Y => 
        \cpqqthwwhvc_i_0_a2_0[2]\);
    
    \cbfkvdxnxdp_i_0\ : MX2
      port map(A => fhpsqkqvwpw, B => dxpbcqjpxtw, S => N_4402_0, 
        Y => cbfkvdxnxdp_i_0);
    
    \srdtcqsfmmm_i_0[13]\ : XA1
      port map(A => N178, B => ADD_13x13_slow_I8_S_0, C => 
        twxmgvvpbgm_iv_0_0_o3_i_o3_net_1, Y => srdtcqsfmmm_i_0_13);
    
    \hrdbnmzftpn_0_0_a2_0_0[16]\ : NOR2A
      port map(A => vvpkfmmpprk(2), B => N_4070, Y => 
        \hrdbnmzftpn_0_0_a2_0_0[16]_net_1\);
    
    \hrdbnmzftpn_0_0_0_o3_1[10]\ : OR2
      port map(A => gpbbjwsprsj(1), B => gpbbjwsprsj(0), Y => 
        N_4029);
    
    \hrdbnmzftpn_0_0_0_a2_1_0[15]\ : NOR3
      port map(A => vvpkfmmpprk(0), B => bfcqzvtnrcn_0, C => 
        vvpkfmmpprk(1), Y => N_4135_1);
    
    \pqmzrrnpzvx_i_a2_2_a2[0]\ : NOR3
      port map(A => bfcqzvtnrcn, B => vvpkfmmpprk(1), C => 
        vvpkfmmpprk(2), Y => N_4193);
    
    \hrdbnmzftpn_0_0_0[12]\ : OR2B
      port map(A => \hrdbnmzftpn_0_0_0_3[12]_net_1\, B => 
        \hrdbnmzftpn_0_0_0_2[12]_net_1\, Y => mmsccbknsvn_0);
    
    \cpqqthwwhvc_i_0_a2_0[8]\ : OR3
      port map(A => N_4318, B => 
        \cpqqthwwhvc_i_0_a2_0_0[8]_net_1\, C => N_4141_1, Y => 
        N_4225);
    
    \grmnwjxgzvw_i_m_14_i_0_0_a3[9]\ : OR2A
      port map(A => gpbbjwsprsj(1), B => gpbbjwsprsj(0), Y => 
        N_4102_i);
    
    \srdtcqsfmmm_i_0[12]\ : XA1
      port map(A => N_24, B => ADD_13x13_slow_I7_S_0, C => 
        N_4402_0, Y => srdtcqsfmmm_i_0_12);
    
    \pqmzrrnpzvx_i_a2_2[0]\ : NOR2A
      port map(A => N_4190, B => N_4193, Y => 
        \wqwwrvhjsjn_i[0]_net_1\);
    
    \grmnwjxgzvw_i_m_16_i_0_0_a2_0[9]\ : NOR2A
      port map(A => vvpkfmmpprk(1), B => vvpkfmmpprk(2), Y => 
        \N_4294\);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    \hbqjhzzrbnr_i_0_a2_1[0]\ : OR3A
      port map(A => bfcqzvtnrcn, B => gpbbjwsprsj(1), C => 
        gpbbjwsprsj(2), Y => N_4190);
    
    \hrdbnmzftpn_0_0_0_a2_1_1[14]\ : OR2A
      port map(A => N_4406, B => N_4036, Y => \N_4134_1\);
    
    \grmnwjxgzvw_i_m_6_i_0_0_a3[9]\ : OR2A
      port map(A => vvpkfmmpprk(1), B => vvpkfmmpprk(0), Y => 
        N_4070);
    
    \srdtcqsfmmm_i_0[0]\ : NOR3B
      port map(A => \INIT_ADDR_int[0]\, B => N_4402_0, C => 
        \wqwwrvhjsjn_i[0]_net_1\, Y => srdtcqsfmmm_i_0_0);
    
    \hrdbnmzftpn_0_0_0_a3_0[15]\ : OR3A
      port map(A => bfcqzvtnrcn, B => N_7, C => \N_4432\, Y => 
        N_4375_i);
    
    \hrdbnmzftpn_0_0_0_1[12]\ : NOR2
      port map(A => N_4081, B => N_4380, Y => 
        \hrdbnmzftpn_0_0_0_1[12]_net_1\);
    
    \cpqqthwwhvc_i_0_a2_0[6]\ : OR3A
      port map(A => bfcqzvtnrcn_0, B => N_70, C => N_4354_1, Y
         => N_4221);
    
    \hbqjhzzrbnr_i_0[0]\ : NOR2B
      port map(A => twxmgvvpbgm_iv_0_0_o3_i_o3_net_1, B => N_4190, 
        Y => N_2497_i_0);
    
    \cpqqthwwhvc_i_0_a2_0_0_0[8]\ : OR2B
      port map(A => bfcqzvtnrcn_0, B => N_4102_i, Y => 
        \cpqqthwwhvc_i_0_a2_0_0[8]_net_1\);
    
    \hrdbnmzftpn_0_0_0_a3_1[12]\ : NOR2B
      port map(A => N_4406, B => \N_4294\, Y => N_4081);
    
    \srdtcqsfmmm_i_0[3]\ : NOR2A
      port map(A => N_4402_0, B => ggfvvjbkdws(3), Y => 
        srdtcqsfmmm_i_0_3);
    
    \srdtcqsfmmm_i_0[14]\ : XA1
      port map(A => N_20, B => ADD_13x13_slow_I9_S_0, C => 
        N_4402_0, Y => srdtcqsfmmm_i_0_14);
    
    \hrdbnmzftpn_i_i_i_2[4]\ : AO1C
      port map(A => \gpbbjwsprsj_0[0]\, B => gpbbjwsprsj(2), C
         => gpbbjwsprsj(1), Y => \hrdbnmzftpn_i_i_i_2[4]_net_1\);
    
    \hrdbnmzftpn_0_0_0_a2_0[15]\ : OR3B
      port map(A => vvpkfmmpprk(0), B => N_4406, C => 
        vvpkfmmpprk(2), Y => N_4136_i);
    
    \srdtcqsfmmm_i_0[6]\ : XA1
      port map(A => ggfvvjbkdws(6), B => N_44, C => N_4402_0, Y
         => srdtcqsfmmm_i_0_6);
    
    \hrdbnmzftpn_0_0_0_a3_0[12]\ : OR3B
      port map(A => \hrdbnmzftpn_0_0_0_a3_0_0[12]_net_1\, B => 
        N_4354_1, C => \gpbbjwsprsj_0[0]\, Y => N_4382);
    
    \hrdbnmzftpn_0_0_0_a3[15]\ : NOR2A
      port map(A => N_4379_1, B => N_4029, Y => N_4374);
    
    \cpqqthwwhvc_i_0_a2_0_0_0[7]\ : AOI1B
      port map(A => \gpbbjwsprsj_0[0]\, B => gpbbjwsprsj(2), C
         => bfcqzvtnrcn_0, Y => \cpqqthwwhvc_i_0_a2_0_0[7]\);
    
    \cpqqthwwhvc_i_0_a2_0_1[2]\ : NOR3C
      port map(A => \N_4432\, B => bfcqzvtnrcn_0, C => 
        \mwtvnzdkmwm_ns_1[6]\, Y => 
        \cpqqthwwhvc_i_0_a2_0_1[2]_net_1\);
    
    \cpqqthwwhvc_i_0_a2[7]\ : OR3B
      port map(A => N_4070, B => N_4036, C => bfcqzvtnrcn_0, Y
         => N_4222);
    
    \hrdbnmzftpn_i_0_0_o2[5]\ : NOR2B
      port map(A => gpbbjwsprsj(2), B => bfcqzvtnrcn, Y => 
        N_4100_i_0);
    
    \twxmgvvpbgm_iv_0_0_0_a2_0\ : OA1
      port map(A => mwtvnzdkmwm(4), B => xbkjkgdsbfh, C => 
        N_4402_0, Y => twxmgvvpbgm_iv_0_0_0_a2_0);
    
    \srdtcqsfmmm_i_0[2]\ : NOR2A
      port map(A => N_4402_0, B => ggfvvjbkdws(2), Y => 
        srdtcqsfmmm_i_0_2);
    
    \hrdbnmzftpn_0_0_0_3[12]\ : NOR3C
      port map(A => N_4381, B => N_4382, C => 
        \hrdbnmzftpn_0_0_0_1[12]_net_1\, Y => 
        \hrdbnmzftpn_0_0_0_3[12]_net_1\);
    
    \grmnwjxgzvw_i_m_14_i_0_0_a2_3[9]\ : NOR2A
      port map(A => gpbbjwsprsj(0), B => \mwtvnzdkmwm_ns_1[6]\, Y
         => N_4318);
    
    \hrdbnmzftpn_0_0_0_a3_1[13]\ : NOR2A
      port map(A => bfcqzvtnrcn, B => N_4048, Y => N_4379_1);
    
    GND_i : GND
      port map(Y => \GND\);
    
    un1_gmvpvmvhgsk_3_0_a2_0_a2_1 : NOR3B
      port map(A => vvpkfmmpprk(0), B => vvpkfmmpprk(2), C => 
        vvpkfmmpprk(1), Y => N_4302_1);
    
    \hrdbnmzftpn_0_0_0_1[14]\ : OR3B
      port map(A => \N_4378\, B => \N_4134_1\, C => N_4135, Y => 
        \hrdbnmzftpn_0_0_0_1[14]_net_1\);
    
    \hrdbnmzftpn_i_i_i_0[4]\ : OA1
      port map(A => \gpbbjwsprsj_0[0]\, B => gpbbjwsprsj(2), C
         => bfcqzvtnrcn_0, Y => \hrdbnmzftpn_i_i_i_0[4]_net_1\);
    
    \hrdbnmzftpn_i_0_0_a3[1]\ : XO1A
      port map(A => \gpbbjwsprsj_0[0]\, B => gpbbjwsprsj(2), C
         => gpbbjwsprsj(1), Y => N_4449);
    
    \hrdbnmzftpn_0_0_0_a2[12]\ : OR2B
      port map(A => vvpkfmmpprk(2), B => N_4131_1, Y => N_4129);
    
    \cpqqthwwhvc_i_0_a2_0_0[8]\ : AO1A
      port map(A => \grmnwjxgzvw_i_m_8_i_0_0_a2_0_0[9]_net_1\, B
         => vvpkfmmpprk(1), C => bfcqzvtnrcn_0, Y => 
        \cpqqthwwhvc_i_0_a2_0[8]_net_1\);
    
    \hrdbnmzftpn_0_0_0_a3_0[13]\ : NOR3B
      port map(A => bfcqzvtnrcn, B => gpbbjwsprsj(1), C => 
        \N_4432\, Y => N_4380);
    
    \cpqqthwwhvc_i_0_a2_0_1[6]\ : AO1D
      port map(A => vvpkfmmpprk(1), B => vvpkfmmpprk(2), C => 
        bfcqzvtnrcn_0, Y => \cpqqthwwhvc_i_0_a2_0[6]_net_1\);
    
    ttkcwbrgmjd_i_0_a2_0_0 : OR2
      port map(A => mwtvnzdkmwm(6), B => mwtvnzdkmwm(5), Y => 
        \ttkcwbrgmjd_i_0_a2_0_0\);
    
    twxmgvvpbgm_iv_0_0_o3_i_o3_0 : NOR2A
      port map(A => bfcqzvtnrcn_0, B => qdvwjxmzbpf, Y => 
        N_4402_0);
    
    \hrdbnmzftpn_0_0_0_o3_1[7]\ : OR2B
      port map(A => vvpkfmmpprk(2), B => vvpkfmmpprk(0), Y => 
        N_4036);
    
    \hrdbnmzftpn_0_0_0_o3[15]\ : OA1A
      port map(A => N_4321, B => gpbbjwsprsj(2), C => 
        \mwtvnzdkmwm_ns_1[6]\, Y => N_4048);
    
    \cpqqthwwhvc_i_0[2]\ : AO1B
      port map(A => N_4102_i, B => 
        \cpqqthwwhvc_i_0_a2_0_1[2]_net_1\, C => N_4212, Y => 
        N_2772);
    
    \srdtcqsfmmm_i_0[1]\ : NOR2A
      port map(A => N_4402_0, B => ggfvvjbkdws(1), Y => 
        srdtcqsfmmm_i_0_1);
    
    \hrdbnmzftpn_0_0_0_a2_1[13]\ : NOR3A
      port map(A => vvpkfmmpprk(0), B => bfcqzvtnrcn, C => 
        vvpkfmmpprk(1), Y => N_4131_1);
    
    \hrdbnmzftpn_0_0_0_2[12]\ : AOI1B
      port map(A => N_4051, B => N_4135_1, C => N_4129, Y => 
        \hrdbnmzftpn_0_0_0_2[12]_net_1\);
    
    \cpqqthwwhvc_i_0_a2[2]\ : AO1B
      port map(A => N_4036, B => vvpkfmmpprk(1), C => 
        \cpqqthwwhvc_i_0_a2_0[2]\, Y => N_4212);
    
    \hrdbnmzftpn_0_0_0[15]\ : OR2A
      port map(A => \hrdbnmzftpn_0_0_0_1[15]_net_1\, B => N_4374, 
        Y => \mmsccbknsvn[15]\);
    
    \hrdbnmzftpn_0_0_a2_2_1[16]\ : NOR2
      port map(A => N_4102_i, B => N_4030, Y => N_4141_1);
    
    \cpqqthwwhvc_i_0_a2_0[7]\ : OR3C
      port map(A => \cpqqthwwhvc_i_0_a2_0_0[7]\, B => N_4030, C
         => N_4102_i, Y => N_4223);
    
    \hrdbnmzftpn_0_0_0_a3_0_0[12]\ : NOR2A
      port map(A => bfcqzvtnrcn_0, B => gpbbjwsprsj(1), Y => 
        \hrdbnmzftpn_0_0_0_a3_0_0[12]_net_1\);
    
    \grmnwjxgzvw_i_m_8_i_0_0_a2_0_0[9]\ : OR2A
      port map(A => vvpkfmmpprk(2), B => vvpkfmmpprk(0), Y => 
        \grmnwjxgzvw_i_m_8_i_0_0_a2_0_0[9]_net_1\);
    
    \cpqqthwwhvc_i_0_a2[6]\ : OR3A
      port map(A => N_4070, B => \cpqqthwwhvc_i_0_a2_0[6]_net_1\, 
        C => N_4302_1, Y => N_4220);
    
    \hrdbnmzftpn_0_0_0_1[13]\ : OR3
      port map(A => N_4081, B => N_4131_1, C => N_4380, Y => 
        \hrdbnmzftpn_0_0_0_1[13]_net_1\);
    
    \srdtcqsfmmm_i_0[15]\ : XA1
      port map(A => N_25, B => \mmsccbknsvn[15]\, C => 
        twxmgvvpbgm_iv_0_0_o3_i_o3_net_1, Y => srdtcqsfmmm_i_0_15);
    
    \cpqqthwwhvc_i_0_a2[8]\ : OR3A
      port map(A => N_4070, B => \cpqqthwwhvc_i_0_a2_0[8]_net_1\, 
        C => N_4302_1, Y => N_4224);
    
    \hrdbnmzftpn_0_0_a2_2[16]\ : OR2B
      port map(A => N_4141_1, B => N_4100_i_0, Y => N_4141);
    
    \hrdbnmzftpn_0_0_0_a3_4[7]\ : NOR2A
      port map(A => vvpkfmmpprk(1), B => bfcqzvtnrcn, Y => N_4406);
    
    \hrdbnmzftpn_0_0_0[13]\ : AO1
      port map(A => N_4321, B => N_4379_1, C => 
        \hrdbnmzftpn_0_0_0_1[13]_net_1\, Y => mmsccbknsvn_1);
    
    \hrdbnmzftpn_i_i_i[4]\ : OR3C
      port map(A => \hrdbnmzftpn_i_i_i_0[4]_net_1\, B => N_4449, 
        C => \hrdbnmzftpn_i_i_i_2[4]_net_1\, Y => \N_3943\);
    
    \hrdbnmzftpn_0_0_0_1[15]\ : NOR3B
      port map(A => N_4375_i, B => N_4136_i, C => N_4135, Y => 
        \hrdbnmzftpn_0_0_0_1[15]_net_1\);
    
    \srdtcqsfmmm_i_0[4]\ : XA1A
      port map(A => ggfvvjbkdws(4), B => \N_3943\, C => 
        twxmgvvpbgm_iv_0_0_o3_i_o3_net_1, Y => srdtcqsfmmm_i_0_4);
    
    \grmnwjxgzvw_i_m_11_i_0_0_a2_0_1[9]\ : NOR2
      port map(A => N_4029, B => gpbbjwsprsj(2), Y => N_4354_1);
    
    \srdtcqsfmmm_i_0[5]\ : XA1
      port map(A => ggfvvjbkdws(5), B => N_41, C => N_4402_0, Y
         => srdtcqsfmmm_i_0_5);
    
    \grmnwjxgzvw_i_m_14_0_i_0_o3[9]\ : NOR2
      port map(A => vvpkfmmpprk(2), B => vvpkfmmpprk(0), Y => 
        N_4051);
    
    \srdtcqsfmmm_i_0[16]\ : XA1A
      port map(A => N_31, B => \mmsccbknsvn[16]\, C => 
        twxmgvvpbgm_iv_0_0_o3_i_o3_net_1, Y => srdtcqsfmmm_i_0_16);
    
    \hrdbnmzftpn_0_0_0_a2[15]\ : NOR2A
      port map(A => N_4135_1, B => N_4051, Y => N_4135);
    
    \grmnwjxgzvw_i_m_10_i_0_0_a3_1[9]\ : OR2A
      port map(A => gpbbjwsprsj(1), B => gpbbjwsprsj(2), Y => 
        \N_4432\);
    
    \twxmgvvpbgm_iv_0_0_o3_i_o3\ : NOR2A
      port map(A => bfcqzvtnrcn, B => qdvwjxmzbpf, Y => 
        twxmgvvpbgm_iv_0_0_o3_i_o3_net_1);
    
    ttkcwbrgmjd_i_0_a2_0_1 : OR2
      port map(A => crpwsdfhcqq_4, B => crpwsdfhcqq_0, Y => 
        ttkcwbrgmjd_i_0_a2_0);
    
    \hrdbnmzftpn_0_0_0[14]\ : OR2
      port map(A => \hrdbnmzftpn_0_0_0_1[14]_net_1\, B => N_4374, 
        Y => mmsccbknsvn_2);
    
    \hrdbnmzftpn_0_0[16]\ : AO1B
      port map(A => N_4406, B => 
        \hrdbnmzftpn_0_0_a2_0_0[16]_net_1\, C => N_4141, Y => 
        \mmsccbknsvn[16]\);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity INITCFG_XC is

    port(mwtvnzdkmwm : in std_logic_vector(6 downto 4); 
        gpbbjwsprsj : in std_logic_vector(2 downto 0); 
        vvpkfmmpprk : in std_logic_vector(2 downto 0); 
        mmsccbknsvn : out std_logic_vector(15 downto 12); 
        ggfvvjbkdws : in std_logic_vector(6 downto 1); 
        crpwsdfhcqq_4, crpwsdfhcqq_0, \gpbbjwsprsj_0[0]\, 
        \mwtvnzdkmwm_ns_1[6]\ : in std_logic; srdtcqsfmmm_i_0_0, 
        srdtcqsfmmm_i_0_5, srdtcqsfmmm_i_0_1, srdtcqsfmmm_i_0_2, 
        srdtcqsfmmm_i_0_14, srdtcqsfmmm_i_0_12, srdtcqsfmmm_i_0_6, 
        srdtcqsfmmm_i_0_3, srdtcqsfmmm_i_0_4, srdtcqsfmmm_i_0_15, 
        srdtcqsfmmm_i_0_16, srdtcqsfmmm_i_0_13 : out std_logic; 
        \INIT_ADDR_int[0]\ : in std_logic; \wqwwrvhjsjn_i[0]\ : 
        out std_logic; bfcqzvtnrcn_0, qdvwjxmzbpf : in std_logic; 
        N_4378, N_4134_1 : out std_logic; N_101 : in std_logic; 
        N_4432, N_4220, N_4221 : out std_logic; N_70 : in
         std_logic; N_4222, N_3943, N_4224, N_4225, N_4223 : out
         std_logic; bfcqzvtnrcn, N_7 : in std_logic; N_4294 : out
         std_logic; N_41, N_20, ADD_13x13_slow_I9_S_0, N_24, 
        ADD_13x13_slow_I7_S_0, N_44 : in std_logic; 
        twxmgvvpbgm_iv_0_0_0_a2_0 : out std_logic; xbkjkgdsbfh : 
        in std_logic; N_2772, cbfkvdxnxdp_i_0 : out std_logic; 
        fhpsqkqvwpw, dxpbcqjpxtw : in std_logic; 
        twxmgvvpbgm_iv_0_0_o3_i_o3 : out std_logic; N_25 : in
         std_logic; xkvhpkkjhmx_i_0 : out std_logic; N_31, N178, 
        ADD_13x13_slow_I8_S_0 : in std_logic; N_2497_i_0 : out
         std_logic);

end INITCFG_XC;

architecture DEF_ARCH of INITCFG_XC is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component ffppvfwnggt
    port(ggfvvjbkdws : 
        in std_logic_vector(6 downto 1) := (others => 'U'); 
        vvpkfmmpprk : 
        in std_logic_vector(2 downto 0) := (others => 'U'); 
        gpbbjwsprsj : 
        in std_logic_vector(2 downto 0) := (others => 'U'); 
        mwtvnzdkmwm : 
        in std_logic_vector(6 downto 4) := (others => 'U'); 
        \wqwwrvhjsjn_i[0]\ : out std_logic; \INIT_ADDR_int[0]\ : 
        in std_logic := 'U'; srdtcqsfmmm_i_0_13, 
        srdtcqsfmmm_i_0_16, srdtcqsfmmm_i_0_15, srdtcqsfmmm_i_0_4, 
        srdtcqsfmmm_i_0_3, srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_12, 
        srdtcqsfmmm_i_0_14, srdtcqsfmmm_i_0_2, srdtcqsfmmm_i_0_1, 
        srdtcqsfmmm_i_0_5, srdtcqsfmmm_i_0_0, mmsccbknsvn_3, 
        mmsccbknsvn_1, mmsccbknsvn_2, mmsccbknsvn_0 : out
         std_logic; \mwtvnzdkmwm_ns_1[6]\, \gpbbjwsprsj_0[0]\, 
        crpwsdfhcqq_0, crpwsdfhcqq_4 : in std_logic := 'U'; 
        N_2497_i_0 : out std_logic; ADD_13x13_slow_I8_S_0, N178, 
        N_31 : in std_logic := 'U'; xkvhpkkjhmx_i_0 : out
         std_logic; N_25 : in std_logic := 'U'; 
        twxmgvvpbgm_iv_0_0_o3_i_o3 : out std_logic; dxpbcqjpxtw, 
        fhpsqkqvwpw : in std_logic := 'U'; cbfkvdxnxdp_i_0, 
        N_2772 : out std_logic; xbkjkgdsbfh : in std_logic := 'U'; 
        twxmgvvpbgm_iv_0_0_0_a2_0 : out std_logic; N_44, 
        ADD_13x13_slow_I7_S_0, N_24, ADD_13x13_slow_I9_S_0, N_20, 
        N_41 : in std_logic := 'U'; N_4294 : out std_logic; N_7, 
        bfcqzvtnrcn : in std_logic := 'U'; N_4223, N_4225, N_4224, 
        N_3943, N_4222 : out std_logic; N_70 : in
         std_logic := 'U'; N_4221, N_4220, N_4432 : out std_logic; 
        N_101 : in std_logic := 'U'; N_4134_1, N_4378 : out
         std_logic; qdvwjxmzbpf, bfcqzvtnrcn_0 : in
         std_logic := 'U');
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : ffppvfwnggt
	Use entity work.ffppvfwnggt(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    u_ffppvfwnggt : ffppvfwnggt
      port map(ggfvvjbkdws(6) => ggfvvjbkdws(6), ggfvvjbkdws(5)
         => ggfvvjbkdws(5), ggfvvjbkdws(4) => ggfvvjbkdws(4), 
        ggfvvjbkdws(3) => ggfvvjbkdws(3), ggfvvjbkdws(2) => 
        ggfvvjbkdws(2), ggfvvjbkdws(1) => ggfvvjbkdws(1), 
        vvpkfmmpprk(2) => vvpkfmmpprk(2), vvpkfmmpprk(1) => 
        vvpkfmmpprk(1), vvpkfmmpprk(0) => vvpkfmmpprk(0), 
        gpbbjwsprsj(2) => gpbbjwsprsj(2), gpbbjwsprsj(1) => 
        gpbbjwsprsj(1), gpbbjwsprsj(0) => gpbbjwsprsj(0), 
        mwtvnzdkmwm(6) => mwtvnzdkmwm(6), mwtvnzdkmwm(5) => 
        mwtvnzdkmwm(5), mwtvnzdkmwm(4) => mwtvnzdkmwm(4), 
        \wqwwrvhjsjn_i[0]\ => \wqwwrvhjsjn_i[0]\, 
        \INIT_ADDR_int[0]\ => \INIT_ADDR_int[0]\, 
        srdtcqsfmmm_i_0_13 => srdtcqsfmmm_i_0_13, 
        srdtcqsfmmm_i_0_16 => srdtcqsfmmm_i_0_16, 
        srdtcqsfmmm_i_0_15 => srdtcqsfmmm_i_0_15, 
        srdtcqsfmmm_i_0_4 => srdtcqsfmmm_i_0_4, srdtcqsfmmm_i_0_3
         => srdtcqsfmmm_i_0_3, srdtcqsfmmm_i_0_6 => 
        srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_12 => 
        srdtcqsfmmm_i_0_12, srdtcqsfmmm_i_0_14 => 
        srdtcqsfmmm_i_0_14, srdtcqsfmmm_i_0_2 => 
        srdtcqsfmmm_i_0_2, srdtcqsfmmm_i_0_1 => srdtcqsfmmm_i_0_1, 
        srdtcqsfmmm_i_0_5 => srdtcqsfmmm_i_0_5, srdtcqsfmmm_i_0_0
         => srdtcqsfmmm_i_0_0, mmsccbknsvn_3 => mmsccbknsvn(15), 
        mmsccbknsvn_1 => mmsccbknsvn(13), mmsccbknsvn_2 => 
        mmsccbknsvn(14), mmsccbknsvn_0 => mmsccbknsvn(12), 
        \mwtvnzdkmwm_ns_1[6]\ => \mwtvnzdkmwm_ns_1[6]\, 
        \gpbbjwsprsj_0[0]\ => \gpbbjwsprsj_0[0]\, crpwsdfhcqq_0
         => crpwsdfhcqq_0, crpwsdfhcqq_4 => crpwsdfhcqq_4, 
        N_2497_i_0 => N_2497_i_0, ADD_13x13_slow_I8_S_0 => 
        ADD_13x13_slow_I8_S_0, N178 => N178, N_31 => N_31, 
        xkvhpkkjhmx_i_0 => xkvhpkkjhmx_i_0, N_25 => N_25, 
        twxmgvvpbgm_iv_0_0_o3_i_o3 => twxmgvvpbgm_iv_0_0_o3_i_o3, 
        dxpbcqjpxtw => dxpbcqjpxtw, fhpsqkqvwpw => fhpsqkqvwpw, 
        cbfkvdxnxdp_i_0 => cbfkvdxnxdp_i_0, N_2772 => N_2772, 
        xbkjkgdsbfh => xbkjkgdsbfh, twxmgvvpbgm_iv_0_0_0_a2_0 => 
        twxmgvvpbgm_iv_0_0_0_a2_0, N_44 => N_44, 
        ADD_13x13_slow_I7_S_0 => ADD_13x13_slow_I7_S_0, N_24 => 
        N_24, ADD_13x13_slow_I9_S_0 => ADD_13x13_slow_I9_S_0, 
        N_20 => N_20, N_41 => N_41, N_4294 => N_4294, N_7 => N_7, 
        bfcqzvtnrcn => bfcqzvtnrcn, N_4223 => N_4223, N_4225 => 
        N_4225, N_4224 => N_4224, N_3943 => N_3943, N_4222 => 
        N_4222, N_70 => N_70, N_4221 => N_4221, N_4220 => N_4220, 
        N_4432 => N_4432, N_101 => N_101, N_4134_1 => N_4134_1, 
        N_4378 => N_4378, qdvwjxmzbpf => qdvwjxmzbpf, 
        bfcqzvtnrcn_0 => bfcqzvtnrcn_0);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity wbsfnwwctpn is

    port(vvpkfmmpprk : out std_logic_vector(2 downto 0); 
        crpwsdfhcqq_7, crpwsdfhcqq_3 : out std_logic; 
        sys_reset_gl, ACM_CLK_c : in std_logic; fhpsqkqvwpw, 
        npgnwznthhx, N_101 : out std_logic; NVM_BUSY_net, 
        mhxwcfnvbgt : in std_logic);

end wbsfnwwctpn;

architecture DEF_ARCH of wbsfnwwctpn is 

  component DFN1C0
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component AO1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component OR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component OR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MAJ3X
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1E1C0
    port(D, CLK, CLR, E : in std_logic := 'U'; Q : out std_logic);
        
  end component;

  component XOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AXOI5
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \crpwsdfhcqq[7]_net_1\, \crpwsdfhcqq[3]_net_1\, 
        \vvpkfmmpprk[0]\, \vvpkfmmpprk[1]\, \vvpkfmmpprk[2]\, 
        \fhpsqkqvwpw\, \npgnwznthhx\, \N_101\, N_111, jspcghhzwmg, 
        \crpwsdfhcqq_ns[6]\, mfwjfgrqchd, 
        \crpwsdfhcqq_ns_a3_0_a2[1]_net_1\, N_87, 
        \bzcdnkppzhj_2_i_0\, N_81, \crpwsdfhcqq_ns[5]\, 
        \crpwsdfhcqq[0]_net_1\, \crpwsdfhcqq_ns_a3_0_a2[4]_net_1\, 
        \crpwsdfhcqq_ns_i[7]_net_1\, \crpwsdfhcqq[1]_net_1\, 
        \crpwsdfhcqq_ns[3]\, \crpwsdfhcqq_ns_a3_0_a2[8]_net_1\, 
        \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

begin 

    vvpkfmmpprk(2) <= \vvpkfmmpprk[2]\;
    vvpkfmmpprk(1) <= \vvpkfmmpprk[1]\;
    vvpkfmmpprk(0) <= \vvpkfmmpprk[0]\;
    crpwsdfhcqq_7 <= \crpwsdfhcqq[7]_net_1\;
    crpwsdfhcqq_3 <= \crpwsdfhcqq[3]_net_1\;
    fhpsqkqvwpw <= \fhpsqkqvwpw\;
    npgnwznthhx <= \npgnwznthhx\;
    N_101 <= \N_101\;

    \crpwsdfhcqq[0]\ : DFN1C0
      port map(D => \crpwsdfhcqq_ns_a3_0_a2[8]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \crpwsdfhcqq[0]_net_1\);
    
    \crpwsdfhcqq[3]\ : DFN1C0
      port map(D => \crpwsdfhcqq_ns[5]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \crpwsdfhcqq[3]_net_1\);
    
    \crpwsdfhcqq_ns_0[5]\ : AO1
      port map(A => mhxwcfnvbgt, B => \crpwsdfhcqq[0]_net_1\, C
         => \crpwsdfhcqq[7]_net_1\, Y => \crpwsdfhcqq_ns[5]\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    \crpwsdfhcqq_ns_0_a2[6]\ : OR2B
      port map(A => mhxwcfnvbgt, B => jspcghhzwmg, Y => N_111);
    
    \crpwsdfhcqq_ns_0[6]\ : AO1B
      port map(A => NVM_BUSY_net, B => mfwjfgrqchd, C => N_111, Y
         => \crpwsdfhcqq_ns[6]\);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    \bzcdnkppzhj[0]\ : DFN1C0
      port map(D => \bzcdnkppzhj_2_i_0\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \vvpkfmmpprk[0]\);
    
    bzcdnkppzhj_2_i_0_o3 : OR2A
      port map(A => \vvpkfmmpprk[2]\, B => \vvpkfmmpprk[1]\, Y
         => \N_101\);
    
    \crpwsdfhcqq_ns_0_a2[3]\ : OA1B
      port map(A => jspcghhzwmg, B => \crpwsdfhcqq[0]_net_1\, C
         => mhxwcfnvbgt, Y => \fhpsqkqvwpw\);
    
    \bzcdnkppzhj_3_i[2]\ : MAJ3X
      port map(A => \vvpkfmmpprk[1]\, B => \vvpkfmmpprk[0]\, C
         => \vvpkfmmpprk[2]\, Y => N_87);
    
    GND_i : GND
      port map(Y => \GND\);
    
    \crpwsdfhcqq[1]\ : DFN1C0
      port map(D => \crpwsdfhcqq_ns_i[7]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \crpwsdfhcqq[1]_net_1\);
    
    \crpwsdfhcqq_ns_i[7]\ : MX2
      port map(A => mfwjfgrqchd, B => \crpwsdfhcqq[1]_net_1\, S
         => NVM_BUSY_net, Y => \crpwsdfhcqq_ns_i[7]_net_1\);
    
    \crpwsdfhcqq[4]\ : DFN1C0
      port map(D => \crpwsdfhcqq_ns_a3_0_a2[4]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => jspcghhzwmg);
    
    \crpwsdfhcqq[2]\ : DFN1C0
      port map(D => \crpwsdfhcqq_ns[6]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => mfwjfgrqchd);
    
    \bzcdnkppzhj[2]\ : DFN1E1C0
      port map(D => N_87, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        E => \crpwsdfhcqq[3]_net_1\, Q => \vvpkfmmpprk[2]\);
    
    un6_bzcdnkppzhj_1_SUM1_0_x2 : XOR2
      port map(A => \vvpkfmmpprk[1]\, B => \vvpkfmmpprk[0]\, Y
         => N_81);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    \crpwsdfhcqq_ns_0[3]\ : AO1
      port map(A => NVM_BUSY_net, B => \npgnwznthhx\, C => 
        \fhpsqkqvwpw\, Y => \crpwsdfhcqq_ns[3]\);
    
    bzcdnkppzhj_2_i_0 : AXOI5
      port map(A => \N_101\, B => \crpwsdfhcqq[3]_net_1\, C => 
        \vvpkfmmpprk[0]\, Y => \bzcdnkppzhj_2_i_0\);
    
    \crpwsdfhcqq_ns_a3_0_a2[4]\ : NOR2A
      port map(A => \npgnwznthhx\, B => NVM_BUSY_net, Y => 
        \crpwsdfhcqq_ns_a3_0_a2[4]_net_1\);
    
    \crpwsdfhcqq[7]\ : DFN1C0
      port map(D => \crpwsdfhcqq_ns_a3_0_a2[1]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \crpwsdfhcqq[7]_net_1\);
    
    \bzcdnkppzhj[1]\ : DFN1E1C0
      port map(D => N_81, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        E => \crpwsdfhcqq[3]_net_1\, Q => \vvpkfmmpprk[1]\);
    
    \crpwsdfhcqq_ns_a3_0_a2[1]\ : OA1
      port map(A => \N_101\, B => \vvpkfmmpprk[0]\, C => 
        \crpwsdfhcqq[3]_net_1\, Y => 
        \crpwsdfhcqq_ns_a3_0_a2[1]_net_1\);
    
    \crpwsdfhcqq_ns_a3_0_a2[8]\ : NOR2A
      port map(A => \crpwsdfhcqq[1]_net_1\, B => NVM_BUSY_net, Y
         => \crpwsdfhcqq_ns_a3_0_a2[8]_net_1\);
    
    \crpwsdfhcqq[5]\ : DFN1C0
      port map(D => \crpwsdfhcqq_ns[3]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \npgnwznthhx\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity INITCFG_XB is

    port(vvpkfmmpprk : out std_logic_vector(2 downto 0); 
        crpwsdfhcqq_0, crpwsdfhcqq_4 : out std_logic; mhxwcfnvbgt, 
        NVM_BUSY_net : in std_logic; N_101, npgnwznthhx, 
        fhpsqkqvwpw : out std_logic; ACM_CLK_c, sys_reset_gl : in
         std_logic);

end INITCFG_XB;

architecture DEF_ARCH of INITCFG_XB is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component wbsfnwwctpn
    port(vvpkfmmpprk : out std_logic_vector(2 downto 0); 
        crpwsdfhcqq_7, crpwsdfhcqq_3 : out std_logic; 
        sys_reset_gl, ACM_CLK_c : in std_logic := 'U'; 
        fhpsqkqvwpw, npgnwznthhx, N_101 : out std_logic; 
        NVM_BUSY_net, mhxwcfnvbgt : in std_logic := 'U');
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : wbsfnwwctpn
	Use entity work.wbsfnwwctpn(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    u_wbsfnwwctpn : wbsfnwwctpn
      port map(vvpkfmmpprk(2) => vvpkfmmpprk(2), vvpkfmmpprk(1)
         => vvpkfmmpprk(1), vvpkfmmpprk(0) => vvpkfmmpprk(0), 
        crpwsdfhcqq_7 => crpwsdfhcqq_4, crpwsdfhcqq_3 => 
        crpwsdfhcqq_0, sys_reset_gl => sys_reset_gl, ACM_CLK_c
         => ACM_CLK_c, fhpsqkqvwpw => fhpsqkqvwpw, npgnwznthhx
         => npgnwznthhx, N_101 => N_101, NVM_BUSY_net => 
        NVM_BUSY_net, mhxwcfnvbgt => mhxwcfnvbgt);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity grwzcnqmqzm is

    port(INIT_ADDR_int : out std_logic_vector(8 downto 0); 
        srdtcqsfmmm_i_0_13, srdtcqsfmmm_i_0_16, 
        srdtcqsfmmm_i_0_15, srdtcqsfmmm_i_0_4, srdtcqsfmmm_i_0_3, 
        srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_12, srdtcqsfmmm_i_0_14, 
        srdtcqsfmmm_i_0_2, srdtcqsfmmm_i_0_1, srdtcqsfmmm_i_0_5, 
        srdtcqsfmmm_i_0_0, hrgmtcpsdzr, N_12, INIT_EV_WEN, N_14, 
        INIT_ASSC_WEN, N_2497_i_0, twxmgvvpbgm_iv_0_0_o3_i_o3, 
        twxmgvvpbgm_iv_0_0_0_a2_0 : out std_logic; NVM_BUSY_net, 
        sys_reset_gl, ACM_CLK_c : in std_logic);

end grwzcnqmqzm;

architecture DEF_ARCH of grwzcnqmqzm is 

  component VCC
    port(Y : out std_logic);
  end component;

  component INITCFG_XF
    port(bfcqzvtnrcn_0 : out std_logic; ACM_CLK_c, sys_reset_gl : 
        in std_logic := 'U'; bfcqzvtnrcn : out std_logic; 
        qdvwjxmzbpf : in std_logic := 'U'; hrgmtcpsdzr : out
         std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component INITCFG_XA
    port(mwtvnzdkmwm : out std_logic_vector(6 downto 4); 
        gpbbjwsprsj : out std_logic_vector(2 downto 0); 
        \gpbbjwsprsj_0[0]\, \mwtvnzdkmwm_ns_1[6]\ : out std_logic; 
        ACM_CLK_c, sys_reset_gl, NVM_BUSY_net : in
         std_logic := 'U'; dxpbcqjpxtw, xbkjkgdsbfh : out
         std_logic; mhxwcfnvbgt, bfcqzvtnrcn : in
         std_logic := 'U'; N_7, N_70, qdvwjxmzbpf : out std_logic);
        
  end component;

  component INITCFG_XD
    port(INIT_ADDR_int : out std_logic_vector(8 downto 0); 
        mmsccbknsvn : 
        in std_logic_vector(15 downto 12) := (others => 'U'); 
        ggfvvjbkdws : out std_logic_vector(6 downto 1); 
        \wqwwrvhjsjn_i[0]\ : in std_logic := 'U'; 
        ADD_13x13_slow_I9_S_0, ADD_13x13_slow_I8_S_0, 
        ADD_13x13_slow_I7_S_0 : out std_logic; N_2772 : in
         std_logic := 'U'; N178, N_24 : out std_logic; 
        xkvhpkkjhmx_i_0 : in std_logic := 'U'; N_44, N_41 : out
         std_logic; N_4222, N_4223, N_4220, N_4221, N_4224, 
        N_4225, N_4134_1, N_4378, N_3943 : in std_logic := 'U'; 
        N_20, N_25, N_31, mhxwcfnvbgt : out std_logic; ACM_CLK_c, 
        sys_reset_gl, cbfkvdxnxdp_i_0 : in std_logic := 'U');
  end component;

  component INITCFG_XE
    port(\gpbbjwsprsj_0[0]\, \vvpkfmmpprk[0]\, \gpbbjwsprsj[0]\, 
        \mwtvnzdkmwm_ns_1[6]\, npgnwznthhx, N_4432, N_4294 : in
         std_logic := 'U'; INIT_ASSC_WEN, N_14 : out std_logic; 
        xbkjkgdsbfh, NVM_BUSY_net : in std_logic := 'U'; 
        INIT_EV_WEN, N_12 : out std_logic; N_101 : in
         std_logic := 'U');
  end component;

  component INITCFG_XC
    port(mwtvnzdkmwm : 
        in std_logic_vector(6 downto 4) := (others => 'U'); 
        gpbbjwsprsj : 
        in std_logic_vector(2 downto 0) := (others => 'U'); 
        vvpkfmmpprk : 
        in std_logic_vector(2 downto 0) := (others => 'U'); 
        mmsccbknsvn : out std_logic_vector(15 downto 12); 
        ggfvvjbkdws : 
        in std_logic_vector(6 downto 1) := (others => 'U'); 
        crpwsdfhcqq_4, crpwsdfhcqq_0, \gpbbjwsprsj_0[0]\, 
        \mwtvnzdkmwm_ns_1[6]\ : in std_logic := 'U'; 
        srdtcqsfmmm_i_0_0, srdtcqsfmmm_i_0_5, srdtcqsfmmm_i_0_1, 
        srdtcqsfmmm_i_0_2, srdtcqsfmmm_i_0_14, srdtcqsfmmm_i_0_12, 
        srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_3, srdtcqsfmmm_i_0_4, 
        srdtcqsfmmm_i_0_15, srdtcqsfmmm_i_0_16, 
        srdtcqsfmmm_i_0_13 : out std_logic; \INIT_ADDR_int[0]\ : 
        in std_logic := 'U'; \wqwwrvhjsjn_i[0]\ : out std_logic; 
        bfcqzvtnrcn_0, qdvwjxmzbpf : in std_logic := 'U'; N_4378, 
        N_4134_1 : out std_logic; N_101 : in std_logic := 'U'; 
        N_4432, N_4220, N_4221 : out std_logic; N_70 : in
         std_logic := 'U'; N_4222, N_3943, N_4224, N_4225, N_4223 : 
        out std_logic; bfcqzvtnrcn, N_7 : in std_logic := 'U'; 
        N_4294 : out std_logic; N_41, N_20, ADD_13x13_slow_I9_S_0, 
        N_24, ADD_13x13_slow_I7_S_0, N_44 : in std_logic := 'U'; 
        twxmgvvpbgm_iv_0_0_0_a2_0 : out std_logic; xbkjkgdsbfh : 
        in std_logic := 'U'; N_2772, cbfkvdxnxdp_i_0 : out
         std_logic; fhpsqkqvwpw, dxpbcqjpxtw : in
         std_logic := 'U'; twxmgvvpbgm_iv_0_0_o3_i_o3 : out
         std_logic; N_25 : in std_logic := 'U'; xkvhpkkjhmx_i_0 : 
        out std_logic; N_31, N178, ADD_13x13_slow_I8_S_0 : in
         std_logic := 'U'; N_2497_i_0 : out std_logic);
  end component;

  component INITCFG_XB
    port(vvpkfmmpprk : out std_logic_vector(2 downto 0); 
        crpwsdfhcqq_0, crpwsdfhcqq_4 : out std_logic; mhxwcfnvbgt, 
        NVM_BUSY_net : in std_logic := 'U'; N_101, npgnwznthhx, 
        fhpsqkqvwpw : out std_logic; ACM_CLK_c, sys_reset_gl : in
         std_logic := 'U');
  end component;

    signal \INIT_ADDR_int[0]\, \gpbbjwsprsj_0[0]\, 
        \mwtvnzdkmwm_ns_1[6]\, \mwtvnzdkmwm[4]\, \mwtvnzdkmwm[5]\, 
        \mwtvnzdkmwm[6]\, \gpbbjwsprsj[0]\, \gpbbjwsprsj[1]\, 
        \gpbbjwsprsj[2]\, dxpbcqjpxtw, xbkjkgdsbfh, mhxwcfnvbgt, 
        bfcqzvtnrcn, N_7, N_70, qdvwjxmzbpf, \vvpkfmmpprk[0]\, 
        \vvpkfmmpprk[1]\, \vvpkfmmpprk[2]\, \crpwsdfhcqq[3]\, 
        \crpwsdfhcqq[7]\, N_101, npgnwznthhx, fhpsqkqvwpw, 
        \mmsccbknsvn[12]\, \mmsccbknsvn[13]\, \mmsccbknsvn[14]\, 
        \mmsccbknsvn[15]\, \wqwwrvhjsjn_i[0]\, \ggfvvjbkdws[1]\, 
        \ggfvvjbkdws[2]\, \ggfvvjbkdws[3]\, \ggfvvjbkdws[4]\, 
        \ggfvvjbkdws[5]\, \ggfvvjbkdws[6]\, bfcqzvtnrcn_0, N_4378, 
        N_4134_1, N_4432, N_4220, N_4221, N_4222, N_3943, N_4224, 
        N_4225, N_4223, N_4294, N_41, N_20, ADD_13x13_slow_I9_S_0, 
        N_24, ADD_13x13_slow_I7_S_0, N_44, N_2772, 
        cbfkvdxnxdp_i_0, N_25, xkvhpkkjhmx_i_0, N_31, N178, 
        ADD_13x13_slow_I8_S_0, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

    for all : INITCFG_XF
	Use entity work.INITCFG_XF(DEF_ARCH);
    for all : INITCFG_XA
	Use entity work.INITCFG_XA(DEF_ARCH);
    for all : INITCFG_XD
	Use entity work.INITCFG_XD(DEF_ARCH);
    for all : INITCFG_XE
	Use entity work.INITCFG_XE(DEF_ARCH);
    for all : INITCFG_XC
	Use entity work.INITCFG_XC(DEF_ARCH);
    for all : INITCFG_XB
	Use entity work.INITCFG_XB(DEF_ARCH);
begin 

    INIT_ADDR_int(0) <= \INIT_ADDR_int[0]\;

    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    user_clk_sel : INITCFG_XF
      port map(bfcqzvtnrcn_0 => bfcqzvtnrcn_0, ACM_CLK_c => 
        ACM_CLK_c, sys_reset_gl => sys_reset_gl, bfcqzvtnrcn => 
        bfcqzvtnrcn, qdvwjxmzbpf => qdvwjxmzbpf, hrgmtcpsdzr => 
        hrgmtcpsdzr);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    init_fsm : INITCFG_XA
      port map(mwtvnzdkmwm(6) => \mwtvnzdkmwm[6]\, mwtvnzdkmwm(5)
         => \mwtvnzdkmwm[5]\, mwtvnzdkmwm(4) => \mwtvnzdkmwm[4]\, 
        gpbbjwsprsj(2) => \gpbbjwsprsj[2]\, gpbbjwsprsj(1) => 
        \gpbbjwsprsj[1]\, gpbbjwsprsj(0) => \gpbbjwsprsj[0]\, 
        \gpbbjwsprsj_0[0]\ => \gpbbjwsprsj_0[0]\, 
        \mwtvnzdkmwm_ns_1[6]\ => \mwtvnzdkmwm_ns_1[6]\, ACM_CLK_c
         => ACM_CLK_c, sys_reset_gl => sys_reset_gl, NVM_BUSY_net
         => NVM_BUSY_net, dxpbcqjpxtw => dxpbcqjpxtw, xbkjkgdsbfh
         => xbkjkgdsbfh, mhxwcfnvbgt => mhxwcfnvbgt, bfcqzvtnrcn
         => bfcqzvtnrcn, N_7 => N_7, N_70 => N_70, qdvwjxmzbpf
         => qdvwjxmzbpf);
    
    address_gen : INITCFG_XD
      port map(INIT_ADDR_int(8) => INIT_ADDR_int(8), 
        INIT_ADDR_int(7) => INIT_ADDR_int(7), INIT_ADDR_int(6)
         => INIT_ADDR_int(6), INIT_ADDR_int(5) => 
        INIT_ADDR_int(5), INIT_ADDR_int(4) => INIT_ADDR_int(4), 
        INIT_ADDR_int(3) => INIT_ADDR_int(3), INIT_ADDR_int(2)
         => INIT_ADDR_int(2), INIT_ADDR_int(1) => 
        INIT_ADDR_int(1), INIT_ADDR_int(0) => \INIT_ADDR_int[0]\, 
        mmsccbknsvn(15) => \mmsccbknsvn[15]\, mmsccbknsvn(14) => 
        \mmsccbknsvn[14]\, mmsccbknsvn(13) => \mmsccbknsvn[13]\, 
        mmsccbknsvn(12) => \mmsccbknsvn[12]\, ggfvvjbkdws(6) => 
        \ggfvvjbkdws[6]\, ggfvvjbkdws(5) => \ggfvvjbkdws[5]\, 
        ggfvvjbkdws(4) => \ggfvvjbkdws[4]\, ggfvvjbkdws(3) => 
        \ggfvvjbkdws[3]\, ggfvvjbkdws(2) => \ggfvvjbkdws[2]\, 
        ggfvvjbkdws(1) => \ggfvvjbkdws[1]\, \wqwwrvhjsjn_i[0]\
         => \wqwwrvhjsjn_i[0]\, ADD_13x13_slow_I9_S_0 => 
        ADD_13x13_slow_I9_S_0, ADD_13x13_slow_I8_S_0 => 
        ADD_13x13_slow_I8_S_0, ADD_13x13_slow_I7_S_0 => 
        ADD_13x13_slow_I7_S_0, N_2772 => N_2772, N178 => N178, 
        N_24 => N_24, xkvhpkkjhmx_i_0 => xkvhpkkjhmx_i_0, N_44
         => N_44, N_41 => N_41, N_4222 => N_4222, N_4223 => 
        N_4223, N_4220 => N_4220, N_4221 => N_4221, N_4224 => 
        N_4224, N_4225 => N_4225, N_4134_1 => N_4134_1, N_4378
         => N_4378, N_3943 => N_3943, N_20 => N_20, N_25 => N_25, 
        N_31 => N_31, mhxwcfnvbgt => mhxwcfnvbgt, ACM_CLK_c => 
        ACM_CLK_c, sys_reset_gl => sys_reset_gl, cbfkvdxnxdp_i_0
         => cbfkvdxnxdp_i_0);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    valid_client : INITCFG_XE
      port map(\gpbbjwsprsj_0[0]\ => \gpbbjwsprsj_0[0]\, 
        \vvpkfmmpprk[0]\ => \vvpkfmmpprk[0]\, \gpbbjwsprsj[0]\
         => \gpbbjwsprsj[0]\, \mwtvnzdkmwm_ns_1[6]\ => 
        \mwtvnzdkmwm_ns_1[6]\, npgnwznthhx => npgnwznthhx, N_4432
         => N_4432, N_4294 => N_4294, INIT_ASSC_WEN => 
        INIT_ASSC_WEN, N_14 => N_14, xbkjkgdsbfh => xbkjkgdsbfh, 
        NVM_BUSY_net => NVM_BUSY_net, INIT_EV_WEN => INIT_EV_WEN, 
        N_12 => N_12, N_101 => N_101);
    
    user_control : INITCFG_XC
      port map(mwtvnzdkmwm(6) => \mwtvnzdkmwm[6]\, mwtvnzdkmwm(5)
         => \mwtvnzdkmwm[5]\, mwtvnzdkmwm(4) => \mwtvnzdkmwm[4]\, 
        gpbbjwsprsj(2) => \gpbbjwsprsj[2]\, gpbbjwsprsj(1) => 
        \gpbbjwsprsj[1]\, gpbbjwsprsj(0) => \gpbbjwsprsj[0]\, 
        vvpkfmmpprk(2) => \vvpkfmmpprk[2]\, vvpkfmmpprk(1) => 
        \vvpkfmmpprk[1]\, vvpkfmmpprk(0) => \vvpkfmmpprk[0]\, 
        mmsccbknsvn(15) => \mmsccbknsvn[15]\, mmsccbknsvn(14) => 
        \mmsccbknsvn[14]\, mmsccbknsvn(13) => \mmsccbknsvn[13]\, 
        mmsccbknsvn(12) => \mmsccbknsvn[12]\, ggfvvjbkdws(6) => 
        \ggfvvjbkdws[6]\, ggfvvjbkdws(5) => \ggfvvjbkdws[5]\, 
        ggfvvjbkdws(4) => \ggfvvjbkdws[4]\, ggfvvjbkdws(3) => 
        \ggfvvjbkdws[3]\, ggfvvjbkdws(2) => \ggfvvjbkdws[2]\, 
        ggfvvjbkdws(1) => \ggfvvjbkdws[1]\, crpwsdfhcqq_4 => 
        \crpwsdfhcqq[7]\, crpwsdfhcqq_0 => \crpwsdfhcqq[3]\, 
        \gpbbjwsprsj_0[0]\ => \gpbbjwsprsj_0[0]\, 
        \mwtvnzdkmwm_ns_1[6]\ => \mwtvnzdkmwm_ns_1[6]\, 
        srdtcqsfmmm_i_0_0 => srdtcqsfmmm_i_0_0, srdtcqsfmmm_i_0_5
         => srdtcqsfmmm_i_0_5, srdtcqsfmmm_i_0_1 => 
        srdtcqsfmmm_i_0_1, srdtcqsfmmm_i_0_2 => srdtcqsfmmm_i_0_2, 
        srdtcqsfmmm_i_0_14 => srdtcqsfmmm_i_0_14, 
        srdtcqsfmmm_i_0_12 => srdtcqsfmmm_i_0_12, 
        srdtcqsfmmm_i_0_6 => srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_3
         => srdtcqsfmmm_i_0_3, srdtcqsfmmm_i_0_4 => 
        srdtcqsfmmm_i_0_4, srdtcqsfmmm_i_0_15 => 
        srdtcqsfmmm_i_0_15, srdtcqsfmmm_i_0_16 => 
        srdtcqsfmmm_i_0_16, srdtcqsfmmm_i_0_13 => 
        srdtcqsfmmm_i_0_13, \INIT_ADDR_int[0]\ => 
        \INIT_ADDR_int[0]\, \wqwwrvhjsjn_i[0]\ => 
        \wqwwrvhjsjn_i[0]\, bfcqzvtnrcn_0 => bfcqzvtnrcn_0, 
        qdvwjxmzbpf => qdvwjxmzbpf, N_4378 => N_4378, N_4134_1
         => N_4134_1, N_101 => N_101, N_4432 => N_4432, N_4220
         => N_4220, N_4221 => N_4221, N_70 => N_70, N_4222 => 
        N_4222, N_3943 => N_3943, N_4224 => N_4224, N_4225 => 
        N_4225, N_4223 => N_4223, bfcqzvtnrcn => bfcqzvtnrcn, N_7
         => N_7, N_4294 => N_4294, N_41 => N_41, N_20 => N_20, 
        ADD_13x13_slow_I9_S_0 => ADD_13x13_slow_I9_S_0, N_24 => 
        N_24, ADD_13x13_slow_I7_S_0 => ADD_13x13_slow_I7_S_0, 
        N_44 => N_44, twxmgvvpbgm_iv_0_0_0_a2_0 => 
        twxmgvvpbgm_iv_0_0_0_a2_0, xbkjkgdsbfh => xbkjkgdsbfh, 
        N_2772 => N_2772, cbfkvdxnxdp_i_0 => cbfkvdxnxdp_i_0, 
        fhpsqkqvwpw => fhpsqkqvwpw, dxpbcqjpxtw => dxpbcqjpxtw, 
        twxmgvvpbgm_iv_0_0_o3_i_o3 => twxmgvvpbgm_iv_0_0_o3_i_o3, 
        N_25 => N_25, xkvhpkkjhmx_i_0 => xkvhpkkjhmx_i_0, N_31
         => N_31, N178 => N178, ADD_13x13_slow_I8_S_0 => 
        ADD_13x13_slow_I8_S_0, N_2497_i_0 => N_2497_i_0);
    
    save_fsm : INITCFG_XB
      port map(vvpkfmmpprk(2) => \vvpkfmmpprk[2]\, vvpkfmmpprk(1)
         => \vvpkfmmpprk[1]\, vvpkfmmpprk(0) => \vvpkfmmpprk[0]\, 
        crpwsdfhcqq_0 => \crpwsdfhcqq[3]\, crpwsdfhcqq_4 => 
        \crpwsdfhcqq[7]\, mhxwcfnvbgt => mhxwcfnvbgt, 
        NVM_BUSY_net => NVM_BUSY_net, N_101 => N_101, npgnwznthhx
         => npgnwznthhx, fhpsqkqvwpw => fhpsqkqvwpw, ACM_CLK_c
         => ACM_CLK_c, sys_reset_gl => sys_reset_gl);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity INITCFG is

    port(INIT_ADDR_int : out std_logic_vector(8 downto 0); 
        srdtcqsfmmm_i_0_0, srdtcqsfmmm_i_0_5, srdtcqsfmmm_i_0_1, 
        srdtcqsfmmm_i_0_2, srdtcqsfmmm_i_0_14, srdtcqsfmmm_i_0_12, 
        srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_3, srdtcqsfmmm_i_0_4, 
        srdtcqsfmmm_i_0_15, srdtcqsfmmm_i_0_16, 
        srdtcqsfmmm_i_0_13 : out std_logic; ACM_CLK_c, 
        sys_reset_gl, NVM_BUSY_net : in std_logic; 
        twxmgvvpbgm_iv_0_0_0_a2_0, twxmgvvpbgm_iv_0_0_o3_i_o3, 
        N_2497_i_0, INIT_ASSC_WEN, N_14, INIT_EV_WEN, N_12, 
        hrgmtcpsdzr : out std_logic);

end INITCFG;

architecture DEF_ARCH of INITCFG is 

  component VCC
    port(Y : out std_logic);
  end component;

  component grwzcnqmqzm
    port(INIT_ADDR_int : out std_logic_vector(8 downto 0); 
        srdtcqsfmmm_i_0_13, srdtcqsfmmm_i_0_16, 
        srdtcqsfmmm_i_0_15, srdtcqsfmmm_i_0_4, srdtcqsfmmm_i_0_3, 
        srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_12, srdtcqsfmmm_i_0_14, 
        srdtcqsfmmm_i_0_2, srdtcqsfmmm_i_0_1, srdtcqsfmmm_i_0_5, 
        srdtcqsfmmm_i_0_0, hrgmtcpsdzr, N_12, INIT_EV_WEN, N_14, 
        INIT_ASSC_WEN, N_2497_i_0, twxmgvvpbgm_iv_0_0_o3_i_o3, 
        twxmgvvpbgm_iv_0_0_0_a2_0 : out std_logic; NVM_BUSY_net, 
        sys_reset_gl, ACM_CLK_c : in std_logic := 'U');
  end component;

  component GND
    port(Y : out std_logic);
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : grwzcnqmqzm
	Use entity work.grwzcnqmqzm(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    u_grwzcnqmqzm : grwzcnqmqzm
      port map(INIT_ADDR_int(8) => INIT_ADDR_int(8), 
        INIT_ADDR_int(7) => INIT_ADDR_int(7), INIT_ADDR_int(6)
         => INIT_ADDR_int(6), INIT_ADDR_int(5) => 
        INIT_ADDR_int(5), INIT_ADDR_int(4) => INIT_ADDR_int(4), 
        INIT_ADDR_int(3) => INIT_ADDR_int(3), INIT_ADDR_int(2)
         => INIT_ADDR_int(2), INIT_ADDR_int(1) => 
        INIT_ADDR_int(1), INIT_ADDR_int(0) => INIT_ADDR_int(0), 
        srdtcqsfmmm_i_0_13 => srdtcqsfmmm_i_0_13, 
        srdtcqsfmmm_i_0_16 => srdtcqsfmmm_i_0_16, 
        srdtcqsfmmm_i_0_15 => srdtcqsfmmm_i_0_15, 
        srdtcqsfmmm_i_0_4 => srdtcqsfmmm_i_0_4, srdtcqsfmmm_i_0_3
         => srdtcqsfmmm_i_0_3, srdtcqsfmmm_i_0_6 => 
        srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_12 => 
        srdtcqsfmmm_i_0_12, srdtcqsfmmm_i_0_14 => 
        srdtcqsfmmm_i_0_14, srdtcqsfmmm_i_0_2 => 
        srdtcqsfmmm_i_0_2, srdtcqsfmmm_i_0_1 => srdtcqsfmmm_i_0_1, 
        srdtcqsfmmm_i_0_5 => srdtcqsfmmm_i_0_5, srdtcqsfmmm_i_0_0
         => srdtcqsfmmm_i_0_0, hrgmtcpsdzr => hrgmtcpsdzr, N_12
         => N_12, INIT_EV_WEN => INIT_EV_WEN, N_14 => N_14, 
        INIT_ASSC_WEN => INIT_ASSC_WEN, N_2497_i_0 => N_2497_i_0, 
        twxmgvvpbgm_iv_0_0_o3_i_o3 => twxmgvvpbgm_iv_0_0_o3_i_o3, 
        twxmgvvpbgm_iv_0_0_0_a2_0 => twxmgvvpbgm_iv_0_0_0_a2_0, 
        NVM_BUSY_net => NVM_BUSY_net, sys_reset_gl => 
        sys_reset_gl, ACM_CLK_c => ACM_CLK_c);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity NVM_analog_init_wrapper is

    port(INIT_ADDR_int : out std_logic_vector(8 downto 0); 
        srdtcqsfmmm_i_0_13, srdtcqsfmmm_i_0_16, 
        srdtcqsfmmm_i_0_15, srdtcqsfmmm_i_0_4, srdtcqsfmmm_i_0_3, 
        srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_12, srdtcqsfmmm_i_0_14, 
        srdtcqsfmmm_i_0_2, srdtcqsfmmm_i_0_1, srdtcqsfmmm_i_0_5, 
        srdtcqsfmmm_i_0_0, hrgmtcpsdzr, N_12, INIT_EV_WEN, N_14, 
        INIT_ASSC_WEN, N_2497_i_0, twxmgvvpbgm_iv_0_0_o3_i_o3, 
        twxmgvvpbgm_iv_0_0_0_a2_0 : out std_logic; NVM_BUSY_net, 
        sys_reset_gl, ACM_CLK_c : in std_logic);

end NVM_analog_init_wrapper;

architecture DEF_ARCH of NVM_analog_init_wrapper is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component INITCFG
    port(INIT_ADDR_int : out std_logic_vector(8 downto 0); 
        srdtcqsfmmm_i_0_0, srdtcqsfmmm_i_0_5, srdtcqsfmmm_i_0_1, 
        srdtcqsfmmm_i_0_2, srdtcqsfmmm_i_0_14, srdtcqsfmmm_i_0_12, 
        srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_3, srdtcqsfmmm_i_0_4, 
        srdtcqsfmmm_i_0_15, srdtcqsfmmm_i_0_16, 
        srdtcqsfmmm_i_0_13 : out std_logic; ACM_CLK_c, 
        sys_reset_gl, NVM_BUSY_net : in std_logic := 'U'; 
        twxmgvvpbgm_iv_0_0_0_a2_0, twxmgvvpbgm_iv_0_0_o3_i_o3, 
        N_2497_i_0, INIT_ASSC_WEN, N_14, INIT_EV_WEN, N_12, 
        hrgmtcpsdzr : out std_logic);
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : INITCFG
	Use entity work.INITCFG(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    U0 : INITCFG
      port map(INIT_ADDR_int(8) => INIT_ADDR_int(8), 
        INIT_ADDR_int(7) => INIT_ADDR_int(7), INIT_ADDR_int(6)
         => INIT_ADDR_int(6), INIT_ADDR_int(5) => 
        INIT_ADDR_int(5), INIT_ADDR_int(4) => INIT_ADDR_int(4), 
        INIT_ADDR_int(3) => INIT_ADDR_int(3), INIT_ADDR_int(2)
         => INIT_ADDR_int(2), INIT_ADDR_int(1) => 
        INIT_ADDR_int(1), INIT_ADDR_int(0) => INIT_ADDR_int(0), 
        srdtcqsfmmm_i_0_0 => srdtcqsfmmm_i_0_0, srdtcqsfmmm_i_0_5
         => srdtcqsfmmm_i_0_5, srdtcqsfmmm_i_0_1 => 
        srdtcqsfmmm_i_0_1, srdtcqsfmmm_i_0_2 => srdtcqsfmmm_i_0_2, 
        srdtcqsfmmm_i_0_14 => srdtcqsfmmm_i_0_14, 
        srdtcqsfmmm_i_0_12 => srdtcqsfmmm_i_0_12, 
        srdtcqsfmmm_i_0_6 => srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_3
         => srdtcqsfmmm_i_0_3, srdtcqsfmmm_i_0_4 => 
        srdtcqsfmmm_i_0_4, srdtcqsfmmm_i_0_15 => 
        srdtcqsfmmm_i_0_15, srdtcqsfmmm_i_0_16 => 
        srdtcqsfmmm_i_0_16, srdtcqsfmmm_i_0_13 => 
        srdtcqsfmmm_i_0_13, ACM_CLK_c => ACM_CLK_c, sys_reset_gl
         => sys_reset_gl, NVM_BUSY_net => NVM_BUSY_net, 
        twxmgvvpbgm_iv_0_0_0_a2_0 => twxmgvvpbgm_iv_0_0_0_a2_0, 
        twxmgvvpbgm_iv_0_0_o3_i_o3 => twxmgvvpbgm_iv_0_0_o3_i_o3, 
        N_2497_i_0 => N_2497_i_0, INIT_ASSC_WEN => INIT_ASSC_WEN, 
        N_14 => N_14, INIT_EV_WEN => INIT_EV_WEN, N_12 => N_12, 
        hrgmtcpsdzr => hrgmtcpsdzr);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity NVM_analog is

    port(INIT_ADDR_int : out std_logic_vector(8 downto 0); 
        INIT_DATA_int : out std_logic_vector(8 downto 0); 
        INIT_ASSC_WEN, N_14, INIT_EV_WEN, N_12, hrgmtcpsdzr : out
         std_logic; sys_reset_gl, ACM_CLK_c : in std_logic);

end NVM_analog;

architecture DEF_ARCH of NVM_analog is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component NVM
    generic (MEMORYFILE:string := "");

    port(RD : out std_logic_vector(31 downto 0); BUSY : out
         std_logic; STATUS : out std_logic_vector(1 downto 0); 
        ADDR : 
        in std_logic_vector(17 downto 0) := (others => 'U'); WD
         : in std_logic_vector(31 downto 0) := (others => 'U'); 
        DATAWIDTH : 
        in std_logic_vector(1 downto 0) := (others => 'U'); REN, 
        WEN, PROGRAM, SPAREPAGE, AUXBLOCK, READNEXT, ERASEPAGE, 
        UNPROTECTPAGE, DISCARDPAGE, OVERWRITEPROTECT, 
        PAGELOSSPROTECT, CLK, RESET, PAGESTATUS, OVERWRITEPAGE, 
        PIPE, LOCKREQUEST : in std_logic := 'U');
  end component;

  component NVM_analog_init_wrapper
    port(INIT_ADDR_int : out std_logic_vector(8 downto 0); 
        srdtcqsfmmm_i_0_13, srdtcqsfmmm_i_0_16, 
        srdtcqsfmmm_i_0_15, srdtcqsfmmm_i_0_4, srdtcqsfmmm_i_0_3, 
        srdtcqsfmmm_i_0_6, srdtcqsfmmm_i_0_12, srdtcqsfmmm_i_0_14, 
        srdtcqsfmmm_i_0_2, srdtcqsfmmm_i_0_1, srdtcqsfmmm_i_0_5, 
        srdtcqsfmmm_i_0_0, hrgmtcpsdzr, N_12, INIT_EV_WEN, N_14, 
        INIT_ASSC_WEN, N_2497_i_0, twxmgvvpbgm_iv_0_0_o3_i_o3, 
        twxmgvvpbgm_iv_0_0_0_a2_0 : out std_logic; NVM_BUSY_net, 
        sys_reset_gl, ACM_CLK_c : in std_logic := 'U');
  end component;

    signal NVM_BUSY_net, NVM_DAT_FROM_net_net_21, 
        NVM_DAT_FROM_net_net_20, NVM_DAT_FROM_net_net_19, 
        NVM_DAT_FROM_net_net_18, NVM_DAT_FROM_net_net_17, 
        NVM_DAT_FROM_net_net_16, NVM_DAT_FROM_net_net_15, 
        NVM_DAT_FROM_net_net_14, NVM_DAT_FROM_net_net_13, 
        NVM_DAT_FROM_net_net_12, NVM_DAT_FROM_net_net_11, 
        NVM_DAT_FROM_net_net_10, NVM_DAT_FROM_net_net_9, 
        NVM_DAT_FROM_net_net_8, NVM_DAT_FROM_net_net_7, 
        NVM_DAT_FROM_net_net_6, NVM_DAT_FROM_net_net_5, 
        NVM_DAT_FROM_net_net_4, NVM_DAT_FROM_net_net_3, 
        NVM_DAT_FROM_net_net_2, NVM_DAT_FROM_net_net_1, 
        \Z\\NVM_DAT_FROM_net\\\, \NVM_DAT_FROM_net[31]\, 
        \Z\\NVM_STATUS_net\\\, \NVM_STATUS_net[1]\, 
        \srdtcqsfmmm_i_0[0]\, \srdtcqsfmmm_i_0[1]\, 
        \srdtcqsfmmm_i_0[2]\, \srdtcqsfmmm_i_0[3]\, 
        \srdtcqsfmmm_i_0[4]\, \srdtcqsfmmm_i_0[5]\, 
        \srdtcqsfmmm_i_0[6]\, \GND\, \srdtcqsfmmm_i_0[12]\, 
        \srdtcqsfmmm_i_0[13]\, \srdtcqsfmmm_i_0[14]\, 
        \srdtcqsfmmm_i_0[15]\, \srdtcqsfmmm_i_0[16]\, N_2497_i_0, 
        twxmgvvpbgm_iv_0_0_o3_i_o3, twxmgvvpbgm_iv_0_0_0_a2_0, 
        \VCC\, GND_net_1, VCC_net_1 : std_logic;

    for all : NVM_analog_init_wrapper
	Use entity work.NVM_analog_init_wrapper(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    NVM_INST : NVM
      generic map(MEMORYFILE => "NVM_analog.mem")

      port map(RD(31) => \NVM_DAT_FROM_net[31]\, RD(30) => 
        \Z\\NVM_DAT_FROM_net\\\, RD(29) => NVM_DAT_FROM_net_net_1, 
        RD(28) => NVM_DAT_FROM_net_net_2, RD(27) => 
        NVM_DAT_FROM_net_net_3, RD(26) => NVM_DAT_FROM_net_net_4, 
        RD(25) => NVM_DAT_FROM_net_net_5, RD(24) => 
        NVM_DAT_FROM_net_net_6, RD(23) => NVM_DAT_FROM_net_net_7, 
        RD(22) => NVM_DAT_FROM_net_net_8, RD(21) => 
        NVM_DAT_FROM_net_net_9, RD(20) => NVM_DAT_FROM_net_net_10, 
        RD(19) => NVM_DAT_FROM_net_net_11, RD(18) => 
        NVM_DAT_FROM_net_net_12, RD(17) => 
        NVM_DAT_FROM_net_net_13, RD(16) => 
        NVM_DAT_FROM_net_net_14, RD(15) => 
        NVM_DAT_FROM_net_net_15, RD(14) => 
        NVM_DAT_FROM_net_net_16, RD(13) => 
        NVM_DAT_FROM_net_net_17, RD(12) => 
        NVM_DAT_FROM_net_net_18, RD(11) => 
        NVM_DAT_FROM_net_net_19, RD(10) => 
        NVM_DAT_FROM_net_net_20, RD(9) => NVM_DAT_FROM_net_net_21, 
        RD(8) => INIT_DATA_int(8), RD(7) => INIT_DATA_int(7), 
        RD(6) => INIT_DATA_int(6), RD(5) => INIT_DATA_int(5), 
        RD(4) => INIT_DATA_int(4), RD(3) => INIT_DATA_int(3), 
        RD(2) => INIT_DATA_int(2), RD(1) => INIT_DATA_int(1), 
        RD(0) => INIT_DATA_int(0), BUSY => NVM_BUSY_net, 
        STATUS(1) => \NVM_STATUS_net[1]\, STATUS(0) => 
        \Z\\NVM_STATUS_net\\\, ADDR(17) => \GND\, ADDR(16) => 
        \srdtcqsfmmm_i_0[16]\, ADDR(15) => \srdtcqsfmmm_i_0[15]\, 
        ADDR(14) => \srdtcqsfmmm_i_0[14]\, ADDR(13) => 
        \srdtcqsfmmm_i_0[13]\, ADDR(12) => \srdtcqsfmmm_i_0[12]\, 
        ADDR(11) => \GND\, ADDR(10) => \GND\, ADDR(9) => \GND\, 
        ADDR(8) => \GND\, ADDR(7) => \GND\, ADDR(6) => 
        \srdtcqsfmmm_i_0[6]\, ADDR(5) => \srdtcqsfmmm_i_0[5]\, 
        ADDR(4) => \srdtcqsfmmm_i_0[4]\, ADDR(3) => 
        \srdtcqsfmmm_i_0[3]\, ADDR(2) => \srdtcqsfmmm_i_0[2]\, 
        ADDR(1) => \srdtcqsfmmm_i_0[1]\, ADDR(0) => 
        \srdtcqsfmmm_i_0[0]\, WD(31) => \GND\, WD(30) => \GND\, 
        WD(29) => \GND\, WD(28) => \GND\, WD(27) => \GND\, WD(26)
         => \GND\, WD(25) => \GND\, WD(24) => \GND\, WD(23) => 
        \GND\, WD(22) => \GND\, WD(21) => \GND\, WD(20) => \GND\, 
        WD(19) => \GND\, WD(18) => \GND\, WD(17) => \GND\, WD(16)
         => \GND\, WD(15) => \GND\, WD(14) => \GND\, WD(13) => 
        \GND\, WD(12) => \GND\, WD(11) => \GND\, WD(10) => \GND\, 
        WD(9) => \GND\, WD(8) => \GND\, WD(7) => \GND\, WD(6) => 
        \GND\, WD(5) => \GND\, WD(4) => \GND\, WD(3) => \GND\, 
        WD(2) => \GND\, WD(1) => \GND\, WD(0) => \GND\, 
        DATAWIDTH(1) => \GND\, DATAWIDTH(0) => N_2497_i_0, REN
         => twxmgvvpbgm_iv_0_0_0_a2_0, WEN => \GND\, PROGRAM => 
        \GND\, SPAREPAGE => twxmgvvpbgm_iv_0_0_o3_i_o3, AUXBLOCK
         => \GND\, READNEXT => twxmgvvpbgm_iv_0_0_0_a2_0, 
        ERASEPAGE => \GND\, UNPROTECTPAGE => \GND\, DISCARDPAGE
         => \GND\, OVERWRITEPROTECT => \GND\, PAGELOSSPROTECT => 
        \GND\, CLK => ACM_CLK_c, RESET => sys_reset_gl, 
        PAGESTATUS => \GND\, OVERWRITEPAGE => \GND\, PIPE => 
        \GND\, LOCKREQUEST => twxmgvvpbgm_iv_0_0_o3_i_o3);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    NVM_analog_init_wrapper_inst : NVM_analog_init_wrapper
      port map(INIT_ADDR_int(8) => INIT_ADDR_int(8), 
        INIT_ADDR_int(7) => INIT_ADDR_int(7), INIT_ADDR_int(6)
         => INIT_ADDR_int(6), INIT_ADDR_int(5) => 
        INIT_ADDR_int(5), INIT_ADDR_int(4) => INIT_ADDR_int(4), 
        INIT_ADDR_int(3) => INIT_ADDR_int(3), INIT_ADDR_int(2)
         => INIT_ADDR_int(2), INIT_ADDR_int(1) => 
        INIT_ADDR_int(1), INIT_ADDR_int(0) => INIT_ADDR_int(0), 
        srdtcqsfmmm_i_0_13 => \srdtcqsfmmm_i_0[13]\, 
        srdtcqsfmmm_i_0_16 => \srdtcqsfmmm_i_0[16]\, 
        srdtcqsfmmm_i_0_15 => \srdtcqsfmmm_i_0[15]\, 
        srdtcqsfmmm_i_0_4 => \srdtcqsfmmm_i_0[4]\, 
        srdtcqsfmmm_i_0_3 => \srdtcqsfmmm_i_0[3]\, 
        srdtcqsfmmm_i_0_6 => \srdtcqsfmmm_i_0[6]\, 
        srdtcqsfmmm_i_0_12 => \srdtcqsfmmm_i_0[12]\, 
        srdtcqsfmmm_i_0_14 => \srdtcqsfmmm_i_0[14]\, 
        srdtcqsfmmm_i_0_2 => \srdtcqsfmmm_i_0[2]\, 
        srdtcqsfmmm_i_0_1 => \srdtcqsfmmm_i_0[1]\, 
        srdtcqsfmmm_i_0_5 => \srdtcqsfmmm_i_0[5]\, 
        srdtcqsfmmm_i_0_0 => \srdtcqsfmmm_i_0[0]\, hrgmtcpsdzr
         => hrgmtcpsdzr, N_12 => N_12, INIT_EV_WEN => INIT_EV_WEN, 
        N_14 => N_14, INIT_ASSC_WEN => INIT_ASSC_WEN, N_2497_i_0
         => N_2497_i_0, twxmgvvpbgm_iv_0_0_o3_i_o3 => 
        twxmgvvpbgm_iv_0_0_o3_i_o3, twxmgvvpbgm_iv_0_0_0_a2_0 => 
        twxmgvvpbgm_iv_0_0_0_a2_0, NVM_BUSY_net => NVM_BUSY_net, 
        sys_reset_gl => sys_reset_gl, ACM_CLK_c => ACM_CLK_c);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity Shift8_reg is

    port(sys_reset_gl, POWER_down_c, ACM_CLK_c : in std_logic; 
        PWR_dwn : out std_logic);

end Shift8_reg;

architecture DEF_ARCH of Shift8_reg is 

  component DFN1C1
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component MX2
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component BUFF
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

    signal \PWR_dwn\, MX2_1_Y, DFN1C1_6_Q, DFN1C1_4_Q, BUFF_0_Y, 
        MX2_7_Y, DFN1C1_1_Q, BUFF_1_Y, MX2_2_Y, DFN1C1_3_Q, 
        DFN1C1_0_Q, MX2_5_Y, DFN1C1_5_Q, DFN1C1_2_Q, INV_0_Y, 
        MX2_6_Y, \VCC\, MX2_4_Y, MX2_0_Y, MX2_3_Y, \GND\, 
        GND_net_1, VCC_net_1 : std_logic;

begin 

    PWR_dwn <= \PWR_dwn\;

    DFN1C1_6 : DFN1C1
      port map(D => MX2_1_Y, CLK => ACM_CLK_c, CLR => INV_0_Y, Q
         => DFN1C1_6_Q);
    
    DFN1C1_3 : DFN1C1
      port map(D => MX2_2_Y, CLK => ACM_CLK_c, CLR => INV_0_Y, Q
         => DFN1C1_3_Q);
    
    DFN1C1_2 : DFN1C1
      port map(D => MX2_4_Y, CLK => ACM_CLK_c, CLR => INV_0_Y, Q
         => DFN1C1_2_Q);
    
    MX2_0 : MX2
      port map(A => DFN1C1_0_Q, B => DFN1C1_5_Q, S => BUFF_1_Y, Y
         => MX2_0_Y);
    
    BUFF_1 : BUFF
      port map(A => \VCC\, Y => BUFF_1_Y);
    
    INV_0 : INV
      port map(A => sys_reset_gl, Y => INV_0_Y);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    DFN1C1_5 : DFN1C1
      port map(D => MX2_5_Y, CLK => ACM_CLK_c, CLR => INV_0_Y, Q
         => DFN1C1_5_Q);
    
    MX2_6 : MX2
      port map(A => DFN1C1_4_Q, B => POWER_down_c, S => BUFF_0_Y, 
        Y => MX2_6_Y);
    
    DFN1C1_Shiftout : DFN1C1
      port map(D => MX2_7_Y, CLK => ACM_CLK_c, CLR => INV_0_Y, Q
         => \PWR_dwn\);
    
    DFN1C1_1 : DFN1C1
      port map(D => MX2_3_Y, CLK => ACM_CLK_c, CLR => INV_0_Y, Q
         => DFN1C1_1_Q);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    MX2_3 : MX2
      port map(A => DFN1C1_1_Q, B => DFN1C1_3_Q, S => BUFF_1_Y, Y
         => MX2_3_Y);
    
    DFN1C1_0 : DFN1C1
      port map(D => MX2_0_Y, CLK => ACM_CLK_c, CLR => INV_0_Y, Q
         => DFN1C1_0_Q);
    
    MX2_4 : MX2
      port map(A => DFN1C1_2_Q, B => DFN1C1_6_Q, S => BUFF_0_Y, Y
         => MX2_4_Y);
    
    GND_i : GND
      port map(Y => \GND\);
    
    BUFF_0 : BUFF
      port map(A => \VCC\, Y => BUFF_0_Y);
    
    MX2_5 : MX2
      port map(A => DFN1C1_5_Q, B => DFN1C1_2_Q, S => BUFF_0_Y, Y
         => MX2_5_Y);
    
    DFN1C1_4 : DFN1C1
      port map(D => MX2_6_Y, CLK => ACM_CLK_c, CLR => INV_0_Y, Q
         => DFN1C1_4_Q);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    MX2_2 : MX2
      port map(A => DFN1C1_3_Q, B => DFN1C1_0_Q, S => BUFF_1_Y, Y
         => MX2_2_Y);
    
    MX2_7 : MX2
      port map(A => \PWR_dwn\, B => DFN1C1_1_Q, S => BUFF_1_Y, Y
         => MX2_7_Y);
    
    MX2_1 : MX2
      port map(A => DFN1C1_6_Q, B => DFN1C1_4_Q, S => BUFF_0_Y, Y
         => MX2_1_Y);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity equal0 is

    port(RTC_count : in std_logic_vector(31 downto 0); rtc_eq0 : 
        out std_logic);

end equal0;

architecture DEF_ARCH of equal0 is 

  component AND3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AND3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component NOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

    signal Temp_0_net, AND3_3_Y, Temp_5_net, Temp_6_net, 
        Temp_7_net, AND3_0_Y, AND3_1_Y, Temp_9_net, Temp_1_net, 
        Temp_8_net, Temp_10_net, Temp_4_net, AND3_2_Y, Temp_3_net, 
        Temp_2_net, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

begin 


    AND3_2 : AND3
      port map(A => Temp_0_net, B => Temp_1_net, C => Temp_2_net, 
        Y => AND3_2_Y);
    
    AND3C_Temp_8_inst : AND3C
      port map(A => RTC_count(24), B => RTC_count(25), C => 
        RTC_count(26), Y => Temp_8_net);
    
    AND3C_Temp_1_inst : AND3C
      port map(A => RTC_count(3), B => RTC_count(4), C => 
        RTC_count(5), Y => Temp_1_net);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    NOR2_Temp_10_inst : NOR2
      port map(A => RTC_count(30), B => RTC_count(31), Y => 
        Temp_10_net);
    
    AND3_0 : AND3
      port map(A => AND3_2_Y, B => Temp_3_net, C => Temp_4_net, Y
         => AND3_0_Y);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    AND3_1 : AND3
      port map(A => Temp_8_net, B => Temp_9_net, C => Temp_10_net, 
        Y => AND3_1_Y);
    
    GND_i : GND
      port map(Y => \GND\);
    
    AND3C_Temp_5_inst : AND3C
      port map(A => RTC_count(15), B => RTC_count(16), C => 
        RTC_count(17), Y => Temp_5_net);
    
    AND3C_Temp_2_inst : AND3C
      port map(A => RTC_count(6), B => RTC_count(7), C => 
        RTC_count(8), Y => Temp_2_net);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    AND3_AEB : AND3
      port map(A => AND3_0_Y, B => AND3_3_Y, C => AND3_1_Y, Y => 
        rtc_eq0);
    
    AND3C_Temp_9_inst : AND3C
      port map(A => RTC_count(27), B => RTC_count(28), C => 
        RTC_count(29), Y => Temp_9_net);
    
    AND3C_Temp_6_inst : AND3C
      port map(A => RTC_count(18), B => RTC_count(19), C => 
        RTC_count(20), Y => Temp_6_net);
    
    AND3C_Temp_3_inst : AND3C
      port map(A => RTC_count(9), B => RTC_count(10), C => 
        RTC_count(11), Y => Temp_3_net);
    
    AND3C_Temp_7_inst : AND3C
      port map(A => RTC_count(21), B => RTC_count(22), C => 
        RTC_count(23), Y => Temp_7_net);
    
    AND3C_Temp_0_inst : AND3C
      port map(A => RTC_count(0), B => RTC_count(1), C => 
        RTC_count(2), Y => Temp_0_net);
    
    AND3_3 : AND3
      port map(A => Temp_5_net, B => Temp_6_net, C => Temp_7_net, 
        Y => AND3_3_Y);
    
    AND3C_Temp_4_inst : AND3C
      port map(A => RTC_count(12), B => RTC_count(13), C => 
        RTC_count(14), Y => Temp_4_net);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity RIP_COUNT32 is

    port(RTC_load : in std_logic_vector(31 downto 0); rtc_eq0 : 
        out std_logic; clr_sec_flag_1_sqmuxa, I_296, ACM_CLK_c, 
        sys_reset_gl : in std_logic);

end RIP_COUNT32;

architecture DEF_ARCH of RIP_COUNT32 is 

  component MX2A
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1E1C0
    port(D, CLK, CLR, E : in std_logic := 'U'; Q : out std_logic);
        
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component OR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component BUFF
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component equal0
    port(RTC_count : 
        in std_logic_vector(31 downto 0) := (others => 'U'); 
        rtc_eq0 : out std_logic);
  end component;

    signal \Z$1N156_1\, \Z$1N156\, \Z$1N156_0\, \RTC_count[21]\, 
        CLK_net_9, DIN_net_10, \RTC_count[3]\, CLK_net_27, 
        DIN_net_28, CLK_net_12, \RTC_count[17]\, \RTC_count[11]\, 
        CLK_net_19, DIN_net_20, \RTC_count[26]\, CLK_net_4, 
        DIN_net_5, \RTC_count[25]\, CLK_net_5, DIN_net_6, 
        DIN_net_11, \RTC_count[20]\, DIN_net_18, \RTC_count[13]\, 
        CLK_net_22, \RTC_count[7]\, DIN_net_14, DIN_net_15, 
        \RTC_count[16]\, CLK_net_3, CLK_net_6, \RTC_count[23]\, 
        DIN_net_8, \RTC_count[19]\, CLK_net_11, DIN_net_12, 
        DIN_net_3, \RTC_count[28]\, \RTC_count[9]\, CLK_net_21, 
        DIN_net_22, DIN_net_16, \RTC_count[15]\, DIN_net_29, 
        \RTC_count[2]\, \RTC_count[10]\, CLK_net_15, DIN_net_1, 
        \RTC_count[30]\, CLK_net_14, \RTC_count[5]\, CLK_net_25, 
        DIN_net_26, \RTC_count[6]\, CLK_net_24, DIN_net_25, 
        DIN_net_30, \RTC_count[1]\, CLK_net_10, \RTC_count[18]\, 
        DIN_net_13, DIN_net_17, \RTC_count[14]\, CLK_net_29, 
        \RTC_count[0]\, CLK_net_20, DIN_net_21, \RTC_count[8]\, 
        DIN_net_23, DIN_net_7, \RTC_count[24]\, DIN_net_9, 
        \RTC_count[22]\, CLK_net_16, DIN_net_24, CLK_net_28, 
        \RTC_count[29]\, CLK_net_1, DIN_net_2, CLK_net_26, 
        CLK_net_7, \CLK[31]\, CLK_net_8, \Z\\CLK\\\, CLK_net_18, 
        DIN_net_27, \RTC_count[4]\, CLK_net_13, CLK_net_2, 
        CLK_net_17, \RTC_count[12]\, DIN_net_19, CLK_net_23, 
        \RTC_count[27]\, DIN_net_4, \DIN[0]\, \Z\\DIN\\\, 
        \RTC_count[31]\, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

    for all : equal0
	Use entity work.equal0(DEF_ARCH);
begin 


    \$1I37_ARRAY_24\ : MX2A
      port map(A => \RTC_count[8]\, B => RTC_load(8), S => I_296, 
        Y => DIN_net_23);
    
    \$1I82_ARRAY_4\ : MX2
      port map(A => \RTC_count[27]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_2);
    
    \$1I113_ARRAY_19\ : DFN1E1C0
      port map(D => DIN_net_18, CLK => CLK_net_17, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[13]\);
    
    \$1I113_ARRAY_2\ : DFN1E1C0
      port map(D => DIN_net_1, CLK => \Z\\CLK\\\, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[30]\);
    
    \$1I113_ARRAY_20\ : DFN1E1C0
      port map(D => DIN_net_19, CLK => CLK_net_18, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[12]\);
    
    \$1I37_ARRAY_23\ : MX2A
      port map(A => \RTC_count[9]\, B => RTC_load(9), S => I_296, 
        Y => DIN_net_22);
    
    \$1I113_ARRAY_1\ : DFN1E1C0
      port map(D => \Z\\DIN\\\, CLK => \CLK[31]\, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[31]\);
    
    \$1I37_ARRAY_5\ : MX2A
      port map(A => \RTC_count[27]\, B => RTC_load(27), S => 
        I_296, Y => DIN_net_4);
    
    \$1I37_ARRAY_1\ : MX2A
      port map(A => \RTC_count[31]\, B => RTC_load(31), S => 
        I_296, Y => \Z\\DIN\\\);
    
    \$1I113_ARRAY_10\ : DFN1E1C0
      port map(D => DIN_net_9, CLK => CLK_net_8, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[22]\);
    
    \$1I1\ : DFN1E1C0
      port map(D => \DIN[0]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[0]\);
    
    \$1I82_ARRAY_9\ : MX2
      port map(A => \RTC_count[22]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_7);
    
    \$1I37_ARRAY_32\ : MX2A
      port map(A => \RTC_count[0]\, B => RTC_load(0), S => I_296, 
        Y => \DIN[0]\);
    
    \$1I37_ARRAY_21\ : MX2A
      port map(A => \RTC_count[11]\, B => RTC_load(11), S => 
        I_296, Y => DIN_net_20);
    
    \$1I37_ARRAY_6\ : MX2A
      port map(A => \RTC_count[26]\, B => RTC_load(26), S => 
        I_296, Y => DIN_net_5);
    
    \$1I113_ARRAY_28\ : DFN1E1C0
      port map(D => DIN_net_27, CLK => CLK_net_26, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[4]\);
    
    \$1I82_ARRAY_27\ : MX2
      port map(A => \RTC_count[4]\, B => ACM_CLK_c, S => I_296, Y
         => CLK_net_25);
    
    \$1I37_ARRAY_7\ : MX2A
      port map(A => \RTC_count[25]\, B => RTC_load(25), S => 
        I_296, Y => DIN_net_6);
    
    \$1I113_ARRAY_5\ : DFN1E1C0
      port map(D => DIN_net_4, CLK => CLK_net_3, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[27]\);
    
    \$1I113_ARRAY_25\ : DFN1E1C0
      port map(D => DIN_net_24, CLK => CLK_net_23, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[7]\);
    
    \$1I82_ARRAY_7\ : MX2
      port map(A => \RTC_count[24]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_5);
    
    \$1I113_ARRAY_18\ : DFN1E1C0
      port map(D => DIN_net_17, CLK => CLK_net_16, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[14]\);
    
    \$1I82_ARRAY_29\ : MX2
      port map(A => \RTC_count[2]\, B => ACM_CLK_c, S => I_296, Y
         => CLK_net_27);
    
    \$1I82_ARRAY_30\ : MX2
      port map(A => \RTC_count[1]\, B => ACM_CLK_c, S => I_296, Y
         => CLK_net_28);
    
    \$1I82_ARRAY_25\ : MX2
      port map(A => \RTC_count[6]\, B => ACM_CLK_c, S => I_296, Y
         => CLK_net_23);
    
    \$1I82_ARRAY_17\ : MX2
      port map(A => \RTC_count[14]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_15);
    
    \$1I113_ARRAY_15\ : DFN1E1C0
      port map(D => DIN_net_14, CLK => CLK_net_13, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[17]\);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    \$1I82_ARRAY_3\ : MX2
      port map(A => \RTC_count[28]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_1);
    
    \$1I37_ARRAY_20\ : MX2A
      port map(A => \RTC_count[12]\, B => RTC_load(12), S => 
        I_296, Y => DIN_net_19);
    
    \$1I82_ARRAY_19\ : MX2
      port map(A => \RTC_count[12]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_17);
    
    \$1I113_ARRAY_4\ : DFN1E1C0
      port map(D => DIN_net_3, CLK => CLK_net_2, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[28]\);
    
    \$1I82_ARRAY_15\ : MX2
      port map(A => \RTC_count[16]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_13);
    
    \$1I82_ARRAY_26\ : MX2
      port map(A => \RTC_count[5]\, B => ACM_CLK_c, S => I_296, Y
         => CLK_net_24);
    
    \$1I37_ARRAY_3\ : MX2A
      port map(A => \RTC_count[29]\, B => RTC_load(29), S => 
        I_296, Y => DIN_net_2);
    
    \$1I37_ARRAY_28\ : MX2A
      port map(A => \RTC_count[4]\, B => RTC_load(4), S => I_296, 
        Y => DIN_net_27);
    
    \$1I37_ARRAY_14\ : MX2A
      port map(A => \RTC_count[18]\, B => RTC_load(18), S => 
        I_296, Y => DIN_net_13);
    
    \$1I82_ARRAY_20\ : MX2
      port map(A => \RTC_count[11]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_18);
    
    \$1I82_ARRAY_16\ : MX2
      port map(A => \RTC_count[15]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_14);
    
    \$1I82_ARRAY_2\ : MX2
      port map(A => \RTC_count[29]\, B => ACM_CLK_c, S => I_296, 
        Y => \Z\\CLK\\\);
    
    \$1I37_ARRAY_13\ : MX2A
      port map(A => \RTC_count[19]\, B => RTC_load(19), S => 
        I_296, Y => DIN_net_12);
    
    \$1I113_ARRAY_31\ : DFN1E1C0
      port map(D => DIN_net_30, CLK => CLK_net_29, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[1]\);
    
    \$1I82_ARRAY_10\ : MX2
      port map(A => \RTC_count[21]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_8);
    
    \$1I82_ARRAY_1\ : MX2
      port map(A => \RTC_count[30]\, B => ACM_CLK_c, S => I_296, 
        Y => \CLK[31]\);
    
    \$1I113_ARRAY_9\ : DFN1E1C0
      port map(D => DIN_net_8, CLK => CLK_net_7, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[23]\);
    
    \$1I37_ARRAY_11\ : MX2A
      port map(A => \RTC_count[21]\, B => RTC_load(21), S => 
        I_296, Y => DIN_net_10);
    
    \$1I37_ARRAY_27\ : MX2A
      port map(A => \RTC_count[5]\, B => RTC_load(5), S => I_296, 
        Y => DIN_net_26);
    
    \$1I82_ARRAY_28\ : MX2
      port map(A => \RTC_count[3]\, B => ACM_CLK_c, S => I_296, Y
         => CLK_net_26);
    
    \$1I113_ARRAY_3\ : DFN1E1C0
      port map(D => DIN_net_2, CLK => CLK_net_1, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[29]\);
    
    \$1I82_ARRAY_23\ : MX2
      port map(A => \RTC_count[8]\, B => ACM_CLK_c, S => I_296, Y
         => CLK_net_21);
    
    \$1I113_ARRAY_30\ : DFN1E1C0
      port map(D => DIN_net_29, CLK => CLK_net_28, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[2]\);
    
    \$1I82_ARRAY_22\ : MX2
      port map(A => \RTC_count[9]\, B => ACM_CLK_c, S => I_296, Y
         => CLK_net_20);
    
    \$1I37_ARRAY_26\ : MX2A
      port map(A => \RTC_count[6]\, B => RTC_load(6), S => I_296, 
        Y => DIN_net_25);
    
    \$1I37_ARRAY_25\ : MX2A
      port map(A => \RTC_count[7]\, B => RTC_load(7), S => I_296, 
        Y => DIN_net_24);
    
    GND_i : GND
      port map(Y => \GND\);
    
    \$1I82_ARRAY_18\ : MX2
      port map(A => \RTC_count[13]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_16);
    
    \$1I37_ARRAY_29\ : MX2A
      port map(A => \RTC_count[3]\, B => RTC_load(3), S => I_296, 
        Y => DIN_net_28);
    
    \$1I37_ARRAY_22\ : MX2A
      port map(A => \RTC_count[10]\, B => RTC_load(10), S => 
        I_296, Y => DIN_net_21);
    
    \$1I37_ARRAY_10\ : MX2A
      port map(A => \RTC_count[22]\, B => RTC_load(22), S => 
        I_296, Y => DIN_net_9);
    
    \$1I153\ : OR2
      port map(A => I_296, B => clr_sec_flag_1_sqmuxa, Y => 
        \Z$1N156\);
    
    \$1I82_ARRAY_13\ : MX2
      port map(A => \RTC_count[18]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_11);
    
    \$1I113_ARRAY_8\ : DFN1E1C0
      port map(D => DIN_net_7, CLK => CLK_net_6, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[24]\);
    
    \$1I82_ARRAY_12\ : MX2
      port map(A => \RTC_count[19]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_10);
    
    \$1I37_ARRAY_8\ : MX2A
      port map(A => \RTC_count[24]\, B => RTC_load(24), S => 
        I_296, Y => DIN_net_7);
    
    \$1I153_1\ : BUFF
      port map(A => \Z$1N156\, Y => \Z$1N156_1\);
    
    \$1I113_ARRAY_24\ : DFN1E1C0
      port map(D => DIN_net_23, CLK => CLK_net_22, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[8]\);
    
    \$1I113_ARRAY_22\ : DFN1E1C0
      port map(D => DIN_net_21, CLK => CLK_net_20, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[10]\);
    
    \$1I82_ARRAY_31\ : MX2
      port map(A => \RTC_count[0]\, B => ACM_CLK_c, S => I_296, Y
         => CLK_net_29);
    
    \$1I37_ARRAY_18\ : MX2A
      port map(A => \RTC_count[14]\, B => RTC_load(14), S => 
        I_296, Y => DIN_net_17);
    
    \$1I113_ARRAY_14\ : DFN1E1C0
      port map(D => DIN_net_13, CLK => CLK_net_12, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[18]\);
    
    \$1I113_ARRAY_12\ : DFN1E1C0
      port map(D => DIN_net_11, CLK => CLK_net_10, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[20]\);
    
    \$1I37_ARRAY_31\ : MX2A
      port map(A => \RTC_count[1]\, B => RTC_load(1), S => I_296, 
        Y => DIN_net_30);
    
    \$1I113_ARRAY_26\ : DFN1E1C0
      port map(D => DIN_net_25, CLK => CLK_net_24, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[6]\);
    
    \$1I113_ARRAY_27\ : DFN1E1C0
      port map(D => DIN_net_26, CLK => CLK_net_25, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[5]\);
    
    \$1I113_ARRAY_16\ : DFN1E1C0
      port map(D => DIN_net_15, CLK => CLK_net_14, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[16]\);
    
    \$1I37_ARRAY_2\ : MX2A
      port map(A => \RTC_count[30]\, B => RTC_load(30), S => 
        I_296, Y => DIN_net_1);
    
    \$1I113_ARRAY_17\ : DFN1E1C0
      port map(D => DIN_net_16, CLK => CLK_net_15, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[15]\);
    
    \$1I82_ARRAY_21\ : MX2
      port map(A => \RTC_count[10]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_19);
    
    \$1I37_ARRAY_30\ : MX2A
      port map(A => \RTC_count[2]\, B => RTC_load(2), S => I_296, 
        Y => DIN_net_29);
    
    \$1I37_ARRAY_17\ : MX2A
      port map(A => \RTC_count[15]\, B => RTC_load(15), S => 
        I_296, Y => DIN_net_16);
    
    \$1I113_ARRAY_23\ : DFN1E1C0
      port map(D => DIN_net_22, CLK => CLK_net_21, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[9]\);
    
    \$1I153_0\ : BUFF
      port map(A => \Z$1N156\, Y => \Z$1N156_0\);
    
    \$1I37_ARRAY_4\ : MX2A
      port map(A => \RTC_count[28]\, B => RTC_load(28), S => 
        I_296, Y => DIN_net_3);
    
    \$1I149\ : equal0
      port map(RTC_count(31) => \RTC_count[31]\, RTC_count(30)
         => \RTC_count[30]\, RTC_count(29) => \RTC_count[29]\, 
        RTC_count(28) => \RTC_count[28]\, RTC_count(27) => 
        \RTC_count[27]\, RTC_count(26) => \RTC_count[26]\, 
        RTC_count(25) => \RTC_count[25]\, RTC_count(24) => 
        \RTC_count[24]\, RTC_count(23) => \RTC_count[23]\, 
        RTC_count(22) => \RTC_count[22]\, RTC_count(21) => 
        \RTC_count[21]\, RTC_count(20) => \RTC_count[20]\, 
        RTC_count(19) => \RTC_count[19]\, RTC_count(18) => 
        \RTC_count[18]\, RTC_count(17) => \RTC_count[17]\, 
        RTC_count(16) => \RTC_count[16]\, RTC_count(15) => 
        \RTC_count[15]\, RTC_count(14) => \RTC_count[14]\, 
        RTC_count(13) => \RTC_count[13]\, RTC_count(12) => 
        \RTC_count[12]\, RTC_count(11) => \RTC_count[11]\, 
        RTC_count(10) => \RTC_count[10]\, RTC_count(9) => 
        \RTC_count[9]\, RTC_count(8) => \RTC_count[8]\, 
        RTC_count(7) => \RTC_count[7]\, RTC_count(6) => 
        \RTC_count[6]\, RTC_count(5) => \RTC_count[5]\, 
        RTC_count(4) => \RTC_count[4]\, RTC_count(3) => 
        \RTC_count[3]\, RTC_count(2) => \RTC_count[2]\, 
        RTC_count(1) => \RTC_count[1]\, RTC_count(0) => 
        \RTC_count[0]\, rtc_eq0 => rtc_eq0);
    
    \$1I113_ARRAY_13\ : DFN1E1C0
      port map(D => DIN_net_12, CLK => CLK_net_11, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[19]\);
    
    \$1I82_ARRAY_11\ : MX2
      port map(A => \RTC_count[20]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_9);
    
    \$1I37_ARRAY_9\ : MX2A
      port map(A => \RTC_count[23]\, B => RTC_load(23), S => 
        I_296, Y => DIN_net_8);
    
    \$1I82_ARRAY_8\ : MX2
      port map(A => \RTC_count[23]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_6);
    
    \$1I82_ARRAY_5\ : MX2
      port map(A => \RTC_count[26]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_3);
    
    \$1I37_ARRAY_16\ : MX2A
      port map(A => \RTC_count[16]\, B => RTC_load(16), S => 
        I_296, Y => DIN_net_15);
    
    \$1I37_ARRAY_15\ : MX2A
      port map(A => \RTC_count[17]\, B => RTC_load(17), S => 
        I_296, Y => DIN_net_14);
    
    \$1I82_ARRAY_6\ : MX2
      port map(A => \RTC_count[25]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_4);
    
    \$1I82_ARRAY_24\ : MX2
      port map(A => \RTC_count[7]\, B => ACM_CLK_c, S => I_296, Y
         => CLK_net_22);
    
    \$1I37_ARRAY_19\ : MX2A
      port map(A => \RTC_count[13]\, B => RTC_load(13), S => 
        I_296, Y => DIN_net_18);
    
    \$1I37_ARRAY_12\ : MX2A
      port map(A => \RTC_count[20]\, B => RTC_load(20), S => 
        I_296, Y => DIN_net_11);
    
    \$1I113_ARRAY_7\ : DFN1E1C0
      port map(D => DIN_net_6, CLK => CLK_net_5, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[25]\);
    
    \$1I113_ARRAY_6\ : DFN1E1C0
      port map(D => DIN_net_5, CLK => CLK_net_4, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[26]\);
    
    \$1I113_ARRAY_21\ : DFN1E1C0
      port map(D => DIN_net_20, CLK => CLK_net_19, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[11]\);
    
    \$1I82_ARRAY_14\ : MX2
      port map(A => \RTC_count[17]\, B => ACM_CLK_c, S => I_296, 
        Y => CLK_net_12);
    
    \$1I113_ARRAY_29\ : DFN1E1C0
      port map(D => DIN_net_28, CLK => CLK_net_27, CLR => 
        sys_reset_gl, E => \Z$1N156_1\, Q => \RTC_count[3]\);
    
    \$1I113_ARRAY_11\ : DFN1E1C0
      port map(D => DIN_net_10, CLK => CLK_net_9, CLR => 
        sys_reset_gl, E => \Z$1N156_0\, Q => \RTC_count[21]\);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity RTC_calendar is

    port(ACMWDATA : out std_logic_vector(7 downto 0); ACMRDATA : 
        in std_logic_vector(7 downto 0); RTC_year_c : 
        out std_logic_vector(7 downto 0); RTC_seconds_c : 
        out std_logic_vector(7 downto 0); RTC_hours_c : 
        out std_logic_vector(7 downto 0); RTC_date_c : 
        out std_logic_vector(7 downto 0); RTC_month_c : 
        out std_logic_vector(7 downto 0); RTC_minutes_c : 
        out std_logic_vector(7 downto 0); RTC_day_c : 
        out std_logic_vector(7 downto 0); ACMADDR_3, ACMADDR_6, 
        ACMADDR_2, ACMADDR_4, ACMADDR_0, ACMADDR_1 : out
         std_logic; I_296, POWER_down_c : in std_logic; 
        un1_rtc_init_0 : out std_logic; INIT_DONE_c, un1_rtc_init : 
        in std_logic; N_410, N_417, RTC_act_i, count_ld_i_0 : out
         std_logic; sys_reset_gl, ACM_CLK_c : in std_logic);

end RTC_calendar;

architecture DEF_ARCH of RTC_calendar is 

  component OR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AOI1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1E0P0
    port(D, CLK, PRE, E : in std_logic := 'U'; Q : out std_logic);
        
  end component;

  component OR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1E0C0
    port(D, CLK, CLR, E : in std_logic := 'U'; Q : out std_logic);
        
  end component;

  component NOR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XNOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1C0
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component NOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AND2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AND3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1E1C0
    port(D, CLK, CLR, E : in std_logic := 'U'; Q : out std_logic);
        
  end component;

  component OAI1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2C
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component AX1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO18
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XO1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AOI1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AX1C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XO1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XA1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1D
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XAI1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component RIP_COUNT32
    port(RTC_load : 
        in std_logic_vector(31 downto 0) := (others => 'U'); 
        rtc_eq0 : out std_logic; clr_sec_flag_1_sqmuxa, I_296, 
        ACM_CLK_c, sys_reset_gl : in std_logic := 'U');
  end component;

  component DFN1P0
    port(D, CLK, PRE : in std_logic := 'U'; Q : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component AXO7
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XA1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \RTC_year_c[0]\, \RTC_year_c[1]\, \RTC_year_c[2]\, 
        \RTC_year_c[3]\, \RTC_year_c[4]\, \RTC_year_c[5]\, 
        \RTC_year_c[6]\, \RTC_year_c[7]\, \RTC_seconds_c[0]\, 
        \RTC_seconds_c[1]\, \RTC_seconds_c[2]\, 
        \RTC_seconds_c[3]\, \RTC_seconds_c[4]\, 
        \RTC_seconds_c[5]\, \RTC_seconds_c[6]\, 
        \RTC_seconds_c[7]\, \RTC_hours_c[0]\, \RTC_hours_c[1]\, 
        \RTC_hours_c[2]\, \RTC_hours_c[3]\, \RTC_hours_c[4]\, 
        \RTC_hours_c[5]\, \RTC_hours_c[6]\, \RTC_hours_c[7]\, 
        \RTC_date_c[0]\, \RTC_date_c[1]\, \RTC_date_c[2]\, 
        \RTC_date_c[3]\, \RTC_date_c[4]\, \RTC_date_c[5]\, 
        \RTC_date_c[6]\, \RTC_date_c[7]\, \RTC_month_c[0]\, 
        \RTC_month_c[1]\, \RTC_month_c[2]\, \RTC_month_c[3]\, 
        \RTC_month_c[4]\, \RTC_month_c[5]\, \RTC_month_c[6]\, 
        \RTC_month_c[7]\, \RTC_minutes_c[0]\, \RTC_minutes_c[1]\, 
        \RTC_minutes_c[2]\, \RTC_minutes_c[3]\, 
        \RTC_minutes_c[4]\, \RTC_minutes_c[5]\, 
        \RTC_minutes_c[6]\, \RTC_minutes_c[7]\, \RTC_day_c[0]\, 
        \RTC_day_c[1]\, \RTC_day_c[2]\, \RTC_day_c[3]\, 
        \RTC_day_c[4]\, \RTC_day_c[5]\, \RTC_day_c[6]\, 
        \RTC_day_c[7]\, \N_410\, \N_417\, RTC_act_i_net_1, 
        \RTCcal_present_0[0]_net_1\, \RTCcal_present_ns[0]\, 
        \RTCcal_present_0[1]_net_1\, N_479_i_0, 
        \RTCcal_present_0[2]_net_1\, N_481_i_0, 
        \RTCcal_present_0[3]_net_1\, \RTCcal_present_ns[3]\, 
        \RTCcal_present_0[4]_net_1\, \RTCcal_present_ns[4]\, 
        N_509_1_1, N_509_1_0, \RTCcal_present_s21_0_a4_0_1\, 
        N_458, \RTCcal_present_s21_0_a4_0_0\, 
        \DWACT_ADD_CI_0_g_array_1[0]\, \DWACT_ADD_CI_0_TMP[0]\, 
        \numdays_month16_i\, \numdays_month16\, 
        \count_minutes_0_sqmuxa_2\, \count_minutes_0_sqmuxa_1\, 
        N_623_1_i, \count_minutes_0_sqmuxa_0\, 
        \clr_sec_flag_0_a2_0_0\, \sec_flag\, N_455, N_402, 
        N_400_i_0, \count_month_0_sqmuxa_2\, \numdays_month17_1\, 
        \count_month_0_sqmuxa_1\, \I_243_1\, \I_243_0\, 
        \count_month_12_sn_m1_0_a2_1\, numdays_month25, 
        \count_date_0_sqmuxa_4\, un8_inc_day_1_i_0, 
        \count_date_0_sqmuxa_0\, \count_date_0_sqmuxa_3\, 
        un8_inc_day_2_i_0, un8_inc_day_3_i_0, un8_inc_day_0_i_0, 
        \numdays_month[5]_net_1\, un8_inc_day_4_i_0, 
        \count_hours_14_0_iv_0[0]_net_1\, \un6_inc_hour\, 
        \count_minutes_0_sqmuxa\, 
        \count_hours_14_0_iv_0[1]_net_1\, \count_hours_2_sqmuxa\, 
        \count_hours_5_sqmuxa_3\, \count_hours_5_sqmuxa_1\, 
        \count_hours_5_sqmuxa_2\, un3_inc_hour_i_0, 
        \un18_inc_hour\, \un12_inc_hour\, \un24_inc_hour\, 
        \count_minutes_0_sqmuxa_3\, N_623_2, 
        \ACMADDR_1_0_1[0]_net_1\, N_884, N_883, N_886, 
        \RTCcal_present_ns_0_1[0]_net_1\, N_499_2, 
        \RTCcal_present_ns_0_0[0]_net_1\, N_499_1, 
        \RTCcal_present_ns_0_a5_0[0]_net_1\, N_493, 
        \RTCcal_present_s21_0_a4_0\, \I_263_0\, 
        \count_seconds_0_sqmuxa_2\, \count_seconds_0_sqmuxa_1\, 
        \un3_inc_sec_2\, \ACMWDATA_iv_2[2]_net_1\, N_496_1, 
        \count_minutes_m[3]_net_1\, \ACMWDATA_iv_1[2]_net_1\, 
        \count_month_m[4]_net_1\, \count_seconds_m[2]_net_1\, 
        \count_date_m[2]_net_1\, \ACMWDATA_iv_2[4]_net_1\, 
        \count_minutes_m[5]_net_1\, \ACMWDATA_iv_1[4]_net_1\, 
        \count_year_m[1]_net_1\, \count_seconds_m[4]_net_1\, 
        \count_date_m[4]_net_1\, \ACMWDATA_iv_2[5]_net_1\, 
        \count_minutes_m[6]_net_1\, \ACMWDATA_iv_1[5]_net_1\, 
        \count_year_m[2]_net_1\, \count_seconds_m[5]_net_1\, 
        \count_date_m[5]_net_1\, \ACMWDATA_iv_2[1]_net_1\, 
        \N_692_m\, \N_601_m\, \ACMWDATA_iv_1[1]_net_1\, N_672, 
        \N_598_m\, \ACMWDATA_iv_2[3]_net_1\, 
        \count_minutes_i_m[4]_net_1\, 
        \count_seconds_i_m[3]_net_1\, \ACMWDATA_iv_1[3]_net_1\, 
        \count_date_i_m[3]_net_1\, \ACMWDATA_iv_2[7]_net_1\, 
        \count_month_m[1]_net_1\, \count_hours_m[1]_net_1\, 
        \ACMWDATA_iv_0[7]_net_1\, N_646, \ACMWDATA_iv_2[0]_net_1\, 
        \count_seconds_i_m[0]_net_1\, \N_602_m\, \N_563_m\, 
        \ACMWDATA_iv_1[0]_net_1\, N_653, 
        \count_date_i_m[0]_net_1\, \ACMWDATA_iv_2[6]_net_1\, 
        \count_seconds_i_m[6]_net_1\, \count_day_i_m[1]_net_1\, 
        \count_year_i_m[3]_net_1\, \ACMWDATA_iv_1[6]_net_1\, 
        \un1_count_month_m[0]_net_1\, 
        \RTCcal_present_ns_0_0_3[3]_net_1\, N_902_2, 
        \RTCcal_present_ns_0_0_a2_2_0[3]_net_1\, N_897, 
        \RTCcal_present_ns_0_0_2[3]_net_1\, 
        \RTCcal_present_ns_0_0_a2_0[3]_net_1\, N_499, 
        \RTCcal_present_ns_0_0_1[3]_net_1\, N_896, N_878, N_887, 
        \RTCcal_present_ns_i_2[2]_net_1\, N_505, N_506, 
        \RTCcal_present_ns_i_1[2]_net_1\, 
        \RTCcal_present_ns_i_a5_4_0[2]_net_1\, 
        \RTCcal_present_ns_i_0[2]_net_1\, N_485_i, N_504, 
        \RTCcal_present_ns_i_1[1]_net_1\, N_408, 
        \RTCcal_present_ns_i_a5_0_0[1]_net_1\, 
        \RTCcal_present_ns_i_0[1]_net_1\, 
        \RTCcal_present_ns_i_a5_3_1[1]_net_1\, N_484_i_0, N_497, 
        \RTCcal_present_ns_0_0_0[4]_net_1\, N_899, 
        \ACMADDR_1_0[3]_net_1\, N_874, N_894, N_416, 
        \clr_sec_flag_0_a3_0\, N_904, \ACMADDR_1_0_1[6]_net_1\, 
        \RTC_match_45_e_1\, N_882, \numdays_month_7_i_0[0]_net_1\, 
        rtc_eq0, \RTC_match[39]_net_1\, 
        \RTCcal_present_ns_i_a5_3_0[2]_net_1\, 
        \un1_numdays_month15_2\, \numdays_month15_1\, 
        \un1_numdays_month15_0\, \ACMADDR_1_0_a2_0[1]_net_1\, 
        \un12_inc_hour_0\, \un18_inc_hour_0\, \un3_inc_hour_2\, 
        \un6_inc_hour_1_0\, \un12_inc_hour_2_0\, 
        \un3_inc_sec_2_0\, N_524, \leap_year\, 
        \count_hours_14[1]\, \count_hours_3_sqmuxa\, 
        \un30_count_hours_m[1]_net_1\, \count_hours_14[0]\, 
        \count_hours_4_sqmuxa\, \un30_count_hours_m[0]_net_1\, 
        N_665, N_665_1, N_503, N_496_i, clr_sec_flag_1_sqmuxa, 
        N_382_1, N_463_1, \count_hours_5_sqmuxa\, 
        \un1_numdays_month15\, \numdays_month26\, 
        \numdays_month17\, \numdays_month26_1\, N_586_1, N_438, 
        \RTCcal_present_d_i[18]\, N_901, N_902, N_902_1, 
        \rr_pwrdwn\, N_903, N_987_i, N_905, \count_ld_i_0_a2\, 
        N_889, \RTCcal_present_d[14]\, N_443, N_444, 
        \RTCcal_present_d_i[17]\, N_441, \RTCcal_present_d_i[23]\, 
        \un5_acmrdata\, \reg_rtc7\, \un28_count_year\, 
        \un3_inc_day\, \RTCcal_present_d_i[19]\, N_407_i_0, N_398, 
        N_453, N_454, N_623, \count_seconds_0_sqmuxa\, N_690_3, 
        N_586, N_776, \count_date_0_sqmuxa\, 
        \count_month_0_sqmuxa\, \un1_rtc_init_1\, 
        \count_day_1_sqmuxa\, \reg_rtc7_2\, 
        \count_day_10[1]_net_1\, I_13, \count_day_10[2]_net_1\, 
        I_14, N_415, N_403, clr_sec_flag, 
        \RTCcal_present[2]_net_1\, \RTCcal_present[3]_net_1\, 
        \count_month_13[7]_net_1\, \count_month_13[6]_net_1\, 
        \count_month_13[5]_net_1\, \count_day_11[7]_net_1\, 
        \count_day_11[6]_net_1\, \count_day_11[5]_net_1\, 
        \count_day_11[4]_net_1\, \count_day_11[3]_net_1\, 
        \count_date_13[7]_net_1\, \count_date_13[6]_net_1\, 
        \count_year_8[7]_net_1\, \count_year_8[6]_net_1\, 
        \count_year_8[5]_net_1\, \count_year_8[4]_net_1\, 
        \count_hours_15[7]_net_1\, \count_minutes_13[7]_net_1\, 
        \count_seconds_13[7]_net_1\, \count_month_13[0]_net_1\, 
        N_711, \count_month_13[1]_net_1\, N_712, 
        \count_month_13[2]_net_1\, N_713, 
        \count_month_13[3]_net_1\, N_714, 
        \count_month_13[4]_net_1\, N_715, \count_day_11[0]_net_1\, 
        N_706, \count_day_11[1]_net_1\, N_707, 
        \count_day_11[2]_net_1\, N_708, \count_date_13[0]_net_1\, 
        N_698, \count_date_13[1]_net_1\, N_699, 
        \count_date_13[2]_net_1\, N_700, \count_date_13[3]_net_1\, 
        N_701, \count_date_13[4]_net_1\, N_702, 
        \count_date_13[5]_net_1\, N_703, \count_year_8[0]_net_1\, 
        N_794, \count_year_8[1]_net_1\, N_795, 
        \count_year_8[2]_net_1\, N_796, \count_year_8[3]_net_1\, 
        N_797, \count_hours_15[0]_net_1\, N_785, 
        \count_hours_15[1]_net_1\, N_786, 
        \count_hours_15[2]_net_1\, N_787, 
        \count_hours_15[3]_net_1\, N_788, 
        \count_hours_15[4]_net_1\, N_789, 
        \count_hours_15[5]_net_1\, N_790, 
        \count_hours_15[6]_net_1\, N_791, \count_month_12[0]\, 
        N_772, \count_month_12[1]\, N_773, \count_month_12[4]\, 
        \count_minutes_13[0]_net_1\, N_764, 
        \count_minutes_13[1]_net_1\, N_765, 
        \count_minutes_13[2]_net_1\, N_766, 
        \count_minutes_13[3]_net_1\, N_767, 
        \count_minutes_13[4]_net_1\, N_509_1, N_768, 
        \count_minutes_13[5]_net_1\, N_769, 
        \count_minutes_13[6]_net_1\, N_770, \count_date_12[2]\, 
        N_752, \count_date_12[3]\, N_753, \count_date_12[4]\, 
        \count_seconds_13[0]_net_1\, N_732, 
        \count_seconds_13[1]_net_1\, N_733, 
        \count_seconds_13[2]_net_1\, N_734, 
        \count_seconds_13[3]_net_1\, N_735, 
        \count_seconds_13[4]_net_1\, N_736, 
        \count_seconds_13[5]_net_1\, N_737, 
        \count_seconds_13[6]_net_1\, N_738, 
        \count_minutes_12[5]_net_1\, N_728, 
        \count_minutes_12[6]_net_1\, N_729, \N_624_m\, 
        \un1_count_hours_m[5]_net_1\, \count_day_m[2]_net_1\, 
        sec_flag_2, \count_seconds_12[5]_net_1\, N_810, N_688, 
        N_686, N_687, \RTC_match[24]_net_1\, \count_date_12[0]\, 
        \RTC_match[25]_net_1\, \count_date_12[1]\, 
        \RTC_match[26]_net_1\, \RTC_match[27]_net_1\, 
        \RTC_match[28]_net_1\, \RTC_match[29]_net_1\, 
        \count_date_12[5]_net_1\, \RTC_match[21]_net_1\, 
        \DWACT_ADD_CI_0_partial_sum[0]\, \RTC_match[22]_net_1\, 
        \RTC_match[23]_net_1\, \RTC_match[30]_net_1\, 
        \RTC_match[31]_net_1\, \RTC_match[32]_net_1\, 
        \count_month_12[2]\, \RTC_match[34]_net_1\, N_7, 
        \RTC_match[0]_net_1\, \count_seconds_12[0]_net_1\, 
        \RTC_match[1]_net_1\, \count_seconds_12[1]_net_1\, 
        \RTC_match[2]_net_1\, N_807, \RTC_match[3]_net_1\, 
        \count_seconds_12[3]_net_1\, \RTC_match[4]_net_1\, N_809, 
        \RTC_match[5]_net_1\, \RTC_match[6]_net_1\, 
        \count_seconds_12[6]_net_1\, I_5_2, 
        \count_hours_m[2]_net_1\, \count_hours_14[2]\, I_9_2, 
        \count_hours_m[3]_net_1\, \count_hours_14[3]\, I_13_3, 
        N_744, \count_hours_14_1_iv[4]_net_1\, 
        \count_hours_14[5]\, \count_year_7[0]_net_1\, 
        \count_year_7[1]_net_1\, I_5, \count_year_7[3]_net_1\, 
        I_13_0, I_9_1, \RTC_match[7]_net_1\, N_723, 
        \RTC_match[8]_net_1\, N_724, \RTC_match[9]_net_1\, N_725, 
        \RTC_match[10]_net_1\, N_726, \RTC_match[11]_net_1\, 
        N_727, \RTC_match[12]_net_1\, \RTC_match[13]_net_1\, 
        I_5_0, \RTC_match[14]_net_1\, \RTC_match[15]_net_1\, 
        \RTC_match[16]_net_1\, \RTC_match[17]_net_1\, 
        \RTC_match[18]_net_1\, \RTC_match[19]_net_1\, 
        \RTC_match[20]_net_1\, \RTC_match[35]_net_1\, 
        \RTC_match[36]_net_1\, \RTC_match[38]_net_1\, N_801, 
        I_9_4, N_690, \numdays_month_79\, 
        \numdays_month[1]_net_1\, \numdays_month_80\, 
        \numdays_month[2]_net_1\, N_717, N_718, N_719, N_720, 
        \numdays_month_83\, \numdays_month[4]_net_1\, 
        \numdays_month[3]_net_1\, \numdays_month[0]_net_1\, 
        \RTCcal_present[4]_net_1\, \RTCcal_present[0]_net_1\, 
        \RTCcal_present[1]_net_1\, \reg_INIT_done\, 
        \RTCcal_present_d[22]\, \RTCcal_present_d[24]\, N_413_i_i, 
        N_461, N_802, N_800, N_799, N_7_0, N_811, I_13_5, I_5_4, 
        I_13_4, I_9_3, I_5_3, \RTC_match[37]_net_1\, 
        \count_year_7[2]_net_1\, N_775, I_13_1, N_774, I_9_0, 
        N_761, I_13_2, N_751, I_5_1, N_750, I_9, 
        \RTC_match[33]_net_1\, \count_month_12[3]\, \reg_pwrdwn\, 
        \RTC_load[0]_net_1\, \RTC_load[1]_net_1\, 
        \RTC_load[2]_net_1\, \RTC_load[3]_net_1\, 
        \RTC_load[4]_net_1\, \RTC_load[5]_net_1\, 
        \RTC_load[6]_net_1\, \RTC_load[7]_net_1\, 
        \RTC_load[8]_net_1\, \RTC_load[9]_net_1\, 
        \RTC_load[10]_net_1\, \RTC_load[11]_net_1\, 
        \RTC_load[12]_net_1\, \RTC_load[13]_net_1\, 
        \RTC_load[14]_net_1\, \RTC_load[15]_net_1\, 
        \RTC_load[16]_net_1\, \RTC_load[17]_net_1\, 
        \RTC_load[18]_net_1\, \RTC_load[19]_net_1\, 
        \RTC_load[20]_net_1\, \RTC_load[21]_net_1\, 
        \RTC_load[22]_net_1\, \RTC_load[23]_net_1\, 
        \RTC_load[24]_net_1\, \RTC_load[25]_net_1\, 
        \RTC_load[26]_net_1\, \RTC_load[27]_net_1\, 
        \RTC_load[28]_net_1\, \RTC_load[29]_net_1\, 
        \RTC_load[30]_net_1\, \RTC_load[31]_net_1\, N_7_1, N_4, 
        N_7_2, N_4_0, N_7_3, N_4_1, N_7_4, N_4_2, N_7_5, N_4_3, 
        N_7_6, N_4_4, \VCC\, \GND\, GND_net_1, VCC_net_1
         : std_logic;

    for all : RIP_COUNT32
	Use entity work.RIP_COUNT32(DEF_ARCH);
begin 

    RTC_year_c(7) <= \RTC_year_c[7]\;
    RTC_year_c(6) <= \RTC_year_c[6]\;
    RTC_year_c(5) <= \RTC_year_c[5]\;
    RTC_year_c(4) <= \RTC_year_c[4]\;
    RTC_year_c(3) <= \RTC_year_c[3]\;
    RTC_year_c(2) <= \RTC_year_c[2]\;
    RTC_year_c(1) <= \RTC_year_c[1]\;
    RTC_year_c(0) <= \RTC_year_c[0]\;
    RTC_seconds_c(7) <= \RTC_seconds_c[7]\;
    RTC_seconds_c(6) <= \RTC_seconds_c[6]\;
    RTC_seconds_c(5) <= \RTC_seconds_c[5]\;
    RTC_seconds_c(4) <= \RTC_seconds_c[4]\;
    RTC_seconds_c(3) <= \RTC_seconds_c[3]\;
    RTC_seconds_c(2) <= \RTC_seconds_c[2]\;
    RTC_seconds_c(1) <= \RTC_seconds_c[1]\;
    RTC_seconds_c(0) <= \RTC_seconds_c[0]\;
    RTC_hours_c(7) <= \RTC_hours_c[7]\;
    RTC_hours_c(6) <= \RTC_hours_c[6]\;
    RTC_hours_c(5) <= \RTC_hours_c[5]\;
    RTC_hours_c(4) <= \RTC_hours_c[4]\;
    RTC_hours_c(3) <= \RTC_hours_c[3]\;
    RTC_hours_c(2) <= \RTC_hours_c[2]\;
    RTC_hours_c(1) <= \RTC_hours_c[1]\;
    RTC_hours_c(0) <= \RTC_hours_c[0]\;
    RTC_date_c(7) <= \RTC_date_c[7]\;
    RTC_date_c(6) <= \RTC_date_c[6]\;
    RTC_date_c(5) <= \RTC_date_c[5]\;
    RTC_date_c(4) <= \RTC_date_c[4]\;
    RTC_date_c(3) <= \RTC_date_c[3]\;
    RTC_date_c(2) <= \RTC_date_c[2]\;
    RTC_date_c(1) <= \RTC_date_c[1]\;
    RTC_date_c(0) <= \RTC_date_c[0]\;
    RTC_month_c(7) <= \RTC_month_c[7]\;
    RTC_month_c(6) <= \RTC_month_c[6]\;
    RTC_month_c(5) <= \RTC_month_c[5]\;
    RTC_month_c(4) <= \RTC_month_c[4]\;
    RTC_month_c(3) <= \RTC_month_c[3]\;
    RTC_month_c(2) <= \RTC_month_c[2]\;
    RTC_month_c(1) <= \RTC_month_c[1]\;
    RTC_month_c(0) <= \RTC_month_c[0]\;
    RTC_minutes_c(7) <= \RTC_minutes_c[7]\;
    RTC_minutes_c(6) <= \RTC_minutes_c[6]\;
    RTC_minutes_c(5) <= \RTC_minutes_c[5]\;
    RTC_minutes_c(4) <= \RTC_minutes_c[4]\;
    RTC_minutes_c(3) <= \RTC_minutes_c[3]\;
    RTC_minutes_c(2) <= \RTC_minutes_c[2]\;
    RTC_minutes_c(1) <= \RTC_minutes_c[1]\;
    RTC_minutes_c(0) <= \RTC_minutes_c[0]\;
    RTC_day_c(7) <= \RTC_day_c[7]\;
    RTC_day_c(6) <= \RTC_day_c[6]\;
    RTC_day_c(5) <= \RTC_day_c[5]\;
    RTC_day_c(4) <= \RTC_day_c[4]\;
    RTC_day_c(3) <= \RTC_day_c[3]\;
    RTC_day_c(2) <= \RTC_day_c[2]\;
    RTC_day_c(1) <= \RTC_day_c[1]\;
    RTC_day_c(0) <= \RTC_day_c[0]\;
    N_410 <= \N_410\;
    N_417 <= \N_417\;
    RTC_act_i <= RTC_act_i_net_1;

    \RTCcal_present_ns_0[0]\ : OR3C
      port map(A => \RTCcal_present_ns_0_1[0]_net_1\, B => 
        \RTCcal_present_ns_0_0[0]_net_1\, C => N_496_i, Y => 
        \RTCcal_present_ns[0]\);
    
    \count_date_13_r[4]\ : AOI1B
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_702, Y => \count_date_13[4]_net_1\);
    
    \numdays_month[0]\ : DFN1E0P0
      port map(D => N_524, CLK => ACM_CLK_c, PRE => sys_reset_gl, 
        E => \un1_rtc_init_1\, Q => \numdays_month[0]_net_1\);
    
    \count_seconds_i_m[0]\ : OR2A
      port map(A => N_646, B => \RTC_seconds_c[0]\, Y => 
        \count_seconds_i_m[0]_net_1\);
    
    \RTC_load[21]\ : DFN1E0C0
      port map(D => ACMRDATA(5), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[17]\, Q => 
        \RTC_load[21]_net_1\);
    
    \RTCcal_present_ns_i_a5_2[2]\ : OR2A
      port map(A => N_499_1, B => \RTCcal_present[2]_net_1\, Y
         => N_505);
    
    \count_minutes_m[3]\ : OR2A
      port map(A => \RTC_minutes_c[3]\, B => N_653, Y => 
        \count_minutes_m[3]_net_1\);
    
    \count_date_i_m[3]\ : OR2A
      port map(A => N_665, B => \RTC_date_c[3]\, Y => 
        \count_date_i_m[3]_net_1\);
    
    \count_date_12_r[3]\ : NOR3A
      port map(A => \count_date_0_sqmuxa\, B => N_586, C => N_753, 
        Y => \count_date_12[3]\);
    
    clr_sec_flag_0_a3_0 : OR3C
      port map(A => \RTCcal_present_0[2]_net_1\, B => 
        \RTCcal_present_0[4]_net_1\, C => \sec_flag\, Y => 
        \clr_sec_flag_0_a3_0\);
    
    \RTCcal_present_ns_i[1]\ : NOR3C
      port map(A => \RTCcal_present_ns_i_0[1]_net_1\, B => N_499, 
        C => \RTCcal_present_ns_i_1[1]_net_1\, Y => N_479_i_0);
    
    \count_date_13[3]\ : MX2
      port map(A => \RTC_match[27]_net_1\, B => 
        \count_date_12[3]\, S => un1_rtc_init, Y => N_701);
    
    \count_minutes_12[4]\ : XOR2
      port map(A => \RTC_minutes_c[4]\, B => N_623, Y => N_727);
    
    un8_inc_day_4 : XNOR2
      port map(A => \RTC_date_c[4]\, B => 
        \numdays_month[4]_net_1\, Y => un8_inc_day_4_i_0);
    
    count_date_0_sqmuxa : OR2A
      port map(A => \count_date_0_sqmuxa_4\, B => 
        \count_hours_3_sqmuxa\, Y => \count_date_0_sqmuxa\);
    
    \RTCcal_present[4]\ : DFN1C0
      port map(D => \RTCcal_present_ns[4]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \RTCcal_present[4]_net_1\);
    
    \count_seconds_12[3]\ : NOR2
      port map(A => N_802, B => N_690, Y => 
        \count_seconds_12[3]_net_1\);
    
    \count_month_13[0]\ : MX2
      port map(A => \RTC_match[30]_net_1\, B => 
        \count_month_12[0]\, S => un1_rtc_init, Y => N_711);
    
    \count_seconds_m[5]\ : OR2B
      port map(A => \RTC_seconds_c[5]\, B => N_646, Y => 
        \count_seconds_m[5]_net_1\);
    
    \count_day[6]\ : DFN1C0
      port map(D => \count_day_11[6]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_day_c[6]\);
    
    un1_count_day_I_9 : XOR2
      port map(A => \RTC_day_c[0]\, B => \count_day_1_sqmuxa\, Y
         => \DWACT_ADD_CI_0_partial_sum[0]\);
    
    un5_acmrdata : OR3B
      port map(A => RTC_act_i_net_1, B => \reg_rtc7\, C => 
        ACMRDATA(7), Y => \un5_acmrdata\);
    
    \RTC_load[31]\ : DFN1E0C0
      port map(D => ACMRDATA(7), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[19]\, Q => 
        \RTC_load[31]_net_1\);
    
    rr_pwrdwn : DFN1C0
      port map(D => \reg_pwrdwn\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \rr_pwrdwn\);
    
    clr_sec_flag_0_o4 : OA1B
      port map(A => N_463_1, B => \clr_sec_flag_0_a3_0\, C => 
        N_455, Y => clr_sec_flag);
    
    un23_count_seconds_I_5 : XOR2
      port map(A => \RTC_seconds_c[0]\, B => \RTC_seconds_c[1]\, 
        Y => I_5_4);
    
    \count_minutes[0]\ : DFN1C0
      port map(D => \count_minutes_13[0]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_minutes_c[0]\);
    
    \count_month_13[5]\ : NOR2B
      port map(A => \RTC_month_c[5]\, B => un1_rtc_init, Y => 
        \count_month_13[5]_net_1\);
    
    \count_seconds_13[6]\ : MX2
      port map(A => \RTC_match[6]_net_1\, B => 
        \count_seconds_12[6]_net_1\, S => un1_rtc_init, Y => 
        N_738);
    
    \ACMWDATA_iv_1[0]\ : OA1
      port map(A => N_653, B => \RTC_minutes_c[1]\, C => 
        \count_date_i_m[0]_net_1\, Y => \ACMWDATA_iv_1[0]_net_1\);
    
    \RTCcal_present_ns_0_0_a2_1[4]\ : OR2A
      port map(A => N_509_1, B => \RTCcal_present[1]_net_1\, Y
         => N_901);
    
    \numdays_month_7_i[0]\ : OA1
      port map(A => \leap_year\, B => \numdays_month16\, C => 
        \numdays_month_7_i_0[0]_net_1\, Y => N_524);
    
    RTC_act_i_o4 : NOR2A
      port map(A => \RTCcal_present_0[2]_net_1\, B => 
        \RTCcal_present_0[0]_net_1\, Y => N_403);
    
    \count_seconds_13_s[0]\ : AO1
      port map(A => N_509_1, B => \RTCcal_present_s21_0_a4_0\, C
         => N_732, Y => \count_seconds_13[0]_net_1\);
    
    \RTCcal_present_ns_i[2]\ : NOR3C
      port map(A => \RTCcal_present_ns_i_1[2]_net_1\, B => N_503, 
        C => \RTCcal_present_ns_i_2[2]_net_1\, Y => N_481_i_0);
    
    I_263_1 : NOR2A
      port map(A => \RTC_minutes_c[3]\, B => \RTC_minutes_c[2]\, 
        Y => N_623_1_i);
    
    N_624_m : OR2
      port map(A => \RTC_minutes_c[2]\, B => N_653, Y => 
        \N_624_m\);
    
    un1_count_day_I_1 : AND2
      port map(A => \RTC_day_c[0]\, B => \count_day_1_sqmuxa\, Y
         => \DWACT_ADD_CI_0_TMP[0]\);
    
    \count_hours_15_s[5]\ : AO1
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_790, Y => \count_hours_15[5]_net_1\);
    
    \count_hours_15_s[6]\ : AO1
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_791, Y => \count_hours_15[6]_net_1\);
    
    \RTCcal_present_ns_0_0_a2_2_2[4]\ : NOR2A
      port map(A => INIT_DONE_c, B => \reg_INIT_done\, Y => 
        N_902_2);
    
    \ACMWDATA_r[3]\ : NOR3C
      port map(A => \un1_count_hours_m[5]_net_1\, B => 
        \ACMWDATA_iv_2[3]_net_1\, C => N_398, Y => ACMWDATA(3));
    
    un23_count_seconds_I_12 : AND3
      port map(A => \RTC_seconds_c[0]\, B => \RTC_seconds_c[1]\, 
        C => \RTC_seconds_c[2]\, Y => N_4_1);
    
    \RTC_match[17]\ : DFN1E0C0
      port map(D => ACMRDATA(1), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_496_i, Q => \RTC_match[17]_net_1\);
    
    \RTCcal_present_ns_i_a5_2_0_a2[1]\ : NOR2A
      port map(A => \RTCcal_present[4]_net_1\, B => N_408, Y => 
        N_903);
    
    un22_count_month_I_9 : XOR2
      port map(A => N_7_5, B => \RTC_month_c[2]\, Y => I_9_0);
    
    \RTC_load[6]\ : DFN1E0C0
      port map(D => ACMRDATA(6), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[14]\, Q => 
        \RTC_load[6]_net_1\);
    
    \count_seconds_12_0[0]\ : XOR2
      port map(A => N_690_3, B => \RTC_seconds_c[0]\, Y => N_799);
    
    \RTCcal_present_ns_0_a5_3_1[0]\ : OR2A
      port map(A => \RTCcal_present[2]_net_1\, B => N_408, Y => 
        N_496_1);
    
    I_297_1 : OR2A
      port map(A => \RTCcal_present_0[0]_net_1\, B => 
        \RTCcal_present[2]_net_1\, Y => N_665_1);
    
    \count_month[5]\ : DFN1C0
      port map(D => \count_month_13[5]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_month_c[5]\);
    
    un24_count_minutes_I_13 : XOR2
      port map(A => N_4_0, B => \RTC_minutes_c[3]\, Y => I_13_4);
    
    I_263_0 : OR3A
      port map(A => \RTC_minutes_c[3]\, B => \RTC_minutes_c[2]\, 
        C => N_623_2, Y => \I_263_0\);
    
    \un1_count_hours_m[5]\ : OR2
      port map(A => \RTC_hours_c[5]\, B => N_496_1, Y => 
        \un1_count_hours_m[5]_net_1\);
    
    \ACMADDR_1_o2[4]\ : NOR2
      port map(A => N_400_i_0, B => \RTCcal_present_0[0]_net_1\, 
        Y => N_402);
    
    \count_year_8[0]\ : MX2
      port map(A => \RTC_match[35]_net_1\, B => 
        \count_year_7[0]_net_1\, S => un1_rtc_init, Y => N_794);
    
    \count_month_12_0[0]\ : XOR2
      port map(A => \count_date_0_sqmuxa\, B => \RTC_month_c[0]\, 
        Y => N_772);
    
    \count_year_8_r[0]\ : AOI1B
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_794, Y => \count_year_8[0]_net_1\);
    
    \count_minutes_m[6]\ : OR2A
      port map(A => \RTC_minutes_c[6]\, B => N_653, Y => 
        \count_minutes_m[6]_net_1\);
    
    \count_date_13[6]\ : NOR2B
      port map(A => \RTC_date_c[6]\, B => un1_rtc_init, Y => 
        \count_date_13[6]_net_1\);
    
    \count_month_12_r[1]\ : NOR3
      port map(A => N_773, B => N_776, C => 
        \count_month_0_sqmuxa\, Y => \count_month_12[1]\);
    
    un11_count_year_I_12 : AND3
      port map(A => \RTC_year_c[0]\, B => \RTC_year_c[1]\, C => 
        \RTC_year_c[2]\, Y => N_4_2);
    
    \count_hours_m[3]\ : OR2B
      port map(A => \RTC_hours_c[3]\, B => 
        \count_minutes_0_sqmuxa\, Y => \count_hours_m[3]_net_1\);
    
    un12_inc_hour_0 : OR2A
      port map(A => \RTC_hours_c[0]\, B => \RTC_hours_c[5]\, Y
         => \un12_inc_hour_0\);
    
    \count_seconds_i_m[6]\ : OR2A
      port map(A => N_646, B => \RTC_seconds_c[6]\, Y => 
        \count_seconds_i_m[6]_net_1\);
    
    \count_date_13[5]\ : MX2
      port map(A => \RTC_match[29]_net_1\, B => 
        \count_date_12[5]_net_1\, S => un1_rtc_init, Y => N_703);
    
    un28_count_year : NOR3A
      port map(A => \RTC_year_c[3]\, B => \RTC_year_c[1]\, C => 
        \RTC_year_c[0]\, Y => \un28_count_year\);
    
    \RTC_match[37]\ : DFN1E0C0
      port map(D => ACMRDATA(5), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[24]\, Q => 
        \RTC_match[37]_net_1\);
    
    \count_year_7[3]\ : MX2
      port map(A => \RTC_year_c[3]\, B => I_13_0, S => 
        \count_month_0_sqmuxa\, Y => \count_year_7[3]_net_1\);
    
    \RTC_load[9]\ : DFN1E0C0
      port map(D => ACMRDATA(1), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[18]\, Q => 
        \RTC_load[9]_net_1\);
    
    N_484_i_0_a2 : OR2B
      port map(A => \RTCcal_present[3]_net_1\, B => 
        \RTCcal_present[2]_net_1\, Y => N_484_i_0);
    
    RTCcal_present_s21_0_a4_0 : NOR2A
      port map(A => \RTCcal_present_0[2]_net_1\, B => N_458, Y
         => \RTCcal_present_s21_0_a4_0\);
    
    \RTC_match[15]\ : DFN1E1C0
      port map(D => ACMRDATA(7), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[22]\, Q => 
        \RTC_match[15]_net_1\);
    
    \count_day_11[3]\ : NOR2B
      port map(A => \RTC_day_c[3]\, B => un1_rtc_init, Y => 
        \count_day_11[3]_net_1\);
    
    \count_date_13[1]\ : MX2
      port map(A => \RTC_match[25]_net_1\, B => 
        \count_date_12[1]\, S => un1_rtc_init, Y => N_699);
    
    \RTC_match[14]\ : DFN1E1C0
      port map(D => ACMRDATA(6), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[22]\, Q => 
        \RTC_match[14]_net_1\);
    
    \ACMADDR_1_a3[3]\ : NOR2A
      port map(A => \RTCcal_present[3]_net_1\, B => N_400_i_0, Y
         => N_461);
    
    \RTC_match[7]\ : DFN1E0C0
      port map(D => ACMRDATA(7), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_987_i, Q => \RTC_match[7]_net_1\);
    
    \count_seconds_13_r[1]\ : AOI1B
      port map(A => N_509_1, B => \RTCcal_present_s21_0_a4_0\, C
         => N_733, Y => \count_seconds_13[1]_net_1\);
    
    \count_date_m[2]\ : OR2B
      port map(A => \RTC_date_c[2]\, B => N_665, Y => 
        \count_date_m[2]_net_1\);
    
    \count_year_7[2]\ : MX2
      port map(A => \RTC_year_c[2]\, B => I_9, S => 
        \count_month_0_sqmuxa\, Y => \count_year_7[2]_net_1\);
    
    \count_month_13_r[2]\ : AOI1B
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_713, Y => \count_month_13[2]_net_1\);
    
    un1_count_day_I_14 : XOR2
      port map(A => \RTC_day_c[2]\, B => 
        \DWACT_ADD_CI_0_g_array_1[0]\, Y => I_14);
    
    \count_minutes[4]\ : DFN1C0
      port map(D => \count_minutes_13[4]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_minutes_c[4]\);
    
    \count_minutes_13[6]\ : MX2
      port map(A => \RTC_match[13]_net_1\, B => 
        \count_minutes_12[6]_net_1\, S => un1_rtc_init, Y => 
        N_770);
    
    un6_inc_hour_1_0 : NOR2A
      port map(A => \RTC_hours_c[4]\, B => \RTC_hours_c[0]\, Y
         => \un6_inc_hour_1_0\);
    
    \count_hours_15[2]\ : MX2
      port map(A => \RTC_match[16]_net_1\, B => 
        \count_hours_14[2]\, S => un1_rtc_init, Y => N_787);
    
    \RTCcal_present_0[2]\ : DFN1C0
      port map(D => N_481_i_0, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \RTCcal_present_0[2]_net_1\);
    
    \count_seconds_13_r[3]\ : AOI1B
      port map(A => N_509_1, B => \RTCcal_present_s21_0_a4_0\, C
         => N_735, Y => \count_seconds_13[3]_net_1\);
    
    un24_count_minutes_I_5 : XOR2
      port map(A => \RTC_minutes_c[0]\, B => \RTC_minutes_c[1]\, 
        Y => I_5_3);
    
    \RTC_act_i\ : NOR2
      port map(A => N_485_i, B => N_382_1, Y => RTC_act_i_net_1);
    
    \count_date_13_r[2]\ : AOI1B
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_700, Y => \count_date_13[2]_net_1\);
    
    un5_rtc_init_i_x2_0_a2 : XNOR2
      port map(A => \RTCcal_present[4]_net_1\, B => 
        \RTCcal_present[1]_net_1\, Y => N_413_i_i);
    
    un1_numdays_month15 : OR3C
      port map(A => \numdays_month26\, B => \numdays_month17\, C
         => \un1_numdays_month15_2\, Y => \un1_numdays_month15\);
    
    un16_count_seconds_1_CO2_0_o2 : OR2B
      port map(A => \RTC_seconds_c[5]\, B => \RTC_seconds_c[4]\, 
        Y => N_7_0);
    
    \RTC_match[27]\ : DFN1E0C0
      port map(D => ACMRDATA(3), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[23]\, Q => 
        \RTC_match[27]_net_1\);
    
    un3_inc_hour : OR3A
      port map(A => \un3_inc_hour_2\, B => \RTC_hours_c[1]\, C
         => \RTC_hours_c[2]\, Y => un3_inc_hour_i_0);
    
    \count_date_12_s[0]\ : OAI1
      port map(A => N_586, B => N_750, C => \count_date_0_sqmuxa\, 
        Y => \count_date_12[0]\);
    
    ACMWEN_0_a3 : OR2A
      port map(A => \RTCcal_present_0[4]_net_1\, B => N_894, Y
         => \N_417\);
    
    \count_minutes_13[3]\ : MX2
      port map(A => \RTC_match[10]_net_1\, B => N_726, S => 
        un1_rtc_init, Y => N_767);
    
    \count_date_12_0[3]\ : MX2C
      port map(A => I_13_2, B => \RTC_date_c[3]\, S => 
        \count_hours_3_sqmuxa\, Y => N_753);
    
    \RTC_match[35]\ : DFN1E0C0
      port map(D => ACMRDATA(3), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[24]\, Q => 
        \RTC_match[35]_net_1\);
    
    clr_sec_flag_1_sqmuxa_0_a4 : NOR3
      port map(A => N_382_1, B => \sec_flag\, C => N_463_1, Y => 
        clr_sec_flag_1_sqmuxa);
    
    \RTC_match[34]\ : DFN1E0C0
      port map(D => ACMRDATA(2), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[24]\, Q => 
        \RTC_match[34]_net_1\);
    
    \ACMADDR_1_a4_0[4]\ : OR3A
      port map(A => \RTCcal_present_0[3]_net_1\, B => N_400_i_0, 
        C => \RTCcal_present_0[0]_net_1\, Y => N_444);
    
    \count_seconds_12[4]\ : XOR2
      port map(A => \RTC_seconds_c[4]\, B => N_690, Y => N_809);
    
    \count_minutes_12[2]\ : NOR2
      port map(A => N_719, B => N_623, Y => N_725);
    
    \RTC_load[26]\ : DFN1E0C0
      port map(D => ACMRDATA(2), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[19]\, Q => 
        \RTC_load[26]_net_1\);
    
    \RTC_load[20]\ : DFN1E0C0
      port map(D => ACMRDATA(4), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[17]\, Q => 
        \RTC_load[20]_net_1\);
    
    I_316 : OR2A
      port map(A => N_499_2, B => rtc_eq0, Y => N_687);
    
    \ACMADDR_1_a4[4]\ : OR2B
      port map(A => \RTCcal_present_0[4]_net_1\, B => N_415, Y
         => N_443);
    
    \count_hours[3]\ : DFN1C0
      port map(D => \count_hours_15[3]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_hours_c[3]\);
    
    un1_rtc_init_1 : OR2B
      port map(A => un1_rtc_init, B => \un1_numdays_month15\, Y
         => \un1_rtc_init_1\);
    
    \count_month[7]\ : DFN1C0
      port map(D => \count_month_13[7]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_month_c[7]\);
    
    \un1_rtc_init_0\ : OR2B
      port map(A => N_415, B => N_413_i_i, Y => un1_rtc_init_0);
    
    \RTC_match[25]\ : DFN1E0C0
      port map(D => ACMRDATA(1), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[23]\, Q => 
        \RTC_match[25]_net_1\);
    
    count_hours_5_sqmuxa_1 : NOR2
      port map(A => \un12_inc_hour\, B => \un24_inc_hour\, Y => 
        \count_hours_5_sqmuxa_1\);
    
    \RTC_load[30]\ : DFN1E0C0
      port map(D => ACMRDATA(6), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[19]\, Q => 
        \RTC_load[30]_net_1\);
    
    \RTC_match[24]\ : DFN1E0C0
      port map(D => ACMRDATA(0), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[23]\, Q => 
        \RTC_match[24]_net_1\);
    
    \ACMWDATA_r[6]\ : NOR3C
      port map(A => \ACMWDATA_iv_1[6]_net_1\, B => 
        \ACMWDATA_iv_2[6]_net_1\, C => N_398, Y => ACMWDATA(6));
    
    \RTC_match[8]\ : DFN1E1C0
      port map(D => ACMRDATA(0), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[22]\, Q => 
        \RTC_match[8]_net_1\);
    
    RTCcal_present_s9_0_a4 : OR3
      port map(A => N_407_i_0, B => \RTCcal_present_0[1]_net_1\, 
        C => N_874, Y => \RTCcal_present_d_i[19]\);
    
    \count_month_12_r[2]\ : NOR3
      port map(A => N_774, B => N_776, C => 
        \count_month_0_sqmuxa\, Y => \count_month_12[2]\);
    
    reg_rtc7_2 : NOR2B
      port map(A => RTC_act_i_net_1, B => ACMRDATA(7), Y => 
        \reg_rtc7_2\);
    
    \count_minutes_12_0[0]\ : XOR2
      port map(A => \count_seconds_0_sqmuxa\, B => 
        \RTC_minutes_c[0]\, Y => N_717);
    
    numdays_month17 : OR3B
      port map(A => \RTC_month_c[1]\, B => \numdays_month17_1\, C
         => \RTC_month_c[4]\, Y => \numdays_month17\);
    
    \count_day_11[7]\ : NOR2B
      port map(A => \RTC_day_c[7]\, B => un1_rtc_init, Y => 
        \count_day_11[7]_net_1\);
    
    \count_date_m[5]\ : OR2B
      port map(A => \RTC_date_c[5]\, B => N_665, Y => 
        \count_date_m[5]_net_1\);
    
    RTCcal_present_s4_0_a4 : OR3
      port map(A => N_874, B => \RTCcal_present[0]_net_1\, C => 
        N_894, Y => \RTCcal_present_d[24]\);
    
    leap_year : DFN1E1C0
      port map(D => \un28_count_year\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => un1_rtc_init, Q => \leap_year\);
    
    count_seconds_0_sqmuxa : OR2A
      port map(A => \count_seconds_0_sqmuxa_2\, B => N_690_3, Y
         => \count_seconds_0_sqmuxa\);
    
    \count_hours[5]\ : DFN1C0
      port map(D => \count_hours_15[5]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_hours_c[5]\);
    
    \count_hours[2]\ : DFN1C0
      port map(D => \count_hours_15[2]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_hours_c[2]\);
    
    \count_hours_14_0[4]\ : NOR2
      port map(A => \count_hours_5_sqmuxa_3\, B => 
        \count_minutes_0_sqmuxa\, Y => N_744);
    
    \count_year[5]\ : DFN1C0
      port map(D => \count_year_8[5]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_year_c[5]\);
    
    count_hours_5_sqmuxa_3 : NOR2B
      port map(A => \count_hours_5_sqmuxa_1\, B => 
        \count_hours_5_sqmuxa_2\, Y => \count_hours_5_sqmuxa_3\);
    
    \count_hours_15[6]\ : MX2
      port map(A => \RTC_match[20]_net_1\, B => \RTC_hours_c[6]\, 
        S => un1_rtc_init, Y => N_791);
    
    \count_date_12_r[5]\ : NOR2B
      port map(A => \count_date_0_sqmuxa\, B => N_761, Y => 
        \count_date_12[5]_net_1\);
    
    GND_i : GND
      port map(Y => \GND\);
    
    \count_minutes_13_s[4]\ : AO1
      port map(A => N_509_1, B => \RTCcal_present_s21_0_a4_0\, C
         => N_768, Y => \count_minutes_13[4]_net_1\);
    
    \count_day_i_m[1]\ : OR2
      port map(A => \RTC_day_c[1]\, B => N_496_1, Y => 
        \count_day_i_m[1]_net_1\);
    
    RTCcal_present_s6_0_o2 : OR2
      port map(A => \RTCcal_present_0[3]_net_1\, B => 
        \RTCcal_present_0[1]_net_1\, Y => N_408);
    
    \count_minutes_13_s[6]\ : AO1
      port map(A => N_509_1, B => \RTCcal_present_s21_0_a4_0\, C
         => N_770, Y => \count_minutes_13[6]_net_1\);
    
    \count_seconds[3]\ : DFN1C0
      port map(D => \count_seconds_13[3]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_seconds_c[3]\);
    
    \count_date[7]\ : DFN1C0
      port map(D => \count_date_13[7]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_date_c[7]\);
    
    \count_minutes_12_0[2]\ : MX2C
      port map(A => I_9_3, B => \RTC_minutes_c[2]\, S => 
        \count_seconds_0_sqmuxa\, Y => N_719);
    
    \RTC_load[13]\ : DFN1E0C0
      port map(D => ACMRDATA(5), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[18]\, Q => 
        \RTC_load[13]_net_1\);
    
    reg_rtc7 : DFN1E1C0
      port map(D => \reg_rtc7_2\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => un1_rtc_init, Q => \reg_rtc7\);
    
    \count_seconds_12[6]\ : AX1
      port map(A => N_7_0, B => N_690, C => \RTC_seconds_c[6]\, Y
         => N_811);
    
    count_date_0_sqmuxa_3 : NOR3C
      port map(A => un8_inc_day_2_i_0, B => un8_inc_day_3_i_0, C
         => un8_inc_day_0_i_0, Y => \count_date_0_sqmuxa_3\);
    
    \RTC_load[8]\ : DFN1E0C0
      port map(D => ACMRDATA(0), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[18]\, Q => 
        \RTC_load[8]_net_1\);
    
    un1_numdays_month15_2 : AOI1B
      port map(A => \numdays_month17_1\, B => \numdays_month15_1\, 
        C => \un1_numdays_month15_0\, Y => 
        \un1_numdays_month15_2\);
    
    count_minutes_0_sqmuxa_1 : AND2
      port map(A => \count_minutes_0_sqmuxa_0\, B => 
        \RTC_minutes_c[4]\, Y => \count_minutes_0_sqmuxa_1\);
    
    \count_seconds_13_s[6]\ : AO1
      port map(A => N_509_1, B => \RTCcal_present_s21_0_a4_0\, C
         => N_738, Y => \count_seconds_13[6]_net_1\);
    
    count_hours_5_sqmuxa_2 : NOR3A
      port map(A => un3_inc_hour_i_0, B => \un18_inc_hour\, C => 
        \un6_inc_hour\, Y => \count_hours_5_sqmuxa_2\);
    
    un5_rtc_init_i_o3_i_a2 : OR2B
      port map(A => \RTCcal_present[3]_net_1\, B => 
        \RTCcal_present[0]_net_1\, Y => N_407_i_0);
    
    \RTC_load[12]\ : DFN1E0C0
      port map(D => ACMRDATA(4), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[18]\, Q => 
        \RTC_load[12]_net_1\);
    
    \RTCcal_present_ns_i_a5_1_1[1]\ : NOR2
      port map(A => \RTCcal_present[4]_net_1\, B => 
        \RTCcal_present[0]_net_1\, Y => N_499_1);
    
    \count_hours[0]\ : DFN1C0
      port map(D => \count_hours_15[0]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_hours_c[0]\);
    
    \RTCcal_present_ns_i_a5_3[2]\ : OR2A
      port map(A => \RTCcal_present_ns_i_a5_3_0[2]_net_1\, B => 
        N_894, Y => N_506);
    
    RTC_act_i_a3_0_a2 : OR2A
      port map(A => \RTCcal_present[1]_net_1\, B => 
        \RTCcal_present[3]_net_1\, Y => N_894);
    
    un1_count_day_I_13 : XOR2
      port map(A => \RTC_day_c[1]\, B => \DWACT_ADD_CI_0_TMP[0]\, 
        Y => I_13);
    
    un1_count_day_I_15 : NOR2B
      port map(A => \DWACT_ADD_CI_0_TMP[0]\, B => \RTC_day_c[1]\, 
        Y => \DWACT_ADD_CI_0_g_array_1[0]\);
    
    un12_inc_hour : NOR3B
      port map(A => N_586_1, B => \un12_inc_hour_2_0\, C => 
        \un12_inc_hour_0\, Y => \un12_inc_hour\);
    
    count_seconds_0_sqmuxa_2 : NOR2A
      port map(A => \count_seconds_0_sqmuxa_1\, B => 
        \un3_inc_sec_2\, Y => \count_seconds_0_sqmuxa_2\);
    
    \RTCcal_present_ns_i_1[1]\ : AO18
      port map(A => \RTCcal_present_0[4]_net_1\, B => N_408, C
         => \RTCcal_present_ns_i_a5_0_0[1]_net_1\, Y => 
        \RTCcal_present_ns_i_1[1]_net_1\);
    
    N_602_m : OR2
      port map(A => \RTC_hours_c[2]\, B => N_496_1, Y => 
        \N_602_m\);
    
    \count_seconds[4]\ : DFN1C0
      port map(D => \count_seconds_13[4]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_seconds_c[4]\);
    
    \count_minutes[2]\ : DFN1C0
      port map(D => \count_minutes_13[2]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_minutes_c[2]\);
    
    \RTC_load[27]\ : DFN1E0C0
      port map(D => ACMRDATA(3), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[19]\, Q => 
        \RTC_load[27]_net_1\);
    
    \ACMADDR_1_a4[2]\ : OR3A
      port map(A => \RTCcal_present_0[4]_net_1\, B => 
        \RTCcal_present_0[2]_net_1\, C => N_458, Y => N_441);
    
    \RTC_load[5]\ : DFN1E0C0
      port map(D => ACMRDATA(5), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[14]\, Q => 
        \RTC_load[5]_net_1\);
    
    \count_seconds_12_0[2]\ : MX2C
      port map(A => I_9_4, B => \RTC_seconds_c[2]\, S => N_690_3, 
        Y => N_801);
    
    clr_sec_flag_0_a2_0 : AND2
      port map(A => N_402, B => \clr_sec_flag_0_a2_0_0\, Y => 
        N_455);
    
    \RTC_load[24]\ : DFN1E0C0
      port map(D => ACMRDATA(0), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[19]\, Q => 
        \RTC_load[24]_net_1\);
    
    count_hours_3_sqmuxa : OR2A
      port map(A => \un18_inc_hour\, B => 
        \count_minutes_0_sqmuxa\, Y => \count_hours_3_sqmuxa\);
    
    un23_count_seconds_I_9 : XOR2
      port map(A => N_7_3, B => \RTC_seconds_c[2]\, Y => I_9_4);
    
    \RTC_load[25]\ : DFN1E0C0
      port map(D => ACMRDATA(1), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[19]\, Q => 
        \RTC_load[25]_net_1\);
    
    RTCcal_present_s21_0_a4_0_1 : NOR2A
      port map(A => \RTCcal_present_0[2]_net_1\, B => N_458, Y
         => \RTCcal_present_s21_0_a4_0_1\);
    
    \un30_count_hours_m[0]\ : OR2A
      port map(A => \count_hours_5_sqmuxa\, B => \RTC_hours_c[0]\, 
        Y => \un30_count_hours_m[0]_net_1\);
    
    \count_minutes_12[3]\ : NOR2
      port map(A => N_720, B => N_623, Y => N_726);
    
    N_563_m : OR2A
      port map(A => N_672, B => \RTC_month_c[2]\, Y => \N_563_m\);
    
    \count_day[0]\ : DFN1C0
      port map(D => \count_day_11[0]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_day_c[0]\);
    
    \RTC_load[1]\ : DFN1E0C0
      port map(D => ACMRDATA(1), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[14]\, Q => 
        \RTC_load[1]_net_1\);
    
    \count_month_13_r[0]\ : AOI1B
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_711, Y => \count_month_13[0]_net_1\);
    
    \RTCcal_present_ns_i_a5_0_0[1]\ : NOR2B
      port map(A => \RTCcal_present_0[2]_net_1\, B => 
        \RTCcal_present_0[0]_net_1\, Y => 
        \RTCcal_present_ns_i_a5_0_0[1]_net_1\);
    
    un1_numdays_month15_0 : XO1A
      port map(A => \RTC_month_c[3]\, B => \RTC_month_c[2]\, C
         => \RTC_month_c[4]\, Y => \un1_numdays_month15_0\);
    
    \count_hours_15_s[0]\ : AO1
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_785, Y => \count_hours_15[0]_net_1\);
    
    \count_year_8[1]\ : MX2
      port map(A => \RTC_match[36]_net_1\, B => 
        \count_year_7[1]_net_1\, S => un1_rtc_init, Y => N_795);
    
    I_317 : OR2B
      port map(A => \sec_flag\, B => N_889, Y => N_686);
    
    \count_day_11_s[1]\ : AO1
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_707, Y => \count_day_11[1]_net_1\);
    
    \count_day[2]\ : DFN1C0
      port map(D => \count_day_11[2]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_day_c[2]\);
    
    \RTC_match[10]\ : DFN1E1C0
      port map(D => ACMRDATA(2), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[22]\, Q => 
        \RTC_match[10]_net_1\);
    
    \count_month_13[6]\ : NOR2B
      port map(A => \RTC_month_c[6]\, B => un1_rtc_init, Y => 
        \count_month_13[6]_net_1\);
    
    \ACMWDATA_iv_1[3]\ : OA1A
      port map(A => N_672, B => \RTC_year_c[0]\, C => 
        \count_date_i_m[3]_net_1\, Y => \ACMWDATA_iv_1[3]_net_1\);
    
    \count_minutes_13_s[0]\ : AO1
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_764, Y => \count_minutes_13[0]_net_1\);
    
    \count_month_m[1]\ : OR2B
      port map(A => \RTC_month_c[1]\, B => N_665, Y => 
        \count_month_m[1]_net_1\);
    
    \count_minutes_12_r[5]\ : NOR2B
      port map(A => \count_minutes_0_sqmuxa\, B => N_728, Y => 
        \count_minutes_12[5]_net_1\);
    
    \ACMWDATA_r[0]\ : NOR3C
      port map(A => \ACMWDATA_iv_1[0]_net_1\, B => 
        \ACMWDATA_iv_2[0]_net_1\, C => N_398, Y => ACMWDATA(0));
    
    un3_inc_sec_2_0 : OR2A
      port map(A => \RTC_seconds_c[3]\, B => \RTC_seconds_c[2]\, 
        Y => \un3_inc_sec_2_0\);
    
    \ACMWDATA_iv_0[7]\ : AOI1
      port map(A => N_646, B => \RTC_minutes_c[0]\, C => N_672, Y
         => \ACMWDATA_iv_0[7]_net_1\);
    
    \count_month_12_0[1]\ : MX2C
      port map(A => I_5_0, B => \RTC_month_c[1]\, S => 
        \count_date_0_sqmuxa\, Y => N_773);
    
    numdays_month_80 : NOR2B
      port map(A => \un1_rtc_init_1\, B => 
        \numdays_month[2]_net_1\, Y => \numdays_month_80\);
    
    \RTC_match[18]\ : DFN1E0C0
      port map(D => ACMRDATA(2), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_496_i, Q => \RTC_match[18]_net_1\);
    
    \count_month[3]\ : DFN1C0
      port map(D => \count_month_13[3]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_month_c[3]\);
    
    \RTCcal_present_ns_0_a5_0_1_0_a2_0[3]\ : NOR2B
      port map(A => \RTCcal_present_0[4]_net_1\, B => 
        \RTCcal_present_0[0]_net_1\, Y => N_509_1_0);
    
    \count_year[0]\ : DFN1C0
      port map(D => \count_year_8[0]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_year_c[0]\);
    
    \count_minutes_13[5]\ : MX2
      port map(A => \RTC_match[12]_net_1\, B => 
        \count_minutes_12[5]_net_1\, S => un1_rtc_init, Y => 
        N_769);
    
    \count_minutes_13_r[1]\ : AOI1B
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_765, Y => \count_minutes_13[1]_net_1\);
    
    \count_hours_m[1]\ : OR2A
      port map(A => \RTC_hours_c[1]\, B => N_653, Y => 
        \count_hours_m[1]_net_1\);
    
    \count_year_m[1]\ : OR2B
      port map(A => \RTC_year_c[1]\, B => N_672, Y => 
        \count_year_m[1]_net_1\);
    
    \count_minutes_13_r[5]\ : AOI1B
      port map(A => N_509_1, B => \RTCcal_present_s21_0_a4_0\, C
         => N_769, Y => \count_minutes_13[5]_net_1\);
    
    \count_day_11[2]\ : MX2
      port map(A => \RTC_match[23]_net_1\, B => 
        \count_day_10[2]_net_1\, S => un1_rtc_init, Y => N_708);
    
    un8_inc_day_3 : XNOR2
      port map(A => \RTC_date_c[3]\, B => 
        \numdays_month[3]_net_1\, Y => un8_inc_day_3_i_0);
    
    \count_hours[6]\ : DFN1C0
      port map(D => \count_hours_15[6]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_hours_c[6]\);
    
    \ACMWDATA_iv_1[6]\ : OA1
      port map(A => N_653, B => \RTC_hours_c[0]\, C => 
        \un1_count_month_m[0]_net_1\, Y => 
        \ACMWDATA_iv_1[6]_net_1\);
    
    un22_count_month_I_12 : AND3
      port map(A => \RTC_month_c[0]\, B => \RTC_month_c[1]\, C
         => \RTC_month_c[2]\, Y => N_4_3);
    
    \count_date_13_r[0]\ : AOI1B
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_698, Y => \count_date_13[0]_net_1\);
    
    \RTC_match[30]\ : DFN1E0C0
      port map(D => ACMRDATA(6), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[23]\, Q => 
        \RTC_match[30]_net_1\);
    
    \ACMADDR_1_a3_1[3]\ : OR2A
      port map(A => \RTCcal_present_0[0]_net_1\, B => 
        \RTCcal_present_0[3]_net_1\, Y => N_416);
    
    un11_count_year_I_9 : XOR2
      port map(A => N_7_4, B => \RTC_year_c[2]\, Y => I_9);
    
    count_month_12_sn_m1_0_a2_1 : OR3
      port map(A => \RTC_month_c[1]\, B => \RTC_month_c[2]\, C
         => numdays_month25, Y => \count_month_12_sn_m1_0_a2_1\);
    
    \count_hours_15[7]\ : NOR2B
      port map(A => \RTC_hours_c[7]\, B => un1_rtc_init, Y => 
        \count_hours_15[7]_net_1\);
    
    \RTC_load[3]\ : DFN1E0C0
      port map(D => ACMRDATA(3), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[14]\, Q => 
        \RTC_load[3]_net_1\);
    
    \RTC_load[29]\ : DFN1E0C0
      port map(D => ACMRDATA(5), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[19]\, Q => 
        \RTC_load[29]_net_1\);
    
    RTCcal_present_s5_0_a4 : OR3A
      port map(A => \RTCcal_present[0]_net_1\, B => N_874, C => 
        N_894, Y => \RTCcal_present_d_i[23]\);
    
    \count_day[4]\ : DFN1C0
      port map(D => \count_day_11[4]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_day_c[4]\);
    
    \RTCcal_present[3]\ : DFN1C0
      port map(D => \RTCcal_present_ns[3]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \RTCcal_present[3]_net_1\);
    
    \count_seconds_12_r[5]\ : NOR2B
      port map(A => \count_seconds_0_sqmuxa\, B => N_810, Y => 
        \count_seconds_12[5]_net_1\);
    
    \count_date[5]\ : DFN1C0
      port map(D => \count_date_13[5]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_date_c[5]\);
    
    \RTCcal_present_ns_0_0_2[3]\ : OA1A
      port map(A => \RTCcal_present_ns_0_0_a2_0[3]_net_1\, B => 
        N_458, C => N_499, Y => 
        \RTCcal_present_ns_0_0_2[3]_net_1\);
    
    \RTC_match[38]\ : DFN1E0C0
      port map(D => ACMRDATA(6), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[24]\, Q => 
        \RTC_match[38]_net_1\);
    
    \un1_count_month_m[0]\ : OR2A
      port map(A => N_665, B => \RTC_month_c[0]\, Y => 
        \un1_count_month_m[0]_net_1\);
    
    \count_minutes[3]\ : DFN1C0
      port map(D => \count_minutes_13[3]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_minutes_c[3]\);
    
    \RTC_load[18]\ : DFN1E0C0
      port map(D => ACMRDATA(2), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[17]\, Q => 
        \RTC_load[18]_net_1\);
    
    RTC_match_45_e_1 : OA1B
      port map(A => N_882, B => \RTCcal_present_0[2]_net_1\, C
         => N_894, Y => \RTC_match_45_e_1\);
    
    un30_count_hours_I_13 : XOR2
      port map(A => N_4, B => \RTC_hours_c[3]\, Y => I_13_3);
    
    \count_date_i_m[0]\ : OR2A
      port map(A => N_665, B => \RTC_date_c[0]\, Y => 
        \count_date_i_m[0]_net_1\);
    
    \count_date_13_r[1]\ : AOI1B
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_699, Y => \count_date_13[1]_net_1\);
    
    \count_year_8[6]\ : NOR2B
      port map(A => \RTC_year_c[6]\, B => un1_rtc_init, Y => 
        \count_year_8[6]_net_1\);
    
    count_seconds_0_sqmuxa_1 : NOR3B
      port map(A => \RTC_seconds_c[6]\, B => \RTC_seconds_c[4]\, 
        C => \RTC_seconds_c[5]\, Y => \count_seconds_0_sqmuxa_1\);
    
    un23_count_seconds_I_13 : XOR2
      port map(A => N_4_1, B => \RTC_seconds_c[3]\, Y => I_13_5);
    
    \count_seconds[7]\ : DFN1C0
      port map(D => \count_seconds_13[7]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_seconds_c[7]\);
    
    \RTCcal_present_ns_i_a5_4_0[2]\ : OR3A
      port map(A => \RTC_match[39]_net_1\, B => 
        \RTCcal_present_0[1]_net_1\, C => 
        \RTCcal_present_0[0]_net_1\, Y => 
        \RTCcal_present_ns_i_a5_4_0[2]_net_1\);
    
    \RTC_match[3]\ : DFN1E0C0
      port map(D => ACMRDATA(3), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_987_i, Q => \RTC_match[3]_net_1\);
    
    \RTC_match[20]\ : DFN1E0C0
      port map(D => ACMRDATA(4), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_496_i, Q => \RTC_match[20]_net_1\);
    
    \count_month_13[2]\ : MX2
      port map(A => \RTC_match[32]_net_1\, B => 
        \count_month_12[2]\, S => un1_rtc_init, Y => N_713);
    
    \RTCcal_present[0]\ : DFN1C0
      port map(D => \RTCcal_present_ns[0]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \RTCcal_present[0]_net_1\);
    
    sec_flag_2_r : OA1A
      port map(A => \un5_acmrdata\, B => \sec_flag\, C => 
        clr_sec_flag, Y => sec_flag_2);
    
    numdays_month25_numdays_month25 : OR2B
      port map(A => \RTC_month_c[3]\, B => \RTC_month_c[0]\, Y
         => numdays_month25);
    
    \count_month_13_s[1]\ : AO1
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_712, Y => \count_month_13[1]_net_1\);
    
    \count_hours_15[3]\ : MX2
      port map(A => \RTC_match[17]_net_1\, B => 
        \count_hours_14[3]\, S => un1_rtc_init, Y => N_788);
    
    un6_inc_hour : NOR3B
      port map(A => N_586_1, B => \un6_inc_hour_1_0\, C => 
        \RTC_hours_c[1]\, Y => \un6_inc_hour\);
    
    \count_date[2]\ : DFN1C0
      port map(D => \count_date_13[2]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_date_c[2]\);
    
    I_263 : NOR2
      port map(A => \I_263_0\, B => \count_seconds_0_sqmuxa\, Y
         => N_623);
    
    \count_date_12_r[2]\ : NOR3A
      port map(A => \count_date_0_sqmuxa\, B => N_586, C => N_752, 
        Y => \count_date_12[2]\);
    
    un5_rtc_init_i_o4 : NOR2A
      port map(A => \RTCcal_present[2]_net_1\, B => N_407_i_0, Y
         => N_415);
    
    \count_month_13[1]\ : MX2
      port map(A => \RTC_match[31]_net_1\, B => 
        \count_month_12[1]\, S => un1_rtc_init, Y => N_712);
    
    \count_day[1]\ : DFN1C0
      port map(D => \count_day_11[1]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_day_c[1]\);
    
    \count_month_12_0[3]\ : MX2C
      port map(A => I_13_1, B => \RTC_month_c[3]\, S => 
        \count_date_0_sqmuxa\, Y => N_775);
    
    \RTC_match[28]\ : DFN1E0C0
      port map(D => ACMRDATA(4), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[23]\, Q => 
        \RTC_match[28]_net_1\);
    
    \RTCcal_present_ns_0_0_a2_0[3]\ : OR2A
      port map(A => N_882, B => N_484_i_0, Y => N_896);
    
    \count_minutes_13[0]\ : MX2
      port map(A => \RTC_match[7]_net_1\, B => N_723, S => 
        un1_rtc_init, Y => N_764);
    
    \count_month_12_0[2]\ : MX2C
      port map(A => I_9_0, B => \RTC_month_c[2]\, S => 
        \count_date_0_sqmuxa\, Y => N_774);
    
    \RTCcal_present_ns_i_a5_3_1[1]\ : NOR3C
      port map(A => \RTCcal_present_0[4]_net_1\, B => 
        \RTCcal_present_0[1]_net_1\, C => rtc_eq0, Y => 
        \RTCcal_present_ns_i_a5_3_1[1]_net_1\);
    
    \count_minutes_12_r[6]\ : NOR2B
      port map(A => \count_minutes_0_sqmuxa\, B => N_729, Y => 
        \count_minutes_12[6]_net_1\);
    
    un24_inc_hour : NOR3C
      port map(A => N_586_1, B => \un6_inc_hour_1_0\, C => 
        \RTC_hours_c[1]\, Y => \un24_inc_hour\);
    
    un24_count_minutes_I_8 : NOR2B
      port map(A => \RTC_minutes_c[1]\, B => \RTC_minutes_c[0]\, 
        Y => N_7_2);
    
    \RTCcal_present_ns_0_a5_0[0]\ : OR2A
      port map(A => N_485_i, B => \RTCcal_present_0[2]_net_1\, Y
         => N_493);
    
    un23_count_seconds_I_8 : NOR2B
      port map(A => \RTC_seconds_c[1]\, B => \RTC_seconds_c[0]\, 
        Y => N_7_3);
    
    un8_inc_day_0 : XNOR2
      port map(A => \RTC_date_c[0]\, B => 
        \numdays_month[0]_net_1\, Y => un8_inc_day_0_i_0);
    
    \count_year[3]\ : DFN1C0
      port map(D => \count_year_8[3]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_year_c[3]\);
    
    un28_i_a4 : OR2
      port map(A => N_403, B => N_400_i_0, Y => N_453);
    
    \RTC_match[2]\ : DFN1E0C0
      port map(D => ACMRDATA(2), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_987_i, Q => \RTC_match[2]_net_1\);
    
    un25_count_date_I_12 : AND3
      port map(A => \RTC_date_c[0]\, B => \RTC_date_c[1]\, C => 
        \RTC_date_c[2]\, Y => N_4_4);
    
    I_283 : OR2B
      port map(A => \RTCcal_present[0]_net_1\, B => N_499_2, Y
         => N_653);
    
    \un30_count_hours_m[1]\ : OR2B
      port map(A => I_5_2, B => \count_hours_5_sqmuxa\, Y => 
        \un30_count_hours_m[1]_net_1\);
    
    \count_hours[7]\ : DFN1C0
      port map(D => \count_hours_15[7]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_hours_c[7]\);
    
    \count_minutes[7]\ : DFN1C0
      port map(D => \count_minutes_13[7]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_minutes_c[7]\);
    
    \count_day_10[2]\ : OA1A
      port map(A => \un3_inc_day\, B => \count_hours_3_sqmuxa\, C
         => I_14, Y => \count_day_10[2]_net_1\);
    
    \ACMADDR_1[3]\ : OR3C
      port map(A => \ACMADDR_1_0[3]_net_1\, B => N_438, C => 
        \N_410\, Y => ACMADDR_3);
    
    numdays_month16_i : INV
      port map(A => \numdays_month16\, Y => \numdays_month16_i\);
    
    RTCcal_present_s11_0_a4 : OR3
      port map(A => N_874, B => \RTCcal_present[0]_net_1\, C => 
        N_458, Y => \RTCcal_present_d_i[17]\);
    
    \ACMWDATA_iv_2[0]\ : NOR3C
      port map(A => \count_seconds_i_m[0]_net_1\, B => \N_602_m\, 
        C => \N_563_m\, Y => \ACMWDATA_iv_2[0]_net_1\);
    
    \RTCcal_present_ns_0_0_3[3]\ : AOI1B
      port map(A => N_902_2, B => 
        \RTCcal_present_ns_0_0_a2_2_0[3]_net_1\, C => N_897, Y
         => \RTCcal_present_ns_0_0_3[3]_net_1\);
    
    \count_ld_i_0\ : NOR3A
      port map(A => \count_ld_i_0_a2\, B => N_889, C => N_878, Y
         => count_ld_i_0);
    
    \count_hours_15_s[4]\ : AO1
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_789, Y => \count_hours_15[4]_net_1\);
    
    \RTCcal_present_ns_i_a5_1[2]\ : OR3B
      port map(A => \RTCcal_present[3]_net_1\, B => 
        \RTCcal_present_0[4]_net_1\, C => 
        \RTCcal_present[2]_net_1\, Y => N_504);
    
    \count_seconds[2]\ : DFN1C0
      port map(D => \count_seconds_13[2]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_seconds_c[2]\);
    
    \count_seconds_12[5]\ : AX1C
      port map(A => \RTC_seconds_c[4]\, B => N_690, C => 
        \RTC_seconds_c[5]\, Y => N_810);
    
    \count_seconds_13_r[4]\ : AOI1B
      port map(A => N_509_1, B => \RTCcal_present_s21_0_a4_0\, C
         => N_736, Y => \count_seconds_13[4]_net_1\);
    
    \RTCcal_present_ns_0_0[4]\ : OR3C
      port map(A => N_901, B => 
        \RTCcal_present_ns_0_0_0[4]_net_1\, C => N_902, Y => 
        \RTCcal_present_ns[4]\);
    
    \count_hours_14_1_iv[4]\ : OA1
      port map(A => N_744, B => \RTC_hours_c[4]\, C => 
        \count_hours_4_sqmuxa\, Y => 
        \count_hours_14_1_iv[4]_net_1\);
    
    \count_year_8_s[2]\ : AO1
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_796, Y => \count_year_8[2]_net_1\);
    
    \count_date_12[5]\ : AX1C
      port map(A => \RTC_date_c[4]\, B => N_586, C => 
        \RTC_date_c[5]\, Y => N_761);
    
    \ACMWDATA_iv_1[5]\ : NOR3C
      port map(A => \count_year_m[2]_net_1\, B => 
        \count_seconds_m[5]_net_1\, C => \count_date_m[5]_net_1\, 
        Y => \ACMWDATA_iv_1[5]_net_1\);
    
    count_minutes_0_sqmuxa_2 : AND2
      port map(A => \count_minutes_0_sqmuxa_1\, B => N_623_1_i, Y
         => \count_minutes_0_sqmuxa_2\);
    
    \ACMADDR_1_0_a2_0[1]\ : NOR2A
      port map(A => \RTCcal_present_0[0]_net_1\, B => 
        \RTCcal_present_0[1]_net_1\, Y => 
        \ACMADDR_1_0_a2_0[1]_net_1\);
    
    \count_hours_14_0_iv_0[0]\ : MX2C
      port map(A => \un6_inc_hour\, B => \RTC_hours_c[0]\, S => 
        \count_minutes_0_sqmuxa\, Y => 
        \count_hours_14_0_iv_0[0]_net_1\);
    
    un3_inc_hour_2 : NOR3B
      port map(A => \RTC_hours_c[3]\, B => \RTC_hours_c[0]\, C
         => \RTC_hours_c[4]\, Y => \un3_inc_hour_2\);
    
    count_ld_i_0_a2 : OR2A
      port map(A => \RTCcal_present[1]_net_1\, B => 
        \RTCcal_present[2]_net_1\, Y => \count_ld_i_0_a2\);
    
    \count_hours_15[0]\ : MX2
      port map(A => \RTC_match[14]_net_1\, B => 
        \count_hours_14[0]\, S => un1_rtc_init, Y => N_785);
    
    \RTCcal_present_ns_0_a5_0_0[0]\ : OA1A
      port map(A => \RTC_match[39]_net_1\, B => 
        \RTCcal_present_0[1]_net_1\, C => 
        \RTCcal_present_0[3]_net_1\, Y => 
        \RTCcal_present_ns_0_a5_0[0]_net_1\);
    
    RTCcal_present_s6_0_a4 : NOR3A
      port map(A => \RTCcal_present[0]_net_1\, B => N_874, C => 
        N_408, Y => \RTCcal_present_d[22]\);
    
    \RTCcal_present_ns_i_a5[1]\ : XO1
      port map(A => \RTCcal_present_0[0]_net_1\, B => 
        \RTCcal_present_0[1]_net_1\, C => 
        \RTCcal_present[2]_net_1\, Y => N_497);
    
    count_date_0_sqmuxa_4 : NOR3C
      port map(A => un8_inc_day_1_i_0, B => 
        \count_date_0_sqmuxa_0\, C => \count_date_0_sqmuxa_3\, Y
         => \count_date_0_sqmuxa_4\);
    
    \count_day_11[0]\ : MX2
      port map(A => \RTC_match[21]_net_1\, B => 
        \DWACT_ADD_CI_0_partial_sum[0]\, S => un1_rtc_init, Y => 
        N_706);
    
    \count_date_13[7]\ : NOR2B
      port map(A => \RTC_date_c[7]\, B => un1_rtc_init, Y => 
        \count_date_13[7]_net_1\);
    
    numdays_month26 : OR3A
      port map(A => \numdays_month26_1\, B => \RTC_month_c[0]\, C
         => \RTC_month_c[1]\, Y => \numdays_month26\);
    
    \RTC_load[11]\ : DFN1E0C0
      port map(D => ACMRDATA(3), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[18]\, Q => 
        \RTC_load[11]_net_1\);
    
    \count_month_13_r[3]\ : AOI1B
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_714, Y => \count_month_13[3]_net_1\);
    
    \RTCcal_present_ns_i_0[2]\ : OA1
      port map(A => N_485_i, B => \RTCcal_present_0[2]_net_1\, C
         => N_504, Y => \RTCcal_present_ns_i_0[2]_net_1\);
    
    \count_day[5]\ : DFN1C0
      port map(D => \count_day_11[5]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_day_c[5]\);
    
    un25_count_date_I_8 : NOR2B
      port map(A => \RTC_date_c[1]\, B => \RTC_date_c[0]\, Y => 
        N_7_6);
    
    \count_minutes[5]\ : DFN1C0
      port map(D => \count_minutes_13[5]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_minutes_c[5]\);
    
    \count_hours_15[1]\ : MX2
      port map(A => \RTC_match[15]_net_1\, B => 
        \count_hours_14[1]\, S => un1_rtc_init, Y => N_786);
    
    I_278 : NOR2
      port map(A => \RTCcal_present[2]_net_1\, B => N_894, Y => 
        N_646);
    
    \count_day[7]\ : DFN1C0
      port map(D => \count_day_11[7]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_day_c[7]\);
    
    un3_inc_day : NOR3C
      port map(A => \RTC_day_c[1]\, B => \RTC_day_c[2]\, C => 
        \RTC_day_c[0]\, Y => \un3_inc_day\);
    
    \RTC_load[0]\ : DFN1E0C0
      port map(D => ACMRDATA(0), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[14]\, Q => 
        \RTC_load[0]_net_1\);
    
    \RTCcal_present_ns_0_0_a2_2_1[4]\ : NOR2A
      port map(A => N_904, B => \RTCcal_present[0]_net_1\, Y => 
        N_902_1);
    
    \count_date_13[2]\ : MX2
      port map(A => \RTC_match[26]_net_1\, B => 
        \count_date_12[2]\, S => un1_rtc_init, Y => N_700);
    
    \count_month[1]\ : DFN1C0
      port map(D => \count_month_13[1]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_month_c[1]\);
    
    \count_date_m[4]\ : OR2B
      port map(A => \RTC_date_c[4]\, B => N_665, Y => 
        \count_date_m[4]_net_1\);
    
    \count_date_13_s[3]\ : AO1
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_701, Y => \count_date_13[3]_net_1\);
    
    \RTCcal_present_ns_i_0[1]\ : OA1A
      port map(A => \RTCcal_present_ns_i_a5_3_1[1]_net_1\, B => 
        N_484_i_0, C => N_497, Y => 
        \RTCcal_present_ns_i_0[1]_net_1\);
    
    count_month_0_sqmuxa : NOR2A
      port map(A => \count_month_0_sqmuxa_2\, B => 
        \count_date_0_sqmuxa\, Y => \count_month_0_sqmuxa\);
    
    un24_count_minutes_I_9 : XOR2
      port map(A => N_7_2, B => \RTC_minutes_c[2]\, Y => I_9_3);
    
    un30_count_hours_I_5 : XOR2
      port map(A => \RTC_hours_c[0]\, B => \RTC_hours_c[1]\, Y
         => I_5_2);
    
    \RTCcal_present_ns_0_0[0]\ : AOI1B
      port map(A => N_499_1, B => 
        \RTCcal_present_ns_0_a5_0[0]_net_1\, C => N_493, Y => 
        \RTCcal_present_ns_0_0[0]_net_1\);
    
    \count_day_11[6]\ : NOR2B
      port map(A => \RTC_day_c[6]\, B => un1_rtc_init, Y => 
        \count_day_11[6]_net_1\);
    
    \count_date_12_r[4]\ : XA1
      port map(A => N_586, B => \RTC_date_c[4]\, C => 
        \count_date_0_sqmuxa\, Y => \count_date_12[4]\);
    
    RTC_act_i_1 : OR2B
      port map(A => \RTCcal_present[4]_net_1\, B => N_403, Y => 
        N_382_1);
    
    count_hours_2_sqmuxa : OR2A
      port map(A => \un12_inc_hour\, B => 
        \count_minutes_0_sqmuxa\, Y => \count_hours_2_sqmuxa\);
    
    \RTCcal_present_0[4]\ : DFN1C0
      port map(D => \RTCcal_present_ns[4]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \RTCcal_present_0[4]_net_1\);
    
    un17_count_minutes_1_CO2_0_o2 : OR2B
      port map(A => \RTC_minutes_c[5]\, B => \RTC_minutes_c[4]\, 
        Y => N_7);
    
    \count_hours_14_0_iv[0]\ : OR3C
      port map(A => \count_hours_14_0_iv_0[0]_net_1\, B => 
        \count_hours_4_sqmuxa\, C => 
        \un30_count_hours_m[0]_net_1\, Y => \count_hours_14[0]\);
    
    \count_day[3]\ : DFN1C0
      port map(D => \count_day_11[3]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_day_c[3]\);
    
    \count_year_i_m[3]\ : OR2A
      port map(A => N_672, B => \RTC_year_c[3]\, Y => 
        \count_year_i_m[3]_net_1\);
    
    \ACMWDATA_iv_2[5]\ : OA1A
      port map(A => \RTC_day_c[0]\, B => N_496_1, C => 
        \count_minutes_m[6]_net_1\, Y => \ACMWDATA_iv_2[5]_net_1\);
    
    \ACMWDATA_iv_2[4]\ : OA1A
      port map(A => \RTC_hours_c[6]\, B => N_496_1, C => 
        \count_minutes_m[5]_net_1\, Y => \ACMWDATA_iv_2[4]_net_1\);
    
    I_315 : NOR2
      port map(A => \un3_inc_sec_2\, B => N_690_3, Y => N_690);
    
    \RTC_match[13]\ : DFN1E1C0
      port map(D => ACMRDATA(5), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[22]\, Q => 
        \RTC_match[13]_net_1\);
    
    \count_minutes_12_0[3]\ : MX2C
      port map(A => I_13_4, B => \RTC_minutes_c[3]\, S => 
        \count_seconds_0_sqmuxa\, Y => N_720);
    
    \ACMWDATA_iv_1[1]\ : OA1A
      port map(A => N_672, B => \RTC_month_c[3]\, C => \N_598_m\, 
        Y => \ACMWDATA_iv_1[1]_net_1\);
    
    \count_day_11[5]\ : NOR2B
      port map(A => \RTC_day_c[5]\, B => un1_rtc_init, Y => 
        \count_day_11[5]_net_1\);
    
    \count_seconds_13[5]\ : MX2
      port map(A => \RTC_match[5]_net_1\, B => 
        \count_seconds_12[5]_net_1\, S => un1_rtc_init, Y => 
        N_737);
    
    \count_year[6]\ : DFN1C0
      port map(D => \count_year_8[6]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_year_c[6]\);
    
    \count_seconds_i_m[3]\ : OR2A
      port map(A => N_646, B => \RTC_seconds_c[3]\, Y => 
        \count_seconds_i_m[3]_net_1\);
    
    \count_minutes[1]\ : DFN1C0
      port map(D => \count_minutes_13[1]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_minutes_c[1]\);
    
    \count_date_13[4]\ : MX2
      port map(A => \RTC_match[28]_net_1\, B => 
        \count_date_12[4]\, S => un1_rtc_init, Y => N_702);
    
    \ACMADDR_1_0_1[6]\ : NOR3B
      port map(A => N_458, B => N_416, C => 
        \RTCcal_present_0[4]_net_1\, Y => 
        \ACMADDR_1_0_1[6]_net_1\);
    
    \count_minutes_12[0]\ : NOR2
      port map(A => N_717, B => N_623, Y => N_723);
    
    \count_minutes_13[2]\ : MX2
      port map(A => \RTC_match[9]_net_1\, B => N_725, S => 
        un1_rtc_init, Y => N_766);
    
    \count_hours_14_0_iv[1]\ : OR3C
      port map(A => \count_hours_14_0_iv_0[1]_net_1\, B => 
        \count_hours_3_sqmuxa\, C => 
        \un30_count_hours_m[1]_net_1\, Y => \count_hours_14[1]\);
    
    \count_month[0]\ : DFN1C0
      port map(D => \count_month_13[0]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_month_c[0]\);
    
    \count_seconds_13_r[5]\ : AOI1B
      port map(A => N_509_1, B => \RTCcal_present_s21_0_a4_0\, C
         => N_737, Y => \count_seconds_13[5]_net_1\);
    
    un22_count_month_I_8 : NOR2B
      port map(A => \RTC_month_c[1]\, B => \RTC_month_c[0]\, Y
         => N_7_5);
    
    \count_seconds_12[1]\ : NOR2
      port map(A => N_800, B => N_690, Y => 
        \count_seconds_12[1]_net_1\);
    
    un25_count_date_I_5 : XOR2
      port map(A => \RTC_date_c[0]\, B => \RTC_date_c[1]\, Y => 
        I_5_1);
    
    \count_minutes_m[5]\ : OR2A
      port map(A => \RTC_minutes_c[5]\, B => N_653, Y => 
        \count_minutes_m[5]_net_1\);
    
    numdays_month16 : OR2A
      port map(A => \numdays_month17_1\, B => \RTC_month_c[0]\, Y
         => \numdays_month16\);
    
    \ACMADDR_1_0_1[0]\ : NOR3C
      port map(A => N_884, B => N_883, C => N_886, Y => 
        \ACMADDR_1_0_1[0]_net_1\);
    
    \count_year_8[2]\ : MX2
      port map(A => \RTC_match[37]_net_1\, B => 
        \count_year_7[2]_net_1\, S => un1_rtc_init, Y => N_796);
    
    un3_inc_sec_2 : OR3A
      port map(A => \RTC_seconds_c[0]\, B => \RTC_seconds_c[1]\, 
        C => \un3_inc_sec_2_0\, Y => \un3_inc_sec_2\);
    
    \RTCcal_present_ns_i_2[2]\ : NOR2B
      port map(A => N_505, B => N_506, Y => 
        \RTCcal_present_ns_i_2[2]_net_1\);
    
    \count_hours_15_r[2]\ : AOI1B
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_787, Y => \count_hours_15[2]_net_1\);
    
    \RTC_match[33]\ : DFN1E0C0
      port map(D => ACMRDATA(1), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[24]\, Q => 
        \RTC_match[33]_net_1\);
    
    \ACMWDATA_iv_2[1]\ : NOR3C
      port map(A => \N_692_m\, B => \N_601_m\, C => 
        \ACMWDATA_iv_1[1]_net_1\, Y => \ACMWDATA_iv_2[1]_net_1\);
    
    count_hours_4_sqmuxa : OR2A
      port map(A => \un24_inc_hour\, B => 
        \count_minutes_0_sqmuxa\, Y => \count_hours_4_sqmuxa\);
    
    \count_seconds_12_0[3]\ : MX2C
      port map(A => I_13_5, B => \RTC_seconds_c[3]\, S => N_690_3, 
        Y => N_802);
    
    \ACMWDATA_r[7]\ : AOI1B
      port map(A => \count_day_m[2]_net_1\, B => 
        \ACMWDATA_iv_2[7]_net_1\, C => N_398, Y => ACMWDATA(7));
    
    \count_minutes_13[1]\ : MX2
      port map(A => \RTC_match[8]_net_1\, B => N_724, S => 
        un1_rtc_init, Y => N_765);
    
    \count_year_8[3]\ : MX2
      port map(A => \RTC_match[38]_net_1\, B => 
        \count_year_7[3]_net_1\, S => un1_rtc_init, Y => N_797);
    
    \RTCcal_present_ns_0_0_a2_2[4]\ : OR3B
      port map(A => N_902_2, B => N_902_1, C => 
        \RTCcal_present_0[3]_net_1\, Y => N_902);
    
    \ACMADDR_1_0[3]\ : OA1A
      port map(A => N_874, B => N_894, C => N_416, Y => 
        \ACMADDR_1_0[3]_net_1\);
    
    numdays_month_79 : NOR2B
      port map(A => \un1_rtc_init_1\, B => 
        \numdays_month[1]_net_1\, Y => \numdays_month_79\);
    
    \count_seconds_m[4]\ : OR2B
      port map(A => \RTC_seconds_c[4]\, B => N_646, Y => 
        \count_seconds_m[4]_net_1\);
    
    un30_count_hours_I_9 : XOR2
      port map(A => N_7_1, B => \RTC_hours_c[2]\, Y => I_9_2);
    
    \RTC_load[7]\ : DFN1E0C0
      port map(D => ACMRDATA(7), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[14]\, Q => 
        \RTC_load[7]_net_1\);
    
    \numdays_month[4]\ : DFN1E0P0
      port map(D => \numdays_month16\, CLK => ACM_CLK_c, PRE => 
        sys_reset_gl, E => \un1_rtc_init_1\, Q => 
        \numdays_month[4]_net_1\);
    
    \count_day_11[1]\ : MX2
      port map(A => \RTC_match[22]_net_1\, B => 
        \count_day_10[1]_net_1\, S => un1_rtc_init, Y => N_707);
    
    \RTCcal_present[2]\ : DFN1C0
      port map(D => N_481_i_0, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \RTCcal_present[2]_net_1\);
    
    \numdays_month[3]\ : DFN1E0C0
      port map(D => \numdays_month16_i\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \un1_rtc_init_1\, Q => 
        \numdays_month[3]_net_1\);
    
    count_month_0_sqmuxa_2 : NOR2B
      port map(A => \numdays_month17_1\, B => 
        \count_month_0_sqmuxa_1\, Y => \count_month_0_sqmuxa_2\);
    
    count_minutes_0_sqmuxa_0 : NOR2A
      port map(A => \RTC_minutes_c[6]\, B => \RTC_minutes_c[5]\, 
        Y => \count_minutes_0_sqmuxa_0\);
    
    \RTCcal_present_ns_0_0_1[3]\ : NOR3C
      port map(A => N_896, B => N_878, C => N_887, Y => 
        \RTCcal_present_ns_0_0_1[3]_net_1\);
    
    \count_date_12_r[1]\ : NOR3A
      port map(A => \count_date_0_sqmuxa\, B => N_586, C => N_751, 
        Y => \count_date_12[1]\);
    
    \RTC_match[23]\ : DFN1E0C0
      port map(D => ACMRDATA(7), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_496_i, Q => \RTC_match[23]_net_1\);
    
    N_601_m : OR2
      port map(A => \RTC_hours_c[3]\, B => N_496_1, Y => 
        \N_601_m\);
    
    \count_month_12_s[0]\ : AO1D
      port map(A => N_772, B => N_776, C => 
        \count_month_0_sqmuxa\, Y => \count_month_12[0]\);
    
    \count_date_12_0[2]\ : MX2C
      port map(A => I_9_1, B => \RTC_date_c[2]\, S => 
        \count_hours_3_sqmuxa\, Y => N_752);
    
    \RTCcal_present_ns_0_0_a2_1[3]\ : OR3C
      port map(A => \RTCcal_present_0[2]_net_1\, B => \rr_pwrdwn\, 
        C => N_903, Y => N_897);
    
    \count_year_m[2]\ : OR2B
      port map(A => \RTC_year_c[2]\, B => N_672, Y => 
        \count_year_m[2]_net_1\);
    
    \count_seconds_13[3]\ : MX2
      port map(A => \RTC_match[3]_net_1\, B => 
        \count_seconds_12[3]_net_1\, S => un1_rtc_init, Y => 
        N_735);
    
    \count_seconds[6]\ : DFN1C0
      port map(D => \count_seconds_13[6]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_seconds_c[6]\);
    
    \count_date_12_0[0]\ : XOR2
      port map(A => \count_hours_3_sqmuxa\, B => \RTC_date_c[0]\, 
        Y => N_750);
    
    \RTC_match[5]\ : DFN1E0C0
      port map(D => ACMRDATA(5), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_987_i, Q => \RTC_match[5]_net_1\);
    
    \RTCcal_present_0[0]\ : DFN1C0
      port map(D => \RTCcal_present_ns[0]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \RTCcal_present_0[0]_net_1\);
    
    \count_date[6]\ : DFN1C0
      port map(D => \count_date_13[6]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_date_c[6]\);
    
    N_598_m : OR2A
      port map(A => N_665, B => \RTC_date_c[1]\, Y => \N_598_m\);
    
    \count_month[2]\ : DFN1C0
      port map(D => \count_month_13[2]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_month_c[2]\);
    
    \count_minutes_12[5]\ : AX1C
      port map(A => \RTC_minutes_c[4]\, B => N_623, C => 
        \RTC_minutes_c[5]\, Y => N_728);
    
    un1_RTCcal_present_1_i_0_o2 : NOR2B
      port map(A => \RTCcal_present[1]_net_1\, B => 
        \RTCcal_present[0]_net_1\, Y => N_882);
    
    \RTCcal_present_ns_0_0_a2_2_0[3]\ : NOR2B
      port map(A => N_904, B => N_499_1, Y => 
        \RTCcal_present_ns_0_0_a2_2_0[3]_net_1\);
    
    \count_year[2]\ : DFN1C0
      port map(D => \count_year_8[2]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_year_c[2]\);
    
    \count_month[6]\ : DFN1C0
      port map(D => \count_month_13[6]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_month_c[6]\);
    
    reg_pwrdwn : DFN1C0
      port map(D => POWER_down_c, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \reg_pwrdwn\);
    
    \count_minutes_13_s[3]\ : AO1
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0\, 
        C => N_767, Y => \count_minutes_13[3]_net_1\);
    
    \RTCcal_present_ns_i_1[2]\ : OA1
      port map(A => N_878, B => 
        \RTCcal_present_ns_i_a5_4_0[2]_net_1\, C => 
        \RTCcal_present_ns_i_0[2]_net_1\, Y => 
        \RTCcal_present_ns_i_1[2]_net_1\);
    
    \count_hours_14_0_iv[2]\ : AO1B
      port map(A => \count_hours_5_sqmuxa\, B => I_9_2, C => 
        \count_hours_m[2]_net_1\, Y => \count_hours_14[2]\);
    
    \RTC_load[2]\ : DFN1E0C0
      port map(D => ACMRDATA(2), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[14]\, Q => 
        \RTC_load[2]_net_1\);
    
    \count_year_8_r[3]\ : AOI1B
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_797, Y => \count_year_8[3]_net_1\);
    
    un25_count_date_I_9 : XOR2
      port map(A => N_7_6, B => \RTC_date_c[2]\, Y => I_9_1);
    
    \count_year_7[1]\ : MX2
      port map(A => \RTC_year_c[1]\, B => I_5, S => 
        \count_month_0_sqmuxa\, Y => \count_year_7[1]_net_1\);
    
    RTCcal_present_s14_0_a4_0_a2 : OR3B
      port map(A => N_904, B => \RTCcal_present_0[0]_net_1\, C
         => N_878, Y => \RTCcal_present_d[14]\);
    
    numdays_month_83 : OR2A
      port map(A => \un1_rtc_init_1\, B => 
        \numdays_month[5]_net_1\, Y => \numdays_month_83\);
    
    \numdays_month[2]\ : DFN1C0
      port map(D => \numdays_month_80\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \numdays_month[2]_net_1\);
    
    \ACMWDATA_r[5]\ : AOI1B
      port map(A => \ACMWDATA_iv_1[5]_net_1\, B => 
        \ACMWDATA_iv_2[5]_net_1\, C => N_398, Y => ACMWDATA(5));
    
    \RTCcal_present_ns_i_a5_0[2]\ : OR2A
      port map(A => N_509_1, B => N_408, Y => N_503);
    
    \RTC_match[11]\ : DFN1E1C0
      port map(D => ACMRDATA(3), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[22]\, Q => 
        \RTC_match[11]_net_1\);
    
    count_minutes_0_sqmuxa_3 : NOR2A
      port map(A => \count_minutes_0_sqmuxa_2\, B => N_623_2, Y
         => \count_minutes_0_sqmuxa_3\);
    
    \RTC_load[16]\ : DFN1E0C0
      port map(D => ACMRDATA(0), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[17]\, Q => 
        \RTC_load[16]_net_1\);
    
    \RTC_load[10]\ : DFN1E0C0
      port map(D => ACMRDATA(2), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[18]\, Q => 
        \RTC_load[10]_net_1\);
    
    \ACMWDATA_iv_1[2]\ : NOR3C
      port map(A => \count_month_m[4]_net_1\, B => 
        \count_seconds_m[2]_net_1\, C => \count_date_m[2]_net_1\, 
        Y => \ACMWDATA_iv_1[2]_net_1\);
    
    \ACMWDATA_iv_2[3]\ : NOR3C
      port map(A => \count_minutes_i_m[4]_net_1\, B => 
        \count_seconds_i_m[3]_net_1\, C => 
        \ACMWDATA_iv_1[3]_net_1\, Y => \ACMWDATA_iv_2[3]_net_1\);
    
    \count_date[3]\ : DFN1C0
      port map(D => \count_date_13[3]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_date_c[3]\);
    
    \ACMWDATA_r[1]\ : NOR3C
      port map(A => \N_624_m\, B => \ACMWDATA_iv_2[1]_net_1\, C
         => N_398, Y => ACMWDATA(1));
    
    clr_sec_flag_0_a2_0_0 : NOR2A
      port map(A => \sec_flag\, B => \RTCcal_present_0[3]_net_1\, 
        Y => \clr_sec_flag_0_a2_0_0\);
    
    \RTCcal_present_ns_i_a5_3_0[2]\ : NOR3B
      port map(A => \RTCcal_present_0[2]_net_1\, B => 
        \RTCcal_present_0[4]_net_1\, C => 
        \RTCcal_present_0[0]_net_1\, Y => 
        \RTCcal_present_ns_i_a5_3_0[2]_net_1\);
    
    count_day_1_sqmuxa : NOR2
      port map(A => \un3_inc_day\, B => \count_hours_3_sqmuxa\, Y
         => \count_day_1_sqmuxa\);
    
    I_315_3 : OR2B
      port map(A => \RTCcal_present_0[4]_net_1\, B => N_688, Y
         => N_690_3);
    
    \count_date[0]\ : DFN1C0
      port map(D => \count_date_13[0]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_date_c[0]\);
    
    \ACMADDR_1_o4[3]\ : OA1C
      port map(A => \RTCcal_present[4]_net_1\, B => N_403, C => 
        N_461, Y => \N_410\);
    
    \count_month_12_r[3]\ : NOR3
      port map(A => N_775, B => N_776, C => 
        \count_month_0_sqmuxa\, Y => \count_month_12[3]\);
    
    \count_minutes_12_0[1]\ : MX2C
      port map(A => I_5_3, B => \RTC_minutes_c[1]\, S => 
        \count_seconds_0_sqmuxa\, Y => N_718);
    
    un28_i : OR3C
      port map(A => N_453, B => N_454, C => \N_417\, Y => N_398);
    
    \count_year_7[0]\ : XOR2
      port map(A => \count_month_0_sqmuxa\, B => \RTC_year_c[0]\, 
        Y => \count_year_7[0]_net_1\);
    
    \RTC_load[23]\ : DFN1E0C0
      port map(D => ACMRDATA(7), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[17]\, Q => 
        \RTC_load[23]_net_1\);
    
    \count_seconds[0]\ : DFN1C0
      port map(D => \count_seconds_13[0]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_seconds_c[0]\);
    
    I_243_1_0 : NOR2A
      port map(A => \RTC_date_c[0]\, B => \RTC_date_c[1]\, Y => 
        \I_243_1\);
    
    \count_day_11_r[0]\ : AOI1B
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_706, Y => \count_day_11[0]_net_1\);
    
    \RTCcal_present_ns_0_1[0]\ : AO1C
      port map(A => N_499_2, B => \RTCcal_present_0[3]_net_1\, C
         => N_509_1_0, Y => \RTCcal_present_ns_0_1[0]_net_1\);
    
    \ACMADDR_1[2]\ : OR3C
      port map(A => \RTCcal_present_d_i[17]\, B => N_441, C => 
        \RTCcal_present_d_i[23]\, Y => ACMADDR_2);
    
    trigger_0_a3_1 : OR2
      port map(A => rtc_eq0, B => N_458, Y => N_463_1);
    
    \ACMADDR_1_0[6]\ : XAI1A
      port map(A => N_408, B => \RTCcal_present_0[2]_net_1\, C
         => \ACMADDR_1_0_1[6]_net_1\, Y => ACMADDR_6);
    
    I_243_1 : NOR2
      port map(A => \RTC_hours_c[3]\, B => \RTC_hours_c[2]\, Y
         => N_586_1);
    
    \RTC_match[31]\ : DFN1E0C0
      port map(D => ACMRDATA(7), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[23]\, Q => 
        \RTC_match[31]_net_1\);
    
    \ACMWDATA_iv_2[7]\ : NOR3C
      port map(A => \count_month_m[1]_net_1\, B => 
        \count_hours_m[1]_net_1\, C => \ACMWDATA_iv_0[7]_net_1\, 
        Y => \ACMWDATA_iv_2[7]_net_1\);
    
    count_minutes_0_sqmuxa : OR2A
      port map(A => \count_minutes_0_sqmuxa_3\, B => 
        \count_seconds_0_sqmuxa\, Y => \count_minutes_0_sqmuxa\);
    
    \RTC_load[22]\ : DFN1E0C0
      port map(D => ACMRDATA(6), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[17]\, Q => 
        \RTC_load[22]_net_1\);
    
    \count_month_13_r[4]\ : AOI1B
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_715, Y => \count_month_13[4]_net_1\);
    
    \ACMADDR_1_0_a2[0]\ : OR3B
      port map(A => N_882, B => N_874, C => 
        \RTCcal_present_0[3]_net_1\, Y => N_883);
    
    \count_hours_15_r[3]\ : AOI1B
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_788, Y => \count_hours_15[3]_net_1\);
    
    \ACMADDR_1_0[0]\ : OR3C
      port map(A => \RTCcal_present_d_i[18]\, B => 
        \ACMADDR_1_0_1[0]_net_1\, C => N_496_i, Y => ACMADDR_0);
    
    \count_day_11_r[2]\ : AOI1B
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_708, Y => \count_day_11[2]_net_1\);
    
    \count_year_8_s[1]\ : AO1
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_795, Y => \count_year_8[1]_net_1\);
    
    rtccount : RIP_COUNT32
      port map(RTC_load(31) => \RTC_load[31]_net_1\, RTC_load(30)
         => \RTC_load[30]_net_1\, RTC_load(29) => 
        \RTC_load[29]_net_1\, RTC_load(28) => 
        \RTC_load[28]_net_1\, RTC_load(27) => 
        \RTC_load[27]_net_1\, RTC_load(26) => 
        \RTC_load[26]_net_1\, RTC_load(25) => 
        \RTC_load[25]_net_1\, RTC_load(24) => 
        \RTC_load[24]_net_1\, RTC_load(23) => 
        \RTC_load[23]_net_1\, RTC_load(22) => 
        \RTC_load[22]_net_1\, RTC_load(21) => 
        \RTC_load[21]_net_1\, RTC_load(20) => 
        \RTC_load[20]_net_1\, RTC_load(19) => 
        \RTC_load[19]_net_1\, RTC_load(18) => 
        \RTC_load[18]_net_1\, RTC_load(17) => 
        \RTC_load[17]_net_1\, RTC_load(16) => 
        \RTC_load[16]_net_1\, RTC_load(15) => 
        \RTC_load[15]_net_1\, RTC_load(14) => 
        \RTC_load[14]_net_1\, RTC_load(13) => 
        \RTC_load[13]_net_1\, RTC_load(12) => 
        \RTC_load[12]_net_1\, RTC_load(11) => 
        \RTC_load[11]_net_1\, RTC_load(10) => 
        \RTC_load[10]_net_1\, RTC_load(9) => \RTC_load[9]_net_1\, 
        RTC_load(8) => \RTC_load[8]_net_1\, RTC_load(7) => 
        \RTC_load[7]_net_1\, RTC_load(6) => \RTC_load[6]_net_1\, 
        RTC_load(5) => \RTC_load[5]_net_1\, RTC_load(4) => 
        \RTC_load[4]_net_1\, RTC_load(3) => \RTC_load[3]_net_1\, 
        RTC_load(2) => \RTC_load[2]_net_1\, RTC_load(1) => 
        \RTC_load[1]_net_1\, RTC_load(0) => \RTC_load[0]_net_1\, 
        rtc_eq0 => rtc_eq0, clr_sec_flag_1_sqmuxa => 
        clr_sec_flag_1_sqmuxa, I_296 => I_296, ACM_CLK_c => 
        ACM_CLK_c, sys_reset_gl => sys_reset_gl);
    
    \count_year[7]\ : DFN1C0
      port map(D => \count_year_8[7]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_year_c[7]\);
    
    \count_date[1]\ : DFN1C0
      port map(D => \count_date_13[1]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_date_c[1]\);
    
    I_263_2 : OR2A
      port map(A => \RTC_minutes_c[0]\, B => \RTC_minutes_c[1]\, 
        Y => N_623_2);
    
    \RTCcal_present_ns_0_0_a2[4]\ : AO1C
      port map(A => \RTCcal_present[2]_net_1\, B => 
        \RTCcal_present[3]_net_1\, C => \RTCcal_present[4]_net_1\, 
        Y => N_899);
    
    count_ld_i_0_a2_0 : NOR2
      port map(A => \RTCcal_present[1]_net_1\, B => 
        \RTCcal_present[0]_net_1\, Y => N_889);
    
    N_692_m : OR2A
      port map(A => N_646, B => \RTC_seconds_c[1]\, Y => 
        \N_692_m\);
    
    un18_inc_hour_0 : OR2B
      port map(A => \RTC_hours_c[5]\, B => \RTC_hours_c[0]\, Y
         => \un18_inc_hour_0\);
    
    \count_seconds[1]\ : DFN1C0
      port map(D => \count_seconds_13[1]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_seconds_c[1]\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    I_243_0 : NOR2A
      port map(A => \RTC_date_c[3]\, B => \RTC_date_c[2]\, Y => 
        \I_243_0\);
    
    \RTC_match[12]\ : DFN1E1C0
      port map(D => ACMRDATA(4), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[22]\, Q => 
        \RTC_match[12]_net_1\);
    
    \RTCcal_present_ns_0_a5_0_1_0_a2_1[3]\ : NOR2B
      port map(A => \RTCcal_present_0[4]_net_1\, B => 
        \RTCcal_present_0[0]_net_1\, Y => N_509_1_1);
    
    \RTCcal_present_ns_i_a5_1_2[1]\ : NOR2B
      port map(A => \RTCcal_present[2]_net_1\, B => 
        \RTCcal_present[1]_net_1\, Y => N_499_2);
    
    \ACMADDR_1_0_a3[6]\ : OR2B
      port map(A => \RTCcal_present_0[3]_net_1\, B => 
        \RTCcal_present_0[1]_net_1\, Y => N_458);
    
    \count_minutes_i_m[4]\ : OR2
      port map(A => \RTC_minutes_c[4]\, B => N_653, Y => 
        \count_minutes_i_m[4]_net_1\);
    
    \RTCcal_present_ns_0_a5_3[0]\ : OR2A
      port map(A => N_499_1, B => N_496_1, Y => N_496_i);
    
    \count_minutes_12[6]\ : AX1
      port map(A => N_7, B => N_623, C => \RTC_minutes_c[6]\, Y
         => N_729);
    
    un18_inc_hour : NOR3B
      port map(A => N_586_1, B => \un12_inc_hour_2_0\, C => 
        \un18_inc_hour_0\, Y => \un18_inc_hour\);
    
    \RTC_match[21]\ : DFN1E0C0
      port map(D => ACMRDATA(5), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_496_i, Q => \RTC_match[21]_net_1\);
    
    un25_count_date_I_13 : XOR2
      port map(A => N_4_4, B => \RTC_date_c[3]\, Y => I_13_2);
    
    \count_seconds_13[1]\ : MX2
      port map(A => \RTC_match[1]_net_1\, B => 
        \count_seconds_12[1]_net_1\, S => un1_rtc_init, Y => 
        N_733);
    
    count_hours_5_sqmuxa : NOR2A
      port map(A => \count_hours_5_sqmuxa_3\, B => 
        \count_minutes_0_sqmuxa\, Y => \count_hours_5_sqmuxa\);
    
    \count_seconds_13[0]\ : MX2
      port map(A => \RTC_match[0]_net_1\, B => 
        \count_seconds_12[0]_net_1\, S => un1_rtc_init, Y => 
        N_732);
    
    un8_inc_day_1 : XNOR2
      port map(A => \RTC_date_c[1]\, B => 
        \numdays_month[1]_net_1\, Y => un8_inc_day_1_i_0);
    
    \RTC_match[0]\ : DFN1E0C0
      port map(D => ACMRDATA(0), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_987_i, Q => \RTC_match[0]_net_1\);
    
    \RTCcal_present[1]\ : DFN1C0
      port map(D => N_479_i_0, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \RTCcal_present[1]_net_1\);
    
    \RTC_match[1]\ : DFN1E0C0
      port map(D => ACMRDATA(1), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_987_i, Q => \RTC_match[1]_net_1\);
    
    \count_minutes_13_r[2]\ : AOI1B
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_766, Y => \count_minutes_13[2]_net_1\);
    
    un11_count_year_I_5 : XOR2
      port map(A => \RTC_year_c[0]\, B => \RTC_year_c[1]\, Y => 
        I_5);
    
    \count_seconds_m[2]\ : OR2B
      port map(A => \RTC_seconds_c[2]\, B => N_646, Y => 
        \count_seconds_m[2]_net_1\);
    
    un22_count_month_I_5 : XOR2
      port map(A => \RTC_month_c[0]\, B => \RTC_month_c[1]\, Y
         => I_5_0);
    
    \RTCcal_present_ns_0_0[3]\ : OR3C
      port map(A => \RTCcal_present_ns_0_0_2[3]_net_1\, B => 
        \RTCcal_present_ns_0_0_1[3]_net_1\, C => 
        \RTCcal_present_ns_0_0_3[3]_net_1\, Y => 
        \RTCcal_present_ns[3]\);
    
    \count_month_m[4]\ : OR2B
      port map(A => \RTC_month_c[4]\, B => N_672, Y => 
        \count_month_m[4]_net_1\);
    
    \count_seconds_13[4]\ : MX2
      port map(A => \RTC_match[4]_net_1\, B => N_809, S => 
        un1_rtc_init, Y => N_736);
    
    \count_seconds_13[7]\ : NOR2B
      port map(A => \RTC_seconds_c[7]\, B => un1_rtc_init, Y => 
        \count_seconds_13[7]_net_1\);
    
    un24_count_minutes_I_12 : AND3
      port map(A => \RTC_minutes_c[0]\, B => \RTC_minutes_c[1]\, 
        C => \RTC_minutes_c[2]\, Y => N_4_0);
    
    \RTC_match[32]\ : DFN1E0C0
      port map(D => ACMRDATA(0), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[24]\, Q => 
        \RTC_match[32]_net_1\);
    
    I_297 : NOR3A
      port map(A => \RTCcal_present_0[3]_net_1\, B => 
        \RTCcal_present_0[1]_net_1\, C => N_665_1, Y => N_665);
    
    \ACMADDR_1_0_o2[0]\ : OR2A
      port map(A => \RTCcal_present[2]_net_1\, B => 
        \RTCcal_present[4]_net_1\, Y => N_874);
    
    \ACMWDATA_iv_1[4]\ : NOR3C
      port map(A => \count_year_m[1]_net_1\, B => 
        \count_seconds_m[4]_net_1\, C => \count_date_m[4]_net_1\, 
        Y => \ACMWDATA_iv_1[4]_net_1\);
    
    \count_date_13_s[5]\ : AO1
      port map(A => N_509_1_0, B => \RTCcal_present_s21_0_a4_0_0\, 
        C => N_703, Y => \count_date_13[5]_net_1\);
    
    \RTCcal_present_0[1]\ : DFN1C0
      port map(D => N_479_i_0, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \RTCcal_present_0[1]_net_1\);
    
    \count_minutes_13[7]\ : NOR2B
      port map(A => \RTC_minutes_c[7]\, B => un1_rtc_init, Y => 
        \count_minutes_13[7]_net_1\);
    
    \count_year_8[5]\ : NOR2B
      port map(A => \RTC_year_c[5]\, B => un1_rtc_init, Y => 
        \count_year_8[5]_net_1\);
    
    \RTCcal_present_ns_i_x2_0_a2[2]\ : XOR2
      port map(A => \RTCcal_present[3]_net_1\, B => 
        \RTCcal_present_0[1]_net_1\, Y => N_485_i);
    
    \RTCcal_present_ns_0_0_a2_0_0[3]\ : AOI1
      port map(A => \RTCcal_present_0[2]_net_1\, B => rtc_eq0, C
         => \RTCcal_present_0[0]_net_1\, Y => 
        \RTCcal_present_ns_0_0_a2_0[3]_net_1\);
    
    \count_hours_14_0_iv_0[1]\ : AOI1B
      port map(A => \count_minutes_0_sqmuxa\, B => 
        \RTC_hours_c[1]\, C => \count_hours_2_sqmuxa\, Y => 
        \count_hours_14_0_iv_0[1]_net_1\);
    
    \count_seconds_13[2]\ : MX2
      port map(A => \RTC_match[2]_net_1\, B => N_807, S => 
        un1_rtc_init, Y => N_734);
    
    un22_count_month_I_13 : XOR2
      port map(A => N_4_3, B => \RTC_month_c[3]\, Y => I_13_1);
    
    \count_day_11[4]\ : NOR2B
      port map(A => \RTC_day_c[4]\, B => un1_rtc_init, Y => 
        \count_day_11[4]_net_1\);
    
    \RTC_load[4]\ : DFN1E0C0
      port map(D => ACMRDATA(4), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[14]\, Q => 
        \RTC_load[4]_net_1\);
    
    \RTC_match[19]\ : DFN1E0C0
      port map(D => ACMRDATA(3), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_496_i, Q => \RTC_match[19]_net_1\);
    
    \RTC_load[17]\ : DFN1E0C0
      port map(D => ACMRDATA(1), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[17]\, Q => 
        \RTC_load[17]_net_1\);
    
    \count_seconds_12[2]\ : NOR2
      port map(A => N_801, B => N_690, Y => N_807);
    
    \numdays_month[5]\ : DFN1P0
      port map(D => \numdays_month_83\, CLK => ACM_CLK_c, PRE => 
        sys_reset_gl, Q => \numdays_month[5]_net_1\);
    
    \count_hours[4]\ : DFN1C0
      port map(D => \count_hours_15[4]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_hours_c[4]\);
    
    \RTCcal_present_ns_0_a5_0_1_0_a2[3]\ : NOR2B
      port map(A => \RTCcal_present[4]_net_1\, B => 
        \RTCcal_present[0]_net_1\, Y => N_509_1);
    
    \count_year[1]\ : DFN1C0
      port map(D => \count_year_8[1]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_year_c[1]\);
    
    \RTC_load[14]\ : DFN1E0C0
      port map(D => ACMRDATA(6), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[18]\, Q => 
        \RTC_load[14]_net_1\);
    
    \count_minutes_13[4]\ : MX2
      port map(A => \RTC_match[11]_net_1\, B => N_727, S => 
        un1_rtc_init, Y => N_768);
    
    \ACMADDR_1_0_a3[0]\ : NOR2
      port map(A => \RTCcal_present[2]_net_1\, B => 
        \RTCcal_present[1]_net_1\, Y => N_904);
    
    \RTC_load[15]\ : DFN1E0C0
      port map(D => ACMRDATA(7), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[18]\, Q => 
        \RTC_load[15]_net_1\);
    
    \numdays_month[1]\ : DFN1C0
      port map(D => \numdays_month_79\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \numdays_month[1]_net_1\);
    
    \count_month_13[4]\ : MX2
      port map(A => \RTC_match[34]_net_1\, B => 
        \count_month_12[4]\, S => un1_rtc_init, Y => N_715);
    
    \RTC_match[22]\ : DFN1E0C0
      port map(D => ACMRDATA(6), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_496_i, Q => \RTC_match[22]_net_1\);
    
    un11_count_year_I_8 : NOR2B
      port map(A => \RTC_year_c[1]\, B => \RTC_year_c[0]\, Y => 
        N_7_4);
    
    \count_year_8[7]\ : NOR2B
      port map(A => \RTC_year_c[7]\, B => un1_rtc_init, Y => 
        \count_year_8[7]_net_1\);
    
    numdays_month17_1 : NOR2
      port map(A => \RTC_month_c[3]\, B => \RTC_month_c[2]\, Y
         => \numdays_month17_1\);
    
    \RTC_match[16]\ : DFN1E0C0
      port map(D => ACMRDATA(0), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_496_i, Q => \RTC_match[16]_net_1\);
    
    un30_count_hours_I_12 : AND3
      port map(A => \RTC_hours_c[0]\, B => \RTC_hours_c[1]\, C
         => \RTC_hours_c[2]\, Y => N_4);
    
    \ACMADDR_1_0_a2_0[0]\ : OR2B
      port map(A => N_904, B => N_509_1, Y => N_884);
    
    \count_hours_m[2]\ : OR2B
      port map(A => \RTC_hours_c[2]\, B => 
        \count_minutes_0_sqmuxa\, Y => \count_hours_m[2]_net_1\);
    
    \count_hours_14_0_iv[3]\ : AO1B
      port map(A => \count_hours_5_sqmuxa\, B => I_13_3, C => 
        \count_hours_m[3]_net_1\, Y => \count_hours_14[3]\);
    
    \RTC_match[9]\ : DFN1E1C0
      port map(D => ACMRDATA(1), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[22]\, Q => 
        \RTC_match[9]_net_1\);
    
    \RTC_match[6]\ : DFN1E0C0
      port map(D => ACMRDATA(6), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_987_i, Q => \RTC_match[6]_net_1\);
    
    un12_inc_hour_2_0 : NOR2A
      port map(A => \RTC_hours_c[4]\, B => \RTC_hours_c[1]\, Y
         => \un12_inc_hour_2_0\);
    
    \count_hours_15[4]\ : MX2
      port map(A => \RTC_match[18]_net_1\, B => 
        \count_hours_14_1_iv[4]_net_1\, S => un1_rtc_init, Y => 
        N_789);
    
    \ACMADDR_1_0_a2[1]\ : OR3C
      port map(A => N_484_i_0, B => \ACMADDR_1_0_a2_0[1]_net_1\, 
        C => N_905, Y => N_887);
    
    \count_seconds[5]\ : DFN1C0
      port map(D => \count_seconds_13[5]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_seconds_c[5]\);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    \RTC_load[28]\ : DFN1E0C0
      port map(D => ACMRDATA(4), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[19]\, Q => 
        \RTC_load[28]_net_1\);
    
    \RTC_match[39]\ : DFN1E0C0
      port map(D => ACMRDATA(7), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[24]\, Q => 
        \RTC_match[39]_net_1\);
    
    \count_hours[1]\ : DFN1C0
      port map(D => \count_hours_15[1]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_hours_c[1]\);
    
    un8_inc_day_2 : XNOR2
      port map(A => \RTC_date_c[2]\, B => 
        \numdays_month[2]_net_1\, Y => un8_inc_day_2_i_0);
    
    \RTCcal_present_ns_0_0_0[4]\ : OA1
      port map(A => N_484_i_0, B => \RTCcal_present_0[1]_net_1\, 
        C => N_899, Y => \RTCcal_present_ns_0_0_0[4]_net_1\);
    
    numdays_month15_1 : NOR3A
      port map(A => \RTC_month_c[0]\, B => \RTC_month_c[4]\, C
         => \RTC_month_c[1]\, Y => \numdays_month15_1\);
    
    \ACMWDATA_r[2]\ : AOI1B
      port map(A => \ACMWDATA_iv_1[2]_net_1\, B => 
        \ACMWDATA_iv_2[2]_net_1\, C => N_398, Y => ACMWDATA(2));
    
    \count_hours_14_0_f0[5]\ : OA1A
      port map(A => \count_hours_2_sqmuxa\, B => \RTC_hours_c[5]\, 
        C => \count_hours_3_sqmuxa\, Y => \count_hours_14[5]\);
    
    \RTCcal_present_0[3]\ : DFN1C0
      port map(D => \RTCcal_present_ns[3]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \RTCcal_present_0[3]_net_1\);
    
    \count_month_12_r[4]\ : OA1B
      port map(A => N_776, B => \RTC_month_c[4]\, C => 
        \count_month_0_sqmuxa\, Y => \count_month_12[4]\);
    
    \count_day_m[2]\ : OR2A
      port map(A => \RTC_day_c[2]\, B => N_496_1, Y => 
        \count_day_m[2]_net_1\);
    
    \count_month[4]\ : DFN1C0
      port map(D => \count_month_13[4]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_month_c[4]\);
    
    \ACMADDR_1_0_a2_2[0]\ : OR2A
      port map(A => N_902_1, B => N_878, Y => N_886);
    
    I_303 : NOR2A
      port map(A => \RTCcal_present[1]_net_1\, B => N_665_1, Y
         => N_672);
    
    \count_year[4]\ : DFN1C0
      port map(D => \count_year_8[4]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_year_c[4]\);
    
    \RTC_match[36]\ : DFN1E0C0
      port map(D => ACMRDATA(4), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d[24]\, Q => 
        \RTC_match[36]_net_1\);
    
    \count_seconds_12_0[1]\ : MX2C
      port map(A => I_5_4, B => \RTC_seconds_c[1]\, S => N_690_3, 
        Y => N_800);
    
    \ACMADDR_1[4]\ : OR3C
      port map(A => N_506, B => N_443, C => N_444, Y => ACMADDR_4);
    
    numdays_month26_1 : NOR2A
      port map(A => \RTC_month_c[3]\, B => \RTC_month_c[4]\, Y
         => \numdays_month26_1\);
    
    \count_month_13[3]\ : MX2
      port map(A => \RTC_match[33]_net_1\, B => 
        \count_month_12[3]\, S => un1_rtc_init, Y => N_714);
    
    \count_minutes_12[1]\ : NOR2
      port map(A => N_718, B => N_623, Y => N_724);
    
    \ACMWDATA_iv_2[6]\ : NOR3C
      port map(A => \count_seconds_i_m[6]_net_1\, B => 
        \count_day_i_m[1]_net_1\, C => \count_year_i_m[3]_net_1\, 
        Y => \ACMWDATA_iv_2[6]_net_1\);
    
    RTC_match_45_e : OR3A
      port map(A => \RTC_match_45_e_1\, B => N_665_1, C => N_905, 
        Y => N_987_i);
    
    \RTC_match[4]\ : DFN1E0C0
      port map(D => ACMRDATA(4), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => N_987_i, Q => \RTC_match[4]_net_1\);
    
    \count_date_13[0]\ : MX2
      port map(A => \RTC_match[24]_net_1\, B => 
        \count_date_12[0]\, S => un1_rtc_init, Y => N_698);
    
    \count_seconds_13_s[2]\ : AO1
      port map(A => N_509_1, B => \RTCcal_present_s21_0_a4_0\, C
         => N_734, Y => \count_seconds_13[2]_net_1\);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    \ACMWDATA_iv_2[2]\ : OA1A
      port map(A => \RTC_hours_c[4]\, B => N_496_1, C => 
        \count_minutes_m[3]_net_1\, Y => \ACMWDATA_iv_2[2]_net_1\);
    
    \RTC_match[29]\ : DFN1E0C0
      port map(D => ACMRDATA(5), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[23]\, Q => 
        \RTC_match[29]_net_1\);
    
    \RTCcal_present_ns_i_a5_1[1]\ : OR2B
      port map(A => N_499_2, B => N_499_1, Y => N_499);
    
    \numdays_month_7_i_0[0]\ : AXO7
      port map(A => \RTC_month_c[2]\, B => \RTC_month_c[3]\, C
         => \RTC_month_c[0]\, Y => \numdays_month_7_i_0[0]_net_1\);
    
    un28_i_a4_0 : OR2A
      port map(A => \RTCcal_present[4]_net_1\, B => N_665_1, Y
         => N_454);
    
    reg_INIT_done : DFN1C0
      port map(D => INIT_DONE_c, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \reg_INIT_done\);
    
    RTC_match_45_e_0 : OR2
      port map(A => \RTCcal_present_0[4]_net_1\, B => 
        \RTCcal_present_0[3]_net_1\, Y => N_905);
    
    un11_count_year_I_13 : XOR2
      port map(A => N_4_2, B => \RTC_year_c[3]\, Y => I_13_0);
    
    \count_seconds_12[0]\ : NOR2
      port map(A => N_799, B => N_690, Y => 
        \count_seconds_12[0]_net_1\);
    
    \RTC_load[19]\ : DFN1E0C0
      port map(D => ACMRDATA(3), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[17]\, Q => 
        \RTC_load[19]_net_1\);
    
    \ACMADDR_1_0[1]\ : OR3C
      port map(A => N_438, B => N_887, C => 
        \RTCcal_present_d_i[18]\, Y => ACMADDR_1);
    
    \ACMADDR_1_a4[1]\ : OR2
      port map(A => N_874, B => N_408, Y => N_438);
    
    sec_flag : DFN1E1C0
      port map(D => sec_flag_2, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => un1_rtc_init, Q => \sec_flag\);
    
    \count_day_10[1]\ : OA1A
      port map(A => \un3_inc_day\, B => \count_hours_3_sqmuxa\, C
         => I_13, Y => \count_day_10[1]_net_1\);
    
    \RTC_match[26]\ : DFN1E0C0
      port map(D => ACMRDATA(2), CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \RTCcal_present_d_i[23]\, Q => 
        \RTC_match[26]_net_1\);
    
    RTCcal_present_s17_0_o2_i_a2 : OR2A
      port map(A => \RTCcal_present[4]_net_1\, B => 
        \RTCcal_present[1]_net_1\, Y => N_400_i_0);
    
    \count_month_13[7]\ : NOR2B
      port map(A => \RTC_month_c[7]\, B => un1_rtc_init, Y => 
        \count_month_13[7]_net_1\);
    
    count_month_0_sqmuxa_1 : NOR3B
      port map(A => \RTC_month_c[4]\, B => \RTC_month_c[1]\, C
         => \RTC_month_c[0]\, Y => \count_month_0_sqmuxa_1\);
    
    RTCcal_present_s10_0_a4 : OR3A
      port map(A => \RTCcal_present[0]_net_1\, B => N_874, C => 
        N_458, Y => \RTCcal_present_d_i[18]\);
    
    \count_hours_15[5]\ : MX2
      port map(A => \RTC_match[19]_net_1\, B => 
        \count_hours_14[5]\, S => un1_rtc_init, Y => N_790);
    
    I_243 : NOR3B
      port map(A => \I_243_1\, B => \I_243_0\, C => 
        \count_hours_3_sqmuxa\, Y => N_586);
    
    count_month_12_sn_m1_0_a2 : NOR2
      port map(A => \count_month_12_sn_m1_0_a2_1\, B => 
        \count_date_0_sqmuxa\, Y => N_776);
    
    \ACMWDATA_r[4]\ : AOI1B
      port map(A => \ACMWDATA_iv_1[4]_net_1\, B => 
        \ACMWDATA_iv_2[4]_net_1\, C => N_398, Y => ACMWDATA(4));
    
    I_308 : MX2C
      port map(A => N_686, B => N_687, S => 
        \RTCcal_present[3]_net_1\, Y => N_688);
    
    \count_minutes[6]\ : DFN1C0
      port map(D => \count_minutes_13[6]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_minutes_c[6]\);
    
    \count_hours_15_r[1]\ : AOI1B
      port map(A => N_509_1_1, B => \RTCcal_present_s21_0_a4_0_1\, 
        C => N_786, Y => \count_hours_15[1]_net_1\);
    
    count_ld_i_0_o2 : OR2A
      port map(A => \RTCcal_present[3]_net_1\, B => 
        \RTCcal_present[4]_net_1\, Y => N_878);
    
    un30_count_hours_I_8 : NOR2B
      port map(A => \RTC_hours_c[1]\, B => \RTC_hours_c[0]\, Y
         => N_7_1);
    
    \count_date_12_0[1]\ : MX2C
      port map(A => I_5_1, B => \RTC_date_c[1]\, S => 
        \count_hours_3_sqmuxa\, Y => N_751);
    
    count_date_0_sqmuxa_0 : XA1A
      port map(A => \numdays_month[5]_net_1\, B => 
        \RTC_date_c[5]\, C => un8_inc_day_4_i_0, Y => 
        \count_date_0_sqmuxa_0\);
    
    \count_date[4]\ : DFN1C0
      port map(D => \count_date_13[4]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \RTC_date_c[4]\);
    
    RTCcal_present_s21_0_a4_0_0 : NOR2A
      port map(A => \RTCcal_present_0[2]_net_1\, B => N_458, Y
         => \RTCcal_present_s21_0_a4_0_0\);
    
    \count_seconds_12_r[6]\ : NOR2B
      port map(A => \count_seconds_0_sqmuxa\, B => N_811, Y => 
        \count_seconds_12[6]_net_1\);
    
    \count_year_8[4]\ : NOR2B
      port map(A => \RTC_year_c[4]\, B => un1_rtc_init, Y => 
        \count_year_8[4]_net_1\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity cwgmnvwgkwq is

    port(dsjtwjhnpmm_i : out std_logic_vector(8 downto 0); 
        \ADC_MODE_net[3]\, \ADC_TVC_net[7]\, \ADC_STC_net[7]\, 
        \ADC_CHNR_net[4]\, \tkcbxxqbtfx_ns[11]\, 
        \ASSC_SEQOUT_net[5]\, \ASSC_RAM_ADDR_B_net[8]\ : out
         std_logic; \EV_ASSCADDR_net[8]\, INIT_ADDR_int_0, 
        INIT_ADDR_int_2, INIT_ADDR_int_3, INIT_ADDR_3, 
        INIT_ADDR_2, INIT_ADDR_4, INIT_ADDR_6, INIT_ADDR_1, 
        INIT_ADDR_0 : in std_logic; \ASSC_RAM_DI_B_net[8]\ : out
         std_logic; \ADC_RESULT_net[11]\, \ASSC_RAM_DO_B_net[8]\ : 
        in std_logic; \Z\\ADC_MODE_net\\\, \Z\\ADC_TVC_net\\\, 
        ADC_TVC_net_net_1, ADC_TVC_net_net_2, ADC_TVC_net_net_3, 
        ADC_TVC_net_net_4, ADC_TVC_net_net_5, ADC_TVC_net_net_6, 
        \Z\\ADC_STC_net\\\, ADC_STC_net_net_1, ADC_STC_net_net_2, 
        ADC_STC_net_net_3, ADC_STC_net_net_4, ADC_STC_net_net_5, 
        ADC_STC_net_net_6 : out std_logic; 
        ASSC_RAM_DO_B_net_net_3 : in std_logic; 
        \Z\\ADC_CHNR_net\\\, ADC_CHNR_net_net_1, 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_3, ADC_VREFSEL_net, 
        ASSC_SAMPFLAG_net, ASSC_CHSAT_c, ADC_PDOWN_net, 
        ADC_RESET_net : out std_logic; ADC_CALIBRATE_net : in
         std_logic; ASSC_RAM_DI_B_net_net_3, ASSC_DONE_c, 
        ASSC_RAM_RW_B_net, rhbzpsxhwms : out std_logic; 
        EV_ASSCADDR_net_net_2 : in std_logic; 
        ASSC_SEQOUT_net_net_4, ASSC_RAM_ADDR_B_net_net_4, 
        ASSC_RAM_ADDR_B_net_net_3, ASSC_RAM_ADDR_B_net_net_2, 
        ASSC_SEQOUT_net_net_1, ASSC_RAM_ADDR_B_net_net_1, 
        \Z\\ASSC_RAM_ADDR_B_net\\\, ASSC_RAM_ADDR_B_net_net_5 : 
        out std_logic; EV_ASSCADDR_net_net_4, 
        \Z\\EV_ASSCADDR_net\\\, ASSC_RAM_DO_B_net_net_1, 
        ASSC_RAM_DO_B_net_net_2 : in std_logic; 
        ASSC_RAM_CSN_A_net : out std_logic; INIT_ASSC_WEN : in
         std_logic; ttxwkrvqgng_i_a3 : out std_logic; 
        EV_ASSCADDR_net_net_3, EV_ASSCADDR_net_net_1 : in
         std_logic; ADC_MODE_net_net_2 : out std_logic; 
        ADC_RESULT_net_net_7 : in std_logic; 
        \Z\\ASSC_RAM_DI_B_net\\\, ASSC_RAM_DI_B_net_net_1, 
        ASSC_RAM_DI_B_net_net_2, ASSC_RAM_DI_B_net_net_4, 
        ASSC_RAM_DI_B_net_net_5, ASSC_RAM_DI_B_net_net_6, 
        ASSC_RAM_DI_B_net_net_7 : out std_logic; 
        ADC_RESULT_net_net_8, ADC_RESULT_net_net_9, 
        ADC_RESULT_net_net_10, EV_DONE_net_0 : in std_logic; 
        ASSC_RAM_ADDR_B_net_net_7, ASSC_RAM_ADDR_B_net_net_6 : 
        out std_logic; bskrjhjvqpf_1_sqmuxa, 
        EV_ASSCADDR_net_net_5 : in std_logic; N_247_i_0 : out
         std_logic; EV_EVFLAG_net, TR_DONE_net, TR_TRFLAG_net, 
        ADC_RESULT_net_net_4, ADC_RESULT_net_net_6, 
        ADC_RESULT_net_net_5, ADC_RESULT_net_net_2, 
        ADC_RESULT_net_net_3, ADC_RESULT_net_net_1, 
        \Z\\ADC_RESULT_net\\\ : in std_logic; ADC_MODE_net_net_1 : 
        out std_logic; \Z\\ASSC_RAM_DO_B_net\\\, ADC_BUSY_net, 
        INIT_DONE_c : in std_logic; jngrzvhtjvn_0, 
        ASSC_CHLATD_c_i, ASSC_SEQOUT_net_net_2, 
        \Z\\ASSC_SEQOUT_net\\\, ASSC_SEQOUT_net_net_3 : out
         std_logic; ASSC_RAM_DO_B_net_net_4, 
        ASSC_RAM_DO_B_net_net_5, ASSC_RAM_DO_B_net_net_6 : in
         std_logic; ADC_START_net : out std_logic; 
        ASSC_RAM_DO_B_net_net_7, sys_reset_gl, ACM_CLK_c : in
         std_logic);

end cwgmnvwgkwq;

architecture DEF_ARCH of cwgmnvwgkwq is 

  component MX2
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1E1C0
    port(D, CLK, CLR, E : in std_logic := 'U'; Q : out std_logic);
        
  end component;

  component OR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1C0
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component NOR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2B
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2C
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AND2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1E0C0
    port(D, CLK, CLR, E : in std_logic := 'U'; Q : out std_logic);
        
  end component;

  component XA1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component OR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component OR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AOI1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1P0
    port(D, CLK, PRE : in std_logic := 'U'; Q : out std_logic);
  end component;

  component OA1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OAI1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AOI1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1D
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \ASSC_SEQOUT_net[5]_net_1\, \ASSC_DONE_c\, 
        rhbzpsxhwms_net_1, \ASSC_SEQOUT_net_net_4\, 
        \ASSC_SEQOUT_net_net_1\, \ADC_MODE_net_net_2\, 
        \ADC_MODE_net_net_1\, \ASSC_SEQOUT_net_net_2\, 
        \Z\\ASSC_SEQOUT_net\\_net_1\, \ASSC_SEQOUT_net_net_3\, 
        \ADC_START_net\, \jrdxqdwndbg_0[0]_net_1\, 
        \jrdxqdwndbg_0[1]_net_1\, \jrdxqdwndbg_0[2]_net_1\, 
        \jrdxqdwndbg_0[3]_net_1\, \DWACT_ADD_CI_0_g_array_2[0]\, 
        \DWACT_ADD_CI_0_g_array_1[0]\, 
        \DWACT_ADD_CI_0_pog_array_1[0]\, \DWACT_ADD_CI_0_TMP[0]\, 
        \DWACT_ADD_CI_0_g_array_12_1[0]\, 
        \DWACT_ADD_CI_0_g_array_12[0]\, \fkgscznxtsm_2_i\, 
        \fkgscznxtsm_2\, ASSC_CHLATD_c, \drsbgqmzxjf_i_1\, N_809, 
        \zwnnqcvfccb[13]_net_1\, \zwnnqcvfccb[10]_net_1\, 
        \un1_cstdbndzxqp\, mkcbzxzcrgq, \thvjrqnxtqb_iv_0\, 
        \jjqqfszbdzb\, gzmcjjcpmtm_8_sqmuxa, 
        \zwnnqcvfccb_ns_2[0]_net_1\, \zwnnqcvfccb_ns_0[0]_net_1\, 
        N_308_3, \prmfbzxmsxf\, \zwnnqcvfccb[0]_net_1\, 
        \zwnnqcvfccb[12]_net_1\, \un1_znktskmxcbp_2_i_0\, 
        N_279_i_0, N_308_1, \fkgscznxtsm_2_0\, \bvzjfmgvnzp\, 
        \zbbgcrmtqtc_1[1]_net_1\, \zwnnqcvfccb[6]_net_1\, 
        \zwnnqcvfccb[3]_net_1\, \zwnnqcvfccb_ns_i_a3_0[13]_net_1\, 
        \zwnnqcvfccb[1]_net_1\, \zwnnqcvfccb[14]_net_1\, 
        \zwnnqcvfccb_ns_i_0[7]_net_1\, \zwnnqcvfccb[7]_net_1\, 
        \zwnnqcvfccb[8]_net_1\, \un1_nvmzjtrjcbj_0_0_a3_0_0\, 
        \gsbzbrrsfpj_2_i_8\, N_824, \zwnnqcvfccb[11]_net_1\, 
        \gsbzbrrsfpj_2_i_7\, \gsbzbrrsfpj_2_i_5\, N_830, 
        \gsbzbrrsfpj_2_i_3\, \gsbzbrrsfpj_2_i_1\, 
        \un6_wsshgnvvnmk_1\, \wrmchxtnrsq[2]\, N_289, 
        \zwnnqcvfccb_ns_i_0[3]_net_1\, \prgqnmxpkfr\, 
        un1_btxdzwbbqvw_1_sqmuxa, 
        un1_Analog_RTC_assc_wrapper_inst, 
        \zwnnqcvfccb_ns_i[7]_net_1\, N_336, N_870, 
        \zwnnqcvfccb_ns_i[13]_net_1\, N_338_1, \pdmrrrmtgwp[0]\, 
        \zwnnqcvfccb[4]_net_1\, \zwnnqcvfccb[5]_net_1\, 
        thvjrqnxtqb, N_269, \zwnnqcvfccb_ns_i_1[8]_net_1\, N_861, 
        N_863, N_271_i_0, N_807_i, N_862, N_892, N_275, 
        \zwnnqcvfccb_ns_i[14]_net_1\, N_341, N_339, 
        \zwnnqcvfccb_ns_i[12]_net_1\, N_267, N_299, N_740, N_395, 
        \bsrvxsspcfd\, \zwnnqcvfccb_ns[0]_net_1\, N_745_i_0, 
        N_829, \hdfsppmkkfr[15]\, N_495, \hdfsppmkkfr[16]\, N_496, 
        \un3_vpprztfztfv\, \vpprztfztfv\, \un5_vpprztfztfv\, 
        N_298, \qkzsmdfkvxj\, N_303, N_812, \prgqnmxpkfr_7_i\, 
        N_887, \hdfsppmkkfr[0]\, \hdfsppmkkfr[9]\, N_888, 
        \hdfsppmkkfr[1]\, \hdfsppmkkfr[10]\, N_889, 
        \hdfsppmkkfr[2]\, \hdfsppmkkfr[11]\, N_422, 
        \hdfsppmkkfr[3]\, \hdfsppmkkfr[12]\, N_424, 
        \hdfsppmkkfr[5]\, \hdfsppmkkfr[14]\, N_425, 
        \hdfsppmkkfr[6]\, N_426, \hdfsppmkkfr[7]\, N_427, 
        \hdfsppmkkfr[8]\, \jrdxqdwndbg[4]_net_1\, N_430, N_431, 
        N_432, N_439_i, N_433, N_435, N_436, N_437, N_438, N_486, 
        N_526, N_866, N_487, N_527, N_525, N_488, N_528, N_489, 
        N_529, N_490, N_530, N_491, N_531, N_492, N_532, N_493, 
        N_533, N_494, N_534, N_516, N_517, N_557, N_608, N_559, 
        N_610, N_560, N_561, N_562, N_563, N_566, N_498, N_569, 
        N_570, N_586, \jrdxqdwndbg[0]_net_1\, N_587, N_603, N_604, 
        N_605, N_620, N_607, N_637, \jrdxqdwndbg[1]_net_1\, N_621, 
        N_638, N_609, N_639, N_640, N_611, N_641, N_612, N_642, 
        N_671, N_672, N_673, N_674, N_606, N_676, N_485, N_678, 
        N_614, N_842, N_553, N_554, N_555, N_558, N_677, 
        \jrdxqdwndbg[2]_net_1\, N_679, \jrdxqdwndbg[3]_net_1\, 
        N_680, N_681, N_682, N_613, N_872, 
        \zwnnqcvfccb_ns_i_0[1]_net_1\, 
        \zwnnqcvfccb_ns_i_0[2]_net_1\, 
        \zwnnqcvfccb_ns_i_1[6]_net_1\, 
        \zwnnqcvfccb_ns_i_1[10]_net_1\, gzmcjjcpmtm_5_sqmuxa, 
        N_280_i_0, N_274_i_0, \jjqqfszbdzb_46\, 
        \zwnnqcvfccb[2]_net_1\, \zwnnqcvfccb_ns_i_1[11]_net_1\, 
        N_564, \prmfbzxmsxf_44\, \un4_cstdbndzxqp\, N_394_i_0, 
        un1_znktskmxcbp_9, N_348, N_841, 
        \zwnnqcvfccb_ns_i_i_a3[4]_net_1\, 
        \gjpxswjjjjj_4[0]_net_1\, N_411, \gjpxswjjjjj_4[1]_net_1\, 
        N_412, \gjpxswjjjjj_4[2]_net_1\, N_413, 
        \gjpxswjjjjj_4[3]_net_1\, N_414, \gjpxswjjjjj_4[4]_net_1\, 
        N_415, \gjpxswjjjjj_4[5]_net_1\, N_416, 
        \jqpswnvgwgd[0]_net_1\, \DWACT_ADD_CI_0_partial_sum[0]\, 
        \jqpswnvgwgd[1]_net_1\, I_21, \jqpswnvgwgd[2]_net_1\, 
        I_22, \jqpswnvgwgd[3]_net_1\, I_23, 
        \jqpswnvgwgd[4]_net_1\, I_24, \jqpswnvgwgd[5]_net_1\, 
        I_26, N_743, N_822, \zwnnqcvfccb_ns_i_1[5]_net_1\, 
        \zwnnqcvfccb_ns_i_1[9]_net_1\, gkwwhjwcbtz, 
        un1_nvmzjtrjcbj, \hdfsppmkkfr[13]\, N_565, N_497, N_434, 
        \hdfsppmkkfr[4]\, N_556, N_675, N_423, N_391, \VCC\, 
        \GND\, GND_net_1, VCC_net_1 : std_logic;

begin 

    \ASSC_SEQOUT_net[5]\ <= \ASSC_SEQOUT_net[5]_net_1\;
    ASSC_DONE_c <= \ASSC_DONE_c\;
    rhbzpsxhwms <= rhbzpsxhwms_net_1;
    ASSC_SEQOUT_net_net_4 <= \ASSC_SEQOUT_net_net_4\;
    ASSC_SEQOUT_net_net_1 <= \ASSC_SEQOUT_net_net_1\;
    ADC_MODE_net_net_2 <= \ADC_MODE_net_net_2\;
    ADC_MODE_net_net_1 <= \ADC_MODE_net_net_1\;
    ASSC_SEQOUT_net_net_2 <= \ASSC_SEQOUT_net_net_2\;
    \Z\\ASSC_SEQOUT_net\\\ <= \Z\\ASSC_SEQOUT_net\\_net_1\;
    ASSC_SEQOUT_net_net_3 <= \ASSC_SEQOUT_net_net_3\;
    ADC_START_net <= \ADC_START_net\;

    \hdfsppmkkfr_3[6]\ : MX2
      port map(A => N_530, B => N_528, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_490);
    
    ddzrbwxtmtz_sn_m2_0_a2 : OR2
      port map(A => \zwnnqcvfccb[4]_net_1\, B => 
        \zwnnqcvfccb[3]_net_1\, Y => N_439_i);
    
    \hdfsppmkkfr_3[14]\ : NOR2B
      port map(A => \jrdxqdwndbg[1]_net_1\, B => N_517, Y => 
        N_498);
    
    \mcccbsrsbmn[3]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_1, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[13]_net_1\, Q => 
        \ADC_MODE_net[3]\);
    
    \hdfsppmkkfr_4[15]\ : MX2
      port map(A => \ADC_RESULT_net[11]\, B => 
        \Z\\ADC_RESULT_net\\\, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_516);
    
    \ddzrbwxtmtz_1[4]\ : OR2A
      port map(A => ADC_RESULT_net_net_6, B => 
        \zwnnqcvfccb[5]_net_1\, Y => N_434);
    
    \zwnnqcvfccb[11]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i_0[3]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \zwnnqcvfccb[11]_net_1\);
    
    \zwnnqcvfccb_ns_i[13]\ : NOR3A
      port map(A => INIT_DONE_c, B => N_338_1, C => N_336, Y => 
        \zwnnqcvfccb_ns_i[13]_net_1\);
    
    prgqnmxpkfr : DFN1C0
      port map(D => \prgqnmxpkfr_7_i\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \prgqnmxpkfr\);
    
    \zwnnqcvfccb_ns_i_0[3]\ : NOR3B
      port map(A => \prgqnmxpkfr\, B => INIT_DONE_c, C => N_809, 
        Y => \zwnnqcvfccb_ns_i_0[3]_net_1\);
    
    \hdfsppmkkfr_12[3]\ : MX2
      port map(A => ADC_RESULT_net_net_1, B => 
        \Z\\ADC_RESULT_net\\\, S => \jrdxqdwndbg[0]_net_1\, Y => 
        N_640);
    
    \zwnnqcvfccb_ns_a3_3_3[0]\ : OR3C
      port map(A => \Z\\ASSC_RAM_DO_B_net\\\, B => 
        \zwnnqcvfccb[11]_net_1\, C => N_308_1, Y => N_308_3);
    
    gsbzbrrsfpj_2_i_a3_0 : AO1C
      port map(A => \ADC_MODE_net_net_1\, B => 
        ADC_RESULT_net_net_10, C => \ADC_MODE_net_net_2\, Y => 
        N_829);
    
    \hdfsppmkkfr_5[8]\ : MX2
      port map(A => ADC_RESULT_net_net_8, B => 
        ADC_RESULT_net_net_9, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_526);
    
    un1_znktskmxcbp_5_i_o3 : NOR2A
      port map(A => INIT_DONE_c, B => \ASSC_DONE_c\, Y => N_822);
    
    \jqpswnvgwgd[3]\ : DFN1C0
      port map(D => \jqpswnvgwgd[3]_net_1\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \jqpswnvgwgd[3]_net_1\);
    
    \hdfsppmkkfr_15[13]\ : NOR2A
      port map(A => N_565, B => \jrdxqdwndbg[3]_net_1\, Y => 
        \hdfsppmkkfr[13]\);
    
    \dsjtwjhnpmm_i_a3[1]\ : NOR2B
      port map(A => INIT_DONE_c, B => EV_ASSCADDR_net_net_5, Y
         => \tkcbxxqbtfx_ns[11]\);
    
    \zwnnqcvfccb[10]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i_i_a3[4]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \zwnnqcvfccb[10]_net_1\);
    
    I_186 : NOR2B
      port map(A => \zwnnqcvfccb[1]_net_1\, B => \qkzsmdfkvxj\, Y
         => N_391);
    
    \hdfsppmkkfr_14[3]\ : MX2
      port map(A => N_606, B => N_610, S => 
        \jrdxqdwndbg_0[2]_net_1\, Y => N_674);
    
    \hdfsppmkkfr_12[5]\ : NOR2A
      port map(A => \ADC_RESULT_net[11]\, B => 
        \jrdxqdwndbg[0]_net_1\, Y => N_642);
    
    un1_gjpxswjjjjj_I_31 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_1[0]\, B => 
        \ASSC_SEQOUT_net_net_2\, Y => 
        \DWACT_ADD_CI_0_g_array_12[0]\);
    
    \mcccbsrsbmn[0]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_4, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[13]_net_1\, Q => 
        \ADC_MODE_net_net_2\);
    
    cnczrfjdnnk : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_7, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[13]_net_1\, Q => 
        ADC_VREFSEL_net);
    
    \gjpxswjjjjj_4[2]\ : MX2
      port map(A => \jqpswnvgwgd[2]_net_1\, B => I_22, S => 
        N_394_i_0, Y => N_413);
    
    \hdfsppmkkfr_15[7]\ : MX2
      port map(A => N_559, B => N_678, S => 
        \jrdxqdwndbg[2]_net_1\, Y => \hdfsppmkkfr[7]\);
    
    gsbzbrrsfpj_2_i_8 : OA1B
      port map(A => \ADC_MODE_net_net_1\, B => N_824, C => 
        \zwnnqcvfccb[11]_net_1\, Y => \gsbzbrrsfpj_2_i_8\);
    
    \dsjtwjhnpmm_i[0]\ : MX2B
      port map(A => INIT_ADDR_0, B => EV_ASSCADDR_net_net_5, S
         => INIT_DONE_c, Y => dsjtwjhnpmm_i(0));
    
    \gjpxswjjjjj_4_r[1]\ : NOR3C
      port map(A => \un4_cstdbndzxqp\, B => INIT_DONE_c, C => 
        N_412, Y => \gjpxswjjjjj_4[1]_net_1\);
    
    \hdfsppmkkfr_14[11]\ : NOR2A
      port map(A => N_614, B => \jrdxqdwndbg[2]_net_1\, Y => 
        N_682);
    
    \pdmrrrmtgwp_0[0]\ : NOR2
      port map(A => N_812, B => N_303, Y => \pdmrrrmtgwp[0]\);
    
    \gjpxswjjjjj_4_r[0]\ : NOR3C
      port map(A => \un4_cstdbndzxqp\, B => INIT_DONE_c, C => 
        N_411, Y => \gjpxswjjjjj_4[0]_net_1\);
    
    un1_nvmzjtrjcbj_0_0_a3_0_0 : OR2
      port map(A => \ASSC_RAM_DO_B_net[8]\, B => 
        \Z\\ASSC_RAM_DO_B_net\\\, Y => 
        \un1_nvmzjtrjcbj_0_0_a3_0_0\);
    
    \zbbgcrmtqtc[6]\ : NOR2B
      port map(A => \pdmrrrmtgwp[0]\, B => 
        \ASSC_SEQOUT_net_net_1\, Y => ASSC_RAM_ADDR_B_net_net_1);
    
    drsbgqmzxjf_i_1 : NOR3A
      port map(A => N_809, B => \zwnnqcvfccb[13]_net_1\, C => 
        \zwnnqcvfccb[10]_net_1\, Y => \drsbgqmzxjf_i_1\);
    
    \hnfknqnpjhb[1]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_6, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[12]_net_1\, Q => 
        ADC_TVC_net_net_5);
    
    I_197 : OR2B
      port map(A => \prmfbzxmsxf\, B => N_275, Y => N_394_i_0);
    
    \gjpxswjjjjj_4_r[3]\ : NOR3C
      port map(A => \un4_cstdbndzxqp\, B => INIT_DONE_c, C => 
        N_414, Y => \gjpxswjjjjj_4[3]_net_1\);
    
    \ddzrbwxtmtz[3]\ : MX2C
      port map(A => N_433, B => N_422, S => N_439_i, Y => 
        ASSC_RAM_DI_B_net_net_4);
    
    \hdfsppmkkfr_5[14]\ : MX2
      port map(A => ADC_RESULT_net_net_2, B => 
        ADC_RESULT_net_net_3, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_532);
    
    gzmcjjcpmtm_8_sqmuxa_0_o3_0_i_a3 : OR2A
      port map(A => N_271_i_0, B => \Z\\ASSC_RAM_DO_B_net\\\, Y
         => N_274_i_0);
    
    \hdfsppmkkfr_7[10]\ : MX2
      port map(A => N_494, B => N_490, S => 
        \jrdxqdwndbg_0[2]_net_1\, Y => N_562);
    
    un1_gjpxswjjjjj_I_34 : AND2
      port map(A => \ASSC_SEQOUT_net_net_2\, B => 
        \ASSC_SEQOUT_net_net_1\, Y => 
        \DWACT_ADD_CI_0_pog_array_1[0]\);
    
    \hdfsppmkkfr_15[9]\ : MX2
      port map(A => N_671, B => N_680, S => 
        \zwnnqcvfccb[3]_net_1\, Y => \hdfsppmkkfr[9]\);
    
    \jrdxqdwndbg_0[0]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_7, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ADC_START_net\, Q => 
        \jrdxqdwndbg_0[0]_net_1\);
    
    \hdfsppmkkfr_10[9]\ : MX2
      port map(A => N_640, B => N_642, S => 
        \jrdxqdwndbg[1]_net_1\, Y => N_612);
    
    \gjpxswjjjjj_4[0]\ : MX2
      port map(A => \jqpswnvgwgd[0]_net_1\, B => 
        \DWACT_ADD_CI_0_partial_sum[0]\, S => N_394_i_0, Y => 
        N_411);
    
    \jqpswnvgwgd[0]\ : DFN1C0
      port map(D => \jqpswnvgwgd[0]_net_1\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \jqpswnvgwgd[0]_net_1\);
    
    \zwnnqcvfccb[2]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i[12]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \zwnnqcvfccb[2]_net_1\);
    
    xqhtcpzgsdn : DFN1C0
      port map(D => gkwwhjwcbtz, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \ASSC_DONE_c\);
    
    \hdfsppmkkfr_3[10]\ : MX2
      port map(A => N_534, B => N_532, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_494);
    
    \ngwjhwffdrb[2]\ : DFN1E0C0
      port map(D => ASSC_RAM_DO_B_net_net_5, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => ASSC_CHLATD_c, Q => 
        ADC_CHNR_net_net_1);
    
    gzmcjjcpmtm_8_sqmuxa_0_o3 : XA1A
      port map(A => \ASSC_RAM_DO_B_net[8]\, B => N_274_i_0, C => 
        \zwnnqcvfccb[11]_net_1\, Y => gzmcjjcpmtm_8_sqmuxa);
    
    un1_gjpxswjjjjj_I_30 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_2[0]\, B => 
        \Z\\ASSC_SEQOUT_net\\_net_1\, Y => 
        \DWACT_ADD_CI_0_g_array_12_1[0]\);
    
    \hdfsppmkkfr_5[16]\ : MX2
      port map(A => \Z\\ADC_RESULT_net\\\, B => 
        ADC_RESULT_net_net_1, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_534);
    
    \hdfsppmkkfr_15[5]\ : MX2
      port map(A => N_557, B => N_676, S => 
        \jrdxqdwndbg_0[2]_net_1\, Y => \hdfsppmkkfr[5]\);
    
    \gjpxswjjjjj_4_r[5]\ : NOR3C
      port map(A => \un4_cstdbndzxqp\, B => INIT_DONE_c, C => 
        N_416, Y => \gjpxswjjjjj_4[5]_net_1\);
    
    \ddzrbwxtmtz[5]\ : MX2C
      port map(A => N_435, B => N_424, S => N_439_i, Y => 
        ASSC_RAM_DI_B_net_net_2);
    
    un1_nvmzjtrjcbj_0_0_a3_1 : NOR3B
      port map(A => \Z\\ASSC_RAM_DO_B_net\\\, B => N_870, C => 
        \ASSC_RAM_DO_B_net[8]\, Y => N_861);
    
    \hdfsppmkkfr_12[2]\ : MX2
      port map(A => ADC_RESULT_net_net_2, B => 
        ADC_RESULT_net_net_1, S => \jrdxqdwndbg[0]_net_1\, Y => 
        N_639);
    
    \hdfsppmkkfr_10[11]\ : NOR2A
      port map(A => N_642, B => \jrdxqdwndbg[1]_net_1\, Y => 
        N_614);
    
    \hdfsppmkkfr_7[12]\ : MX2
      port map(A => N_496, B => N_492, S => 
        \jrdxqdwndbg[2]_net_1\, Y => N_564);
    
    \hdfsppmkkfr_3_0_a3[0]\ : NOR2A
      port map(A => N_866, B => \jrdxqdwndbg[1]_net_1\, Y => 
        N_872);
    
    \jrdxqdwndbg_0[2]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_5, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ADC_START_net\, Q => 
        \jrdxqdwndbg_0[2]_net_1\);
    
    \hdfsppmkkfr_14[4]\ : MX2
      port map(A => N_872, B => N_611, S => 
        \jrdxqdwndbg[3]_net_1\, Y => N_675);
    
    \ddzrbwxtmtz_1[7]\ : OR2A
      port map(A => ADC_RESULT_net_net_3, B => 
        \zwnnqcvfccb[5]_net_1\, Y => N_437);
    
    \zwnnqcvfccb_ns_i_a3_1[14]\ : NOR2A
      port map(A => \prmfbzxmsxf\, B => \zwnnqcvfccb[11]_net_1\, 
        Y => N_341);
    
    \hdfsppmkkfr_7[8]\ : MX2
      port map(A => N_492, B => N_488, S => 
        \jrdxqdwndbg_0[2]_net_1\, Y => N_560);
    
    \hdfsppmkkfr_10[4]\ : MX2
      port map(A => N_620, B => N_637, S => 
        \jrdxqdwndbg[1]_net_1\, Y => N_607);
    
    \ngwjhwffdrb[4]\ : DFN1E0C0
      port map(D => ASSC_RAM_DO_B_net_net_3, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => ASSC_CHLATD_c, Q => 
        \ADC_CHNR_net[4]\);
    
    GND_i : GND
      port map(Y => \GND\);
    
    \hdfsppmkkfr_3[12]\ : MX2
      port map(A => N_517, B => N_534, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_496);
    
    \ddzrbwxtmtz_0[8]\ : MX2C
      port map(A => \hdfsppmkkfr[8]\, B => \jrdxqdwndbg[4]_net_1\, 
        S => \zwnnqcvfccb[3]_net_1\, Y => N_427);
    
    \hdfsppmkkfr_7[9]\ : MX2
      port map(A => N_493, B => N_489, S => 
        \jrdxqdwndbg_0[2]_net_1\, Y => N_561);
    
    \ddzrbwxtmtz_0[6]\ : MX2C
      port map(A => \hdfsppmkkfr[6]\, B => \hdfsppmkkfr[15]\, S
         => \zwnnqcvfccb[3]_net_1\, Y => N_425);
    
    \zwnnqcvfccb[7]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i[7]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \zwnnqcvfccb[7]_net_1\);
    
    \zwnnqcvfccb_ns[0]\ : OR3A
      port map(A => \zwnnqcvfccb_ns_2[0]_net_1\, B => 
        gzmcjjcpmtm_8_sqmuxa, C => N_269, Y => 
        \zwnnqcvfccb_ns[0]_net_1\);
    
    \ddzrbwxtmtz[4]\ : MX2C
      port map(A => N_434, B => N_423, S => N_439_i, Y => 
        ASSC_RAM_DI_B_net_net_3);
    
    \ddzrbwxtmtz[6]\ : MX2C
      port map(A => N_436, B => N_425, S => N_439_i, Y => 
        ASSC_RAM_DI_B_net_net_1);
    
    \hdfsppmkkfr_15[4]\ : MX2
      port map(A => N_556, B => N_675, S => 
        \jrdxqdwndbg[2]_net_1\, Y => \hdfsppmkkfr[4]\);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    \jrdxqdwndbg_0[1]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_6, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ADC_START_net\, Q => 
        \jrdxqdwndbg_0[1]_net_1\);
    
    \hdfsppmkkfr_7[13]\ : MX2
      port map(A => N_497, B => N_493, S => 
        \jrdxqdwndbg[2]_net_1\, Y => N_565);
    
    \hdfsppmkkfr_15[11]\ : MX2
      port map(A => N_563, B => N_682, S => 
        \jrdxqdwndbg[3]_net_1\, Y => \hdfsppmkkfr[11]\);
    
    gvghxwwqjrn : DFN1E1C0
      port map(D => N_841, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        E => un1_znktskmxcbp_9, Q => ADC_PDOWN_net);
    
    drsbgqmzxjf_i_o3 : OR3
      port map(A => \zwnnqcvfccb[5]_net_1\, B => 
        \zwnnqcvfccb[4]_net_1\, C => \zwnnqcvfccb[3]_net_1\, Y
         => \wrmchxtnrsq[2]\);
    
    \gjpxswjjjjj[3]\ : DFN1C0
      port map(D => \gjpxswjjjjj_4[3]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \ASSC_SEQOUT_net_net_1\);
    
    \dsjtwjhnpmm_i[8]\ : MX2
      port map(A => INIT_ADDR_int_3, B => \EV_ASSCADDR_net[8]\, S
         => INIT_DONE_c, Y => dsjtwjhnpmm_i(8));
    
    \zbbgcrmtqtc[0]\ : OR3A
      port map(A => \pdmrrrmtgwp[0]\, B => \zwnnqcvfccb[4]_net_1\, 
        C => N_289, Y => ASSC_RAM_ADDR_B_net_net_7);
    
    gsbzbrrsfpj_2_i : NOR3C
      port map(A => N_829, B => \gsbzbrrsfpj_2_i_7\, C => 
        \gsbzbrrsfpj_2_i_8\, Y => N_745_i_0);
    
    un5_vpprztfztfv : OR2
      port map(A => \un3_vpprztfztfv\, B => \fkgscznxtsm_2\, Y
         => \un5_vpprztfztfv\);
    
    \hnfknqnpjhb[3]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_4, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[12]_net_1\, Q => 
        ADC_TVC_net_net_3);
    
    \hdfsppmkkfr_7_0_a3[0]\ : NOR2A
      port map(A => N_872, B => \jrdxqdwndbg[2]_net_1\, Y => 
        N_842);
    
    \gjpxswjjjjj[5]\ : DFN1C0
      port map(D => \gjpxswjjjjj_4[5]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \ASSC_SEQOUT_net[5]_net_1\);
    
    \jrdxqdwndbg_0[3]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_4, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ADC_START_net\, Q => 
        \jrdxqdwndbg_0[3]_net_1\);
    
    \ddzrbwxtmtz[8]\ : MX2C
      port map(A => N_438, B => N_427, S => N_439_i, Y => 
        \ASSC_RAM_DI_B_net[8]\);
    
    \zwnnqcvfccb_ns_i[14]\ : NOR3A
      port map(A => INIT_DONE_c, B => N_341, C => N_339, Y => 
        \zwnnqcvfccb_ns_i[14]_net_1\);
    
    \hdfsppmkkfr_3[13]\ : NOR2B
      port map(A => \jrdxqdwndbg[1]_net_1\, B => N_516, Y => 
        N_497);
    
    \jrdxqdwndbg[0]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_7, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ADC_START_net\, Q => 
        \jrdxqdwndbg[0]_net_1\);
    
    \zbbgcrmtqtc[1]\ : OR2A
      port map(A => \pdmrrrmtgwp[0]\, B => 
        \zbbgcrmtqtc_1[1]_net_1\, Y => ASSC_RAM_ADDR_B_net_net_6);
    
    \ddzrbwxtmtz_1[1]\ : MX2C
      port map(A => ADC_RESULT_net_net_9, B => 
        \Z\\ADC_RESULT_net\\\, S => \zwnnqcvfccb[5]_net_1\, Y => 
        N_431);
    
    \zbbgcrmtqtc[3]\ : MX2
      port map(A => \zwnnqcvfccb[13]_net_1\, B => 
        \ASSC_SEQOUT_net_net_4\, S => \pdmrrrmtgwp[0]\, Y => 
        ASSC_RAM_ADDR_B_net_net_4);
    
    \hdfsppmkkfr_12[0]\ : MX2
      port map(A => ADC_RESULT_net_net_4, B => 
        ADC_RESULT_net_net_3, S => \jrdxqdwndbg[0]_net_1\, Y => 
        N_637);
    
    \zbbgcrmtqtc[2]\ : OR2A
      port map(A => \pdmrrrmtgwp[0]\, B => \wrmchxtnrsq[2]\, Y
         => ASSC_RAM_ADDR_B_net_net_5);
    
    jjqqfszbdzb_46 : AO1A
      port map(A => thvjrqnxtqb, B => INIT_DONE_c, C => 
        \jjqqfszbdzb\, Y => \jjqqfszbdzb_46\);
    
    \zwnnqcvfccb_ns_i_1[6]\ : NOR2B
      port map(A => INIT_DONE_c, B => \ADC_START_net\, Y => 
        \zwnnqcvfccb_ns_i_1[6]_net_1\);
    
    \hdfsppmkkfr_5[10]\ : MX2
      port map(A => ADC_RESULT_net_net_6, B => 
        ADC_RESULT_net_net_7, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_528);
    
    un1_znktskmxcbp_5_i : OR2B
      port map(A => N_822, B => ASSC_CHLATD_c, Y => N_743);
    
    un3_vpprztfztfv : NOR2A
      port map(A => \vpprztfztfv\, B => \bvzjfmgvnzp\, Y => 
        \un3_vpprztfztfv\);
    
    \ddzrbwxtmtz_0[5]\ : MX2C
      port map(A => \hdfsppmkkfr[5]\, B => \hdfsppmkkfr[14]\, S
         => \zwnnqcvfccb[3]_net_1\, Y => N_424);
    
    un1_znktskmxcbp_9_0_a3 : NOR2B
      port map(A => N_280_i_0, B => N_271_i_0, Y => N_348);
    
    \hdfsppmkkfr_5[15]\ : MX2
      port map(A => ADC_RESULT_net_net_1, B => 
        ADC_RESULT_net_net_2, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_533);
    
    \jrdxqdwndbg[2]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_5, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ADC_START_net\, Q => 
        \jrdxqdwndbg[2]_net_1\);
    
    \zwnnqcvfccb_ns_i_0_o3_0[1]\ : NOR2
      port map(A => \prgqnmxpkfr\, B => N_809, Y => N_812);
    
    \hdfsppmkkfr_15[16]\ : NOR3B
      port map(A => \jrdxqdwndbg_0[2]_net_1\, B => N_496, C => 
        \jrdxqdwndbg_0[3]_net_1\, Y => \hdfsppmkkfr[16]\);
    
    un1_nvmzjtrjcbj_0_0 : OR2B
      port map(A => N_395, B => ASSC_CHLATD_c, Y => 
        un1_nvmzjtrjcbj);
    
    \hnfknqnpjhb[0]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_7, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[12]_net_1\, Q => 
        ADC_TVC_net_net_6);
    
    \cmdprjtzcrv[7]\ : DFN1E1C0
      port map(D => \Z\\ASSC_RAM_DO_B_net\\\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[10]_net_1\, Q => 
        \ADC_STC_net[7]\);
    
    un6_wsshgnvvnmk : NOR2
      port map(A => \un6_wsshgnvvnmk_1\, B => EV_DONE_net_0, Y
         => N_892);
    
    \hnfknqnpjhb[2]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_5, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[12]_net_1\, Q => 
        ADC_TVC_net_net_4);
    
    un1_gjpxswjjjjj_I_1 : AND2
      port map(A => \ASSC_SEQOUT_net_net_4\, B => 
        rhbzpsxhwms_net_1, Y => \DWACT_ADD_CI_0_TMP[0]\);
    
    \hdfsppmkkfr_3[3]\ : MX2
      port map(A => N_527, B => N_525, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_487);
    
    \hdfsppmkkfr_7[3]\ : NOR2A
      port map(A => N_487, B => \jrdxqdwndbg[2]_net_1\, Y => 
        N_555);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    \hdfsppmkkfr_3[9]\ : MX2
      port map(A => N_533, B => N_531, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_493);
    
    sdqvwvrqjsk_0_a3 : NOR2
      port map(A => \zwnnqcvfccb[6]_net_1\, B => \wrmchxtnrsq[2]\, 
        Y => ASSC_RAM_RW_B_net);
    
    \jqpswnvgwgd[1]\ : DFN1C0
      port map(D => \jqpswnvgwgd[1]_net_1\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \jqpswnvgwgd[1]_net_1\);
    
    \ddzrbwxtmtz_0[0]\ : MX2C
      port map(A => \hdfsppmkkfr[0]\, B => \hdfsppmkkfr[9]\, S
         => \jrdxqdwndbg_0[3]_net_1\, Y => N_887);
    
    \ttxwkrvqgng_i_a3\ : OR2A
      port map(A => INIT_ASSC_WEN, B => INIT_DONE_c, Y => 
        ttxwkrvqgng_i_a3);
    
    \hdfsppmkkfr_5[12]\ : MX2
      port map(A => ADC_RESULT_net_net_4, B => 
        ADC_RESULT_net_net_5, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_530);
    
    un1_gjpxswjjjjj_I_21 : XOR2
      port map(A => \ASSC_SEQOUT_net_net_3\, B => 
        \DWACT_ADD_CI_0_TMP[0]\, Y => I_21);
    
    \gjpxswjjjjj_4[1]\ : MX2
      port map(A => \jqpswnvgwgd[1]_net_1\, B => I_21, S => 
        N_394_i_0, Y => N_412);
    
    \zwnnqcvfccb_ns_a3_3_1[0]\ : NOR2A
      port map(A => ASSC_RAM_DO_B_net_net_1, B => 
        ASSC_RAM_DO_B_net_net_2, Y => N_308_1);
    
    \pdmrrrmtgwp_0_a3[0]\ : NOR2A
      port map(A => \zwnnqcvfccb[13]_net_1\, B => 
        \zwnnqcvfccb[14]_net_1\, Y => N_303);
    
    N_280_i_0_a3 : NOR3B
      port map(A => \zwnnqcvfccb[11]_net_1\, B => 
        \Z\\ASSC_RAM_DO_B_net\\\, C => \ASSC_RAM_DO_B_net[8]\, Y
         => N_280_i_0);
    
    \hdfsppmkkfr_12[1]\ : MX2
      port map(A => ADC_RESULT_net_net_3, B => 
        ADC_RESULT_net_net_2, S => \jrdxqdwndbg[0]_net_1\, Y => 
        N_638);
    
    \hdfsppmkkfr_10[5]\ : MX2
      port map(A => N_621, B => N_638, S => 
        \jrdxqdwndbg[1]_net_1\, Y => N_608);
    
    gsbzbrrsfpj_2_i_a3_1 : OR2A
      port map(A => \ADC_MODE_net_net_2\, B => 
        ADC_RESULT_net_net_9, Y => N_830);
    
    \zwnnqcvfccb_ns_i_1[11]\ : NOR2B
      port map(A => INIT_DONE_c, B => \zwnnqcvfccb[4]_net_1\, Y
         => \zwnnqcvfccb_ns_i_1[11]_net_1\);
    
    un1_gzmcjjcpmtm_1_sqmuxa_1_0_o3_i_a3 : NOR3A
      port map(A => \zwnnqcvfccb[11]_net_1\, B => 
        \ASSC_RAM_DO_B_net[8]\, C => \Z\\ASSC_RAM_DO_B_net\\\, Y
         => N_279_i_0);
    
    gsbzbrrsfpj_2_i_7 : NOR3C
      port map(A => \Z\\ADC_RESULT_net\\\, B => 
        \gsbzbrrsfpj_2_i_5\, C => N_830, Y => \gsbzbrrsfpj_2_i_7\);
    
    gsbzbrrsfpj_2_i_1 : NOR3C
      port map(A => ADC_RESULT_net_net_5, B => 
        ADC_RESULT_net_net_6, C => ADC_RESULT_net_net_4, Y => 
        \gsbzbrrsfpj_2_i_1\);
    
    \dsjtwjhnpmm_i[1]\ : MX2B
      port map(A => INIT_ADDR_1, B => EV_ASSCADDR_net_net_5, S
         => INIT_DONE_c, Y => dsjtwjhnpmm_i(1));
    
    \zwnnqcvfccb_ns_i_0[1]\ : NOR2B
      port map(A => INIT_DONE_c, B => N_812, Y => 
        \zwnnqcvfccb_ns_i_0[1]_net_1\);
    
    \hdfsppmkkfr_15[3]\ : MX2
      port map(A => N_555, B => N_674, S => 
        \jrdxqdwndbg_0[3]_net_1\, Y => \hdfsppmkkfr[3]\);
    
    \zwnnqcvfccb_ns_i_a3_0[13]\ : OR2
      port map(A => \zwnnqcvfccb[1]_net_1\, B => 
        \zwnnqcvfccb[14]_net_1\, Y => 
        \zwnnqcvfccb_ns_i_a3_0[13]_net_1\);
    
    \zwnnqcvfccb_ns_i_a3[14]\ : OA1C
      port map(A => N_280_i_0, B => ASSC_RAM_DO_B_net_net_1, C
         => \zwnnqcvfccb[0]_net_1\, Y => N_339);
    
    un1_gjpxswjjjjj_I_28 : NOR2B
      port map(A => \DWACT_ADD_CI_0_TMP[0]\, B => 
        \ASSC_SEQOUT_net_net_3\, Y => 
        \DWACT_ADD_CI_0_g_array_1[0]\);
    
    un6_wsshgnvvnmk_1 : OR3
      port map(A => TR_TRFLAG_net, B => TR_DONE_net, C => 
        EV_EVFLAG_net, Y => \un6_wsshgnvvnmk_1\);
    
    \hdfsppmkkfr_15[15]\ : NOR3B
      port map(A => \jrdxqdwndbg_0[2]_net_1\, B => N_495, C => 
        \jrdxqdwndbg_0[3]_net_1\, Y => \hdfsppmkkfr[15]\);
    
    gsbzbrrsfpj : DFN1E1C0
      port map(D => N_745_i_0, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => un1_nvmzjtrjcbj, Q => ASSC_CHSAT_c);
    
    \zwnnqcvfccb_ns_i_1[5]\ : NOR2B
      port map(A => INIT_DONE_c, B => \zwnnqcvfccb[10]_net_1\, Y
         => \zwnnqcvfccb_ns_i_1[5]_net_1\);
    
    \ngwjhwffdrb[3]\ : DFN1E0C0
      port map(D => ASSC_RAM_DO_B_net_net_4, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => ASSC_CHLATD_c, Q => 
        \Z\\ADC_CHNR_net\\\);
    
    \zwnnqcvfccb_ns_i_0[2]\ : NOR2B
      port map(A => INIT_DONE_c, B => \zwnnqcvfccb[13]_net_1\, Y
         => \zwnnqcvfccb_ns_i_0[2]_net_1\);
    
    \zwnnqcvfccb_ns_i_i_a3[4]\ : NOR2A
      port map(A => INIT_DONE_c, B => ASSC_CHLATD_c, Y => 
        \zwnnqcvfccb_ns_i_i_a3[4]_net_1\);
    
    gsbzbrrsfpj_2_i_o3_0 : NOR2B
      port map(A => ADC_RESULT_net_net_8, B => 
        ADC_RESULT_net_net_7, Y => N_824);
    
    \hdfsppmkkfr_5[7]\ : MX2
      port map(A => ADC_RESULT_net_net_9, B => 
        ADC_RESULT_net_net_10, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_525);
    
    fkgscznxtsm_2 : NOR2
      port map(A => \fkgscznxtsm_2_0\, B => 
        un1_btxdzwbbqvw_1_sqmuxa, Y => \fkgscznxtsm_2\);
    
    \hdfsppmkkfr_5[13]\ : MX2
      port map(A => ADC_RESULT_net_net_3, B => 
        ADC_RESULT_net_net_4, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_531);
    
    \hdfsppmkkfr_10[7]\ : MX2
      port map(A => N_638, B => N_640, S => 
        \jrdxqdwndbg[1]_net_1\, Y => N_610);
    
    \zwnnqcvfccb_ns_i_a3[13]\ : AOI1
      port map(A => N_279_i_0, B => N_870, C => 
        \zwnnqcvfccb_ns_i_a3_0[13]_net_1\, Y => N_336);
    
    \zwnnqcvfccb[5]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i_1[9]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \zwnnqcvfccb[5]_net_1\);
    
    \dsjtwjhnpmm_i[6]\ : MX2
      port map(A => INIT_ADDR_6, B => EV_ASSCADDR_net_net_1, S
         => INIT_DONE_c, Y => dsjtwjhnpmm_i(6));
    
    un1_gjpxswjjjjj_I_24 : XOR2
      port map(A => \Z\\ASSC_SEQOUT_net\\_net_1\, B => 
        \DWACT_ADD_CI_0_g_array_2[0]\, Y => I_24);
    
    \jrdxqdwndbg[4]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_3, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ADC_START_net\, Q => 
        \jrdxqdwndbg[4]_net_1\);
    
    \zwnnqcvfccb_ns_i_0_o3[3]\ : OR2B
      port map(A => \zwnnqcvfccb[14]_net_1\, B => \qkzsmdfkvxj\, 
        Y => N_809);
    
    vhvdmsthqvg : DFN1E1C0
      port map(D => N_822, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        E => N_743, Q => ASSC_SAMPFLAG_net);
    
    \jqpswnvgwgd[5]\ : DFN1C0
      port map(D => \jqpswnvgwgd[5]_net_1\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \jqpswnvgwgd[5]_net_1\);
    
    un1_znktskmxcbp_2_i : AO1A
      port map(A => N_395, B => \bsrvxsspcfd\, C => 
        \un1_znktskmxcbp_2_i_0\, Y => N_740);
    
    \zbbgcrmtqtc_1[1]\ : OR3
      port map(A => \zwnnqcvfccb[10]_net_1\, B => 
        \zwnnqcvfccb[6]_net_1\, C => \zwnnqcvfccb[3]_net_1\, Y
         => \zbbgcrmtqtc_1[1]_net_1\);
    
    I_195 : OR2A
      port map(A => N_892, B => N_267, Y => N_395);
    
    \zwnnqcvfccb[14]\ : DFN1P0
      port map(D => \zwnnqcvfccb_ns[0]_net_1\, CLK => ACM_CLK_c, 
        PRE => sys_reset_gl, Q => \zwnnqcvfccb[14]_net_1\);
    
    prgqnmxpkfr_7_i : OA1
      port map(A => \prgqnmxpkfr\, B => \zwnnqcvfccb[12]_net_1\, 
        C => INIT_DONE_c, Y => \prgqnmxpkfr_7_i\);
    
    \hdfsppmkkfr_7[1]\ : NOR2A
      port map(A => N_485, B => \jrdxqdwndbg[2]_net_1\, Y => 
        N_553);
    
    \hdfsppmkkfr_10[6]\ : MX2
      port map(A => N_637, B => N_639, S => 
        \jrdxqdwndbg[1]_net_1\, Y => N_609);
    
    \hdfsppmkkfr_14[10]\ : NOR2A
      port map(A => N_613, B => \jrdxqdwndbg[2]_net_1\, Y => 
        N_681);
    
    gsbzbrrsfpj_2_i_3 : NOR3C
      port map(A => ADC_RESULT_net_net_3, B => 
        \gsbzbrrsfpj_2_i_1\, C => ADC_RESULT_net_net_2, Y => 
        \gsbzbrrsfpj_2_i_3\);
    
    \hdfsppmkkfr_15[0]\ : MX2
      port map(A => N_842, B => N_561, S => 
        \zwnnqcvfccb[3]_net_1\, Y => \hdfsppmkkfr[0]\);
    
    \hdfsppmkkfr_14[7]\ : MX2
      port map(A => N_487, B => N_614, S => 
        \jrdxqdwndbg_0[3]_net_1\, Y => N_678);
    
    \zbbgcrmtqtc[8]\ : NOR2B
      port map(A => \pdmrrrmtgwp[0]\, B => 
        \ASSC_SEQOUT_net[5]_net_1\, Y => \ASSC_RAM_ADDR_B_net[8]\);
    
    \hnfknqnpjhb[7]\ : DFN1E1C0
      port map(D => \Z\\ASSC_RAM_DO_B_net\\\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[12]_net_1\, Q => 
        \ADC_TVC_net[7]\);
    
    \hdfsppmkkfr_14[9]\ : NOR2A
      port map(A => N_612, B => \jrdxqdwndbg[2]_net_1\, Y => 
        N_680);
    
    \zwnnqcvfccb[0]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i[14]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \zwnnqcvfccb[0]_net_1\);
    
    \gjpxswjjjjj_4[4]\ : MX2
      port map(A => \jqpswnvgwgd[4]_net_1\, B => I_24, S => 
        N_394_i_0, Y => N_415);
    
    fkgscznxtsm : DFN1P0
      port map(D => \fkgscznxtsm_2\, CLK => ACM_CLK_c, PRE => 
        sys_reset_gl, Q => ADC_RESET_net);
    
    vpprztfztfv : DFN1C0
      port map(D => \bvzjfmgvnzp\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \vpprztfztfv\);
    
    un1_nvmzjtrjcbj_0_0_a3_2 : NOR2A
      port map(A => \ASSC_RAM_DO_B_net[8]\, B => N_274_i_0, Y => 
        N_862);
    
    \wrmchxtnrsq_o3[0]\ : OR2A
      port map(A => ASSC_CHLATD_c, B => \zwnnqcvfccb[6]_net_1\, Y
         => N_289);
    
    \hdfsppmkkfr_10[8]\ : MX2
      port map(A => N_639, B => N_641, S => 
        \jrdxqdwndbg[1]_net_1\, Y => N_611);
    
    \hdfsppmkkfr_14[2]\ : MX2
      port map(A => N_605, B => N_609, S => 
        \jrdxqdwndbg_0[2]_net_1\, Y => N_673);
    
    \ddzrbwxtmtz_1[6]\ : OR2A
      port map(A => ADC_RESULT_net_net_4, B => 
        \zwnnqcvfccb[5]_net_1\, Y => N_436);
    
    \cmdprjtzcrv[0]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_7, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[10]_net_1\, Q => 
        ADC_STC_net_net_6);
    
    un1_cstdbndzxqp : AO1A
      port map(A => N_395, B => \bsrvxsspcfd\, C => 
        gzmcjjcpmtm_5_sqmuxa, Y => \un1_cstdbndzxqp\);
    
    \ddzrbwxtmtz_1[0]\ : MX2C
      port map(A => ADC_RESULT_net_net_10, B => 
        ADC_RESULT_net_net_1, S => \zwnnqcvfccb[5]_net_1\, Y => 
        N_430);
    
    \hdfsppmkkfr_3[1]\ : NOR2A
      port map(A => N_525, B => \jrdxqdwndbg[1]_net_1\, Y => 
        N_485);
    
    \ASSC_CHLATD_c_i\ : INV
      port map(A => ASSC_CHLATD_c, Y => ASSC_CHLATD_c_i);
    
    un1_btxdzwbbqvw_1_sqmuxa_0_a3 : NOR2A
      port map(A => \zwnnqcvfccb[14]_net_1\, B => \qkzsmdfkvxj\, 
        Y => N_298);
    
    \ddzrbwxtmtz[7]\ : MX2C
      port map(A => N_437, B => N_426, S => N_439_i, Y => 
        \Z\\ASSC_RAM_DI_B_net\\\);
    
    thvjrqnxtqb_iv : OA1C
      port map(A => N_269, B => \prmfbzxmsxf\, C => 
        \thvjrqnxtqb_iv_0\, Y => thvjrqnxtqb);
    
    \hdfsppmkkfr_14[0]\ : MX2
      port map(A => N_603, B => N_607, S => 
        \jrdxqdwndbg_0[2]_net_1\, Y => N_671);
    
    fjvxxdqqwgk_1_sqmuxa_0_o3 : NOR2
      port map(A => \zwnnqcvfccb[3]_net_1\, B => 
        \zwnnqcvfccb[2]_net_1\, Y => N_267);
    
    un1_gzmcjjcpmtm_3_sqmuxa_1_0_a3_0_a3 : NOR2B
      port map(A => INIT_DONE_c, B => N_348, Y => N_841);
    
    \gjpxswjjjjj_4[5]\ : MX2
      port map(A => \jqpswnvgwgd[5]_net_1\, B => I_26, S => 
        N_394_i_0, Y => N_416);
    
    \dsjtwjhnpmm_i[5]\ : MX2
      port map(A => INIT_ADDR_int_0, B => EV_ASSCADDR_net_net_2, 
        S => INIT_DONE_c, Y => dsjtwjhnpmm_i(5));
    
    \zwnnqcvfccb[3]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i_1[11]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \zwnnqcvfccb[3]_net_1\);
    
    \hdfsppmkkfr_14[6]\ : MX2
      port map(A => N_486, B => N_613, S => 
        \jrdxqdwndbg[3]_net_1\, Y => N_677);
    
    \zwnnqcvfccb_ns_i_1[8]\ : NOR3B
      port map(A => \zwnnqcvfccb[7]_net_1\, B => INIT_DONE_c, C
         => ADC_BUSY_net, Y => \zwnnqcvfccb_ns_i_1[8]_net_1\);
    
    gvvstrjqmht_iv : OAI1
      port map(A => INIT_ASSC_WEN, B => INIT_DONE_c, C => 
        un1_Analog_RTC_assc_wrapper_inst, Y => ASSC_RAM_CSN_A_net);
    
    un1_nvmzjtrjcbj_0_0_a3 : OR2B
      port map(A => \zwnnqcvfccb[11]_net_1\, B => N_807_i, Y => 
        ASSC_CHLATD_c);
    
    \hdfsppmkkfr_15[1]\ : MX2
      port map(A => N_553, B => N_562, S => 
        \zwnnqcvfccb[3]_net_1\, Y => \hdfsppmkkfr[1]\);
    
    \hdfsppmkkfr_10[10]\ : NOR2A
      port map(A => N_641, B => \jrdxqdwndbg[1]_net_1\, Y => 
        N_613);
    
    \ddzrbwxtmtz[0]\ : MX2C
      port map(A => N_430, B => N_887, S => N_439_i, Y => 
        ASSC_RAM_DI_B_net_net_7);
    
    \gjpxswjjjjj_4_r[2]\ : NOR3C
      port map(A => \un4_cstdbndzxqp\, B => INIT_DONE_c, C => 
        N_413, Y => \gjpxswjjjjj_4[2]_net_1\);
    
    un1_gjpxswjjjjj_I_33 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_1[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_1[0]\, Y => 
        \DWACT_ADD_CI_0_g_array_2[0]\);
    
    \zwnnqcvfccb[4]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i_1[10]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \zwnnqcvfccb[4]_net_1\);
    
    un1_znktskmxcbp_1_i_o3 : AO1C
      port map(A => ASSC_RAM_DO_B_net_net_1, B => N_280_i_0, C
         => N_395, Y => gkwwhjwcbtz);
    
    \hdfsppmkkfr_15[12]\ : NOR2A
      port map(A => N_564, B => \jrdxqdwndbg[3]_net_1\, Y => 
        \hdfsppmkkfr[12]\);
    
    gsbzbrrsfpj_2_i_5 : NOR3C
      port map(A => ADC_RESULT_net_net_1, B => 
        \gsbzbrrsfpj_2_i_3\, C => \ADC_RESULT_net[11]\, Y => 
        \gsbzbrrsfpj_2_i_5\);
    
    \hdfsppmkkfr_9[1]\ : MX2
      port map(A => ADC_RESULT_net_net_7, B => 
        ADC_RESULT_net_net_6, S => \jrdxqdwndbg[0]_net_1\, Y => 
        N_587);
    
    \zwnnqcvfccb_ns_i_1[10]\ : NOR2B
      port map(A => INIT_DONE_c, B => \zwnnqcvfccb[5]_net_1\, Y
         => \zwnnqcvfccb_ns_i_1[10]_net_1\);
    
    \ddzrbwxtmtz_0[1]\ : MX2C
      port map(A => \hdfsppmkkfr[1]\, B => \hdfsppmkkfr[10]\, S
         => \jrdxqdwndbg_0[3]_net_1\, Y => N_888);
    
    \cmdprjtzcrv[4]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_3, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[10]_net_1\, Q => 
        ADC_STC_net_net_2);
    
    \hdfsppmkkfr_7[5]\ : MX2
      port map(A => N_489, B => N_608, S => 
        \jrdxqdwndbg_0[3]_net_1\, Y => N_557);
    
    \cmdprjtzcrv[2]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_5, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[10]_net_1\, Q => 
        ADC_STC_net_net_4);
    
    \hdfsppmkkfr_7[2]\ : NOR2A
      port map(A => N_486, B => \jrdxqdwndbg[2]_net_1\, Y => 
        N_554);
    
    \mcccbsrsbmn[2]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_2, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[13]_net_1\, Q => 
        \Z\\ADC_MODE_net\\\);
    
    \zwnnqcvfccb_ns_i_1[9]\ : NOR2B
      port map(A => INIT_DONE_c, B => \zwnnqcvfccb[6]_net_1\, Y
         => \zwnnqcvfccb_ns_i_1[9]_net_1\);
    
    \hdfsppmkkfr_3[7]\ : MX2
      port map(A => N_531, B => N_529, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_491);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    \dsjtwjhnpmm_i[3]\ : MX2
      port map(A => INIT_ADDR_3, B => EV_ASSCADDR_net_net_4, S
         => INIT_DONE_c, Y => dsjtwjhnpmm_i(3));
    
    drsbgqmzxjf_i : NOR3A
      port map(A => \drsbgqmzxjf_i_1\, B => \wrmchxtnrsq[2]\, C
         => N_289, Y => N_247_i_0);
    
    \zwnnqcvfccb[13]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i_0[1]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \zwnnqcvfccb[13]_net_1\);
    
    \hdfsppmkkfr_15[10]\ : MX2
      port map(A => N_672, B => N_681, S => 
        \zwnnqcvfccb[3]_net_1\, Y => \hdfsppmkkfr[10]\);
    
    \zwnnqcvfccb_ns_i[12]\ : NOR3A
      port map(A => INIT_DONE_c, B => N_267, C => N_892, Y => 
        \zwnnqcvfccb_ns_i[12]_net_1\);
    
    un1_nvmzjtrjcbj_0_0_a3_0 : NOR3
      port map(A => N_271_i_0, B => \un1_nvmzjtrjcbj_0_0_a3_0_0\, 
        C => N_870, Y => N_863);
    
    \hdfsppmkkfr_15[2]\ : MX2
      port map(A => N_554, B => N_673, S => 
        \jrdxqdwndbg_0[3]_net_1\, Y => \hdfsppmkkfr[2]\);
    
    un1_gjpxswjjjjj_I_15 : XOR2
      port map(A => \ASSC_SEQOUT_net_net_4\, B => 
        rhbzpsxhwms_net_1, Y => \DWACT_ADD_CI_0_partial_sum[0]\);
    
    \hdfsppmkkfr_7[7]\ : MX2
      port map(A => N_491, B => N_610, S => 
        \jrdxqdwndbg_0[3]_net_1\, Y => N_559);
    
    \hdfsppmkkfr_8[1]\ : MX2
      port map(A => ADC_RESULT_net_net_9, B => 
        ADC_RESULT_net_net_8, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_570);
    
    \hdfsppmkkfr_3[8]\ : MX2
      port map(A => N_532, B => N_530, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_492);
    
    gzmcjjcpmtm_5_sqmuxa_0_a3 : NOR2B
      port map(A => N_308_1, B => N_280_i_0, Y => 
        gzmcjjcpmtm_5_sqmuxa);
    
    \cmdprjtzcrv[1]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_6, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[10]_net_1\, Q => 
        ADC_STC_net_net_5);
    
    \dsjtwjhnpmm_i[4]\ : MX2
      port map(A => INIT_ADDR_4, B => EV_ASSCADDR_net_net_3, S
         => INIT_DONE_c, Y => dsjtwjhnpmm_i(4));
    
    \zwnnqcvfccb_ns_i_0[7]\ : AOI1
      port map(A => ADC_BUSY_net, B => \zwnnqcvfccb[7]_net_1\, C
         => \zwnnqcvfccb[8]_net_1\, Y => 
        \zwnnqcvfccb_ns_i_0[7]_net_1\);
    
    un1_mbqdggcpxtd_0_sqmuxa_0_o3 : OR2A
      port map(A => N_395, B => N_391, Y => N_269);
    
    un1_gzmcjjcpmtm_1_sqmuxa_1_0_a3 : NOR3C
      port map(A => INIT_DONE_c, B => N_308_1, C => N_279_i_0, Y
         => N_299);
    
    \ddzrbwxtmtz_0[7]\ : MX2C
      port map(A => \hdfsppmkkfr[7]\, B => \hdfsppmkkfr[16]\, S
         => \zwnnqcvfccb[3]_net_1\, Y => N_426);
    
    \hdfsppmkkfr_14[8]\ : NOR2A
      port map(A => N_611, B => \jrdxqdwndbg[2]_net_1\, Y => 
        N_679);
    
    fkgscznxtsm_2_0 : OR2A
      port map(A => INIT_DONE_c, B => \bvzjfmgvnzp\, Y => 
        \fkgscznxtsm_2_0\);
    
    \zwnnqcvfccb_ns_i[7]\ : NOR2A
      port map(A => INIT_DONE_c, B => 
        \zwnnqcvfccb_ns_i_0[7]_net_1\, Y => 
        \zwnnqcvfccb_ns_i[7]_net_1\);
    
    un4_cstdbndzxqp : OR3C
      port map(A => \un1_cstdbndzxqp\, B => INIT_DONE_c, C => 
        N_394_i_0, Y => \un4_cstdbndzxqp\);
    
    \jrdxqdwndbg[3]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_4, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ADC_START_net\, Q => 
        \jrdxqdwndbg[3]_net_1\);
    
    \zwnnqcvfccb_ns_0[0]\ : AOI1
      port map(A => \prmfbzxmsxf\, B => \zwnnqcvfccb[0]_net_1\, C
         => \zwnnqcvfccb[12]_net_1\, Y => 
        \zwnnqcvfccb_ns_0[0]_net_1\);
    
    \hdfsppmkkfr_7[4]\ : MX2
      port map(A => N_488, B => N_607, S => 
        \jrdxqdwndbg[3]_net_1\, Y => N_556);
    
    \ddzrbwxtmtz_0[3]\ : MX2C
      port map(A => \hdfsppmkkfr[3]\, B => \hdfsppmkkfr[12]\, S
         => \zwnnqcvfccb[3]_net_1\, Y => N_422);
    
    \zwnnqcvfccb[6]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i_1[8]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \zwnnqcvfccb[6]_net_1\);
    
    un1_znktskmxcbp_2_i_0 : AO1B
      port map(A => N_279_i_0, B => N_308_1, C => INIT_DONE_c, Y
         => \un1_znktskmxcbp_2_i_0\);
    
    \gjpxswjjjjj[0]\ : DFN1C0
      port map(D => \gjpxswjjjjj_4[0]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \ASSC_SEQOUT_net_net_4\);
    
    prmfbzxmsxf : DFN1C0
      port map(D => \prmfbzxmsxf_44\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \prmfbzxmsxf\);
    
    \ddzrbwxtmtz_1[5]\ : OR2A
      port map(A => ADC_RESULT_net_net_5, B => 
        \zwnnqcvfccb[5]_net_1\, Y => N_435);
    
    \ddzrbwxtmtz_0[2]\ : MX2C
      port map(A => \hdfsppmkkfr[2]\, B => \hdfsppmkkfr[11]\, S
         => \zwnnqcvfccb[3]_net_1\, Y => N_889);
    
    \cmdprjtzcrv[6]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_1, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[10]_net_1\, Q => 
        \Z\\ADC_STC_net\\\);
    
    un1_znktskmxcbp_9_0 : OR3B
      port map(A => mkcbzxzcrgq, B => INIT_DONE_c, C => N_348, Y
         => un1_znktskmxcbp_9);
    
    un1_btxdzwbbqvw_1_sqmuxa_0_o3_0_i_a3 : NOR2B
      port map(A => ASSC_RAM_DO_B_net_net_2, B => 
        ASSC_RAM_DO_B_net_net_1, Y => N_870);
    
    \hdfsppmkkfr_5[9]\ : MX2
      port map(A => ADC_RESULT_net_net_7, B => 
        ADC_RESULT_net_net_8, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_527);
    
    mkcbzxzcrgq_0_o3 : OR2
      port map(A => \zwnnqcvfccb[0]_net_1\, B => N_269, Y => 
        N_275);
    
    \ddzrbwxtmtz_1[3]\ : OR2A
      port map(A => ADC_RESULT_net_net_7, B => 
        \zwnnqcvfccb[5]_net_1\, Y => N_433);
    
    \zwnnqcvfccb[8]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i_1[6]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \zwnnqcvfccb[8]_net_1\);
    
    \jngrzvhtjvn_0\ : AOI1B
      port map(A => \un1_cstdbndzxqp\, B => INIT_DONE_c, C => 
        mkcbzxzcrgq, Y => jngrzvhtjvn_0);
    
    \hdfsppmkkfr_11[0]\ : MX2
      port map(A => ADC_RESULT_net_net_6, B => 
        ADC_RESULT_net_net_5, S => \jrdxqdwndbg[0]_net_1\, Y => 
        N_620);
    
    \hdfsppmkkfr_10[3]\ : MX2
      port map(A => N_587, B => N_621, S => 
        \jrdxqdwndbg[1]_net_1\, Y => N_606);
    
    \gjpxswjjjjj[2]\ : DFN1C0
      port map(D => \gjpxswjjjjj_4[2]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \ASSC_SEQOUT_net_net_2\);
    
    fkgscznxtsm_2_i : INV
      port map(A => \fkgscznxtsm_2\, Y => \fkgscznxtsm_2_i\);
    
    \gjpxswjjjjj_4[3]\ : MX2
      port map(A => \jqpswnvgwgd[3]_net_1\, B => I_23, S => 
        N_394_i_0, Y => N_414);
    
    \gjpxswjjjjj[4]\ : DFN1C0
      port map(D => \gjpxswjjjjj_4[4]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \Z\\ASSC_SEQOUT_net\\_net_1\);
    
    \rhbzpsxhwms\ : NOR3B
      port map(A => INIT_DONE_c, B => \jjqqfszbdzb\, C => 
        thvjrqnxtqb, Y => rhbzpsxhwms_net_1);
    
    \hnfknqnpjhb[6]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_1, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[12]_net_1\, Q => 
        \Z\\ADC_TVC_net\\\);
    
    \hdfsppmkkfr_10[0]\ : MX2
      port map(A => N_569, B => N_586, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_603);
    
    \ddzrbwxtmtz[2]\ : MX2C
      port map(A => N_432, B => N_889, S => N_439_i, Y => 
        ASSC_RAM_DI_B_net_net_5);
    
    \ngwjhwffdrb[1]\ : DFN1E0C0
      port map(D => ASSC_RAM_DO_B_net_net_6, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => ASSC_CHLATD_c, Q => 
        ADC_CHNR_net_net_2);
    
    \hdfsppmkkfr_14[1]\ : MX2
      port map(A => N_604, B => N_608, S => 
        \jrdxqdwndbg_0[2]_net_1\, Y => N_672);
    
    \ngwjhwffdrb[0]\ : DFN1E0C0
      port map(D => ASSC_RAM_DO_B_net_net_7, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => ASSC_CHLATD_c, Q => 
        ADC_CHNR_net_net_3);
    
    \hdfsppmkkfr_5_0_a3[6]\ : NOR2A
      port map(A => ADC_RESULT_net_net_10, B => 
        \jrdxqdwndbg[0]_net_1\, Y => N_866);
    
    \zbbgcrmtqtc[4]\ : NOR2B
      port map(A => \pdmrrrmtgwp[0]\, B => 
        \ASSC_SEQOUT_net_net_3\, Y => ASSC_RAM_ADDR_B_net_net_3);
    
    \jrdxqdwndbg[1]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_6, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ADC_START_net\, Q => 
        \jrdxqdwndbg[1]_net_1\);
    
    thvjrqnxtqb_iv_0 : AO1D
      port map(A => N_809, B => \jjqqfszbdzb\, C => 
        gzmcjjcpmtm_8_sqmuxa, Y => \thvjrqnxtqb_iv_0\);
    
    \zwnnqcvfccb[12]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i_0[2]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \zwnnqcvfccb[12]_net_1\);
    
    \zbbgcrmtqtc[5]\ : NOR2B
      port map(A => \pdmrrrmtgwp[0]\, B => 
        \ASSC_SEQOUT_net_net_2\, Y => ASSC_RAM_ADDR_B_net_net_2);
    
    bvzjfmgvnzp : DFN1C0
      port map(D => ADC_CALIBRATE_net, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \bvzjfmgvnzp\);
    
    \zbbgcrmtqtc[7]\ : NOR2B
      port map(A => \pdmrrrmtgwp[0]\, B => 
        \Z\\ASSC_SEQOUT_net\\_net_1\, Y => 
        \Z\\ASSC_RAM_ADDR_B_net\\\);
    
    \hdfsppmkkfr_7[11]\ : MX2
      port map(A => N_495, B => N_491, S => 
        \jrdxqdwndbg_0[2]_net_1\, Y => N_563);
    
    \cmdprjtzcrv[5]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_2, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[10]_net_1\, Q => 
        ADC_STC_net_net_1);
    
    mxgtgkgzknq_un6_qbfzhmnchpj : OR3A
      port map(A => INIT_DONE_c, B => EV_ASSCADDR_net_net_5, C
         => bskrjhjvqpf_1_sqmuxa, Y => 
        un1_Analog_RTC_assc_wrapper_inst);
    
    \dsjtwjhnpmm_i[7]\ : MX2
      port map(A => INIT_ADDR_int_2, B => \Z\\EV_ASSCADDR_net\\\, 
        S => INIT_DONE_c, Y => dsjtwjhnpmm_i(7));
    
    \zwnnqcvfccb_ns_i_a3_1_1[13]\ : NOR2A
      port map(A => \qkzsmdfkvxj\, B => \zwnnqcvfccb[11]_net_1\, 
        Y => N_338_1);
    
    \zwnnqcvfccb[9]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i_1[5]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => \ADC_START_net\);
    
    \hnfknqnpjhb[5]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_2, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[12]_net_1\, Q => 
        ADC_TVC_net_net_1);
    
    \hdfsppmkkfr_9[0]\ : MX2
      port map(A => ADC_RESULT_net_net_8, B => 
        ADC_RESULT_net_net_7, S => \jrdxqdwndbg[0]_net_1\, Y => 
        N_586);
    
    \gjpxswjjjjj[1]\ : DFN1C0
      port map(D => \gjpxswjjjjj_4[1]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \ASSC_SEQOUT_net_net_3\);
    
    \ddzrbwxtmtz_1[8]\ : OR2A
      port map(A => ADC_RESULT_net_net_2, B => 
        \zwnnqcvfccb[5]_net_1\, Y => N_438);
    
    \hdfsppmkkfr_3[11]\ : MX2
      port map(A => N_516, B => N_533, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_495);
    
    \gjpxswjjjjj_4_r[4]\ : NOR3C
      port map(A => \un4_cstdbndzxqp\, B => INIT_DONE_c, C => 
        N_415, Y => \gjpxswjjjjj_4[4]_net_1\);
    
    \dsjtwjhnpmm_i[2]\ : MX2
      port map(A => INIT_ADDR_2, B => EV_ASSCADDR_net_net_5, S
         => INIT_DONE_c, Y => dsjtwjhnpmm_i(2));
    
    \hdfsppmkkfr_3[2]\ : MX2
      port map(A => N_526, B => N_866, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_486);
    
    \hdfsppmkkfr_4[16]\ : NOR2B
      port map(A => \jrdxqdwndbg[0]_net_1\, B => 
        \ADC_RESULT_net[11]\, Y => N_517);
    
    \hdfsppmkkfr_3[4]\ : MX2
      port map(A => N_528, B => N_526, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_488);
    
    un1_nvmzjtrjcbj_0_0_o3 : OR3
      port map(A => N_863, B => N_861, C => N_862, Y => N_807_i);
    
    un1_btxdzwbbqvw_1_sqmuxa_0 : AOI1
      port map(A => N_279_i_0, B => N_870, C => N_298, Y => 
        un1_btxdzwbbqvw_1_sqmuxa);
    
    \jqpswnvgwgd[2]\ : DFN1C0
      port map(D => \jqpswnvgwgd[2]_net_1\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \jqpswnvgwgd[2]_net_1\);
    
    \zwnnqcvfccb[1]\ : DFN1C0
      port map(D => \zwnnqcvfccb_ns_i[13]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \zwnnqcvfccb[1]_net_1\);
    
    bsrvxsspcfd : DFN1E1C0
      port map(D => N_299, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        E => N_740, Q => \bsrvxsspcfd\);
    
    prmfbzxmsxf_44 : NOR3B
      port map(A => INIT_DONE_c, B => \prmfbzxmsxf\, C => N_275, 
        Y => \prmfbzxmsxf_44\);
    
    \ddzrbwxtmtz_0[4]\ : MX2C
      port map(A => \hdfsppmkkfr[4]\, B => \hdfsppmkkfr[13]\, S
         => \zwnnqcvfccb[3]_net_1\, Y => N_423);
    
    \zwnnqcvfccb_ns_2[0]\ : NOR3C
      port map(A => INIT_DONE_c, B => \zwnnqcvfccb_ns_0[0]_net_1\, 
        C => N_308_3, Y => \zwnnqcvfccb_ns_2[0]_net_1\);
    
    \hdfsppmkkfr_12[4]\ : MX2
      port map(A => \Z\\ADC_RESULT_net\\\, B => 
        \ADC_RESULT_net[11]\, S => \jrdxqdwndbg[0]_net_1\, Y => 
        N_641);
    
    un1_gjpxswjjjjj_I_26 : XOR2
      port map(A => \ASSC_SEQOUT_net[5]_net_1\, B => 
        \DWACT_ADD_CI_0_g_array_12_1[0]\, Y => I_26);
    
    \hdfsppmkkfr_15[8]\ : MX2
      port map(A => N_560, B => N_679, S => 
        \jrdxqdwndbg[3]_net_1\, Y => \hdfsppmkkfr[8]\);
    
    \jqpswnvgwgd[4]\ : DFN1C0
      port map(D => \jqpswnvgwgd[4]_net_1\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \jqpswnvgwgd[4]_net_1\);
    
    \hdfsppmkkfr_15[6]\ : MX2
      port map(A => N_558, B => N_677, S => 
        \jrdxqdwndbg_0[2]_net_1\, Y => \hdfsppmkkfr[6]\);
    
    un1_gjpxswjjjjj_I_23 : XOR2
      port map(A => \ASSC_SEQOUT_net_net_1\, B => 
        \DWACT_ADD_CI_0_g_array_12[0]\, Y => I_23);
    
    mkcbzxzcrgq_0_a3 : OR3C
      port map(A => INIT_DONE_c, B => \prmfbzxmsxf\, C => N_275, 
        Y => mkcbzxzcrgq);
    
    \hdfsppmkkfr_11[1]\ : MX2
      port map(A => ADC_RESULT_net_net_5, B => 
        ADC_RESULT_net_net_4, S => \jrdxqdwndbg[0]_net_1\, Y => 
        N_621);
    
    \ddzrbwxtmtz[1]\ : MX2C
      port map(A => N_431, B => N_888, S => N_439_i, Y => 
        ASSC_RAM_DI_B_net_net_6);
    
    qkzsmdfkvxj : DFN1E1C0
      port map(D => \fkgscznxtsm_2_i\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \un5_vpprztfztfv\, Q => \qkzsmdfkvxj\);
    
    \hdfsppmkkfr_8[0]\ : MX2
      port map(A => ADC_RESULT_net_net_10, B => 
        ADC_RESULT_net_net_9, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_569);
    
    \cmdprjtzcrv[3]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_4, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[10]_net_1\, Q => 
        ADC_STC_net_net_3);
    
    un1_gjpxswjjjjj_I_22 : XOR2
      port map(A => \ASSC_SEQOUT_net_net_2\, B => 
        \DWACT_ADD_CI_0_g_array_1[0]\, Y => I_22);
    
    \hdfsppmkkfr_10[1]\ : MX2
      port map(A => N_570, B => N_587, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_604);
    
    \hdfsppmkkfr_10[2]\ : MX2
      port map(A => N_586, B => N_620, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_605);
    
    \hnfknqnpjhb[4]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_3, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[12]_net_1\, Q => 
        ADC_TVC_net_net_2);
    
    \mcccbsrsbmn[1]\ : DFN1E1C0
      port map(D => ASSC_RAM_DO_B_net_net_3, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \zwnnqcvfccb[13]_net_1\, Q => 
        \ADC_MODE_net_net_1\);
    
    jjqqfszbdzb : DFN1C0
      port map(D => \jjqqfszbdzb_46\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \jjqqfszbdzb\);
    
    \hdfsppmkkfr_7[14]\ : MX2
      port map(A => N_498, B => N_494, S => 
        \jrdxqdwndbg_0[2]_net_1\, Y => N_566);
    
    N_271_i_0_a3 : NOR2
      port map(A => ASSC_RAM_DO_B_net_net_2, B => 
        ASSC_RAM_DO_B_net_net_1, Y => N_271_i_0);
    
    \hdfsppmkkfr_3[5]\ : MX2
      port map(A => N_529, B => N_527, S => 
        \jrdxqdwndbg_0[1]_net_1\, Y => N_489);
    
    \hdfsppmkkfr_5[11]\ : MX2
      port map(A => ADC_RESULT_net_net_5, B => 
        ADC_RESULT_net_net_6, S => \jrdxqdwndbg_0[0]_net_1\, Y
         => N_529);
    
    \hdfsppmkkfr_15[14]\ : NOR2A
      port map(A => N_566, B => \jrdxqdwndbg[3]_net_1\, Y => 
        \hdfsppmkkfr[14]\);
    
    \hdfsppmkkfr_14[5]\ : MX2
      port map(A => N_485, B => N_612, S => 
        \jrdxqdwndbg_0[3]_net_1\, Y => N_676);
    
    \hdfsppmkkfr_7[6]\ : MX2
      port map(A => N_490, B => N_609, S => 
        \jrdxqdwndbg[3]_net_1\, Y => N_558);
    
    \ddzrbwxtmtz_1[2]\ : MX2C
      port map(A => ADC_RESULT_net_net_8, B => 
        \ADC_RESULT_net[11]\, S => \zwnnqcvfccb[5]_net_1\, Y => 
        N_432);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity ASSC is

    port(dsjtwjhnpmm_i : out std_logic_vector(8 downto 0); 
        \ASSC_RAM_DO_B_net[8]\, \ADC_RESULT_net[11]\ : in
         std_logic; \ASSC_RAM_DI_B_net[8]\ : out std_logic; 
        INIT_ADDR_0, INIT_ADDR_1, INIT_ADDR_6, INIT_ADDR_4, 
        INIT_ADDR_2, INIT_ADDR_3, INIT_ADDR_int_3, 
        INIT_ADDR_int_2, INIT_ADDR_int_0, \EV_ASSCADDR_net[8]\ : 
        in std_logic; \ASSC_RAM_ADDR_B_net[8]\, 
        \ASSC_SEQOUT_net[5]\, \tkcbxxqbtfx_ns[11]\, 
        \ADC_CHNR_net[4]\, \ADC_STC_net[7]\, \ADC_TVC_net[7]\, 
        \ADC_MODE_net[3]\ : out std_logic; ACM_CLK_c, 
        sys_reset_gl, ASSC_RAM_DO_B_net_net_7 : in std_logic; 
        ADC_START_net : out std_logic; ASSC_RAM_DO_B_net_net_6, 
        ASSC_RAM_DO_B_net_net_5, ASSC_RAM_DO_B_net_net_4 : in
         std_logic; ASSC_SEQOUT_net_net_3, \Z\\ASSC_SEQOUT_net\\\, 
        ASSC_SEQOUT_net_net_2, ASSC_CHLATD_c_i, jngrzvhtjvn_0 : 
        out std_logic; INIT_DONE_c, ADC_BUSY_net, 
        \Z\\ASSC_RAM_DO_B_net\\\ : in std_logic; 
        ADC_MODE_net_net_1 : out std_logic; \Z\\ADC_RESULT_net\\\, 
        ADC_RESULT_net_net_1, ADC_RESULT_net_net_3, 
        ADC_RESULT_net_net_2, ADC_RESULT_net_net_5, 
        ADC_RESULT_net_net_6, ADC_RESULT_net_net_4, TR_TRFLAG_net, 
        TR_DONE_net, EV_EVFLAG_net : in std_logic; N_247_i_0 : 
        out std_logic; EV_ASSCADDR_net_net_5, 
        bskrjhjvqpf_1_sqmuxa : in std_logic; 
        ASSC_RAM_ADDR_B_net_net_6, ASSC_RAM_ADDR_B_net_net_7 : 
        out std_logic; EV_DONE_net_0, ADC_RESULT_net_net_10, 
        ADC_RESULT_net_net_9, ADC_RESULT_net_net_8 : in std_logic; 
        ASSC_RAM_DI_B_net_net_7, ASSC_RAM_DI_B_net_net_6, 
        ASSC_RAM_DI_B_net_net_5, ASSC_RAM_DI_B_net_net_4, 
        ASSC_RAM_DI_B_net_net_2, ASSC_RAM_DI_B_net_net_1, 
        \Z\\ASSC_RAM_DI_B_net\\\ : out std_logic; 
        ADC_RESULT_net_net_7 : in std_logic; ADC_MODE_net_net_2 : 
        out std_logic; EV_ASSCADDR_net_net_1, 
        EV_ASSCADDR_net_net_3 : in std_logic; ttxwkrvqgng_i_a3 : 
        out std_logic; INIT_ASSC_WEN : in std_logic; 
        ASSC_RAM_CSN_A_net : out std_logic; 
        ASSC_RAM_DO_B_net_net_2, ASSC_RAM_DO_B_net_net_1, 
        \Z\\EV_ASSCADDR_net\\\, EV_ASSCADDR_net_net_4 : in
         std_logic; ASSC_RAM_ADDR_B_net_net_5, 
        \Z\\ASSC_RAM_ADDR_B_net\\\, ASSC_RAM_ADDR_B_net_net_1, 
        ASSC_SEQOUT_net_net_1, ASSC_RAM_ADDR_B_net_net_2, 
        ASSC_RAM_ADDR_B_net_net_3, ASSC_RAM_ADDR_B_net_net_4, 
        ASSC_SEQOUT_net_net_4 : out std_logic; 
        EV_ASSCADDR_net_net_2 : in std_logic; rhbzpsxhwms, 
        ASSC_RAM_RW_B_net, ASSC_DONE_c, ASSC_RAM_DI_B_net_net_3 : 
        out std_logic; ADC_CALIBRATE_net : in std_logic; 
        ADC_RESET_net, ADC_PDOWN_net, ASSC_CHSAT_c, 
        ASSC_SAMPFLAG_net, ADC_VREFSEL_net, ADC_CHNR_net_net_3, 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_1, 
        \Z\\ADC_CHNR_net\\\ : out std_logic; 
        ASSC_RAM_DO_B_net_net_3 : in std_logic; ADC_STC_net_net_6, 
        ADC_STC_net_net_5, ADC_STC_net_net_4, ADC_STC_net_net_3, 
        ADC_STC_net_net_2, ADC_STC_net_net_1, \Z\\ADC_STC_net\\\, 
        ADC_TVC_net_net_6, ADC_TVC_net_net_5, ADC_TVC_net_net_4, 
        ADC_TVC_net_net_3, ADC_TVC_net_net_2, ADC_TVC_net_net_1, 
        \Z\\ADC_TVC_net\\\, \Z\\ADC_MODE_net\\\ : out std_logic);

        
end ASSC;

architecture DEF_ARCH of ASSC is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component cwgmnvwgkwq
    port(dsjtwjhnpmm_i : out std_logic_vector(8 downto 0); 
        \ADC_MODE_net[3]\, \ADC_TVC_net[7]\, \ADC_STC_net[7]\, 
        \ADC_CHNR_net[4]\, \tkcbxxqbtfx_ns[11]\, 
        \ASSC_SEQOUT_net[5]\, \ASSC_RAM_ADDR_B_net[8]\ : out
         std_logic; \EV_ASSCADDR_net[8]\, INIT_ADDR_int_0, 
        INIT_ADDR_int_2, INIT_ADDR_int_3, INIT_ADDR_3, 
        INIT_ADDR_2, INIT_ADDR_4, INIT_ADDR_6, INIT_ADDR_1, 
        INIT_ADDR_0 : in std_logic := 'U'; \ASSC_RAM_DI_B_net[8]\ : 
        out std_logic; \ADC_RESULT_net[11]\, 
        \ASSC_RAM_DO_B_net[8]\ : in std_logic := 'U'; 
        \Z\\ADC_MODE_net\\\, \Z\\ADC_TVC_net\\\, 
        ADC_TVC_net_net_1, ADC_TVC_net_net_2, ADC_TVC_net_net_3, 
        ADC_TVC_net_net_4, ADC_TVC_net_net_5, ADC_TVC_net_net_6, 
        \Z\\ADC_STC_net\\\, ADC_STC_net_net_1, ADC_STC_net_net_2, 
        ADC_STC_net_net_3, ADC_STC_net_net_4, ADC_STC_net_net_5, 
        ADC_STC_net_net_6 : out std_logic; 
        ASSC_RAM_DO_B_net_net_3 : in std_logic := 'U'; 
        \Z\\ADC_CHNR_net\\\, ADC_CHNR_net_net_1, 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_3, ADC_VREFSEL_net, 
        ASSC_SAMPFLAG_net, ASSC_CHSAT_c, ADC_PDOWN_net, 
        ADC_RESET_net : out std_logic; ADC_CALIBRATE_net : in
         std_logic := 'U'; ASSC_RAM_DI_B_net_net_3, ASSC_DONE_c, 
        ASSC_RAM_RW_B_net, rhbzpsxhwms : out std_logic; 
        EV_ASSCADDR_net_net_2 : in std_logic := 'U'; 
        ASSC_SEQOUT_net_net_4, ASSC_RAM_ADDR_B_net_net_4, 
        ASSC_RAM_ADDR_B_net_net_3, ASSC_RAM_ADDR_B_net_net_2, 
        ASSC_SEQOUT_net_net_1, ASSC_RAM_ADDR_B_net_net_1, 
        \Z\\ASSC_RAM_ADDR_B_net\\\, ASSC_RAM_ADDR_B_net_net_5 : 
        out std_logic; EV_ASSCADDR_net_net_4, 
        \Z\\EV_ASSCADDR_net\\\, ASSC_RAM_DO_B_net_net_1, 
        ASSC_RAM_DO_B_net_net_2 : in std_logic := 'U'; 
        ASSC_RAM_CSN_A_net : out std_logic; INIT_ASSC_WEN : in
         std_logic := 'U'; ttxwkrvqgng_i_a3 : out std_logic; 
        EV_ASSCADDR_net_net_3, EV_ASSCADDR_net_net_1 : in
         std_logic := 'U'; ADC_MODE_net_net_2 : out std_logic; 
        ADC_RESULT_net_net_7 : in std_logic := 'U'; 
        \Z\\ASSC_RAM_DI_B_net\\\, ASSC_RAM_DI_B_net_net_1, 
        ASSC_RAM_DI_B_net_net_2, ASSC_RAM_DI_B_net_net_4, 
        ASSC_RAM_DI_B_net_net_5, ASSC_RAM_DI_B_net_net_6, 
        ASSC_RAM_DI_B_net_net_7 : out std_logic; 
        ADC_RESULT_net_net_8, ADC_RESULT_net_net_9, 
        ADC_RESULT_net_net_10, EV_DONE_net_0 : in
         std_logic := 'U'; ASSC_RAM_ADDR_B_net_net_7, 
        ASSC_RAM_ADDR_B_net_net_6 : out std_logic; 
        bskrjhjvqpf_1_sqmuxa, EV_ASSCADDR_net_net_5 : in
         std_logic := 'U'; N_247_i_0 : out std_logic; 
        EV_EVFLAG_net, TR_DONE_net, TR_TRFLAG_net, 
        ADC_RESULT_net_net_4, ADC_RESULT_net_net_6, 
        ADC_RESULT_net_net_5, ADC_RESULT_net_net_2, 
        ADC_RESULT_net_net_3, ADC_RESULT_net_net_1, 
        \Z\\ADC_RESULT_net\\\ : in std_logic := 'U'; 
        ADC_MODE_net_net_1 : out std_logic; 
        \Z\\ASSC_RAM_DO_B_net\\\, ADC_BUSY_net, INIT_DONE_c : in
         std_logic := 'U'; jngrzvhtjvn_0, ASSC_CHLATD_c_i, 
        ASSC_SEQOUT_net_net_2, \Z\\ASSC_SEQOUT_net\\\, 
        ASSC_SEQOUT_net_net_3 : out std_logic; 
        ASSC_RAM_DO_B_net_net_4, ASSC_RAM_DO_B_net_net_5, 
        ASSC_RAM_DO_B_net_net_6 : in std_logic := 'U'; 
        ADC_START_net : out std_logic; ASSC_RAM_DO_B_net_net_7, 
        sys_reset_gl, ACM_CLK_c : in std_logic := 'U');
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : cwgmnvwgkwq
	Use entity work.cwgmnvwgkwq(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    u_cwgmnvwgkwq : cwgmnvwgkwq
      port map(dsjtwjhnpmm_i(8) => dsjtwjhnpmm_i(8), 
        dsjtwjhnpmm_i(7) => dsjtwjhnpmm_i(7), dsjtwjhnpmm_i(6)
         => dsjtwjhnpmm_i(6), dsjtwjhnpmm_i(5) => 
        dsjtwjhnpmm_i(5), dsjtwjhnpmm_i(4) => dsjtwjhnpmm_i(4), 
        dsjtwjhnpmm_i(3) => dsjtwjhnpmm_i(3), dsjtwjhnpmm_i(2)
         => dsjtwjhnpmm_i(2), dsjtwjhnpmm_i(1) => 
        dsjtwjhnpmm_i(1), dsjtwjhnpmm_i(0) => dsjtwjhnpmm_i(0), 
        \ADC_MODE_net[3]\ => \ADC_MODE_net[3]\, \ADC_TVC_net[7]\
         => \ADC_TVC_net[7]\, \ADC_STC_net[7]\ => 
        \ADC_STC_net[7]\, \ADC_CHNR_net[4]\ => \ADC_CHNR_net[4]\, 
        \tkcbxxqbtfx_ns[11]\ => \tkcbxxqbtfx_ns[11]\, 
        \ASSC_SEQOUT_net[5]\ => \ASSC_SEQOUT_net[5]\, 
        \ASSC_RAM_ADDR_B_net[8]\ => \ASSC_RAM_ADDR_B_net[8]\, 
        \EV_ASSCADDR_net[8]\ => \EV_ASSCADDR_net[8]\, 
        INIT_ADDR_int_0 => INIT_ADDR_int_0, INIT_ADDR_int_2 => 
        INIT_ADDR_int_2, INIT_ADDR_int_3 => INIT_ADDR_int_3, 
        INIT_ADDR_3 => INIT_ADDR_3, INIT_ADDR_2 => INIT_ADDR_2, 
        INIT_ADDR_4 => INIT_ADDR_4, INIT_ADDR_6 => INIT_ADDR_6, 
        INIT_ADDR_1 => INIT_ADDR_1, INIT_ADDR_0 => INIT_ADDR_0, 
        \ASSC_RAM_DI_B_net[8]\ => \ASSC_RAM_DI_B_net[8]\, 
        \ADC_RESULT_net[11]\ => \ADC_RESULT_net[11]\, 
        \ASSC_RAM_DO_B_net[8]\ => \ASSC_RAM_DO_B_net[8]\, 
        \Z\\ADC_MODE_net\\\ => \Z\\ADC_MODE_net\\\, 
        \Z\\ADC_TVC_net\\\ => \Z\\ADC_TVC_net\\\, 
        ADC_TVC_net_net_1 => ADC_TVC_net_net_1, ADC_TVC_net_net_2
         => ADC_TVC_net_net_2, ADC_TVC_net_net_3 => 
        ADC_TVC_net_net_3, ADC_TVC_net_net_4 => ADC_TVC_net_net_4, 
        ADC_TVC_net_net_5 => ADC_TVC_net_net_5, ADC_TVC_net_net_6
         => ADC_TVC_net_net_6, \Z\\ADC_STC_net\\\ => 
        \Z\\ADC_STC_net\\\, ADC_STC_net_net_1 => 
        ADC_STC_net_net_1, ADC_STC_net_net_2 => ADC_STC_net_net_2, 
        ADC_STC_net_net_3 => ADC_STC_net_net_3, ADC_STC_net_net_4
         => ADC_STC_net_net_4, ADC_STC_net_net_5 => 
        ADC_STC_net_net_5, ADC_STC_net_net_6 => ADC_STC_net_net_6, 
        ASSC_RAM_DO_B_net_net_3 => ASSC_RAM_DO_B_net_net_3, 
        \Z\\ADC_CHNR_net\\\ => \Z\\ADC_CHNR_net\\\, 
        ADC_CHNR_net_net_1 => ADC_CHNR_net_net_1, 
        ADC_CHNR_net_net_2 => ADC_CHNR_net_net_2, 
        ADC_CHNR_net_net_3 => ADC_CHNR_net_net_3, ADC_VREFSEL_net
         => ADC_VREFSEL_net, ASSC_SAMPFLAG_net => 
        ASSC_SAMPFLAG_net, ASSC_CHSAT_c => ASSC_CHSAT_c, 
        ADC_PDOWN_net => ADC_PDOWN_net, ADC_RESET_net => 
        ADC_RESET_net, ADC_CALIBRATE_net => ADC_CALIBRATE_net, 
        ASSC_RAM_DI_B_net_net_3 => ASSC_RAM_DI_B_net_net_3, 
        ASSC_DONE_c => ASSC_DONE_c, ASSC_RAM_RW_B_net => 
        ASSC_RAM_RW_B_net, rhbzpsxhwms => rhbzpsxhwms, 
        EV_ASSCADDR_net_net_2 => EV_ASSCADDR_net_net_2, 
        ASSC_SEQOUT_net_net_4 => ASSC_SEQOUT_net_net_4, 
        ASSC_RAM_ADDR_B_net_net_4 => ASSC_RAM_ADDR_B_net_net_4, 
        ASSC_RAM_ADDR_B_net_net_3 => ASSC_RAM_ADDR_B_net_net_3, 
        ASSC_RAM_ADDR_B_net_net_2 => ASSC_RAM_ADDR_B_net_net_2, 
        ASSC_SEQOUT_net_net_1 => ASSC_SEQOUT_net_net_1, 
        ASSC_RAM_ADDR_B_net_net_1 => ASSC_RAM_ADDR_B_net_net_1, 
        \Z\\ASSC_RAM_ADDR_B_net\\\ => \Z\\ASSC_RAM_ADDR_B_net\\\, 
        ASSC_RAM_ADDR_B_net_net_5 => ASSC_RAM_ADDR_B_net_net_5, 
        EV_ASSCADDR_net_net_4 => EV_ASSCADDR_net_net_4, 
        \Z\\EV_ASSCADDR_net\\\ => \Z\\EV_ASSCADDR_net\\\, 
        ASSC_RAM_DO_B_net_net_1 => ASSC_RAM_DO_B_net_net_1, 
        ASSC_RAM_DO_B_net_net_2 => ASSC_RAM_DO_B_net_net_2, 
        ASSC_RAM_CSN_A_net => ASSC_RAM_CSN_A_net, INIT_ASSC_WEN
         => INIT_ASSC_WEN, ttxwkrvqgng_i_a3 => ttxwkrvqgng_i_a3, 
        EV_ASSCADDR_net_net_3 => EV_ASSCADDR_net_net_3, 
        EV_ASSCADDR_net_net_1 => EV_ASSCADDR_net_net_1, 
        ADC_MODE_net_net_2 => ADC_MODE_net_net_2, 
        ADC_RESULT_net_net_7 => ADC_RESULT_net_net_7, 
        \Z\\ASSC_RAM_DI_B_net\\\ => \Z\\ASSC_RAM_DI_B_net\\\, 
        ASSC_RAM_DI_B_net_net_1 => ASSC_RAM_DI_B_net_net_1, 
        ASSC_RAM_DI_B_net_net_2 => ASSC_RAM_DI_B_net_net_2, 
        ASSC_RAM_DI_B_net_net_4 => ASSC_RAM_DI_B_net_net_4, 
        ASSC_RAM_DI_B_net_net_5 => ASSC_RAM_DI_B_net_net_5, 
        ASSC_RAM_DI_B_net_net_6 => ASSC_RAM_DI_B_net_net_6, 
        ASSC_RAM_DI_B_net_net_7 => ASSC_RAM_DI_B_net_net_7, 
        ADC_RESULT_net_net_8 => ADC_RESULT_net_net_8, 
        ADC_RESULT_net_net_9 => ADC_RESULT_net_net_9, 
        ADC_RESULT_net_net_10 => ADC_RESULT_net_net_10, 
        EV_DONE_net_0 => EV_DONE_net_0, ASSC_RAM_ADDR_B_net_net_7
         => ASSC_RAM_ADDR_B_net_net_7, ASSC_RAM_ADDR_B_net_net_6
         => ASSC_RAM_ADDR_B_net_net_6, bskrjhjvqpf_1_sqmuxa => 
        bskrjhjvqpf_1_sqmuxa, EV_ASSCADDR_net_net_5 => 
        EV_ASSCADDR_net_net_5, N_247_i_0 => N_247_i_0, 
        EV_EVFLAG_net => EV_EVFLAG_net, TR_DONE_net => 
        TR_DONE_net, TR_TRFLAG_net => TR_TRFLAG_net, 
        ADC_RESULT_net_net_4 => ADC_RESULT_net_net_4, 
        ADC_RESULT_net_net_6 => ADC_RESULT_net_net_6, 
        ADC_RESULT_net_net_5 => ADC_RESULT_net_net_5, 
        ADC_RESULT_net_net_2 => ADC_RESULT_net_net_2, 
        ADC_RESULT_net_net_3 => ADC_RESULT_net_net_3, 
        ADC_RESULT_net_net_1 => ADC_RESULT_net_net_1, 
        \Z\\ADC_RESULT_net\\\ => \Z\\ADC_RESULT_net\\\, 
        ADC_MODE_net_net_1 => ADC_MODE_net_net_1, 
        \Z\\ASSC_RAM_DO_B_net\\\ => \Z\\ASSC_RAM_DO_B_net\\\, 
        ADC_BUSY_net => ADC_BUSY_net, INIT_DONE_c => INIT_DONE_c, 
        jngrzvhtjvn_0 => jngrzvhtjvn_0, ASSC_CHLATD_c_i => 
        ASSC_CHLATD_c_i, ASSC_SEQOUT_net_net_2 => 
        ASSC_SEQOUT_net_net_2, \Z\\ASSC_SEQOUT_net\\\ => 
        \Z\\ASSC_SEQOUT_net\\\, ASSC_SEQOUT_net_net_3 => 
        ASSC_SEQOUT_net_net_3, ASSC_RAM_DO_B_net_net_4 => 
        ASSC_RAM_DO_B_net_net_4, ASSC_RAM_DO_B_net_net_5 => 
        ASSC_RAM_DO_B_net_net_5, ASSC_RAM_DO_B_net_net_6 => 
        ASSC_RAM_DO_B_net_net_6, ADC_START_net => ADC_START_net, 
        ASSC_RAM_DO_B_net_net_7 => ASSC_RAM_DO_B_net_net_7, 
        sys_reset_gl => sys_reset_gl, ACM_CLK_c => ACM_CLK_c);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity Analog_RTC_assc_wrapper is

    port(dsjtwjhnpmm_i : out std_logic_vector(8 downto 0); 
        \ADC_MODE_net[3]\, \ADC_TVC_net[7]\, \ADC_STC_net[7]\, 
        \ADC_CHNR_net[4]\, \tkcbxxqbtfx_ns[11]\, 
        \ASSC_SEQOUT_net[5]\, \ASSC_RAM_ADDR_B_net[8]\ : out
         std_logic; \EV_ASSCADDR_net[8]\, INIT_ADDR_int_0, 
        INIT_ADDR_int_2, INIT_ADDR_int_3, INIT_ADDR_3, 
        INIT_ADDR_2, INIT_ADDR_4, INIT_ADDR_6, INIT_ADDR_1, 
        INIT_ADDR_0 : in std_logic; \ASSC_RAM_DI_B_net[8]\ : out
         std_logic; \ADC_RESULT_net[11]\, \ASSC_RAM_DO_B_net[8]\ : 
        in std_logic; \Z\\ADC_MODE_net\\\, \Z\\ADC_TVC_net\\\, 
        ADC_TVC_net_net_1, ADC_TVC_net_net_2, ADC_TVC_net_net_3, 
        ADC_TVC_net_net_4, ADC_TVC_net_net_5, ADC_TVC_net_net_6, 
        \Z\\ADC_STC_net\\\, ADC_STC_net_net_1, ADC_STC_net_net_2, 
        ADC_STC_net_net_3, ADC_STC_net_net_4, ADC_STC_net_net_5, 
        ADC_STC_net_net_6 : out std_logic; 
        ASSC_RAM_DO_B_net_net_3 : in std_logic; 
        \Z\\ADC_CHNR_net\\\, ADC_CHNR_net_net_1, 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_3, ADC_VREFSEL_net, 
        ASSC_SAMPFLAG_net, ASSC_CHSAT_c, ADC_PDOWN_net, 
        ADC_RESET_net : out std_logic; ADC_CALIBRATE_net : in
         std_logic; ASSC_RAM_DI_B_net_net_3, ASSC_DONE_c, 
        ASSC_RAM_RW_B_net, rhbzpsxhwms : out std_logic; 
        EV_ASSCADDR_net_net_2 : in std_logic; 
        ASSC_SEQOUT_net_net_4, ASSC_RAM_ADDR_B_net_net_4, 
        ASSC_RAM_ADDR_B_net_net_3, ASSC_RAM_ADDR_B_net_net_2, 
        ASSC_SEQOUT_net_net_1, ASSC_RAM_ADDR_B_net_net_1, 
        \Z\\ASSC_RAM_ADDR_B_net\\\, ASSC_RAM_ADDR_B_net_net_5 : 
        out std_logic; EV_ASSCADDR_net_net_4, 
        \Z\\EV_ASSCADDR_net\\\, ASSC_RAM_DO_B_net_net_1, 
        ASSC_RAM_DO_B_net_net_2 : in std_logic; 
        ASSC_RAM_CSN_A_net : out std_logic; INIT_ASSC_WEN : in
         std_logic; ttxwkrvqgng_i_a3 : out std_logic; 
        EV_ASSCADDR_net_net_3, EV_ASSCADDR_net_net_1 : in
         std_logic; ADC_MODE_net_net_2 : out std_logic; 
        ADC_RESULT_net_net_7 : in std_logic; 
        \Z\\ASSC_RAM_DI_B_net\\\, ASSC_RAM_DI_B_net_net_1, 
        ASSC_RAM_DI_B_net_net_2, ASSC_RAM_DI_B_net_net_4, 
        ASSC_RAM_DI_B_net_net_5, ASSC_RAM_DI_B_net_net_6, 
        ASSC_RAM_DI_B_net_net_7 : out std_logic; 
        ADC_RESULT_net_net_8, ADC_RESULT_net_net_9, 
        ADC_RESULT_net_net_10, EV_DONE_net_0 : in std_logic; 
        ASSC_RAM_ADDR_B_net_net_7, ASSC_RAM_ADDR_B_net_net_6 : 
        out std_logic; bskrjhjvqpf_1_sqmuxa, 
        EV_ASSCADDR_net_net_5 : in std_logic; N_247_i_0 : out
         std_logic; EV_EVFLAG_net, TR_DONE_net, TR_TRFLAG_net, 
        ADC_RESULT_net_net_4, ADC_RESULT_net_net_6, 
        ADC_RESULT_net_net_5, ADC_RESULT_net_net_2, 
        ADC_RESULT_net_net_3, ADC_RESULT_net_net_1, 
        \Z\\ADC_RESULT_net\\\ : in std_logic; ADC_MODE_net_net_1 : 
        out std_logic; \Z\\ASSC_RAM_DO_B_net\\\, ADC_BUSY_net, 
        INIT_DONE_c : in std_logic; jngrzvhtjvn_0, 
        ASSC_CHLATD_c_i, ASSC_SEQOUT_net_net_2, 
        \Z\\ASSC_SEQOUT_net\\\, ASSC_SEQOUT_net_net_3 : out
         std_logic; ASSC_RAM_DO_B_net_net_4, 
        ASSC_RAM_DO_B_net_net_5, ASSC_RAM_DO_B_net_net_6 : in
         std_logic; ADC_START_net : out std_logic; 
        ASSC_RAM_DO_B_net_net_7, sys_reset_gl, ACM_CLK_c : in
         std_logic);

end Analog_RTC_assc_wrapper;

architecture DEF_ARCH of Analog_RTC_assc_wrapper is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component ASSC
    port(dsjtwjhnpmm_i : out std_logic_vector(8 downto 0); 
        \ASSC_RAM_DO_B_net[8]\, \ADC_RESULT_net[11]\ : in
         std_logic := 'U'; \ASSC_RAM_DI_B_net[8]\ : out std_logic; 
        INIT_ADDR_0, INIT_ADDR_1, INIT_ADDR_6, INIT_ADDR_4, 
        INIT_ADDR_2, INIT_ADDR_3, INIT_ADDR_int_3, 
        INIT_ADDR_int_2, INIT_ADDR_int_0, \EV_ASSCADDR_net[8]\ : 
        in std_logic := 'U'; \ASSC_RAM_ADDR_B_net[8]\, 
        \ASSC_SEQOUT_net[5]\, \tkcbxxqbtfx_ns[11]\, 
        \ADC_CHNR_net[4]\, \ADC_STC_net[7]\, \ADC_TVC_net[7]\, 
        \ADC_MODE_net[3]\ : out std_logic; ACM_CLK_c, 
        sys_reset_gl, ASSC_RAM_DO_B_net_net_7 : in
         std_logic := 'U'; ADC_START_net : out std_logic; 
        ASSC_RAM_DO_B_net_net_6, ASSC_RAM_DO_B_net_net_5, 
        ASSC_RAM_DO_B_net_net_4 : in std_logic := 'U'; 
        ASSC_SEQOUT_net_net_3, \Z\\ASSC_SEQOUT_net\\\, 
        ASSC_SEQOUT_net_net_2, ASSC_CHLATD_c_i, jngrzvhtjvn_0 : 
        out std_logic; INIT_DONE_c, ADC_BUSY_net, 
        \Z\\ASSC_RAM_DO_B_net\\\ : in std_logic := 'U'; 
        ADC_MODE_net_net_1 : out std_logic; \Z\\ADC_RESULT_net\\\, 
        ADC_RESULT_net_net_1, ADC_RESULT_net_net_3, 
        ADC_RESULT_net_net_2, ADC_RESULT_net_net_5, 
        ADC_RESULT_net_net_6, ADC_RESULT_net_net_4, TR_TRFLAG_net, 
        TR_DONE_net, EV_EVFLAG_net : in std_logic := 'U'; 
        N_247_i_0 : out std_logic; EV_ASSCADDR_net_net_5, 
        bskrjhjvqpf_1_sqmuxa : in std_logic := 'U'; 
        ASSC_RAM_ADDR_B_net_net_6, ASSC_RAM_ADDR_B_net_net_7 : 
        out std_logic; EV_DONE_net_0, ADC_RESULT_net_net_10, 
        ADC_RESULT_net_net_9, ADC_RESULT_net_net_8 : in
         std_logic := 'U'; ASSC_RAM_DI_B_net_net_7, 
        ASSC_RAM_DI_B_net_net_6, ASSC_RAM_DI_B_net_net_5, 
        ASSC_RAM_DI_B_net_net_4, ASSC_RAM_DI_B_net_net_2, 
        ASSC_RAM_DI_B_net_net_1, \Z\\ASSC_RAM_DI_B_net\\\ : out
         std_logic; ADC_RESULT_net_net_7 : in std_logic := 'U'; 
        ADC_MODE_net_net_2 : out std_logic; EV_ASSCADDR_net_net_1, 
        EV_ASSCADDR_net_net_3 : in std_logic := 'U'; 
        ttxwkrvqgng_i_a3 : out std_logic; INIT_ASSC_WEN : in
         std_logic := 'U'; ASSC_RAM_CSN_A_net : out std_logic; 
        ASSC_RAM_DO_B_net_net_2, ASSC_RAM_DO_B_net_net_1, 
        \Z\\EV_ASSCADDR_net\\\, EV_ASSCADDR_net_net_4 : in
         std_logic := 'U'; ASSC_RAM_ADDR_B_net_net_5, 
        \Z\\ASSC_RAM_ADDR_B_net\\\, ASSC_RAM_ADDR_B_net_net_1, 
        ASSC_SEQOUT_net_net_1, ASSC_RAM_ADDR_B_net_net_2, 
        ASSC_RAM_ADDR_B_net_net_3, ASSC_RAM_ADDR_B_net_net_4, 
        ASSC_SEQOUT_net_net_4 : out std_logic; 
        EV_ASSCADDR_net_net_2 : in std_logic := 'U'; rhbzpsxhwms, 
        ASSC_RAM_RW_B_net, ASSC_DONE_c, ASSC_RAM_DI_B_net_net_3 : 
        out std_logic; ADC_CALIBRATE_net : in std_logic := 'U'; 
        ADC_RESET_net, ADC_PDOWN_net, ASSC_CHSAT_c, 
        ASSC_SAMPFLAG_net, ADC_VREFSEL_net, ADC_CHNR_net_net_3, 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_1, 
        \Z\\ADC_CHNR_net\\\ : out std_logic; 
        ASSC_RAM_DO_B_net_net_3 : in std_logic := 'U'; 
        ADC_STC_net_net_6, ADC_STC_net_net_5, ADC_STC_net_net_4, 
        ADC_STC_net_net_3, ADC_STC_net_net_2, ADC_STC_net_net_1, 
        \Z\\ADC_STC_net\\\, ADC_TVC_net_net_6, ADC_TVC_net_net_5, 
        ADC_TVC_net_net_4, ADC_TVC_net_net_3, ADC_TVC_net_net_2, 
        ADC_TVC_net_net_1, \Z\\ADC_TVC_net\\\, 
        \Z\\ADC_MODE_net\\\ : out std_logic);
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : ASSC
	Use entity work.ASSC(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    U0 : ASSC
      port map(dsjtwjhnpmm_i(8) => dsjtwjhnpmm_i(8), 
        dsjtwjhnpmm_i(7) => dsjtwjhnpmm_i(7), dsjtwjhnpmm_i(6)
         => dsjtwjhnpmm_i(6), dsjtwjhnpmm_i(5) => 
        dsjtwjhnpmm_i(5), dsjtwjhnpmm_i(4) => dsjtwjhnpmm_i(4), 
        dsjtwjhnpmm_i(3) => dsjtwjhnpmm_i(3), dsjtwjhnpmm_i(2)
         => dsjtwjhnpmm_i(2), dsjtwjhnpmm_i(1) => 
        dsjtwjhnpmm_i(1), dsjtwjhnpmm_i(0) => dsjtwjhnpmm_i(0), 
        \ASSC_RAM_DO_B_net[8]\ => \ASSC_RAM_DO_B_net[8]\, 
        \ADC_RESULT_net[11]\ => \ADC_RESULT_net[11]\, 
        \ASSC_RAM_DI_B_net[8]\ => \ASSC_RAM_DI_B_net[8]\, 
        INIT_ADDR_0 => INIT_ADDR_0, INIT_ADDR_1 => INIT_ADDR_1, 
        INIT_ADDR_6 => INIT_ADDR_6, INIT_ADDR_4 => INIT_ADDR_4, 
        INIT_ADDR_2 => INIT_ADDR_2, INIT_ADDR_3 => INIT_ADDR_3, 
        INIT_ADDR_int_3 => INIT_ADDR_int_3, INIT_ADDR_int_2 => 
        INIT_ADDR_int_2, INIT_ADDR_int_0 => INIT_ADDR_int_0, 
        \EV_ASSCADDR_net[8]\ => \EV_ASSCADDR_net[8]\, 
        \ASSC_RAM_ADDR_B_net[8]\ => \ASSC_RAM_ADDR_B_net[8]\, 
        \ASSC_SEQOUT_net[5]\ => \ASSC_SEQOUT_net[5]\, 
        \tkcbxxqbtfx_ns[11]\ => \tkcbxxqbtfx_ns[11]\, 
        \ADC_CHNR_net[4]\ => \ADC_CHNR_net[4]\, \ADC_STC_net[7]\
         => \ADC_STC_net[7]\, \ADC_TVC_net[7]\ => 
        \ADC_TVC_net[7]\, \ADC_MODE_net[3]\ => \ADC_MODE_net[3]\, 
        ACM_CLK_c => ACM_CLK_c, sys_reset_gl => sys_reset_gl, 
        ASSC_RAM_DO_B_net_net_7 => ASSC_RAM_DO_B_net_net_7, 
        ADC_START_net => ADC_START_net, ASSC_RAM_DO_B_net_net_6
         => ASSC_RAM_DO_B_net_net_6, ASSC_RAM_DO_B_net_net_5 => 
        ASSC_RAM_DO_B_net_net_5, ASSC_RAM_DO_B_net_net_4 => 
        ASSC_RAM_DO_B_net_net_4, ASSC_SEQOUT_net_net_3 => 
        ASSC_SEQOUT_net_net_3, \Z\\ASSC_SEQOUT_net\\\ => 
        \Z\\ASSC_SEQOUT_net\\\, ASSC_SEQOUT_net_net_2 => 
        ASSC_SEQOUT_net_net_2, ASSC_CHLATD_c_i => ASSC_CHLATD_c_i, 
        jngrzvhtjvn_0 => jngrzvhtjvn_0, INIT_DONE_c => 
        INIT_DONE_c, ADC_BUSY_net => ADC_BUSY_net, 
        \Z\\ASSC_RAM_DO_B_net\\\ => \Z\\ASSC_RAM_DO_B_net\\\, 
        ADC_MODE_net_net_1 => ADC_MODE_net_net_1, 
        \Z\\ADC_RESULT_net\\\ => \Z\\ADC_RESULT_net\\\, 
        ADC_RESULT_net_net_1 => ADC_RESULT_net_net_1, 
        ADC_RESULT_net_net_3 => ADC_RESULT_net_net_3, 
        ADC_RESULT_net_net_2 => ADC_RESULT_net_net_2, 
        ADC_RESULT_net_net_5 => ADC_RESULT_net_net_5, 
        ADC_RESULT_net_net_6 => ADC_RESULT_net_net_6, 
        ADC_RESULT_net_net_4 => ADC_RESULT_net_net_4, 
        TR_TRFLAG_net => TR_TRFLAG_net, TR_DONE_net => 
        TR_DONE_net, EV_EVFLAG_net => EV_EVFLAG_net, N_247_i_0
         => N_247_i_0, EV_ASSCADDR_net_net_5 => 
        EV_ASSCADDR_net_net_5, bskrjhjvqpf_1_sqmuxa => 
        bskrjhjvqpf_1_sqmuxa, ASSC_RAM_ADDR_B_net_net_6 => 
        ASSC_RAM_ADDR_B_net_net_6, ASSC_RAM_ADDR_B_net_net_7 => 
        ASSC_RAM_ADDR_B_net_net_7, EV_DONE_net_0 => EV_DONE_net_0, 
        ADC_RESULT_net_net_10 => ADC_RESULT_net_net_10, 
        ADC_RESULT_net_net_9 => ADC_RESULT_net_net_9, 
        ADC_RESULT_net_net_8 => ADC_RESULT_net_net_8, 
        ASSC_RAM_DI_B_net_net_7 => ASSC_RAM_DI_B_net_net_7, 
        ASSC_RAM_DI_B_net_net_6 => ASSC_RAM_DI_B_net_net_6, 
        ASSC_RAM_DI_B_net_net_5 => ASSC_RAM_DI_B_net_net_5, 
        ASSC_RAM_DI_B_net_net_4 => ASSC_RAM_DI_B_net_net_4, 
        ASSC_RAM_DI_B_net_net_2 => ASSC_RAM_DI_B_net_net_2, 
        ASSC_RAM_DI_B_net_net_1 => ASSC_RAM_DI_B_net_net_1, 
        \Z\\ASSC_RAM_DI_B_net\\\ => \Z\\ASSC_RAM_DI_B_net\\\, 
        ADC_RESULT_net_net_7 => ADC_RESULT_net_net_7, 
        ADC_MODE_net_net_2 => ADC_MODE_net_net_2, 
        EV_ASSCADDR_net_net_1 => EV_ASSCADDR_net_net_1, 
        EV_ASSCADDR_net_net_3 => EV_ASSCADDR_net_net_3, 
        ttxwkrvqgng_i_a3 => ttxwkrvqgng_i_a3, INIT_ASSC_WEN => 
        INIT_ASSC_WEN, ASSC_RAM_CSN_A_net => ASSC_RAM_CSN_A_net, 
        ASSC_RAM_DO_B_net_net_2 => ASSC_RAM_DO_B_net_net_2, 
        ASSC_RAM_DO_B_net_net_1 => ASSC_RAM_DO_B_net_net_1, 
        \Z\\EV_ASSCADDR_net\\\ => \Z\\EV_ASSCADDR_net\\\, 
        EV_ASSCADDR_net_net_4 => EV_ASSCADDR_net_net_4, 
        ASSC_RAM_ADDR_B_net_net_5 => ASSC_RAM_ADDR_B_net_net_5, 
        \Z\\ASSC_RAM_ADDR_B_net\\\ => \Z\\ASSC_RAM_ADDR_B_net\\\, 
        ASSC_RAM_ADDR_B_net_net_1 => ASSC_RAM_ADDR_B_net_net_1, 
        ASSC_SEQOUT_net_net_1 => ASSC_SEQOUT_net_net_1, 
        ASSC_RAM_ADDR_B_net_net_2 => ASSC_RAM_ADDR_B_net_net_2, 
        ASSC_RAM_ADDR_B_net_net_3 => ASSC_RAM_ADDR_B_net_net_3, 
        ASSC_RAM_ADDR_B_net_net_4 => ASSC_RAM_ADDR_B_net_net_4, 
        ASSC_SEQOUT_net_net_4 => ASSC_SEQOUT_net_net_4, 
        EV_ASSCADDR_net_net_2 => EV_ASSCADDR_net_net_2, 
        rhbzpsxhwms => rhbzpsxhwms, ASSC_RAM_RW_B_net => 
        ASSC_RAM_RW_B_net, ASSC_DONE_c => ASSC_DONE_c, 
        ASSC_RAM_DI_B_net_net_3 => ASSC_RAM_DI_B_net_net_3, 
        ADC_CALIBRATE_net => ADC_CALIBRATE_net, ADC_RESET_net => 
        ADC_RESET_net, ADC_PDOWN_net => ADC_PDOWN_net, 
        ASSC_CHSAT_c => ASSC_CHSAT_c, ASSC_SAMPFLAG_net => 
        ASSC_SAMPFLAG_net, ADC_VREFSEL_net => ADC_VREFSEL_net, 
        ADC_CHNR_net_net_3 => ADC_CHNR_net_net_3, 
        ADC_CHNR_net_net_2 => ADC_CHNR_net_net_2, 
        ADC_CHNR_net_net_1 => ADC_CHNR_net_net_1, 
        \Z\\ADC_CHNR_net\\\ => \Z\\ADC_CHNR_net\\\, 
        ASSC_RAM_DO_B_net_net_3 => ASSC_RAM_DO_B_net_net_3, 
        ADC_STC_net_net_6 => ADC_STC_net_net_6, ADC_STC_net_net_5
         => ADC_STC_net_net_5, ADC_STC_net_net_4 => 
        ADC_STC_net_net_4, ADC_STC_net_net_3 => ADC_STC_net_net_3, 
        ADC_STC_net_net_2 => ADC_STC_net_net_2, ADC_STC_net_net_1
         => ADC_STC_net_net_1, \Z\\ADC_STC_net\\\ => 
        \Z\\ADC_STC_net\\\, ADC_TVC_net_net_6 => 
        ADC_TVC_net_net_6, ADC_TVC_net_net_5 => ADC_TVC_net_net_5, 
        ADC_TVC_net_net_4 => ADC_TVC_net_net_4, ADC_TVC_net_net_3
         => ADC_TVC_net_net_3, ADC_TVC_net_net_2 => 
        ADC_TVC_net_net_2, ADC_TVC_net_net_1 => ADC_TVC_net_net_1, 
        \Z\\ADC_TVC_net\\\ => \Z\\ADC_TVC_net\\\, 
        \Z\\ADC_MODE_net\\\ => \Z\\ADC_MODE_net\\\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity Analog_RTC_smtr_ram is

    port(INIT_DATA : in std_logic_vector(8 downto 0); INIT_ADDR
         : in std_logic_vector(8 downto 0); \TR_RAM_DI_B_net[8]\, 
        \TR_RAM_ADDR_B_net[8]\ : in std_logic; 
        \TR_RAM_DO_B_net[8]\ : out std_logic; sys_reset_gl, 
        \Z\\TR_RAM_DI_B_net\\\, TR_RAM_DI_B_net_net_1, 
        TR_RAM_DI_B_net_net_2, TR_RAM_DI_B_net_net_3, 
        TR_RAM_DI_B_net_net_4, TR_RAM_DI_B_net_net_5, 
        TR_RAM_DI_B_net_net_6, TR_RAM_DI_B_net_net_7, ACM_CLK_c, 
        TR_RAM_RW_B_net, TR_RAM_CSN_B_net, cnwzzrhdsqv_i_a2, 
        \Z\\TR_RAM_ADDR_B_net\\\, TR_RAM_ADDR_B_net_net_1, 
        TR_RAM_ADDR_B_net_net_2, TR_RAM_ADDR_B_net_net_3, 
        TR_RAM_ADDR_B_net_net_4, TR_RAM_ADDR_B_net_net_5, 
        TR_RAM_ADDR_B_net_net_6, TR_RAM_ADDR_B_net_net_7 : in
         std_logic; \Z\\TR_RAM_DO_B_net\\\, TR_RAM_DO_B_net_net_1, 
        TR_RAM_DO_B_net_net_2, TR_RAM_DO_B_net_net_3, 
        TR_RAM_DO_B_net_net_4, TR_RAM_DO_B_net_net_5, 
        TR_RAM_DO_B_net_net_6, TR_RAM_DO_B_net_net_7 : out
         std_logic);

end Analog_RTC_smtr_ram;

architecture DEF_ARCH of Analog_RTC_smtr_ram is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component RAM4K9
    generic (MEMORYFILE:string := "");

    port(ADDRA11, ADDRA10, ADDRA9, ADDRA8, ADDRA7, ADDRA6, ADDRA5, 
        ADDRA4, ADDRA3, ADDRA2, ADDRA1, ADDRA0, ADDRB11, ADDRB10, 
        ADDRB9, ADDRB8, ADDRB7, ADDRB6, ADDRB5, ADDRB4, ADDRB3, 
        ADDRB2, ADDRB1, ADDRB0, DINA8, DINA7, DINA6, DINA5, DINA4, 
        DINA3, DINA2, DINA1, DINA0, DINB8, DINB7, DINB6, DINB5, 
        DINB4, DINB3, DINB2, DINB1, DINB0, WIDTHA0, WIDTHA1, 
        WIDTHB0, WIDTHB1, PIPEA, PIPEB, WMODEA, WMODEB, BLKA, 
        BLKB, WENA, WENB, CLKA, CLKB, RESET : in std_logic := 'U'; 
        DOUTA8, DOUTA7, DOUTA6, DOUTA5, DOUTA4, DOUTA3, DOUTA2, 
        DOUTA1, DOUTA0, DOUTB8, DOUTB7, DOUTB6, DOUTB5, DOUTB4, 
        DOUTB3, DOUTB2, DOUTB1, DOUTB0 : out std_logic);
  end component;

    signal Analog_RTC_smtr_ram_R0C0_DOUTA0, 
        Analog_RTC_smtr_ram_R0C0_DOUTA1, 
        Analog_RTC_smtr_ram_R0C0_DOUTA2, 
        Analog_RTC_smtr_ram_R0C0_DOUTA3, 
        Analog_RTC_smtr_ram_R0C0_DOUTA4, 
        Analog_RTC_smtr_ram_R0C0_DOUTA5, 
        Analog_RTC_smtr_ram_R0C0_DOUTA6, 
        Analog_RTC_smtr_ram_R0C0_DOUTA7, 
        Analog_RTC_smtr_ram_R0C0_DOUTA8, \GND\, \VCC\, GND_net_1, 
        VCC_net_1 : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    Analog_RTC_smtr_ram_R0C0 : RAM4K9
      port map(ADDRA11 => \GND\, ADDRA10 => \GND\, ADDRA9 => 
        \GND\, ADDRA8 => INIT_ADDR(8), ADDRA7 => INIT_ADDR(7), 
        ADDRA6 => INIT_ADDR(6), ADDRA5 => INIT_ADDR(5), ADDRA4
         => INIT_ADDR(4), ADDRA3 => INIT_ADDR(3), ADDRA2 => 
        INIT_ADDR(2), ADDRA1 => INIT_ADDR(1), ADDRA0 => 
        INIT_ADDR(0), ADDRB11 => \GND\, ADDRB10 => \GND\, ADDRB9
         => \GND\, ADDRB8 => \TR_RAM_ADDR_B_net[8]\, ADDRB7 => 
        \Z\\TR_RAM_ADDR_B_net\\\, ADDRB6 => 
        TR_RAM_ADDR_B_net_net_1, ADDRB5 => 
        TR_RAM_ADDR_B_net_net_2, ADDRB4 => 
        TR_RAM_ADDR_B_net_net_3, ADDRB3 => 
        TR_RAM_ADDR_B_net_net_4, ADDRB2 => 
        TR_RAM_ADDR_B_net_net_5, ADDRB1 => 
        TR_RAM_ADDR_B_net_net_6, ADDRB0 => 
        TR_RAM_ADDR_B_net_net_7, DINA8 => INIT_DATA(8), DINA7 => 
        INIT_DATA(7), DINA6 => INIT_DATA(6), DINA5 => 
        INIT_DATA(5), DINA4 => INIT_DATA(4), DINA3 => 
        INIT_DATA(3), DINA2 => INIT_DATA(2), DINA1 => 
        INIT_DATA(1), DINA0 => INIT_DATA(0), DINB8 => 
        \TR_RAM_DI_B_net[8]\, DINB7 => \Z\\TR_RAM_DI_B_net\\\, 
        DINB6 => TR_RAM_DI_B_net_net_1, DINB5 => 
        TR_RAM_DI_B_net_net_2, DINB4 => TR_RAM_DI_B_net_net_3, 
        DINB3 => TR_RAM_DI_B_net_net_4, DINB2 => 
        TR_RAM_DI_B_net_net_5, DINB1 => TR_RAM_DI_B_net_net_6, 
        DINB0 => TR_RAM_DI_B_net_net_7, WIDTHA0 => \VCC\, WIDTHA1
         => \VCC\, WIDTHB0 => \VCC\, WIDTHB1 => \VCC\, PIPEA => 
        \GND\, PIPEB => \GND\, WMODEA => \VCC\, WMODEB => \VCC\, 
        BLKA => cnwzzrhdsqv_i_a2, BLKB => TR_RAM_CSN_B_net, WENA
         => cnwzzrhdsqv_i_a2, WENB => TR_RAM_RW_B_net, CLKA => 
        ACM_CLK_c, CLKB => ACM_CLK_c, RESET => sys_reset_gl, 
        DOUTA8 => Analog_RTC_smtr_ram_R0C0_DOUTA8, DOUTA7 => 
        Analog_RTC_smtr_ram_R0C0_DOUTA7, DOUTA6 => 
        Analog_RTC_smtr_ram_R0C0_DOUTA6, DOUTA5 => 
        Analog_RTC_smtr_ram_R0C0_DOUTA5, DOUTA4 => 
        Analog_RTC_smtr_ram_R0C0_DOUTA4, DOUTA3 => 
        Analog_RTC_smtr_ram_R0C0_DOUTA3, DOUTA2 => 
        Analog_RTC_smtr_ram_R0C0_DOUTA2, DOUTA1 => 
        Analog_RTC_smtr_ram_R0C0_DOUTA1, DOUTA0 => 
        Analog_RTC_smtr_ram_R0C0_DOUTA0, DOUTB8 => 
        \TR_RAM_DO_B_net[8]\, DOUTB7 => \Z\\TR_RAM_DO_B_net\\\, 
        DOUTB6 => TR_RAM_DO_B_net_net_1, DOUTB5 => 
        TR_RAM_DO_B_net_net_2, DOUTB4 => TR_RAM_DO_B_net_net_3, 
        DOUTB3 => TR_RAM_DO_B_net_net_4, DOUTB2 => 
        TR_RAM_DO_B_net_net_5, DOUTB1 => TR_RAM_DO_B_net_net_6, 
        DOUTB0 => TR_RAM_DO_B_net_net_7);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity Analog_RTC_assc_ram is

    port(INIT_DATA : in std_logic_vector(8 downto 0); 
        dsjtwjhnpmm_i : in std_logic_vector(8 downto 0); 
        \ASSC_RAM_DI_B_net[8]\, \ASSC_RAM_ADDR_B_net[8]\ : in
         std_logic; \ASSC_RAM_DO_B_net[8]\, 
        \ASSC_RAM_DO_A_net[8]\ : out std_logic; sys_reset_gl, 
        \Z\\ASSC_RAM_DI_B_net\\\, ASSC_RAM_DI_B_net_net_1, 
        ASSC_RAM_DI_B_net_net_2, ASSC_RAM_DI_B_net_net_3, 
        ASSC_RAM_DI_B_net_net_4, ASSC_RAM_DI_B_net_net_5, 
        ASSC_RAM_DI_B_net_net_6, ASSC_RAM_DI_B_net_net_7, 
        ACM_CLK_c, ASSC_RAM_RW_B_net, N_247_i_0, ttxwkrvqgng_i_a3, 
        ASSC_RAM_CSN_A_net, \Z\\ASSC_RAM_ADDR_B_net\\\, 
        ASSC_RAM_ADDR_B_net_net_1, ASSC_RAM_ADDR_B_net_net_2, 
        ASSC_RAM_ADDR_B_net_net_3, ASSC_RAM_ADDR_B_net_net_4, 
        ASSC_RAM_ADDR_B_net_net_5, ASSC_RAM_ADDR_B_net_net_6, 
        ASSC_RAM_ADDR_B_net_net_7 : in std_logic; 
        \Z\\ASSC_RAM_DO_B_net\\\, ASSC_RAM_DO_B_net_net_1, 
        ASSC_RAM_DO_B_net_net_2, ASSC_RAM_DO_B_net_net_3, 
        ASSC_RAM_DO_B_net_net_4, ASSC_RAM_DO_B_net_net_5, 
        ASSC_RAM_DO_B_net_net_6, ASSC_RAM_DO_B_net_net_7, 
        \Z\\ASSC_RAM_DO_A_net\\\, ASSC_RAM_DO_A_net_net_1, 
        ASSC_RAM_DO_A_net_net_2, ASSC_RAM_DO_A_net_net_3, 
        ASSC_RAM_DO_A_net_net_4, ASSC_RAM_DO_A_net_net_5, 
        ASSC_RAM_DO_A_net_net_6, ASSC_RAM_DO_A_net_net_7 : out
         std_logic);

end Analog_RTC_assc_ram;

architecture DEF_ARCH of Analog_RTC_assc_ram is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component RAM4K9
    generic (MEMORYFILE:string := "");

    port(ADDRA11, ADDRA10, ADDRA9, ADDRA8, ADDRA7, ADDRA6, ADDRA5, 
        ADDRA4, ADDRA3, ADDRA2, ADDRA1, ADDRA0, ADDRB11, ADDRB10, 
        ADDRB9, ADDRB8, ADDRB7, ADDRB6, ADDRB5, ADDRB4, ADDRB3, 
        ADDRB2, ADDRB1, ADDRB0, DINA8, DINA7, DINA6, DINA5, DINA4, 
        DINA3, DINA2, DINA1, DINA0, DINB8, DINB7, DINB6, DINB5, 
        DINB4, DINB3, DINB2, DINB1, DINB0, WIDTHA0, WIDTHA1, 
        WIDTHB0, WIDTHB1, PIPEA, PIPEB, WMODEA, WMODEB, BLKA, 
        BLKB, WENA, WENB, CLKA, CLKB, RESET : in std_logic := 'U'; 
        DOUTA8, DOUTA7, DOUTA6, DOUTA5, DOUTA4, DOUTA3, DOUTA2, 
        DOUTA1, DOUTA0, DOUTB8, DOUTB7, DOUTB6, DOUTB5, DOUTB4, 
        DOUTB3, DOUTB2, DOUTB1, DOUTB0 : out std_logic);
  end component;

    signal \GND\, \VCC\, GND_net_1, VCC_net_1 : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Analog_RTC_assc_ram_R0C0 : RAM4K9
      port map(ADDRA11 => \GND\, ADDRA10 => \GND\, ADDRA9 => 
        \GND\, ADDRA8 => dsjtwjhnpmm_i(8), ADDRA7 => 
        dsjtwjhnpmm_i(7), ADDRA6 => dsjtwjhnpmm_i(6), ADDRA5 => 
        dsjtwjhnpmm_i(5), ADDRA4 => dsjtwjhnpmm_i(4), ADDRA3 => 
        dsjtwjhnpmm_i(3), ADDRA2 => dsjtwjhnpmm_i(2), ADDRA1 => 
        dsjtwjhnpmm_i(1), ADDRA0 => dsjtwjhnpmm_i(0), ADDRB11 => 
        \GND\, ADDRB10 => \GND\, ADDRB9 => \GND\, ADDRB8 => 
        \ASSC_RAM_ADDR_B_net[8]\, ADDRB7 => 
        \Z\\ASSC_RAM_ADDR_B_net\\\, ADDRB6 => 
        ASSC_RAM_ADDR_B_net_net_1, ADDRB5 => 
        ASSC_RAM_ADDR_B_net_net_2, ADDRB4 => 
        ASSC_RAM_ADDR_B_net_net_3, ADDRB3 => 
        ASSC_RAM_ADDR_B_net_net_4, ADDRB2 => 
        ASSC_RAM_ADDR_B_net_net_5, ADDRB1 => 
        ASSC_RAM_ADDR_B_net_net_6, ADDRB0 => 
        ASSC_RAM_ADDR_B_net_net_7, DINA8 => INIT_DATA(8), DINA7
         => INIT_DATA(7), DINA6 => INIT_DATA(6), DINA5 => 
        INIT_DATA(5), DINA4 => INIT_DATA(4), DINA3 => 
        INIT_DATA(3), DINA2 => INIT_DATA(2), DINA1 => 
        INIT_DATA(1), DINA0 => INIT_DATA(0), DINB8 => 
        \ASSC_RAM_DI_B_net[8]\, DINB7 => \Z\\ASSC_RAM_DI_B_net\\\, 
        DINB6 => ASSC_RAM_DI_B_net_net_1, DINB5 => 
        ASSC_RAM_DI_B_net_net_2, DINB4 => ASSC_RAM_DI_B_net_net_3, 
        DINB3 => ASSC_RAM_DI_B_net_net_4, DINB2 => 
        ASSC_RAM_DI_B_net_net_5, DINB1 => ASSC_RAM_DI_B_net_net_6, 
        DINB0 => ASSC_RAM_DI_B_net_net_7, WIDTHA0 => \VCC\, 
        WIDTHA1 => \VCC\, WIDTHB0 => \VCC\, WIDTHB1 => \VCC\, 
        PIPEA => \GND\, PIPEB => \GND\, WMODEA => \VCC\, WMODEB
         => \VCC\, BLKA => ASSC_RAM_CSN_A_net, BLKB => N_247_i_0, 
        WENA => ttxwkrvqgng_i_a3, WENB => ASSC_RAM_RW_B_net, CLKA
         => ACM_CLK_c, CLKB => ACM_CLK_c, RESET => sys_reset_gl, 
        DOUTA8 => \ASSC_RAM_DO_A_net[8]\, DOUTA7 => 
        \Z\\ASSC_RAM_DO_A_net\\\, DOUTA6 => 
        ASSC_RAM_DO_A_net_net_1, DOUTA5 => 
        ASSC_RAM_DO_A_net_net_2, DOUTA4 => 
        ASSC_RAM_DO_A_net_net_3, DOUTA3 => 
        ASSC_RAM_DO_A_net_net_4, DOUTA2 => 
        ASSC_RAM_DO_A_net_net_5, DOUTA1 => 
        ASSC_RAM_DO_A_net_net_6, DOUTA0 => 
        ASSC_RAM_DO_A_net_net_7, DOUTB8 => \ASSC_RAM_DO_B_net[8]\, 
        DOUTB7 => \Z\\ASSC_RAM_DO_B_net\\\, DOUTB6 => 
        ASSC_RAM_DO_B_net_net_1, DOUTB5 => 
        ASSC_RAM_DO_B_net_net_2, DOUTB4 => 
        ASSC_RAM_DO_B_net_net_3, DOUTB3 => 
        ASSC_RAM_DO_B_net_net_4, DOUTB2 => 
        ASSC_RAM_DO_B_net_net_5, DOUTB1 => 
        ASSC_RAM_DO_B_net_net_6, DOUTB0 => 
        ASSC_RAM_DO_B_net_net_7);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity nbnjbfrkqwq is

    port(INIT_ADDR_int : in std_logic_vector(8 downto 5); 
        INIT_ADDR : in std_logic_vector(4 downto 0); 
        rdgwgtpkbgc_0_3, \EV_RAM_ADDR_B_net[8]\ : out std_logic; 
        \ADC_CHNR_net[4]\, \ASSC_SEQOUT_net[5]\ : in std_logic; 
        \EV_ASSCADDR_net[8]\ : out std_logic; tkcbxxqbtfx_ns_11 : 
        in std_logic; \EV_RAM_DI_B_net[8]\ : out std_logic; 
        \ASSC_RAM_DO_A_net[8]\, \TR_EV_ADDR_net[8]\ : in
         std_logic; \EV_RAM_ADDR_A_net[8]\ : out std_logic; 
        rdgwgtpkbgc_3 : in std_logic; \EV_CHHOLD_net[4]\ : out
         std_logic; \EV_RAM_DO_B_net[8]\ : in std_logic; 
        \Z\\EV_RAM_ADDR_B_net\\\, EV_RAM_ADDR_B_net_net_2, 
        EV_RAM_ADDR_B_net_net_4, EV_RAM_ADDR_B_net_net_7 : out
         std_logic; \Z\\ADC_CHNR_net\\\, ADC_CHNR_net_net_1, 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_3, 
        \Z\\ASSC_SEQOUT_net\\\ : in std_logic; 
        \Z\\EV_ASSCADDR_net\\\ : out std_logic; 
        ASSC_SEQOUT_net_net_1 : in std_logic; 
        EV_ASSCADDR_net_net_1 : out std_logic; 
        ASSC_SEQOUT_net_net_2 : in std_logic; 
        EV_ASSCADDR_net_net_2 : out std_logic; 
        ASSC_SEQOUT_net_net_3 : in std_logic; 
        EV_ASSCADDR_net_net_3 : out std_logic; 
        ASSC_SEQOUT_net_net_4 : in std_logic; 
        EV_ASSCADDR_net_net_4, EV_EVFLAG_net, EV_DONE_net, 
        EV_RAM_WR_BUSY_B_net, EV_RAM_DI_B_net_net_7, 
        EV_RAM_DI_B_net_net_6, EV_RAM_DI_B_net_net_2, 
        \Z\\EV_RAM_DI_B_net\\\, EV_RAM_DI_B_net_net_1 : out
         std_logic; TR_EV_RD_net : in std_logic; EV_RAM_CSN_A_net : 
        out std_logic; INIT_EV_WEN : in std_logic; 
        EV_RAM_RW_A_net : out std_logic; ASSC_DONE_c, 
        ASSC_SAMPFLAG_net, jngrzvhtjvn_0, rhbzpsxhwms, 
        TR_EV_ADDR_net_net_7 : in std_logic; 
        EV_RAM_ADDR_A_net_net_7 : out std_logic; 
        TR_EV_ADDR_net_net_6 : in std_logic; 
        EV_RAM_ADDR_A_net_net_6 : out std_logic; 
        TR_EV_ADDR_net_net_4 : in std_logic; 
        EV_RAM_ADDR_A_net_net_4 : out std_logic; 
        TR_EV_ADDR_net_net_3 : in std_logic; 
        EV_RAM_ADDR_A_net_net_3 : out std_logic; 
        TR_EV_ADDR_net_net_2 : in std_logic; 
        EV_RAM_ADDR_A_net_net_2 : out std_logic; 
        TR_EV_ADDR_net_net_1 : in std_logic; 
        EV_RAM_ADDR_A_net_net_1 : out std_logic; 
        \Z\\TR_EV_ADDR_net\\\ : in std_logic; 
        \Z\\EV_RAM_ADDR_A_net\\\ : out std_logic; 
        TR_EV_ADDR_net_net_5 : in std_logic; 
        EV_RAM_ADDR_A_net_net_5 : out std_logic; 
        ASSC_RAM_DO_A_net_net_7, \Z\\ASSC_RAM_DO_A_net\\\, 
        ASSC_RAM_DO_A_net_net_1, ASSC_RAM_DO_A_net_net_2, 
        ASSC_RAM_DO_A_net_net_3, ASSC_RAM_DO_A_net_net_4, 
        ASSC_RAM_DO_A_net_net_5, ASSC_RAM_DO_A_net_net_6, 
        EV_RAM_DO_B_net_net_3, EV_RAM_DO_B_net_net_4 : in
         std_logic; EV_RAM_DI_B_net_net_4, EV_RAM_DI_B_net_net_3, 
        EV_RAM_DI_B_net_net_5 : out std_logic; INIT_DONE_c : in
         std_logic; EV_RAM_CSN_B_net, EV_ASSCADDR_net_net_5 : out
         std_logic; EV_RAM_DO_B_net_net_2 : in std_logic; 
        EV_CHHOLD_net_net_3, EV_CHHOLD_net_net_2, 
        EV_CHHOLD_net_net_1, \Z\\EV_CHHOLD_net\\\ : out std_logic; 
        EV_RAM_DO_B_net_net_1, \Z\\EV_RAM_DO_B_net\\\ : in
         std_logic; EV_RAM_ADDR_B_net_net_1, 
        EV_RAM_ADDR_B_net_net_5, EV_RAM_ADDR_B_net_net_3, 
        EV_RAM_ADDR_B_net_net_6 : out std_logic; 
        EV_RAM_DO_B_net_net_5, EV_RAM_DO_B_net_net_6 : in
         std_logic; bskrjhjvqpf_1_sqmuxa : out std_logic; 
        EV_RAM_DO_B_net_net_7, sys_reset_gl, ACM_CLK_c : in
         std_logic; EV_DONE_net_0 : out std_logic);

end nbnjbfrkqwq;

architecture DEF_ARCH of nbnjbfrkqwq is 

  component DFN1E1C0
    port(D, CLK, CLR, E : in std_logic := 'U'; Q : out std_logic);
        
  end component;

  component NOR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XNOR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AOI1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XNOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2C
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AOI1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1C0
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component NOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MIN3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO13
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XOR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AND2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MAJ3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1E0C0
    port(D, CLK, CLR, E : in std_logic := 'U'; Q : out std_logic);
        
  end component;

  component AND2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component OR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OAI1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AXOI3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO18
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XA1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AND3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1P0
    port(D, CLK, PRE : in std_logic := 'U'; Q : out std_logic);
  end component;

  component AOI1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2B
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AXO5
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

    signal \EV_RAM_ADDR_B_net[8]_net_1\, 
        \EV_CHHOLD_net[4]_net_1\, \Z\\EV_RAM_ADDR_B_net\\_net_1\, 
        \EV_RAM_ADDR_B_net_net_2\, \EV_RAM_ADDR_B_net_net_4\, 
        \EV_RAM_ADDR_B_net_net_7\, \EV_ASSCADDR_net_net_5\, 
        \EV_CHHOLD_net_net_3\, \EV_CHHOLD_net_net_2\, 
        \EV_CHHOLD_net_net_1\, \Z\\EV_CHHOLD_net\\_net_1\, 
        \EV_RAM_ADDR_B_net_net_1\, \EV_RAM_ADDR_B_net_net_5\, 
        \EV_RAM_ADDR_B_net_net_3\, \EV_RAM_ADDR_B_net_net_6\, 
        \bskrjhjvqpf_1_sqmuxa\, bskrjhjvqpf_6_sqmuxa, 
        \rdgwgtpkbgc_0[0]_net_1\, \rdgwgtpkbgc_0[1]_net_1\, 
        \rdgwgtpkbgc_1[2]_net_1\, \rdgwgtpkbgc_0[2]_net_1\, 
        \rgmpzqrbghh_0\, hmzcpnxsdwg, \tkcbxxqbtfx_0[4]_net_1\, 
        \tkcbxxqbtfx_ns[13]\, \DWACT_ADD_CI_0_g_array_2[0]\, 
        \DWACT_ADD_CI_0_g_array_1[0]\, 
        \DWACT_ADD_CI_0_pog_array_1[0]\, 
        \DWACT_ADD_CI_0_g_array_3[0]\, 
        \DWACT_ADD_CI_0_pog_array_2[0]\, 
        \DWACT_ADD_CI_0_g_array_11[0]\, 
        \DWACT_ADD_CI_0_pog_array_1_1[0]\, 
        \DWACT_ADD_CI_0_TMP[0]\, \DWACT_ADD_CI_0_g_array_12_1[0]\, 
        \DWACT_ADD_CI_0_g_array_12[0]\, 
        \DWACT_ADD_CI_0_g_array_12_2[0]\, \kkjrqmngvjc_0_0_1\, 
        \kkjrqmngvjc_0_0_0\, N_180_5, \tkcbxxqbtfx[1]_net_1\, 
        \tkcbxxqbtfx[11]_net_1\, N_1083, \kkjrqmngvjc_0_0_3_1\, 
        ADD_18x18_slow_I17_Y_2, ADD_18x18_slow_I17_Y_1, N_317, 
        \bnvtvdkcnpp[11]_net_1\, \rqrqhcwpqdn[11]_net_1\, N_940, 
        \zbqggxmnpbs_0_iv_0[1]_net_1\, \tkcbxxqbtfx[5]_net_1\, 
        \fqkdsnjfpdb_i[7]\, \ckghxwbsztp_m[0]_net_1\, 
        \zbqggxmnpbs_0_iv_1[0]_net_1\, \fqkdsnjfpdb[6]\, 
        \zbqggxmnpbs_0_iv_0[0]_net_1\, \tddhwjzbsrj\, 
        \tkcbxxqbtfx[0]_net_1\, un2_pbzxzjhqhwz_m, 
        ADD_18x18_slow_I13_S_0, N_301, N_297, 
        \zbqggxmnpbs_0_iv_0_0[5]_net_1\, \ckghxwbsztp[4]_net_1\, 
        N_1149, ADD_18x18_slow_I11_S_0, N_291, N_287, 
        \zbqggxmnpbs_0_iv_0[4]_net_1\, \fqkdsnjfpdb[1]\, 
        \ckghxwbsztp_m[3]_net_1\, \zbqggxmnpbs_0_iv_0_0[3]_net_1\, 
        \fqkdsnjfpdb[0]\, N_1147, ADD_18x18_slow_I8_S_0, N_276_i, 
        N_272_i, ADD_18x18_slow_I6_S_0, N_266, N_262, 
        ADD_18x18_slow_I5_CO1_i_o3_0, N_261, N_41, N_257, 
        ADD_18x18_slow_I4_un1_CO1_i_o3_0, N_252_i, N_35, N_256_i, 
        ADD_18x18_slow_I4_S_0, \dhzrwbnrbpw_7_iv_0_0[7]\, N_1088, 
        N_1077, \dhzrwbnrbpw_7_iv_0_0[6]\, N_1074, 
        \dhzrwbnrbpw_7_iv_0_0[8]\, N_1080, 
        \dhzrwbnrbpw_7_iv_0_0[5]\, N_1071, 
        \dhzrwbnrbpw_7_iv_0_0[4]\, N_1067, ADD_18x18_slow_I2_S_0, 
        N_246, N_242, \dhzrwbnrbpw_7_iv_0_0[3]\, N_1063, 
        \dhzrwbnrbpw_7_iv_0_0[2]\, N_1059, 
        \dhzrwbnrbpw_7_iv_0_0[1]\, N_1055, 
        \dhzrwbnrbpw_7_iv_0_0[0]\, N_1051, \G_101_0\, 
        \ftbkrvpxdbp[2]_net_1\, \gjkdbmpgrtc[2]\, 
        \dhzrwbnrbpw_7_iv_0_a3_0[0]\, \tkcbxxqbtfx[8]_net_1\, 
        \bskrjhjvqpf_i_0_2\, \bskrjhjvqpf_i_0_1\, 
        \tkcbxxqbtfx[9]_net_1\, \tkcbxxqbtfx[15]_net_1\, 
        \bskrjhjvqpf_i_0_a2_0\, \tkcbxxqbtfx[12]_net_1\, 
        \gpjgkmhxbkn_NE_2\, gpjgkmhxbkn_1_i, gpjgkmhxbkn_2_i, 
        gpjgkmhxbkn_0_i, \gpjgkmhxbkn_NE_0\, 
        \rqrqhcwpqdn[5]_net_1\, gpjgkmhxbkn_3_i, 
        \un1_fskhwddcbbg_i_o3_0\, \tkcbxxqbtfx[3]_net_1\, 
        \wqpbnjzjqsx_NE_2\, wqpbnjzjqsx_4_i, wqpbnjzjqsx_3_i, 
        wqpbnjzjqsx_0_i, \wqpbnjzjqsx_NE_1\, 
        \bnvtvdkcnpp[3]_net_1\, \rqrqhcwpqdn[3]_net_1\, 
        wqpbnjzjqsx_1_i, \tkcbxxqbtfx[6]_net_1\, 
        \tkcbxxqbtfx[17]_net_1\, N_1136, N_1129_i, 
        \tkcbxxqbtfx_ns_1[14]\, \dhzrwbnrbpw_7[5]\, N_1069, 
        N_1072, \dhzrwbnrbpw_7[3]\, N_1061, N_1064, 
        \dhzrwbnrbpw_7[1]\, N_1053, N_1056, \bskrjhjvqpf_i_0\, 
        N_1160, N_1161, \tkcbxxqbtfx_ns[1]\, 
        \rqrqhcwpqdn_9_i[11]_net_1\, N_1007, N_1009, N_1008, 
        \rqrqhcwpqdn_9_i[10]_net_1\, N_1004, N_1006, N_1005, 
        \rqrqhcwpqdn_9_i[9]_net_1\, N_1001, N_1003, N_1002, 
        \rqrqhcwpqdn_5_i[7]_net_1\, N_996, N_997, N_995, 
        \rqrqhcwpqdn_5_i[6]_net_1\, N_993, N_994, N_992, 
        \rqrqhcwpqdn_5_i[5]_net_1\, N_990, N_991, N_989, 
        \rqrqhcwpqdn_5_i[4]_net_1\, N_987, N_988, N_986, 
        \rqrqhcwpqdn_5_i[3]_net_1\, N_984, N_985, N_983, 
        \rqrqhcwpqdn_5_i[2]_net_1\, N_981, N_982, N_980, 
        \rqrqhcwpqdn_5_i[1]_net_1\, N_978, N_979, N_977, N_241_i, 
        \ftbkrvpxdbp[0]_net_1\, \ftbkrvpxdbp[1]_net_1\, 
        \gjkdbmpgrtc[1]\, \bnwcvzjbgtc[2]\, N_251, 
        \gjkdbmpgrtc[3]\, \ftbkrvpxdbp[3]_net_1\, 
        \bnwcvzjbgtc[3]\, \gjkdbmpgrtc[4]\, 
        \ftbkrvpxdbp[4]_net_1\, \bnwcvzjbgtc[4]\, 
        \gjkdbmpgrtc[6]\, \bnvtvdkcnpp[0]_net_1\, 
        \bnwcvzjbgtc[6]\, \gjkdbmpgrtc[8]\, 
        \bnvtvdkcnpp[2]_net_1\, \bnwcvzjbgtc[8]\, N_281, 
        \gjkdbmpgrtc[9]\, \bnwcvzjbgtc[9]\, N_286, 
        \gjkdbmpgrtc[10]\, \bnvtvdkcnpp[4]_net_1\, 
        \bnwcvzjbgtc[10]\, \gjkdbmpgrtc[11]\, 
        \bnvtvdkcnpp[5]_net_1\, \bnwcvzjbgtc[11]\, N_296_i, 
        \gjkdbmpgrtc[12]\, \bnvtvdkcnpp[6]_net_1\, 
        \bnwcvzjbgtc[12]\, \gjkdbmpgrtc[13]\, 
        \bnvtvdkcnpp[7]_net_1\, \bnwcvzjbgtc[13]\, N_306, 
        \gjkdbmpgrtc[14]\, \bnvtvdkcnpp[8]_net_1\, 
        \bnwcvzjbgtc[14]\, N_311, \gjkdbmpgrtc[15]\, 
        \bnvtvdkcnpp[9]_net_1\, \bnwcvzjbgtc[15]\, N_316_i_0, 
        \gjkdbmpgrtc[16]\, \bnvtvdkcnpp[10]_net_1\, 
        \bnwcvzjbgtc[16]\, N_50, N231, N_48, N_282, N247, N_47, 
        N_302, N255, N_39, N_277, N_40, \bnwcvzjbgtc[1]\, N_42, 
        \bnwcvzjbgtc[0]\, \gjkdbmpgrtc_i[0]\, N_72, N_36, N_292_i, 
        N_37, \fqkdsnjfpdb[2]\, \fqkdsnjfpdb[3]\, N_247, 
        \fqkdsnjfpdb_i[4]\, \fqkdsnjfpdb[8]\, \fqkdsnjfpdb[9]\, 
        \fqkdsnjfpdb[10]\, \fqkdsnjfpdb_i[11]\, \fqkdsnjfpdb[12]\, 
        \fqkdsnjfpdb_i[13]\, \fqkdsnjfpdb[14]\, \fqkdsnjfpdb[15]\, 
        N_307, \fqkdsnjfpdb_i[16]\, N_312_i_0, N_38, 
        \fqkdsnjfpdb[17]\, I16_un1_CO1_i, \bnvtvdkcnpp[1]_net_1\, 
        \bnwcvzjbgtc[7]\, \gjkdbmpgrtc[7]\, N_267, N_271, N_49, 
        \rqrqhcwpqdn_5_i[8]_net_1\, N_999, N_1000, N_998, N_1086, 
        N_1044, \dhzrwbnrbpw_7[0]\, 
        \DWACT_ADD_CI_0_partial_sum[0]\, \dhzrwbnrbpw_7[6]\, 
        I_38_1, \dhzrwbnrbpw_7[7]\, I_34_1, \dhzrwbnrbpw_7[8]\, 
        I_36_1, \dhzrwbnrbpw_7[2]\, N_1057, N_1060, 
        \dhzrwbnrbpw_7[4]\, N_1065, N_1068, 
        \ftbkrvpxdbp[5]_net_1\, \gjkdbmpgrtc[5]\, 
        \bnwcvzjbgtc[5]\, N239, \fqkdsnjfpdb[5]\, N_81_i, 
        \fqkdsnjfpdb_m[8]_net_1\, \ckghxwbsztp_m[1]_net_1\, 
        \fqkdsnjfpdb_m[17]_net_1\, N_199, \un1_kkgccghmrzf\, 
        \rfrdbqfdrhx[0]_net_1\, \rfrdbqfdrhx[1]_net_1\, 
        \ftbkrvpxdbp_1_sqmuxa\, \tkcbxxqbtfx_ns[7]\, 
        \tkcbxxqbtfx_ns[8]\, \tkcbxxqbtfx[10]_net_1\, 
        \bnvtvdkcnpp_5[0]_net_1\, un3_qvqqfftqsfc, 
        \bnvtvdkcnpp_5[1]_net_1\, \bnvtvdkcnpp_5[2]_net_1\, 
        \bnvtvdkcnpp_5[3]_net_1\, \bnvtvdkcnpp_5[4]_net_1\, 
        \bnvtvdkcnpp_5[5]_net_1\, \bnvtvdkcnpp_5[6]_net_1\, 
        \bnvtvdkcnpp_5[7]_net_1\, \bnvtvdkcnpp_5[8]_net_1\, N_433, 
        N_960, N_531, N_434, N_530, N_532, N_435, N_533, N_436, 
        N_534, N_437, N_535, N_438, N_536, N_439, N_537, N_440, 
        N_538, N_441, N_539, N_442, N_540, N_443, N_961, N_499, 
        N_954, N_500, N_955, N_501, N_503, N_504, N_444, N_505, 
        N_956, \rqrqhcwpqdn[0]_net_1\, \rqrqhcwpqdn[1]_net_1\, 
        \rqrqhcwpqdn[2]_net_1\, \rqrqhcwpqdn[4]_net_1\, 
        \rqrqhcwpqdn[6]_net_1\, \rqrqhcwpqdn[7]_net_1\, 
        \rqrqhcwpqdn[8]_net_1\, \rqrqhcwpqdn[9]_net_1\, 
        \rqrqhcwpqdn[10]_net_1\, N_571, N_572, N_573, N_574, 
        N_575, N_576, N_577, N_496, N_962, N_957, N_963, N_965, 
        N_568, N_966, N_959, N_507, N_508, N_509, N_510, N_967, 
        N_511, N_968, N_598, N_599, N_615, N_616, N_632, N_633, 
        N_634, N_717, N_635, N_718, N_636, N_734, N_637, N_735, 
        \rdgwgtpkbgc[1]_net_1\, N_638, N_736, N_639, N_737, N_640, 
        N_738, N_641, N_739, N_642, N_740, N_643, N_741, N_644, 
        N_742, N_645, N_743, N_646, N_744, N_647, N_974, N_700, 
        N_701, N_702, N_703, N_704, N_705, N_706, N_707, N_708, 
        N_970, N_709, N_971, \rdgwgtpkbgc[0]_net_1\, N_768, N_769, 
        N_770, N_771, N_772, N_773, N_774, N_775, N_776, N_777, 
        N_778, N_779, N_780, N_781, N_710, N_711, 
        \rdgwgtpkbgc[2]_net_1\, N_712, N_713, N_714, N_782, N_972, 
        N_975, N_973, N_976, \ftbkrvpxdbp_5[0]_net_1\, 
        \ftbkrvpxdbp_5[1]_net_1\, \ftbkrvpxdbp_5[2]_net_1\, 
        \ftbkrvpxdbp_5[3]_net_1\, \ftbkrvpxdbp_5[4]_net_1\, 
        \ftbkrvpxdbp_5[5]_net_1\, \bnvtvdkcnpp_9[0]_net_1\, 
        \bnvtvdkcnpp_9[1]_net_1\, \bnvtvdkcnpp_9[2]_net_1\, 
        \bnvtvdkcnpp_9[3]_net_1\, \bnvtvdkcnpp_9[4]_net_1\, 
        \bnvtvdkcnpp_9[5]_net_1\, \tkcbxxqbtfx[4]_net_1\, 
        \bnvtvdkcnpp_9[6]_net_1\, \bnvtvdkcnpp_9[7]_net_1\, 
        \bnvtvdkcnpp_9[8]_net_1\, \bnvtvdkcnpp_9[9]_net_1\, 
        \bnvtvdkcnpp_8[9]_net_1\, \bnvtvdkcnpp_9[10]_net_1\, 
        \bnvtvdkcnpp_8[10]_net_1\, \bnvtvdkcnpp_9[11]_net_1\, 
        \bnvtvdkcnpp_8[11]_net_1\, \fqkdsnjfpdb_m[15]_net_1\, 
        \fqkdsnjfpdb_m[16]_net_1\, \fqkdsnjfpdb_m[4]_net_1\, 
        \tkcbxxqbtfx_ns[3]\, \tkcbxxqbtfx_ns[12]\, N_1011_i, 
        N_1010_i, N_1127, N_937, N_1012_1, \qrbxknbdvjj[1]_net_1\, 
        I_32_1, \qrbxknbdvjj[3]_net_1\, I_37_1, 
        \qrbxknbdvjj[5]_net_1\, I_33_1, \tkcbxxqbtfx[14]_net_1\, 
        \tkcbxxqbtfx_ns[0]\, \rgmpzqrbghh\, N_1048, 
        \tkcbxxqbtfx_ns[9]\, N_1152, N_1159, \tkcbxxqbtfx_ns[4]\, 
        N_1130, N_1107, N_1128, \spbznmtdmpd\, 
        \tkcbxxqbtfx[2]_net_1\, \ckghxwbsztp_3[3]\, I_13_6, 
        \tkcbxxqbtfx_ns[14]\, \tkcbxxqbtfx_ns[6]\, 
        bskrjhjvqpf_6_sqmuxa_2, bskrjhjvqpf_0_sqmuxa, N_1133, 
        \tkcbxxqbtfx_ns[10]\, N_969, N_964, N_958, N_512, 
        \tkcbxxqbtfx_ns[2]\, \tkcbxxqbtfx[16]_net_1\, N_872, 
        qmmwhjxxxmv, N_1122, \rqrqhcwpqdn_5_i[0]_net_1\, N_502, 
        N_570, \qrbxknbdvjj[8]_net_1\, \qrbxknbdvjj[7]_net_1\, 
        \qrbxknbdvjj[6]_net_1\, I_30_1, \qrbxknbdvjj[4]_net_1\, 
        I_35_1, \qrbxknbdvjj[2]_net_1\, \qrbxknbdvjj[0]_net_1\, 
        \tkcbxxqbtfx_ns[5]\, \tkcbxxqbtfx[13]_net_1\, 
        dbdctxjdvkp_2_sqmuxa, N_1144, \tkcbxxqbtfx_ns[16]\, 
        bskrjhjvqpf_6_sqmuxa_1, \tkcbxxqbtfx_ns[15]\, 
        \ckghxwbsztp_3[4]\, I_20, \ckghxwbsztp_3[2]\, I_9_5, 
        \ckghxwbsztp_3[1]\, I_5_5, \ckghxwbsztp_3[0]\, N_1151, 
        N_1148, \ckghxwbsztp[2]_net_1\, \fqkdsnjfpdb_m[14]_net_1\, 
        \fqkdsnjfpdb_m[13]_net_1\, \fqkdsnjfpdb_m[12]_net_1\, 
        \ckghxwbsztp[3]_net_1\, \ckghxwbsztp[1]_net_1\, 
        \ckghxwbsztp[0]_net_1\, un2_pbzxzjhqhwz, cmkddgvkrrs, 
        N_185, N_195, N_191, \tkcbxxqbtfx_ns[17]\, N_12, N_4, 
        \DWACT_FINC_E[0]\, N_9, \DWACT_COMP0_E[1]\, 
        \DWACT_COMP0_E[2]\, \DWACT_COMP0_E[0]\, 
        \DWACT_CMPLE_PO2_DWACT_COMP0_E[1]\, 
        \DWACT_CMPLE_PO2_DWACT_COMP0_E[2]\, 
        \DWACT_CMPLE_PO2_DWACT_COMP0_E[0]\, \ACT_LT4_E[3]\, 
        \ACT_LT4_E[6]\, \ACT_LT4_E[10]\, \ACT_LT4_E[7]\, 
        \ACT_LT4_E[8]\, \ACT_LT4_E[5]\, \ACT_LT4_E[4]\, 
        \ACT_LT4_E[0]\, \ACT_LT4_E[1]\, \ACT_LT4_E[2]\, 
        \ACT_LT3_E[3]\, \ACT_LT3_E[4]\, \ACT_LT3_E[5]\, 
        \ACT_LT3_E[0]\, \ACT_LT3_E[1]\, \ACT_LT3_E[2]\, 
        \DWACT_BL_EQUAL_0_E[2]\, \DWACT_BL_EQUAL_0_E[1]\, 
        \DWACT_BL_EQUAL_0_E[0]\, N_37_0, N_36_0, N_35_0, N_32, 
        N_34, N_33, N_31, N_28, N_29, N_30, 
        \DWACT_BL_EQUAL_0_DWACT_ANDTREE_E[1]\, 
        \DWACT_BL_EQUAL_0_DWACT_ANDTREE_E[0]\, 
        \DWACT_BL_EQUAL_0_E[4]\, \DWACT_BL_EQUAL_0_E[3]\, 
        \DWACT_BL_EQUAL_0_E_0[0]\, \DWACT_BL_EQUAL_0_E_0[1]\, 
        \DWACT_BL_EQUAL_0_E_0[2]\, 
        \DWACT_ADD_CI_0_pog_array_1_2[0]\, \VCC\, \GND\, 
        GND_net_1, VCC_net_1 : std_logic;

begin 

    \EV_RAM_ADDR_B_net[8]\ <= \EV_RAM_ADDR_B_net[8]_net_1\;
    \EV_CHHOLD_net[4]\ <= \EV_CHHOLD_net[4]_net_1\;
    \Z\\EV_RAM_ADDR_B_net\\\ <= \Z\\EV_RAM_ADDR_B_net\\_net_1\;
    EV_RAM_ADDR_B_net_net_2 <= \EV_RAM_ADDR_B_net_net_2\;
    EV_RAM_ADDR_B_net_net_4 <= \EV_RAM_ADDR_B_net_net_4\;
    EV_RAM_ADDR_B_net_net_7 <= \EV_RAM_ADDR_B_net_net_7\;
    EV_ASSCADDR_net_net_5 <= \EV_ASSCADDR_net_net_5\;
    EV_CHHOLD_net_net_3 <= \EV_CHHOLD_net_net_3\;
    EV_CHHOLD_net_net_2 <= \EV_CHHOLD_net_net_2\;
    EV_CHHOLD_net_net_1 <= \EV_CHHOLD_net_net_1\;
    \Z\\EV_CHHOLD_net\\\ <= \Z\\EV_CHHOLD_net\\_net_1\;
    EV_RAM_ADDR_B_net_net_1 <= \EV_RAM_ADDR_B_net_net_1\;
    EV_RAM_ADDR_B_net_net_5 <= \EV_RAM_ADDR_B_net_net_5\;
    EV_RAM_ADDR_B_net_net_3 <= \EV_RAM_ADDR_B_net_net_3\;
    EV_RAM_ADDR_B_net_net_6 <= \EV_RAM_ADDR_B_net_net_6\;
    bskrjhjvqpf_1_sqmuxa <= \bskrjhjvqpf_1_sqmuxa\;

    \qrbxknbdvjj[4]\ : DFN1E1C0
      port map(D => \EV_RAM_ADDR_B_net_net_3\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \tkcbxxqbtfx[11]_net_1\, Q => 
        \qrbxknbdvjj[4]_net_1\);
    
    \bnwcvzjbgtc_7[10]\ : NOR2A
      port map(A => N_642, B => rdgwgtpkbgc_3, Y => N_710);
    
    G_166 : XNOR3
      port map(A => \gjkdbmpgrtc[15]\, B => 
        \bnvtvdkcnpp[9]_net_1\, C => \bnwcvzjbgtc[15]\, Y => 
        N_311);
    
    \rdgwgtpkbgc[2]\ : DFN1E1C0
      port map(D => EV_RAM_DO_B_net_net_5, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \bskrjhjvqpf_1_sqmuxa\, Q => 
        \rdgwgtpkbgc[2]_net_1\);
    
    \zbqggxmnpbs_r[5]\ : AOI1B
      port map(A => N_1148, B => \zbqggxmnpbs_0_iv_0_0[5]_net_1\, 
        C => N_185, Y => EV_RAM_DI_B_net_net_2);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0[2]\ : OR3C
      port map(A => N_1057, B => \dhzrwbnrbpw_7_iv_0_0[2]\, C => 
        N_1060, Y => \dhzrwbnrbpw_7[2]\);
    
    wqpbnjzjqsx_1_0_a2 : XNOR2
      port map(A => \rqrqhcwpqdn[2]_net_1\, B => 
        \bnvtvdkcnpp[2]_net_1\, Y => wqpbnjzjqsx_1_i);
    
    \bnvtvdkcnpp_5[1]\ : MX2C
      port map(A => \bnvtvdkcnpp[1]_net_1\, B => 
        EV_RAM_DO_B_net_net_6, S => un3_qvqqfftqsfc, Y => 
        \bnvtvdkcnpp_5[1]_net_1\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_1 : XNOR2
      port map(A => \bnvtvdkcnpp[8]_net_1\, B => 
        \rqrqhcwpqdn[8]_net_1\, Y => \DWACT_BL_EQUAL_0_E_0[1]\);
    
    \gjkdbmpgrtc_11[12]\ : MX2
      port map(A => N_444, B => \rqrqhcwpqdn[6]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_576);
    
    \gjkdbmpgrtc_11[13]\ : MX2
      port map(A => N_956, B => \rqrqhcwpqdn[7]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_577);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_58 : AOI1A
      port map(A => \ACT_LT4_E[0]\, B => \ACT_LT4_E[1]\, C => 
        \ACT_LT4_E[2]\, Y => \ACT_LT4_E[3]\);
    
    un3_ckghxwbsztp_I_5 : XOR2
      port map(A => \rqrqhcwpqdn[1]_net_1\, B => 
        \rqrqhcwpqdn[2]_net_1\, Y => I_5_5);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_35 : XNOR2
      port map(A => \bnvtvdkcnpp[5]_net_1\, B => 
        \rqrqhcwpqdn[5]_net_1\, Y => \DWACT_BL_EQUAL_0_E[1]\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_60 : NOR2A
      port map(A => \rqrqhcwpqdn[3]_net_1\, B => 
        \bnvtvdkcnpp[3]_net_1\, Y => \ACT_LT4_E[5]\);
    
    cmkddgvkrrs_0_iv_i_o2 : AO1B
      port map(A => \gpjgkmhxbkn_NE_0\, B => \gpjgkmhxbkn_NE_2\, 
        C => \rqrqhcwpqdn[0]_net_1\, Y => N_1128);
    
    tddhwjzbsrj : DFN1E1C0
      port map(D => cmkddgvkrrs, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \tkcbxxqbtfx[1]_net_1\, Q => 
        \tddhwjzbsrj\);
    
    \tkcbxxqbtfx_ns_a3_0_a2[1]\ : NOR3C
      port map(A => \rgmpzqrbghh_0\, B => \tkcbxxqbtfx[17]_net_1\, 
        C => INIT_DONE_c, Y => \tkcbxxqbtfx_ns[1]\);
    
    G_136 : XNOR3
      port map(A => \gjkdbmpgrtc[9]\, B => \bnvtvdkcnpp[3]_net_1\, 
        C => \bnwcvzjbgtc[9]\, Y => N_281);
    
    \gjkdbmpgrtc_7_0_a2[1]\ : NOR2B
      port map(A => rdgwgtpkbgc_3, B => N_437, Y => N_957);
    
    \gjkdbmpgrtc_0_a3_1[17]\ : OR2
      port map(A => \rdgwgtpkbgc[1]_net_1\, B => 
        \rdgwgtpkbgc[0]_net_1\, Y => N_1012_1);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_20 : OR2A
      port map(A => \rqrqhcwpqdn[11]_net_1\, B => 
        \bnvtvdkcnpp[11]_net_1\, Y => N_31);
    
    \rfrdbqfdrhx[0]\ : DFN1E1C0
      port map(D => EV_RAM_DO_B_net_net_3, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => bskrjhjvqpf_0_sqmuxa, Q => 
        \rfrdbqfdrhx[0]_net_1\);
    
    \bnwcvzjbgtc_7[2]\ : MX2
      port map(A => N_634, B => N_642, S => rdgwgtpkbgc_3, Y => 
        N_702);
    
    \bnwcvzjbgtc_9[3]\ : MX2
      port map(A => \bnvtvdkcnpp[3]_net_1\, B => 
        \bnvtvdkcnpp[4]_net_1\, S => \rdgwgtpkbgc[0]_net_1\, Y
         => N_737);
    
    \tkcbxxqbtfx_0[4]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[13]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx_0[4]_net_1\);
    
    rgmpzqrbghh : DFN1C0
      port map(D => hmzcpnxsdwg, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \rgmpzqrbghh\);
    
    \gjkdbmpgrtc_12[9]\ : MX2
      port map(A => N_505, B => N_573, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc[9]\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_46 : OR2A
      port map(A => \rqrqhcwpqdn[6]_net_1\, B => 
        \bnvtvdkcnpp[6]_net_1\, Y => \ACT_LT3_E[4]\);
    
    \bnwcvzjbgtc_11[11]\ : MX2
      port map(A => N_647, B => \bnvtvdkcnpp[5]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_779);
    
    \qrbxknbdvjj[2]\ : DFN1E1C0
      port map(D => \EV_RAM_ADDR_B_net_net_5\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \tkcbxxqbtfx[11]_net_1\, Q => 
        \qrbxknbdvjj[2]_net_1\);
    
    kkjrqmngvjc_0_0_a2_0_0_a2 : NOR2
      port map(A => bskrjhjvqpf_6_sqmuxa_2, B => 
        bskrjhjvqpf_6_sqmuxa_1, Y => bskrjhjvqpf_6_sqmuxa);
    
    \gjkdbmpgrtc_7[6]\ : MX2
      port map(A => N_434, B => N_442, S => rdgwgtpkbgc_3, Y => 
        N_502);
    
    \bnvtvdkcnpp_5[4]\ : MX2
      port map(A => \bnvtvdkcnpp[4]_net_1\, B => 
        EV_RAM_DO_B_net_net_3, S => un3_qvqqfftqsfc, Y => 
        \bnvtvdkcnpp_5[4]_net_1\);
    
    un1_dhzrwbnrbpw_I_47 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_2[0]\, B => 
        \EV_RAM_ADDR_B_net_net_3\, Y => 
        \DWACT_ADD_CI_0_g_array_12_1[0]\);
    
    \bnwcvzjbgtc_9[5]\ : MX2
      port map(A => \bnvtvdkcnpp[5]_net_1\, B => 
        \bnvtvdkcnpp[6]_net_1\, S => \rdgwgtpkbgc[0]_net_1\, Y
         => N_739);
    
    \bnwcvzjbgtc_3[8]\ : MX2
      port map(A => N_736, B => N_738, S => 
        \rdgwgtpkbgc[1]_net_1\, Y => N_640);
    
    \bnwcvzjbgtc_11[9]\ : MX2
      port map(A => N_645, B => \bnvtvdkcnpp[3]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_777);
    
    \ckghxwbsztp_3_0_a2[4]\ : OA1A
      port map(A => N_1107, B => \tkcbxxqbtfx[17]_net_1\, C => 
        I_20, Y => \ckghxwbsztp_3[4]\);
    
    \bnwcvzjbgtc_3[14]\ : MX2
      port map(A => N_742, B => N_744, S => 
        \rdgwgtpkbgc[1]_net_1\, Y => N_646);
    
    \bnwcvzjbgtc_9[1]\ : MX2
      port map(A => \bnvtvdkcnpp[1]_net_1\, B => 
        \bnvtvdkcnpp[2]_net_1\, S => \rdgwgtpkbgc[0]_net_1\, Y
         => N_735);
    
    \gjkdbmpgrtc_3[16]\ : NOR2A
      port map(A => N_540, B => \rdgwgtpkbgc[1]_net_1\, Y => 
        N_444);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_5 : XNOR2
      port map(A => \bnvtvdkcnpp[7]_net_1\, B => 
        \rqrqhcwpqdn[7]_net_1\, Y => \DWACT_BL_EQUAL_0_E_0[0]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I15_CO1_i_o2 : MIN3
      port map(A => N_307, B => N_311, C => N_47, Y => N_38);
    
    un1_dhzrwbnrbpw_I_33 : XOR2
      port map(A => \EV_RAM_ADDR_B_net_net_2\, B => 
        \DWACT_ADD_CI_0_g_array_12_1[0]\, Y => I_33_1);
    
    \bnwcvzjbgtc_11[2]\ : MX2
      port map(A => N_638, B => \ftbkrvpxdbp[2]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_770);
    
    G_105 : AO13
      port map(A => \gjkdbmpgrtc[2]\, B => \ftbkrvpxdbp[2]_net_1\, 
        C => \bnwcvzjbgtc[2]\, Y => N_247);
    
    un3_ckghxwbsztp_I_19 : NOR2B
      port map(A => \rqrqhcwpqdn[4]_net_1\, B => 
        \DWACT_FINC_E[0]\, Y => N_4);
    
    wqpbnjzjqsx_3_0_a2 : XNOR2
      port map(A => \rqrqhcwpqdn[4]_net_1\, B => 
        \bnvtvdkcnpp[4]_net_1\, Y => wqpbnjzjqsx_3_i);
    
    \ckghxwbsztp_3_0_a2[2]\ : OA1A
      port map(A => N_1107, B => \tkcbxxqbtfx[17]_net_1\, C => 
        I_9_5, Y => \ckghxwbsztp_3[2]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I11_CO1_i_o2 : MIN3
      port map(A => N_287, B => N_291, C => N_48, Y => N_37);
    
    un1_dhzrwbnrbpw_I_44 : NOR2B
      port map(A => \DWACT_ADD_CI_0_TMP[0]\, B => 
        \EV_RAM_ADDR_B_net_net_6\, Y => 
        \DWACT_ADD_CI_0_g_array_1[0]\);
    
    \rqrqhcwpqdn_5_i[0]\ : MX2
      port map(A => EV_RAM_DO_B_net_net_7, B => N_1122, S => 
        N_1127, Y => \rqrqhcwpqdn_5_i[0]_net_1\);
    
    \bnwcvzjbgtc_7_0_a2[15]\ : NOR2A
      port map(A => N_647, B => rdgwgtpkbgc_3, Y => N_972);
    
    \rqrqhcwpqdn_5_i_a2_0[6]\ : NOR2
      port map(A => \rqrqhcwpqdn[6]_net_1\, B => N_1010_i, Y => 
        N_993);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_0[6]\ : OR3B
      port map(A => \tkcbxxqbtfx[8]_net_1\, B => 
        \qrbxknbdvjj[6]_net_1\, C => \rgmpzqrbghh\, Y => N_1074);
    
    \gjkdbmpgrtc_9[9]\ : MX2
      port map(A => \rqrqhcwpqdn[9]_net_1\, B => 
        \rqrqhcwpqdn[10]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_539);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_17 : OR2A
      port map(A => \rqrqhcwpqdn[8]_net_1\, B => 
        \bnvtvdkcnpp[8]_net_1\, Y => N_28);
    
    \bnwcvzjbgtc_3[4]\ : MX2
      port map(A => N_717, B => N_734, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_636);
    
    \bnvtvdkcnpp_9[2]\ : MX2
      port map(A => \bnvtvdkcnpp_5[2]_net_1\, B => 
        \fqkdsnjfpdb[8]\, S => \tkcbxxqbtfx_0[4]_net_1\, Y => 
        \bnvtvdkcnpp_9[2]_net_1\);
    
    bskrjhjvqpf_i_0_1 : NOR3
      port map(A => \tkcbxxqbtfx[9]_net_1\, B => 
        \tkcbxxqbtfx[15]_net_1\, C => \tkcbxxqbtfx[11]_net_1\, Y
         => \bskrjhjvqpf_i_0_1\);
    
    G_125 : AO13
      port map(A => \bnvtvdkcnpp[0]_net_1\, B => \gjkdbmpgrtc[6]\, 
        C => \bnwcvzjbgtc[6]\, Y => N_267);
    
    \dhzrwbnrbpw[0]\ : DFN1C0
      port map(D => \dhzrwbnrbpw_7[0]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \EV_RAM_ADDR_B_net_net_7\);
    
    un1_kkgccghmrzf : OR3B
      port map(A => \rfrdbqfdrhx[0]_net_1\, B => 
        \tkcbxxqbtfx[8]_net_1\, C => \rfrdbqfdrhx[1]_net_1\, Y
         => \un1_kkgccghmrzf\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I16_S : XOR3
      port map(A => N_312_i_0, B => N_316_i_0, C => N_38, Y => 
        \fqkdsnjfpdb_i[16]\);
    
    \gjkdbmpgrtc_6_0_a2[9]\ : NOR2A
      port map(A => \rqrqhcwpqdn[11]_net_1\, B => N_1012_1, Y => 
        N_956);
    
    \bnwcvzjbgtc_9_0_a2[11]\ : NOR2A
      port map(A => \bnvtvdkcnpp[11]_net_1\, B => 
        \rdgwgtpkbgc[0]_net_1\, Y => N_974);
    
    un1_dhzrwbnrbpw_I_53 : AND2
      port map(A => \EV_RAM_ADDR_B_net_net_3\, B => 
        \EV_RAM_ADDR_B_net_net_2\, Y => 
        \DWACT_ADD_CI_0_pog_array_1_1[0]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I14_un1_CO1_i_o3 : MAJ3
      port map(A => N_306, B => N_302, C => N255, Y => N_47);
    
    \zbqggxmnpbs_0_iv[4]\ : AO1B
      port map(A => \fqkdsnjfpdb[10]\, B => 
        \tkcbxxqbtfx[5]_net_1\, C => 
        \zbqggxmnpbs_0_iv_0[4]_net_1\, Y => EV_RAM_DI_B_net_net_3);
    
    \ckghxwbsztp_3_0_a2[1]\ : OA1A
      port map(A => N_1107, B => \tkcbxxqbtfx[17]_net_1\, C => 
        I_5_5, Y => \ckghxwbsztp_3[1]\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_36 : XNOR2
      port map(A => \bnvtvdkcnpp[6]_net_1\, B => 
        \rqrqhcwpqdn[6]_net_1\, Y => \DWACT_BL_EQUAL_0_E[2]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I17_Y_2 : XOR2
      port map(A => ADD_18x18_slow_I17_Y_1, B => N_317, Y => 
        ADD_18x18_slow_I17_Y_2);
    
    \bnvtvdkcnpp_5[3]\ : MX2
      port map(A => \bnvtvdkcnpp[3]_net_1\, B => 
        EV_RAM_DO_B_net_net_4, S => un3_qvqqfftqsfc, Y => 
        \bnvtvdkcnpp_5[3]_net_1\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I17_Y : XNOR2
      port map(A => ADD_18x18_slow_I17_Y_2, B => I16_un1_CO1_i, Y
         => \fqkdsnjfpdb[17]\);
    
    \bnwcvzjbgtc_12[2]\ : MX2
      port map(A => N_702, B => N_770, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => \bnwcvzjbgtc[2]\);
    
    \tkcbxxqbtfx_ns_a3_0_a2[12]\ : NOR2B
      port map(A => INIT_DONE_c, B => \tkcbxxqbtfx[6]_net_1\, Y
         => \tkcbxxqbtfx_ns[12]\);
    
    \gjkdbmpgrtc_12[3]\ : MX2
      port map(A => N_499, B => N_965, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc[3]\);
    
    xwhwqsjfmgh_0 : DFN1C0
      port map(D => bskrjhjvqpf_6_sqmuxa, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => EV_DONE_net_0);
    
    \bnvtvdkcnpp[5]\ : DFN1C0
      port map(D => \bnvtvdkcnpp_9[5]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \bnvtvdkcnpp[5]_net_1\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_1[4]\ : OR3B
      port map(A => \tkcbxxqbtfx[8]_net_1\, B => 
        \qrbxknbdvjj[4]_net_1\, C => \rgmpzqrbghh\, Y => N_1067);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I8_S_0 : XOR2
      port map(A => N_276_i, B => N_272_i, Y => 
        ADD_18x18_slow_I8_S_0);
    
    \rdgwgtpkbgc_1[2]\ : DFN1E1C0
      port map(D => EV_RAM_DO_B_net_net_5, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \bskrjhjvqpf_1_sqmuxa\, Q => 
        \rdgwgtpkbgc_1[2]_net_1\);
    
    \gjkdbmpgrtc_3_0_a2[4]\ : NOR2B
      port map(A => \rdgwgtpkbgc[1]_net_1\, B => N_530, Y => 
        N_955);
    
    \tkcbxxqbtfx_ns_a3_0_a2[6]\ : NOR3
      port map(A => \Z\\EV_RAM_DO_B_net\\\, B => N_1160, C => 
        \tkcbxxqbtfx_ns_1[14]\, Y => \tkcbxxqbtfx_ns[6]\);
    
    \ckghxwbsztp[1]\ : DFN1E1C0
      port map(D => \ckghxwbsztp_3[1]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \tkcbxxqbtfx[1]_net_1\, Q => 
        \ckghxwbsztp[1]_net_1\);
    
    un1_dhzrwbnrbpw_I_49 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_1[0]\, B => 
        \EV_RAM_ADDR_B_net_net_5\, Y => 
        \DWACT_ADD_CI_0_g_array_12[0]\);
    
    \bnwcvzjbgtc_8[1]\ : MX2
      port map(A => \ftbkrvpxdbp[5]_net_1\, B => 
        \bnvtvdkcnpp[0]_net_1\, S => \rdgwgtpkbgc[0]_net_1\, Y
         => N_718);
    
    \rqrqhcwpqdn_5_i_a2[3]\ : NOR2
      port map(A => N_1011_i, B => ASSC_RAM_DO_A_net_net_4, Y => 
        N_983);
    
    G_101_0 : XOR2
      port map(A => \ftbkrvpxdbp[2]_net_1\, B => \gjkdbmpgrtc[2]\, 
        Y => \G_101_0\);
    
    \ftbkrvpxdbp[4]\ : DFN1E0C0
      port map(D => \ftbkrvpxdbp_5[4]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ftbkrvpxdbp_1_sqmuxa\, Q => 
        \ftbkrvpxdbp[4]_net_1\);
    
    \gjkdbmpgrtc_3[9]\ : MX2
      port map(A => N_533, B => N_535, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_437);
    
    \fqkdsnjfpdb_m[13]\ : OR2A
      port map(A => \tkcbxxqbtfx[5]_net_1\, B => 
        \fqkdsnjfpdb_i[13]\, Y => \fqkdsnjfpdb_m[13]_net_1\);
    
    \bnwcvzjbgtc_12[7]\ : MX2
      port map(A => N_707, B => N_775, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => \bnwcvzjbgtc[7]\);
    
    \bnwcvzjbgtc_3[1]\ : MX2
      port map(A => N_599, B => N_616, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_633);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_57 : AND2A
      port map(A => \rqrqhcwpqdn[1]_net_1\, B => 
        \bnvtvdkcnpp[1]_net_1\, Y => \ACT_LT4_E[2]\);
    
    \bnwcvzjbgtc_12[12]\ : MX2
      port map(A => N_712, B => N_780, S => 
        \rdgwgtpkbgc[2]_net_1\, Y => \bnwcvzjbgtc[12]\);
    
    \bnwcvzjbgtc_3_0_a2[16]\ : NOR2A
      port map(A => N_744, B => \rdgwgtpkbgc[1]_net_1\, Y => 
        N_970);
    
    \kkkmkxwwscv[0]\ : DFN1E0C0
      port map(D => ASSC_SEQOUT_net_net_4, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => qmmwhjxxxmv, Q => 
        EV_ASSCADDR_net_net_4);
    
    \bnwcvzjbgtc_11[7]\ : MX2
      port map(A => N_643, B => \bnvtvdkcnpp[1]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_775);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I3_S : XOR3
      port map(A => N_247, B => N_251, C => N_50, Y => 
        \fqkdsnjfpdb[3]\);
    
    \bnvtvdkcnpp_9[4]\ : MX2
      port map(A => \bnvtvdkcnpp_5[4]_net_1\, B => 
        \fqkdsnjfpdb[10]\, S => \tkcbxxqbtfx_0[4]_net_1\, Y => 
        \bnvtvdkcnpp_9[4]_net_1\);
    
    \rqrqhcwpqdn[2]\ : DFN1C0
      port map(D => \rqrqhcwpqdn_5_i[2]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \rqrqhcwpqdn[2]_net_1\);
    
    \gjkdbmpgrtc_3[12]\ : MX2
      port map(A => N_536, B => N_538, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_440);
    
    \bnwcvzjbgtc_7[5]\ : MX2
      port map(A => N_637, B => N_645, S => rdgwgtpkbgc_3, Y => 
        N_705);
    
    \tkcbxxqbtfx[2]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[15]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[2]_net_1\);
    
    \rqrqhcwpqdn_5_i_a2[1]\ : NOR2
      port map(A => N_1011_i, B => ASSC_RAM_DO_A_net_net_6, Y => 
        N_977);
    
    G_140 : AO13
      port map(A => \bnvtvdkcnpp[3]_net_1\, B => \gjkdbmpgrtc[9]\, 
        C => \bnwcvzjbgtc[9]\, Y => N_282);
    
    \bnvtvdkcnpp_9[10]\ : MX2C
      port map(A => \bnvtvdkcnpp_8[10]_net_1\, B => 
        \fqkdsnjfpdb_i[16]\, S => \tkcbxxqbtfx[4]_net_1\, Y => 
        \bnvtvdkcnpp_9[10]_net_1\);
    
    xwhwqsjfmgh : DFN1C0
      port map(D => bskrjhjvqpf_6_sqmuxa, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => EV_DONE_net);
    
    \rqrqhcwpqdn_5_i[8]\ : NOR3
      port map(A => N_999, B => N_1000, C => N_998, Y => 
        \rqrqhcwpqdn_5_i[8]_net_1\);
    
    un1_dhzrwbnrbpw_I_37 : XOR2
      port map(A => \EV_RAM_ADDR_B_net_net_4\, B => 
        \DWACT_ADD_CI_0_g_array_12[0]\, Y => I_37_1);
    
    \gjkdbmpgrtc_7[8]\ : MX2
      port map(A => N_436, B => N_444, S => rdgwgtpkbgc_3, Y => 
        N_504);
    
    un3_ckghxwbsztp_I_20 : XOR2
      port map(A => N_4, B => \rqrqhcwpqdn[5]_net_1\, Y => I_20);
    
    \ftbkrvpxdbp_5[3]\ : MX2
      port map(A => EV_RAM_DO_B_net_net_1, B => \fqkdsnjfpdb[3]\, 
        S => \tkcbxxqbtfx_0[4]_net_1\, Y => 
        \ftbkrvpxdbp_5[3]_net_1\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a3_1[0]\ : NOR2
      port map(A => \rgmpzqrbghh\, B => N_1044, Y => N_1088);
    
    bskrjhjvqpf_5_sqmuxa_0_a3_0_0_a3_0_a2 : OR2A
      port map(A => \tkcbxxqbtfx[12]_net_1\, B => 
        EV_RAM_DO_B_net_net_1, Y => N_1159);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I4_un1_CO1_i_o3_0 : AO1
      port map(A => N_252_i, B => N_35, C => N_256_i, Y => 
        ADD_18x18_slow_I4_un1_CO1_i_o3_0);
    
    \mkmkbnghgkd[5]\ : MX2
      port map(A => INIT_ADDR_int(5), B => TR_EV_ADDR_net_net_2, 
        S => INIT_DONE_c, Y => EV_RAM_ADDR_A_net_net_2);
    
    \tkcbxxqbtfx[13]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[4]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[13]_net_1\);
    
    \rqrqhcwpqdn_5_i_a2_1[3]\ : NOR2
      port map(A => EV_RAM_DO_B_net_net_4, B => N_1127, Y => 
        N_985);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0[1]\ : OR3C
      port map(A => N_1053, B => \dhzrwbnrbpw_7_iv_0_0[1]\, C => 
        N_1056, Y => \dhzrwbnrbpw_7[1]\);
    
    \bnwcvzjbgtc_3[3]\ : MX2
      port map(A => N_616, B => N_718, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_635);
    
    \gjkdbmpgrtc_3[5]\ : MX2
      port map(A => N_960, B => N_531, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_433);
    
    \rqrqhcwpqdn_5_i_a2_1[8]\ : NOR2
      port map(A => \EV_RAM_DO_B_net[8]\, B => N_1127, Y => 
        N_1000);
    
    \bnwcvzjbgtc_11[4]\ : MX2
      port map(A => N_640, B => \ftbkrvpxdbp[4]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_772);
    
    un1_dhzrwbnrbpw_I_34 : XOR2
      port map(A => \Z\\EV_RAM_ADDR_B_net\\_net_1\, B => 
        \DWACT_ADD_CI_0_g_array_12_2[0]\, Y => I_34_1);
    
    \tkcbxxqbtfx_ns_a3_0_a2[10]\ : NOR2B
      port map(A => INIT_DONE_c, B => \bskrjhjvqpf_1_sqmuxa\, Y
         => \tkcbxxqbtfx_ns[10]\);
    
    \bnwcvzjbgtc_11[13]\ : MX2
      port map(A => N_971, B => \bnvtvdkcnpp[7]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_781);
    
    \rqrqhcwpqdn_5_i[1]\ : NOR3
      port map(A => N_978, B => N_979, C => N_977, Y => 
        \rqrqhcwpqdn_5_i[1]_net_1\);
    
    \rqrqhcwpqdn_5_i_a2_0[2]\ : NOR2
      port map(A => \rqrqhcwpqdn[2]_net_1\, B => N_1010_i, Y => 
        N_981);
    
    \rqrqhcwpqdn_5_i_a2_1[6]\ : NOR2
      port map(A => EV_RAM_DO_B_net_net_1, B => N_1127, Y => 
        N_994);
    
    \ckghxwbsztp_3_0_a2[3]\ : OA1A
      port map(A => N_1107, B => \tkcbxxqbtfx[17]_net_1\, C => 
        I_13_6, Y => \ckghxwbsztp_3[3]\);
    
    \tkcbxxqbtfx_ns_a3_0_a2[9]\ : NOR2B
      port map(A => INIT_DONE_c, B => \tkcbxxqbtfx[9]_net_1\, Y
         => \tkcbxxqbtfx_ns[9]\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0[0]\ : AO1B
      port map(A => N_1086, B => \DWACT_ADD_CI_0_partial_sum[0]\, 
        C => \dhzrwbnrbpw_7_iv_0_0[0]\, Y => \dhzrwbnrbpw_7[0]\);
    
    \rqrqhcwpqdn_5_i[6]\ : NOR3
      port map(A => N_993, B => N_994, C => N_992, Y => 
        \rqrqhcwpqdn_5_i[6]_net_1\);
    
    \ftbkrvpxdbp[5]\ : DFN1E0C0
      port map(D => \ftbkrvpxdbp_5[5]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ftbkrvpxdbp_1_sqmuxa\, Q => 
        \ftbkrvpxdbp[5]_net_1\);
    
    \bnwcvzjbgtc_12[8]\ : MX2
      port map(A => N_708, B => N_776, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => \bnwcvzjbgtc[8]\);
    
    \zbqggxmnpbs_0_iv_0[4]\ : AOI1B
      port map(A => \fqkdsnjfpdb[1]\, B => 
        \tkcbxxqbtfx_0[4]_net_1\, C => \ckghxwbsztp_m[3]_net_1\, 
        Y => \zbqggxmnpbs_0_iv_0[4]_net_1\);
    
    un3_qvqqfftqsfc_0_a2 : OR3B
      port map(A => \rfrdbqfdrhx[0]_net_1\, B => 
        \tkcbxxqbtfx[9]_net_1\, C => \rfrdbqfdrhx[1]_net_1\, Y
         => N_1133);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I10_S : XOR3
      port map(A => N_282, B => N_286, C => N247, Y => 
        \fqkdsnjfpdb[10]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I11_S : XNOR2
      port map(A => ADD_18x18_slow_I11_S_0, B => N_48, Y => 
        \fqkdsnjfpdb_i[11]\);
    
    \rqrqhcwpqdn_9_i_a2[9]\ : NOR3B
      port map(A => N_937, B => \tkcbxxqbtfx[6]_net_1\, C => 
        ASSC_RAM_DO_A_net_net_7, Y => N_1001);
    
    bskrjhjvqpf_i_0_o3_i_a2 : OR2A
      port map(A => EV_RAM_DO_B_net_net_1, B => 
        \EV_RAM_DO_B_net[8]\, Y => N_1160);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_72 : AO1
      port map(A => \DWACT_CMPLE_PO2_DWACT_COMP0_E[1]\, B => 
        \DWACT_CMPLE_PO2_DWACT_COMP0_E[2]\, C => 
        \DWACT_CMPLE_PO2_DWACT_COMP0_E[0]\, Y => 
        \DWACT_COMP0_E[2]\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0[3]\ : OR3C
      port map(A => N_1061, B => \dhzrwbnrbpw_7_iv_0_0[3]\, C => 
        N_1064, Y => \dhzrwbnrbpw_7[3]\);
    
    \ckghxwbsztp[0]\ : DFN1E1C0
      port map(D => \ckghxwbsztp_3[0]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \tkcbxxqbtfx[1]_net_1\, Q => 
        \ckghxwbsztp[0]_net_1\);
    
    \bnvtvdkcnpp_5[7]\ : MX2C
      port map(A => \bnvtvdkcnpp[7]_net_1\, B => 
        \Z\\EV_RAM_DO_B_net\\\, S => un3_qvqqfftqsfc, Y => 
        \bnvtvdkcnpp_5[7]_net_1\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_61 : NOR2A
      port map(A => \ACT_LT4_E[4]\, B => \ACT_LT4_E[5]\, Y => 
        \ACT_LT4_E[6]\);
    
    \bnwcvzjbgtc_11[10]\ : MX2
      port map(A => N_646, B => \bnvtvdkcnpp[4]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_778);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_0[6]\ : AOI1B
      port map(A => N_1088, B => EV_RAM_DO_B_net_net_1, C => 
        N_1074, Y => \dhzrwbnrbpw_7_iv_0_0[6]\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_0[1]\ : AOI1B
      port map(A => \rgmpzqrbghh_0\, B => \EV_CHHOLD_net_net_3\, 
        C => N_1055, Y => \dhzrwbnrbpw_7_iv_0_0[1]\);
    
    un1_dhzrwbnrbpw_I_54 : AND2
      port map(A => \EV_RAM_ADDR_B_net_net_1\, B => 
        \Z\\EV_RAM_ADDR_B_net\\_net_1\, Y => 
        \DWACT_ADD_CI_0_pog_array_1_2[0]\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_21 : AO1C
      port map(A => \rqrqhcwpqdn[8]_net_1\, B => 
        \bnvtvdkcnpp[8]_net_1\, C => N_30, Y => N_32);
    
    G_115 : AO13
      port map(A => \ftbkrvpxdbp[4]_net_1\, B => \gjkdbmpgrtc[4]\, 
        C => \bnwcvzjbgtc[4]\, Y => N_257);
    
    \gjkdbmpgrtc_7[0]\ : NOR2B
      port map(A => rdgwgtpkbgc_3, B => N_436, Y => N_496);
    
    gpjgkmhxbkn_0_0_a2 : XNOR2
      port map(A => EV_RAM_DO_B_net_net_6, B => 
        \rqrqhcwpqdn[1]_net_1\, Y => gpjgkmhxbkn_0_i);
    
    \bnwcvzjbgtc_11[1]\ : MX2
      port map(A => N_637, B => \ftbkrvpxdbp[1]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_769);
    
    \tkcbxxqbtfx[10]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[7]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[10]_net_1\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_m : OR2B
      port map(A => un2_pbzxzjhqhwz, B => dbdctxjdvkp_2_sqmuxa, Y
         => un2_pbzxzjhqhwz_m);
    
    \mtbjcfqsbqf[1]\ : DFN1E0C0
      port map(D => ADC_CHNR_net_net_2, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => qmmwhjxxxmv, Q => 
        \EV_CHHOLD_net_net_2\);
    
    qmmwhjxxxmv_0_a2 : AO1C
      port map(A => rhbzpsxhwms, B => jngrzvhtjvn_0, C => 
        ASSC_SAMPFLAG_net, Y => qmmwhjxxxmv);
    
    \tkcbxxqbtfx[7]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[10]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \EV_ASSCADDR_net_net_5\);
    
    \rqrqhcwpqdn_5_i[2]\ : NOR3
      port map(A => N_981, B => N_982, C => N_980, Y => 
        \rqrqhcwpqdn_5_i[2]_net_1\);
    
    \rqrqhcwpqdn_5_i_a2_0[5]\ : NOR2
      port map(A => \rqrqhcwpqdn[5]_net_1\, B => N_1010_i, Y => 
        N_990);
    
    \bnvtvdkcnpp_5[2]\ : MX2
      port map(A => \bnvtvdkcnpp[2]_net_1\, B => 
        EV_RAM_DO_B_net_net_5, S => un3_qvqqfftqsfc, Y => 
        \bnvtvdkcnpp_5[2]_net_1\);
    
    gpjgkmhxbkn_NE_2 : NOR3C
      port map(A => gpjgkmhxbkn_1_i, B => gpjgkmhxbkn_2_i, C => 
        gpjgkmhxbkn_0_i, Y => \gpjgkmhxbkn_NE_2\);
    
    zcjzqnkdqwb_iv : MX2C
      port map(A => INIT_EV_WEN, B => TR_EV_RD_net, S => 
        INIT_DONE_c, Y => EV_RAM_CSN_A_net);
    
    \rqrqhcwpqdn_5_i_a3[1]\ : OR2A
      port map(A => N_1127, B => \EV_ASSCADDR_net_net_5\, Y => 
        N_1010_i);
    
    \dhzrwbnrbpw[2]\ : DFN1C0
      port map(D => \dhzrwbnrbpw_7[2]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \EV_RAM_ADDR_B_net_net_5\);
    
    \ckghxwbsztp_m[1]\ : OR2B
      port map(A => \tkcbxxqbtfx[0]_net_1\, B => 
        \ckghxwbsztp[1]_net_1\, Y => \ckghxwbsztp_m[1]_net_1\);
    
    \rqrqhcwpqdn[3]\ : DFN1C0
      port map(D => \rqrqhcwpqdn_5_i[3]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \rqrqhcwpqdn[3]_net_1\);
    
    \dhzrwbnrbpw[1]\ : DFN1C0
      port map(D => \dhzrwbnrbpw_7[1]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \EV_RAM_ADDR_B_net_net_6\);
    
    \rqrqhcwpqdn_5_i_a2_0[1]\ : NOR2
      port map(A => \rqrqhcwpqdn[1]_net_1\, B => N_1010_i, Y => 
        N_978);
    
    \rqrqhcwpqdn_5_i_a2[4]\ : NOR2
      port map(A => N_1011_i, B => ASSC_RAM_DO_A_net_net_3, Y => 
        N_986);
    
    \bnvtvdkcnpp[8]\ : DFN1C0
      port map(D => \bnvtvdkcnpp_9[8]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \bnvtvdkcnpp[8]_net_1\);
    
    \tkcbxxqbtfx[15]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[2]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[15]_net_1\);
    
    \rqrqhcwpqdn_9_i_a2[11]\ : NOR3B
      port map(A => N_937, B => \tkcbxxqbtfx[6]_net_1\, C => 
        ASSC_RAM_DO_A_net_net_5, Y => N_1007);
    
    un1_bskrjhjvqpf_6_sqmuxa_1_i : OR2B
      port map(A => qmmwhjxxxmv, B => N_1048, Y => N_872);
    
    \rqrqhcwpqdn_5_i[3]\ : NOR3
      port map(A => N_984, B => N_985, C => N_983, Y => 
        \rqrqhcwpqdn_5_i[3]_net_1\);
    
    GND_i : GND
      port map(Y => \GND\);
    
    \ftbkrvpxdbp_5[5]\ : MX2
      port map(A => \EV_RAM_DO_B_net[8]\, B => \fqkdsnjfpdb[5]\, 
        S => \tkcbxxqbtfx_0[4]_net_1\, Y => 
        \ftbkrvpxdbp_5[5]_net_1\);
    
    G_165 : AO13
      port map(A => \bnvtvdkcnpp[8]_net_1\, B => 
        \gjkdbmpgrtc[14]\, C => \bnwcvzjbgtc[14]\, Y => N_307);
    
    \tkcbxxqbtfx_ns_a3[2]\ : NOR2B
      port map(A => INIT_DONE_c, B => \tkcbxxqbtfx[16]_net_1\, Y
         => \tkcbxxqbtfx_ns[2]\);
    
    \tkcbxxqbtfx_ns_0_o2[4]\ : OR3
      port map(A => N_199, B => \tkcbxxqbtfx[2]_net_1\, C => 
        N_1083, Y => N_1130);
    
    \bnvtvdkcnpp[2]\ : DFN1C0
      port map(D => \bnvtvdkcnpp_9[2]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \bnvtvdkcnpp[2]_net_1\);
    
    \tkcbxxqbtfx_ns_0_o2[0]\ : NOR2A
      port map(A => INIT_DONE_c, B => bskrjhjvqpf_6_sqmuxa, Y => 
        N_1048);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a3_0_0[0]\ : NOR2
      port map(A => \tkcbxxqbtfx[8]_net_1\, B => \rgmpzqrbghh_0\, 
        Y => \dhzrwbnrbpw_7_iv_0_a3_0[0]\);
    
    \bnwcvzjbgtc_7[14]\ : NOR2A
      port map(A => N_646, B => rdgwgtpkbgc_3, Y => N_714);
    
    \rqrqhcwpqdn_5_i_a3_0[1]\ : OR2B
      port map(A => \EV_ASSCADDR_net_net_5\, B => N_1127, Y => 
        N_1011_i);
    
    \gjkdbmpgrtc_11_0_a2[14]\ : NOR2B
      port map(A => \rqrqhcwpqdn[8]_net_1\, B => rdgwgtpkbgc_3, Y
         => N_967);
    
    \rqrqhcwpqdn[5]\ : DFN1C0
      port map(D => \rqrqhcwpqdn_5_i[5]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \rqrqhcwpqdn[5]_net_1\);
    
    \bnwcvzjbgtc_12[11]\ : MX2
      port map(A => N_711, B => N_779, S => 
        \rdgwgtpkbgc[2]_net_1\, Y => \bnwcvzjbgtc[11]\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_64 : AOI1A
      port map(A => \ACT_LT4_E[7]\, B => \ACT_LT4_E[8]\, C => 
        \ACT_LT4_E[5]\, Y => \ACT_LT4_E[10]\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0[6]\ : AO1B
      port map(A => N_1086, B => I_38_1, C => 
        \dhzrwbnrbpw_7_iv_0_0[6]\, Y => \dhzrwbnrbpw_7[6]\);
    
    G_135 : AO13
      port map(A => \bnvtvdkcnpp[2]_net_1\, B => \gjkdbmpgrtc[8]\, 
        C => \bnwcvzjbgtc[8]\, Y => N_277);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_24 : OR2A
      port map(A => \bnvtvdkcnpp[11]_net_1\, B => 
        \rqrqhcwpqdn[11]_net_1\, Y => N_35_0);
    
    \tkcbxxqbtfx[0]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[17]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[0]_net_1\);
    
    \tkcbxxqbtfx_ns_0[0]\ : AO1C
      port map(A => \rgmpzqrbghh\, B => \tkcbxxqbtfx[17]_net_1\, 
        C => N_1048, Y => \tkcbxxqbtfx_ns[0]\);
    
    \rqrqhcwpqdn_5_i_a2[8]\ : NOR2
      port map(A => N_1011_i, B => \ASSC_RAM_DO_A_net[8]\, Y => 
        N_998);
    
    zbcrjjxqjgg_i_o3 : AO1A
      port map(A => N_191, B => \tkcbxxqbtfx[12]_net_1\, C => 
        \tkcbxxqbtfx[5]_net_1\, Y => N_195);
    
    un3_qvqqfftqsfc_0 : OAI1
      port map(A => N_1159, B => bskrjhjvqpf_6_sqmuxa_1, C => 
        N_1133, Y => un3_qvqqfftqsfc);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_o3[0]\ : NOR2
      port map(A => \tkcbxxqbtfx[14]_net_1\, B => 
        \tkcbxxqbtfx[11]_net_1\, Y => N_1044);
    
    \bnwcvzjbgtc_3[13]\ : MX2
      port map(A => N_741, B => N_743, S => 
        \rdgwgtpkbgc[1]_net_1\, Y => N_645);
    
    \gjkdbmpgrtc_0_o3[17]\ : AXOI3
      port map(A => N_1012_1, B => \rdgwgtpkbgc[2]_net_1\, C => 
        rdgwgtpkbgc_3, Y => N_940);
    
    \bnwcvzjbgtc_12[16]\ : MX2
      port map(A => N_973, B => N_976, S => 
        \rdgwgtpkbgc[2]_net_1\, Y => \bnwcvzjbgtc[16]\);
    
    \mkmkbnghgkd[0]\ : MX2
      port map(A => INIT_ADDR(0), B => TR_EV_ADDR_net_net_7, S
         => INIT_DONE_c, Y => EV_RAM_ADDR_A_net_net_7);
    
    \gjkdbmpgrtc_11_0_a2[0]\ : NOR2A
      port map(A => N_955, B => rdgwgtpkbgc_3, Y => N_962);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I6_S : XOR2
      port map(A => ADD_18x18_slow_I6_S_0, B => N239, Y => 
        \fqkdsnjfpdb[6]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I16_un1_CO1_i : MAJ3
      port map(A => N_312_i_0, B => N_316_i_0, C => N_38, Y => 
        I16_un1_CO1_i);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I12_S : XNOR3
      port map(A => N_292_i, B => N_296_i, C => N_37, Y => 
        \fqkdsnjfpdb[12]\);
    
    bskrjhjvqpf_i_0 : AOI1B
      port map(A => N_1160, B => \bskrjhjvqpf_i_0_a2_0\, C => 
        \bskrjhjvqpf_i_0_2\, Y => \bskrjhjvqpf_i_0\);
    
    \bnwcvzjbgtc_3[2]\ : MX2
      port map(A => N_615, B => N_717, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_634);
    
    un1_dhzrwbnrbpw_I_21 : XOR2
      port map(A => \EV_RAM_ADDR_B_net_net_7\, B => 
        \bskrjhjvqpf_i_0\, Y => \DWACT_ADD_CI_0_partial_sum[0]\);
    
    \gjkdbmpgrtc_11[7]\ : MX2
      port map(A => N_439, B => \rqrqhcwpqdn[1]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_571);
    
    kkjrqmngvjc_0_0_1 : AND2
      port map(A => \kkjrqmngvjc_0_0_0\, B => N_180_5, Y => 
        \kkjrqmngvjc_0_0_1\);
    
    G_150 : AO18
      port map(A => \bnvtvdkcnpp[5]_net_1\, B => 
        \bnwcvzjbgtc[11]\, C => \gjkdbmpgrtc[11]\, Y => N_292_i);
    
    \rqrqhcwpqdn_5_i[7]\ : NOR3
      port map(A => N_996, B => N_997, C => N_995, Y => 
        \rqrqhcwpqdn_5_i[7]_net_1\);
    
    \gjkdbmpgrtc_7[5]\ : MX2
      port map(A => N_433, B => N_441, S => rdgwgtpkbgc_3, Y => 
        N_501);
    
    \rqrqhcwpqdn[8]\ : DFN1C0
      port map(D => \rqrqhcwpqdn_5_i[8]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \rqrqhcwpqdn[8]_net_1\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I17_Y_1 : MX2
      port map(A => \bnvtvdkcnpp[11]_net_1\, B => 
        \rqrqhcwpqdn[11]_net_1\, S => N_940, Y => 
        ADD_18x18_slow_I17_Y_1);
    
    \qrbxknbdvjj[8]\ : DFN1E1C0
      port map(D => \EV_RAM_ADDR_B_net[8]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, E => 
        \tkcbxxqbtfx[11]_net_1\, Q => \qrbxknbdvjj[8]_net_1\);
    
    bskrjhjvqpf_0_sqmuxa_0_a3_0_a2 : NOR3A
      port map(A => \tkcbxxqbtfx[12]_net_1\, B => 
        \Z\\EV_RAM_DO_B_net\\\, C => N_1160, Y => 
        bskrjhjvqpf_0_sqmuxa);
    
    \dhzrwbnrbpw[7]\ : DFN1C0
      port map(D => \dhzrwbnrbpw_7[7]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \Z\\EV_RAM_ADDR_B_net\\_net_1\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I7_S : XNOR3
      port map(A => N_267, B => N_271, C => N_49, Y => 
        \fqkdsnjfpdb_i[7]\);
    
    \gjkdbmpgrtc_7_0_a2[10]\ : NOR2A
      port map(A => N_438, B => rdgwgtpkbgc_3, Y => N_959);
    
    G_175 : AO18
      port map(A => \bnvtvdkcnpp[10]_net_1\, B => 
        \bnwcvzjbgtc[16]\, C => \gjkdbmpgrtc[16]\, Y => N_317);
    
    \gjkdbmpgrtc_12[16]\ : MX2
      port map(A => N_512, B => N_969, S => 
        \rdgwgtpkbgc[2]_net_1\, Y => \gjkdbmpgrtc[16]\);
    
    \gjkdbmpgrtc_12[15]\ : MX2
      port map(A => N_511, B => N_968, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => \gjkdbmpgrtc[15]\);
    
    bskrjhjvqpf_i_0_a2_0 : NOR2B
      port map(A => \tkcbxxqbtfx[12]_net_1\, B => 
        \Z\\EV_RAM_DO_B_net\\\, Y => \bskrjhjvqpf_i_0_a2_0\);
    
    \rdgwgtpkbgc_0[2]\ : DFN1E1C0
      port map(D => EV_RAM_DO_B_net_net_5, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \bskrjhjvqpf_1_sqmuxa\, Q => 
        \rdgwgtpkbgc_0[2]_net_1\);
    
    \bnwcvzjbgtc_9[10]\ : MX2
      port map(A => \bnvtvdkcnpp[10]_net_1\, B => 
        \bnvtvdkcnpp[11]_net_1\, S => \rdgwgtpkbgc[0]_net_1\, Y
         => N_744);
    
    \bnvtvdkcnpp[3]\ : DFN1C0
      port map(D => \bnvtvdkcnpp_9[3]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \bnvtvdkcnpp[3]_net_1\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2[2]\ : OR2B
      port map(A => EV_RAM_DO_B_net_net_5, B => N_1088, Y => 
        N_1057);
    
    \zbqggxmnpbs_0_iv_0_0[3]\ : AOI1B
      port map(A => \fqkdsnjfpdb[0]\, B => 
        \tkcbxxqbtfx_0[4]_net_1\, C => N_1147, Y => 
        \zbqggxmnpbs_0_iv_0_0[3]_net_1\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I1_S : XNOR3
      port map(A => N_241_i, B => \bnwcvzjbgtc[1]\, C => N_42, Y
         => \fqkdsnjfpdb[1]\);
    
    \rqrqhcwpqdn_9_i[10]\ : NOR3
      port map(A => N_1004, B => N_1006, C => N_1005, Y => 
        \rqrqhcwpqdn_9_i[10]_net_1\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I5_CO1_i_a3 : OR2B
      port map(A => N_41, B => N_261, Y => N_81_i);
    
    \gjkdbmpgrtc_11[8]\ : MX2
      port map(A => N_440, B => \rqrqhcwpqdn[2]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_572);
    
    \gjkdbmpgrtc_8_0_a2[1]\ : NOR2B
      port map(A => \rqrqhcwpqdn[0]_net_1\, B => 
        \rdgwgtpkbgc[0]_net_1\, Y => N_960);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_2[4]\ : OR2B
      port map(A => I_30_1, B => N_1086, Y => N_1068);
    
    \bnwcvzjbgtc_11_0_a2[15]\ : NOR2B
      port map(A => rdgwgtpkbgc_3, B => \bnvtvdkcnpp[9]_net_1\, Y
         => N_975);
    
    dbdctxjdvkp_2_sqmuxa_0_a3 : NOR2
      port map(A => bskrjhjvqpf_6_sqmuxa_2, B => N_1144, Y => 
        dbdctxjdvkp_2_sqmuxa);
    
    \mkmkbnghgkd[8]\ : MX2
      port map(A => INIT_ADDR_int(8), B => \TR_EV_ADDR_net[8]\, S
         => INIT_DONE_c, Y => \EV_RAM_ADDR_A_net[8]\);
    
    \gjkdbmpgrtc_3[7]\ : MX2
      port map(A => N_531, B => N_533, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_435);
    
    \bnwcvzjbgtc_7[3]\ : MX2
      port map(A => N_635, B => N_643, S => rdgwgtpkbgc_3, Y => 
        N_703);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_55 : NOR2A
      port map(A => \rqrqhcwpqdn[0]_net_1\, B => 
        \bnvtvdkcnpp[0]_net_1\, Y => \ACT_LT4_E[0]\);
    
    \qrbxknbdvjj[7]\ : DFN1E1C0
      port map(D => \Z\\EV_RAM_ADDR_B_net\\_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, E => 
        \tkcbxxqbtfx[11]_net_1\, Q => \qrbxknbdvjj[7]_net_1\);
    
    \zbqggxmnpbs_0_iv_0[1]\ : OA1A
      port map(A => \tkcbxxqbtfx[5]_net_1\, B => 
        \fqkdsnjfpdb_i[7]\, C => \ckghxwbsztp_m[0]_net_1\, Y => 
        \zbqggxmnpbs_0_iv_0[1]_net_1\);
    
    \zbqggxmnpbs_r[7]\ : AOI1B
      port map(A => \fqkdsnjfpdb_m[4]_net_1\, B => 
        \fqkdsnjfpdb_m[13]_net_1\, C => N_185, Y => 
        \Z\\EV_RAM_DI_B_net\\\);
    
    \bnvtvdkcnpp_8[10]\ : MX2C
      port map(A => EV_RAM_DO_B_net_net_6, B => 
        \bnvtvdkcnpp[10]_net_1\, S => \un1_kkgccghmrzf\, Y => 
        \bnvtvdkcnpp_8[10]_net_1\);
    
    \gjkdbmpgrtc_11[9]\ : MX2
      port map(A => N_441, B => \rqrqhcwpqdn[3]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_573);
    
    \ckghxwbsztp_m[3]\ : OR2B
      port map(A => \tkcbxxqbtfx[0]_net_1\, B => 
        \ckghxwbsztp[3]_net_1\, Y => \ckghxwbsztp_m[3]_net_1\);
    
    bskrjhjvqpf_i_o3_1_i_a2 : OR2
      port map(A => \tkcbxxqbtfx[14]_net_1\, B => 
        \tkcbxxqbtfx[8]_net_1\, Y => N_1083);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_7 : AND2
      port map(A => \DWACT_BL_EQUAL_0_E[4]\, B => 
        \DWACT_BL_EQUAL_0_E[3]\, Y => 
        \DWACT_BL_EQUAL_0_DWACT_ANDTREE_E[1]\);
    
    \tkcbxxqbtfx[4]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[13]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[4]_net_1\);
    
    \gjkdbmpgrtc_11_0_a2[3]\ : NOR2A
      port map(A => N_435, B => rdgwgtpkbgc_3, Y => N_965);
    
    \fqkdsnjfpdb_m[12]\ : OR2B
      port map(A => \tkcbxxqbtfx[5]_net_1\, B => 
        \fqkdsnjfpdb[12]\, Y => \fqkdsnjfpdb_m[12]_net_1\);
    
    \gjkdbmpgrtc_12[11]\ : MX2
      port map(A => N_507, B => N_575, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc[11]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I2_S : XOR2
      port map(A => ADD_18x18_slow_I2_S_0, B => N231, Y => 
        \fqkdsnjfpdb[2]\);
    
    \tkcbxxqbtfx_ns_a3[5]\ : NOR2B
      port map(A => INIT_DONE_c, B => \tkcbxxqbtfx[13]_net_1\, Y
         => \tkcbxxqbtfx_ns[5]\);
    
    \gjkdbmpgrtc_11[6]\ : MX2
      port map(A => N_438, B => \rqrqhcwpqdn[0]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_570);
    
    un1_dhzrwbnrbpw_I_42 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_2[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_1_1[0]\, Y => 
        \DWACT_ADD_CI_0_g_array_11[0]\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_62 : NOR2A
      port map(A => \bnvtvdkcnpp[2]_net_1\, B => 
        \rqrqhcwpqdn[2]_net_1\, Y => \ACT_LT4_E[7]\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_8 : AND2
      port map(A => \DWACT_BL_EQUAL_0_DWACT_ANDTREE_E[1]\, B => 
        \DWACT_BL_EQUAL_0_DWACT_ANDTREE_E[0]\, Y => 
        \DWACT_COMP0_E[1]\);
    
    G_141 : XNOR3
      port map(A => \gjkdbmpgrtc[10]\, B => 
        \bnvtvdkcnpp[4]_net_1\, C => \bnwcvzjbgtc[10]\, Y => 
        N_286);
    
    \fqkdsnjfpdb_m[8]\ : OR2B
      port map(A => \tkcbxxqbtfx[5]_net_1\, B => \fqkdsnjfpdb[8]\, 
        Y => \fqkdsnjfpdb_m[8]_net_1\);
    
    \gjkdbmpgrtc_12[10]\ : MX2
      port map(A => N_959, B => N_574, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc[10]\);
    
    \rqrqhcwpqdn[9]\ : DFN1C0
      port map(D => \rqrqhcwpqdn_9_i[9]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \rqrqhcwpqdn[9]_net_1\);
    
    gpjgkmhxbkn_2_0_a2 : XNOR2
      port map(A => EV_RAM_DO_B_net_net_4, B => 
        \rqrqhcwpqdn[3]_net_1\, Y => gpjgkmhxbkn_2_i);
    
    \tkcbxxqbtfx[3]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[14]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[3]_net_1\);
    
    \rqrqhcwpqdn_9_i_a2_0[9]\ : NOR3A
      port map(A => N_937, B => \tkcbxxqbtfx[6]_net_1\, C => 
        \rqrqhcwpqdn[9]_net_1\, Y => N_1002);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_22 : OA1A
      port map(A => \bnvtvdkcnpp[10]_net_1\, B => 
        \rqrqhcwpqdn[10]_net_1\, C => N_29, Y => N_33);
    
    \gjkdbmpgrtc_9_0_a2[11]\ : NOR2A
      port map(A => \rqrqhcwpqdn[11]_net_1\, B => 
        \rdgwgtpkbgc[0]_net_1\, Y => N_961);
    
    \bnwcvzjbgtc_12[0]\ : MX2
      port map(A => N_700, B => N_768, S => rdgwgtpkbgc_3, Y => 
        \bnwcvzjbgtc[0]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I15_S : XOR3
      port map(A => N_307, B => N_311, C => N_47, Y => 
        \fqkdsnjfpdb[15]\);
    
    \bnwcvzjbgtc_12[4]\ : MX2
      port map(A => N_704, B => N_772, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => \bnwcvzjbgtc[4]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I11_S_0 : XOR2
      port map(A => N_291, B => N_287, Y => 
        ADD_18x18_slow_I11_S_0);
    
    \kkkmkxwwscv[1]\ : DFN1E0C0
      port map(D => ASSC_SEQOUT_net_net_3, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => qmmwhjxxxmv, Q => 
        EV_ASSCADDR_net_net_3);
    
    \tkcbxxqbtfx[6]\ : DFN1C0
      port map(D => tkcbxxqbtfx_ns_11, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \tkcbxxqbtfx[6]_net_1\);
    
    \tkcbxxqbtfx[12]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[5]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[12]_net_1\);
    
    \gjkdbmpgrtc_9[4]\ : MX2
      port map(A => \rqrqhcwpqdn[4]_net_1\, B => 
        \rqrqhcwpqdn[5]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_534);
    
    wqpbnjzjqsx_NE_1 : XA1A
      port map(A => \bnvtvdkcnpp[3]_net_1\, B => 
        \rqrqhcwpqdn[3]_net_1\, C => wqpbnjzjqsx_1_i, Y => 
        \wqpbnjzjqsx_NE_1\);
    
    \gjkdbmpgrtc_7[7]\ : MX2
      port map(A => N_435, B => N_443, S => rdgwgtpkbgc_3, Y => 
        N_503);
    
    \rqrqhcwpqdn[4]\ : DFN1C0
      port map(D => \rqrqhcwpqdn_5_i[4]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \rqrqhcwpqdn[4]_net_1\);
    
    \bnwcvzjbgtc_7_0_a2[16]\ : NOR2A
      port map(A => N_970, B => rdgwgtpkbgc_3, Y => N_973);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_44 : AND2A
      port map(A => \rqrqhcwpqdn[5]_net_1\, B => 
        \bnvtvdkcnpp[5]_net_1\, Y => \ACT_LT3_E[2]\);
    
    \tkcbxxqbtfx_ns_a3[13]\ : NOR2B
      port map(A => INIT_DONE_c, B => \tkcbxxqbtfx[5]_net_1\, Y
         => \tkcbxxqbtfx_ns[13]\);
    
    un3_ckghxwbsztp_I_16 : AND3
      port map(A => \rqrqhcwpqdn[1]_net_1\, B => 
        \rqrqhcwpqdn[2]_net_1\, C => \rqrqhcwpqdn[3]_net_1\, Y
         => \DWACT_FINC_E[0]\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_0[0]\ : OR3B
      port map(A => \tkcbxxqbtfx[8]_net_1\, B => 
        \qrbxknbdvjj[0]_net_1\, C => \rgmpzqrbghh\, Y => N_1051);
    
    \bnwcvzjbgtc_9[0]\ : MX2
      port map(A => \bnvtvdkcnpp[0]_net_1\, B => 
        \bnvtvdkcnpp[1]_net_1\, S => \rdgwgtpkbgc[0]_net_1\, Y
         => N_734);
    
    \qrbxknbdvjj[6]\ : DFN1E1C0
      port map(D => \EV_RAM_ADDR_B_net_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \tkcbxxqbtfx[11]_net_1\, Q => 
        \qrbxknbdvjj[6]_net_1\);
    
    \bnvtvdkcnpp[9]\ : DFN1C0
      port map(D => \bnvtvdkcnpp_9[9]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \bnvtvdkcnpp[9]_net_1\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I5_CO1_i_o3 : OR2B
      port map(A => ADD_18x18_slow_I5_CO1_i_o3_0, B => N_81_i, Y
         => N239);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I13_S_0 : XOR2
      port map(A => N_301, B => N_297, Y => 
        ADD_18x18_slow_I13_S_0);
    
    \gjkdbmpgrtc_11_0_a2[2]\ : NOR2A
      port map(A => N_434, B => rdgwgtpkbgc_3, Y => N_964);
    
    \bnwcvzjbgtc_12[13]\ : MX2
      port map(A => N_713, B => N_781, S => 
        \rdgwgtpkbgc[2]_net_1\, Y => \bnwcvzjbgtc[13]\);
    
    \tkcbxxqbtfx_ns_a3_0_a2_1[14]\ : OR2B
      port map(A => INIT_DONE_c, B => \tkcbxxqbtfx[12]_net_1\, Y
         => \tkcbxxqbtfx_ns_1[14]\);
    
    \bnwcvzjbgtc_2[1]\ : MX2
      port map(A => \ftbkrvpxdbp[3]_net_1\, B => 
        \ftbkrvpxdbp[4]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_616);
    
    \tkcbxxqbtfx[11]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[6]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[11]_net_1\);
    
    \kkkmkxwwscv[2]\ : DFN1E0C0
      port map(D => ASSC_SEQOUT_net_net_2, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => qmmwhjxxxmv, Q => 
        EV_ASSCADDR_net_net_2);
    
    G_96 : XNOR3
      port map(A => \ftbkrvpxdbp[0]_net_1\, B => 
        \ftbkrvpxdbp[1]_net_1\, C => \gjkdbmpgrtc[1]\, Y => 
        N_241_i);
    
    un1_fskhwddcbbg_i : OR2
      port map(A => N_199, B => N_195, Y => N_185);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0[7]\ : AO1B
      port map(A => N_1086, B => I_34_1, C => 
        \dhzrwbnrbpw_7_iv_0_0[7]\, Y => \dhzrwbnrbpw_7[7]\);
    
    \gjkdbmpgrtc_12[7]\ : MX2
      port map(A => N_503, B => N_571, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc[7]\);
    
    G_100 : MAJ3
      port map(A => \ftbkrvpxdbp[0]_net_1\, B => 
        \ftbkrvpxdbp[1]_net_1\, C => \gjkdbmpgrtc[1]\, Y => N_242);
    
    un3_ckghxwbsztp_I_13 : XOR2
      port map(A => N_9, B => \rqrqhcwpqdn[4]_net_1\, Y => I_13_6);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_77 : AO1
      port map(A => \DWACT_COMP0_E[1]\, B => \DWACT_COMP0_E[2]\, 
        C => \DWACT_COMP0_E[0]\, Y => un2_pbzxzjhqhwz);
    
    \tkcbxxqbtfx[14]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[3]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[14]_net_1\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_56 : OR2A
      port map(A => \rqrqhcwpqdn[1]_net_1\, B => 
        \bnvtvdkcnpp[1]_net_1\, Y => \ACT_LT4_E[1]\);
    
    \bnwcvzjbgtc_3[5]\ : MX2
      port map(A => N_718, B => N_735, S => 
        \rdgwgtpkbgc[1]_net_1\, Y => N_637);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I14_S : XOR3
      port map(A => N_302, B => N_306, C => N255, Y => 
        \fqkdsnjfpdb[14]\);
    
    \bnvtvdkcnpp_9[5]\ : MX2C
      port map(A => \bnvtvdkcnpp_5[5]_net_1\, B => 
        \fqkdsnjfpdb_i[11]\, S => \tkcbxxqbtfx[4]_net_1\, Y => 
        \bnvtvdkcnpp_9[5]_net_1\);
    
    \ftbkrvpxdbp[2]\ : DFN1E0C0
      port map(D => \ftbkrvpxdbp_5[2]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ftbkrvpxdbp_1_sqmuxa\, Q => 
        \ftbkrvpxdbp[2]_net_1\);
    
    \rqrqhcwpqdn_9_i[11]\ : NOR3
      port map(A => N_1007, B => N_1009, C => N_1008, Y => 
        \rqrqhcwpqdn_9_i[11]_net_1\);
    
    \mkmkbnghgkd[6]\ : MX2
      port map(A => INIT_ADDR_int(6), B => TR_EV_ADDR_net_net_1, 
        S => INIT_DONE_c, Y => EV_RAM_ADDR_A_net_net_1);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0[4]\ : OR3C
      port map(A => N_1065, B => \dhzrwbnrbpw_7_iv_0_0[4]\, C => 
        N_1068, Y => \dhzrwbnrbpw_7[4]\);
    
    \rqrqhcwpqdn_5_i_a2_1[5]\ : NOR2
      port map(A => EV_RAM_DO_B_net_net_2, B => N_1127, Y => 
        N_991);
    
    \bnwcvzjbgtc_3[11]\ : MX2
      port map(A => N_739, B => N_741, S => 
        \rdgwgtpkbgc[1]_net_1\, Y => N_643);
    
    \bnwcvzjbgtc_12[10]\ : MX2
      port map(A => N_710, B => N_778, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => \bnwcvzjbgtc[10]\);
    
    kkjrqmngvjc_0_0 : OR3B
      port map(A => N_1161, B => \kkjrqmngvjc_0_0_1\, C => 
        bskrjhjvqpf_6_sqmuxa, Y => EV_RAM_CSN_B_net);
    
    G_120 : AO13
      port map(A => \ftbkrvpxdbp[5]_net_1\, B => \gjkdbmpgrtc[5]\, 
        C => \bnwcvzjbgtc[5]\, Y => N_262);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I0_un1_CO1_i_o3 : MAJ3
      port map(A => \ftbkrvpxdbp[0]_net_1\, B => \bnwcvzjbgtc[0]\, 
        C => \gjkdbmpgrtc_i[0]\, Y => N_42);
    
    \rqrqhcwpqdn_9_i_a2_0[10]\ : NOR3A
      port map(A => N_937, B => \tkcbxxqbtfx[6]_net_1\, C => 
        \rqrqhcwpqdn[10]_net_1\, Y => N_1005);
    
    \gjkdbmpgrtc_12[12]\ : MX2
      port map(A => N_508, B => N_576, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc[12]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I4_S_0 : XOR2
      port map(A => N_256_i, B => N_252_i, Y => 
        ADD_18x18_slow_I4_S_0);
    
    \qrbxknbdvjj[1]\ : DFN1E1C0
      port map(D => \EV_RAM_ADDR_B_net_net_6\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \tkcbxxqbtfx[11]_net_1\, Q => 
        \qrbxknbdvjj[1]_net_1\);
    
    kkjrqmngvjc_0_0_a2_0_a2 : OR2
      port map(A => \EV_RAM_DO_B_net[8]\, B => N_1159, Y => 
        N_1161);
    
    \gjkdbmpgrtc_12[13]\ : MX2
      port map(A => N_509, B => N_577, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc[13]\);
    
    \bnwcvzjbgtc_11[14]\ : NOR2B
      port map(A => rdgwgtpkbgc_3, B => \bnvtvdkcnpp[8]_net_1\, Y
         => N_782);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I7_CO1_i_o2 : MIN3
      port map(A => N_267, B => N_271, C => N_49, Y => N_36);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I3_CO1_i_o2 : MIN3
      port map(A => N_247, B => N_251, C => N_50, Y => N_35);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_34 : XNOR2
      port map(A => \bnvtvdkcnpp[4]_net_1\, B => 
        \rqrqhcwpqdn[4]_net_1\, Y => \DWACT_BL_EQUAL_0_E[0]\);
    
    \kkkmkxwwscv[3]\ : DFN1E0C0
      port map(D => ASSC_SEQOUT_net_net_1, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => qmmwhjxxxmv, Q => 
        EV_ASSCADDR_net_net_1);
    
    \gjkdbmpgrtc_9[3]\ : MX2
      port map(A => \rqrqhcwpqdn[3]_net_1\, B => 
        \rqrqhcwpqdn[4]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_533);
    
    \bnwcvzjbgtc_3[0]\ : MX2
      port map(A => N_598, B => N_615, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_632);
    
    \tkcbxxqbtfx_ns_a3_0_a2[16]\ : NOR3A
      port map(A => INIT_DONE_c, B => N_1159, C => 
        bskrjhjvqpf_6_sqmuxa_1, Y => \tkcbxxqbtfx_ns[16]\);
    
    \rqrqhcwpqdn_5_i_a2[2]\ : NOR2
      port map(A => N_1011_i, B => ASSC_RAM_DO_A_net_net_5, Y => 
        N_980);
    
    \rqrqhcwpqdn[11]\ : DFN1C0
      port map(D => \rqrqhcwpqdn_9_i[11]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \rqrqhcwpqdn[11]_net_1\);
    
    \gjkdbmpgrtc_9[10]\ : MX2
      port map(A => \rqrqhcwpqdn[10]_net_1\, B => 
        \rqrqhcwpqdn[11]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_540);
    
    \bnvtvdkcnpp[6]\ : DFN1C0
      port map(D => \bnvtvdkcnpp_9[6]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \bnvtvdkcnpp[6]_net_1\);
    
    \rqrqhcwpqdn_9_i_a2_1[9]\ : NOR2
      port map(A => EV_RAM_DO_B_net_net_7, B => N_937, Y => 
        N_1003);
    
    un1_dhzrwbnrbpw_I_32 : XOR2
      port map(A => \EV_RAM_ADDR_B_net_net_6\, B => 
        \DWACT_ADD_CI_0_TMP[0]\, Y => I_32_1);
    
    \rqrqhcwpqdn_9_i_a2[10]\ : NOR3B
      port map(A => N_937, B => \tkcbxxqbtfx[6]_net_1\, C => 
        ASSC_RAM_DO_A_net_net_6, Y => N_1004);
    
    \ckghxwbsztp[4]\ : DFN1E1C0
      port map(D => \ckghxwbsztp_3[4]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \tkcbxxqbtfx[1]_net_1\, Q => 
        \ckghxwbsztp[4]_net_1\);
    
    gpjgkmhxbkn_1_0_a2 : XNOR2
      port map(A => EV_RAM_DO_B_net_net_5, B => 
        \rqrqhcwpqdn[2]_net_1\, Y => gpjgkmhxbkn_1_i);
    
    \rqrqhcwpqdn_5_i_a2_1[4]\ : NOR2
      port map(A => EV_RAM_DO_B_net_net_3, B => N_1127, Y => 
        N_988);
    
    \gjkdbmpgrtc_9[6]\ : MX2
      port map(A => \rqrqhcwpqdn[6]_net_1\, B => 
        \rqrqhcwpqdn[7]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_536);
    
    \bnvtvdkcnpp[4]\ : DFN1C0
      port map(D => \bnvtvdkcnpp_9[4]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \bnvtvdkcnpp[4]_net_1\);
    
    ftbkrvpxdbp_1_sqmuxa : NOR2A
      port map(A => \un1_kkgccghmrzf\, B => 
        \tkcbxxqbtfx_0[4]_net_1\, Y => \ftbkrvpxdbp_1_sqmuxa\);
    
    \tkcbxxqbtfx[17]\ : DFN1P0
      port map(D => \tkcbxxqbtfx_ns[0]\, CLK => ACM_CLK_c, PRE
         => sys_reset_gl, Q => \tkcbxxqbtfx[17]_net_1\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_42 : NOR2A
      port map(A => \rqrqhcwpqdn[4]_net_1\, B => 
        \bnvtvdkcnpp[4]_net_1\, Y => \ACT_LT3_E[0]\);
    
    \bnwcvzjbgtc_12[6]\ : MX2
      port map(A => N_706, B => N_774, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => \bnwcvzjbgtc[6]\);
    
    \rdgwgtpkbgc_0[0]\ : DFN1E1C0
      port map(D => EV_RAM_DO_B_net_net_7, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \bskrjhjvqpf_1_sqmuxa\, Q => 
        \rdgwgtpkbgc_0[0]_net_1\);
    
    \bnwcvzjbgtc_11[6]\ : MX2
      port map(A => N_642, B => \bnvtvdkcnpp[0]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_774);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_63 : OR2A
      port map(A => \bnvtvdkcnpp[3]_net_1\, B => 
        \rqrqhcwpqdn[3]_net_1\, Y => \ACT_LT4_E[8]\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2[3]\ : OR2B
      port map(A => EV_RAM_DO_B_net_net_4, B => N_1088, Y => 
        N_1061);
    
    G_146 : XNOR3
      port map(A => \gjkdbmpgrtc[11]\, B => 
        \bnvtvdkcnpp[5]_net_1\, C => \bnwcvzjbgtc[11]\, Y => 
        N_291);
    
    njhxtffzjds : DFN1E1C0
      port map(D => N_1048, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        E => N_872, Q => EV_EVFLAG_net);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_23 : AO1C
      port map(A => \bnvtvdkcnpp[9]_net_1\, B => 
        \rqrqhcwpqdn[9]_net_1\, C => N_28, Y => N_34);
    
    \gjkdbmpgrtc_9[5]\ : MX2
      port map(A => \rqrqhcwpqdn[5]_net_1\, B => 
        \rqrqhcwpqdn[6]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_535);
    
    \mkmkbnghgkd[7]\ : MX2
      port map(A => INIT_ADDR_int(7), B => \Z\\TR_EV_ADDR_net\\\, 
        S => INIT_DONE_c, Y => \Z\\EV_RAM_ADDR_A_net\\\);
    
    un1_dhzrwbnrbpw_I_52 : AND2
      port map(A => \EV_RAM_ADDR_B_net_net_5\, B => 
        \EV_RAM_ADDR_B_net_net_4\, Y => 
        \DWACT_ADD_CI_0_pog_array_1[0]\);
    
    \kkkmkxwwscv[4]\ : DFN1E0C0
      port map(D => \Z\\ASSC_SEQOUT_net\\\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => qmmwhjxxxmv, Q => 
        \Z\\EV_ASSCADDR_net\\\);
    
    kkjrqmngvjc_0_0_a2_0_0_a2_2 : OR2B
      port map(A => EV_RAM_DO_B_net_net_1, B => 
        \tkcbxxqbtfx[12]_net_1\, Y => bskrjhjvqpf_6_sqmuxa_2);
    
    \bnvtvdkcnpp[11]\ : DFN1C0
      port map(D => \bnvtvdkcnpp_9[11]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \bnvtvdkcnpp[11]_net_1\);
    
    \rqrqhcwpqdn_9_i_a2_1[10]\ : NOR2
      port map(A => EV_RAM_DO_B_net_net_6, B => N_937, Y => 
        N_1006);
    
    \zbqggxmnpbs_0_iv[8]\ : AO1B
      port map(A => \fqkdsnjfpdb[5]\, B => \tkcbxxqbtfx[4]_net_1\, 
        C => \fqkdsnjfpdb_m[14]_net_1\, Y => \EV_RAM_DI_B_net[8]\);
    
    un3_ckghxwbsztp_I_8 : NOR2B
      port map(A => \rqrqhcwpqdn[2]_net_1\, B => 
        \rqrqhcwpqdn[1]_net_1\, Y => N_12);
    
    \rqrqhcwpqdn[1]\ : DFN1C0
      port map(D => \rqrqhcwpqdn_5_i[1]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \rqrqhcwpqdn[1]_net_1\);
    
    \ftbkrvpxdbp[3]\ : DFN1E0C0
      port map(D => \ftbkrvpxdbp_5[3]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ftbkrvpxdbp_1_sqmuxa\, Q => 
        \ftbkrvpxdbp[3]_net_1\);
    
    \bnwcvzjbgtc_7[13]\ : NOR2A
      port map(A => N_645, B => rdgwgtpkbgc_3, Y => N_713);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I0_S : XNOR3
      port map(A => \gjkdbmpgrtc_i[0]\, B => 
        \ftbkrvpxdbp[0]_net_1\, C => \bnwcvzjbgtc[0]\, Y => 
        \fqkdsnjfpdb[0]\);
    
    \bnvtvdkcnpp_5[6]\ : MX2
      port map(A => \bnvtvdkcnpp[6]_net_1\, B => 
        EV_RAM_DO_B_net_net_1, S => un3_qvqqfftqsfc, Y => 
        \bnvtvdkcnpp_5[6]_net_1\);
    
    un1_dhzrwbnrbpw_I_35 : XOR2
      port map(A => \EV_RAM_ADDR_B_net_net_5\, B => 
        \DWACT_ADD_CI_0_g_array_1[0]\, Y => I_35_1);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_0[7]\ : AOI1B
      port map(A => \Z\\EV_RAM_DO_B_net\\\, B => N_1088, C => 
        N_1077, Y => \dhzrwbnrbpw_7_iv_0_0[7]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I6_un1_CO1_i_o3 : MAJ3
      port map(A => N_266, B => N_262, C => N239, Y => N_49);
    
    \bnwcvzjbgtc_7[1]\ : MX2
      port map(A => N_633, B => N_641, S => rdgwgtpkbgc_3, Y => 
        N_701);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_6 : AND3
      port map(A => \DWACT_BL_EQUAL_0_E_0[0]\, B => 
        \DWACT_BL_EQUAL_0_E_0[1]\, C => \DWACT_BL_EQUAL_0_E_0[2]\, 
        Y => \DWACT_BL_EQUAL_0_DWACT_ANDTREE_E[0]\);
    
    \tkcbxxqbtfx_ns_0_x3[4]\ : XOR2
      port map(A => EV_RAM_DO_B_net_net_1, B => 
        \EV_RAM_DO_B_net[8]\, Y => N_1129_i);
    
    \qrbxknbdvjj[3]\ : DFN1E1C0
      port map(D => \EV_RAM_ADDR_B_net_net_4\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \tkcbxxqbtfx[11]_net_1\, Q => 
        \qrbxknbdvjj[3]_net_1\);
    
    gpjgkmhxbkn_3_0_a2 : XNOR2
      port map(A => EV_RAM_DO_B_net_net_3, B => 
        \rqrqhcwpqdn[4]_net_1\, Y => gpjgkmhxbkn_3_i);
    
    \bnvtvdkcnpp_5[5]\ : MX2C
      port map(A => \bnvtvdkcnpp[5]_net_1\, B => 
        EV_RAM_DO_B_net_net_2, S => un3_qvqqfftqsfc, Y => 
        \bnvtvdkcnpp_5[5]_net_1\);
    
    \bnwcvzjbgtc_9[4]\ : MX2
      port map(A => \bnvtvdkcnpp[4]_net_1\, B => 
        \bnvtvdkcnpp[5]_net_1\, S => \rdgwgtpkbgc[0]_net_1\, Y
         => N_738);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_2 : XNOR2
      port map(A => \bnvtvdkcnpp[11]_net_1\, B => 
        \rqrqhcwpqdn[11]_net_1\, Y => \DWACT_BL_EQUAL_0_E[4]\);
    
    \mkmkbnghgkd[1]\ : MX2
      port map(A => INIT_ADDR(1), B => TR_EV_ADDR_net_net_6, S
         => INIT_DONE_c, Y => EV_RAM_ADDR_A_net_net_6);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a3[0]\ : NOR2B
      port map(A => \dhzrwbnrbpw_7_iv_0_a3_0[0]\, B => N_1044, Y
         => N_1086);
    
    \ckghxwbsztp_m[0]\ : OR2B
      port map(A => \tkcbxxqbtfx[0]_net_1\, B => 
        \ckghxwbsztp[0]_net_1\, Y => \ckghxwbsztp_m[0]_net_1\);
    
    \rqrqhcwpqdn[6]\ : DFN1C0
      port map(D => \rqrqhcwpqdn_5_i[6]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \rqrqhcwpqdn[6]_net_1\);
    
    G_151 : XOR3
      port map(A => \gjkdbmpgrtc[12]\, B => 
        \bnvtvdkcnpp[6]_net_1\, C => \bnwcvzjbgtc[12]\, Y => 
        N_296_i);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_1[2]\ : OR3B
      port map(A => \tkcbxxqbtfx[8]_net_1\, B => 
        \qrbxknbdvjj[2]_net_1\, C => \rgmpzqrbghh\, Y => N_1059);
    
    \gjkdbmpgrtc_11_0_a2[15]\ : NOR2B
      port map(A => \rqrqhcwpqdn[9]_net_1\, B => rdgwgtpkbgc_3, Y
         => N_968);
    
    \bnvtvdkcnpp[10]\ : DFN1C0
      port map(D => \bnvtvdkcnpp_9[10]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \bnvtvdkcnpp[10]_net_1\);
    
    un1_dhzrwbnrbpw_I_46 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_2[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_2[0]\, Y => 
        \DWACT_ADD_CI_0_g_array_3[0]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I9_CO1_i_o3 : MAJ3
      port map(A => N_281, B => N_277, C => N_40, Y => N247);
    
    \rqrqhcwpqdn_5_i_a2[7]\ : NOR2
      port map(A => N_1011_i, B => \Z\\ASSC_RAM_DO_A_net\\\, Y
         => N_995);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I4_S : XOR2
      port map(A => ADD_18x18_slow_I4_S_0, B => N_35, Y => 
        \fqkdsnjfpdb_i[4]\);
    
    \ftbkrvpxdbp[1]\ : DFN1E0C0
      port map(D => \ftbkrvpxdbp_5[1]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ftbkrvpxdbp_1_sqmuxa\, Q => 
        \ftbkrvpxdbp[1]_net_1\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_0[0]\ : AOI1B
      port map(A => N_1088, B => EV_RAM_DO_B_net_net_7, C => 
        N_1051, Y => \dhzrwbnrbpw_7_iv_0_0[0]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I13_CO1_i_o3 : MAJ3
      port map(A => N_301, B => N_297, C => N_39, Y => N255);
    
    cmkddgvkrrs_0_iv_i_a2_0 : AOI1
      port map(A => \wqpbnjzjqsx_NE_1\, B => \wqpbnjzjqsx_NE_2\, 
        C => \rqrqhcwpqdn[0]_net_1\, Y => N_1152);
    
    \mtbjcfqsbqf[0]\ : DFN1E0C0
      port map(D => ADC_CHNR_net_net_3, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => qmmwhjxxxmv, Q => 
        \EV_CHHOLD_net_net_3\);
    
    \bnwcvzjbgtc_9[2]\ : MX2
      port map(A => \bnvtvdkcnpp[2]_net_1\, B => 
        \bnvtvdkcnpp[3]_net_1\, S => \rdgwgtpkbgc[0]_net_1\, Y
         => N_736);
    
    G_110 : AO18
      port map(A => \ftbkrvpxdbp[3]_net_1\, B => \bnwcvzjbgtc[3]\, 
        C => \gjkdbmpgrtc[3]\, Y => N_252_i);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I2_un1_CO1_i_o3 : MAJ3
      port map(A => N_246, B => N_242, C => N231, Y => N_50);
    
    \gjkdbmpgrtc_7[3]\ : MX2
      port map(A => N_954, B => N_439, S => rdgwgtpkbgc_3, Y => 
        N_499);
    
    \bnvtvdkcnpp_9[9]\ : MX2
      port map(A => \bnvtvdkcnpp_8[9]_net_1\, B => 
        \fqkdsnjfpdb[15]\, S => \tkcbxxqbtfx[4]_net_1\, Y => 
        \bnvtvdkcnpp_9[9]_net_1\);
    
    \fqkdsnjfpdb_m[17]\ : OR2B
      port map(A => \tkcbxxqbtfx[4]_net_1\, B => 
        \fqkdsnjfpdb[17]\, Y => \fqkdsnjfpdb_m[17]_net_1\);
    
    \rqrqhcwpqdn_5_i_a2[5]\ : NOR2
      port map(A => N_1011_i, B => ASSC_RAM_DO_A_net_net_2, Y => 
        N_989);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0[5]\ : OR3C
      port map(A => N_1069, B => \dhzrwbnrbpw_7_iv_0_0[5]\, C => 
        N_1072, Y => \dhzrwbnrbpw_7[5]\);
    
    \fqkdsnjfpdb_m[16]\ : OR2A
      port map(A => \tkcbxxqbtfx[4]_net_1\, B => 
        \fqkdsnjfpdb_i[16]\, Y => \fqkdsnjfpdb_m[16]_net_1\);
    
    un3_ckghxwbsztp_I_12 : AND3
      port map(A => \rqrqhcwpqdn[1]_net_1\, B => 
        \rqrqhcwpqdn[2]_net_1\, C => \rqrqhcwpqdn[3]_net_1\, Y
         => N_9);
    
    un1_dhzrwbnrbpw_I_51 : AND2
      port map(A => \DWACT_ADD_CI_0_pog_array_1_1[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_1_2[0]\, Y => 
        \DWACT_ADD_CI_0_pog_array_2[0]\);
    
    \tkcbxxqbtfx[5]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[12]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[5]_net_1\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_0[4]\ : AOI1B
      port map(A => \Z\\EV_CHHOLD_net\\_net_1\, B => 
        \rgmpzqrbghh_0\, C => N_1067, Y => 
        \dhzrwbnrbpw_7_iv_0_0[4]\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_27 : OA1
      port map(A => N_37_0, B => N_36_0, C => N_35_0, Y => 
        \DWACT_COMP0_E[0]\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_48 : AOI1A
      port map(A => \ACT_LT3_E[3]\, B => \ACT_LT3_E[4]\, C => 
        \ACT_LT3_E[5]\, Y => \DWACT_CMPLE_PO2_DWACT_COMP0_E[0]\);
    
    \gjkdbmpgrtc_7[4]\ : MX2
      port map(A => N_955, B => N_440, S => rdgwgtpkbgc_3, Y => 
        N_500);
    
    \gjkdbmpgrtc_3_0_a2[3]\ : NOR2B
      port map(A => \rdgwgtpkbgc[1]_net_1\, B => N_960, Y => 
        N_954);
    
    \bnvtvdkcnpp_9[0]\ : MX2
      port map(A => \bnvtvdkcnpp_5[0]_net_1\, B => 
        \fqkdsnjfpdb[6]\, S => \tkcbxxqbtfx_0[4]_net_1\, Y => 
        \bnvtvdkcnpp_9[0]_net_1\);
    
    \rqrqhcwpqdn_5_i_a2_0[8]\ : NOR2
      port map(A => \rqrqhcwpqdn[8]_net_1\, B => N_1010_i, Y => 
        N_999);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_43 : OR2A
      port map(A => \rqrqhcwpqdn[5]_net_1\, B => 
        \bnvtvdkcnpp[5]_net_1\, Y => \ACT_LT3_E[1]\);
    
    cmkddgvkrrs_r : NOR3
      port map(A => N_1151, B => N_1152, C => 
        \tkcbxxqbtfx[17]_net_1\, Y => cmkddgvkrrs);
    
    \rqrqhcwpqdn_5_i[4]\ : NOR3
      port map(A => N_987, B => N_988, C => N_986, Y => 
        \rqrqhcwpqdn_5_i[4]_net_1\);
    
    \bnvtvdkcnpp_9[1]\ : MX2C
      port map(A => \bnvtvdkcnpp_5[1]_net_1\, B => 
        \fqkdsnjfpdb_i[7]\, S => \tkcbxxqbtfx_0[4]_net_1\, Y => 
        \bnvtvdkcnpp_9[1]_net_1\);
    
    G_160 : AO13
      port map(A => \bnvtvdkcnpp[7]_net_1\, B => 
        \gjkdbmpgrtc[13]\, C => \bnwcvzjbgtc[13]\, Y => N_302);
    
    \bnvtvdkcnpp[1]\ : DFN1C0
      port map(D => \bnvtvdkcnpp_9[1]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \bnvtvdkcnpp[1]_net_1\);
    
    \rqrqhcwpqdn_5_i_a2_0[3]\ : NOR2
      port map(A => \rqrqhcwpqdn[3]_net_1\, B => N_1010_i, Y => 
        N_984);
    
    \bnvtvdkcnpp_5[0]\ : MX2
      port map(A => \bnvtvdkcnpp[0]_net_1\, B => 
        EV_RAM_DO_B_net_net_7, S => un3_qvqqfftqsfc, Y => 
        \bnvtvdkcnpp_5[0]_net_1\);
    
    frhjqwkkhgk_0 : NOR3
      port map(A => N_199, B => \tkcbxxqbtfx[2]_net_1\, C => 
        N_195, Y => EV_RAM_WR_BUSY_B_net);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_2[5]\ : OR2B
      port map(A => I_33_1, B => N_1086, Y => N_1072);
    
    un1_dhzrwbnrbpw_I_1 : AND2
      port map(A => \EV_RAM_ADDR_B_net_net_7\, B => 
        \bskrjhjvqpf_i_0\, Y => \DWACT_ADD_CI_0_TMP[0]\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0[8]\ : AO1B
      port map(A => N_1086, B => I_36_1, C => 
        \dhzrwbnrbpw_7_iv_0_0[8]\, Y => \dhzrwbnrbpw_7[8]\);
    
    un1_fskhwddcbbg_i_o3 : OR2
      port map(A => \un1_fskhwddcbbg_i_o3_0\, B => 
        \tkcbxxqbtfx_0[4]_net_1\, Y => N_199);
    
    \gjkdbmpgrtc_9[7]\ : MX2
      port map(A => \rqrqhcwpqdn[7]_net_1\, B => 
        \rqrqhcwpqdn[8]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_537);
    
    \bnvtvdkcnpp[0]\ : DFN1C0
      port map(D => \bnvtvdkcnpp_9[0]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \bnvtvdkcnpp[0]_net_1\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_1[5]\ : OR3B
      port map(A => \tkcbxxqbtfx[8]_net_1\, B => 
        \qrbxknbdvjj[5]_net_1\, C => \rgmpzqrbghh_0\, Y => N_1071);
    
    \dhzrwbnrbpw[3]\ : DFN1C0
      port map(D => \dhzrwbnrbpw_7[3]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \EV_RAM_ADDR_B_net_net_4\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I8_un1_CO1_i_o3 : MIN3
      port map(A => N_276_i, B => N_272_i, C => N_36, Y => N_40);
    
    un1_dhzrwbnrbpw_I_38 : XOR2
      port map(A => \EV_RAM_ADDR_B_net_net_1\, B => 
        \DWACT_ADD_CI_0_g_array_11[0]\, Y => I_38_1);
    
    \bnwcvzjbgtc_11_0_a2[16]\ : NOR2B
      port map(A => rdgwgtpkbgc_3, B => \bnvtvdkcnpp[10]_net_1\, 
        Y => N_976);
    
    \gjkdbmpgrtc_7[13]\ : NOR2A
      port map(A => N_441, B => rdgwgtpkbgc_3, Y => N_509);
    
    un3_ckghxwbsztp_I_9 : XOR2
      port map(A => N_12, B => \rqrqhcwpqdn[3]_net_1\, Y => I_9_5);
    
    G_130 : AO18
      port map(A => \bnvtvdkcnpp[1]_net_1\, B => \bnwcvzjbgtc[7]\, 
        C => \gjkdbmpgrtc[7]\, Y => N_272_i);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_4 : XNOR2
      port map(A => \bnvtvdkcnpp[10]_net_1\, B => 
        \rqrqhcwpqdn[10]_net_1\, Y => \DWACT_BL_EQUAL_0_E[3]\);
    
    G_101 : XNOR2
      port map(A => \G_101_0\, B => \bnwcvzjbgtc[2]\, Y => N_246);
    
    \fqkdsnjfpdb_m[15]\ : OR2B
      port map(A => \tkcbxxqbtfx[4]_net_1\, B => 
        \fqkdsnjfpdb[15]\, Y => \fqkdsnjfpdb_m[15]_net_1\);
    
    \bnwcvzjbgtc_7[11]\ : NOR2A
      port map(A => N_643, B => rdgwgtpkbgc_3, Y => N_711);
    
    \rqrqhcwpqdn_9_i_a2_0[11]\ : NOR3A
      port map(A => N_937, B => \tkcbxxqbtfx[6]_net_1\, C => 
        \rqrqhcwpqdn[11]_net_1\, Y => N_1008);
    
    \bnwcvzjbgtc_7[7]\ : MX2
      port map(A => N_639, B => N_647, S => rdgwgtpkbgc_3, Y => 
        N_707);
    
    \mkmkbnghgkd[4]\ : MX2
      port map(A => INIT_ADDR(4), B => TR_EV_ADDR_net_net_3, S
         => INIT_DONE_c, Y => EV_RAM_ADDR_A_net_net_3);
    
    \gjkdbmpgrtc_7[15]\ : NOR2A
      port map(A => N_443, B => rdgwgtpkbgc_3, Y => N_511);
    
    sphhbprckfk_1_iv_i : MX2B
      port map(A => N_1128, B => N_1152, S => \spbznmtdmpd\, Y
         => N_1107);
    
    un1_dhzrwbnrbpw_I_36 : XOR2
      port map(A => \EV_RAM_ADDR_B_net[8]_net_1\, B => 
        \DWACT_ADD_CI_0_g_array_3[0]\, Y => I_36_1);
    
    \gjkdbmpgrtc_7[9]\ : MX2
      port map(A => N_437, B => N_956, S => rdgwgtpkbgc_3, Y => 
        N_505);
    
    \rdgwgtpkbgc[0]\ : DFN1E1C0
      port map(D => EV_RAM_DO_B_net_net_7, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \bskrjhjvqpf_1_sqmuxa\, Q => 
        \rdgwgtpkbgc[0]_net_1\);
    
    \rqrqhcwpqdn_9_i_o3[9]\ : OR3A
      port map(A => \tkcbxxqbtfx[8]_net_1\, B => 
        \rfrdbqfdrhx[0]_net_1\, C => \rfrdbqfdrhx[1]_net_1\, Y
         => N_937);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I4_un1_CO1_i_a3 : OR2
      port map(A => N_35, B => N_252_i, Y => N_72);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_1[3]\ : OR3B
      port map(A => \tkcbxxqbtfx[8]_net_1\, B => 
        \qrbxknbdvjj[3]_net_1\, C => \rgmpzqrbghh_0\, Y => N_1063);
    
    \bnwcvzjbgtc_7[8]\ : MX2
      port map(A => N_640, B => N_970, S => rdgwgtpkbgc_3, Y => 
        N_708);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_2[2]\ : OR2B
      port map(A => I_35_1, B => N_1086, Y => N_1060);
    
    \bnwcvzjbgtc_3[12]\ : MX2
      port map(A => N_740, B => N_742, S => 
        \rdgwgtpkbgc[1]_net_1\, Y => N_644);
    
    \rqrqhcwpqdn_5_i_a2_1[2]\ : NOR2
      port map(A => EV_RAM_DO_B_net_net_5, B => N_1127, Y => 
        N_982);
    
    \bnwcvzjbgtc_9[7]\ : MX2
      port map(A => \bnvtvdkcnpp[7]_net_1\, B => 
        \bnvtvdkcnpp[8]_net_1\, S => \rdgwgtpkbgc[0]_net_1\, Y
         => N_741);
    
    \bnwcvzjbgtc_3[6]\ : MX2
      port map(A => N_734, B => N_736, S => 
        \rdgwgtpkbgc[1]_net_1\, Y => N_638);
    
    pfbntnkqtmt : OR2A
      port map(A => INIT_EV_WEN, B => INIT_DONE_c, Y => 
        EV_RAM_RW_A_net);
    
    G_121 : XNOR3
      port map(A => \gjkdbmpgrtc[6]\, B => \bnvtvdkcnpp[0]_net_1\, 
        C => \bnwcvzjbgtc[6]\, Y => N_266);
    
    G_156 : XNOR3
      port map(A => \gjkdbmpgrtc[13]\, B => 
        \bnvtvdkcnpp[7]_net_1\, C => \bnwcvzjbgtc[13]\, Y => 
        N_301);
    
    \gjkdbmpgrtc_12[6]\ : MX2
      port map(A => N_502, B => N_570, S => 
        \rdgwgtpkbgc[2]_net_1\, Y => \gjkdbmpgrtc[6]\);
    
    \bnwcvzjbgtc_1[0]\ : MX2
      port map(A => \ftbkrvpxdbp[0]_net_1\, B => 
        \ftbkrvpxdbp[1]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_598);
    
    un1_dhzrwbnrbpw_I_30 : XOR2
      port map(A => \EV_RAM_ADDR_B_net_net_3\, B => 
        \DWACT_ADD_CI_0_g_array_2[0]\, Y => I_30_1);
    
    \bnvtvdkcnpp_9[11]\ : MX2
      port map(A => \bnvtvdkcnpp_8[11]_net_1\, B => 
        \fqkdsnjfpdb[17]\, S => \tkcbxxqbtfx[4]_net_1\, Y => 
        \bnvtvdkcnpp_9[11]_net_1\);
    
    \gjkdbmpgrtc_9[1]\ : MX2
      port map(A => \rqrqhcwpqdn[1]_net_1\, B => 
        \rqrqhcwpqdn[2]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_531);
    
    \bnvtvdkcnpp_9[3]\ : MX2
      port map(A => \bnvtvdkcnpp_5[3]_net_1\, B => 
        \fqkdsnjfpdb[9]\, S => \tkcbxxqbtfx_0[4]_net_1\, Y => 
        \bnvtvdkcnpp_9[3]_net_1\);
    
    \rqrqhcwpqdn_5_i_a2_1[7]\ : NOR2
      port map(A => N_1127, B => \Z\\EV_RAM_DO_B_net\\\, Y => 
        N_997);
    
    \dhzrwbnrbpw[8]\ : DFN1C0
      port map(D => \dhzrwbnrbpw_7[8]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \EV_RAM_ADDR_B_net[8]_net_1\);
    
    \gjkdbmpgrtc_7[11]\ : NOR2A
      port map(A => N_439, B => rdgwgtpkbgc_3, Y => N_507);
    
    \tkcbxxqbtfx_ns_a3_0_a2[14]\ : NOR3A
      port map(A => \Z\\EV_RAM_DO_B_net\\\, B => N_1160, C => 
        \tkcbxxqbtfx_ns_1[14]\, Y => \tkcbxxqbtfx_ns[14]\);
    
    \bnwcvzjbgtc_12[14]\ : MX2
      port map(A => N_714, B => N_782, S => 
        \rdgwgtpkbgc[2]_net_1\, Y => \bnwcvzjbgtc[14]\);
    
    G_170 : AO18
      port map(A => \bnvtvdkcnpp[9]_net_1\, B => 
        \bnwcvzjbgtc[15]\, C => \gjkdbmpgrtc[15]\, Y => N_312_i_0);
    
    \bnwcvzjbgtc_9[6]\ : MX2
      port map(A => \bnvtvdkcnpp[6]_net_1\, B => 
        \bnvtvdkcnpp[7]_net_1\, S => \rdgwgtpkbgc[0]_net_1\, Y
         => N_740);
    
    \rqrqhcwpqdn_5_i[5]\ : NOR3
      port map(A => N_990, B => N_991, C => N_989, Y => 
        \rqrqhcwpqdn_5_i[5]_net_1\);
    
    \mtbjcfqsbqf[4]\ : DFN1E0C0
      port map(D => \ADC_CHNR_net[4]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => qmmwhjxxxmv, Q => 
        \EV_CHHOLD_net[4]_net_1\);
    
    \zbqggxmnpbs_0_iv[2]\ : OR3C
      port map(A => \fqkdsnjfpdb_m[8]_net_1\, B => 
        \ckghxwbsztp_m[1]_net_1\, C => \fqkdsnjfpdb_m[17]_net_1\, 
        Y => EV_RAM_DI_B_net_net_5);
    
    \gjkdbmpgrtc_11[4]\ : NOR2A
      port map(A => N_436, B => rdgwgtpkbgc_3, Y => N_568);
    
    \fqkdsnjfpdb_m[4]\ : OR2A
      port map(A => \tkcbxxqbtfx[4]_net_1\, B => 
        \fqkdsnjfpdb_i[4]\, Y => \fqkdsnjfpdb_m[4]_net_1\);
    
    \bnwcvzjbgtc_9[9]\ : MX2
      port map(A => \bnvtvdkcnpp[9]_net_1\, B => 
        \bnvtvdkcnpp[10]_net_1\, S => \rdgwgtpkbgc[0]_net_1\, Y
         => N_743);
    
    \fqkdsnjfpdb_m[14]\ : OR2B
      port map(A => \tkcbxxqbtfx[5]_net_1\, B => 
        \fqkdsnjfpdb[14]\, Y => \fqkdsnjfpdb_m[14]_net_1\);
    
    \zbqggxmnpbs_0_iv_0[0]\ : AOI1B
      port map(A => \tddhwjzbsrj\, B => \tkcbxxqbtfx[0]_net_1\, C
         => un2_pbzxzjhqhwz_m, Y => \zbqggxmnpbs_0_iv_0[0]_net_1\);
    
    \rqrqhcwpqdn_5_i_a2_1[1]\ : NOR2
      port map(A => EV_RAM_DO_B_net_net_6, B => N_1127, Y => 
        N_979);
    
    \bnvtvdkcnpp_9[8]\ : MX2
      port map(A => \bnvtvdkcnpp_5[8]_net_1\, B => 
        \fqkdsnjfpdb[14]\, S => \tkcbxxqbtfx[4]_net_1\, Y => 
        \bnvtvdkcnpp_9[8]_net_1\);
    
    \mtbjcfqsbqf[2]\ : DFN1E0C0
      port map(D => ADC_CHNR_net_net_1, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => qmmwhjxxxmv, Q => 
        \EV_CHHOLD_net_net_1\);
    
    \ckghxwbsztp[2]\ : DFN1E1C0
      port map(D => \ckghxwbsztp_3[2]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \tkcbxxqbtfx[1]_net_1\, Q => 
        \ckghxwbsztp[2]_net_1\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_3 : XNOR2
      port map(A => \bnvtvdkcnpp[9]_net_1\, B => 
        \rqrqhcwpqdn[9]_net_1\, Y => \DWACT_BL_EQUAL_0_E_0[2]\);
    
    \bnvtvdkcnpp_5[8]\ : MX2
      port map(A => \bnvtvdkcnpp[8]_net_1\, B => 
        \EV_RAM_DO_B_net[8]\, S => un3_qvqqfftqsfc, Y => 
        \bnvtvdkcnpp_5[8]_net_1\);
    
    \dhzrwbnrbpw[5]\ : DFN1C0
      port map(D => \dhzrwbnrbpw_7[5]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \EV_RAM_ADDR_B_net_net_2\);
    
    \rfrdbqfdrhx[1]\ : DFN1E1C0
      port map(D => EV_RAM_DO_B_net_net_2, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => bskrjhjvqpf_0_sqmuxa, Q => 
        \rfrdbqfdrhx[1]_net_1\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_0[8]\ : AOI1B
      port map(A => N_1088, B => \EV_RAM_DO_B_net[8]\, C => 
        N_1080, Y => \dhzrwbnrbpw_7_iv_0_0[8]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I4_un1_CO1_i_o3 : OR2B
      port map(A => ADD_18x18_slow_I4_un1_CO1_i_o3_0, B => N_72, 
        Y => N_41);
    
    un1_dhzrwbnrbpw_I_50 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_11[0]\, B => 
        \EV_RAM_ADDR_B_net_net_1\, Y => 
        \DWACT_ADD_CI_0_g_array_12_2[0]\);
    
    \tkcbxxqbtfx_ns_0[4]\ : AO1B
      port map(A => N_1130, B => INIT_DONE_c, C => N_1136, Y => 
        \tkcbxxqbtfx_ns[4]\);
    
    \rqrqhcwpqdn_5_i_o2[0]\ : OR3A
      port map(A => \tkcbxxqbtfx[9]_net_1\, B => 
        \rfrdbqfdrhx[0]_net_1\, C => \rfrdbqfdrhx[1]_net_1\, Y
         => N_1127);
    
    \bnwcvzjbgtc_1[1]\ : MX2
      port map(A => \ftbkrvpxdbp[1]_net_1\, B => 
        \ftbkrvpxdbp[2]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_599);
    
    \tkcbxxqbtfx_ns_0_a2_0[4]\ : OR3
      port map(A => N_1129_i, B => \Z\\EV_RAM_DO_B_net\\\, C => 
        \tkcbxxqbtfx_ns_1[14]\, Y => N_1136);
    
    \rqrqhcwpqdn_9_i_a2_1[11]\ : NOR2
      port map(A => EV_RAM_DO_B_net_net_5, B => N_937, Y => 
        N_1009);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_47 : AND2A
      port map(A => \rqrqhcwpqdn[6]_net_1\, B => 
        \bnvtvdkcnpp[6]_net_1\, Y => \ACT_LT3_E[5]\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_0[7]\ : OR3B
      port map(A => \tkcbxxqbtfx[8]_net_1\, B => 
        \qrbxknbdvjj[7]_net_1\, C => \rgmpzqrbghh\, Y => N_1077);
    
    rgmpzqrbghh_0 : DFN1C0
      port map(D => hmzcpnxsdwg, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \rgmpzqrbghh_0\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_0[5]\ : AOI1B
      port map(A => \rgmpzqrbghh_0\, B => 
        \EV_CHHOLD_net[4]_net_1\, C => N_1071, Y => 
        \dhzrwbnrbpw_7_iv_0_0[5]\);
    
    \bnwcvzjbgtc_7[4]\ : MX2
      port map(A => N_636, B => N_644, S => rdgwgtpkbgc_3, Y => 
        N_704);
    
    \rdgwgtpkbgc[1]\ : DFN1E1C0
      port map(D => EV_RAM_DO_B_net_net_6, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \bskrjhjvqpf_1_sqmuxa\, Q => 
        \rdgwgtpkbgc[1]_net_1\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2[4]\ : OR2B
      port map(A => EV_RAM_DO_B_net_net_3, B => N_1088, Y => 
        N_1065);
    
    \bnwcvzjbgtc_9[8]\ : MX2
      port map(A => \bnvtvdkcnpp[8]_net_1\, B => 
        \bnvtvdkcnpp[9]_net_1\, S => \rdgwgtpkbgc[0]_net_1\, Y
         => N_742);
    
    \ftbkrvpxdbp_5[1]\ : MX2
      port map(A => EV_RAM_DO_B_net_net_3, B => \fqkdsnjfpdb[1]\, 
        S => \tkcbxxqbtfx_0[4]_net_1\, Y => 
        \ftbkrvpxdbp_5[1]_net_1\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I5_S : XOR3
      port map(A => N_257, B => N_261, C => N_41, Y => 
        \fqkdsnjfpdb[5]\);
    
    \gjkdbmpgrtc_3[6]\ : MX2
      port map(A => N_530, B => N_532, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_434);
    
    \tkcbxxqbtfx_ns_a3[17]\ : NOR2B
      port map(A => INIT_DONE_c, B => \tkcbxxqbtfx[1]_net_1\, Y
         => \tkcbxxqbtfx_ns[17]\);
    
    \kkkmkxwwscv[5]\ : DFN1E0C0
      port map(D => \ASSC_SEQOUT_net[5]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => qmmwhjxxxmv, Q => 
        \EV_ASSCADDR_net[8]\);
    
    \gjkdbmpgrtc_12[2]\ : MX2
      port map(A => N_958, B => N_964, S => 
        \rdgwgtpkbgc[2]_net_1\, Y => \gjkdbmpgrtc[2]\);
    
    \dhzrwbnrbpw[6]\ : DFN1C0
      port map(D => \dhzrwbnrbpw_7[6]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \EV_RAM_ADDR_B_net_net_1\);
    
    \gjkdbmpgrtc_12[8]\ : MX2
      port map(A => N_504, B => N_572, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc[8]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I1_CO1_i_o3 : MIN3
      port map(A => N_241_i, B => \bnwcvzjbgtc[1]\, C => N_42, Y
         => N231);
    
    \gjkdbmpgrtc_12[0]\ : MX2C
      port map(A => N_496, B => N_962, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc_i[0]\);
    
    \zbqggxmnpbs_0_iv[6]\ : AO1B
      port map(A => \fqkdsnjfpdb[3]\, B => \tkcbxxqbtfx[4]_net_1\, 
        C => \fqkdsnjfpdb_m[12]_net_1\, Y => 
        EV_RAM_DI_B_net_net_1);
    
    G_145 : AO13
      port map(A => \bnvtvdkcnpp[4]_net_1\, B => 
        \gjkdbmpgrtc[10]\, C => \bnwcvzjbgtc[10]\, Y => N_287);
    
    \rqrqhcwpqdn_9_i[9]\ : NOR3
      port map(A => N_1001, B => N_1003, C => N_1002, Y => 
        \rqrqhcwpqdn_9_i[9]_net_1\);
    
    \mkmkbnghgkd[3]\ : MX2
      port map(A => INIT_ADDR(3), B => TR_EV_ADDR_net_net_4, S
         => INIT_DONE_c, Y => EV_RAM_ADDR_A_net_net_4);
    
    \bnwcvzjbgtc_3[10]\ : MX2
      port map(A => N_738, B => N_740, S => 
        \rdgwgtpkbgc[1]_net_1\, Y => N_642);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2[5]\ : OR2B
      port map(A => EV_RAM_DO_B_net_net_2, B => N_1088, Y => 
        N_1069);
    
    \zbqggxmnpbs_0_iv_0_0[5]\ : AOI1B
      port map(A => \ckghxwbsztp[4]_net_1\, B => 
        \tkcbxxqbtfx[0]_net_1\, C => N_1149, Y => 
        \zbqggxmnpbs_0_iv_0_0[5]_net_1\);
    
    \rqrqhcwpqdn_5_i_a2[6]\ : NOR2
      port map(A => N_1011_i, B => ASSC_RAM_DO_A_net_net_1, Y => 
        N_992);
    
    \zbqggxmnpbs_0_iv_0_a2_0[5]\ : OR2B
      port map(A => \tkcbxxqbtfx[4]_net_1\, B => \fqkdsnjfpdb[2]\, 
        Y => N_1149);
    
    \zbqggxmnpbs_r[0]\ : AOI1B
      port map(A => \fqkdsnjfpdb_m[15]_net_1\, B => 
        \zbqggxmnpbs_0_iv_1[0]_net_1\, C => N_185, Y => 
        EV_RAM_DI_B_net_net_7);
    
    wqpbnjzjqsx_0_0_a2 : XNOR2
      port map(A => \rqrqhcwpqdn[1]_net_1\, B => 
        \bnvtvdkcnpp[1]_net_1\, Y => wqpbnjzjqsx_0_i);
    
    \gjkdbmpgrtc_12[1]\ : MX2
      port map(A => N_957, B => N_963, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc[1]\);
    
    \bnvtvdkcnpp_9[7]\ : MX2C
      port map(A => \bnvtvdkcnpp_5[7]_net_1\, B => 
        \fqkdsnjfpdb_i[13]\, S => \tkcbxxqbtfx[4]_net_1\, Y => 
        \bnvtvdkcnpp_9[7]_net_1\);
    
    \bnwcvzjbgtc_2[0]\ : MX2
      port map(A => \ftbkrvpxdbp[2]_net_1\, B => 
        \ftbkrvpxdbp[3]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_615);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    \ftbkrvpxdbp_5[0]\ : MX2
      port map(A => EV_RAM_DO_B_net_net_4, B => \fqkdsnjfpdb[0]\, 
        S => \tkcbxxqbtfx_0[4]_net_1\, Y => 
        \ftbkrvpxdbp_5[0]_net_1\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I6_S_0 : XOR2
      port map(A => N_266, B => N_262, Y => ADD_18x18_slow_I6_S_0);
    
    \bnwcvzjbgtc_3[15]\ : MX2
      port map(A => N_743, B => N_974, S => 
        \rdgwgtpkbgc[1]_net_1\, Y => N_647);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_37 : AND3
      port map(A => \DWACT_BL_EQUAL_0_E[2]\, B => 
        \DWACT_BL_EQUAL_0_E[1]\, C => \DWACT_BL_EQUAL_0_E[0]\, Y
         => \DWACT_CMPLE_PO2_DWACT_COMP0_E[1]\);
    
    \ftbkrvpxdbp_5[2]\ : MX2
      port map(A => EV_RAM_DO_B_net_net_2, B => \fqkdsnjfpdb[2]\, 
        S => \tkcbxxqbtfx_0[4]_net_1\, Y => 
        \ftbkrvpxdbp_5[2]_net_1\);
    
    \ftbkrvpxdbp[0]\ : DFN1E0C0
      port map(D => \ftbkrvpxdbp_5[0]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \ftbkrvpxdbp_1_sqmuxa\, Q => 
        \ftbkrvpxdbp[0]_net_1\);
    
    \gjkdbmpgrtc_11_0_a2[5]\ : NOR2A
      port map(A => N_437, B => rdgwgtpkbgc_3, Y => N_966);
    
    G_106 : XNOR3
      port map(A => \gjkdbmpgrtc[3]\, B => \ftbkrvpxdbp[3]_net_1\, 
        C => \bnwcvzjbgtc[3]\, Y => N_251);
    
    spbznmtdmpd : DFN1E1C0
      port map(D => un2_pbzxzjhqhwz, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => dbdctxjdvkp_2_sqmuxa, Q => 
        \spbznmtdmpd\);
    
    \gjkdbmpgrtc_9[0]\ : MX2
      port map(A => \rqrqhcwpqdn[0]_net_1\, B => 
        \rqrqhcwpqdn[1]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_530);
    
    \bnwcvzjbgtc_11[8]\ : MX2
      port map(A => N_644, B => \bnvtvdkcnpp[2]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_776);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_1[1]\ : OR3B
      port map(A => \tkcbxxqbtfx[8]_net_1\, B => 
        \qrbxknbdvjj[1]_net_1\, C => \rgmpzqrbghh_0\, Y => N_1055);
    
    \rdgwgtpkbgc_0[1]\ : DFN1E1C0
      port map(D => EV_RAM_DO_B_net_net_6, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \bskrjhjvqpf_1_sqmuxa\, Q => 
        \rdgwgtpkbgc_0[1]_net_1\);
    
    \gjkdbmpgrtc_7[14]\ : NOR2A
      port map(A => N_442, B => rdgwgtpkbgc_3, Y => N_510);
    
    \gjkdbmpgrtc_11_0_a2[1]\ : NOR2A
      port map(A => N_433, B => rdgwgtpkbgc_3, Y => N_963);
    
    \bnwcvzjbgtc_12[1]\ : MX2
      port map(A => N_701, B => N_769, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => \bnwcvzjbgtc[1]\);
    
    gpjgkmhxbkn_NE_0 : XA1A
      port map(A => \rqrqhcwpqdn[5]_net_1\, B => 
        EV_RAM_DO_B_net_net_2, C => gpjgkmhxbkn_3_i, Y => 
        \gpjgkmhxbkn_NE_0\);
    
    \bnwcvzjbgtc_3[7]\ : MX2
      port map(A => N_735, B => N_737, S => 
        \rdgwgtpkbgc[1]_net_1\, Y => N_639);
    
    G_126 : XNOR3
      port map(A => \gjkdbmpgrtc[7]\, B => \bnvtvdkcnpp[1]_net_1\, 
        C => \bnwcvzjbgtc[7]\, Y => N_271);
    
    \ckghxwbsztp_3_0_a2[0]\ : OA1C
      port map(A => N_1107, B => \tkcbxxqbtfx[17]_net_1\, C => 
        \rqrqhcwpqdn[1]_net_1\, Y => \ckghxwbsztp_3[0]\);
    
    G_111 : XOR3
      port map(A => \gjkdbmpgrtc[4]\, B => \ftbkrvpxdbp[4]_net_1\, 
        C => \bnwcvzjbgtc[4]\, Y => N_256_i);
    
    \tkcbxxqbtfx[16]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[1]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[16]_net_1\);
    
    zbcrjjxqjgg_i_o3_1 : AXO5
      port map(A => EV_RAM_DO_B_net_net_1, B => 
        \Z\\EV_RAM_DO_B_net\\\, C => \EV_RAM_DO_B_net[8]\, Y => 
        N_191);
    
    \gjkdbmpgrtc_3[13]\ : MX2
      port map(A => N_537, B => N_539, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_441);
    
    \bnwcvzjbgtc_3[9]\ : MX2
      port map(A => N_737, B => N_739, S => 
        \rdgwgtpkbgc[1]_net_1\, Y => N_641);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_65 : AOI1A
      port map(A => \ACT_LT4_E[3]\, B => \ACT_LT4_E[6]\, C => 
        \ACT_LT4_E[10]\, Y => \DWACT_CMPLE_PO2_DWACT_COMP0_E[2]\);
    
    \gjkdbmpgrtc_12[14]\ : MX2
      port map(A => N_510, B => N_967, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc[14]\);
    
    \bnwcvzjbgtc_6_0_a2[9]\ : NOR2A
      port map(A => \bnvtvdkcnpp[11]_net_1\, B => N_1012_1, Y => 
        N_971);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I10_un1_CO1_i_o3 : MAJ3
      port map(A => N_282, B => N_286, C => N247, Y => N_48);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_25 : AO1C
      port map(A => \bnvtvdkcnpp[10]_net_1\, B => 
        \rqrqhcwpqdn[10]_net_1\, C => N_31, Y => N_36_0);
    
    \mkmkbnghgkd[2]\ : MX2
      port map(A => INIT_ADDR(2), B => TR_EV_ADDR_net_net_5, S
         => INIT_DONE_c, Y => EV_RAM_ADDR_A_net_net_5);
    
    dbdctxjdvkp_2_sqmuxa_0_a3_0_0_a2 : OR2A
      port map(A => \EV_RAM_DO_B_net[8]\, B => 
        \Z\\EV_RAM_DO_B_net\\\, Y => N_1144);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2[1]\ : OR2B
      port map(A => EV_RAM_DO_B_net_net_6, B => N_1088, Y => 
        N_1053);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_0[2]\ : AOI1B
      port map(A => \rgmpzqrbghh_0\, B => \EV_CHHOLD_net_net_2\, 
        C => N_1059, Y => \dhzrwbnrbpw_7_iv_0_0[2]\);
    
    \gjkdbmpgrtc_3[15]\ : MX2
      port map(A => N_539, B => N_961, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_443);
    
    \gjkdbmpgrtc_12[5]\ : MX2
      port map(A => N_501, B => N_966, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc[5]\);
    
    \tkcbxxqbtfx_ns_a3_0_a2[3]\ : NOR2B
      port map(A => INIT_DONE_c, B => \tkcbxxqbtfx[15]_net_1\, Y
         => \tkcbxxqbtfx_ns[3]\);
    
    \bnwcvzjbgtc_11[3]\ : MX2
      port map(A => N_639, B => \ftbkrvpxdbp[3]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_771);
    
    \mtbjcfqsbqf[3]\ : DFN1E0C0
      port map(D => \Z\\ADC_CHNR_net\\\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => qmmwhjxxxmv, Q => 
        \Z\\EV_CHHOLD_net\\_net_1\);
    
    \tkcbxxqbtfx[1]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[16]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[1]_net_1\);
    
    \rqrqhcwpqdn[0]\ : DFN1C0
      port map(D => \rqrqhcwpqdn_5_i[0]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \rqrqhcwpqdn[0]_net_1\);
    
    \bnvtvdkcnpp_9[6]\ : MX2
      port map(A => \bnvtvdkcnpp_5[6]_net_1\, B => 
        \fqkdsnjfpdb[12]\, S => \tkcbxxqbtfx[4]_net_1\, Y => 
        \bnvtvdkcnpp_9[6]_net_1\);
    
    \zbqggxmnpbs_r[1]\ : AOI1B
      port map(A => \fqkdsnjfpdb_m[16]_net_1\, B => 
        \zbqggxmnpbs_0_iv_0[1]_net_1\, C => N_185, Y => 
        EV_RAM_DI_B_net_net_6);
    
    \zbqggxmnpbs_0_iv_0_a2_1[3]\ : OR2B
      port map(A => \tkcbxxqbtfx[0]_net_1\, B => 
        \ckghxwbsztp[2]_net_1\, Y => N_1147);
    
    \ckghxwbsztp[3]\ : DFN1E1C0
      port map(D => \ckghxwbsztp_3[3]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => \tkcbxxqbtfx[1]_net_1\, Q => 
        \ckghxwbsztp[3]_net_1\);
    
    G_161 : XNOR3
      port map(A => \gjkdbmpgrtc[14]\, B => 
        \bnvtvdkcnpp[8]_net_1\, C => \bnwcvzjbgtc[14]\, Y => 
        N_306);
    
    \bnwcvzjbgtc_7[9]\ : MX2
      port map(A => N_641, B => N_971, S => rdgwgtpkbgc_3, Y => 
        N_709);
    
    cmkddgvkrrs_0_iv_i_a2 : NOR2A
      port map(A => N_1128, B => \spbznmtdmpd\, Y => N_1151);
    
    \bnwcvzjbgtc_7[6]\ : MX2
      port map(A => N_638, B => N_646, S => rdgwgtpkbgc_3, Y => 
        N_706);
    
    \rdgwgtpkbgc[3]\ : DFN1E1C0
      port map(D => EV_RAM_DO_B_net_net_4, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \bskrjhjvqpf_1_sqmuxa\, Q => 
        rdgwgtpkbgc_0_3);
    
    kkjrqmngvjc_0_0_3 : NOR2
      port map(A => N_1083, B => \kkjrqmngvjc_0_0_3_1\, Y => 
        N_180_5);
    
    wqpbnjzjqsx_NE_2 : NOR3C
      port map(A => wqpbnjzjqsx_4_i, B => wqpbnjzjqsx_3_i, C => 
        wqpbnjzjqsx_0_i, Y => \wqpbnjzjqsx_NE_2\);
    
    \ftbkrvpxdbp_5[4]\ : MX2B
      port map(A => \Z\\EV_RAM_DO_B_net\\\, B => 
        \fqkdsnjfpdb_i[4]\, S => \tkcbxxqbtfx_0[4]_net_1\, Y => 
        \ftbkrvpxdbp_5[4]_net_1\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_19 : NOR2A
      port map(A => \rqrqhcwpqdn[7]_net_1\, B => 
        \bnvtvdkcnpp[7]_net_1\, Y => N_30);
    
    hmzcpnxsdwg_0_a2 : NOR2B
      port map(A => ASSC_SAMPFLAG_net, B => ASSC_DONE_c, Y => 
        hmzcpnxsdwg);
    
    \qrbxknbdvjj[5]\ : DFN1E1C0
      port map(D => \EV_RAM_ADDR_B_net_net_2\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \tkcbxxqbtfx[11]_net_1\, Q => 
        \qrbxknbdvjj[5]_net_1\);
    
    \bnwcvzjbgtc_12[5]\ : MX2
      port map(A => N_705, B => N_773, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => \bnwcvzjbgtc[5]\);
    
    \gjkdbmpgrtc_3[11]\ : MX2
      port map(A => N_535, B => N_537, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_439);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I12_un1_CO1_i_o3 : MIN3
      port map(A => N_292_i, B => N_296_i, C => N_37, Y => N_39);
    
    ddbbjvkdkcq_0_0_a2 : NOR2A
      port map(A => \Z\\EV_RAM_DO_B_net\\\, B => N_1161, Y => 
        \bskrjhjvqpf_1_sqmuxa\);
    
    \gjkdbmpgrtc_7[16]\ : NOR2A
      port map(A => N_444, B => rdgwgtpkbgc_3, Y => N_512);
    
    \gjkdbmpgrtc_7_0_a2[2]\ : NOR2B
      port map(A => rdgwgtpkbgc_3, B => N_438, Y => N_958);
    
    \bnwcvzjbgtc_11[12]\ : MX2
      port map(A => N_970, B => \bnvtvdkcnpp[6]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_780);
    
    \bnwcvzjbgtc_8[0]\ : MX2
      port map(A => \ftbkrvpxdbp[4]_net_1\, B => 
        \ftbkrvpxdbp[5]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_717);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_2[1]\ : OR2B
      port map(A => I_32_1, B => N_1086, Y => N_1056);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    G_131 : XOR3
      port map(A => \gjkdbmpgrtc[8]\, B => \bnvtvdkcnpp[2]_net_1\, 
        C => \bnwcvzjbgtc[8]\, Y => N_276_i);
    
    \rqrqhcwpqdn_5_i_a2_0[4]\ : NOR2
      port map(A => \rqrqhcwpqdn[4]_net_1\, B => N_1010_i, Y => 
        N_987);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I8_S : XNOR2
      port map(A => ADD_18x18_slow_I8_S_0, B => N_36, Y => 
        \fqkdsnjfpdb[8]\);
    
    \bnwcvzjbgtc_11[5]\ : MX2
      port map(A => N_641, B => \ftbkrvpxdbp[5]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_773);
    
    \gjkdbmpgrtc_12[4]\ : MX2
      port map(A => N_500, B => N_568, S => 
        \rdgwgtpkbgc_0[2]_net_1\, Y => \gjkdbmpgrtc[4]\);
    
    \tkcbxxqbtfx_ns_a3[7]\ : NOR2B
      port map(A => INIT_DONE_c, B => \tkcbxxqbtfx[11]_net_1\, Y
         => \tkcbxxqbtfx_ns[7]\);
    
    \bnwcvzjbgtc_7[12]\ : NOR2A
      port map(A => N_644, B => rdgwgtpkbgc_3, Y => N_712);
    
    \bnwcvzjbgtc_12[15]\ : MX2
      port map(A => N_972, B => N_975, S => 
        \rdgwgtpkbgc[2]_net_1\, Y => \bnwcvzjbgtc[15]\);
    
    \tkcbxxqbtfx[9]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[8]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[9]_net_1\);
    
    \bnvtvdkcnpp_8[11]\ : MX2
      port map(A => EV_RAM_DO_B_net_net_5, B => 
        \bnvtvdkcnpp[11]_net_1\, S => \un1_kkgccghmrzf\, Y => 
        \bnvtvdkcnpp_8[11]_net_1\);
    
    bskrjhjvqpf_i_0_2 : NOR3A
      port map(A => \bskrjhjvqpf_i_0_1\, B => N_1083, C => 
        \kkjrqmngvjc_0_0_3_1\, Y => \bskrjhjvqpf_i_0_2\);
    
    \gjkdbmpgrtc_11_0_a2[16]\ : NOR2B
      port map(A => \rqrqhcwpqdn[10]_net_1\, B => rdgwgtpkbgc_3, 
        Y => N_969);
    
    \tkcbxxqbtfx_ns_a3[8]\ : NOR2B
      port map(A => INIT_DONE_c, B => \tkcbxxqbtfx[10]_net_1\, Y
         => \tkcbxxqbtfx_ns[8]\);
    
    kkjrqmngvjc_0_0_3_1 : OR3
      port map(A => \tkcbxxqbtfx[6]_net_1\, B => 
        \tkcbxxqbtfx[17]_net_1\, C => \EV_ASSCADDR_net_net_5\, Y
         => \kkjrqmngvjc_0_0_3_1\);
    
    \tkcbxxqbtfx[8]\ : DFN1C0
      port map(D => \tkcbxxqbtfx_ns[9]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \tkcbxxqbtfx[8]_net_1\);
    
    kkjrqmngvjc_0_0_0 : NOR2
      port map(A => \tkcbxxqbtfx[1]_net_1\, B => 
        \tkcbxxqbtfx[11]_net_1\, Y => \kkjrqmngvjc_0_0_0\);
    
    \gjkdbmpgrtc_11[11]\ : MX2
      port map(A => N_443, B => \rqrqhcwpqdn[5]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_575);
    
    \bnwcvzjbgtc_11[0]\ : MX2
      port map(A => N_640, B => \ftbkrvpxdbp[0]_net_1\, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => N_768);
    
    kkjrqmngvjc_0_0_a2_0_0_a2_1 : OR2B
      port map(A => \EV_RAM_DO_B_net[8]\, B => 
        \Z\\EV_RAM_DO_B_net\\\, Y => bskrjhjvqpf_6_sqmuxa_1);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I2_S_0 : XOR2
      port map(A => N_246, B => N_242, Y => ADD_18x18_slow_I2_S_0);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_59 : OR2A
      port map(A => \rqrqhcwpqdn[2]_net_1\, B => 
        \bnvtvdkcnpp[2]_net_1\, Y => \ACT_LT4_E[4]\);
    
    un1_fskhwddcbbg_i_o3_0 : OR2
      port map(A => \tkcbxxqbtfx[3]_net_1\, B => 
        \tkcbxxqbtfx[0]_net_1\, Y => \un1_fskhwddcbbg_i_o3_0\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_26 : OA1A
      port map(A => N_32, B => N_34, C => N_33, Y => N_37_0);
    
    \tkcbxxqbtfx_ns_a3_0_a2[15]\ : NOR3A
      port map(A => INIT_DONE_c, B => N_1159, C => N_1144, Y => 
        \tkcbxxqbtfx_ns[15]\);
    
    \gjkdbmpgrtc_11[10]\ : MX2
      port map(A => N_442, B => \rqrqhcwpqdn[4]_net_1\, S => 
        rdgwgtpkbgc_3, Y => N_574);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_0[3]\ : AOI1B
      port map(A => \rgmpzqrbghh_0\, B => \EV_CHHOLD_net_net_1\, 
        C => N_1063, Y => \dhzrwbnrbpw_7_iv_0_0[3]\);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_2[3]\ : OR2B
      port map(A => I_37_1, B => N_1086, Y => N_1064);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    G_171 : XOR3
      port map(A => \gjkdbmpgrtc[16]\, B => 
        \bnvtvdkcnpp[10]_net_1\, C => \bnwcvzjbgtc[16]\, Y => 
        N_316_i_0);
    
    \qrbxknbdvjj[0]\ : DFN1E1C0
      port map(D => \EV_RAM_ADDR_B_net_net_7\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, E => \tkcbxxqbtfx[11]_net_1\, Q => 
        \qrbxknbdvjj[0]_net_1\);
    
    \bnwcvzjbgtc_7[0]\ : MX2
      port map(A => N_632, B => N_636, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => N_700);
    
    \zbqggxmnpbs_0_iv_0[3]\ : AO1B
      port map(A => \fqkdsnjfpdb[9]\, B => \tkcbxxqbtfx[5]_net_1\, 
        C => \zbqggxmnpbs_0_iv_0_0[3]_net_1\, Y => 
        EV_RAM_DI_B_net_net_4);
    
    un1_dhzrwbnrbpw_I_43 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_1[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_1[0]\, Y => 
        \DWACT_ADD_CI_0_g_array_2[0]\);
    
    G_155 : AO13
      port map(A => \bnvtvdkcnpp[6]_net_1\, B => 
        \gjkdbmpgrtc[12]\, C => \bnwcvzjbgtc[12]\, Y => N_297);
    
    \gjkdbmpgrtc_9[8]\ : MX2
      port map(A => \rqrqhcwpqdn[8]_net_1\, B => 
        \rqrqhcwpqdn[9]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_538);
    
    \gjkdbmpgrtc_3[8]\ : MX2
      port map(A => N_532, B => N_534, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_436);
    
    \rqrqhcwpqdn[7]\ : DFN1C0
      port map(D => \rqrqhcwpqdn_5_i[7]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \rqrqhcwpqdn[7]_net_1\);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_45 : AOI1A
      port map(A => \ACT_LT3_E[0]\, B => \ACT_LT3_E[1]\, C => 
        \ACT_LT3_E[2]\, Y => \ACT_LT3_E[3]\);
    
    G_116 : XNOR3
      port map(A => \gjkdbmpgrtc[5]\, B => \ftbkrvpxdbp[5]_net_1\, 
        C => \bnwcvzjbgtc[5]\, Y => N_261);
    
    \pptwzsjcmcz_dhzrwbnrbpw_7_iv_0_a2_0[8]\ : OR3B
      port map(A => \tkcbxxqbtfx[8]_net_1\, B => 
        \qrbxknbdvjj[8]_net_1\, C => \rgmpzqrbghh\, Y => N_1080);
    
    \bnvtvdkcnpp_8[9]\ : MX2
      port map(A => EV_RAM_DO_B_net_net_7, B => 
        \bnvtvdkcnpp[9]_net_1\, S => \un1_kkgccghmrzf\, Y => 
        \bnvtvdkcnpp_8[9]_net_1\);
    
    \zbqggxmnpbs_0_iv_0_a2[5]\ : OR2A
      port map(A => \tkcbxxqbtfx[5]_net_1\, B => 
        \fqkdsnjfpdb_i[11]\, Y => N_1148);
    
    pvtdpmmqdpp_un2_pbzxzjhqhwz_0_I_18 : OR2A
      port map(A => \bnvtvdkcnpp[9]_net_1\, B => 
        \rqrqhcwpqdn[9]_net_1\, Y => N_29);
    
    \gjkdbmpgrtc_7[12]\ : NOR2A
      port map(A => N_440, B => rdgwgtpkbgc_3, Y => N_508);
    
    \bnwcvzjbgtc_12[3]\ : MX2
      port map(A => N_703, B => N_771, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => \bnwcvzjbgtc[3]\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I13_S : XNOR2
      port map(A => ADD_18x18_slow_I13_S_0, B => N_39, Y => 
        \fqkdsnjfpdb_i[13]\);
    
    \dhzrwbnrbpw[4]\ : DFN1C0
      port map(D => \dhzrwbnrbpw_7[4]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \EV_RAM_ADDR_B_net_net_3\);
    
    \rqrqhcwpqdn[10]\ : DFN1C0
      port map(D => \rqrqhcwpqdn_9_i[10]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \rqrqhcwpqdn[10]_net_1\);
    
    \gjkdbmpgrtc_9[2]\ : MX2
      port map(A => \rqrqhcwpqdn[2]_net_1\, B => 
        \rqrqhcwpqdn[3]_net_1\, S => \rdgwgtpkbgc_0[0]_net_1\, Y
         => N_532);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I9_S : XOR3
      port map(A => N_277, B => N_281, C => N_40, Y => 
        \fqkdsnjfpdb[9]\);
    
    \bnvtvdkcnpp[7]\ : DFN1C0
      port map(D => \bnvtvdkcnpp_9[7]_net_1\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \bnvtvdkcnpp[7]_net_1\);
    
    \rqrqhcwpqdn_5_i_m3[0]\ : MX2
      port map(A => \rqrqhcwpqdn[0]_net_1\, B => 
        ASSC_RAM_DO_A_net_net_7, S => \EV_ASSCADDR_net_net_5\, Y
         => N_1122);
    
    wqpbnjzjqsx_4_0_a2 : XNOR2
      port map(A => \rqrqhcwpqdn[5]_net_1\, B => 
        \bnvtvdkcnpp[5]_net_1\, Y => wqpbnjzjqsx_4_i);
    
    \gjkdbmpgrtc_3[14]\ : MX2
      port map(A => N_538, B => N_540, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_442);
    
    \rqrqhcwpqdn_5_i_a2_0[7]\ : NOR2
      port map(A => \rqrqhcwpqdn[7]_net_1\, B => N_1010_i, Y => 
        N_996);
    
    \bnwcvzjbgtc_12[9]\ : MX2
      port map(A => N_709, B => N_777, S => 
        \rdgwgtpkbgc_1[2]_net_1\, Y => \bnwcvzjbgtc[9]\);
    
    \gjkdbmpgrtc_3[10]\ : MX2
      port map(A => N_534, B => N_536, S => 
        \rdgwgtpkbgc_0[1]_net_1\, Y => N_438);
    
    \zbqggxmnpbs_0_iv_1[0]\ : AOI1B
      port map(A => \fqkdsnjfpdb[6]\, B => \tkcbxxqbtfx[5]_net_1\, 
        C => \zbqggxmnpbs_0_iv_0[0]_net_1\, Y => 
        \zbqggxmnpbs_0_iv_1[0]_net_1\);
    
    fqkdsnjfpdb_0_0_ADD_18x18_slow_I5_CO1_i_o3_0 : OAI1
      port map(A => N_261, B => N_41, C => N_257, Y => 
        ADD_18x18_slow_I5_CO1_i_o3_0);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity SMEV is

    port(INIT_ADDR : in std_logic_vector(4 downto 0); 
        INIT_ADDR_int : in std_logic_vector(8 downto 5); 
        \EV_RAM_DO_B_net[8]\ : in std_logic; \EV_CHHOLD_net[4]\ : 
        out std_logic; \rdgwgtpkbgc_0[3]\ : in std_logic; 
        \EV_RAM_ADDR_A_net[8]\ : out std_logic; 
        \TR_EV_ADDR_net[8]\, \ASSC_RAM_DO_A_net[8]\ : in
         std_logic; \EV_RAM_DI_B_net[8]\ : out std_logic; 
        \tkcbxxqbtfx_ns[11]\ : in std_logic; \EV_ASSCADDR_net[8]\ : 
        out std_logic; \ASSC_SEQOUT_net[5]\, \ADC_CHNR_net[4]\ : 
        in std_logic; \EV_RAM_ADDR_B_net[8]\, \rdgwgtpkbgc[3]\, 
        EV_DONE_net_0 : out std_logic; ACM_CLK_c, sys_reset_gl, 
        EV_RAM_DO_B_net_net_7 : in std_logic; 
        bskrjhjvqpf_1_sqmuxa : out std_logic; 
        EV_RAM_DO_B_net_net_6, EV_RAM_DO_B_net_net_5 : in
         std_logic; EV_RAM_ADDR_B_net_net_6, 
        EV_RAM_ADDR_B_net_net_3, EV_RAM_ADDR_B_net_net_5, 
        EV_RAM_ADDR_B_net_net_1 : out std_logic; 
        \Z\\EV_RAM_DO_B_net\\\, EV_RAM_DO_B_net_net_1 : in
         std_logic; \Z\\EV_CHHOLD_net\\\, EV_CHHOLD_net_net_1, 
        EV_CHHOLD_net_net_2, EV_CHHOLD_net_net_3 : out std_logic; 
        EV_RAM_DO_B_net_net_2 : in std_logic; 
        EV_ASSCADDR_net_net_5, EV_RAM_CSN_B_net : out std_logic; 
        INIT_DONE_c : in std_logic; EV_RAM_DI_B_net_net_5, 
        EV_RAM_DI_B_net_net_3, EV_RAM_DI_B_net_net_4 : out
         std_logic; EV_RAM_DO_B_net_net_4, EV_RAM_DO_B_net_net_3, 
        ASSC_RAM_DO_A_net_net_6, ASSC_RAM_DO_A_net_net_5, 
        ASSC_RAM_DO_A_net_net_4, ASSC_RAM_DO_A_net_net_3, 
        ASSC_RAM_DO_A_net_net_2, ASSC_RAM_DO_A_net_net_1, 
        \Z\\ASSC_RAM_DO_A_net\\\, ASSC_RAM_DO_A_net_net_7 : in
         std_logic; EV_RAM_ADDR_A_net_net_5 : out std_logic; 
        TR_EV_ADDR_net_net_5 : in std_logic; 
        \Z\\EV_RAM_ADDR_A_net\\\ : out std_logic; 
        \Z\\TR_EV_ADDR_net\\\ : in std_logic; 
        EV_RAM_ADDR_A_net_net_1 : out std_logic; 
        TR_EV_ADDR_net_net_1 : in std_logic; 
        EV_RAM_ADDR_A_net_net_2 : out std_logic; 
        TR_EV_ADDR_net_net_2 : in std_logic; 
        EV_RAM_ADDR_A_net_net_3 : out std_logic; 
        TR_EV_ADDR_net_net_3 : in std_logic; 
        EV_RAM_ADDR_A_net_net_4 : out std_logic; 
        TR_EV_ADDR_net_net_4 : in std_logic; 
        EV_RAM_ADDR_A_net_net_6 : out std_logic; 
        TR_EV_ADDR_net_net_6 : in std_logic; 
        EV_RAM_ADDR_A_net_net_7 : out std_logic; 
        TR_EV_ADDR_net_net_7, rhbzpsxhwms, jngrzvhtjvn_0, 
        ASSC_SAMPFLAG_net, ASSC_DONE_c : in std_logic; 
        EV_RAM_RW_A_net : out std_logic; INIT_EV_WEN : in
         std_logic; EV_RAM_CSN_A_net : out std_logic; 
        TR_EV_RD_net : in std_logic; EV_RAM_DI_B_net_net_1, 
        \Z\\EV_RAM_DI_B_net\\\, EV_RAM_DI_B_net_net_2, 
        EV_RAM_DI_B_net_net_6, EV_RAM_DI_B_net_net_7, 
        EV_RAM_WR_BUSY_B_net, EV_DONE_net, EV_EVFLAG_net, 
        EV_ASSCADDR_net_net_4 : out std_logic; 
        ASSC_SEQOUT_net_net_4 : in std_logic; 
        EV_ASSCADDR_net_net_3 : out std_logic; 
        ASSC_SEQOUT_net_net_3 : in std_logic; 
        EV_ASSCADDR_net_net_2 : out std_logic; 
        ASSC_SEQOUT_net_net_2 : in std_logic; 
        EV_ASSCADDR_net_net_1 : out std_logic; 
        ASSC_SEQOUT_net_net_1 : in std_logic; 
        \Z\\EV_ASSCADDR_net\\\ : out std_logic; 
        \Z\\ASSC_SEQOUT_net\\\, ADC_CHNR_net_net_3, 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_1, 
        \Z\\ADC_CHNR_net\\\ : in std_logic; 
        EV_RAM_ADDR_B_net_net_7, EV_RAM_ADDR_B_net_net_4, 
        EV_RAM_ADDR_B_net_net_2, \Z\\EV_RAM_ADDR_B_net\\\ : out
         std_logic);

end SMEV;

architecture DEF_ARCH of SMEV is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component nbnjbfrkqwq
    port(INIT_ADDR_int : 
        in std_logic_vector(8 downto 5) := (others => 'U'); 
        INIT_ADDR : 
        in std_logic_vector(4 downto 0) := (others => 'U'); 
        rdgwgtpkbgc_0_3, \EV_RAM_ADDR_B_net[8]\ : out std_logic; 
        \ADC_CHNR_net[4]\, \ASSC_SEQOUT_net[5]\ : in
         std_logic := 'U'; \EV_ASSCADDR_net[8]\ : out std_logic; 
        tkcbxxqbtfx_ns_11 : in std_logic := 'U'; 
        \EV_RAM_DI_B_net[8]\ : out std_logic; 
        \ASSC_RAM_DO_A_net[8]\, \TR_EV_ADDR_net[8]\ : in
         std_logic := 'U'; \EV_RAM_ADDR_A_net[8]\ : out std_logic; 
        rdgwgtpkbgc_3 : in std_logic := 'U'; \EV_CHHOLD_net[4]\ : 
        out std_logic; \EV_RAM_DO_B_net[8]\ : in std_logic := 'U'; 
        \Z\\EV_RAM_ADDR_B_net\\\, EV_RAM_ADDR_B_net_net_2, 
        EV_RAM_ADDR_B_net_net_4, EV_RAM_ADDR_B_net_net_7 : out
         std_logic; \Z\\ADC_CHNR_net\\\, ADC_CHNR_net_net_1, 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_3, 
        \Z\\ASSC_SEQOUT_net\\\ : in std_logic := 'U'; 
        \Z\\EV_ASSCADDR_net\\\ : out std_logic; 
        ASSC_SEQOUT_net_net_1 : in std_logic := 'U'; 
        EV_ASSCADDR_net_net_1 : out std_logic; 
        ASSC_SEQOUT_net_net_2 : in std_logic := 'U'; 
        EV_ASSCADDR_net_net_2 : out std_logic; 
        ASSC_SEQOUT_net_net_3 : in std_logic := 'U'; 
        EV_ASSCADDR_net_net_3 : out std_logic; 
        ASSC_SEQOUT_net_net_4 : in std_logic := 'U'; 
        EV_ASSCADDR_net_net_4, EV_EVFLAG_net, EV_DONE_net, 
        EV_RAM_WR_BUSY_B_net, EV_RAM_DI_B_net_net_7, 
        EV_RAM_DI_B_net_net_6, EV_RAM_DI_B_net_net_2, 
        \Z\\EV_RAM_DI_B_net\\\, EV_RAM_DI_B_net_net_1 : out
         std_logic; TR_EV_RD_net : in std_logic := 'U'; 
        EV_RAM_CSN_A_net : out std_logic; INIT_EV_WEN : in
         std_logic := 'U'; EV_RAM_RW_A_net : out std_logic; 
        ASSC_DONE_c, ASSC_SAMPFLAG_net, jngrzvhtjvn_0, 
        rhbzpsxhwms, TR_EV_ADDR_net_net_7 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_7 : out std_logic; 
        TR_EV_ADDR_net_net_6 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_6 : out std_logic; 
        TR_EV_ADDR_net_net_4 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_4 : out std_logic; 
        TR_EV_ADDR_net_net_3 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_3 : out std_logic; 
        TR_EV_ADDR_net_net_2 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_2 : out std_logic; 
        TR_EV_ADDR_net_net_1 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_1 : out std_logic; 
        \Z\\TR_EV_ADDR_net\\\ : in std_logic := 'U'; 
        \Z\\EV_RAM_ADDR_A_net\\\ : out std_logic; 
        TR_EV_ADDR_net_net_5 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_5 : out std_logic; 
        ASSC_RAM_DO_A_net_net_7, \Z\\ASSC_RAM_DO_A_net\\\, 
        ASSC_RAM_DO_A_net_net_1, ASSC_RAM_DO_A_net_net_2, 
        ASSC_RAM_DO_A_net_net_3, ASSC_RAM_DO_A_net_net_4, 
        ASSC_RAM_DO_A_net_net_5, ASSC_RAM_DO_A_net_net_6, 
        EV_RAM_DO_B_net_net_3, EV_RAM_DO_B_net_net_4 : in
         std_logic := 'U'; EV_RAM_DI_B_net_net_4, 
        EV_RAM_DI_B_net_net_3, EV_RAM_DI_B_net_net_5 : out
         std_logic; INIT_DONE_c : in std_logic := 'U'; 
        EV_RAM_CSN_B_net, EV_ASSCADDR_net_net_5 : out std_logic; 
        EV_RAM_DO_B_net_net_2 : in std_logic := 'U'; 
        EV_CHHOLD_net_net_3, EV_CHHOLD_net_net_2, 
        EV_CHHOLD_net_net_1, \Z\\EV_CHHOLD_net\\\ : out std_logic; 
        EV_RAM_DO_B_net_net_1, \Z\\EV_RAM_DO_B_net\\\ : in
         std_logic := 'U'; EV_RAM_ADDR_B_net_net_1, 
        EV_RAM_ADDR_B_net_net_5, EV_RAM_ADDR_B_net_net_3, 
        EV_RAM_ADDR_B_net_net_6 : out std_logic; 
        EV_RAM_DO_B_net_net_5, EV_RAM_DO_B_net_net_6 : in
         std_logic := 'U'; bskrjhjvqpf_1_sqmuxa : out std_logic; 
        EV_RAM_DO_B_net_net_7, sys_reset_gl, ACM_CLK_c : in
         std_logic := 'U'; EV_DONE_net_0 : out std_logic);
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : nbnjbfrkqwq
	Use entity work.nbnjbfrkqwq(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    u_nbnjbfrkqwq : nbnjbfrkqwq
      port map(INIT_ADDR_int(8) => INIT_ADDR_int(8), 
        INIT_ADDR_int(7) => INIT_ADDR_int(7), INIT_ADDR_int(6)
         => INIT_ADDR_int(6), INIT_ADDR_int(5) => 
        INIT_ADDR_int(5), INIT_ADDR(4) => INIT_ADDR(4), 
        INIT_ADDR(3) => INIT_ADDR(3), INIT_ADDR(2) => 
        INIT_ADDR(2), INIT_ADDR(1) => INIT_ADDR(1), INIT_ADDR(0)
         => INIT_ADDR(0), rdgwgtpkbgc_0_3 => \rdgwgtpkbgc[3]\, 
        \EV_RAM_ADDR_B_net[8]\ => \EV_RAM_ADDR_B_net[8]\, 
        \ADC_CHNR_net[4]\ => \ADC_CHNR_net[4]\, 
        \ASSC_SEQOUT_net[5]\ => \ASSC_SEQOUT_net[5]\, 
        \EV_ASSCADDR_net[8]\ => \EV_ASSCADDR_net[8]\, 
        tkcbxxqbtfx_ns_11 => \tkcbxxqbtfx_ns[11]\, 
        \EV_RAM_DI_B_net[8]\ => \EV_RAM_DI_B_net[8]\, 
        \ASSC_RAM_DO_A_net[8]\ => \ASSC_RAM_DO_A_net[8]\, 
        \TR_EV_ADDR_net[8]\ => \TR_EV_ADDR_net[8]\, 
        \EV_RAM_ADDR_A_net[8]\ => \EV_RAM_ADDR_A_net[8]\, 
        rdgwgtpkbgc_3 => \rdgwgtpkbgc_0[3]\, \EV_CHHOLD_net[4]\
         => \EV_CHHOLD_net[4]\, \EV_RAM_DO_B_net[8]\ => 
        \EV_RAM_DO_B_net[8]\, \Z\\EV_RAM_ADDR_B_net\\\ => 
        \Z\\EV_RAM_ADDR_B_net\\\, EV_RAM_ADDR_B_net_net_2 => 
        EV_RAM_ADDR_B_net_net_2, EV_RAM_ADDR_B_net_net_4 => 
        EV_RAM_ADDR_B_net_net_4, EV_RAM_ADDR_B_net_net_7 => 
        EV_RAM_ADDR_B_net_net_7, \Z\\ADC_CHNR_net\\\ => 
        \Z\\ADC_CHNR_net\\\, ADC_CHNR_net_net_1 => 
        ADC_CHNR_net_net_1, ADC_CHNR_net_net_2 => 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_3 => 
        ADC_CHNR_net_net_3, \Z\\ASSC_SEQOUT_net\\\ => 
        \Z\\ASSC_SEQOUT_net\\\, \Z\\EV_ASSCADDR_net\\\ => 
        \Z\\EV_ASSCADDR_net\\\, ASSC_SEQOUT_net_net_1 => 
        ASSC_SEQOUT_net_net_1, EV_ASSCADDR_net_net_1 => 
        EV_ASSCADDR_net_net_1, ASSC_SEQOUT_net_net_2 => 
        ASSC_SEQOUT_net_net_2, EV_ASSCADDR_net_net_2 => 
        EV_ASSCADDR_net_net_2, ASSC_SEQOUT_net_net_3 => 
        ASSC_SEQOUT_net_net_3, EV_ASSCADDR_net_net_3 => 
        EV_ASSCADDR_net_net_3, ASSC_SEQOUT_net_net_4 => 
        ASSC_SEQOUT_net_net_4, EV_ASSCADDR_net_net_4 => 
        EV_ASSCADDR_net_net_4, EV_EVFLAG_net => EV_EVFLAG_net, 
        EV_DONE_net => EV_DONE_net, EV_RAM_WR_BUSY_B_net => 
        EV_RAM_WR_BUSY_B_net, EV_RAM_DI_B_net_net_7 => 
        EV_RAM_DI_B_net_net_7, EV_RAM_DI_B_net_net_6 => 
        EV_RAM_DI_B_net_net_6, EV_RAM_DI_B_net_net_2 => 
        EV_RAM_DI_B_net_net_2, \Z\\EV_RAM_DI_B_net\\\ => 
        \Z\\EV_RAM_DI_B_net\\\, EV_RAM_DI_B_net_net_1 => 
        EV_RAM_DI_B_net_net_1, TR_EV_RD_net => TR_EV_RD_net, 
        EV_RAM_CSN_A_net => EV_RAM_CSN_A_net, INIT_EV_WEN => 
        INIT_EV_WEN, EV_RAM_RW_A_net => EV_RAM_RW_A_net, 
        ASSC_DONE_c => ASSC_DONE_c, ASSC_SAMPFLAG_net => 
        ASSC_SAMPFLAG_net, jngrzvhtjvn_0 => jngrzvhtjvn_0, 
        rhbzpsxhwms => rhbzpsxhwms, TR_EV_ADDR_net_net_7 => 
        TR_EV_ADDR_net_net_7, EV_RAM_ADDR_A_net_net_7 => 
        EV_RAM_ADDR_A_net_net_7, TR_EV_ADDR_net_net_6 => 
        TR_EV_ADDR_net_net_6, EV_RAM_ADDR_A_net_net_6 => 
        EV_RAM_ADDR_A_net_net_6, TR_EV_ADDR_net_net_4 => 
        TR_EV_ADDR_net_net_4, EV_RAM_ADDR_A_net_net_4 => 
        EV_RAM_ADDR_A_net_net_4, TR_EV_ADDR_net_net_3 => 
        TR_EV_ADDR_net_net_3, EV_RAM_ADDR_A_net_net_3 => 
        EV_RAM_ADDR_A_net_net_3, TR_EV_ADDR_net_net_2 => 
        TR_EV_ADDR_net_net_2, EV_RAM_ADDR_A_net_net_2 => 
        EV_RAM_ADDR_A_net_net_2, TR_EV_ADDR_net_net_1 => 
        TR_EV_ADDR_net_net_1, EV_RAM_ADDR_A_net_net_1 => 
        EV_RAM_ADDR_A_net_net_1, \Z\\TR_EV_ADDR_net\\\ => 
        \Z\\TR_EV_ADDR_net\\\, \Z\\EV_RAM_ADDR_A_net\\\ => 
        \Z\\EV_RAM_ADDR_A_net\\\, TR_EV_ADDR_net_net_5 => 
        TR_EV_ADDR_net_net_5, EV_RAM_ADDR_A_net_net_5 => 
        EV_RAM_ADDR_A_net_net_5, ASSC_RAM_DO_A_net_net_7 => 
        ASSC_RAM_DO_A_net_net_7, \Z\\ASSC_RAM_DO_A_net\\\ => 
        \Z\\ASSC_RAM_DO_A_net\\\, ASSC_RAM_DO_A_net_net_1 => 
        ASSC_RAM_DO_A_net_net_1, ASSC_RAM_DO_A_net_net_2 => 
        ASSC_RAM_DO_A_net_net_2, ASSC_RAM_DO_A_net_net_3 => 
        ASSC_RAM_DO_A_net_net_3, ASSC_RAM_DO_A_net_net_4 => 
        ASSC_RAM_DO_A_net_net_4, ASSC_RAM_DO_A_net_net_5 => 
        ASSC_RAM_DO_A_net_net_5, ASSC_RAM_DO_A_net_net_6 => 
        ASSC_RAM_DO_A_net_net_6, EV_RAM_DO_B_net_net_3 => 
        EV_RAM_DO_B_net_net_3, EV_RAM_DO_B_net_net_4 => 
        EV_RAM_DO_B_net_net_4, EV_RAM_DI_B_net_net_4 => 
        EV_RAM_DI_B_net_net_4, EV_RAM_DI_B_net_net_3 => 
        EV_RAM_DI_B_net_net_3, EV_RAM_DI_B_net_net_5 => 
        EV_RAM_DI_B_net_net_5, INIT_DONE_c => INIT_DONE_c, 
        EV_RAM_CSN_B_net => EV_RAM_CSN_B_net, 
        EV_ASSCADDR_net_net_5 => EV_ASSCADDR_net_net_5, 
        EV_RAM_DO_B_net_net_2 => EV_RAM_DO_B_net_net_2, 
        EV_CHHOLD_net_net_3 => EV_CHHOLD_net_net_3, 
        EV_CHHOLD_net_net_2 => EV_CHHOLD_net_net_2, 
        EV_CHHOLD_net_net_1 => EV_CHHOLD_net_net_1, 
        \Z\\EV_CHHOLD_net\\\ => \Z\\EV_CHHOLD_net\\\, 
        EV_RAM_DO_B_net_net_1 => EV_RAM_DO_B_net_net_1, 
        \Z\\EV_RAM_DO_B_net\\\ => \Z\\EV_RAM_DO_B_net\\\, 
        EV_RAM_ADDR_B_net_net_1 => EV_RAM_ADDR_B_net_net_1, 
        EV_RAM_ADDR_B_net_net_5 => EV_RAM_ADDR_B_net_net_5, 
        EV_RAM_ADDR_B_net_net_3 => EV_RAM_ADDR_B_net_net_3, 
        EV_RAM_ADDR_B_net_net_6 => EV_RAM_ADDR_B_net_net_6, 
        EV_RAM_DO_B_net_net_5 => EV_RAM_DO_B_net_net_5, 
        EV_RAM_DO_B_net_net_6 => EV_RAM_DO_B_net_net_6, 
        bskrjhjvqpf_1_sqmuxa => bskrjhjvqpf_1_sqmuxa, 
        EV_RAM_DO_B_net_net_7 => EV_RAM_DO_B_net_net_7, 
        sys_reset_gl => sys_reset_gl, ACM_CLK_c => ACM_CLK_c, 
        EV_DONE_net_0 => EV_DONE_net_0);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity Analog_RTC_smev_wrapper is

    port(INIT_ADDR_int : in std_logic_vector(8 downto 5); 
        INIT_ADDR : in std_logic_vector(4 downto 0); 
        \rdgwgtpkbgc_0[3]\, \EV_RAM_ADDR_B_net[8]\ : out
         std_logic; \ADC_CHNR_net[4]\, \ASSC_SEQOUT_net[5]\ : in
         std_logic; \EV_ASSCADDR_net[8]\ : out std_logic; 
        \tkcbxxqbtfx_ns[11]\ : in std_logic; \EV_RAM_DI_B_net[8]\ : 
        out std_logic; \ASSC_RAM_DO_A_net[8]\, 
        \TR_EV_ADDR_net[8]\ : in std_logic; 
        \EV_RAM_ADDR_A_net[8]\ : out std_logic; \rdgwgtpkbgc[3]\ : 
        in std_logic; \EV_CHHOLD_net[4]\ : out std_logic; 
        \EV_RAM_DO_B_net[8]\ : in std_logic; 
        \Z\\EV_RAM_ADDR_B_net\\\, EV_RAM_ADDR_B_net_net_2, 
        EV_RAM_ADDR_B_net_net_4, EV_RAM_ADDR_B_net_net_7 : out
         std_logic; \Z\\ADC_CHNR_net\\\, ADC_CHNR_net_net_1, 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_3, 
        \Z\\ASSC_SEQOUT_net\\\ : in std_logic; 
        \Z\\EV_ASSCADDR_net\\\ : out std_logic; 
        ASSC_SEQOUT_net_net_1 : in std_logic; 
        EV_ASSCADDR_net_net_1 : out std_logic; 
        ASSC_SEQOUT_net_net_2 : in std_logic; 
        EV_ASSCADDR_net_net_2 : out std_logic; 
        ASSC_SEQOUT_net_net_3 : in std_logic; 
        EV_ASSCADDR_net_net_3 : out std_logic; 
        ASSC_SEQOUT_net_net_4 : in std_logic; 
        EV_ASSCADDR_net_net_4, EV_EVFLAG_net, EV_DONE_net, 
        EV_RAM_WR_BUSY_B_net, EV_RAM_DI_B_net_net_7, 
        EV_RAM_DI_B_net_net_6, EV_RAM_DI_B_net_net_2, 
        \Z\\EV_RAM_DI_B_net\\\, EV_RAM_DI_B_net_net_1 : out
         std_logic; TR_EV_RD_net : in std_logic; EV_RAM_CSN_A_net : 
        out std_logic; INIT_EV_WEN : in std_logic; 
        EV_RAM_RW_A_net : out std_logic; ASSC_DONE_c, 
        ASSC_SAMPFLAG_net, jngrzvhtjvn_0, rhbzpsxhwms, 
        TR_EV_ADDR_net_net_7 : in std_logic; 
        EV_RAM_ADDR_A_net_net_7 : out std_logic; 
        TR_EV_ADDR_net_net_6 : in std_logic; 
        EV_RAM_ADDR_A_net_net_6 : out std_logic; 
        TR_EV_ADDR_net_net_4 : in std_logic; 
        EV_RAM_ADDR_A_net_net_4 : out std_logic; 
        TR_EV_ADDR_net_net_3 : in std_logic; 
        EV_RAM_ADDR_A_net_net_3 : out std_logic; 
        TR_EV_ADDR_net_net_2 : in std_logic; 
        EV_RAM_ADDR_A_net_net_2 : out std_logic; 
        TR_EV_ADDR_net_net_1 : in std_logic; 
        EV_RAM_ADDR_A_net_net_1 : out std_logic; 
        \Z\\TR_EV_ADDR_net\\\ : in std_logic; 
        \Z\\EV_RAM_ADDR_A_net\\\ : out std_logic; 
        TR_EV_ADDR_net_net_5 : in std_logic; 
        EV_RAM_ADDR_A_net_net_5 : out std_logic; 
        ASSC_RAM_DO_A_net_net_7, \Z\\ASSC_RAM_DO_A_net\\\, 
        ASSC_RAM_DO_A_net_net_1, ASSC_RAM_DO_A_net_net_2, 
        ASSC_RAM_DO_A_net_net_3, ASSC_RAM_DO_A_net_net_4, 
        ASSC_RAM_DO_A_net_net_5, ASSC_RAM_DO_A_net_net_6, 
        EV_RAM_DO_B_net_net_3, EV_RAM_DO_B_net_net_4 : in
         std_logic; EV_RAM_DI_B_net_net_4, EV_RAM_DI_B_net_net_3, 
        EV_RAM_DI_B_net_net_5 : out std_logic; INIT_DONE_c : in
         std_logic; EV_RAM_CSN_B_net, EV_ASSCADDR_net_net_5 : out
         std_logic; EV_RAM_DO_B_net_net_2 : in std_logic; 
        EV_CHHOLD_net_net_3, EV_CHHOLD_net_net_2, 
        EV_CHHOLD_net_net_1, \Z\\EV_CHHOLD_net\\\ : out std_logic; 
        EV_RAM_DO_B_net_net_1, \Z\\EV_RAM_DO_B_net\\\ : in
         std_logic; EV_RAM_ADDR_B_net_net_1, 
        EV_RAM_ADDR_B_net_net_5, EV_RAM_ADDR_B_net_net_3, 
        EV_RAM_ADDR_B_net_net_6 : out std_logic; 
        EV_RAM_DO_B_net_net_5, EV_RAM_DO_B_net_net_6 : in
         std_logic; bskrjhjvqpf_1_sqmuxa : out std_logic; 
        EV_RAM_DO_B_net_net_7, sys_reset_gl, ACM_CLK_c : in
         std_logic; EV_DONE_net_0 : out std_logic);

end Analog_RTC_smev_wrapper;

architecture DEF_ARCH of Analog_RTC_smev_wrapper is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component SMEV
    port(INIT_ADDR : 
        in std_logic_vector(4 downto 0) := (others => 'U'); 
        INIT_ADDR_int : 
        in std_logic_vector(8 downto 5) := (others => 'U'); 
        \EV_RAM_DO_B_net[8]\ : in std_logic := 'U'; 
        \EV_CHHOLD_net[4]\ : out std_logic; \rdgwgtpkbgc_0[3]\ : 
        in std_logic := 'U'; \EV_RAM_ADDR_A_net[8]\ : out
         std_logic; \TR_EV_ADDR_net[8]\, \ASSC_RAM_DO_A_net[8]\ : 
        in std_logic := 'U'; \EV_RAM_DI_B_net[8]\ : out std_logic; 
        \tkcbxxqbtfx_ns[11]\ : in std_logic := 'U'; 
        \EV_ASSCADDR_net[8]\ : out std_logic; 
        \ASSC_SEQOUT_net[5]\, \ADC_CHNR_net[4]\ : in
         std_logic := 'U'; \EV_RAM_ADDR_B_net[8]\, 
        \rdgwgtpkbgc[3]\, EV_DONE_net_0 : out std_logic; 
        ACM_CLK_c, sys_reset_gl, EV_RAM_DO_B_net_net_7 : in
         std_logic := 'U'; bskrjhjvqpf_1_sqmuxa : out std_logic; 
        EV_RAM_DO_B_net_net_6, EV_RAM_DO_B_net_net_5 : in
         std_logic := 'U'; EV_RAM_ADDR_B_net_net_6, 
        EV_RAM_ADDR_B_net_net_3, EV_RAM_ADDR_B_net_net_5, 
        EV_RAM_ADDR_B_net_net_1 : out std_logic; 
        \Z\\EV_RAM_DO_B_net\\\, EV_RAM_DO_B_net_net_1 : in
         std_logic := 'U'; \Z\\EV_CHHOLD_net\\\, 
        EV_CHHOLD_net_net_1, EV_CHHOLD_net_net_2, 
        EV_CHHOLD_net_net_3 : out std_logic; 
        EV_RAM_DO_B_net_net_2 : in std_logic := 'U'; 
        EV_ASSCADDR_net_net_5, EV_RAM_CSN_B_net : out std_logic; 
        INIT_DONE_c : in std_logic := 'U'; EV_RAM_DI_B_net_net_5, 
        EV_RAM_DI_B_net_net_3, EV_RAM_DI_B_net_net_4 : out
         std_logic; EV_RAM_DO_B_net_net_4, EV_RAM_DO_B_net_net_3, 
        ASSC_RAM_DO_A_net_net_6, ASSC_RAM_DO_A_net_net_5, 
        ASSC_RAM_DO_A_net_net_4, ASSC_RAM_DO_A_net_net_3, 
        ASSC_RAM_DO_A_net_net_2, ASSC_RAM_DO_A_net_net_1, 
        \Z\\ASSC_RAM_DO_A_net\\\, ASSC_RAM_DO_A_net_net_7 : in
         std_logic := 'U'; EV_RAM_ADDR_A_net_net_5 : out
         std_logic; TR_EV_ADDR_net_net_5 : in std_logic := 'U'; 
        \Z\\EV_RAM_ADDR_A_net\\\ : out std_logic; 
        \Z\\TR_EV_ADDR_net\\\ : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_1 : out std_logic; 
        TR_EV_ADDR_net_net_1 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_2 : out std_logic; 
        TR_EV_ADDR_net_net_2 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_3 : out std_logic; 
        TR_EV_ADDR_net_net_3 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_4 : out std_logic; 
        TR_EV_ADDR_net_net_4 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_6 : out std_logic; 
        TR_EV_ADDR_net_net_6 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_7 : out std_logic; 
        TR_EV_ADDR_net_net_7, rhbzpsxhwms, jngrzvhtjvn_0, 
        ASSC_SAMPFLAG_net, ASSC_DONE_c : in std_logic := 'U'; 
        EV_RAM_RW_A_net : out std_logic; INIT_EV_WEN : in
         std_logic := 'U'; EV_RAM_CSN_A_net : out std_logic; 
        TR_EV_RD_net : in std_logic := 'U'; EV_RAM_DI_B_net_net_1, 
        \Z\\EV_RAM_DI_B_net\\\, EV_RAM_DI_B_net_net_2, 
        EV_RAM_DI_B_net_net_6, EV_RAM_DI_B_net_net_7, 
        EV_RAM_WR_BUSY_B_net, EV_DONE_net, EV_EVFLAG_net, 
        EV_ASSCADDR_net_net_4 : out std_logic; 
        ASSC_SEQOUT_net_net_4 : in std_logic := 'U'; 
        EV_ASSCADDR_net_net_3 : out std_logic; 
        ASSC_SEQOUT_net_net_3 : in std_logic := 'U'; 
        EV_ASSCADDR_net_net_2 : out std_logic; 
        ASSC_SEQOUT_net_net_2 : in std_logic := 'U'; 
        EV_ASSCADDR_net_net_1 : out std_logic; 
        ASSC_SEQOUT_net_net_1 : in std_logic := 'U'; 
        \Z\\EV_ASSCADDR_net\\\ : out std_logic; 
        \Z\\ASSC_SEQOUT_net\\\, ADC_CHNR_net_net_3, 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_1, 
        \Z\\ADC_CHNR_net\\\ : in std_logic := 'U'; 
        EV_RAM_ADDR_B_net_net_7, EV_RAM_ADDR_B_net_net_4, 
        EV_RAM_ADDR_B_net_net_2, \Z\\EV_RAM_ADDR_B_net\\\ : out
         std_logic);
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : SMEV
	Use entity work.SMEV(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    U0 : SMEV
      port map(INIT_ADDR(4) => INIT_ADDR(4), INIT_ADDR(3) => 
        INIT_ADDR(3), INIT_ADDR(2) => INIT_ADDR(2), INIT_ADDR(1)
         => INIT_ADDR(1), INIT_ADDR(0) => INIT_ADDR(0), 
        INIT_ADDR_int(8) => INIT_ADDR_int(8), INIT_ADDR_int(7)
         => INIT_ADDR_int(7), INIT_ADDR_int(6) => 
        INIT_ADDR_int(6), INIT_ADDR_int(5) => INIT_ADDR_int(5), 
        \EV_RAM_DO_B_net[8]\ => \EV_RAM_DO_B_net[8]\, 
        \EV_CHHOLD_net[4]\ => \EV_CHHOLD_net[4]\, 
        \rdgwgtpkbgc_0[3]\ => \rdgwgtpkbgc[3]\, 
        \EV_RAM_ADDR_A_net[8]\ => \EV_RAM_ADDR_A_net[8]\, 
        \TR_EV_ADDR_net[8]\ => \TR_EV_ADDR_net[8]\, 
        \ASSC_RAM_DO_A_net[8]\ => \ASSC_RAM_DO_A_net[8]\, 
        \EV_RAM_DI_B_net[8]\ => \EV_RAM_DI_B_net[8]\, 
        \tkcbxxqbtfx_ns[11]\ => \tkcbxxqbtfx_ns[11]\, 
        \EV_ASSCADDR_net[8]\ => \EV_ASSCADDR_net[8]\, 
        \ASSC_SEQOUT_net[5]\ => \ASSC_SEQOUT_net[5]\, 
        \ADC_CHNR_net[4]\ => \ADC_CHNR_net[4]\, 
        \EV_RAM_ADDR_B_net[8]\ => \EV_RAM_ADDR_B_net[8]\, 
        \rdgwgtpkbgc[3]\ => \rdgwgtpkbgc_0[3]\, EV_DONE_net_0 => 
        EV_DONE_net_0, ACM_CLK_c => ACM_CLK_c, sys_reset_gl => 
        sys_reset_gl, EV_RAM_DO_B_net_net_7 => 
        EV_RAM_DO_B_net_net_7, bskrjhjvqpf_1_sqmuxa => 
        bskrjhjvqpf_1_sqmuxa, EV_RAM_DO_B_net_net_6 => 
        EV_RAM_DO_B_net_net_6, EV_RAM_DO_B_net_net_5 => 
        EV_RAM_DO_B_net_net_5, EV_RAM_ADDR_B_net_net_6 => 
        EV_RAM_ADDR_B_net_net_6, EV_RAM_ADDR_B_net_net_3 => 
        EV_RAM_ADDR_B_net_net_3, EV_RAM_ADDR_B_net_net_5 => 
        EV_RAM_ADDR_B_net_net_5, EV_RAM_ADDR_B_net_net_1 => 
        EV_RAM_ADDR_B_net_net_1, \Z\\EV_RAM_DO_B_net\\\ => 
        \Z\\EV_RAM_DO_B_net\\\, EV_RAM_DO_B_net_net_1 => 
        EV_RAM_DO_B_net_net_1, \Z\\EV_CHHOLD_net\\\ => 
        \Z\\EV_CHHOLD_net\\\, EV_CHHOLD_net_net_1 => 
        EV_CHHOLD_net_net_1, EV_CHHOLD_net_net_2 => 
        EV_CHHOLD_net_net_2, EV_CHHOLD_net_net_3 => 
        EV_CHHOLD_net_net_3, EV_RAM_DO_B_net_net_2 => 
        EV_RAM_DO_B_net_net_2, EV_ASSCADDR_net_net_5 => 
        EV_ASSCADDR_net_net_5, EV_RAM_CSN_B_net => 
        EV_RAM_CSN_B_net, INIT_DONE_c => INIT_DONE_c, 
        EV_RAM_DI_B_net_net_5 => EV_RAM_DI_B_net_net_5, 
        EV_RAM_DI_B_net_net_3 => EV_RAM_DI_B_net_net_3, 
        EV_RAM_DI_B_net_net_4 => EV_RAM_DI_B_net_net_4, 
        EV_RAM_DO_B_net_net_4 => EV_RAM_DO_B_net_net_4, 
        EV_RAM_DO_B_net_net_3 => EV_RAM_DO_B_net_net_3, 
        ASSC_RAM_DO_A_net_net_6 => ASSC_RAM_DO_A_net_net_6, 
        ASSC_RAM_DO_A_net_net_5 => ASSC_RAM_DO_A_net_net_5, 
        ASSC_RAM_DO_A_net_net_4 => ASSC_RAM_DO_A_net_net_4, 
        ASSC_RAM_DO_A_net_net_3 => ASSC_RAM_DO_A_net_net_3, 
        ASSC_RAM_DO_A_net_net_2 => ASSC_RAM_DO_A_net_net_2, 
        ASSC_RAM_DO_A_net_net_1 => ASSC_RAM_DO_A_net_net_1, 
        \Z\\ASSC_RAM_DO_A_net\\\ => \Z\\ASSC_RAM_DO_A_net\\\, 
        ASSC_RAM_DO_A_net_net_7 => ASSC_RAM_DO_A_net_net_7, 
        EV_RAM_ADDR_A_net_net_5 => EV_RAM_ADDR_A_net_net_5, 
        TR_EV_ADDR_net_net_5 => TR_EV_ADDR_net_net_5, 
        \Z\\EV_RAM_ADDR_A_net\\\ => \Z\\EV_RAM_ADDR_A_net\\\, 
        \Z\\TR_EV_ADDR_net\\\ => \Z\\TR_EV_ADDR_net\\\, 
        EV_RAM_ADDR_A_net_net_1 => EV_RAM_ADDR_A_net_net_1, 
        TR_EV_ADDR_net_net_1 => TR_EV_ADDR_net_net_1, 
        EV_RAM_ADDR_A_net_net_2 => EV_RAM_ADDR_A_net_net_2, 
        TR_EV_ADDR_net_net_2 => TR_EV_ADDR_net_net_2, 
        EV_RAM_ADDR_A_net_net_3 => EV_RAM_ADDR_A_net_net_3, 
        TR_EV_ADDR_net_net_3 => TR_EV_ADDR_net_net_3, 
        EV_RAM_ADDR_A_net_net_4 => EV_RAM_ADDR_A_net_net_4, 
        TR_EV_ADDR_net_net_4 => TR_EV_ADDR_net_net_4, 
        EV_RAM_ADDR_A_net_net_6 => EV_RAM_ADDR_A_net_net_6, 
        TR_EV_ADDR_net_net_6 => TR_EV_ADDR_net_net_6, 
        EV_RAM_ADDR_A_net_net_7 => EV_RAM_ADDR_A_net_net_7, 
        TR_EV_ADDR_net_net_7 => TR_EV_ADDR_net_net_7, rhbzpsxhwms
         => rhbzpsxhwms, jngrzvhtjvn_0 => jngrzvhtjvn_0, 
        ASSC_SAMPFLAG_net => ASSC_SAMPFLAG_net, ASSC_DONE_c => 
        ASSC_DONE_c, EV_RAM_RW_A_net => EV_RAM_RW_A_net, 
        INIT_EV_WEN => INIT_EV_WEN, EV_RAM_CSN_A_net => 
        EV_RAM_CSN_A_net, TR_EV_RD_net => TR_EV_RD_net, 
        EV_RAM_DI_B_net_net_1 => EV_RAM_DI_B_net_net_1, 
        \Z\\EV_RAM_DI_B_net\\\ => \Z\\EV_RAM_DI_B_net\\\, 
        EV_RAM_DI_B_net_net_2 => EV_RAM_DI_B_net_net_2, 
        EV_RAM_DI_B_net_net_6 => EV_RAM_DI_B_net_net_6, 
        EV_RAM_DI_B_net_net_7 => EV_RAM_DI_B_net_net_7, 
        EV_RAM_WR_BUSY_B_net => EV_RAM_WR_BUSY_B_net, EV_DONE_net
         => EV_DONE_net, EV_EVFLAG_net => EV_EVFLAG_net, 
        EV_ASSCADDR_net_net_4 => EV_ASSCADDR_net_net_4, 
        ASSC_SEQOUT_net_net_4 => ASSC_SEQOUT_net_net_4, 
        EV_ASSCADDR_net_net_3 => EV_ASSCADDR_net_net_3, 
        ASSC_SEQOUT_net_net_3 => ASSC_SEQOUT_net_net_3, 
        EV_ASSCADDR_net_net_2 => EV_ASSCADDR_net_net_2, 
        ASSC_SEQOUT_net_net_2 => ASSC_SEQOUT_net_net_2, 
        EV_ASSCADDR_net_net_1 => EV_ASSCADDR_net_net_1, 
        ASSC_SEQOUT_net_net_1 => ASSC_SEQOUT_net_net_1, 
        \Z\\EV_ASSCADDR_net\\\ => \Z\\EV_ASSCADDR_net\\\, 
        \Z\\ASSC_SEQOUT_net\\\ => \Z\\ASSC_SEQOUT_net\\\, 
        ADC_CHNR_net_net_3 => ADC_CHNR_net_net_3, 
        ADC_CHNR_net_net_2 => ADC_CHNR_net_net_2, 
        ADC_CHNR_net_net_1 => ADC_CHNR_net_net_1, 
        \Z\\ADC_CHNR_net\\\ => \Z\\ADC_CHNR_net\\\, 
        EV_RAM_ADDR_B_net_net_7 => EV_RAM_ADDR_B_net_net_7, 
        EV_RAM_ADDR_B_net_net_4 => EV_RAM_ADDR_B_net_net_4, 
        EV_RAM_ADDR_B_net_net_2 => EV_RAM_ADDR_B_net_net_2, 
        \Z\\EV_RAM_ADDR_B_net\\\ => \Z\\EV_RAM_ADDR_B_net\\\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity Analog_RTC_smev_ram is

    port(INIT_DATA : in std_logic_vector(8 downto 0); 
        \EV_RAM_DI_B_net[8]\, \EV_RAM_ADDR_B_net[8]\, 
        \EV_RAM_ADDR_A_net[8]\ : in std_logic; 
        \EV_RAM_DO_B_net[8]\ : out std_logic; sys_reset_gl, 
        \Z\\EV_RAM_DI_B_net\\\, EV_RAM_DI_B_net_net_1, 
        EV_RAM_DI_B_net_net_2, EV_RAM_DI_B_net_net_3, 
        EV_RAM_DI_B_net_net_4, EV_RAM_DI_B_net_net_5, 
        EV_RAM_DI_B_net_net_6, EV_RAM_DI_B_net_net_7, ACM_CLK_c, 
        EV_RAM_WR_BUSY_B_net, EV_RAM_CSN_B_net, EV_RAM_RW_A_net, 
        EV_RAM_CSN_A_net, \Z\\EV_RAM_ADDR_B_net\\\, 
        EV_RAM_ADDR_B_net_net_1, EV_RAM_ADDR_B_net_net_2, 
        EV_RAM_ADDR_B_net_net_3, EV_RAM_ADDR_B_net_net_4, 
        EV_RAM_ADDR_B_net_net_5, EV_RAM_ADDR_B_net_net_6, 
        EV_RAM_ADDR_B_net_net_7, \Z\\EV_RAM_ADDR_A_net\\\, 
        EV_RAM_ADDR_A_net_net_1, EV_RAM_ADDR_A_net_net_2, 
        EV_RAM_ADDR_A_net_net_3, EV_RAM_ADDR_A_net_net_4, 
        EV_RAM_ADDR_A_net_net_5, EV_RAM_ADDR_A_net_net_6, 
        EV_RAM_ADDR_A_net_net_7 : in std_logic; 
        \Z\\EV_RAM_DO_B_net\\\, EV_RAM_DO_B_net_net_1, 
        EV_RAM_DO_B_net_net_2, EV_RAM_DO_B_net_net_3, 
        EV_RAM_DO_B_net_net_4, EV_RAM_DO_B_net_net_5, 
        EV_RAM_DO_B_net_net_6, EV_RAM_DO_B_net_net_7, 
        EV_RAM_DO_A_net_net_7 : out std_logic);

end Analog_RTC_smev_ram;

architecture DEF_ARCH of Analog_RTC_smev_ram is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component RAM4K9
    generic (MEMORYFILE:string := "");

    port(ADDRA11, ADDRA10, ADDRA9, ADDRA8, ADDRA7, ADDRA6, ADDRA5, 
        ADDRA4, ADDRA3, ADDRA2, ADDRA1, ADDRA0, ADDRB11, ADDRB10, 
        ADDRB9, ADDRB8, ADDRB7, ADDRB6, ADDRB5, ADDRB4, ADDRB3, 
        ADDRB2, ADDRB1, ADDRB0, DINA8, DINA7, DINA6, DINA5, DINA4, 
        DINA3, DINA2, DINA1, DINA0, DINB8, DINB7, DINB6, DINB5, 
        DINB4, DINB3, DINB2, DINB1, DINB0, WIDTHA0, WIDTHA1, 
        WIDTHB0, WIDTHB1, PIPEA, PIPEB, WMODEA, WMODEB, BLKA, 
        BLKB, WENA, WENB, CLKA, CLKB, RESET : in std_logic := 'U'; 
        DOUTA8, DOUTA7, DOUTA6, DOUTA5, DOUTA4, DOUTA3, DOUTA2, 
        DOUTA1, DOUTA0, DOUTB8, DOUTB7, DOUTB6, DOUTB5, DOUTB4, 
        DOUTB3, DOUTB2, DOUTB1, DOUTB0 : out std_logic);
  end component;

    signal EV_RAM_DO_A_net_net_6, EV_RAM_DO_A_net_net_5, 
        EV_RAM_DO_A_net_net_4, EV_RAM_DO_A_net_net_3, 
        EV_RAM_DO_A_net_net_2, EV_RAM_DO_A_net_net_1, 
        \Z\\EV_RAM_DO_A_net\\\, \EV_RAM_DO_A_net[8]\, \GND\, 
        \VCC\, GND_net_1, VCC_net_1 : std_logic;

begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    Analog_RTC_smev_ram_R0C0 : RAM4K9
      port map(ADDRA11 => \GND\, ADDRA10 => \GND\, ADDRA9 => 
        \GND\, ADDRA8 => \EV_RAM_ADDR_A_net[8]\, ADDRA7 => 
        \Z\\EV_RAM_ADDR_A_net\\\, ADDRA6 => 
        EV_RAM_ADDR_A_net_net_1, ADDRA5 => 
        EV_RAM_ADDR_A_net_net_2, ADDRA4 => 
        EV_RAM_ADDR_A_net_net_3, ADDRA3 => 
        EV_RAM_ADDR_A_net_net_4, ADDRA2 => 
        EV_RAM_ADDR_A_net_net_5, ADDRA1 => 
        EV_RAM_ADDR_A_net_net_6, ADDRA0 => 
        EV_RAM_ADDR_A_net_net_7, ADDRB11 => \GND\, ADDRB10 => 
        \GND\, ADDRB9 => \GND\, ADDRB8 => \EV_RAM_ADDR_B_net[8]\, 
        ADDRB7 => \Z\\EV_RAM_ADDR_B_net\\\, ADDRB6 => 
        EV_RAM_ADDR_B_net_net_1, ADDRB5 => 
        EV_RAM_ADDR_B_net_net_2, ADDRB4 => 
        EV_RAM_ADDR_B_net_net_3, ADDRB3 => 
        EV_RAM_ADDR_B_net_net_4, ADDRB2 => 
        EV_RAM_ADDR_B_net_net_5, ADDRB1 => 
        EV_RAM_ADDR_B_net_net_6, ADDRB0 => 
        EV_RAM_ADDR_B_net_net_7, DINA8 => INIT_DATA(8), DINA7 => 
        INIT_DATA(7), DINA6 => INIT_DATA(6), DINA5 => 
        INIT_DATA(5), DINA4 => INIT_DATA(4), DINA3 => 
        INIT_DATA(3), DINA2 => INIT_DATA(2), DINA1 => 
        INIT_DATA(1), DINA0 => INIT_DATA(0), DINB8 => 
        \EV_RAM_DI_B_net[8]\, DINB7 => \Z\\EV_RAM_DI_B_net\\\, 
        DINB6 => EV_RAM_DI_B_net_net_1, DINB5 => 
        EV_RAM_DI_B_net_net_2, DINB4 => EV_RAM_DI_B_net_net_3, 
        DINB3 => EV_RAM_DI_B_net_net_4, DINB2 => 
        EV_RAM_DI_B_net_net_5, DINB1 => EV_RAM_DI_B_net_net_6, 
        DINB0 => EV_RAM_DI_B_net_net_7, WIDTHA0 => \VCC\, WIDTHA1
         => \VCC\, WIDTHB0 => \VCC\, WIDTHB1 => \VCC\, PIPEA => 
        \GND\, PIPEB => \GND\, WMODEA => \VCC\, WMODEB => \VCC\, 
        BLKA => EV_RAM_CSN_A_net, BLKB => EV_RAM_CSN_B_net, WENA
         => EV_RAM_RW_A_net, WENB => EV_RAM_WR_BUSY_B_net, CLKA
         => ACM_CLK_c, CLKB => ACM_CLK_c, RESET => sys_reset_gl, 
        DOUTA8 => \EV_RAM_DO_A_net[8]\, DOUTA7 => 
        \Z\\EV_RAM_DO_A_net\\\, DOUTA6 => EV_RAM_DO_A_net_net_1, 
        DOUTA5 => EV_RAM_DO_A_net_net_2, DOUTA4 => 
        EV_RAM_DO_A_net_net_3, DOUTA3 => EV_RAM_DO_A_net_net_4, 
        DOUTA2 => EV_RAM_DO_A_net_net_5, DOUTA1 => 
        EV_RAM_DO_A_net_net_6, DOUTA0 => EV_RAM_DO_A_net_net_7, 
        DOUTB8 => \EV_RAM_DO_B_net[8]\, DOUTB7 => 
        \Z\\EV_RAM_DO_B_net\\\, DOUTB6 => EV_RAM_DO_B_net_net_1, 
        DOUTB5 => EV_RAM_DO_B_net_net_2, DOUTB4 => 
        EV_RAM_DO_B_net_net_3, DOUTB3 => EV_RAM_DO_B_net_net_4, 
        DOUTB2 => EV_RAM_DO_B_net_net_5, DOUTB1 => 
        EV_RAM_DO_B_net_net_6, DOUTB0 => EV_RAM_DO_B_net_net_7);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity ccfdxrhchsz is

    port(\TR_RAM_ADDR_B_net[8]\, \TR_EV_ADDR_net[8]\, 
        \TR_RAM_DI_B_net[8]\ : out std_logic; \EV_CHHOLD_net[4]\, 
        \TR_RAM_DO_B_net[8]\ : in std_logic; 
        \Z\\TR_RAM_ADDR_B_net\\\, TR_RAM_ADDR_B_net_net_2, 
        TR_RAM_ADDR_B_net_net_4, TR_RAM_ADDR_B_net_net_7, 
        \Z\\TR_EV_ADDR_net\\\, TR_EV_ADDR_net_net_1, 
        TR_EV_ADDR_net_net_2, TR_EV_ADDR_net_net_3, 
        TR_EV_ADDR_net_net_4, TR_EV_ADDR_net_net_5, 
        TR_EV_ADDR_net_net_6, TR_EV_ADDR_net_net_7, TR_TRFLAG_net, 
        TR_DONE_net : out std_logic; N_14, N_12 : in std_logic; 
        cnwzzrhdsqv_i_a2, \Z\\TR_RAM_DI_B_net\\\, 
        TR_RAM_DI_B_net_net_1, TR_RAM_DI_B_net_net_7, 
        TR_RAM_DI_B_net_net_6, TR_RAM_DI_B_net_net_5, 
        TR_RAM_DI_B_net_net_4, TR_RAM_DI_B_net_net_3, 
        TR_RAM_RW_B_net, TR_RAM_DI_B_net_net_2 : out std_logic; 
        EV_CHHOLD_net_net_2, EV_CHHOLD_net_net_3, 
        \Z\\EV_CHHOLD_net\\\, EV_DONE_net, EV_CHHOLD_net_net_1, 
        EV_RAM_DO_A_net_net_7, TR_RAM_DO_B_net_net_7, 
        EV_DONE_net_0 : in std_logic; TR_RAM_CSN_B_net : out
         std_logic; TR_RAM_DO_B_net_net_3, TR_RAM_DO_B_net_net_5, 
        TR_RAM_DO_B_net_net_4, \Z\\TR_RAM_DO_B_net\\\, 
        TR_RAM_DO_B_net_net_1, TR_RAM_DO_B_net_net_2, INIT_DONE_c, 
        TR_RAM_DO_B_net_net_6 : in std_logic; TR_EV_RD_net, 
        TR_RAM_ADDR_B_net_net_1, TR_RAM_ADDR_B_net_net_5, 
        TR_RAM_ADDR_B_net_net_3, TR_RAM_ADDR_B_net_net_6 : out
         std_logic; sys_reset_gl, ACM_CLK_c : in std_logic);

end ccfdxrhchsz;

architecture DEF_ARCH of ccfdxrhchsz is 

  component NOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AX1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1C0
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component OAI1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AND2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1D
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1E1C0
    port(D, CLK, CLR, E : in std_logic := 'U'; Q : out std_logic);
        
  end component;

  component NOR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AOI1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component OA1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component DFN1P0
    port(D, CLK, PRE : in std_logic := 'U'; Q : out std_logic);
  end component;

  component OR3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2B
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AO1A
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component OA1C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR3C
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

    signal \TR_RAM_ADDR_B_net[8]_net_1\, 
        \Z\\TR_RAM_ADDR_B_net\\_net_1\, \TR_RAM_ADDR_B_net_net_2\, 
        \TR_RAM_ADDR_B_net_net_4\, \TR_RAM_ADDR_B_net_net_7\, 
        \TR_EV_RD_net\, \TR_RAM_ADDR_B_net_net_1\, 
        \TR_RAM_ADDR_B_net_net_5\, \TR_RAM_ADDR_B_net_net_3\, 
        \TR_RAM_ADDR_B_net_net_6\, \frcskdnwvmx_0[8]_net_1\, 
        \frcskdnwvmx_ns_i_0[3]_net_1\, 
        \DWACT_ADD_CI_0_g_array_2[0]\, 
        \DWACT_ADD_CI_0_g_array_1[0]\, 
        \DWACT_ADD_CI_0_pog_array_1[0]\, 
        \DWACT_ADD_CI_0_g_array_3[0]\, 
        \DWACT_ADD_CI_0_pog_array_2[0]\, 
        \DWACT_ADD_CI_0_g_array_11[0]\, 
        \DWACT_ADD_CI_0_pog_array_1_1[0]\, 
        \DWACT_ADD_CI_0_TMP[0]\, \DWACT_ADD_CI_0_g_array_12_1[0]\, 
        \DWACT_ADD_CI_0_g_array_12[0]\, 
        \DWACT_ADD_CI_0_g_array_12_2[0]\, \dttrrgvsnmg_iv_0_3\, 
        N_585_i_i_0, \jnfrmfqggmk\, N_788, \dttrrgvsnmg_iv_0_1\, 
        N_378_i, \dttrrgvsnmg_iv_0_a2_1\, \dttrrgvsnmg_iv_0_0\, 
        N_766, \dttrrgvsnmg_iv_0_a2_0_1\, \qxzrwpjhcww_m[0]\, 
        \dttrrgvsnmg_iv_0_a2_3_0\, N_928, \svpbmqwvncp_0_0_1\, 
        \frcskdnwvmx[11]_net_1\, \hnbsspwxkpd\, N_776, 
        \frcskdnwvmx_ns_1[6]\, \znjtscvsvcs_iv_0_a2_0\, 
        \znjtscvsvcs_iv_0_a2_1\, \jvxfmjbzjmc_0_0_4\, N_406, 
        \jvxfmjbzjmc_0_0_a2_0\, \jvxfmjbzjmc_0_0_2\, 
        \jvxfmjbzjmc_0_0_a2_3_0\, N_385_i, \jvxfmjbzjmc_0_0_1\, 
        N_432, N_401, \frcskdnwvmx[10]_net_1\, 
        \frcskdnwvmx_ns_i_0_0[5]_net_1\, N_460, 
        \mwzbqqvzwjt_12_iv_0_o3_0_1[1]_net_1\, 
        \dttrrgvsnmg_iv_0_a2_0_0\, \znjtscvsvcs_iv_0_a2_0_0\, 
        \frcskdnwvmx_ns_i_0_0[3]_net_1\, N_398, 
        \frcskdnwvmx[9]_net_1\, \frcskdnwvmx_ns_i_0_0[4]_net_1\, 
        un1_xjrbtnwdszs_0_sqmuxa_0_a2_0, 
        \frcskdnwvmx_ns_a3_0_a2_1[7]_net_1\, 
        \frcskdnwvmx_ns_i_0_a2_1_0[5]_net_1\, 
        \frcskdnwvmx[6]_net_1\, \frcskdnwvmx_ns_0_a2_3_0[0]\, 
        \frcskdnwvmx_ns_0_a2_1[0]_net_1\, \frcskdnwvmx[4]_net_1\, 
        \frcskdnwvmx[5]_net_1\, \frcskdnwvmx[7]_net_1\, 
        \frcskdnwvmx[2]_net_1\, un1_xjrbtnwdszs_0_sqmuxa, N_785, 
        N_784, N_783, \frcskdnwvmx_ns[10]\, N_381_i, 
        \frcskdnwvmx_ns[7]\, N_764, N_647, N_391_i, 
        \frcskdnwvmx_ns_i_0[5]_net_1\, N_461, N_459, jvxfmjbzjmc, 
        N_433, \svpbmqwvncp_0_0_a2\, znjtscvsvcs, N_646, 
        \frcskdnwvmx_ns_i_0[4]_net_1\, N_405, 
        \qgnzcpmbksd_7_0_iv_i[8]\, N_397, N_452, N_453, 
        \qgnzcpmbksd_7_0_iv_i[7]\, N_450, N_451, 
        \qgnzcpmbksd_7[0]\, N_438, N_436, N_437, 
        \frcskdnwvmx_ns[6]\, \frcskdnwvmx_ns_1[7]\, 
        \qgnzcpmbksd_7_0_iv_i[6]\, N_448, N_449, 
        \qgnzcpmbksd_7[4]\, N_442, N_444, N_443, 
        \qgnzcpmbksd_7[3]\, N_827, N_829, N_828, 
        \qgnzcpmbksd_7[2]\, N_439, N_441, N_440, 
        \qgnzcpmbksd_7[5]\, N_445, N_447, N_446, 
        un1_xjrbtnwdszs_1_sqmuxa, N_787, N_786, 
        \qgnzcpmbksd_7[1]\, N_825, N_826, N_824, N_822_i, 
        dttrrgvsnmg, \frcskdnwvmx_ns_0_a2_3[0]_net_1\, 
        \frcskdnwvmx[1]_net_1\, \DWACT_ADD_CI_0_partial_sum[0]\, 
        N_446_2, I_34_0, I_36_0, \frcskdnwvmx[8]_net_1\, 
        \frcskdnwvmx_ns[2]\, \frcskdnwvmx_ns[11]\, N_769, 
        \frcskdnwvmx_ns[0]\, \frcskdnwvmx_ns_1[0]\, 
        un1_frcskdnwvmx_3, \frcskdnwvmx_ns[1]\, I_38_0, 
        \frcskdnwvmx_ns[9]\, I_30_0, I_37_0, I_35_0, I_33_0, 
        I_32_0, \dvqzjxhqhrh\, \frcskdnwvmx[0]_net_1\, N_390, 
        \frcskdnwvmx_ns[8]\, cpwfrthtnfr_7_sqmuxa, 
        \DWACT_ADD_CI_0_pog_array_1_2[0]\, \VCC\, \GND\, 
        GND_net_1, VCC_net_1 : std_logic;

begin 

    \TR_RAM_ADDR_B_net[8]\ <= \TR_RAM_ADDR_B_net[8]_net_1\;
    \Z\\TR_RAM_ADDR_B_net\\\ <= \Z\\TR_RAM_ADDR_B_net\\_net_1\;
    TR_RAM_ADDR_B_net_net_2 <= \TR_RAM_ADDR_B_net_net_2\;
    TR_RAM_ADDR_B_net_net_4 <= \TR_RAM_ADDR_B_net_net_4\;
    TR_RAM_ADDR_B_net_net_7 <= \TR_RAM_ADDR_B_net_net_7\;
    TR_EV_RD_net <= \TR_EV_RD_net\;
    TR_RAM_ADDR_B_net_net_1 <= \TR_RAM_ADDR_B_net_net_1\;
    TR_RAM_ADDR_B_net_net_5 <= \TR_RAM_ADDR_B_net_net_5\;
    TR_RAM_ADDR_B_net_net_3 <= \TR_RAM_ADDR_B_net_net_3\;
    TR_RAM_ADDR_B_net_net_6 <= \TR_RAM_ADDR_B_net_net_6\;

    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_i_a2_0[8]\ : NOR2
      port map(A => I_36_0, B => \frcskdnwvmx[8]_net_1\, Y => 
        N_453);
    
    \brxxctcvrmh_qgnzcpmbksd_7_iv_0_a2_0[0]\ : OR2B
      port map(A => \DWACT_ADD_CI_0_partial_sum[0]\, B => 
        \frcskdnwvmx_ns_0_a2_3[0]_net_1\, Y => N_437);
    
    znjtscvsvcs_iv_0_x2 : AX1B
      port map(A => N_928, B => 
        \mwzbqqvzwjt_12_iv_0_o3_0_1[1]_net_1\, C => 
        TR_RAM_DO_B_net_net_2, Y => N_585_i_i_0);
    
    \brxxctcvrmh_qgnzcpmbksd_7_iv_0_a2[0]\ : OR3B
      port map(A => \frcskdnwvmx_0[8]_net_1\, B => 
        TR_RAM_DO_B_net_net_7, C => EV_DONE_net_0, Y => N_436);
    
    ndngwnmwgbm_0_a3_0_a2 : NOR2
      port map(A => \frcskdnwvmx[1]_net_1\, B => 
        \frcskdnwvmx[0]_net_1\, Y => TR_RAM_RW_B_net);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2_1[2]\ : OR3B
      port map(A => \frcskdnwvmx[8]_net_1\, B => 
        TR_RAM_DO_B_net_net_5, C => EV_DONE_net, Y => N_441);
    
    \qgnzcpmbksd[1]\ : DFN1C0
      port map(D => \qgnzcpmbksd_7[1]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \TR_RAM_ADDR_B_net_net_6\);
    
    znjtscvsvcs_iv_0_a2_0_2 : OAI1
      port map(A => \znjtscvsvcs_iv_0_a2_1\, B => 
        TR_RAM_DO_B_net_net_6, C => \jnfrmfqggmk\, Y => 
        \znjtscvsvcs_iv_0_a2_0\);
    
    qgtkpxrgqqs_un1_xjrbtnwdszs_0_sqmuxa_0_a2_0_0 : NOR2A
      port map(A => N_928, B => TR_RAM_DO_B_net_net_2, Y => 
        un1_xjrbtnwdszs_0_sqmuxa_0_a2_0);
    
    qgtkpxrgqqs_un1_xjrbtnwdszs_0_sqmuxa_0_a2_0 : OR2A
      port map(A => \frcskdnwvmx[2]_net_1\, B => \dvqzjxhqhrh\, Y
         => N_784);
    
    un1_qgnzcpmbksd_I_51 : AND2
      port map(A => \DWACT_ADD_CI_0_pog_array_1_1[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_1_2[0]\, Y => 
        \DWACT_ADD_CI_0_pog_array_2[0]\);
    
    dttrrgvsnmg_iv_0_a2_0_0 : NOR2A
      port map(A => TR_RAM_DO_B_net_net_2, B => 
        TR_RAM_DO_B_net_net_1, Y => \dttrrgvsnmg_iv_0_a2_0_0\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2_0[1]\ : OR2B
      port map(A => I_32_0, B => N_446_2, Y => N_825);
    
    \cnwzzrhdsqv_i_a2\ : OR3B
      port map(A => N_12, B => N_14, C => INIT_DONE_c, Y => 
        cnwzzrhdsqv_i_a2);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_o3_0_o2[5]\ : AO1D
      port map(A => N_378_i, B => N_381_i, C => 
        \frcskdnwvmx[1]_net_1\, Y => N_769);
    
    dttrrgvsnmg_iv_0_a2_3 : OR3A
      port map(A => N_766, B => TR_RAM_DO_B_net_net_2, C => 
        \dttrrgvsnmg_iv_0_a2_3_0\, Y => N_822_i);
    
    \frcskdnwvmx_ns_i_0_a2_1_0[5]\ : OR2A
      port map(A => \Z\\TR_RAM_DO_B_net\\\, B => N_401, Y => 
        \frcskdnwvmx_ns_i_0_a2_1_0[5]_net_1\);
    
    N_231_i_0_i2_0_a2 : OR2B
      port map(A => TR_RAM_DO_B_net_net_2, B => 
        \TR_RAM_DO_B_net[8]\, Y => N_381_i);
    
    \rjhhzsggrtr[5]\ : DFN1E1C0
      port map(D => TR_RAM_DO_B_net_net_2, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \frcskdnwvmx[4]_net_1\, Q => 
        TR_EV_ADDR_net_net_2);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_i[7]\ : NOR3
      port map(A => N_397, B => N_450, C => N_451, Y => 
        \qgnzcpmbksd_7_0_iv_i[7]\);
    
    \qgnzcpmbksd[8]\ : DFN1C0
      port map(D => \qgnzcpmbksd_7_0_iv_i[8]\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \TR_RAM_ADDR_B_net[8]_net_1\);
    
    un1_qgnzcpmbksd_I_33 : XOR2
      port map(A => \TR_RAM_ADDR_B_net_net_2\, B => 
        \DWACT_ADD_CI_0_g_array_12_1[0]\, Y => I_33_0);
    
    \frcskdnwvmx_ns_a3_0_a2[11]\ : NOR2B
      port map(A => INIT_DONE_c, B => \frcskdnwvmx[1]_net_1\, Y
         => \frcskdnwvmx_ns[11]\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_i[8]\ : NOR3
      port map(A => N_397, B => N_452, C => N_453, Y => 
        \qgnzcpmbksd_7_0_iv_i[8]\);
    
    \frcskdnwvmx_ns_i_0_a2[5]\ : NOR3
      port map(A => \Z\\TR_RAM_DO_B_net\\\, B => N_401, C => 
        N_391_i, Y => N_459);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_i_a2_0[6]\ : NOR2
      port map(A => I_38_0, B => \frcskdnwvmx[8]_net_1\, Y => 
        N_449);
    
    un1_qgnzcpmbksd_I_32 : XOR2
      port map(A => \TR_RAM_ADDR_B_net_net_6\, B => 
        \DWACT_ADD_CI_0_TMP[0]\, Y => I_32_0);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2_0[2]\ : OR2B
      port map(A => I_35_0, B => N_446_2, Y => N_440);
    
    un1_qgnzcpmbksd_I_37 : XOR2
      port map(A => \TR_RAM_ADDR_B_net_net_4\, B => 
        \DWACT_ADD_CI_0_g_array_12[0]\, Y => I_37_0);
    
    \frcskdnwvmx[7]\ : DFN1C0
      port map(D => \frcskdnwvmx_ns_i_0[4]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \frcskdnwvmx[7]_net_1\);
    
    \frcskdnwvmx_ns_0_a2_1[0]\ : NOR3
      port map(A => \frcskdnwvmx[9]_net_1\, B => 
        \frcskdnwvmx[10]_net_1\, C => \TR_EV_RD_net\, Y => 
        \frcskdnwvmx_ns_0_a2_1[0]_net_1\);
    
    \rjhhzsggrtr[3]\ : DFN1E1C0
      port map(D => TR_RAM_DO_B_net_net_4, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \frcskdnwvmx[4]_net_1\, Q => 
        TR_EV_ADDR_net_net_4);
    
    \frcskdnwvmx[5]\ : DFN1C0
      port map(D => \frcskdnwvmx_ns[6]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \frcskdnwvmx[5]_net_1\);
    
    jvxfmjbzjmc_0_0_o3_0 : OR2
      port map(A => \TR_RAM_DO_B_net[8]\, B => \hnbsspwxkpd\, Y
         => N_406);
    
    \frcskdnwvmx[8]\ : DFN1C0
      port map(D => \frcskdnwvmx_ns_i_0[3]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \frcskdnwvmx[8]_net_1\);
    
    \frcskdnwvmx[1]\ : DFN1C0
      port map(D => \frcskdnwvmx_ns[10]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \frcskdnwvmx[1]_net_1\);
    
    \frcskdnwvmx_ns_a3_0_a2[7]\ : NOR3B
      port map(A => INIT_DONE_c, B => 
        \frcskdnwvmx_ns_a3_0_a2_1[7]_net_1\, C => N_764, Y => 
        \frcskdnwvmx_ns[7]\);
    
    dttrrgvsnmg_iv_0_0 : AOI1B
      port map(A => N_766, B => \dttrrgvsnmg_iv_0_a2_0_1\, C => 
        \qxzrwpjhcww_m[0]\, Y => \dttrrgvsnmg_iv_0_0\);
    
    GND_i : GND
      port map(Y => \GND\);
    
    \frcskdnwvmx_ns_i_0[3]\ : OA1B
      port map(A => N_405, B => \frcskdnwvmx[9]_net_1\, C => 
        \frcskdnwvmx_ns_i_0_0[3]_net_1\, Y => 
        \frcskdnwvmx_ns_i_0[3]_net_1\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2_0_1[2]\ : OA1
      port map(A => N_378_i, B => N_381_i, C => 
        \frcskdnwvmx_ns_0_a2_3[0]_net_1\, Y => N_446_2);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0[2]\ : OR3C
      port map(A => N_439, B => N_441, C => N_440, Y => 
        \qgnzcpmbksd_7[2]\);
    
    \jfkkdmfhcdn_0_a2[3]\ : NOR2A
      port map(A => TR_RAM_DO_B_net_net_4, B => 
        \frcskdnwvmx[0]_net_1\, Y => TR_RAM_DI_B_net_net_4);
    
    \frcskdnwvmx_ns_a3_0_a2[10]\ : NOR3A
      port map(A => INIT_DONE_c, B => N_381_i, C => N_378_i, Y
         => \frcskdnwvmx_ns[10]\);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    jvxfmjbzjmc_0_0 : OR3C
      port map(A => \jvxfmjbzjmc_0_0_1\, B => N_433, C => 
        \jvxfmjbzjmc_0_0_4\, Y => jvxfmjbzjmc);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0[5]\ : OR3C
      port map(A => N_445, B => N_447, C => N_446, Y => 
        \qgnzcpmbksd_7[5]\);
    
    un1_qgnzcpmbksd_I_36 : XOR2
      port map(A => \TR_RAM_ADDR_B_net[8]_net_1\, B => 
        \DWACT_ADD_CI_0_g_array_3[0]\, Y => I_36_0);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0[3]\ : OR3C
      port map(A => N_827, B => N_829, C => N_828, Y => 
        \qgnzcpmbksd_7[3]\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_i_a2[8]\ : NOR2A
      port map(A => \frcskdnwvmx_0[8]_net_1\, B => 
        \TR_RAM_DO_B_net[8]\, Y => N_452);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2_1[3]\ : OR3B
      port map(A => \frcskdnwvmx[8]_net_1\, B => 
        TR_RAM_DO_B_net_net_4, C => EV_DONE_net_0, Y => N_829);
    
    \frcskdnwvmx_ns_i_0_o3[3]\ : NOR2
      port map(A => \TR_RAM_DO_B_net[8]\, B => N_385_i, Y => 
        N_398);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2[1]\ : OAI1
      port map(A => N_769, B => EV_DONE_net, C => 
        EV_CHHOLD_net_net_3, Y => N_824);
    
    \frcskdnwvmx[11]\ : DFN1P0
      port map(D => \frcskdnwvmx_ns[0]\, CLK => ACM_CLK_c, PRE
         => sys_reset_gl, Q => \frcskdnwvmx[11]_net_1\);
    
    jvxfmjbzjmc_0_0_4 : AOI1B
      port map(A => N_406, B => \jvxfmjbzjmc_0_0_a2_0\, C => 
        \jvxfmjbzjmc_0_0_2\, Y => \jvxfmjbzjmc_0_0_4\);
    
    \frcskdnwvmx_ns_i_0[5]\ : NOR3
      port map(A => N_461, B => \frcskdnwvmx_ns_i_0_0[5]_net_1\, 
        C => N_459, Y => \frcskdnwvmx_ns_i_0[5]_net_1\);
    
    vnjtgtptszx_un1_xjrbtnwdszs_1_sqmuxa_0 : OR3
      port map(A => N_788, B => N_787, C => N_786, Y => 
        un1_xjrbtnwdszs_1_sqmuxa);
    
    \frcskdnwvmx[6]\ : DFN1C0
      port map(D => \frcskdnwvmx_ns_i_0[5]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \frcskdnwvmx[6]_net_1\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0[4]\ : OR3C
      port map(A => N_442, B => N_444, C => N_443, Y => 
        \qgnzcpmbksd_7[4]\);
    
    znjtscvsvcs_iv_0_a2_1 : NOR2A
      port map(A => \hnbsspwxkpd\, B => TR_RAM_DO_B_net_net_7, Y
         => \znjtscvsvcs_iv_0_a2_1\);
    
    un1_qgnzcpmbksd_I_30 : XOR2
      port map(A => \TR_RAM_ADDR_B_net_net_3\, B => 
        \DWACT_ADD_CI_0_g_array_2[0]\, Y => I_30_0);
    
    qgtkpxrgqqs_un1_xjrbtnwdszs_0_sqmuxa_0_o3_0 : OR2B
      port map(A => \TR_RAM_DO_B_net[8]\, B => 
        \frcskdnwvmx[6]_net_1\, Y => N_764);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    un1_qgnzcpmbksd_I_49 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_1[0]\, B => 
        \TR_RAM_ADDR_B_net_net_5\, Y => 
        \DWACT_ADD_CI_0_g_array_12[0]\);
    
    \frcskdnwvmx_ns_i_0_o3_0[3]\ : NOR2A
      port map(A => TR_RAM_DO_B_net_net_1, B => N_390, Y => N_405);
    
    svpbmqwvncp_0_0_a2 : OR2
      port map(A => \TR_RAM_DO_B_net[8]\, B => 
        \frcskdnwvmx_ns_1[7]\, Y => \svpbmqwvncp_0_0_a2\);
    
    \frcskdnwvmx_ns_i_0_0[3]\ : OAI1
      port map(A => N_398, B => \frcskdnwvmx[9]_net_1\, C => 
        INIT_DONE_c, Y => \frcskdnwvmx_ns_i_0_0[3]_net_1\);
    
    N_226_i_0_o3_i_a2_0_a2 : OR2A
      port map(A => \frcskdnwvmx[6]_net_1\, B => 
        \Z\\TR_RAM_DO_B_net\\\, Y => N_385_i);
    
    vnjtgtptszx_un1_xjrbtnwdszs_1_sqmuxa_0_o3_0 : NOR2A
      port map(A => \Z\\TR_RAM_DO_B_net\\\, B => N_764, Y => 
        N_766);
    
    vnjtgtptszx_un1_xjrbtnwdszs_1_sqmuxa_0_o3 : NOR2
      port map(A => TR_RAM_DO_B_net_net_7, B => 
        TR_RAM_DO_B_net_net_6, Y => N_776);
    
    un1_qgnzcpmbksd_I_38 : XOR2
      port map(A => \TR_RAM_ADDR_B_net_net_1\, B => 
        \DWACT_ADD_CI_0_g_array_11[0]\, Y => I_38_0);
    
    jvxfmjbzjmc_0_0_a2_0 : OR2A
      port map(A => \frcskdnwvmx[6]_net_1\, B => 
        TR_RAM_DO_B_net_net_1, Y => N_432);
    
    \rjhhzsggrtr[1]\ : DFN1E1C0
      port map(D => TR_RAM_DO_B_net_net_6, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \frcskdnwvmx[4]_net_1\, Q => 
        TR_EV_ADDR_net_net_6);
    
    \brxxctcvrmh_qgnzcpmbksd_7_iv_0[0]\ : OR3C
      port map(A => N_438, B => N_436, C => N_437, Y => 
        \qgnzcpmbksd_7[0]\);
    
    \frcskdnwvmx_ns_a3_1_0_a3_0_a2_0_a2[6]\ : OR2A
      port map(A => \TR_RAM_DO_B_net[8]\, B => 
        TR_RAM_DO_B_net_net_2, Y => \frcskdnwvmx_ns_1[6]\);
    
    dttrrgvsnmg_iv_0 : OR3C
      port map(A => \dttrrgvsnmg_iv_0_1\, B => N_822_i, C => 
        \dttrrgvsnmg_iv_0_3\, Y => dttrrgvsnmg);
    
    znjtscvsvcs_iv_0_a2 : OR3
      port map(A => \frcskdnwvmx_ns_1[6]\, B => 
        \znjtscvsvcs_iv_0_a2_0\, C => N_378_i, Y => N_646);
    
    un1_frcskdnwvmx_3_0_0 : AO1B
      port map(A => \frcskdnwvmx[11]_net_1\, B => EV_DONE_net_0, 
        C => \frcskdnwvmx_ns_1[0]\, Y => un1_frcskdnwvmx_3);
    
    \qgnzcpmbksd[2]\ : DFN1C0
      port map(D => \qgnzcpmbksd_7[2]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \TR_RAM_ADDR_B_net_net_5\);
    
    \frcskdnwvmx_ns_i_0_0[4]\ : OAI1
      port map(A => N_398, B => \frcskdnwvmx_0[8]_net_1\, C => 
        INIT_DONE_c, Y => \frcskdnwvmx_ns_i_0_0[4]_net_1\);
    
    \qgnzcpmbksd[4]\ : DFN1C0
      port map(D => \qgnzcpmbksd_7[4]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \TR_RAM_ADDR_B_net_net_3\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0[1]\ : OR3C
      port map(A => N_825, B => N_826, C => N_824, Y => 
        \qgnzcpmbksd_7[1]\);
    
    \frcskdnwvmx_ns_0[0]\ : AO1B
      port map(A => \frcskdnwvmx_ns_0_a2_3[0]_net_1\, B => 
        \frcskdnwvmx_ns_0_a2_3_0[0]\, C => \frcskdnwvmx_ns_1[0]\, 
        Y => \frcskdnwvmx_ns[0]\);
    
    \rjhhzsggrtr[7]\ : DFN1E1C0
      port map(D => \Z\\TR_RAM_DO_B_net\\\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \frcskdnwvmx[4]_net_1\, Q => 
        \Z\\TR_EV_ADDR_net\\\);
    
    cpwfrthtnfr_7_sqmuxa_0_a3_0_a2 : NOR2
      port map(A => \frcskdnwvmx_ns_1[7]\, B => N_381_i, Y => 
        cpwfrthtnfr_7_sqmuxa);
    
    \qgnzcpmbksd[5]\ : DFN1C0
      port map(D => \qgnzcpmbksd_7[5]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \TR_RAM_ADDR_B_net_net_2\);
    
    \frcskdnwvmx_ns_i_0_m3[3]\ : MX2B
      port map(A => \hnbsspwxkpd\, B => \jnfrmfqggmk\, S => 
        TR_RAM_DO_B_net_net_2, Y => N_390);
    
    \frcskdnwvmx_ns_a3_0_a2[9]\ : NOR2B
      port map(A => INIT_DONE_c, B => \TR_EV_RD_net\, Y => 
        \frcskdnwvmx_ns[9]\);
    
    un1_qgnzcpmbksd_I_34 : XOR2
      port map(A => \Z\\TR_RAM_ADDR_B_net\\_net_1\, B => 
        \DWACT_ADD_CI_0_g_array_12_2[0]\, Y => I_34_0);
    
    dttrrgvsnmg_iv_0_a2_1 : OR2B
      port map(A => EV_RAM_DO_A_net_net_7, B => 
        \frcskdnwvmx[2]_net_1\, Y => \qxzrwpjhcww_m[0]\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_i_o2[6]\ : AO1A
      port map(A => \frcskdnwvmx[8]_net_1\, B => N_769, C => 
        EV_DONE_net, Y => N_397);
    
    jnfrmfqggmk : DFN1E1C0
      port map(D => dttrrgvsnmg, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => un1_xjrbtnwdszs_1_sqmuxa, Q => 
        \jnfrmfqggmk\);
    
    \jfkkdmfhcdn_0_a3_0_a2[5]\ : NOR2A
      port map(A => TR_RAM_DO_B_net_net_2, B => 
        \frcskdnwvmx[0]_net_1\, Y => TR_RAM_DI_B_net_net_2);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_i[6]\ : NOR3
      port map(A => N_397, B => N_448, C => N_449, Y => 
        \qgnzcpmbksd_7_0_iv_i[6]\);
    
    \frcskdnwvmx_ns_a3_0_a2[6]\ : NOR3A
      port map(A => INIT_DONE_c, B => \frcskdnwvmx_ns_1[6]\, C
         => \frcskdnwvmx_ns_1[7]\, Y => \frcskdnwvmx_ns[6]\);
    
    \rjhhzsggrtr[0]\ : DFN1E1C0
      port map(D => TR_RAM_DO_B_net_net_7, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \frcskdnwvmx[4]_net_1\, Q => 
        TR_EV_ADDR_net_net_7);
    
    un1_qgnzcpmbksd_I_53 : AND2
      port map(A => \TR_RAM_ADDR_B_net_net_3\, B => 
        \TR_RAM_ADDR_B_net_net_2\, Y => 
        \DWACT_ADD_CI_0_pog_array_1_1[0]\);
    
    vnjtgtptszx_un1_xjrbtnwdszs_1_sqmuxa_0_a2 : NOR3A
      port map(A => N_766, B => TR_RAM_DO_B_net_net_2, C => N_776, 
        Y => N_786);
    
    \qgnzcpmbksd[7]\ : DFN1C0
      port map(D => \qgnzcpmbksd_7_0_iv_i[7]\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \Z\\TR_RAM_ADDR_B_net\\_net_1\);
    
    \rjhhzsggrtr[6]\ : DFN1E1C0
      port map(D => TR_RAM_DO_B_net_net_1, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \frcskdnwvmx[4]_net_1\, Q => 
        TR_EV_ADDR_net_net_1);
    
    \brxxctcvrmh_qgnzcpmbksd_7_iv_0_a2_1[0]\ : OR3
      port map(A => N_378_i, B => N_381_i, C => EV_DONE_net_0, Y
         => N_438);
    
    un1_qgnzcpmbksd_I_43 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_1[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_1[0]\, Y => 
        \DWACT_ADD_CI_0_g_array_2[0]\);
    
    \frcskdnwvmx_ns_a3_0_a2[2]\ : NOR2B
      port map(A => INIT_DONE_c, B => \frcskdnwvmx[10]_net_1\, Y
         => \frcskdnwvmx_ns[2]\);
    
    un1_qgnzcpmbksd_I_52 : AND2
      port map(A => \TR_RAM_ADDR_B_net_net_5\, B => 
        \TR_RAM_ADDR_B_net_net_4\, Y => 
        \DWACT_ADD_CI_0_pog_array_1[0]\);
    
    \jfkkdmfhcdn_0_a2[2]\ : NOR2A
      port map(A => TR_RAM_DO_B_net_net_5, B => 
        \frcskdnwvmx[0]_net_1\, Y => TR_RAM_DI_B_net_net_5);
    
    \frcskdnwvmx[10]\ : DFN1C0
      port map(D => \frcskdnwvmx_ns[1]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \frcskdnwvmx[10]_net_1\);
    
    \frcskdnwvmx_ns_i_0_0[5]\ : OR2A
      port map(A => INIT_DONE_c, B => N_460, Y => 
        \frcskdnwvmx_ns_i_0_0[5]_net_1\);
    
    \frcskdnwvmx[4]\ : DFN1C0
      port map(D => \frcskdnwvmx_ns[7]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \frcskdnwvmx[4]_net_1\);
    
    qgtkpxrgqqs_un1_xjrbtnwdszs_0_sqmuxa_0 : OR3C
      port map(A => N_785, B => N_784, C => N_783, Y => 
        un1_xjrbtnwdszs_0_sqmuxa);
    
    \qgnzcpmbksd[0]\ : DFN1C0
      port map(D => \qgnzcpmbksd_7[0]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \TR_RAM_ADDR_B_net_net_7\);
    
    znjtscvsvcs_iv_0_a2_0_0 : NOR2A
      port map(A => \hnbsspwxkpd\, B => N_385_i, Y => 
        \znjtscvsvcs_iv_0_a2_0_0\);
    
    \qgnzcpmbksd[6]\ : DFN1C0
      port map(D => \qgnzcpmbksd_7_0_iv_i[6]\, CLK => ACM_CLK_c, 
        CLR => sys_reset_gl, Q => \TR_RAM_ADDR_B_net_net_1\);
    
    un1_qgnzcpmbksd_I_42 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_2[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_1_1[0]\, Y => 
        \DWACT_ADD_CI_0_g_array_11[0]\);
    
    un1_qgnzcpmbksd_I_47 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_2[0]\, B => 
        \TR_RAM_ADDR_B_net_net_3\, Y => 
        \DWACT_ADD_CI_0_g_array_12_1[0]\);
    
    \frcskdnwvmx_ns_0_a2_3[0]\ : NOR3B
      port map(A => \frcskdnwvmx_ns_0_a2_1[0]_net_1\, B => N_460, 
        C => \frcskdnwvmx[4]_net_1\, Y => 
        \frcskdnwvmx_ns_0_a2_3_0[0]\);
    
    \frcskdnwvmx_ns_a3_1_0_a3_0_a2[7]\ : OR2
      port map(A => TR_RAM_DO_B_net_net_1, B => N_385_i, Y => 
        \frcskdnwvmx_ns_1[7]\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2[5]\ : OAI1
      port map(A => N_769, B => EV_DONE_net, C => 
        \EV_CHHOLD_net[4]\, Y => N_445);
    
    thrxvbszptn : DFN1C0
      port map(D => \frcskdnwvmx[0]_net_1\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => TR_DONE_net);
    
    \jfkkdmfhcdn_0_a2[1]\ : NOR2A
      port map(A => TR_RAM_DO_B_net_net_6, B => 
        \frcskdnwvmx[0]_net_1\, Y => TR_RAM_DI_B_net_net_6);
    
    znjtscvsvcs_iv_0 : OR3C
      port map(A => N_647, B => \qxzrwpjhcww_m[0]\, C => N_646, Y
         => znjtscvsvcs);
    
    \mwzbqqvzwjt_12_iv_0_o3_0_1[1]\ : OR3
      port map(A => TR_RAM_DO_B_net_net_4, B => 
        TR_RAM_DO_B_net_net_5, C => TR_RAM_DO_B_net_net_3, Y => 
        \mwzbqqvzwjt_12_iv_0_o3_0_1[1]_net_1\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    dttrrgvsnmg_iv_0_a2_0_1 : OA1
      port map(A => N_928, B => 
        \mwzbqqvzwjt_12_iv_0_o3_0_1[1]_net_1\, C => 
        \dttrrgvsnmg_iv_0_a2_0_0\, Y => \dttrrgvsnmg_iv_0_a2_0_1\);
    
    \jfkkdmfhcdn_0_a2[4]\ : NOR2A
      port map(A => TR_RAM_DO_B_net_net_3, B => 
        \frcskdnwvmx[0]_net_1\, Y => TR_RAM_DI_B_net_net_3);
    
    \frcskdnwvmx[0]\ : DFN1C0
      port map(D => \frcskdnwvmx_ns[11]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \frcskdnwvmx[0]_net_1\);
    
    \jfkkdmfhcdn_0_a3_0_a2[8]\ : NOR2A
      port map(A => \TR_RAM_DO_B_net[8]\, B => 
        \frcskdnwvmx[0]_net_1\, Y => \TR_RAM_DI_B_net[8]\);
    
    \rjhhzsggrtr[4]\ : DFN1E1C0
      port map(D => TR_RAM_DO_B_net_net_3, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \frcskdnwvmx[4]_net_1\, Q => 
        TR_EV_ADDR_net_net_3);
    
    jvxfmjbzjmc_0_0_a2_0_0 : NOR2A
      port map(A => \frcskdnwvmx[6]_net_1\, B => 
        TR_RAM_DO_B_net_net_2, Y => \jvxfmjbzjmc_0_0_a2_0\);
    
    dttrrgvsnmg_iv_0_a2_3_0 : OR2A
      port map(A => \jnfrmfqggmk\, B => N_928, Y => 
        \dttrrgvsnmg_iv_0_a2_3_0\);
    
    jvxfmjbzjmc_0_0_1 : NOR3A
      port map(A => N_432, B => N_401, C => 
        \frcskdnwvmx[10]_net_1\, Y => \jvxfmjbzjmc_0_0_1\);
    
    znjtscvsvcs_iv_0_a2_0 : OR3C
      port map(A => \znjtscvsvcs_iv_0_a2_0_0\, B => N_391_i, C
         => N_585_i_i_0, Y => N_647);
    
    un1_qgnzcpmbksd_I_46 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_2[0]\, B => 
        \DWACT_ADD_CI_0_pog_array_2[0]\, Y => 
        \DWACT_ADD_CI_0_g_array_3[0]\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2_1[4]\ : OR3B
      port map(A => \frcskdnwvmx[8]_net_1\, B => 
        TR_RAM_DO_B_net_net_3, C => EV_DONE_net_0, Y => N_444);
    
    qgtkpxrgqqs_un1_xjrbtnwdszs_0_sqmuxa_0_a2 : OR3B
      port map(A => TR_RAM_DO_B_net_net_1, B => 
        un1_xjrbtnwdszs_0_sqmuxa_0_a2_0, C => N_764, Y => N_783);
    
    jvxfmjbzjmc_0_0_a2_3_0 : NOR2A
      port map(A => TR_RAM_DO_B_net_net_2, B => \jnfrmfqggmk\, Y
         => \jvxfmjbzjmc_0_0_a2_3_0\);
    
    \frcskdnwvmx_ns_i_0_a2_1[5]\ : NOR3A
      port map(A => TR_RAM_DO_B_net_net_1, B => 
        \frcskdnwvmx_ns_i_0_a2_1_0[5]_net_1\, C => N_381_i, Y => 
        N_461);
    
    vnjtgtptszx_un1_xjrbtnwdszs_1_sqmuxa_0_a2_1 : NOR2A
      port map(A => N_766, B => TR_RAM_DO_B_net_net_1, Y => N_788);
    
    \frcskdnwvmx_ns_0_1[0]\ : NOR2A
      port map(A => INIT_DONE_c, B => \frcskdnwvmx[0]_net_1\, Y
         => \frcskdnwvmx_ns_1[0]\);
    
    dvqzjxhqhrh : DFN1E1C0
      port map(D => TR_RAM_DO_B_net_net_3, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => cpwfrthtnfr_7_sqmuxa, Q => 
        \dvqzjxhqhrh\);
    
    \frcskdnwvmx[9]\ : DFN1C0
      port map(D => \frcskdnwvmx_ns[2]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \frcskdnwvmx[9]_net_1\);
    
    dttrrgvsnmg_iv_0_a2_1_0 : OR3A
      port map(A => \hnbsspwxkpd\, B => N_776, C => 
        \frcskdnwvmx_ns_1[6]\, Y => \dttrrgvsnmg_iv_0_a2_1\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2[4]\ : OAI1
      port map(A => N_769, B => EV_DONE_net, C => 
        \Z\\EV_CHHOLD_net\\\, Y => N_442);
    
    \frcskdnwvmx_ns_i_0[4]\ : OA1C
      port map(A => N_405, B => \frcskdnwvmx_0[8]_net_1\, C => 
        \frcskdnwvmx_ns_i_0_0[4]_net_1\, Y => 
        \frcskdnwvmx_ns_i_0[4]_net_1\);
    
    \qgnzcpmbksd[3]\ : DFN1C0
      port map(D => \qgnzcpmbksd_7[3]\, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, Q => \TR_RAM_ADDR_B_net_net_4\);
    
    un1_qgnzcpmbksd_I_50 : NOR2B
      port map(A => \DWACT_ADD_CI_0_g_array_11[0]\, B => 
        \TR_RAM_ADDR_B_net_net_1\, Y => 
        \DWACT_ADD_CI_0_g_array_12_2[0]\);
    
    \rjhhzsggrtr[8]\ : DFN1E1C0
      port map(D => \TR_RAM_DO_B_net[8]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \frcskdnwvmx[4]_net_1\, Q => 
        \TR_EV_ADDR_net[8]\);
    
    un1_qgnzcpmbksd_I_35 : XOR2
      port map(A => \TR_RAM_ADDR_B_net_net_5\, B => 
        \DWACT_ADD_CI_0_g_array_1[0]\, Y => I_35_0);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2_0[5]\ : OR2B
      port map(A => I_33_0, B => N_446_2, Y => N_446);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2[3]\ : OAI1
      port map(A => N_769, B => EV_DONE_net_0, C => 
        EV_CHHOLD_net_net_1, Y => N_827);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_i_a2_0[7]\ : NOR2
      port map(A => I_34_0, B => \frcskdnwvmx_0[8]_net_1\, Y => 
        N_451);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_i_a2[7]\ : NOR2A
      port map(A => \frcskdnwvmx_0[8]_net_1\, B => 
        \Z\\TR_RAM_DO_B_net\\\, Y => N_450);
    
    un1_qgnzcpmbksd_I_1 : AND2
      port map(A => \TR_RAM_ADDR_B_net_net_7\, B => jvxfmjbzjmc, 
        Y => \DWACT_ADD_CI_0_TMP[0]\);
    
    \frcskdnwvmx[3]\ : DFN1C0
      port map(D => \frcskdnwvmx_ns[8]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \TR_EV_RD_net\);
    
    \frcskdnwvmx_ns_a3_0_a2_1[7]\ : NOR3A
      port map(A => TR_RAM_DO_B_net_net_2, B => 
        \Z\\TR_RAM_DO_B_net\\\, C => TR_RAM_DO_B_net_net_1, Y => 
        \frcskdnwvmx_ns_a3_0_a2_1[7]_net_1\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_i_a2[6]\ : NOR2A
      port map(A => \frcskdnwvmx_0[8]_net_1\, B => 
        TR_RAM_DO_B_net_net_1, Y => N_448);
    
    hnbsspwxkpd : DFN1E1C0
      port map(D => znjtscvsvcs, CLK => ACM_CLK_c, CLR => 
        sys_reset_gl, E => un1_xjrbtnwdszs_0_sqmuxa, Q => 
        \hnbsspwxkpd\);
    
    dttrrgvsnmg_iv_0_1 : OA1
      port map(A => N_378_i, B => \dttrrgvsnmg_iv_0_a2_1\, C => 
        \dttrrgvsnmg_iv_0_0\, Y => \dttrrgvsnmg_iv_0_1\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2_0[3]\ : OR2B
      port map(A => I_37_0, B => N_446_2, Y => N_828);
    
    \rjhhzsggrtr[2]\ : DFN1E1C0
      port map(D => TR_RAM_DO_B_net_net_5, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => \frcskdnwvmx[4]_net_1\, Q => 
        TR_EV_ADDR_net_net_5);
    
    un1_qgnzcpmbksd_I_21 : XOR2
      port map(A => \TR_RAM_ADDR_B_net_net_7\, B => jvxfmjbzjmc, 
        Y => \DWACT_ADD_CI_0_partial_sum[0]\);
    
    jvxfmjbzjmc_0_0_o2 : OR3
      port map(A => \frcskdnwvmx[5]_net_1\, B => 
        \frcskdnwvmx[7]_net_1\, C => \frcskdnwvmx[2]_net_1\, Y
         => N_401);
    
    cpwfrthtnfr_14_sqmuxa_0_i3_0_a3_0_a2 : OR3C
      port map(A => \Z\\TR_RAM_DO_B_net\\\, B => 
        \frcskdnwvmx[6]_net_1\, C => TR_RAM_DO_B_net_net_1, Y => 
        N_378_i);
    
    qgtkpxrgqqs_un1_xjrbtnwdszs_0_sqmuxa_0_o3_i_o2 : OR2A
      port map(A => TR_RAM_DO_B_net_net_7, B => 
        TR_RAM_DO_B_net_net_6, Y => N_928);
    
    \frcskdnwvmx_ns_a3_0_a2[8]\ : NOR2B
      port map(A => INIT_DONE_c, B => \frcskdnwvmx[4]_net_1\, Y
         => \frcskdnwvmx_ns[8]\);
    
    un1_qgnzcpmbksd_I_54 : AND2
      port map(A => \TR_RAM_ADDR_B_net_net_1\, B => 
        \Z\\TR_RAM_ADDR_B_net\\_net_1\, Y => 
        \DWACT_ADD_CI_0_pog_array_1_2[0]\);
    
    vnjtgtptszx_un1_xjrbtnwdszs_1_sqmuxa_0_a2_0 : NOR2B
      port map(A => \frcskdnwvmx[2]_net_1\, B => \dvqzjxhqhrh\, Y
         => N_787);
    
    \jfkkdmfhcdn_0_a3_0_a2[7]\ : NOR2A
      port map(A => \Z\\TR_RAM_DO_B_net\\\, B => 
        \frcskdnwvmx[0]_net_1\, Y => \Z\\TR_RAM_DI_B_net\\\);
    
    \jfkkdmfhcdn_0_a3_0_a2[6]\ : NOR2A
      port map(A => TR_RAM_DO_B_net_net_1, B => 
        \frcskdnwvmx[0]_net_1\, Y => TR_RAM_DI_B_net_net_1);
    
    \frcskdnwvmx[2]\ : DFN1C0
      port map(D => \frcskdnwvmx_ns[9]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, Q => \frcskdnwvmx[2]_net_1\);
    
    qgtkpxrgqqs_un1_xjrbtnwdszs_0_sqmuxa_0_a2_1 : OR3A
      port map(A => TR_RAM_DO_B_net_net_1, B => N_764, C => 
        \Z\\TR_RAM_DO_B_net\\\, Y => N_785);
    
    \frcskdnwvmx_ns_i_0_o3_i_a2[5]\ : NOR2B
      port map(A => TR_RAM_DO_B_net_net_1, B => 
        \TR_RAM_DO_B_net[8]\, Y => N_391_i);
    
    \brxxctcvrmh_qgnzcpmbksd_7_iv_0_a2_0_0[0]\ : NOR3
      port map(A => EV_DONE_net_0, B => \frcskdnwvmx[1]_net_1\, C
         => \frcskdnwvmx_0[8]_net_1\, Y => 
        \frcskdnwvmx_ns_0_a2_3[0]_net_1\);
    
    \frcskdnwvmx_0[8]\ : DFN1C0
      port map(D => \frcskdnwvmx_ns_i_0[3]_net_1\, CLK => 
        ACM_CLK_c, CLR => sys_reset_gl, Q => 
        \frcskdnwvmx_0[8]_net_1\);
    
    un1_qgnzcpmbksd_I_44 : NOR2B
      port map(A => \DWACT_ADD_CI_0_TMP[0]\, B => 
        \TR_RAM_ADDR_B_net_net_6\, Y => 
        \DWACT_ADD_CI_0_g_array_1[0]\);
    
    dttrrgvsnmg_iv_0_3 : OAI1
      port map(A => N_585_i_i_0, B => \jnfrmfqggmk\, C => N_788, 
        Y => \dttrrgvsnmg_iv_0_3\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2_0[4]\ : OR2B
      port map(A => I_30_0, B => N_446_2, Y => N_443);
    
    svpbmqwvncp_0_0 : OR3B
      port map(A => \svpbmqwvncp_0_0_1\, B => 
        \svpbmqwvncp_0_0_a2\, C => \frcskdnwvmx[4]_net_1\, Y => 
        TR_RAM_CSN_B_net);
    
    \frcskdnwvmx_ns_i_0_a2_0[5]\ : NOR2
      port map(A => \frcskdnwvmx[6]_net_1\, B => N_401, Y => 
        N_460);
    
    svpbmqwvncp_0_0_1 : NOR3
      port map(A => \TR_EV_RD_net\, B => \frcskdnwvmx[11]_net_1\, 
        C => \frcskdnwvmx_0[8]_net_1\, Y => \svpbmqwvncp_0_0_1\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2_1[5]\ : OR3B
      port map(A => \frcskdnwvmx[8]_net_1\, B => 
        TR_RAM_DO_B_net_net_2, C => EV_DONE_net, Y => N_447);
    
    jvxfmjbzjmc_0_0_a2_1 : OR3B
      port map(A => \Z\\TR_RAM_DO_B_net\\\, B => 
        \frcskdnwvmx[6]_net_1\, C => \TR_RAM_DO_B_net[8]\, Y => 
        N_433);
    
    \jfkkdmfhcdn_0_a2[0]\ : NOR2A
      port map(A => TR_RAM_DO_B_net_net_7, B => 
        \frcskdnwvmx[0]_net_1\, Y => TR_RAM_DI_B_net_net_7);
    
    un9_nwjvqqsgrjz_0_a3_0_a2 : NOR3C
      port map(A => \frcskdnwvmx[11]_net_1\, B => EV_DONE_net_0, 
        C => INIT_DONE_c, Y => \frcskdnwvmx_ns[1]\);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2[2]\ : OAI1
      port map(A => N_769, B => EV_DONE_net, C => 
        EV_CHHOLD_net_net_2, Y => N_439);
    
    jvxfmjbzjmc_0_0_2 : AO1D
      port map(A => \jvxfmjbzjmc_0_0_a2_3_0\, B => 
        \TR_RAM_DO_B_net[8]\, C => N_385_i, Y => 
        \jvxfmjbzjmc_0_0_2\);
    
    zzzxtvvjqhf : DFN1E1C0
      port map(D => \frcskdnwvmx_ns[1]\, CLK => ACM_CLK_c, CLR
         => sys_reset_gl, E => un1_frcskdnwvmx_3, Q => 
        TR_TRFLAG_net);
    
    \brxxctcvrmh_qgnzcpmbksd_7_0_iv_0_a2_1[1]\ : OR3B
      port map(A => \frcskdnwvmx[8]_net_1\, B => 
        TR_RAM_DO_B_net_net_6, C => EV_DONE_net, Y => N_826);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity SMTR is

    port(\TR_RAM_DO_B_net[8]\, \EV_CHHOLD_net[4]\ : in std_logic; 
        \TR_RAM_DI_B_net[8]\, \TR_EV_ADDR_net[8]\, 
        \TR_RAM_ADDR_B_net[8]\ : out std_logic; ACM_CLK_c, 
        sys_reset_gl : in std_logic; TR_RAM_ADDR_B_net_net_6, 
        TR_RAM_ADDR_B_net_net_3, TR_RAM_ADDR_B_net_net_5, 
        TR_RAM_ADDR_B_net_net_1, TR_EV_RD_net : out std_logic; 
        TR_RAM_DO_B_net_net_6, INIT_DONE_c, TR_RAM_DO_B_net_net_2, 
        TR_RAM_DO_B_net_net_1, \Z\\TR_RAM_DO_B_net\\\, 
        TR_RAM_DO_B_net_net_4, TR_RAM_DO_B_net_net_5, 
        TR_RAM_DO_B_net_net_3 : in std_logic; TR_RAM_CSN_B_net : 
        out std_logic; EV_DONE_net_0, TR_RAM_DO_B_net_net_7, 
        EV_RAM_DO_A_net_net_7, EV_CHHOLD_net_net_1, EV_DONE_net, 
        \Z\\EV_CHHOLD_net\\\, EV_CHHOLD_net_net_3, 
        EV_CHHOLD_net_net_2 : in std_logic; TR_RAM_DI_B_net_net_2, 
        TR_RAM_RW_B_net, TR_RAM_DI_B_net_net_3, 
        TR_RAM_DI_B_net_net_4, TR_RAM_DI_B_net_net_5, 
        TR_RAM_DI_B_net_net_6, TR_RAM_DI_B_net_net_7, 
        TR_RAM_DI_B_net_net_1, \Z\\TR_RAM_DI_B_net\\\, 
        cnwzzrhdsqv_i_a2 : out std_logic; N_12, N_14 : in
         std_logic; TR_DONE_net, TR_TRFLAG_net, 
        TR_EV_ADDR_net_net_7, TR_EV_ADDR_net_net_6, 
        TR_EV_ADDR_net_net_5, TR_EV_ADDR_net_net_4, 
        TR_EV_ADDR_net_net_3, TR_EV_ADDR_net_net_2, 
        TR_EV_ADDR_net_net_1, \Z\\TR_EV_ADDR_net\\\, 
        TR_RAM_ADDR_B_net_net_7, TR_RAM_ADDR_B_net_net_4, 
        TR_RAM_ADDR_B_net_net_2, \Z\\TR_RAM_ADDR_B_net\\\ : out
         std_logic);

end SMTR;

architecture DEF_ARCH of SMTR is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component ccfdxrhchsz
    port(\TR_RAM_ADDR_B_net[8]\, \TR_EV_ADDR_net[8]\, 
        \TR_RAM_DI_B_net[8]\ : out std_logic; \EV_CHHOLD_net[4]\, 
        \TR_RAM_DO_B_net[8]\ : in std_logic := 'U'; 
        \Z\\TR_RAM_ADDR_B_net\\\, TR_RAM_ADDR_B_net_net_2, 
        TR_RAM_ADDR_B_net_net_4, TR_RAM_ADDR_B_net_net_7, 
        \Z\\TR_EV_ADDR_net\\\, TR_EV_ADDR_net_net_1, 
        TR_EV_ADDR_net_net_2, TR_EV_ADDR_net_net_3, 
        TR_EV_ADDR_net_net_4, TR_EV_ADDR_net_net_5, 
        TR_EV_ADDR_net_net_6, TR_EV_ADDR_net_net_7, TR_TRFLAG_net, 
        TR_DONE_net : out std_logic; N_14, N_12 : in
         std_logic := 'U'; cnwzzrhdsqv_i_a2, 
        \Z\\TR_RAM_DI_B_net\\\, TR_RAM_DI_B_net_net_1, 
        TR_RAM_DI_B_net_net_7, TR_RAM_DI_B_net_net_6, 
        TR_RAM_DI_B_net_net_5, TR_RAM_DI_B_net_net_4, 
        TR_RAM_DI_B_net_net_3, TR_RAM_RW_B_net, 
        TR_RAM_DI_B_net_net_2 : out std_logic; 
        EV_CHHOLD_net_net_2, EV_CHHOLD_net_net_3, 
        \Z\\EV_CHHOLD_net\\\, EV_DONE_net, EV_CHHOLD_net_net_1, 
        EV_RAM_DO_A_net_net_7, TR_RAM_DO_B_net_net_7, 
        EV_DONE_net_0 : in std_logic := 'U'; TR_RAM_CSN_B_net : 
        out std_logic; TR_RAM_DO_B_net_net_3, 
        TR_RAM_DO_B_net_net_5, TR_RAM_DO_B_net_net_4, 
        \Z\\TR_RAM_DO_B_net\\\, TR_RAM_DO_B_net_net_1, 
        TR_RAM_DO_B_net_net_2, INIT_DONE_c, TR_RAM_DO_B_net_net_6 : 
        in std_logic := 'U'; TR_EV_RD_net, 
        TR_RAM_ADDR_B_net_net_1, TR_RAM_ADDR_B_net_net_5, 
        TR_RAM_ADDR_B_net_net_3, TR_RAM_ADDR_B_net_net_6 : out
         std_logic; sys_reset_gl, ACM_CLK_c : in std_logic := 'U');
        
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : ccfdxrhchsz
	Use entity work.ccfdxrhchsz(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    u_ccfdxrhchsz : ccfdxrhchsz
      port map(\TR_RAM_ADDR_B_net[8]\ => \TR_RAM_ADDR_B_net[8]\, 
        \TR_EV_ADDR_net[8]\ => \TR_EV_ADDR_net[8]\, 
        \TR_RAM_DI_B_net[8]\ => \TR_RAM_DI_B_net[8]\, 
        \EV_CHHOLD_net[4]\ => \EV_CHHOLD_net[4]\, 
        \TR_RAM_DO_B_net[8]\ => \TR_RAM_DO_B_net[8]\, 
        \Z\\TR_RAM_ADDR_B_net\\\ => \Z\\TR_RAM_ADDR_B_net\\\, 
        TR_RAM_ADDR_B_net_net_2 => TR_RAM_ADDR_B_net_net_2, 
        TR_RAM_ADDR_B_net_net_4 => TR_RAM_ADDR_B_net_net_4, 
        TR_RAM_ADDR_B_net_net_7 => TR_RAM_ADDR_B_net_net_7, 
        \Z\\TR_EV_ADDR_net\\\ => \Z\\TR_EV_ADDR_net\\\, 
        TR_EV_ADDR_net_net_1 => TR_EV_ADDR_net_net_1, 
        TR_EV_ADDR_net_net_2 => TR_EV_ADDR_net_net_2, 
        TR_EV_ADDR_net_net_3 => TR_EV_ADDR_net_net_3, 
        TR_EV_ADDR_net_net_4 => TR_EV_ADDR_net_net_4, 
        TR_EV_ADDR_net_net_5 => TR_EV_ADDR_net_net_5, 
        TR_EV_ADDR_net_net_6 => TR_EV_ADDR_net_net_6, 
        TR_EV_ADDR_net_net_7 => TR_EV_ADDR_net_net_7, 
        TR_TRFLAG_net => TR_TRFLAG_net, TR_DONE_net => 
        TR_DONE_net, N_14 => N_14, N_12 => N_12, cnwzzrhdsqv_i_a2
         => cnwzzrhdsqv_i_a2, \Z\\TR_RAM_DI_B_net\\\ => 
        \Z\\TR_RAM_DI_B_net\\\, TR_RAM_DI_B_net_net_1 => 
        TR_RAM_DI_B_net_net_1, TR_RAM_DI_B_net_net_7 => 
        TR_RAM_DI_B_net_net_7, TR_RAM_DI_B_net_net_6 => 
        TR_RAM_DI_B_net_net_6, TR_RAM_DI_B_net_net_5 => 
        TR_RAM_DI_B_net_net_5, TR_RAM_DI_B_net_net_4 => 
        TR_RAM_DI_B_net_net_4, TR_RAM_DI_B_net_net_3 => 
        TR_RAM_DI_B_net_net_3, TR_RAM_RW_B_net => TR_RAM_RW_B_net, 
        TR_RAM_DI_B_net_net_2 => TR_RAM_DI_B_net_net_2, 
        EV_CHHOLD_net_net_2 => EV_CHHOLD_net_net_2, 
        EV_CHHOLD_net_net_3 => EV_CHHOLD_net_net_3, 
        \Z\\EV_CHHOLD_net\\\ => \Z\\EV_CHHOLD_net\\\, EV_DONE_net
         => EV_DONE_net, EV_CHHOLD_net_net_1 => 
        EV_CHHOLD_net_net_1, EV_RAM_DO_A_net_net_7 => 
        EV_RAM_DO_A_net_net_7, TR_RAM_DO_B_net_net_7 => 
        TR_RAM_DO_B_net_net_7, EV_DONE_net_0 => EV_DONE_net_0, 
        TR_RAM_CSN_B_net => TR_RAM_CSN_B_net, 
        TR_RAM_DO_B_net_net_3 => TR_RAM_DO_B_net_net_3, 
        TR_RAM_DO_B_net_net_5 => TR_RAM_DO_B_net_net_5, 
        TR_RAM_DO_B_net_net_4 => TR_RAM_DO_B_net_net_4, 
        \Z\\TR_RAM_DO_B_net\\\ => \Z\\TR_RAM_DO_B_net\\\, 
        TR_RAM_DO_B_net_net_1 => TR_RAM_DO_B_net_net_1, 
        TR_RAM_DO_B_net_net_2 => TR_RAM_DO_B_net_net_2, 
        INIT_DONE_c => INIT_DONE_c, TR_RAM_DO_B_net_net_6 => 
        TR_RAM_DO_B_net_net_6, TR_EV_RD_net => TR_EV_RD_net, 
        TR_RAM_ADDR_B_net_net_1 => TR_RAM_ADDR_B_net_net_1, 
        TR_RAM_ADDR_B_net_net_5 => TR_RAM_ADDR_B_net_net_5, 
        TR_RAM_ADDR_B_net_net_3 => TR_RAM_ADDR_B_net_net_3, 
        TR_RAM_ADDR_B_net_net_6 => TR_RAM_ADDR_B_net_net_6, 
        sys_reset_gl => sys_reset_gl, ACM_CLK_c => ACM_CLK_c);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity Analog_RTC_smtr_wrapper is

    port(\TR_RAM_ADDR_B_net[8]\, \TR_EV_ADDR_net[8]\, 
        \TR_RAM_DI_B_net[8]\ : out std_logic; \EV_CHHOLD_net[4]\, 
        \TR_RAM_DO_B_net[8]\ : in std_logic; 
        \Z\\TR_RAM_ADDR_B_net\\\, TR_RAM_ADDR_B_net_net_2, 
        TR_RAM_ADDR_B_net_net_4, TR_RAM_ADDR_B_net_net_7, 
        \Z\\TR_EV_ADDR_net\\\, TR_EV_ADDR_net_net_1, 
        TR_EV_ADDR_net_net_2, TR_EV_ADDR_net_net_3, 
        TR_EV_ADDR_net_net_4, TR_EV_ADDR_net_net_5, 
        TR_EV_ADDR_net_net_6, TR_EV_ADDR_net_net_7, TR_TRFLAG_net, 
        TR_DONE_net : out std_logic; N_14, N_12 : in std_logic; 
        cnwzzrhdsqv_i_a2, \Z\\TR_RAM_DI_B_net\\\, 
        TR_RAM_DI_B_net_net_1, TR_RAM_DI_B_net_net_7, 
        TR_RAM_DI_B_net_net_6, TR_RAM_DI_B_net_net_5, 
        TR_RAM_DI_B_net_net_4, TR_RAM_DI_B_net_net_3, 
        TR_RAM_RW_B_net, TR_RAM_DI_B_net_net_2 : out std_logic; 
        EV_CHHOLD_net_net_2, EV_CHHOLD_net_net_3, 
        \Z\\EV_CHHOLD_net\\\, EV_DONE_net, EV_CHHOLD_net_net_1, 
        EV_RAM_DO_A_net_net_7, TR_RAM_DO_B_net_net_7, 
        EV_DONE_net_0 : in std_logic; TR_RAM_CSN_B_net : out
         std_logic; TR_RAM_DO_B_net_net_3, TR_RAM_DO_B_net_net_5, 
        TR_RAM_DO_B_net_net_4, \Z\\TR_RAM_DO_B_net\\\, 
        TR_RAM_DO_B_net_net_1, TR_RAM_DO_B_net_net_2, INIT_DONE_c, 
        TR_RAM_DO_B_net_net_6 : in std_logic; TR_EV_RD_net, 
        TR_RAM_ADDR_B_net_net_1, TR_RAM_ADDR_B_net_net_5, 
        TR_RAM_ADDR_B_net_net_3, TR_RAM_ADDR_B_net_net_6 : out
         std_logic; sys_reset_gl, ACM_CLK_c : in std_logic);

end Analog_RTC_smtr_wrapper;

architecture DEF_ARCH of Analog_RTC_smtr_wrapper is 

  component VCC
    port(Y : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component SMTR
    port(\TR_RAM_DO_B_net[8]\, \EV_CHHOLD_net[4]\ : in
         std_logic := 'U'; \TR_RAM_DI_B_net[8]\, 
        \TR_EV_ADDR_net[8]\, \TR_RAM_ADDR_B_net[8]\ : out
         std_logic; ACM_CLK_c, sys_reset_gl : in std_logic := 'U'; 
        TR_RAM_ADDR_B_net_net_6, TR_RAM_ADDR_B_net_net_3, 
        TR_RAM_ADDR_B_net_net_5, TR_RAM_ADDR_B_net_net_1, 
        TR_EV_RD_net : out std_logic; TR_RAM_DO_B_net_net_6, 
        INIT_DONE_c, TR_RAM_DO_B_net_net_2, TR_RAM_DO_B_net_net_1, 
        \Z\\TR_RAM_DO_B_net\\\, TR_RAM_DO_B_net_net_4, 
        TR_RAM_DO_B_net_net_5, TR_RAM_DO_B_net_net_3 : in
         std_logic := 'U'; TR_RAM_CSN_B_net : out std_logic; 
        EV_DONE_net_0, TR_RAM_DO_B_net_net_7, 
        EV_RAM_DO_A_net_net_7, EV_CHHOLD_net_net_1, EV_DONE_net, 
        \Z\\EV_CHHOLD_net\\\, EV_CHHOLD_net_net_3, 
        EV_CHHOLD_net_net_2 : in std_logic := 'U'; 
        TR_RAM_DI_B_net_net_2, TR_RAM_RW_B_net, 
        TR_RAM_DI_B_net_net_3, TR_RAM_DI_B_net_net_4, 
        TR_RAM_DI_B_net_net_5, TR_RAM_DI_B_net_net_6, 
        TR_RAM_DI_B_net_net_7, TR_RAM_DI_B_net_net_1, 
        \Z\\TR_RAM_DI_B_net\\\, cnwzzrhdsqv_i_a2 : out std_logic; 
        N_12, N_14 : in std_logic := 'U'; TR_DONE_net, 
        TR_TRFLAG_net, TR_EV_ADDR_net_net_7, TR_EV_ADDR_net_net_6, 
        TR_EV_ADDR_net_net_5, TR_EV_ADDR_net_net_4, 
        TR_EV_ADDR_net_net_3, TR_EV_ADDR_net_net_2, 
        TR_EV_ADDR_net_net_1, \Z\\TR_EV_ADDR_net\\\, 
        TR_RAM_ADDR_B_net_net_7, TR_RAM_ADDR_B_net_net_4, 
        TR_RAM_ADDR_B_net_net_2, \Z\\TR_RAM_ADDR_B_net\\\ : out
         std_logic);
  end component;

    signal \VCC\, \GND\, GND_net_1, VCC_net_1 : std_logic;

    for all : SMTR
	Use entity work.SMTR(DEF_ARCH);
begin 


    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    U0 : SMTR
      port map(\TR_RAM_DO_B_net[8]\ => \TR_RAM_DO_B_net[8]\, 
        \EV_CHHOLD_net[4]\ => \EV_CHHOLD_net[4]\, 
        \TR_RAM_DI_B_net[8]\ => \TR_RAM_DI_B_net[8]\, 
        \TR_EV_ADDR_net[8]\ => \TR_EV_ADDR_net[8]\, 
        \TR_RAM_ADDR_B_net[8]\ => \TR_RAM_ADDR_B_net[8]\, 
        ACM_CLK_c => ACM_CLK_c, sys_reset_gl => sys_reset_gl, 
        TR_RAM_ADDR_B_net_net_6 => TR_RAM_ADDR_B_net_net_6, 
        TR_RAM_ADDR_B_net_net_3 => TR_RAM_ADDR_B_net_net_3, 
        TR_RAM_ADDR_B_net_net_5 => TR_RAM_ADDR_B_net_net_5, 
        TR_RAM_ADDR_B_net_net_1 => TR_RAM_ADDR_B_net_net_1, 
        TR_EV_RD_net => TR_EV_RD_net, TR_RAM_DO_B_net_net_6 => 
        TR_RAM_DO_B_net_net_6, INIT_DONE_c => INIT_DONE_c, 
        TR_RAM_DO_B_net_net_2 => TR_RAM_DO_B_net_net_2, 
        TR_RAM_DO_B_net_net_1 => TR_RAM_DO_B_net_net_1, 
        \Z\\TR_RAM_DO_B_net\\\ => \Z\\TR_RAM_DO_B_net\\\, 
        TR_RAM_DO_B_net_net_4 => TR_RAM_DO_B_net_net_4, 
        TR_RAM_DO_B_net_net_5 => TR_RAM_DO_B_net_net_5, 
        TR_RAM_DO_B_net_net_3 => TR_RAM_DO_B_net_net_3, 
        TR_RAM_CSN_B_net => TR_RAM_CSN_B_net, EV_DONE_net_0 => 
        EV_DONE_net_0, TR_RAM_DO_B_net_net_7 => 
        TR_RAM_DO_B_net_net_7, EV_RAM_DO_A_net_net_7 => 
        EV_RAM_DO_A_net_net_7, EV_CHHOLD_net_net_1 => 
        EV_CHHOLD_net_net_1, EV_DONE_net => EV_DONE_net, 
        \Z\\EV_CHHOLD_net\\\ => \Z\\EV_CHHOLD_net\\\, 
        EV_CHHOLD_net_net_3 => EV_CHHOLD_net_net_3, 
        EV_CHHOLD_net_net_2 => EV_CHHOLD_net_net_2, 
        TR_RAM_DI_B_net_net_2 => TR_RAM_DI_B_net_net_2, 
        TR_RAM_RW_B_net => TR_RAM_RW_B_net, TR_RAM_DI_B_net_net_3
         => TR_RAM_DI_B_net_net_3, TR_RAM_DI_B_net_net_4 => 
        TR_RAM_DI_B_net_net_4, TR_RAM_DI_B_net_net_5 => 
        TR_RAM_DI_B_net_net_5, TR_RAM_DI_B_net_net_6 => 
        TR_RAM_DI_B_net_net_6, TR_RAM_DI_B_net_net_7 => 
        TR_RAM_DI_B_net_net_7, TR_RAM_DI_B_net_net_1 => 
        TR_RAM_DI_B_net_net_1, \Z\\TR_RAM_DI_B_net\\\ => 
        \Z\\TR_RAM_DI_B_net\\\, cnwzzrhdsqv_i_a2 => 
        cnwzzrhdsqv_i_a2, N_12 => N_12, N_14 => N_14, TR_DONE_net
         => TR_DONE_net, TR_TRFLAG_net => TR_TRFLAG_net, 
        TR_EV_ADDR_net_net_7 => TR_EV_ADDR_net_net_7, 
        TR_EV_ADDR_net_net_6 => TR_EV_ADDR_net_net_6, 
        TR_EV_ADDR_net_net_5 => TR_EV_ADDR_net_net_5, 
        TR_EV_ADDR_net_net_4 => TR_EV_ADDR_net_net_4, 
        TR_EV_ADDR_net_net_3 => TR_EV_ADDR_net_net_3, 
        TR_EV_ADDR_net_net_2 => TR_EV_ADDR_net_net_2, 
        TR_EV_ADDR_net_net_1 => TR_EV_ADDR_net_net_1, 
        \Z\\TR_EV_ADDR_net\\\ => \Z\\TR_EV_ADDR_net\\\, 
        TR_RAM_ADDR_B_net_net_7 => TR_RAM_ADDR_B_net_net_7, 
        TR_RAM_ADDR_B_net_net_4 => TR_RAM_ADDR_B_net_net_4, 
        TR_RAM_ADDR_B_net_net_2 => TR_RAM_ADDR_B_net_net_2, 
        \Z\\TR_RAM_ADDR_B_net\\\ => \Z\\TR_RAM_ADDR_B_net\\\);
    
    GND_i : GND
      port map(Y => \GND\);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity Analog_RTC is

    port(INIT_ADDR_int : in std_logic_vector(8 downto 5); 
        INIT_DATA : in std_logic_vector(8 downto 0); INIT_ADDR : 
        in std_logic_vector(8 downto 0); RTC_MODE : 
        out std_logic_vector(1 downto 0); ACMRDATA : 
        out std_logic_vector(7 downto 0); \rdgwgtpkbgc_0[3]\ : in
         std_logic; \rdgwgtpkbgc[3]\ : out std_logic; INIT_EV_WEN, 
        N_12, N_14 : in std_logic; ASSC_CHLATD_c_i : out
         std_logic; INIT_DONE_c, INIT_ASSC_WEN : in std_logic; 
        ASSC_DONE_c, ASSC_CHSAT_c : out std_logic; INIT_ACM_WEN, 
        RTCCLK, sys_reset_gl, ACM_CLK_c : in std_logic; SELMODE, 
        DATAVALID_c : out std_logic; VAREF : inout
         std_logic := 'Z');

end Analog_RTC;

architecture DEF_ARCH of Analog_RTC is 

  component AB
    port(DAVOUT0, DACOUT0, DATOUT0, DAVOUT1, DACOUT1, DATOUT1, 
        DAVOUT2, DACOUT2, DATOUT2, DAVOUT3, DACOUT3, DATOUT3, 
        DAVOUT4, DACOUT4, DATOUT4, DAVOUT5, DACOUT5, DATOUT5, 
        DAVOUT6, DACOUT6, DATOUT6, DAVOUT7, DACOUT7, DATOUT7, 
        DAVOUT8, DACOUT8, DATOUT8, DAVOUT9, DACOUT9, DATOUT9 : 
        out std_logic; DENAV0, DENAC0, DENAT0, DENAV1, DENAC1, 
        DENAT1, DENAV2, DENAC2, DENAT2, DENAV3, DENAC3, DENAT3, 
        DENAV4, DENAC4, DENAT4, DENAV5, DENAC5, DENAT5, DENAV6, 
        DENAC6, DENAT6, DENAV7, DENAC7, DENAT7, DENAV8, DENAC8, 
        DENAT8, DENAV9, DENAC9, DENAT9, AV0, AC0, AT0 : in
         std_logic := 'U'; AG0 : out std_logic; AV1, AC1, AT1 : 
        in std_logic := 'U'; AG1 : out std_logic; AV2, AC2, AT2 : 
        in std_logic := 'U'; AG2 : out std_logic; AV3, AC3, AT3 : 
        in std_logic := 'U'; AG3 : out std_logic; AV4, AC4, AT4 : 
        in std_logic := 'U'; AG4 : out std_logic; AV5, AC5, AT5 : 
        in std_logic := 'U'; AG5 : out std_logic; AV6, AC6, AT6 : 
        in std_logic := 'U'; AG6 : out std_logic; AV7, AC7, AT7 : 
        in std_logic := 'U'; AG7 : out std_logic; AV8, AC8, AT8 : 
        in std_logic := 'U'; AG8 : out std_logic; AV9, AC9, AT9 : 
        in std_logic := 'U'; AG9 : out std_logic; ATRETURN01, 
        ATRETURN23, ATRETURN45, ATRETURN67, ATRETURN89 : in
         std_logic := 'U'; VAREF : inout std_logic; GNDREF, GDON0, 
        GDON1, GDON2, GDON3, GDON4, GDON5, GDON6, GDON7, GDON8, 
        GDON9, CMSTB0, CMSTB1, CMSTB2, CMSTB3, CMSTB4, CMSTB5, 
        CMSTB6, CMSTB7, CMSTB8, CMSTB9, TMSTB0, TMSTB1, TMSTB2, 
        TMSTB3, TMSTB4, TMSTB5, TMSTB6, TMSTB7, TMSTB8, TMSTB9, 
        TMSTBINT : in std_logic := 'U'; RESULT : 
        out std_logic_vector(11 downto 0); ACMRDATA : 
        out std_logic_vector(7 downto 0); ACMWDATA : 
        in std_logic_vector(7 downto 0) := (others => 'U'); 
        ACMADDR : 
        in std_logic_vector(7 downto 0) := (others => 'U'); STC
         : in std_logic_vector(7 downto 0) := (others => 'U'); 
        TVC : in std_logic_vector(7 downto 0) := (others => 'U'); 
        CHNUMBER : 
        in std_logic_vector(4 downto 0) := (others => 'U'); MODE
         : in std_logic_vector(3 downto 0) := (others => 'U'); 
        RTCXTLMODE : out std_logic_vector(1 downto 0); ACMRESET, 
        ACMCLK, ACMWEN, VAREFSEL : in std_logic := 'U'; DATAVALID, 
        CALIBRATE, SAMPLE, BUSY : out std_logic; PWRDWN, SYSCLK, 
        ADCRESET, ADCSTART : in std_logic := 'U'; RTCMATCH, 
        RTCPSMMATCH, RTCXTLSEL : out std_logic; RTCCLK : in
         std_logic := 'U');
  end component;

  component Analog_RTC_assc_wrapper
    port(dsjtwjhnpmm_i : out std_logic_vector(8 downto 0); 
        \ADC_MODE_net[3]\, \ADC_TVC_net[7]\, \ADC_STC_net[7]\, 
        \ADC_CHNR_net[4]\, \tkcbxxqbtfx_ns[11]\, 
        \ASSC_SEQOUT_net[5]\, \ASSC_RAM_ADDR_B_net[8]\ : out
         std_logic; \EV_ASSCADDR_net[8]\, INIT_ADDR_int_0, 
        INIT_ADDR_int_2, INIT_ADDR_int_3, INIT_ADDR_3, 
        INIT_ADDR_2, INIT_ADDR_4, INIT_ADDR_6, INIT_ADDR_1, 
        INIT_ADDR_0 : in std_logic := 'U'; \ASSC_RAM_DI_B_net[8]\ : 
        out std_logic; \ADC_RESULT_net[11]\, 
        \ASSC_RAM_DO_B_net[8]\ : in std_logic := 'U'; 
        \Z\\ADC_MODE_net\\\, \Z\\ADC_TVC_net\\\, 
        ADC_TVC_net_net_1, ADC_TVC_net_net_2, ADC_TVC_net_net_3, 
        ADC_TVC_net_net_4, ADC_TVC_net_net_5, ADC_TVC_net_net_6, 
        \Z\\ADC_STC_net\\\, ADC_STC_net_net_1, ADC_STC_net_net_2, 
        ADC_STC_net_net_3, ADC_STC_net_net_4, ADC_STC_net_net_5, 
        ADC_STC_net_net_6 : out std_logic; 
        ASSC_RAM_DO_B_net_net_3 : in std_logic := 'U'; 
        \Z\\ADC_CHNR_net\\\, ADC_CHNR_net_net_1, 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_3, ADC_VREFSEL_net, 
        ASSC_SAMPFLAG_net, ASSC_CHSAT_c, ADC_PDOWN_net, 
        ADC_RESET_net : out std_logic; ADC_CALIBRATE_net : in
         std_logic := 'U'; ASSC_RAM_DI_B_net_net_3, ASSC_DONE_c, 
        ASSC_RAM_RW_B_net, rhbzpsxhwms : out std_logic; 
        EV_ASSCADDR_net_net_2 : in std_logic := 'U'; 
        ASSC_SEQOUT_net_net_4, ASSC_RAM_ADDR_B_net_net_4, 
        ASSC_RAM_ADDR_B_net_net_3, ASSC_RAM_ADDR_B_net_net_2, 
        ASSC_SEQOUT_net_net_1, ASSC_RAM_ADDR_B_net_net_1, 
        \Z\\ASSC_RAM_ADDR_B_net\\\, ASSC_RAM_ADDR_B_net_net_5 : 
        out std_logic; EV_ASSCADDR_net_net_4, 
        \Z\\EV_ASSCADDR_net\\\, ASSC_RAM_DO_B_net_net_1, 
        ASSC_RAM_DO_B_net_net_2 : in std_logic := 'U'; 
        ASSC_RAM_CSN_A_net : out std_logic; INIT_ASSC_WEN : in
         std_logic := 'U'; ttxwkrvqgng_i_a3 : out std_logic; 
        EV_ASSCADDR_net_net_3, EV_ASSCADDR_net_net_1 : in
         std_logic := 'U'; ADC_MODE_net_net_2 : out std_logic; 
        ADC_RESULT_net_net_7 : in std_logic := 'U'; 
        \Z\\ASSC_RAM_DI_B_net\\\, ASSC_RAM_DI_B_net_net_1, 
        ASSC_RAM_DI_B_net_net_2, ASSC_RAM_DI_B_net_net_4, 
        ASSC_RAM_DI_B_net_net_5, ASSC_RAM_DI_B_net_net_6, 
        ASSC_RAM_DI_B_net_net_7 : out std_logic; 
        ADC_RESULT_net_net_8, ADC_RESULT_net_net_9, 
        ADC_RESULT_net_net_10, EV_DONE_net_0 : in
         std_logic := 'U'; ASSC_RAM_ADDR_B_net_net_7, 
        ASSC_RAM_ADDR_B_net_net_6 : out std_logic; 
        bskrjhjvqpf_1_sqmuxa, EV_ASSCADDR_net_net_5 : in
         std_logic := 'U'; N_247_i_0 : out std_logic; 
        EV_EVFLAG_net, TR_DONE_net, TR_TRFLAG_net, 
        ADC_RESULT_net_net_4, ADC_RESULT_net_net_6, 
        ADC_RESULT_net_net_5, ADC_RESULT_net_net_2, 
        ADC_RESULT_net_net_3, ADC_RESULT_net_net_1, 
        \Z\\ADC_RESULT_net\\\ : in std_logic := 'U'; 
        ADC_MODE_net_net_1 : out std_logic; 
        \Z\\ASSC_RAM_DO_B_net\\\, ADC_BUSY_net, INIT_DONE_c : in
         std_logic := 'U'; jngrzvhtjvn_0, ASSC_CHLATD_c_i, 
        ASSC_SEQOUT_net_net_2, \Z\\ASSC_SEQOUT_net\\\, 
        ASSC_SEQOUT_net_net_3 : out std_logic; 
        ASSC_RAM_DO_B_net_net_4, ASSC_RAM_DO_B_net_net_5, 
        ASSC_RAM_DO_B_net_net_6 : in std_logic := 'U'; 
        ADC_START_net : out std_logic; ASSC_RAM_DO_B_net_net_7, 
        sys_reset_gl, ACM_CLK_c : in std_logic := 'U');
  end component;

  component OR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component Analog_RTC_smtr_ram
    port(INIT_DATA : 
        in std_logic_vector(8 downto 0) := (others => 'U'); 
        INIT_ADDR : 
        in std_logic_vector(8 downto 0) := (others => 'U'); 
        \TR_RAM_DI_B_net[8]\, \TR_RAM_ADDR_B_net[8]\ : in
         std_logic := 'U'; \TR_RAM_DO_B_net[8]\ : out std_logic; 
        sys_reset_gl, \Z\\TR_RAM_DI_B_net\\\, 
        TR_RAM_DI_B_net_net_1, TR_RAM_DI_B_net_net_2, 
        TR_RAM_DI_B_net_net_3, TR_RAM_DI_B_net_net_4, 
        TR_RAM_DI_B_net_net_5, TR_RAM_DI_B_net_net_6, 
        TR_RAM_DI_B_net_net_7, ACM_CLK_c, TR_RAM_RW_B_net, 
        TR_RAM_CSN_B_net, cnwzzrhdsqv_i_a2, 
        \Z\\TR_RAM_ADDR_B_net\\\, TR_RAM_ADDR_B_net_net_1, 
        TR_RAM_ADDR_B_net_net_2, TR_RAM_ADDR_B_net_net_3, 
        TR_RAM_ADDR_B_net_net_4, TR_RAM_ADDR_B_net_net_5, 
        TR_RAM_ADDR_B_net_net_6, TR_RAM_ADDR_B_net_net_7 : in
         std_logic := 'U'; \Z\\TR_RAM_DO_B_net\\\, 
        TR_RAM_DO_B_net_net_1, TR_RAM_DO_B_net_net_2, 
        TR_RAM_DO_B_net_net_3, TR_RAM_DO_B_net_net_4, 
        TR_RAM_DO_B_net_net_5, TR_RAM_DO_B_net_net_6, 
        TR_RAM_DO_B_net_net_7 : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component Analog_RTC_assc_ram
    port(INIT_DATA : 
        in std_logic_vector(8 downto 0) := (others => 'U'); 
        dsjtwjhnpmm_i : 
        in std_logic_vector(8 downto 0) := (others => 'U'); 
        \ASSC_RAM_DI_B_net[8]\, \ASSC_RAM_ADDR_B_net[8]\ : in
         std_logic := 'U'; \ASSC_RAM_DO_B_net[8]\, 
        \ASSC_RAM_DO_A_net[8]\ : out std_logic; sys_reset_gl, 
        \Z\\ASSC_RAM_DI_B_net\\\, ASSC_RAM_DI_B_net_net_1, 
        ASSC_RAM_DI_B_net_net_2, ASSC_RAM_DI_B_net_net_3, 
        ASSC_RAM_DI_B_net_net_4, ASSC_RAM_DI_B_net_net_5, 
        ASSC_RAM_DI_B_net_net_6, ASSC_RAM_DI_B_net_net_7, 
        ACM_CLK_c, ASSC_RAM_RW_B_net, N_247_i_0, ttxwkrvqgng_i_a3, 
        ASSC_RAM_CSN_A_net, \Z\\ASSC_RAM_ADDR_B_net\\\, 
        ASSC_RAM_ADDR_B_net_net_1, ASSC_RAM_ADDR_B_net_net_2, 
        ASSC_RAM_ADDR_B_net_net_3, ASSC_RAM_ADDR_B_net_net_4, 
        ASSC_RAM_ADDR_B_net_net_5, ASSC_RAM_ADDR_B_net_net_6, 
        ASSC_RAM_ADDR_B_net_net_7 : in std_logic := 'U'; 
        \Z\\ASSC_RAM_DO_B_net\\\, ASSC_RAM_DO_B_net_net_1, 
        ASSC_RAM_DO_B_net_net_2, ASSC_RAM_DO_B_net_net_3, 
        ASSC_RAM_DO_B_net_net_4, ASSC_RAM_DO_B_net_net_5, 
        ASSC_RAM_DO_B_net_net_6, ASSC_RAM_DO_B_net_net_7, 
        \Z\\ASSC_RAM_DO_A_net\\\, ASSC_RAM_DO_A_net_net_1, 
        ASSC_RAM_DO_A_net_net_2, ASSC_RAM_DO_A_net_net_3, 
        ASSC_RAM_DO_A_net_net_4, ASSC_RAM_DO_A_net_net_5, 
        ASSC_RAM_DO_A_net_net_6, ASSC_RAM_DO_A_net_net_7 : out
         std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component Analog_RTC_smev_wrapper
    port(INIT_ADDR_int : 
        in std_logic_vector(8 downto 5) := (others => 'U'); 
        INIT_ADDR : 
        in std_logic_vector(4 downto 0) := (others => 'U'); 
        \rdgwgtpkbgc_0[3]\, \EV_RAM_ADDR_B_net[8]\ : out
         std_logic; \ADC_CHNR_net[4]\, \ASSC_SEQOUT_net[5]\ : in
         std_logic := 'U'; \EV_ASSCADDR_net[8]\ : out std_logic; 
        \tkcbxxqbtfx_ns[11]\ : in std_logic := 'U'; 
        \EV_RAM_DI_B_net[8]\ : out std_logic; 
        \ASSC_RAM_DO_A_net[8]\, \TR_EV_ADDR_net[8]\ : in
         std_logic := 'U'; \EV_RAM_ADDR_A_net[8]\ : out std_logic; 
        \rdgwgtpkbgc[3]\ : in std_logic := 'U'; 
        \EV_CHHOLD_net[4]\ : out std_logic; \EV_RAM_DO_B_net[8]\ : 
        in std_logic := 'U'; \Z\\EV_RAM_ADDR_B_net\\\, 
        EV_RAM_ADDR_B_net_net_2, EV_RAM_ADDR_B_net_net_4, 
        EV_RAM_ADDR_B_net_net_7 : out std_logic; 
        \Z\\ADC_CHNR_net\\\, ADC_CHNR_net_net_1, 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_3, 
        \Z\\ASSC_SEQOUT_net\\\ : in std_logic := 'U'; 
        \Z\\EV_ASSCADDR_net\\\ : out std_logic; 
        ASSC_SEQOUT_net_net_1 : in std_logic := 'U'; 
        EV_ASSCADDR_net_net_1 : out std_logic; 
        ASSC_SEQOUT_net_net_2 : in std_logic := 'U'; 
        EV_ASSCADDR_net_net_2 : out std_logic; 
        ASSC_SEQOUT_net_net_3 : in std_logic := 'U'; 
        EV_ASSCADDR_net_net_3 : out std_logic; 
        ASSC_SEQOUT_net_net_4 : in std_logic := 'U'; 
        EV_ASSCADDR_net_net_4, EV_EVFLAG_net, EV_DONE_net, 
        EV_RAM_WR_BUSY_B_net, EV_RAM_DI_B_net_net_7, 
        EV_RAM_DI_B_net_net_6, EV_RAM_DI_B_net_net_2, 
        \Z\\EV_RAM_DI_B_net\\\, EV_RAM_DI_B_net_net_1 : out
         std_logic; TR_EV_RD_net : in std_logic := 'U'; 
        EV_RAM_CSN_A_net : out std_logic; INIT_EV_WEN : in
         std_logic := 'U'; EV_RAM_RW_A_net : out std_logic; 
        ASSC_DONE_c, ASSC_SAMPFLAG_net, jngrzvhtjvn_0, 
        rhbzpsxhwms, TR_EV_ADDR_net_net_7 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_7 : out std_logic; 
        TR_EV_ADDR_net_net_6 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_6 : out std_logic; 
        TR_EV_ADDR_net_net_4 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_4 : out std_logic; 
        TR_EV_ADDR_net_net_3 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_3 : out std_logic; 
        TR_EV_ADDR_net_net_2 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_2 : out std_logic; 
        TR_EV_ADDR_net_net_1 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_1 : out std_logic; 
        \Z\\TR_EV_ADDR_net\\\ : in std_logic := 'U'; 
        \Z\\EV_RAM_ADDR_A_net\\\ : out std_logic; 
        TR_EV_ADDR_net_net_5 : in std_logic := 'U'; 
        EV_RAM_ADDR_A_net_net_5 : out std_logic; 
        ASSC_RAM_DO_A_net_net_7, \Z\\ASSC_RAM_DO_A_net\\\, 
        ASSC_RAM_DO_A_net_net_1, ASSC_RAM_DO_A_net_net_2, 
        ASSC_RAM_DO_A_net_net_3, ASSC_RAM_DO_A_net_net_4, 
        ASSC_RAM_DO_A_net_net_5, ASSC_RAM_DO_A_net_net_6, 
        EV_RAM_DO_B_net_net_3, EV_RAM_DO_B_net_net_4 : in
         std_logic := 'U'; EV_RAM_DI_B_net_net_4, 
        EV_RAM_DI_B_net_net_3, EV_RAM_DI_B_net_net_5 : out
         std_logic; INIT_DONE_c : in std_logic := 'U'; 
        EV_RAM_CSN_B_net, EV_ASSCADDR_net_net_5 : out std_logic; 
        EV_RAM_DO_B_net_net_2 : in std_logic := 'U'; 
        EV_CHHOLD_net_net_3, EV_CHHOLD_net_net_2, 
        EV_CHHOLD_net_net_1, \Z\\EV_CHHOLD_net\\\ : out std_logic; 
        EV_RAM_DO_B_net_net_1, \Z\\EV_RAM_DO_B_net\\\ : in
         std_logic := 'U'; EV_RAM_ADDR_B_net_net_1, 
        EV_RAM_ADDR_B_net_net_5, EV_RAM_ADDR_B_net_net_3, 
        EV_RAM_ADDR_B_net_net_6 : out std_logic; 
        EV_RAM_DO_B_net_net_5, EV_RAM_DO_B_net_net_6 : in
         std_logic := 'U'; bskrjhjvqpf_1_sqmuxa : out std_logic; 
        EV_RAM_DO_B_net_net_7, sys_reset_gl, ACM_CLK_c : in
         std_logic := 'U'; EV_DONE_net_0 : out std_logic);
  end component;

  component Analog_RTC_smev_ram
    port(INIT_DATA : 
        in std_logic_vector(8 downto 0) := (others => 'U'); 
        \EV_RAM_DI_B_net[8]\, \EV_RAM_ADDR_B_net[8]\, 
        \EV_RAM_ADDR_A_net[8]\ : in std_logic := 'U'; 
        \EV_RAM_DO_B_net[8]\ : out std_logic; sys_reset_gl, 
        \Z\\EV_RAM_DI_B_net\\\, EV_RAM_DI_B_net_net_1, 
        EV_RAM_DI_B_net_net_2, EV_RAM_DI_B_net_net_3, 
        EV_RAM_DI_B_net_net_4, EV_RAM_DI_B_net_net_5, 
        EV_RAM_DI_B_net_net_6, EV_RAM_DI_B_net_net_7, ACM_CLK_c, 
        EV_RAM_WR_BUSY_B_net, EV_RAM_CSN_B_net, EV_RAM_RW_A_net, 
        EV_RAM_CSN_A_net, \Z\\EV_RAM_ADDR_B_net\\\, 
        EV_RAM_ADDR_B_net_net_1, EV_RAM_ADDR_B_net_net_2, 
        EV_RAM_ADDR_B_net_net_3, EV_RAM_ADDR_B_net_net_4, 
        EV_RAM_ADDR_B_net_net_5, EV_RAM_ADDR_B_net_net_6, 
        EV_RAM_ADDR_B_net_net_7, \Z\\EV_RAM_ADDR_A_net\\\, 
        EV_RAM_ADDR_A_net_net_1, EV_RAM_ADDR_A_net_net_2, 
        EV_RAM_ADDR_A_net_net_3, EV_RAM_ADDR_A_net_net_4, 
        EV_RAM_ADDR_A_net_net_5, EV_RAM_ADDR_A_net_net_6, 
        EV_RAM_ADDR_A_net_net_7 : in std_logic := 'U'; 
        \Z\\EV_RAM_DO_B_net\\\, EV_RAM_DO_B_net_net_1, 
        EV_RAM_DO_B_net_net_2, EV_RAM_DO_B_net_net_3, 
        EV_RAM_DO_B_net_net_4, EV_RAM_DO_B_net_net_5, 
        EV_RAM_DO_B_net_net_6, EV_RAM_DO_B_net_net_7, 
        EV_RAM_DO_A_net_net_7 : out std_logic);
  end component;

  component Analog_RTC_smtr_wrapper
    port(\TR_RAM_ADDR_B_net[8]\, \TR_EV_ADDR_net[8]\, 
        \TR_RAM_DI_B_net[8]\ : out std_logic; \EV_CHHOLD_net[4]\, 
        \TR_RAM_DO_B_net[8]\ : in std_logic := 'U'; 
        \Z\\TR_RAM_ADDR_B_net\\\, TR_RAM_ADDR_B_net_net_2, 
        TR_RAM_ADDR_B_net_net_4, TR_RAM_ADDR_B_net_net_7, 
        \Z\\TR_EV_ADDR_net\\\, TR_EV_ADDR_net_net_1, 
        TR_EV_ADDR_net_net_2, TR_EV_ADDR_net_net_3, 
        TR_EV_ADDR_net_net_4, TR_EV_ADDR_net_net_5, 
        TR_EV_ADDR_net_net_6, TR_EV_ADDR_net_net_7, TR_TRFLAG_net, 
        TR_DONE_net : out std_logic; N_14, N_12 : in
         std_logic := 'U'; cnwzzrhdsqv_i_a2, 
        \Z\\TR_RAM_DI_B_net\\\, TR_RAM_DI_B_net_net_1, 
        TR_RAM_DI_B_net_net_7, TR_RAM_DI_B_net_net_6, 
        TR_RAM_DI_B_net_net_5, TR_RAM_DI_B_net_net_4, 
        TR_RAM_DI_B_net_net_3, TR_RAM_RW_B_net, 
        TR_RAM_DI_B_net_net_2 : out std_logic; 
        EV_CHHOLD_net_net_2, EV_CHHOLD_net_net_3, 
        \Z\\EV_CHHOLD_net\\\, EV_DONE_net, EV_CHHOLD_net_net_1, 
        EV_RAM_DO_A_net_net_7, TR_RAM_DO_B_net_net_7, 
        EV_DONE_net_0 : in std_logic := 'U'; TR_RAM_CSN_B_net : 
        out std_logic; TR_RAM_DO_B_net_net_3, 
        TR_RAM_DO_B_net_net_5, TR_RAM_DO_B_net_net_4, 
        \Z\\TR_RAM_DO_B_net\\\, TR_RAM_DO_B_net_net_1, 
        TR_RAM_DO_B_net_net_2, INIT_DONE_c, TR_RAM_DO_B_net_net_6 : 
        in std_logic := 'U'; TR_EV_RD_net, 
        TR_RAM_ADDR_B_net_net_1, TR_RAM_ADDR_B_net_net_5, 
        TR_RAM_ADDR_B_net_net_3, TR_RAM_ADDR_B_net_net_6 : out
         std_logic; sys_reset_gl, ACM_CLK_c : in std_logic := 'U');
        
  end component;

    signal \ASSC_DONE_c\, AB_INST_AG0, AB_INST_AG1, AB_INST_AG2, 
        AB_INST_AG3, AB_INST_AG4, AB_INST_AG5, AB_INST_AG6, 
        AB_INST_AG7, AB_INST_AG8, AB_INST_AG9, ADC_BUSY_net, 
        ADC_CALIBRATE_net, AB_INST_DACOUT0, AB_INST_DACOUT1, 
        AB_INST_DACOUT2, AB_INST_DACOUT3, AB_INST_DACOUT4, 
        AB_INST_DACOUT5, AB_INST_DACOUT6, AB_INST_DACOUT7, 
        AB_INST_DACOUT8, AB_INST_DACOUT9, AB_INST_DATOUT0, 
        AB_INST_DATOUT1, AB_INST_DATOUT2, AB_INST_DATOUT3, 
        AB_INST_DATOUT4, AB_INST_DATOUT5, AB_INST_DATOUT6, 
        AB_INST_DATOUT7, AB_INST_DATOUT8, AB_INST_DATOUT9, 
        AB_INST_DAVOUT0, AB_INST_DAVOUT1, AB_INST_DAVOUT2, 
        AB_INST_DAVOUT3, AB_INST_DAVOUT4, AB_INST_DAVOUT5, 
        AB_INST_DAVOUT6, AB_INST_DAVOUT7, AB_INST_DAVOUT8, 
        AB_INST_DAVOUT9, ADC_RESULT_net_net_10, 
        ADC_RESULT_net_net_9, ADC_RESULT_net_net_8, 
        ADC_RESULT_net_net_7, ADC_RESULT_net_net_6, 
        ADC_RESULT_net_net_5, ADC_RESULT_net_net_4, 
        ADC_RESULT_net_net_3, ADC_RESULT_net_net_2, 
        ADC_RESULT_net_net_1, \Z\\ADC_RESULT_net\\\, 
        \ADC_RESULT_net[11]\, AB_INST_RTCMATCH, 
        AB_INST_RTCPSMMATCH, AB_INST_SAMPLE, \GND\, 
        INIT_ACM_WEN_OR_net, ADC_RESET_net, ADC_START_net, 
        ADC_CHNR_net_net_3, ADC_CHNR_net_net_2, 
        ADC_CHNR_net_net_1, \Z\\ADC_CHNR_net\\\, 
        \ADC_CHNR_net[4]\, ADC_MODE_net_net_2, ADC_MODE_net_net_1, 
        \Z\\ADC_MODE_net\\\, \ADC_MODE_net[3]\, ADC_PDOWN_net, 
        ADC_STC_net_net_6, ADC_STC_net_net_5, ADC_STC_net_net_4, 
        ADC_STC_net_net_3, ADC_STC_net_net_2, ADC_STC_net_net_1, 
        \Z\\ADC_STC_net\\\, \ADC_STC_net[7]\, ADC_TVC_net_net_6, 
        ADC_TVC_net_net_5, ADC_TVC_net_net_4, ADC_TVC_net_net_3, 
        ADC_TVC_net_net_2, ADC_TVC_net_net_1, \Z\\ADC_TVC_net\\\, 
        \ADC_TVC_net[7]\, ADC_VREFSEL_net, \tkcbxxqbtfx_ns[11]\, 
        \ASSC_SEQOUT_net[5]\, \ASSC_RAM_ADDR_B_net[8]\, 
        \EV_ASSCADDR_net[8]\, \dsjtwjhnpmm_i[0]\, 
        \dsjtwjhnpmm_i[1]\, \dsjtwjhnpmm_i[2]\, 
        \dsjtwjhnpmm_i[3]\, \dsjtwjhnpmm_i[4]\, 
        \dsjtwjhnpmm_i[5]\, \dsjtwjhnpmm_i[6]\, 
        \dsjtwjhnpmm_i[7]\, \dsjtwjhnpmm_i[8]\, 
        \ASSC_RAM_DI_B_net[8]\, \ASSC_RAM_DO_B_net[8]\, 
        ASSC_RAM_DO_B_net_net_3, ASSC_SAMPFLAG_net, 
        ASSC_RAM_DI_B_net_net_3, ASSC_RAM_RW_B_net, rhbzpsxhwms, 
        EV_ASSCADDR_net_net_2, ASSC_SEQOUT_net_net_4, 
        ASSC_RAM_ADDR_B_net_net_4, ASSC_RAM_ADDR_B_net_net_3, 
        ASSC_RAM_ADDR_B_net_net_2, ASSC_SEQOUT_net_net_1, 
        ASSC_RAM_ADDR_B_net_net_1, \Z\\ASSC_RAM_ADDR_B_net\\\, 
        ASSC_RAM_ADDR_B_net_net_5, EV_ASSCADDR_net_net_4, 
        \Z\\EV_ASSCADDR_net\\\, ASSC_RAM_DO_B_net_net_1, 
        ASSC_RAM_DO_B_net_net_2, ASSC_RAM_CSN_A_net, 
        ttxwkrvqgng_i_a3, EV_ASSCADDR_net_net_3, 
        EV_ASSCADDR_net_net_1, \Z\\ASSC_RAM_DI_B_net\\\, 
        ASSC_RAM_DI_B_net_net_1, ASSC_RAM_DI_B_net_net_2, 
        ASSC_RAM_DI_B_net_net_4, ASSC_RAM_DI_B_net_net_5, 
        ASSC_RAM_DI_B_net_net_6, ASSC_RAM_DI_B_net_net_7, 
        EV_DONE_net_0, ASSC_RAM_ADDR_B_net_net_7, 
        ASSC_RAM_ADDR_B_net_net_6, bskrjhjvqpf_1_sqmuxa, 
        EV_ASSCADDR_net_net_5, N_247_i_0, EV_EVFLAG_net, 
        TR_DONE_net, TR_TRFLAG_net, \Z\\ASSC_RAM_DO_B_net\\\, 
        jngrzvhtjvn_0, ASSC_SEQOUT_net_net_2, 
        \Z\\ASSC_SEQOUT_net\\\, ASSC_SEQOUT_net_net_3, 
        ASSC_RAM_DO_B_net_net_4, ASSC_RAM_DO_B_net_net_5, 
        ASSC_RAM_DO_B_net_net_6, ASSC_RAM_DO_B_net_net_7, 
        \TR_RAM_ADDR_B_net[8]\, \TR_EV_ADDR_net[8]\, 
        \TR_RAM_DI_B_net[8]\, \EV_CHHOLD_net[4]\, 
        \TR_RAM_DO_B_net[8]\, \Z\\TR_RAM_ADDR_B_net\\\, 
        TR_RAM_ADDR_B_net_net_2, TR_RAM_ADDR_B_net_net_4, 
        TR_RAM_ADDR_B_net_net_7, \Z\\TR_EV_ADDR_net\\\, 
        TR_EV_ADDR_net_net_1, TR_EV_ADDR_net_net_2, 
        TR_EV_ADDR_net_net_3, TR_EV_ADDR_net_net_4, 
        TR_EV_ADDR_net_net_5, TR_EV_ADDR_net_net_6, 
        TR_EV_ADDR_net_net_7, cnwzzrhdsqv_i_a2, 
        \Z\\TR_RAM_DI_B_net\\\, TR_RAM_DI_B_net_net_1, 
        TR_RAM_DI_B_net_net_7, TR_RAM_DI_B_net_net_6, 
        TR_RAM_DI_B_net_net_5, TR_RAM_DI_B_net_net_4, 
        TR_RAM_DI_B_net_net_3, TR_RAM_RW_B_net, 
        TR_RAM_DI_B_net_net_2, EV_CHHOLD_net_net_2, 
        EV_CHHOLD_net_net_3, \Z\\EV_CHHOLD_net\\\, EV_DONE_net, 
        EV_CHHOLD_net_net_1, EV_RAM_DO_A_net_net_7, 
        TR_RAM_DO_B_net_net_7, TR_RAM_CSN_B_net, 
        TR_RAM_DO_B_net_net_3, TR_RAM_DO_B_net_net_5, 
        TR_RAM_DO_B_net_net_4, \Z\\TR_RAM_DO_B_net\\\, 
        TR_RAM_DO_B_net_net_1, TR_RAM_DO_B_net_net_2, 
        TR_RAM_DO_B_net_net_6, TR_EV_RD_net, 
        TR_RAM_ADDR_B_net_net_1, TR_RAM_ADDR_B_net_net_5, 
        TR_RAM_ADDR_B_net_net_3, TR_RAM_ADDR_B_net_net_6, 
        \EV_RAM_ADDR_B_net[8]\, \EV_RAM_DI_B_net[8]\, 
        \ASSC_RAM_DO_A_net[8]\, \EV_RAM_ADDR_A_net[8]\, 
        \EV_RAM_DO_B_net[8]\, \Z\\EV_RAM_ADDR_B_net\\\, 
        EV_RAM_ADDR_B_net_net_2, EV_RAM_ADDR_B_net_net_4, 
        EV_RAM_ADDR_B_net_net_7, EV_RAM_WR_BUSY_B_net, 
        EV_RAM_DI_B_net_net_7, EV_RAM_DI_B_net_net_6, 
        EV_RAM_DI_B_net_net_2, \Z\\EV_RAM_DI_B_net\\\, 
        EV_RAM_DI_B_net_net_1, EV_RAM_CSN_A_net, EV_RAM_RW_A_net, 
        EV_RAM_ADDR_A_net_net_7, EV_RAM_ADDR_A_net_net_6, 
        EV_RAM_ADDR_A_net_net_4, EV_RAM_ADDR_A_net_net_3, 
        EV_RAM_ADDR_A_net_net_2, EV_RAM_ADDR_A_net_net_1, 
        \Z\\EV_RAM_ADDR_A_net\\\, EV_RAM_ADDR_A_net_net_5, 
        ASSC_RAM_DO_A_net_net_7, \Z\\ASSC_RAM_DO_A_net\\\, 
        ASSC_RAM_DO_A_net_net_1, ASSC_RAM_DO_A_net_net_2, 
        ASSC_RAM_DO_A_net_net_3, ASSC_RAM_DO_A_net_net_4, 
        ASSC_RAM_DO_A_net_net_5, ASSC_RAM_DO_A_net_net_6, 
        EV_RAM_DO_B_net_net_3, EV_RAM_DO_B_net_net_4, 
        EV_RAM_DI_B_net_net_4, EV_RAM_DI_B_net_net_3, 
        EV_RAM_DI_B_net_net_5, EV_RAM_CSN_B_net, 
        EV_RAM_DO_B_net_net_2, EV_RAM_DO_B_net_net_1, 
        \Z\\EV_RAM_DO_B_net\\\, EV_RAM_ADDR_B_net_net_1, 
        EV_RAM_ADDR_B_net_net_5, EV_RAM_ADDR_B_net_net_3, 
        EV_RAM_ADDR_B_net_net_6, EV_RAM_DO_B_net_net_5, 
        EV_RAM_DO_B_net_net_6, EV_RAM_DO_B_net_net_7, \VCC\, 
        GND_net_1, VCC_net_1 : std_logic;

    for all : Analog_RTC_assc_wrapper
	Use entity work.Analog_RTC_assc_wrapper(DEF_ARCH);
    for all : Analog_RTC_smtr_ram
	Use entity work.Analog_RTC_smtr_ram(DEF_ARCH);
    for all : Analog_RTC_assc_ram
	Use entity work.Analog_RTC_assc_ram(DEF_ARCH);
    for all : Analog_RTC_smev_wrapper
	Use entity work.Analog_RTC_smev_wrapper(DEF_ARCH);
    for all : Analog_RTC_smev_ram
	Use entity work.Analog_RTC_smev_ram(DEF_ARCH);
    for all : Analog_RTC_smtr_wrapper
	Use entity work.Analog_RTC_smtr_wrapper(DEF_ARCH);
begin 

    ASSC_DONE_c <= \ASSC_DONE_c\;

    AB_INST : AB
      port map(DAVOUT0 => AB_INST_DAVOUT0, DACOUT0 => 
        AB_INST_DACOUT0, DATOUT0 => AB_INST_DATOUT0, DAVOUT1 => 
        AB_INST_DAVOUT1, DACOUT1 => AB_INST_DACOUT1, DATOUT1 => 
        AB_INST_DATOUT1, DAVOUT2 => AB_INST_DAVOUT2, DACOUT2 => 
        AB_INST_DACOUT2, DATOUT2 => AB_INST_DATOUT2, DAVOUT3 => 
        AB_INST_DAVOUT3, DACOUT3 => AB_INST_DACOUT3, DATOUT3 => 
        AB_INST_DATOUT3, DAVOUT4 => AB_INST_DAVOUT4, DACOUT4 => 
        AB_INST_DACOUT4, DATOUT4 => AB_INST_DATOUT4, DAVOUT5 => 
        AB_INST_DAVOUT5, DACOUT5 => AB_INST_DACOUT5, DATOUT5 => 
        AB_INST_DATOUT5, DAVOUT6 => AB_INST_DAVOUT6, DACOUT6 => 
        AB_INST_DACOUT6, DATOUT6 => AB_INST_DATOUT6, DAVOUT7 => 
        AB_INST_DAVOUT7, DACOUT7 => AB_INST_DACOUT7, DATOUT7 => 
        AB_INST_DATOUT7, DAVOUT8 => AB_INST_DAVOUT8, DACOUT8 => 
        AB_INST_DACOUT8, DATOUT8 => AB_INST_DATOUT8, DAVOUT9 => 
        AB_INST_DAVOUT9, DACOUT9 => AB_INST_DACOUT9, DATOUT9 => 
        AB_INST_DATOUT9, DENAV0 => \GND\, DENAC0 => \GND\, DENAT0
         => \GND\, DENAV1 => \GND\, DENAC1 => \GND\, DENAT1 => 
        \GND\, DENAV2 => \GND\, DENAC2 => \GND\, DENAT2 => \GND\, 
        DENAV3 => \GND\, DENAC3 => \GND\, DENAT3 => \GND\, DENAV4
         => \GND\, DENAC4 => \GND\, DENAT4 => \GND\, DENAV5 => 
        \GND\, DENAC5 => \GND\, DENAT5 => \GND\, DENAV6 => \GND\, 
        DENAC6 => \GND\, DENAT6 => \GND\, DENAV7 => \GND\, DENAC7
         => \GND\, DENAT7 => \GND\, DENAV8 => \GND\, DENAC8 => 
        \GND\, DENAT8 => \GND\, DENAV9 => \GND\, DENAC9 => \GND\, 
        DENAT9 => \GND\, AV0 => \GND\, AC0 => \GND\, AT0 => \GND\, 
        AG0 => AB_INST_AG0, AV1 => \GND\, AC1 => \GND\, AT1 => 
        \GND\, AG1 => AB_INST_AG1, AV2 => \GND\, AC2 => \GND\, 
        AT2 => \GND\, AG2 => AB_INST_AG2, AV3 => \GND\, AC3 => 
        \GND\, AT3 => \GND\, AG3 => AB_INST_AG3, AV4 => \GND\, 
        AC4 => \GND\, AT4 => \GND\, AG4 => AB_INST_AG4, AV5 => 
        \GND\, AC5 => \GND\, AT5 => \GND\, AG5 => AB_INST_AG5, 
        AV6 => \GND\, AC6 => \GND\, AT6 => \GND\, AG6 => 
        AB_INST_AG6, AV7 => \GND\, AC7 => \GND\, AT7 => \GND\, 
        AG7 => AB_INST_AG7, AV8 => \GND\, AC8 => \GND\, AT8 => 
        \GND\, AG8 => AB_INST_AG8, AV9 => \GND\, AC9 => \GND\, 
        AT9 => \GND\, AG9 => AB_INST_AG9, ATRETURN01 => \GND\, 
        ATRETURN23 => \GND\, ATRETURN45 => \GND\, ATRETURN67 => 
        \GND\, ATRETURN89 => \GND\, VAREF => VAREF, GNDREF => 
        \GND\, GDON0 => \GND\, GDON1 => \GND\, GDON2 => \GND\, 
        GDON3 => \GND\, GDON4 => \GND\, GDON5 => \GND\, GDON6 => 
        \GND\, GDON7 => \GND\, GDON8 => \GND\, GDON9 => \GND\, 
        CMSTB0 => \GND\, CMSTB1 => \GND\, CMSTB2 => \GND\, CMSTB3
         => \GND\, CMSTB4 => \GND\, CMSTB5 => \GND\, CMSTB6 => 
        \GND\, CMSTB7 => \GND\, CMSTB8 => \GND\, CMSTB9 => \GND\, 
        TMSTB0 => \GND\, TMSTB1 => \GND\, TMSTB2 => \GND\, TMSTB3
         => \GND\, TMSTB4 => \GND\, TMSTB5 => \GND\, TMSTB6 => 
        \GND\, TMSTB7 => \GND\, TMSTB8 => \GND\, TMSTB9 => \GND\, 
        TMSTBINT => \GND\, RESULT(11) => \ADC_RESULT_net[11]\, 
        RESULT(10) => \Z\\ADC_RESULT_net\\\, RESULT(9) => 
        ADC_RESULT_net_net_1, RESULT(8) => ADC_RESULT_net_net_2, 
        RESULT(7) => ADC_RESULT_net_net_3, RESULT(6) => 
        ADC_RESULT_net_net_4, RESULT(5) => ADC_RESULT_net_net_5, 
        RESULT(4) => ADC_RESULT_net_net_6, RESULT(3) => 
        ADC_RESULT_net_net_7, RESULT(2) => ADC_RESULT_net_net_8, 
        RESULT(1) => ADC_RESULT_net_net_9, RESULT(0) => 
        ADC_RESULT_net_net_10, ACMRDATA(7) => ACMRDATA(7), 
        ACMRDATA(6) => ACMRDATA(6), ACMRDATA(5) => ACMRDATA(5), 
        ACMRDATA(4) => ACMRDATA(4), ACMRDATA(3) => ACMRDATA(3), 
        ACMRDATA(2) => ACMRDATA(2), ACMRDATA(1) => ACMRDATA(1), 
        ACMRDATA(0) => ACMRDATA(0), ACMWDATA(7) => INIT_DATA(7), 
        ACMWDATA(6) => INIT_DATA(6), ACMWDATA(5) => INIT_DATA(5), 
        ACMWDATA(4) => INIT_DATA(4), ACMWDATA(3) => INIT_DATA(3), 
        ACMWDATA(2) => INIT_DATA(2), ACMWDATA(1) => INIT_DATA(1), 
        ACMWDATA(0) => INIT_DATA(0), ACMADDR(7) => INIT_ADDR(7), 
        ACMADDR(6) => INIT_ADDR(6), ACMADDR(5) => INIT_ADDR(5), 
        ACMADDR(4) => INIT_ADDR(4), ACMADDR(3) => INIT_ADDR(3), 
        ACMADDR(2) => INIT_ADDR(2), ACMADDR(1) => INIT_ADDR(1), 
        ACMADDR(0) => INIT_ADDR(0), STC(7) => \ADC_STC_net[7]\, 
        STC(6) => \Z\\ADC_STC_net\\\, STC(5) => ADC_STC_net_net_1, 
        STC(4) => ADC_STC_net_net_2, STC(3) => ADC_STC_net_net_3, 
        STC(2) => ADC_STC_net_net_4, STC(1) => ADC_STC_net_net_5, 
        STC(0) => ADC_STC_net_net_6, TVC(7) => \ADC_TVC_net[7]\, 
        TVC(6) => \Z\\ADC_TVC_net\\\, TVC(5) => ADC_TVC_net_net_1, 
        TVC(4) => ADC_TVC_net_net_2, TVC(3) => ADC_TVC_net_net_3, 
        TVC(2) => ADC_TVC_net_net_4, TVC(1) => ADC_TVC_net_net_5, 
        TVC(0) => ADC_TVC_net_net_6, CHNUMBER(4) => 
        \ADC_CHNR_net[4]\, CHNUMBER(3) => \Z\\ADC_CHNR_net\\\, 
        CHNUMBER(2) => ADC_CHNR_net_net_1, CHNUMBER(1) => 
        ADC_CHNR_net_net_2, CHNUMBER(0) => ADC_CHNR_net_net_3, 
        MODE(3) => \ADC_MODE_net[3]\, MODE(2) => 
        \Z\\ADC_MODE_net\\\, MODE(1) => ADC_MODE_net_net_1, 
        MODE(0) => ADC_MODE_net_net_2, RTCXTLMODE(1) => 
        RTC_MODE(1), RTCXTLMODE(0) => RTC_MODE(0), ACMRESET => 
        sys_reset_gl, ACMCLK => ACM_CLK_c, ACMWEN => 
        INIT_ACM_WEN_OR_net, VAREFSEL => ADC_VREFSEL_net, 
        DATAVALID => DATAVALID_c, CALIBRATE => ADC_CALIBRATE_net, 
        SAMPLE => AB_INST_SAMPLE, BUSY => ADC_BUSY_net, PWRDWN
         => ADC_PDOWN_net, SYSCLK => ACM_CLK_c, ADCRESET => 
        ADC_RESET_net, ADCSTART => ADC_START_net, RTCMATCH => 
        AB_INST_RTCMATCH, RTCPSMMATCH => AB_INST_RTCPSMMATCH, 
        RTCXTLSEL => SELMODE, RTCCLK => RTCCLK);
    
    Analog_RTC_assc_wrapper_inst : Analog_RTC_assc_wrapper
      port map(dsjtwjhnpmm_i(8) => \dsjtwjhnpmm_i[8]\, 
        dsjtwjhnpmm_i(7) => \dsjtwjhnpmm_i[7]\, dsjtwjhnpmm_i(6)
         => \dsjtwjhnpmm_i[6]\, dsjtwjhnpmm_i(5) => 
        \dsjtwjhnpmm_i[5]\, dsjtwjhnpmm_i(4) => 
        \dsjtwjhnpmm_i[4]\, dsjtwjhnpmm_i(3) => 
        \dsjtwjhnpmm_i[3]\, dsjtwjhnpmm_i(2) => 
        \dsjtwjhnpmm_i[2]\, dsjtwjhnpmm_i(1) => 
        \dsjtwjhnpmm_i[1]\, dsjtwjhnpmm_i(0) => 
        \dsjtwjhnpmm_i[0]\, \ADC_MODE_net[3]\ => 
        \ADC_MODE_net[3]\, \ADC_TVC_net[7]\ => \ADC_TVC_net[7]\, 
        \ADC_STC_net[7]\ => \ADC_STC_net[7]\, \ADC_CHNR_net[4]\
         => \ADC_CHNR_net[4]\, \tkcbxxqbtfx_ns[11]\ => 
        \tkcbxxqbtfx_ns[11]\, \ASSC_SEQOUT_net[5]\ => 
        \ASSC_SEQOUT_net[5]\, \ASSC_RAM_ADDR_B_net[8]\ => 
        \ASSC_RAM_ADDR_B_net[8]\, \EV_ASSCADDR_net[8]\ => 
        \EV_ASSCADDR_net[8]\, INIT_ADDR_int_0 => INIT_ADDR_int(5), 
        INIT_ADDR_int_2 => INIT_ADDR_int(7), INIT_ADDR_int_3 => 
        INIT_ADDR_int(8), INIT_ADDR_3 => INIT_ADDR(3), 
        INIT_ADDR_2 => INIT_ADDR(2), INIT_ADDR_4 => INIT_ADDR(4), 
        INIT_ADDR_6 => INIT_ADDR(6), INIT_ADDR_1 => INIT_ADDR(1), 
        INIT_ADDR_0 => INIT_ADDR(0), \ASSC_RAM_DI_B_net[8]\ => 
        \ASSC_RAM_DI_B_net[8]\, \ADC_RESULT_net[11]\ => 
        \ADC_RESULT_net[11]\, \ASSC_RAM_DO_B_net[8]\ => 
        \ASSC_RAM_DO_B_net[8]\, \Z\\ADC_MODE_net\\\ => 
        \Z\\ADC_MODE_net\\\, \Z\\ADC_TVC_net\\\ => 
        \Z\\ADC_TVC_net\\\, ADC_TVC_net_net_1 => 
        ADC_TVC_net_net_1, ADC_TVC_net_net_2 => ADC_TVC_net_net_2, 
        ADC_TVC_net_net_3 => ADC_TVC_net_net_3, ADC_TVC_net_net_4
         => ADC_TVC_net_net_4, ADC_TVC_net_net_5 => 
        ADC_TVC_net_net_5, ADC_TVC_net_net_6 => ADC_TVC_net_net_6, 
        \Z\\ADC_STC_net\\\ => \Z\\ADC_STC_net\\\, 
        ADC_STC_net_net_1 => ADC_STC_net_net_1, ADC_STC_net_net_2
         => ADC_STC_net_net_2, ADC_STC_net_net_3 => 
        ADC_STC_net_net_3, ADC_STC_net_net_4 => ADC_STC_net_net_4, 
        ADC_STC_net_net_5 => ADC_STC_net_net_5, ADC_STC_net_net_6
         => ADC_STC_net_net_6, ASSC_RAM_DO_B_net_net_3 => 
        ASSC_RAM_DO_B_net_net_3, \Z\\ADC_CHNR_net\\\ => 
        \Z\\ADC_CHNR_net\\\, ADC_CHNR_net_net_1 => 
        ADC_CHNR_net_net_1, ADC_CHNR_net_net_2 => 
        ADC_CHNR_net_net_2, ADC_CHNR_net_net_3 => 
        ADC_CHNR_net_net_3, ADC_VREFSEL_net => ADC_VREFSEL_net, 
        ASSC_SAMPFLAG_net => ASSC_SAMPFLAG_net, ASSC_CHSAT_c => 
        ASSC_CHSAT_c, ADC_PDOWN_net => ADC_PDOWN_net, 
        ADC_RESET_net => ADC_RESET_net, ADC_CALIBRATE_net => 
        ADC_CALIBRATE_net, ASSC_RAM_DI_B_net_net_3 => 
        ASSC_RAM_DI_B_net_net_3, ASSC_DONE_c => \ASSC_DONE_c\, 
        ASSC_RAM_RW_B_net => ASSC_RAM_RW_B_net, rhbzpsxhwms => 
        rhbzpsxhwms, EV_ASSCADDR_net_net_2 => 
        EV_ASSCADDR_net_net_2, ASSC_SEQOUT_net_net_4 => 
        ASSC_SEQOUT_net_net_4, ASSC_RAM_ADDR_B_net_net_4 => 
        ASSC_RAM_ADDR_B_net_net_4, ASSC_RAM_ADDR_B_net_net_3 => 
        ASSC_RAM_ADDR_B_net_net_3, ASSC_RAM_ADDR_B_net_net_2 => 
        ASSC_RAM_ADDR_B_net_net_2, ASSC_SEQOUT_net_net_1 => 
        ASSC_SEQOUT_net_net_1, ASSC_RAM_ADDR_B_net_net_1 => 
        ASSC_RAM_ADDR_B_net_net_1, \Z\\ASSC_RAM_ADDR_B_net\\\ => 
        \Z\\ASSC_RAM_ADDR_B_net\\\, ASSC_RAM_ADDR_B_net_net_5 => 
        ASSC_RAM_ADDR_B_net_net_5, EV_ASSCADDR_net_net_4 => 
        EV_ASSCADDR_net_net_4, \Z\\EV_ASSCADDR_net\\\ => 
        \Z\\EV_ASSCADDR_net\\\, ASSC_RAM_DO_B_net_net_1 => 
        ASSC_RAM_DO_B_net_net_1, ASSC_RAM_DO_B_net_net_2 => 
        ASSC_RAM_DO_B_net_net_2, ASSC_RAM_CSN_A_net => 
        ASSC_RAM_CSN_A_net, INIT_ASSC_WEN => INIT_ASSC_WEN, 
        ttxwkrvqgng_i_a3 => ttxwkrvqgng_i_a3, 
        EV_ASSCADDR_net_net_3 => EV_ASSCADDR_net_net_3, 
        EV_ASSCADDR_net_net_1 => EV_ASSCADDR_net_net_1, 
        ADC_MODE_net_net_2 => ADC_MODE_net_net_2, 
        ADC_RESULT_net_net_7 => ADC_RESULT_net_net_7, 
        \Z\\ASSC_RAM_DI_B_net\\\ => \Z\\ASSC_RAM_DI_B_net\\\, 
        ASSC_RAM_DI_B_net_net_1 => ASSC_RAM_DI_B_net_net_1, 
        ASSC_RAM_DI_B_net_net_2 => ASSC_RAM_DI_B_net_net_2, 
        ASSC_RAM_DI_B_net_net_4 => ASSC_RAM_DI_B_net_net_4, 
        ASSC_RAM_DI_B_net_net_5 => ASSC_RAM_DI_B_net_net_5, 
        ASSC_RAM_DI_B_net_net_6 => ASSC_RAM_DI_B_net_net_6, 
        ASSC_RAM_DI_B_net_net_7 => ASSC_RAM_DI_B_net_net_7, 
        ADC_RESULT_net_net_8 => ADC_RESULT_net_net_8, 
        ADC_RESULT_net_net_9 => ADC_RESULT_net_net_9, 
        ADC_RESULT_net_net_10 => ADC_RESULT_net_net_10, 
        EV_DONE_net_0 => EV_DONE_net_0, ASSC_RAM_ADDR_B_net_net_7
         => ASSC_RAM_ADDR_B_net_net_7, ASSC_RAM_ADDR_B_net_net_6
         => ASSC_RAM_ADDR_B_net_net_6, bskrjhjvqpf_1_sqmuxa => 
        bskrjhjvqpf_1_sqmuxa, EV_ASSCADDR_net_net_5 => 
        EV_ASSCADDR_net_net_5, N_247_i_0 => N_247_i_0, 
        EV_EVFLAG_net => EV_EVFLAG_net, TR_DONE_net => 
        TR_DONE_net, TR_TRFLAG_net => TR_TRFLAG_net, 
        ADC_RESULT_net_net_4 => ADC_RESULT_net_net_4, 
        ADC_RESULT_net_net_6 => ADC_RESULT_net_net_6, 
        ADC_RESULT_net_net_5 => ADC_RESULT_net_net_5, 
        ADC_RESULT_net_net_2 => ADC_RESULT_net_net_2, 
        ADC_RESULT_net_net_3 => ADC_RESULT_net_net_3, 
        ADC_RESULT_net_net_1 => ADC_RESULT_net_net_1, 
        \Z\\ADC_RESULT_net\\\ => \Z\\ADC_RESULT_net\\\, 
        ADC_MODE_net_net_1 => ADC_MODE_net_net_1, 
        \Z\\ASSC_RAM_DO_B_net\\\ => \Z\\ASSC_RAM_DO_B_net\\\, 
        ADC_BUSY_net => ADC_BUSY_net, INIT_DONE_c => INIT_DONE_c, 
        jngrzvhtjvn_0 => jngrzvhtjvn_0, ASSC_CHLATD_c_i => 
        ASSC_CHLATD_c_i, ASSC_SEQOUT_net_net_2 => 
        ASSC_SEQOUT_net_net_2, \Z\\ASSC_SEQOUT_net\\\ => 
        \Z\\ASSC_SEQOUT_net\\\, ASSC_SEQOUT_net_net_3 => 
        ASSC_SEQOUT_net_net_3, ASSC_RAM_DO_B_net_net_4 => 
        ASSC_RAM_DO_B_net_net_4, ASSC_RAM_DO_B_net_net_5 => 
        ASSC_RAM_DO_B_net_net_5, ASSC_RAM_DO_B_net_net_6 => 
        ASSC_RAM_DO_B_net_net_6, ADC_START_net => ADC_START_net, 
        ASSC_RAM_DO_B_net_net_7 => ASSC_RAM_DO_B_net_net_7, 
        sys_reset_gl => sys_reset_gl, ACM_CLK_c => ACM_CLK_c);
    
    INIT_ACMWEN_OR : OR2
      port map(A => INIT_ACM_WEN, B => \GND\, Y => 
        INIT_ACM_WEN_OR_net);
    
    Analog_RTC_smtr_ram_inst : Analog_RTC_smtr_ram
      port map(INIT_DATA(8) => INIT_DATA(8), INIT_DATA(7) => 
        INIT_DATA(7), INIT_DATA(6) => INIT_DATA(6), INIT_DATA(5)
         => INIT_DATA(5), INIT_DATA(4) => INIT_DATA(4), 
        INIT_DATA(3) => INIT_DATA(3), INIT_DATA(2) => 
        INIT_DATA(2), INIT_DATA(1) => INIT_DATA(1), INIT_DATA(0)
         => INIT_DATA(0), INIT_ADDR(8) => INIT_ADDR(8), 
        INIT_ADDR(7) => INIT_ADDR(7), INIT_ADDR(6) => 
        INIT_ADDR(6), INIT_ADDR(5) => INIT_ADDR(5), INIT_ADDR(4)
         => INIT_ADDR(4), INIT_ADDR(3) => INIT_ADDR(3), 
        INIT_ADDR(2) => INIT_ADDR(2), INIT_ADDR(1) => 
        INIT_ADDR(1), INIT_ADDR(0) => INIT_ADDR(0), 
        \TR_RAM_DI_B_net[8]\ => \TR_RAM_DI_B_net[8]\, 
        \TR_RAM_ADDR_B_net[8]\ => \TR_RAM_ADDR_B_net[8]\, 
        \TR_RAM_DO_B_net[8]\ => \TR_RAM_DO_B_net[8]\, 
        sys_reset_gl => sys_reset_gl, \Z\\TR_RAM_DI_B_net\\\ => 
        \Z\\TR_RAM_DI_B_net\\\, TR_RAM_DI_B_net_net_1 => 
        TR_RAM_DI_B_net_net_1, TR_RAM_DI_B_net_net_2 => 
        TR_RAM_DI_B_net_net_2, TR_RAM_DI_B_net_net_3 => 
        TR_RAM_DI_B_net_net_3, TR_RAM_DI_B_net_net_4 => 
        TR_RAM_DI_B_net_net_4, TR_RAM_DI_B_net_net_5 => 
        TR_RAM_DI_B_net_net_5, TR_RAM_DI_B_net_net_6 => 
        TR_RAM_DI_B_net_net_6, TR_RAM_DI_B_net_net_7 => 
        TR_RAM_DI_B_net_net_7, ACM_CLK_c => ACM_CLK_c, 
        TR_RAM_RW_B_net => TR_RAM_RW_B_net, TR_RAM_CSN_B_net => 
        TR_RAM_CSN_B_net, cnwzzrhdsqv_i_a2 => cnwzzrhdsqv_i_a2, 
        \Z\\TR_RAM_ADDR_B_net\\\ => \Z\\TR_RAM_ADDR_B_net\\\, 
        TR_RAM_ADDR_B_net_net_1 => TR_RAM_ADDR_B_net_net_1, 
        TR_RAM_ADDR_B_net_net_2 => TR_RAM_ADDR_B_net_net_2, 
        TR_RAM_ADDR_B_net_net_3 => TR_RAM_ADDR_B_net_net_3, 
        TR_RAM_ADDR_B_net_net_4 => TR_RAM_ADDR_B_net_net_4, 
        TR_RAM_ADDR_B_net_net_5 => TR_RAM_ADDR_B_net_net_5, 
        TR_RAM_ADDR_B_net_net_6 => TR_RAM_ADDR_B_net_net_6, 
        TR_RAM_ADDR_B_net_net_7 => TR_RAM_ADDR_B_net_net_7, 
        \Z\\TR_RAM_DO_B_net\\\ => \Z\\TR_RAM_DO_B_net\\\, 
        TR_RAM_DO_B_net_net_1 => TR_RAM_DO_B_net_net_1, 
        TR_RAM_DO_B_net_net_2 => TR_RAM_DO_B_net_net_2, 
        TR_RAM_DO_B_net_net_3 => TR_RAM_DO_B_net_net_3, 
        TR_RAM_DO_B_net_net_4 => TR_RAM_DO_B_net_net_4, 
        TR_RAM_DO_B_net_net_5 => TR_RAM_DO_B_net_net_5, 
        TR_RAM_DO_B_net_net_6 => TR_RAM_DO_B_net_net_6, 
        TR_RAM_DO_B_net_net_7 => TR_RAM_DO_B_net_net_7);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    Analog_RTC_assc_ram_inst : Analog_RTC_assc_ram
      port map(INIT_DATA(8) => INIT_DATA(8), INIT_DATA(7) => 
        INIT_DATA(7), INIT_DATA(6) => INIT_DATA(6), INIT_DATA(5)
         => INIT_DATA(5), INIT_DATA(4) => INIT_DATA(4), 
        INIT_DATA(3) => INIT_DATA(3), INIT_DATA(2) => 
        INIT_DATA(2), INIT_DATA(1) => INIT_DATA(1), INIT_DATA(0)
         => INIT_DATA(0), dsjtwjhnpmm_i(8) => \dsjtwjhnpmm_i[8]\, 
        dsjtwjhnpmm_i(7) => \dsjtwjhnpmm_i[7]\, dsjtwjhnpmm_i(6)
         => \dsjtwjhnpmm_i[6]\, dsjtwjhnpmm_i(5) => 
        \dsjtwjhnpmm_i[5]\, dsjtwjhnpmm_i(4) => 
        \dsjtwjhnpmm_i[4]\, dsjtwjhnpmm_i(3) => 
        \dsjtwjhnpmm_i[3]\, dsjtwjhnpmm_i(2) => 
        \dsjtwjhnpmm_i[2]\, dsjtwjhnpmm_i(1) => 
        \dsjtwjhnpmm_i[1]\, dsjtwjhnpmm_i(0) => 
        \dsjtwjhnpmm_i[0]\, \ASSC_RAM_DI_B_net[8]\ => 
        \ASSC_RAM_DI_B_net[8]\, \ASSC_RAM_ADDR_B_net[8]\ => 
        \ASSC_RAM_ADDR_B_net[8]\, \ASSC_RAM_DO_B_net[8]\ => 
        \ASSC_RAM_DO_B_net[8]\, \ASSC_RAM_DO_A_net[8]\ => 
        \ASSC_RAM_DO_A_net[8]\, sys_reset_gl => sys_reset_gl, 
        \Z\\ASSC_RAM_DI_B_net\\\ => \Z\\ASSC_RAM_DI_B_net\\\, 
        ASSC_RAM_DI_B_net_net_1 => ASSC_RAM_DI_B_net_net_1, 
        ASSC_RAM_DI_B_net_net_2 => ASSC_RAM_DI_B_net_net_2, 
        ASSC_RAM_DI_B_net_net_3 => ASSC_RAM_DI_B_net_net_3, 
        ASSC_RAM_DI_B_net_net_4 => ASSC_RAM_DI_B_net_net_4, 
        ASSC_RAM_DI_B_net_net_5 => ASSC_RAM_DI_B_net_net_5, 
        ASSC_RAM_DI_B_net_net_6 => ASSC_RAM_DI_B_net_net_6, 
        ASSC_RAM_DI_B_net_net_7 => ASSC_RAM_DI_B_net_net_7, 
        ACM_CLK_c => ACM_CLK_c, ASSC_RAM_RW_B_net => 
        ASSC_RAM_RW_B_net, N_247_i_0 => N_247_i_0, 
        ttxwkrvqgng_i_a3 => ttxwkrvqgng_i_a3, ASSC_RAM_CSN_A_net
         => ASSC_RAM_CSN_A_net, \Z\\ASSC_RAM_ADDR_B_net\\\ => 
        \Z\\ASSC_RAM_ADDR_B_net\\\, ASSC_RAM_ADDR_B_net_net_1 => 
        ASSC_RAM_ADDR_B_net_net_1, ASSC_RAM_ADDR_B_net_net_2 => 
        ASSC_RAM_ADDR_B_net_net_2, ASSC_RAM_ADDR_B_net_net_3 => 
        ASSC_RAM_ADDR_B_net_net_3, ASSC_RAM_ADDR_B_net_net_4 => 
        ASSC_RAM_ADDR_B_net_net_4, ASSC_RAM_ADDR_B_net_net_5 => 
        ASSC_RAM_ADDR_B_net_net_5, ASSC_RAM_ADDR_B_net_net_6 => 
        ASSC_RAM_ADDR_B_net_net_6, ASSC_RAM_ADDR_B_net_net_7 => 
        ASSC_RAM_ADDR_B_net_net_7, \Z\\ASSC_RAM_DO_B_net\\\ => 
        \Z\\ASSC_RAM_DO_B_net\\\, ASSC_RAM_DO_B_net_net_1 => 
        ASSC_RAM_DO_B_net_net_1, ASSC_RAM_DO_B_net_net_2 => 
        ASSC_RAM_DO_B_net_net_2, ASSC_RAM_DO_B_net_net_3 => 
        ASSC_RAM_DO_B_net_net_3, ASSC_RAM_DO_B_net_net_4 => 
        ASSC_RAM_DO_B_net_net_4, ASSC_RAM_DO_B_net_net_5 => 
        ASSC_RAM_DO_B_net_net_5, ASSC_RAM_DO_B_net_net_6 => 
        ASSC_RAM_DO_B_net_net_6, ASSC_RAM_DO_B_net_net_7 => 
        ASSC_RAM_DO_B_net_net_7, \Z\\ASSC_RAM_DO_A_net\\\ => 
        \Z\\ASSC_RAM_DO_A_net\\\, ASSC_RAM_DO_A_net_net_1 => 
        ASSC_RAM_DO_A_net_net_1, ASSC_RAM_DO_A_net_net_2 => 
        ASSC_RAM_DO_A_net_net_2, ASSC_RAM_DO_A_net_net_3 => 
        ASSC_RAM_DO_A_net_net_3, ASSC_RAM_DO_A_net_net_4 => 
        ASSC_RAM_DO_A_net_net_4, ASSC_RAM_DO_A_net_net_5 => 
        ASSC_RAM_DO_A_net_net_5, ASSC_RAM_DO_A_net_net_6 => 
        ASSC_RAM_DO_A_net_net_6, ASSC_RAM_DO_A_net_net_7 => 
        ASSC_RAM_DO_A_net_net_7);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    GND_i : GND
      port map(Y => \GND\);
    
    Analog_RTC_smev_wrapper_inst : Analog_RTC_smev_wrapper
      port map(INIT_ADDR_int(8) => INIT_ADDR_int(8), 
        INIT_ADDR_int(7) => INIT_ADDR_int(7), INIT_ADDR_int(6)
         => INIT_ADDR_int(6), INIT_ADDR_int(5) => 
        INIT_ADDR_int(5), INIT_ADDR(4) => INIT_ADDR(4), 
        INIT_ADDR(3) => INIT_ADDR(3), INIT_ADDR(2) => 
        INIT_ADDR(2), INIT_ADDR(1) => INIT_ADDR(1), INIT_ADDR(0)
         => INIT_ADDR(0), \rdgwgtpkbgc_0[3]\ => \rdgwgtpkbgc[3]\, 
        \EV_RAM_ADDR_B_net[8]\ => \EV_RAM_ADDR_B_net[8]\, 
        \ADC_CHNR_net[4]\ => \ADC_CHNR_net[4]\, 
        \ASSC_SEQOUT_net[5]\ => \ASSC_SEQOUT_net[5]\, 
        \EV_ASSCADDR_net[8]\ => \EV_ASSCADDR_net[8]\, 
        \tkcbxxqbtfx_ns[11]\ => \tkcbxxqbtfx_ns[11]\, 
        \EV_RAM_DI_B_net[8]\ => \EV_RAM_DI_B_net[8]\, 
        \ASSC_RAM_DO_A_net[8]\ => \ASSC_RAM_DO_A_net[8]\, 
        \TR_EV_ADDR_net[8]\ => \TR_EV_ADDR_net[8]\, 
        \EV_RAM_ADDR_A_net[8]\ => \EV_RAM_ADDR_A_net[8]\, 
        \rdgwgtpkbgc[3]\ => \rdgwgtpkbgc_0[3]\, 
        \EV_CHHOLD_net[4]\ => \EV_CHHOLD_net[4]\, 
        \EV_RAM_DO_B_net[8]\ => \EV_RAM_DO_B_net[8]\, 
        \Z\\EV_RAM_ADDR_B_net\\\ => \Z\\EV_RAM_ADDR_B_net\\\, 
        EV_RAM_ADDR_B_net_net_2 => EV_RAM_ADDR_B_net_net_2, 
        EV_RAM_ADDR_B_net_net_4 => EV_RAM_ADDR_B_net_net_4, 
        EV_RAM_ADDR_B_net_net_7 => EV_RAM_ADDR_B_net_net_7, 
        \Z\\ADC_CHNR_net\\\ => \Z\\ADC_CHNR_net\\\, 
        ADC_CHNR_net_net_1 => ADC_CHNR_net_net_1, 
        ADC_CHNR_net_net_2 => ADC_CHNR_net_net_2, 
        ADC_CHNR_net_net_3 => ADC_CHNR_net_net_3, 
        \Z\\ASSC_SEQOUT_net\\\ => \Z\\ASSC_SEQOUT_net\\\, 
        \Z\\EV_ASSCADDR_net\\\ => \Z\\EV_ASSCADDR_net\\\, 
        ASSC_SEQOUT_net_net_1 => ASSC_SEQOUT_net_net_1, 
        EV_ASSCADDR_net_net_1 => EV_ASSCADDR_net_net_1, 
        ASSC_SEQOUT_net_net_2 => ASSC_SEQOUT_net_net_2, 
        EV_ASSCADDR_net_net_2 => EV_ASSCADDR_net_net_2, 
        ASSC_SEQOUT_net_net_3 => ASSC_SEQOUT_net_net_3, 
        EV_ASSCADDR_net_net_3 => EV_ASSCADDR_net_net_3, 
        ASSC_SEQOUT_net_net_4 => ASSC_SEQOUT_net_net_4, 
        EV_ASSCADDR_net_net_4 => EV_ASSCADDR_net_net_4, 
        EV_EVFLAG_net => EV_EVFLAG_net, EV_DONE_net => 
        EV_DONE_net, EV_RAM_WR_BUSY_B_net => EV_RAM_WR_BUSY_B_net, 
        EV_RAM_DI_B_net_net_7 => EV_RAM_DI_B_net_net_7, 
        EV_RAM_DI_B_net_net_6 => EV_RAM_DI_B_net_net_6, 
        EV_RAM_DI_B_net_net_2 => EV_RAM_DI_B_net_net_2, 
        \Z\\EV_RAM_DI_B_net\\\ => \Z\\EV_RAM_DI_B_net\\\, 
        EV_RAM_DI_B_net_net_1 => EV_RAM_DI_B_net_net_1, 
        TR_EV_RD_net => TR_EV_RD_net, EV_RAM_CSN_A_net => 
        EV_RAM_CSN_A_net, INIT_EV_WEN => INIT_EV_WEN, 
        EV_RAM_RW_A_net => EV_RAM_RW_A_net, ASSC_DONE_c => 
        \ASSC_DONE_c\, ASSC_SAMPFLAG_net => ASSC_SAMPFLAG_net, 
        jngrzvhtjvn_0 => jngrzvhtjvn_0, rhbzpsxhwms => 
        rhbzpsxhwms, TR_EV_ADDR_net_net_7 => TR_EV_ADDR_net_net_7, 
        EV_RAM_ADDR_A_net_net_7 => EV_RAM_ADDR_A_net_net_7, 
        TR_EV_ADDR_net_net_6 => TR_EV_ADDR_net_net_6, 
        EV_RAM_ADDR_A_net_net_6 => EV_RAM_ADDR_A_net_net_6, 
        TR_EV_ADDR_net_net_4 => TR_EV_ADDR_net_net_4, 
        EV_RAM_ADDR_A_net_net_4 => EV_RAM_ADDR_A_net_net_4, 
        TR_EV_ADDR_net_net_3 => TR_EV_ADDR_net_net_3, 
        EV_RAM_ADDR_A_net_net_3 => EV_RAM_ADDR_A_net_net_3, 
        TR_EV_ADDR_net_net_2 => TR_EV_ADDR_net_net_2, 
        EV_RAM_ADDR_A_net_net_2 => EV_RAM_ADDR_A_net_net_2, 
        TR_EV_ADDR_net_net_1 => TR_EV_ADDR_net_net_1, 
        EV_RAM_ADDR_A_net_net_1 => EV_RAM_ADDR_A_net_net_1, 
        \Z\\TR_EV_ADDR_net\\\ => \Z\\TR_EV_ADDR_net\\\, 
        \Z\\EV_RAM_ADDR_A_net\\\ => \Z\\EV_RAM_ADDR_A_net\\\, 
        TR_EV_ADDR_net_net_5 => TR_EV_ADDR_net_net_5, 
        EV_RAM_ADDR_A_net_net_5 => EV_RAM_ADDR_A_net_net_5, 
        ASSC_RAM_DO_A_net_net_7 => ASSC_RAM_DO_A_net_net_7, 
        \Z\\ASSC_RAM_DO_A_net\\\ => \Z\\ASSC_RAM_DO_A_net\\\, 
        ASSC_RAM_DO_A_net_net_1 => ASSC_RAM_DO_A_net_net_1, 
        ASSC_RAM_DO_A_net_net_2 => ASSC_RAM_DO_A_net_net_2, 
        ASSC_RAM_DO_A_net_net_3 => ASSC_RAM_DO_A_net_net_3, 
        ASSC_RAM_DO_A_net_net_4 => ASSC_RAM_DO_A_net_net_4, 
        ASSC_RAM_DO_A_net_net_5 => ASSC_RAM_DO_A_net_net_5, 
        ASSC_RAM_DO_A_net_net_6 => ASSC_RAM_DO_A_net_net_6, 
        EV_RAM_DO_B_net_net_3 => EV_RAM_DO_B_net_net_3, 
        EV_RAM_DO_B_net_net_4 => EV_RAM_DO_B_net_net_4, 
        EV_RAM_DI_B_net_net_4 => EV_RAM_DI_B_net_net_4, 
        EV_RAM_DI_B_net_net_3 => EV_RAM_DI_B_net_net_3, 
        EV_RAM_DI_B_net_net_5 => EV_RAM_DI_B_net_net_5, 
        INIT_DONE_c => INIT_DONE_c, EV_RAM_CSN_B_net => 
        EV_RAM_CSN_B_net, EV_ASSCADDR_net_net_5 => 
        EV_ASSCADDR_net_net_5, EV_RAM_DO_B_net_net_2 => 
        EV_RAM_DO_B_net_net_2, EV_CHHOLD_net_net_3 => 
        EV_CHHOLD_net_net_3, EV_CHHOLD_net_net_2 => 
        EV_CHHOLD_net_net_2, EV_CHHOLD_net_net_1 => 
        EV_CHHOLD_net_net_1, \Z\\EV_CHHOLD_net\\\ => 
        \Z\\EV_CHHOLD_net\\\, EV_RAM_DO_B_net_net_1 => 
        EV_RAM_DO_B_net_net_1, \Z\\EV_RAM_DO_B_net\\\ => 
        \Z\\EV_RAM_DO_B_net\\\, EV_RAM_ADDR_B_net_net_1 => 
        EV_RAM_ADDR_B_net_net_1, EV_RAM_ADDR_B_net_net_5 => 
        EV_RAM_ADDR_B_net_net_5, EV_RAM_ADDR_B_net_net_3 => 
        EV_RAM_ADDR_B_net_net_3, EV_RAM_ADDR_B_net_net_6 => 
        EV_RAM_ADDR_B_net_net_6, EV_RAM_DO_B_net_net_5 => 
        EV_RAM_DO_B_net_net_5, EV_RAM_DO_B_net_net_6 => 
        EV_RAM_DO_B_net_net_6, bskrjhjvqpf_1_sqmuxa => 
        bskrjhjvqpf_1_sqmuxa, EV_RAM_DO_B_net_net_7 => 
        EV_RAM_DO_B_net_net_7, sys_reset_gl => sys_reset_gl, 
        ACM_CLK_c => ACM_CLK_c, EV_DONE_net_0 => EV_DONE_net_0);
    
    Analog_RTC_smev_ram_inst : Analog_RTC_smev_ram
      port map(INIT_DATA(8) => INIT_DATA(8), INIT_DATA(7) => 
        INIT_DATA(7), INIT_DATA(6) => INIT_DATA(6), INIT_DATA(5)
         => INIT_DATA(5), INIT_DATA(4) => INIT_DATA(4), 
        INIT_DATA(3) => INIT_DATA(3), INIT_DATA(2) => 
        INIT_DATA(2), INIT_DATA(1) => INIT_DATA(1), INIT_DATA(0)
         => INIT_DATA(0), \EV_RAM_DI_B_net[8]\ => 
        \EV_RAM_DI_B_net[8]\, \EV_RAM_ADDR_B_net[8]\ => 
        \EV_RAM_ADDR_B_net[8]\, \EV_RAM_ADDR_A_net[8]\ => 
        \EV_RAM_ADDR_A_net[8]\, \EV_RAM_DO_B_net[8]\ => 
        \EV_RAM_DO_B_net[8]\, sys_reset_gl => sys_reset_gl, 
        \Z\\EV_RAM_DI_B_net\\\ => \Z\\EV_RAM_DI_B_net\\\, 
        EV_RAM_DI_B_net_net_1 => EV_RAM_DI_B_net_net_1, 
        EV_RAM_DI_B_net_net_2 => EV_RAM_DI_B_net_net_2, 
        EV_RAM_DI_B_net_net_3 => EV_RAM_DI_B_net_net_3, 
        EV_RAM_DI_B_net_net_4 => EV_RAM_DI_B_net_net_4, 
        EV_RAM_DI_B_net_net_5 => EV_RAM_DI_B_net_net_5, 
        EV_RAM_DI_B_net_net_6 => EV_RAM_DI_B_net_net_6, 
        EV_RAM_DI_B_net_net_7 => EV_RAM_DI_B_net_net_7, ACM_CLK_c
         => ACM_CLK_c, EV_RAM_WR_BUSY_B_net => 
        EV_RAM_WR_BUSY_B_net, EV_RAM_CSN_B_net => 
        EV_RAM_CSN_B_net, EV_RAM_RW_A_net => EV_RAM_RW_A_net, 
        EV_RAM_CSN_A_net => EV_RAM_CSN_A_net, 
        \Z\\EV_RAM_ADDR_B_net\\\ => \Z\\EV_RAM_ADDR_B_net\\\, 
        EV_RAM_ADDR_B_net_net_1 => EV_RAM_ADDR_B_net_net_1, 
        EV_RAM_ADDR_B_net_net_2 => EV_RAM_ADDR_B_net_net_2, 
        EV_RAM_ADDR_B_net_net_3 => EV_RAM_ADDR_B_net_net_3, 
        EV_RAM_ADDR_B_net_net_4 => EV_RAM_ADDR_B_net_net_4, 
        EV_RAM_ADDR_B_net_net_5 => EV_RAM_ADDR_B_net_net_5, 
        EV_RAM_ADDR_B_net_net_6 => EV_RAM_ADDR_B_net_net_6, 
        EV_RAM_ADDR_B_net_net_7 => EV_RAM_ADDR_B_net_net_7, 
        \Z\\EV_RAM_ADDR_A_net\\\ => \Z\\EV_RAM_ADDR_A_net\\\, 
        EV_RAM_ADDR_A_net_net_1 => EV_RAM_ADDR_A_net_net_1, 
        EV_RAM_ADDR_A_net_net_2 => EV_RAM_ADDR_A_net_net_2, 
        EV_RAM_ADDR_A_net_net_3 => EV_RAM_ADDR_A_net_net_3, 
        EV_RAM_ADDR_A_net_net_4 => EV_RAM_ADDR_A_net_net_4, 
        EV_RAM_ADDR_A_net_net_5 => EV_RAM_ADDR_A_net_net_5, 
        EV_RAM_ADDR_A_net_net_6 => EV_RAM_ADDR_A_net_net_6, 
        EV_RAM_ADDR_A_net_net_7 => EV_RAM_ADDR_A_net_net_7, 
        \Z\\EV_RAM_DO_B_net\\\ => \Z\\EV_RAM_DO_B_net\\\, 
        EV_RAM_DO_B_net_net_1 => EV_RAM_DO_B_net_net_1, 
        EV_RAM_DO_B_net_net_2 => EV_RAM_DO_B_net_net_2, 
        EV_RAM_DO_B_net_net_3 => EV_RAM_DO_B_net_net_3, 
        EV_RAM_DO_B_net_net_4 => EV_RAM_DO_B_net_net_4, 
        EV_RAM_DO_B_net_net_5 => EV_RAM_DO_B_net_net_5, 
        EV_RAM_DO_B_net_net_6 => EV_RAM_DO_B_net_net_6, 
        EV_RAM_DO_B_net_net_7 => EV_RAM_DO_B_net_net_7, 
        EV_RAM_DO_A_net_net_7 => EV_RAM_DO_A_net_net_7);
    
    Analog_RTC_smtr_wrapper_inst : Analog_RTC_smtr_wrapper
      port map(\TR_RAM_ADDR_B_net[8]\ => \TR_RAM_ADDR_B_net[8]\, 
        \TR_EV_ADDR_net[8]\ => \TR_EV_ADDR_net[8]\, 
        \TR_RAM_DI_B_net[8]\ => \TR_RAM_DI_B_net[8]\, 
        \EV_CHHOLD_net[4]\ => \EV_CHHOLD_net[4]\, 
        \TR_RAM_DO_B_net[8]\ => \TR_RAM_DO_B_net[8]\, 
        \Z\\TR_RAM_ADDR_B_net\\\ => \Z\\TR_RAM_ADDR_B_net\\\, 
        TR_RAM_ADDR_B_net_net_2 => TR_RAM_ADDR_B_net_net_2, 
        TR_RAM_ADDR_B_net_net_4 => TR_RAM_ADDR_B_net_net_4, 
        TR_RAM_ADDR_B_net_net_7 => TR_RAM_ADDR_B_net_net_7, 
        \Z\\TR_EV_ADDR_net\\\ => \Z\\TR_EV_ADDR_net\\\, 
        TR_EV_ADDR_net_net_1 => TR_EV_ADDR_net_net_1, 
        TR_EV_ADDR_net_net_2 => TR_EV_ADDR_net_net_2, 
        TR_EV_ADDR_net_net_3 => TR_EV_ADDR_net_net_3, 
        TR_EV_ADDR_net_net_4 => TR_EV_ADDR_net_net_4, 
        TR_EV_ADDR_net_net_5 => TR_EV_ADDR_net_net_5, 
        TR_EV_ADDR_net_net_6 => TR_EV_ADDR_net_net_6, 
        TR_EV_ADDR_net_net_7 => TR_EV_ADDR_net_net_7, 
        TR_TRFLAG_net => TR_TRFLAG_net, TR_DONE_net => 
        TR_DONE_net, N_14 => N_14, N_12 => N_12, cnwzzrhdsqv_i_a2
         => cnwzzrhdsqv_i_a2, \Z\\TR_RAM_DI_B_net\\\ => 
        \Z\\TR_RAM_DI_B_net\\\, TR_RAM_DI_B_net_net_1 => 
        TR_RAM_DI_B_net_net_1, TR_RAM_DI_B_net_net_7 => 
        TR_RAM_DI_B_net_net_7, TR_RAM_DI_B_net_net_6 => 
        TR_RAM_DI_B_net_net_6, TR_RAM_DI_B_net_net_5 => 
        TR_RAM_DI_B_net_net_5, TR_RAM_DI_B_net_net_4 => 
        TR_RAM_DI_B_net_net_4, TR_RAM_DI_B_net_net_3 => 
        TR_RAM_DI_B_net_net_3, TR_RAM_RW_B_net => TR_RAM_RW_B_net, 
        TR_RAM_DI_B_net_net_2 => TR_RAM_DI_B_net_net_2, 
        EV_CHHOLD_net_net_2 => EV_CHHOLD_net_net_2, 
        EV_CHHOLD_net_net_3 => EV_CHHOLD_net_net_3, 
        \Z\\EV_CHHOLD_net\\\ => \Z\\EV_CHHOLD_net\\\, EV_DONE_net
         => EV_DONE_net, EV_CHHOLD_net_net_1 => 
        EV_CHHOLD_net_net_1, EV_RAM_DO_A_net_net_7 => 
        EV_RAM_DO_A_net_net_7, TR_RAM_DO_B_net_net_7 => 
        TR_RAM_DO_B_net_net_7, EV_DONE_net_0 => EV_DONE_net_0, 
        TR_RAM_CSN_B_net => TR_RAM_CSN_B_net, 
        TR_RAM_DO_B_net_net_3 => TR_RAM_DO_B_net_net_3, 
        TR_RAM_DO_B_net_net_5 => TR_RAM_DO_B_net_net_5, 
        TR_RAM_DO_B_net_net_4 => TR_RAM_DO_B_net_net_4, 
        \Z\\TR_RAM_DO_B_net\\\ => \Z\\TR_RAM_DO_B_net\\\, 
        TR_RAM_DO_B_net_net_1 => TR_RAM_DO_B_net_net_1, 
        TR_RAM_DO_B_net_net_2 => TR_RAM_DO_B_net_net_2, 
        INIT_DONE_c => INIT_DONE_c, TR_RAM_DO_B_net_net_6 => 
        TR_RAM_DO_B_net_net_6, TR_EV_RD_net => TR_EV_RD_net, 
        TR_RAM_ADDR_B_net_net_1 => TR_RAM_ADDR_B_net_net_1, 
        TR_RAM_ADDR_B_net_net_5 => TR_RAM_ADDR_B_net_net_5, 
        TR_RAM_ADDR_B_net_net_3 => TR_RAM_ADDR_B_net_net_3, 
        TR_RAM_ADDR_B_net_net_6 => TR_RAM_ADDR_B_net_net_6, 
        sys_reset_gl => sys_reset_gl, ACM_CLK_c => ACM_CLK_c);
    

end DEF_ARCH; 

library ieee;
use ieee.std_logic_1164.all;
library fusion;

entity RTC_demo is

    port(DATA_LCD : out std_logic_vector(7 downto 4); RTC_year : 
        out std_logic_vector(7 downto 0); RTC_month : 
        out std_logic_vector(7 downto 0); RTC_date : 
        out std_logic_vector(7 downto 0); RTC_day : 
        out std_logic_vector(7 downto 0); RTC_hours : 
        out std_logic_vector(7 downto 0); RTC_minutes : 
        out std_logic_vector(7 downto 0); RTC_seconds : 
        out std_logic_vector(7 downto 0); SYS_RESET, POWER_down : 
        in std_logic; INIT_DONE : out std_logic; VAREF : inout
         std_logic := 'Z'; DATAVALID, ASSC_DONE, ASSC_CHSAT, 
        ASSC_CHLATD, RS_LCD, EN_LCD, R_nW_LCD : out std_logic; 
        SW1 : in std_logic; ACM_CLK, clk_78khz, FPGAGOOD, PUCORE : 
        out std_logic; PUB, SW2, XTAL1 : in std_logic; RTCCLK_out, 
        RTC_act : out std_logic);

end RTC_demo;

architecture DEF_ARCH of RTC_demo is 

  component AND3
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component MX2
    port(A, B, S : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XTAL_RTC
    port(RTC_MODE : 
        in std_logic_vector(1 downto 0) := (others => 'U'); XTAL1, 
        SELMODE : in std_logic := 'U'; RTCCLK : out std_logic);
  end component;

  component OUTBUF
    port(D : in std_logic := 'U'; PAD : out std_logic);
  end component;

  component NOR2A
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component BUFF
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component INBUF
    port(PAD : in std_logic := 'U'; Y : out std_logic);
  end component;

  component AOI1B
    port(A, B, C : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NOR2B
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component XOR2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component DFN1C0
    port(D, CLK, CLR : in std_logic := 'U'; Q : out std_logic);
  end component;

  component GND
    port(Y : out std_logic);
  end component;

  component CLKINT
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component VCC
    port(Y : out std_logic);
  end component;

  component PLL_100_10
    port(sys_reset_gl, sys_clk : in std_logic := 'U'; ACM_CLK_c : 
        out std_logic);
  end component;

  component INV
    port(A : in std_logic := 'U'; Y : out std_logic);
  end component;

  component VR_mon
    port(PWR_dwn, PUB : in std_logic := 'U'; PUCORE_c, FPGAGOOD_c : 
        out std_logic);
  end component;

  component LCD_DIS
    port(RTC_minutes_c : 
        in std_logic_vector(6 downto 0) := (others => 'U'); 
        RTC_month_c : 
        in std_logic_vector(7 downto 0) := (others => 'U'); 
        RTC_date_c : 
        in std_logic_vector(5 downto 0) := (others => 'U'); 
        RTC_seconds_c : 
        in std_logic_vector(6 downto 0) := (others => 'U'); 
        RTC_year_c : 
        in std_logic_vector(3 downto 0) := (others => 'U'); 
        RTC_hours_c : 
        in std_logic_vector(4 downto 0) := (others => 'U'); 
        DATA_LCD_c : out std_logic_vector(7 downto 4); SW1_c, 
        SW2_c_0, SW2_c, ACM_CLK_c, SW1_c_1 : in std_logic := 'U'; 
        RS_LCD_c : out std_logic; SW1_c_0 : in std_logic := 'U'; 
        EN_LCD_c : out std_logic);
  end component;

  component RCO
    port(sys_clk : out std_logic);
  end component;

  component AND2
    port(A, B : in std_logic := 'U'; Y : out std_logic);
  end component;

  component NVM_analog
    port(INIT_ADDR_int : out std_logic_vector(8 downto 0); 
        INIT_DATA_int : out std_logic_vector(8 downto 0); 
        INIT_ASSC_WEN, N_14, INIT_EV_WEN, N_12, hrgmtcpsdzr : out
         std_logic; sys_reset_gl, ACM_CLK_c : in std_logic := 'U');
        
  end component;

  component Shift8_reg
    port(sys_reset_gl, POWER_down_c, ACM_CLK_c : in
         std_logic := 'U'; PWR_dwn : out std_logic);
  end component;

  component RTC_calendar
    port(ACMWDATA : out std_logic_vector(7 downto 0); ACMRDATA : 
        in std_logic_vector(7 downto 0) := (others => 'U'); 
        RTC_year_c : out std_logic_vector(7 downto 0); 
        RTC_seconds_c : out std_logic_vector(7 downto 0); 
        RTC_hours_c : out std_logic_vector(7 downto 0); 
        RTC_date_c : out std_logic_vector(7 downto 0); 
        RTC_month_c : out std_logic_vector(7 downto 0); 
        RTC_minutes_c : out std_logic_vector(7 downto 0); 
        RTC_day_c : out std_logic_vector(7 downto 0); ACMADDR_3, 
        ACMADDR_6, ACMADDR_2, ACMADDR_4, ACMADDR_0, ACMADDR_1 : 
        out std_logic; I_296, POWER_down_c : in std_logic := 'U'; 
        un1_rtc_init_0 : out std_logic; INIT_DONE_c, un1_rtc_init : 
        in std_logic := 'U'; N_410, N_417, RTC_act_i, 
        count_ld_i_0 : out std_logic; sys_reset_gl, ACM_CLK_c : 
        in std_logic := 'U');
  end component;

  component Analog_RTC
    port(INIT_ADDR_int : 
        in std_logic_vector(8 downto 5) := (others => 'U'); 
        INIT_DATA : 
        in std_logic_vector(8 downto 0) := (others => 'U'); 
        INIT_ADDR : 
        in std_logic_vector(8 downto 0) := (others => 'U'); 
        RTC_MODE : out std_logic_vector(1 downto 0); ACMRDATA : 
        out std_logic_vector(7 downto 0); \rdgwgtpkbgc_0[3]\ : in
         std_logic := 'U'; \rdgwgtpkbgc[3]\ : out std_logic; 
        INIT_EV_WEN, N_12, N_14 : in std_logic := 'U'; 
        ASSC_CHLATD_c_i : out std_logic; INIT_DONE_c, 
        INIT_ASSC_WEN : in std_logic := 'U'; ASSC_DONE_c, 
        ASSC_CHSAT_c : out std_logic; INIT_ACM_WEN, RTCCLK, 
        sys_reset_gl, ACM_CLK_c : in std_logic := 'U'; SELMODE, 
        DATAVALID_c : out std_logic; VAREF : inout std_logic);
  end component;

    signal SW1_c_1, SW1_c, SW1_c_0, SW2_c_0, SW2_c, 
        \RTC_cal.un1_rtc_init\, un1_rtc_init_0, \I_296\, 
        count_ld_i_0, 
        \AnalogRTC.Analog_RTC_smev_wrapper_inst.U0.u_nbnjbfrkqwq.rdgwgtpkbgc[3]\, 
        \rdgwgtpkbgc[3]\, INIT_DONE_c, hrgmtcpsdzr, 
        \SYS_RESET_c_i\, SYS_RESET_c, \INIT_ACM_WEN\, 
        \RTC_cal.N_410\, \RTC_cal.N_417\, \INIT_ADDR[0]_net_1\, 
        \INIT_ADDR_int[0]\, \ACMADDR[0]\, \INIT_ADDR[1]_net_1\, 
        \INIT_ADDR_int[1]\, \ACMADDR[1]\, \INIT_ADDR[2]_net_1\, 
        \INIT_ADDR_int[2]\, \ACMADDR[2]\, \INIT_ADDR[4]_net_1\, 
        \INIT_ADDR_int[4]\, \ACMADDR[4]\, \INIT_DATA[0]_net_1\, 
        \INIT_DATA_int[0]\, \ACMWDATA[0]\, \INIT_DATA[1]_net_1\, 
        \INIT_DATA_int[1]\, \ACMWDATA[1]\, \INIT_DATA[2]_net_1\, 
        \INIT_DATA_int[2]\, \ACMWDATA[2]\, \INIT_DATA[3]_net_1\, 
        \INIT_DATA_int[3]\, \ACMWDATA[3]\, \INIT_DATA[4]_net_1\, 
        \INIT_DATA_int[4]\, \ACMWDATA[4]\, \INIT_DATA[5]_net_1\, 
        \INIT_DATA_int[5]\, \ACMWDATA[5]\, \INIT_DATA[6]_net_1\, 
        \INIT_DATA_int[6]\, \ACMWDATA[6]\, \INIT_DATA[7]_net_1\, 
        \INIT_DATA_int[7]\, \ACMWDATA[7]\, \INIT_DATA[8]_net_1\, 
        \INIT_DATA_int[8]\, \INIT_ADDR[8]_net_1\, 
        \INIT_ADDR_int[8]\, \INIT_ADDR[7]_net_1\, 
        \INIT_ADDR_int[7]\, \INIT_ADDR[6]_net_1\, 
        \INIT_ADDR_int[6]\, \ACMADDR[6]\, \INIT_ADDR[5]_net_1\, 
        \INIT_ADDR_int[5]\, \INIT_ADDR[3]_net_1\, 
        \INIT_ADDR_int[3]\, \ACMADDR[3]\, \count[0]_net_1\, 
        ACM_CLK_c, sys_reset_gl, I_4, \count[1]_net_1\, I_5_6, 
        \count[2]_net_1\, I_9_6, \count[3]_net_1\, I_13_7, 
        \count[4]_net_1\, I_20_0, \count[5]_net_1\, I_24_0, 
        \count_c[6]\, I_31, \RTC_seconds_c[7]\, 
        \RTC_seconds_c[6]\, \RTC_seconds_c[5]\, 
        \RTC_seconds_c[4]\, \RTC_seconds_c[3]\, 
        \RTC_seconds_c[2]\, \RTC_seconds_c[1]\, 
        \RTC_seconds_c[0]\, \RTC_minutes_c[7]\, 
        \RTC_minutes_c[6]\, \RTC_minutes_c[5]\, 
        \RTC_minutes_c[4]\, \RTC_minutes_c[3]\, 
        \RTC_minutes_c[2]\, \RTC_minutes_c[1]\, 
        \RTC_minutes_c[0]\, \RTC_hours_c[7]\, \RTC_hours_c[6]\, 
        \RTC_hours_c[5]\, \RTC_hours_c[4]\, \RTC_hours_c[3]\, 
        \RTC_hours_c[2]\, \RTC_hours_c[1]\, \RTC_hours_c[0]\, 
        \RTC_day_c[7]\, \RTC_day_c[6]\, \RTC_day_c[5]\, 
        \RTC_day_c[4]\, \RTC_day_c[3]\, \RTC_day_c[2]\, 
        \RTC_day_c[1]\, \RTC_day_c[0]\, \RTC_date_c[7]\, 
        \RTC_date_c[6]\, \RTC_date_c[5]\, \RTC_date_c[4]\, 
        \RTC_date_c[3]\, \RTC_date_c[2]\, \RTC_date_c[1]\, 
        \RTC_date_c[0]\, \RTC_month_c[7]\, \RTC_month_c[6]\, 
        \RTC_month_c[5]\, \RTC_month_c[4]\, \RTC_month_c[3]\, 
        \RTC_month_c[2]\, \RTC_month_c[1]\, \RTC_month_c[0]\, 
        \RTC_year_c[7]\, \RTC_year_c[6]\, \RTC_year_c[5]\, 
        \RTC_year_c[4]\, \RTC_year_c[3]\, \RTC_year_c[2]\, 
        \RTC_year_c[1]\, \RTC_year_c[0]\, RTC_act_i, \GND\, 
        PUCORE_c, FPGAGOOD_c, EN_LCD_c, \DATA_LCD_c[7]\, 
        \DATA_LCD_c[6]\, \DATA_LCD_c[5]\, \DATA_LCD_c[4]\, 
        RS_LCD_c, ASSC_CHLATD_c_i, ASSC_CHSAT_c, ASSC_DONE_c, 
        DATAVALID_c, POWER_down_c, \un2_count.N_20\, 
        \un2_count.N_12\, \un2_count.U1.DWACT_FINC_E[0]\, 
        \un2_count.N_4\, \un2_count.U1.DWACT_FINC_E[1]\, 
        \un2_count.N_9\, \un2_count.N_17\, sys_clk, PWR_dwn, 
        \ACMRDATA[0]\, \ACMRDATA[1]\, \ACMRDATA[2]\, 
        \ACMRDATA[3]\, \ACMRDATA[4]\, \ACMRDATA[5]\, 
        \ACMRDATA[6]\, \ACMRDATA[7]\, \RTC_MODE[0]\, 
        \RTC_MODE[1]\, SELMODE, RTCCLK, INIT_ASSC_WEN, 
        \NVManalog.NVM_analog_init_wrapper_inst.U0.u_grwzcnqmqzm.valid_client.u_gpbdjfttbjd.N_14\, 
        INIT_EV_WEN, 
        \NVManalog.NVM_analog_init_wrapper_inst.U0.u_grwzcnqmqzm.valid_client.u_gpbdjfttbjd.N_12\, 
        \VCC\, GND_net_1, VCC_net_1 : std_logic;

    for all : XTAL_RTC
	Use entity work.XTAL_RTC(DEF_ARCH);
    for all : PLL_100_10
	Use entity work.PLL_100_10(DEF_ARCH);
    for all : VR_mon
	Use entity work.VR_mon(DEF_ARCH);
    for all : LCD_DIS
	Use entity work.LCD_DIS(DEF_ARCH);
    for all : RCO
	Use entity work.RCO(DEF_ARCH);
    for all : NVM_analog
	Use entity work.NVM_analog(DEF_ARCH);
    for all : Shift8_reg
	Use entity work.Shift8_reg(DEF_ARCH);
    for all : RTC_calendar
	Use entity work.RTC_calendar(DEF_ARCH);
    for all : Analog_RTC
	Use entity work.Analog_RTC(DEF_ARCH);
begin 


    un2_count_I_23 : AND3
      port map(A => \un2_count.U1.DWACT_FINC_E[0]\, B => 
        \count[3]_net_1\, C => \count[4]_net_1\, Y => 
        \un2_count.N_9\);
    
    \INIT_ADDR[3]\ : MX2
      port map(A => \INIT_ADDR_int[3]\, B => \ACMADDR[3]\, S => 
        INIT_DONE_c, Y => \INIT_ADDR[3]_net_1\);
    
    XTALRTC : XTAL_RTC
      port map(RTC_MODE(1) => \RTC_MODE[1]\, RTC_MODE(0) => 
        \RTC_MODE[0]\, XTAL1 => XTAL1, SELMODE => SELMODE, RTCCLK
         => RTCCLK);
    
    \DATA_LCD_pad[6]\ : OUTBUF
      port map(D => \DATA_LCD_c[6]\, PAD => DATA_LCD(6));
    
    \RTC_minutes_pad[1]\ : OUTBUF
      port map(D => \RTC_minutes_c[1]\, PAD => RTC_minutes(1));
    
    \RTC_date_pad[5]\ : OUTBUF
      port map(D => \RTC_date_c[5]\, PAD => RTC_date(5));
    
    \RTC_minutes_pad[3]\ : OUTBUF
      port map(D => \RTC_minutes_c[3]\, PAD => RTC_minutes(3));
    
    \INIT_DATA[8]\ : NOR2A
      port map(A => \INIT_DATA_int[8]\, B => INIT_DONE_c, Y => 
        \INIT_DATA[8]_net_1\);
    
    \RTC_year_pad[5]\ : OUTBUF
      port map(D => \RTC_year_c[5]\, PAD => RTC_year(5));
    
    \RTC_year_pad[2]\ : OUTBUF
      port map(D => \RTC_year_c[2]\, PAD => RTC_year(2));
    
    SW1_pad_1 : BUFF
      port map(A => SW1_c, Y => SW1_c_1);
    
    POWER_down_pad : INBUF
      port map(PAD => POWER_down, Y => POWER_down_c);
    
    INIT_ACM_WEN : AOI1B
      port map(A => \RTC_cal.N_410\, B => \RTC_cal.N_417\, C => 
        INIT_DONE_c, Y => \INIT_ACM_WEN\);
    
    un2_count_I_19 : NOR2B
      port map(A => \count[3]_net_1\, B => 
        \un2_count.U1.DWACT_FINC_E[0]\, Y => \un2_count.N_12\);
    
    \RTC_month_pad[7]\ : OUTBUF
      port map(D => \RTC_month_c[7]\, PAD => RTC_month(7));
    
    \RTC_day_pad[6]\ : OUTBUF
      port map(D => \RTC_day_c[6]\, PAD => RTC_day(6));
    
    \RTC_minutes_pad[4]\ : OUTBUF
      port map(D => \RTC_minutes_c[4]\, PAD => RTC_minutes(4));
    
    \RTC_date_pad[7]\ : OUTBUF
      port map(D => \RTC_date_c[7]\, PAD => RTC_date(7));
    
    \RTC_hours_pad[0]\ : OUTBUF
      port map(D => \RTC_hours_c[0]\, PAD => RTC_hours(0));
    
    \RTC_seconds_pad[4]\ : OUTBUF
      port map(D => \RTC_seconds_c[4]\, PAD => RTC_seconds(4));
    
    un2_count_I_20 : XOR2
      port map(A => \un2_count.N_12\, B => \count[4]_net_1\, Y
         => I_20_0);
    
    \count[5]\ : DFN1C0
      port map(D => I_24_0, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        Q => \count[5]_net_1\);
    
    ASSC_CHLATD_pad : OUTBUF
      port map(D => ASSC_CHLATD_c_i, PAD => ASSC_CHLATD);
    
    \RTC_day_pad[5]\ : OUTBUF
      port map(D => \RTC_day_c[5]\, PAD => RTC_day(5));
    
    \INIT_DATA[0]\ : MX2
      port map(A => \INIT_DATA_int[0]\, B => \ACMWDATA[0]\, S => 
        INIT_DONE_c, Y => \INIT_DATA[0]_net_1\);
    
    \INIT_DATA[3]\ : MX2
      port map(A => \INIT_DATA_int[3]\, B => \ACMWDATA[3]\, S => 
        INIT_DONE_c, Y => \INIT_DATA[3]_net_1\);
    
    un2_count_I_5 : XOR2
      port map(A => \count[0]_net_1\, B => \count[1]_net_1\, Y
         => I_5_6);
    
    \RTC_seconds_pad[2]\ : OUTBUF
      port map(D => \RTC_seconds_c[2]\, PAD => RTC_seconds(2));
    
    \INIT_DATA[2]\ : MX2
      port map(A => \INIT_DATA_int[2]\, B => \ACMWDATA[2]\, S => 
        INIT_DONE_c, Y => \INIT_DATA[2]_net_1\);
    
    \count[4]\ : DFN1C0
      port map(D => I_20_0, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        Q => \count[4]_net_1\);
    
    GND_i : GND
      port map(Y => \GND\);
    
    \RTC_day_pad[1]\ : OUTBUF
      port map(D => \RTC_day_c[1]\, PAD => RTC_day(1));
    
    \INIT_DATA[7]\ : MX2
      port map(A => \INIT_DATA_int[7]\, B => \ACMWDATA[7]\, S => 
        INIT_DONE_c, Y => \INIT_DATA[7]_net_1\);
    
    \RTC_date_pad[0]\ : OUTBUF
      port map(D => \RTC_date_c[0]\, PAD => RTC_date(0));
    
    reset : CLKINT
      port map(A => \SYS_RESET_c_i\, Y => sys_reset_gl);
    
    VCC_i_0 : VCC
      port map(Y => VCC_net_1);
    
    \RTC_hours_pad[6]\ : OUTBUF
      port map(D => \RTC_hours_c[6]\, PAD => RTC_hours(6));
    
    \count[0]\ : DFN1C0
      port map(D => I_4, CLK => ACM_CLK_c, CLR => sys_reset_gl, Q
         => \count[0]_net_1\);
    
    \RTC_minutes_pad[5]\ : OUTBUF
      port map(D => \RTC_minutes_c[5]\, PAD => RTC_minutes(5));
    
    \INIT_ADDR[6]\ : MX2
      port map(A => \INIT_ADDR_int[6]\, B => \ACMADDR[6]\, S => 
        INIT_DONE_c, Y => \INIT_ADDR[6]_net_1\);
    
    \RTC_year_pad[6]\ : OUTBUF
      port map(D => \RTC_year_c[6]\, PAD => RTC_year(6));
    
    un2_count_I_12 : AND3
      port map(A => \count[0]_net_1\, B => \count[1]_net_1\, C
         => \count[2]_net_1\, Y => \un2_count.N_17\);
    
    FPGAGOOD_pad : OUTBUF
      port map(D => FPGAGOOD_c, PAD => FPGAGOOD);
    
    \RTC_minutes_pad[2]\ : OUTBUF
      port map(D => \RTC_minutes_c[2]\, PAD => RTC_minutes(2));
    
    un2_count_I_16 : AND3
      port map(A => \count[0]_net_1\, B => \count[1]_net_1\, C
         => \count[2]_net_1\, Y => \un2_count.U1.DWACT_FINC_E[0]\);
    
    ACM_CLK_pad : OUTBUF
      port map(D => ACM_CLK_c, PAD => ACM_CLK);
    
    \DATA_LCD_pad[4]\ : OUTBUF
      port map(D => \DATA_LCD_c[4]\, PAD => DATA_LCD(4));
    
    \RTC_month_pad[1]\ : OUTBUF
      port map(D => \RTC_month_c[1]\, PAD => RTC_month(1));
    
    \RTC_year_pad[3]\ : OUTBUF
      port map(D => \RTC_year_c[3]\, PAD => RTC_year(3));
    
    VCC_i : VCC
      port map(Y => \VCC\);
    
    un2_count_I_13 : XOR2
      port map(A => \un2_count.N_17\, B => \count[3]_net_1\, Y
         => I_13_7);
    
    SW2_pad : INBUF
      port map(PAD => SW2, Y => SW2_c);
    
    \INIT_DATA[6]\ : MX2
      port map(A => \INIT_DATA_int[6]\, B => \ACMWDATA[6]\, S => 
        INIT_DONE_c, Y => \INIT_DATA[6]_net_1\);
    
    \RTC_hours_pad[5]\ : OUTBUF
      port map(D => \RTC_hours_c[5]\, PAD => RTC_hours(5));
    
    \RTC_year_pad[0]\ : OUTBUF
      port map(D => \RTC_year_c[0]\, PAD => RTC_year(0));
    
    SW2_pad_0 : BUFF
      port map(A => SW2_c, Y => SW2_c_0);
    
    RS_LCD_pad : OUTBUF
      port map(D => RS_LCD_c, PAD => RS_LCD);
    
    I_295 : CLKINT
      port map(A => \rdgwgtpkbgc[3]\, Y => 
        \AnalogRTC.Analog_RTC_smev_wrapper_inst.U0.u_nbnjbfrkqwq.rdgwgtpkbgc[3]\);
    
    \RTC_year_pad[4]\ : OUTBUF
      port map(D => \RTC_year_c[4]\, PAD => RTC_year(4));
    
    un2_count_I_31 : XOR2
      port map(A => \un2_count.N_4\, B => \count_c[6]\, Y => I_31);
    
    \RTC_seconds_pad[7]\ : OUTBUF
      port map(D => \RTC_seconds_c[7]\, PAD => RTC_seconds(7));
    
    \RTC_hours_pad[1]\ : OUTBUF
      port map(D => \RTC_hours_c[1]\, PAD => RTC_hours(1));
    
    \RTC_date_pad[4]\ : OUTBUF
      port map(D => \RTC_date_c[4]\, PAD => RTC_date(4));
    
    un2_count_I_9 : XOR2
      port map(A => \un2_count.N_20\, B => \count[2]_net_1\, Y
         => I_9_6);
    
    \RTC_minutes_pad[7]\ : OUTBUF
      port map(D => \RTC_minutes_c[7]\, PAD => RTC_minutes(7));
    
    PLL_acm : PLL_100_10
      port map(sys_reset_gl => sys_reset_gl, sys_clk => sys_clk, 
        ACM_CLK_c => ACM_CLK_c);
    
    \RTC_day_pad[2]\ : OUTBUF
      port map(D => \RTC_day_c[2]\, PAD => RTC_day(2));
    
    SYS_RESET_c_i : INV
      port map(A => SYS_RESET_c, Y => \SYS_RESET_c_i\);
    
    Vreg : VR_mon
      port map(PWR_dwn => PWR_dwn, PUB => PUB, PUCORE_c => 
        PUCORE_c, FPGAGOOD_c => FPGAGOOD_c);
    
    \INIT_ADDR[0]\ : MX2
      port map(A => \INIT_ADDR_int[0]\, B => \ACMADDR[0]\, S => 
        INIT_DONE_c, Y => \INIT_ADDR[0]_net_1\);
    
    \count[1]\ : DFN1C0
      port map(D => I_5_6, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        Q => \count[1]_net_1\);
    
    \RTC_year_pad[7]\ : OUTBUF
      port map(D => \RTC_year_c[7]\, PAD => RTC_year(7));
    
    \INIT_ADDR[2]\ : MX2
      port map(A => \INIT_ADDR_int[2]\, B => \ACMADDR[2]\, S => 
        INIT_DONE_c, Y => \INIT_ADDR[2]_net_1\);
    
    \count[3]\ : DFN1C0
      port map(D => I_13_7, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        Q => \count[3]_net_1\);
    
    \RTC_month_pad[3]\ : OUTBUF
      port map(D => \RTC_month_c[3]\, PAD => RTC_month(3));
    
    \RTC_minutes_pad[6]\ : OUTBUF
      port map(D => \RTC_minutes_c[6]\, PAD => RTC_minutes(6));
    
    SYS_RESET_pad : INBUF
      port map(PAD => SYS_RESET, Y => SYS_RESET_c);
    
    \RTC_year_pad[1]\ : OUTBUF
      port map(D => \RTC_year_c[1]\, PAD => RTC_year(1));
    
    \INIT_ADDR[5]\ : NOR2A
      port map(A => \INIT_ADDR_int[5]\, B => INIT_DONE_c, Y => 
        \INIT_ADDR[5]_net_1\);
    
    R_nW_LCD_pad : OUTBUF
      port map(D => \GND\, PAD => R_nW_LCD);
    
    \RTC_day_pad[0]\ : OUTBUF
      port map(D => \RTC_day_c[0]\, PAD => RTC_day(0));
    
    \RTC_month_pad[0]\ : OUTBUF
      port map(D => \RTC_month_c[0]\, PAD => RTC_month(0));
    
    un2_count_I_30 : AND3
      port map(A => \un2_count.U1.DWACT_FINC_E[0]\, B => 
        \un2_count.U1.DWACT_FINC_E[1]\, C => \count[5]_net_1\, Y
         => \un2_count.N_4\);
    
    RTCCLK_out_pad : OUTBUF
      port map(D => \GND\, PAD => RTCCLK_out);
    
    \RTC_seconds_pad[5]\ : OUTBUF
      port map(D => \RTC_seconds_c[5]\, PAD => RTC_seconds(5));
    
    un2_count_I_24 : XOR2
      port map(A => \un2_count.N_9\, B => \count[5]_net_1\, Y => 
        I_24_0);
    
    LCD_DIS_inst : LCD_DIS
      port map(RTC_minutes_c(6) => \RTC_minutes_c[6]\, 
        RTC_minutes_c(5) => \RTC_minutes_c[5]\, RTC_minutes_c(4)
         => \RTC_minutes_c[4]\, RTC_minutes_c(3) => 
        \RTC_minutes_c[3]\, RTC_minutes_c(2) => 
        \RTC_minutes_c[2]\, RTC_minutes_c(1) => 
        \RTC_minutes_c[1]\, RTC_minutes_c(0) => 
        \RTC_minutes_c[0]\, RTC_month_c(7) => \RTC_month_c[7]\, 
        RTC_month_c(6) => \RTC_month_c[6]\, RTC_month_c(5) => 
        \RTC_month_c[5]\, RTC_month_c(4) => \RTC_month_c[4]\, 
        RTC_month_c(3) => \RTC_month_c[3]\, RTC_month_c(2) => 
        \RTC_month_c[2]\, RTC_month_c(1) => \RTC_month_c[1]\, 
        RTC_month_c(0) => \RTC_month_c[0]\, RTC_date_c(5) => 
        \RTC_date_c[5]\, RTC_date_c(4) => \RTC_date_c[4]\, 
        RTC_date_c(3) => \RTC_date_c[3]\, RTC_date_c(2) => 
        \RTC_date_c[2]\, RTC_date_c(1) => \RTC_date_c[1]\, 
        RTC_date_c(0) => \RTC_date_c[0]\, RTC_seconds_c(6) => 
        \RTC_seconds_c[6]\, RTC_seconds_c(5) => 
        \RTC_seconds_c[5]\, RTC_seconds_c(4) => 
        \RTC_seconds_c[4]\, RTC_seconds_c(3) => 
        \RTC_seconds_c[3]\, RTC_seconds_c(2) => 
        \RTC_seconds_c[2]\, RTC_seconds_c(1) => 
        \RTC_seconds_c[1]\, RTC_seconds_c(0) => 
        \RTC_seconds_c[0]\, RTC_year_c(3) => \RTC_year_c[3]\, 
        RTC_year_c(2) => \RTC_year_c[2]\, RTC_year_c(1) => 
        \RTC_year_c[1]\, RTC_year_c(0) => \RTC_year_c[0]\, 
        RTC_hours_c(4) => \RTC_hours_c[4]\, RTC_hours_c(3) => 
        \RTC_hours_c[3]\, RTC_hours_c(2) => \RTC_hours_c[2]\, 
        RTC_hours_c(1) => \RTC_hours_c[1]\, RTC_hours_c(0) => 
        \RTC_hours_c[0]\, DATA_LCD_c(7) => \DATA_LCD_c[7]\, 
        DATA_LCD_c(6) => \DATA_LCD_c[6]\, DATA_LCD_c(5) => 
        \DATA_LCD_c[5]\, DATA_LCD_c(4) => \DATA_LCD_c[4]\, SW1_c
         => SW1_c, SW2_c_0 => SW2_c_0, SW2_c => SW2_c, ACM_CLK_c
         => ACM_CLK_c, SW1_c_1 => SW1_c_1, RS_LCD_c => RS_LCD_c, 
        SW1_c_0 => SW1_c_0, EN_LCD_c => EN_LCD_c);
    
    \RTC_seconds_pad[6]\ : OUTBUF
      port map(D => \RTC_seconds_c[6]\, PAD => RTC_seconds(6));
    
    SW1_pad : INBUF
      port map(PAD => SW1, Y => SW1_c);
    
    \RTC_date_pad[3]\ : OUTBUF
      port map(D => \RTC_date_c[3]\, PAD => RTC_date(3));
    
    \INIT_DATA[1]\ : MX2
      port map(A => \INIT_DATA_int[1]\, B => \ACMWDATA[1]\, S => 
        INIT_DONE_c, Y => \INIT_DATA[1]_net_1\);
    
    RC_osc : RCO
      port map(sys_clk => sys_clk);
    
    ASSC_DONE_pad : OUTBUF
      port map(D => ASSC_DONE_c, PAD => ASSC_DONE);
    
    I_294 : CLKINT
      port map(A => hrgmtcpsdzr, Y => INIT_DONE_c);
    
    \INIT_ADDR[4]\ : MX2
      port map(A => \INIT_ADDR_int[4]\, B => \ACMADDR[4]\, S => 
        INIT_DONE_c, Y => \INIT_ADDR[4]_net_1\);
    
    GND_i_0 : GND
      port map(Y => GND_net_1);
    
    SW1_pad_0 : BUFF
      port map(A => SW1_c, Y => SW1_c_0);
    
    \RTC_day_pad[4]\ : OUTBUF
      port map(D => \RTC_day_c[4]\, PAD => RTC_day(4));
    
    PUCORE_pad : OUTBUF
      port map(D => PUCORE_c, PAD => PUCORE);
    
    un2_count_I_4 : INV
      port map(A => \count[0]_net_1\, Y => I_4);
    
    \DATA_LCD_pad[7]\ : OUTBUF
      port map(D => \DATA_LCD_c[7]\, PAD => DATA_LCD(7));
    
    \RTC_date_pad[6]\ : OUTBUF
      port map(D => \RTC_date_c[6]\, PAD => RTC_date(6));
    
    \RTC_date_pad[1]\ : OUTBUF
      port map(D => \RTC_date_c[1]\, PAD => RTC_date(1));
    
    \RTC_month_pad[6]\ : OUTBUF
      port map(D => \RTC_month_c[6]\, PAD => RTC_month(6));
    
    INIT_DONE_pad : OUTBUF
      port map(D => INIT_DONE_c, PAD => INIT_DONE);
    
    \count[6]\ : DFN1C0
      port map(D => I_31, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        Q => \count_c[6]\);
    
    \RTC_hours_pad[2]\ : OUTBUF
      port map(D => \RTC_hours_c[2]\, PAD => RTC_hours(2));
    
    \RTC_seconds_pad[1]\ : OUTBUF
      port map(D => \RTC_seconds_c[1]\, PAD => RTC_seconds(1));
    
    \RTC_hours_pad[3]\ : OUTBUF
      port map(D => \RTC_hours_c[3]\, PAD => RTC_hours(3));
    
    \INIT_DATA[4]\ : MX2
      port map(A => \INIT_DATA_int[4]\, B => \ACMWDATA[4]\, S => 
        INIT_DONE_c, Y => \INIT_DATA[4]_net_1\);
    
    \INIT_ADDR[8]\ : NOR2A
      port map(A => \INIT_ADDR_int[8]\, B => INIT_DONE_c, Y => 
        \INIT_ADDR[8]_net_1\);
    
    \RTC_month_pad[5]\ : OUTBUF
      port map(D => \RTC_month_c[5]\, PAD => RTC_month(5));
    
    un2_count_I_27 : AND2
      port map(A => \count[3]_net_1\, B => \count[4]_net_1\, Y
         => \un2_count.U1.DWACT_FINC_E[1]\);
    
    NVManalog : NVM_analog
      port map(INIT_ADDR_int(8) => \INIT_ADDR_int[8]\, 
        INIT_ADDR_int(7) => \INIT_ADDR_int[7]\, INIT_ADDR_int(6)
         => \INIT_ADDR_int[6]\, INIT_ADDR_int(5) => 
        \INIT_ADDR_int[5]\, INIT_ADDR_int(4) => 
        \INIT_ADDR_int[4]\, INIT_ADDR_int(3) => 
        \INIT_ADDR_int[3]\, INIT_ADDR_int(2) => 
        \INIT_ADDR_int[2]\, INIT_ADDR_int(1) => 
        \INIT_ADDR_int[1]\, INIT_ADDR_int(0) => 
        \INIT_ADDR_int[0]\, INIT_DATA_int(8) => 
        \INIT_DATA_int[8]\, INIT_DATA_int(7) => 
        \INIT_DATA_int[7]\, INIT_DATA_int(6) => 
        \INIT_DATA_int[6]\, INIT_DATA_int(5) => 
        \INIT_DATA_int[5]\, INIT_DATA_int(4) => 
        \INIT_DATA_int[4]\, INIT_DATA_int(3) => 
        \INIT_DATA_int[3]\, INIT_DATA_int(2) => 
        \INIT_DATA_int[2]\, INIT_DATA_int(1) => 
        \INIT_DATA_int[1]\, INIT_DATA_int(0) => 
        \INIT_DATA_int[0]\, INIT_ASSC_WEN => INIT_ASSC_WEN, N_14
         => 
        \NVManalog.NVM_analog_init_wrapper_inst.U0.u_grwzcnqmqzm.valid_client.u_gpbdjfttbjd.N_14\, 
        INIT_EV_WEN => INIT_EV_WEN, N_12 => 
        \NVManalog.NVM_analog_init_wrapper_inst.U0.u_grwzcnqmqzm.valid_client.u_gpbdjfttbjd.N_12\, 
        hrgmtcpsdzr => hrgmtcpsdzr, sys_reset_gl => sys_reset_gl, 
        ACM_CLK_c => ACM_CLK_c);
    
    \RTC_day_pad[3]\ : OUTBUF
      port map(D => \RTC_day_c[3]\, PAD => RTC_day(3));
    
    \RTC_hours_pad[7]\ : OUTBUF
      port map(D => \RTC_hours_c[7]\, PAD => RTC_hours(7));
    
    \RTC_month_pad[4]\ : OUTBUF
      port map(D => \RTC_month_c[4]\, PAD => RTC_month(4));
    
    \RTC_seconds_pad[3]\ : OUTBUF
      port map(D => \RTC_seconds_c[3]\, PAD => RTC_seconds(3));
    
    un2_count_I_8 : NOR2B
      port map(A => \count[1]_net_1\, B => \count[0]_net_1\, Y
         => \un2_count.N_20\);
    
    \INIT_ADDR[7]\ : NOR2A
      port map(A => \INIT_ADDR_int[7]\, B => INIT_DONE_c, Y => 
        \INIT_ADDR[7]_net_1\);
    
    DATAVALID_pad : OUTBUF
      port map(D => DATAVALID_c, PAD => DATAVALID);
    
    \RTC_day_pad[7]\ : OUTBUF
      port map(D => \RTC_day_c[7]\, PAD => RTC_day(7));
    
    sreg8 : Shift8_reg
      port map(sys_reset_gl => sys_reset_gl, POWER_down_c => 
        POWER_down_c, ACM_CLK_c => ACM_CLK_c, PWR_dwn => PWR_dwn);
    
    \count[2]\ : DFN1C0
      port map(D => I_9_6, CLK => ACM_CLK_c, CLR => sys_reset_gl, 
        Q => \count[2]_net_1\);
    
    EN_LCD_pad : OUTBUF
      port map(D => EN_LCD_c, PAD => EN_LCD);
    
    RTC_act_pad : OUTBUF
      port map(D => RTC_act_i, PAD => RTC_act);
    
    RTC_cal : RTC_calendar
      port map(ACMWDATA(7) => \ACMWDATA[7]\, ACMWDATA(6) => 
        \ACMWDATA[6]\, ACMWDATA(5) => \ACMWDATA[5]\, ACMWDATA(4)
         => \ACMWDATA[4]\, ACMWDATA(3) => \ACMWDATA[3]\, 
        ACMWDATA(2) => \ACMWDATA[2]\, ACMWDATA(1) => 
        \ACMWDATA[1]\, ACMWDATA(0) => \ACMWDATA[0]\, ACMRDATA(7)
         => \ACMRDATA[7]\, ACMRDATA(6) => \ACMRDATA[6]\, 
        ACMRDATA(5) => \ACMRDATA[5]\, ACMRDATA(4) => 
        \ACMRDATA[4]\, ACMRDATA(3) => \ACMRDATA[3]\, ACMRDATA(2)
         => \ACMRDATA[2]\, ACMRDATA(1) => \ACMRDATA[1]\, 
        ACMRDATA(0) => \ACMRDATA[0]\, RTC_year_c(7) => 
        \RTC_year_c[7]\, RTC_year_c(6) => \RTC_year_c[6]\, 
        RTC_year_c(5) => \RTC_year_c[5]\, RTC_year_c(4) => 
        \RTC_year_c[4]\, RTC_year_c(3) => \RTC_year_c[3]\, 
        RTC_year_c(2) => \RTC_year_c[2]\, RTC_year_c(1) => 
        \RTC_year_c[1]\, RTC_year_c(0) => \RTC_year_c[0]\, 
        RTC_seconds_c(7) => \RTC_seconds_c[7]\, RTC_seconds_c(6)
         => \RTC_seconds_c[6]\, RTC_seconds_c(5) => 
        \RTC_seconds_c[5]\, RTC_seconds_c(4) => 
        \RTC_seconds_c[4]\, RTC_seconds_c(3) => 
        \RTC_seconds_c[3]\, RTC_seconds_c(2) => 
        \RTC_seconds_c[2]\, RTC_seconds_c(1) => 
        \RTC_seconds_c[1]\, RTC_seconds_c(0) => 
        \RTC_seconds_c[0]\, RTC_hours_c(7) => \RTC_hours_c[7]\, 
        RTC_hours_c(6) => \RTC_hours_c[6]\, RTC_hours_c(5) => 
        \RTC_hours_c[5]\, RTC_hours_c(4) => \RTC_hours_c[4]\, 
        RTC_hours_c(3) => \RTC_hours_c[3]\, RTC_hours_c(2) => 
        \RTC_hours_c[2]\, RTC_hours_c(1) => \RTC_hours_c[1]\, 
        RTC_hours_c(0) => \RTC_hours_c[0]\, RTC_date_c(7) => 
        \RTC_date_c[7]\, RTC_date_c(6) => \RTC_date_c[6]\, 
        RTC_date_c(5) => \RTC_date_c[5]\, RTC_date_c(4) => 
        \RTC_date_c[4]\, RTC_date_c(3) => \RTC_date_c[3]\, 
        RTC_date_c(2) => \RTC_date_c[2]\, RTC_date_c(1) => 
        \RTC_date_c[1]\, RTC_date_c(0) => \RTC_date_c[0]\, 
        RTC_month_c(7) => \RTC_month_c[7]\, RTC_month_c(6) => 
        \RTC_month_c[6]\, RTC_month_c(5) => \RTC_month_c[5]\, 
        RTC_month_c(4) => \RTC_month_c[4]\, RTC_month_c(3) => 
        \RTC_month_c[3]\, RTC_month_c(2) => \RTC_month_c[2]\, 
        RTC_month_c(1) => \RTC_month_c[1]\, RTC_month_c(0) => 
        \RTC_month_c[0]\, RTC_minutes_c(7) => \RTC_minutes_c[7]\, 
        RTC_minutes_c(6) => \RTC_minutes_c[6]\, RTC_minutes_c(5)
         => \RTC_minutes_c[5]\, RTC_minutes_c(4) => 
        \RTC_minutes_c[4]\, RTC_minutes_c(3) => 
        \RTC_minutes_c[3]\, RTC_minutes_c(2) => 
        \RTC_minutes_c[2]\, RTC_minutes_c(1) => 
        \RTC_minutes_c[1]\, RTC_minutes_c(0) => 
        \RTC_minutes_c[0]\, RTC_day_c(7) => \RTC_day_c[7]\, 
        RTC_day_c(6) => \RTC_day_c[6]\, RTC_day_c(5) => 
        \RTC_day_c[5]\, RTC_day_c(4) => \RTC_day_c[4]\, 
        RTC_day_c(3) => \RTC_day_c[3]\, RTC_day_c(2) => 
        \RTC_day_c[2]\, RTC_day_c(1) => \RTC_day_c[1]\, 
        RTC_day_c(0) => \RTC_day_c[0]\, ACMADDR_3 => \ACMADDR[3]\, 
        ACMADDR_6 => \ACMADDR[6]\, ACMADDR_2 => \ACMADDR[2]\, 
        ACMADDR_4 => \ACMADDR[4]\, ACMADDR_0 => \ACMADDR[0]\, 
        ACMADDR_1 => \ACMADDR[1]\, I_296 => \I_296\, POWER_down_c
         => POWER_down_c, un1_rtc_init_0 => un1_rtc_init_0, 
        INIT_DONE_c => INIT_DONE_c, un1_rtc_init => 
        \RTC_cal.un1_rtc_init\, N_410 => \RTC_cal.N_410\, N_417
         => \RTC_cal.N_417\, RTC_act_i => RTC_act_i, count_ld_i_0
         => count_ld_i_0, sys_reset_gl => sys_reset_gl, ACM_CLK_c
         => ACM_CLK_c);
    
    \INIT_DATA[5]\ : MX2
      port map(A => \INIT_DATA_int[5]\, B => \ACMWDATA[5]\, S => 
        INIT_DONE_c, Y => \INIT_DATA[5]_net_1\);
    
    \INIT_ADDR[1]\ : MX2
      port map(A => \INIT_ADDR_int[1]\, B => \ACMADDR[1]\, S => 
        INIT_DONE_c, Y => \INIT_ADDR[1]_net_1\);
    
    \RTC_minutes_pad[0]\ : OUTBUF
      port map(D => \RTC_minutes_c[0]\, PAD => RTC_minutes(0));
    
    I_297 : CLKINT
      port map(A => un1_rtc_init_0, Y => \RTC_cal.un1_rtc_init\);
    
    \RTC_month_pad[2]\ : OUTBUF
      port map(D => \RTC_month_c[2]\, PAD => RTC_month(2));
    
    clk_78khz_pad : OUTBUF
      port map(D => \count_c[6]\, PAD => clk_78khz);
    
    AnalogRTC : Analog_RTC
      port map(INIT_ADDR_int(8) => \INIT_ADDR_int[8]\, 
        INIT_ADDR_int(7) => \INIT_ADDR_int[7]\, INIT_ADDR_int(6)
         => \INIT_ADDR_int[6]\, INIT_ADDR_int(5) => 
        \INIT_ADDR_int[5]\, INIT_DATA(8) => \INIT_DATA[8]_net_1\, 
        INIT_DATA(7) => \INIT_DATA[7]_net_1\, INIT_DATA(6) => 
        \INIT_DATA[6]_net_1\, INIT_DATA(5) => 
        \INIT_DATA[5]_net_1\, INIT_DATA(4) => 
        \INIT_DATA[4]_net_1\, INIT_DATA(3) => 
        \INIT_DATA[3]_net_1\, INIT_DATA(2) => 
        \INIT_DATA[2]_net_1\, INIT_DATA(1) => 
        \INIT_DATA[1]_net_1\, INIT_DATA(0) => 
        \INIT_DATA[0]_net_1\, INIT_ADDR(8) => 
        \INIT_ADDR[8]_net_1\, INIT_ADDR(7) => 
        \INIT_ADDR[7]_net_1\, INIT_ADDR(6) => 
        \INIT_ADDR[6]_net_1\, INIT_ADDR(5) => 
        \INIT_ADDR[5]_net_1\, INIT_ADDR(4) => 
        \INIT_ADDR[4]_net_1\, INIT_ADDR(3) => 
        \INIT_ADDR[3]_net_1\, INIT_ADDR(2) => 
        \INIT_ADDR[2]_net_1\, INIT_ADDR(1) => 
        \INIT_ADDR[1]_net_1\, INIT_ADDR(0) => 
        \INIT_ADDR[0]_net_1\, RTC_MODE(1) => \RTC_MODE[1]\, 
        RTC_MODE(0) => \RTC_MODE[0]\, ACMRDATA(7) => 
        \ACMRDATA[7]\, ACMRDATA(6) => \ACMRDATA[6]\, ACMRDATA(5)
         => \ACMRDATA[5]\, ACMRDATA(4) => \ACMRDATA[4]\, 
        ACMRDATA(3) => \ACMRDATA[3]\, ACMRDATA(2) => 
        \ACMRDATA[2]\, ACMRDATA(1) => \ACMRDATA[1]\, ACMRDATA(0)
         => \ACMRDATA[0]\, \rdgwgtpkbgc_0[3]\ => 
        \AnalogRTC.Analog_RTC_smev_wrapper_inst.U0.u_nbnjbfrkqwq.rdgwgtpkbgc[3]\, 
        \rdgwgtpkbgc[3]\ => \rdgwgtpkbgc[3]\, INIT_EV_WEN => 
        INIT_EV_WEN, N_12 => 
        \NVManalog.NVM_analog_init_wrapper_inst.U0.u_grwzcnqmqzm.valid_client.u_gpbdjfttbjd.N_12\, 
        N_14 => 
        \NVManalog.NVM_analog_init_wrapper_inst.U0.u_grwzcnqmqzm.valid_client.u_gpbdjfttbjd.N_14\, 
        ASSC_CHLATD_c_i => ASSC_CHLATD_c_i, INIT_DONE_c => 
        INIT_DONE_c, INIT_ASSC_WEN => INIT_ASSC_WEN, ASSC_DONE_c
         => ASSC_DONE_c, ASSC_CHSAT_c => ASSC_CHSAT_c, 
        INIT_ACM_WEN => \INIT_ACM_WEN\, RTCCLK => RTCCLK, 
        sys_reset_gl => sys_reset_gl, ACM_CLK_c => ACM_CLK_c, 
        SELMODE => SELMODE, DATAVALID_c => DATAVALID_c, VAREF => 
        VAREF);
    
    \RTC_date_pad[2]\ : OUTBUF
      port map(D => \RTC_date_c[2]\, PAD => RTC_date(2));
    
    \RTC_seconds_pad[0]\ : OUTBUF
      port map(D => \RTC_seconds_c[0]\, PAD => RTC_seconds(0));
    
    ASSC_CHSAT_pad : OUTBUF
      port map(D => ASSC_CHSAT_c, PAD => ASSC_CHSAT);
    
    I_296 : CLKINT
      port map(A => count_ld_i_0, Y => \I_296\);
    
    \DATA_LCD_pad[5]\ : OUTBUF
      port map(D => \DATA_LCD_c[5]\, PAD => DATA_LCD(5));
    
    \RTC_hours_pad[4]\ : OUTBUF
      port map(D => \RTC_hours_c[4]\, PAD => RTC_hours(4));
    

end DEF_ARCH; 
